#include "Lasso8ConnectorforApache.h"
#include "POSTReader.h"
#include "osErrors.h"
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <syslog.h>
#include <http_request.h>
#include <http_log.h> 

static char * kConfFile = "/etc/lasso/lasso8apache.conf";
static char * kServer = "server";
static char * kPort = "port";

static bool gHasInitted = false;
static short gPort = 14552;
static char gAddress[128] = "127.0.0.1";

static pool * gChildPool = NULL;

#define INVALID_SOCKET (-1)
#define DEFAULT_BUFFER (1024*1024)
//#define DO_SYSLOG
/*
	Interface variables:
*/
static const handler_rec handlers8[] = 
{
    { "lasso8-handler", lasso8_handler },
	{ NULL }
};

/* 
 * This is the structure that is used to interface with Apache.  Not all 
 * parameters will be used. - CJS 
*/
module MODULE_VAR_EXPORT lasso8_module =
{
	STANDARD_MODULE_STUFF,
    init_module,	/* initializer */
    NULL,			/* dir config creater */
    NULL,			/* dir merger ensure strictness */
    NULL,			/* server config */
    NULL,			/* merge server config */
    NULL,			/* command table */
    handlers8,		/* handlers */
    NULL,			/* filename translation */
    NULL,			/* check_user_id */
    NULL,			/* check auth */
    NULL,			/* check access */
    NULL,			/* type_checker */
    NULL,			/* fixups */
    NULL,			/* logger */
    NULL,			/* header parser */
    child_init_handler,			/* child_init */
    child_term_handler,	/* child_exit */
    NULL			/* post read-request */
};

int lasso8_handler (request_rec *r)
{
	int sock = INVALID_SOCKET;
    int res = 0;	
    LPCommandBlock block;
    char * data = NULL;
    int reqOpen = 1;
    unsigned int timeout = 1000;	//	05/03/2001 GLH upped timeout to 5 minutes
    int openTries = 0;
    int readCount = 0;
	int read = 0;
	int contentLen = -1, contentAdjust = 0;

    LP8POSTReader reader(ap_table_get(r->headers_in, "Content-Type"));
    if (!get_post(sock, r, &reader))
    	return SERVER_ERROR;

	sock = open_connection(gChildPool, r->server);
    if ( sock == INVALID_SOCKET )
		return SERVER_ERROR;
    //printf("DB 1: %d %d %d %d\n", sock, openTries, read, readCount);
    /* start reading command blocks from Lasso */
    while ( reqOpen == 1 && (read = read_block(sock, &block, &data, &timeout)) == 1 )
    {
    	//printf("DB 1.5: %d %d %d %d %d\n", sock, openTries, read, readCount, block.fCmd);
		switch( block.fCmd )
		{
			case cmdGetParam:
			{
				char * myData = data;
				int numParams = 0;
				
				// scan to see how many params
				for (unsigned int pos = 0; pos < block.fDataSize;)
				{
					++numParams;
					pos += sizeof(RequestParamKeyword);
					
					int dSize = *(int*)(data+pos);
					pos += (ntohl(dSize)+sizeof(int));
				}
				
				block.fCmd = cmdGetParamRep;
				block.fDataSize = 0;
				block.fResultCode = 0;
				
				struct ptr_size
				{
					char * data;
					int size;
				};
				
				ptr_size * rec = (ptr_size*)malloc(numParams*sizeof(ptr_size));
				
				for ( int x = 0; x < numParams; ++x )
				{
					RequestParamKeyword word = (RequestParamKeyword)ntohl(*(int*)myData);
					int dLen = ntohl(*(int*)(myData+sizeof(RequestParamKeyword)));
					
					char * pData = NULL;
					int dSize = 0;
					
					myData += (sizeof(RequestParamKeyword) + sizeof(int));
					
					if ( word == rpPostKeyword )
					{
						const std::string * str = reader.GetPostArgs();
						if ( str != NULL )
						{
							pData = (char*)malloc(str->size());
							dSize = str->size();
							memcpy(pData, str->c_str(), str->size());
//							pData[str->size()] = '\0';
						}
					}
					else
					{
						int res = get_param(r, word, dLen > 0 ? myData : NULL, &pData, &dSize);
						if ( res < 0 )
							block.fResultCode = res;
					}
					rec[x].data = pData;
					rec[x].size = dSize;
					
					block.fDataSize += (dSize + sizeof(int));
				}
				
				/* alloc a buffer for the data portion of the block*/
				char * pData = (char*)malloc(block.fDataSize);
				char * ppData = pData; 
				for ( int x = 0; x < numParams; ++x )
				{
					int mSize = htonl(rec[x].size);
					memcpy(ppData, &mSize, sizeof(int));
					memcpy(ppData+sizeof(int), rec[x].data, rec[x].size);
					ppData += (sizeof(int) + rec[x].size);
					
					free(rec[x].data);
				}
									
				res = send_block(sock, &block, pData);
				// dispose rec and each pointer within!
				free(rec);
				free(pData);
			}
			break;
			case cmdPushData:
				if (block.fDataSize > 0)
				{
					if ( contentLen == -1 )
					{
						if ( strncasecmp("HTTP/1.", data, 7) == 0 )
						{
							const char * s = strstr(data, "\r\n\r\n");
							if ( s )
							{
								contentAdjust = (s-data)+4;
							}
						}
						contentLen = 0;
					}
					contentLen += ap_rwrite(data, block.fDataSize, r);
				}
				break;
			case cmdCloseReq:
				reqOpen = 0;
				r->bytes_sent = contentLen-contentAdjust;
				r->sent_bodyct = contentLen-contentAdjust;
				ap_bsetopt(r->connection->client, BO_BYTECT, &r->sent_bodyct);
				ap_finalize_request_protocol(r);
				break;
			default:
				break;
		}
		free(data);
		data = NULL;
    }
    // DEBUGGING CODE
  	if (reqOpen)
  	{
		ap_log_error(APLOG_MARK, APLOG_EMERG, r->server, 
  	  		"Lasso Connector: connection to LP was not properly closed. client ip: \"%s\" server name: \"%s\" file name: \"%s\" search args: \"%s\"", 
  	  			r->connection->remote_ip, ap_get_server_name(r), r->uri?r->uri:"NULL", r->args?r->args:"NULL");
  	}
    // DEBUGGING CODE
  
    if ( read != 1 ) // some error
    {
   		//printf("DB 2: %d %d %d %d\n", sock, openTries, read, readCount);
	    ap_pclosesocket(r->pool, sock);
   		//printf("DB 3: %d %d %d %d\n", sock, openTries, read, readCount);
	    return SERVER_ERROR;
    }
    if (sock != INVALID_SOCKET)
	    ap_pclosesocket(r->pool, sock);
    return OK;
}

static void copy_string(const char * in, char ** out, int * size )
{
	if ( in != NULL )
	{
		*out = (char*)malloc(strlen(in)+1);
		*size = strlen(in);
		strcpy(*out, in);
	}
}

/* 
 * Workaround for Client_Headers/Full_Request tags under Apache. 
 * Produces concatenated sequence of substrings referenced as elements within
 * the array.  The string will be empty if all substrings are empty or null,
 * or if there are no elements in the array.  - 07.04.2001 - AK 
 */
static void concat_headers(const array_header *arr, char ** data, int * outLen)
{
	const int 		kDelimLen = 2;
	const char* 	kFieldDelim = ": ";
	const char* 	kHeaderDelim = "\r\n";
	
    char *cp, **strpp;
    int i, len = 0;

	/* Empty table? */
	
    if (arr->nelts <= 0 || arr->elts == NULL)      
    {
    	*data = (char*)malloc(1);
    	**data = '\0';
    	*outLen = 0;
    	return;
	}
    
    for (i = arr->nelts * 2, strpp = (char **) arr->elts; ; ++strpp) {  
        if (strpp && *strpp != NULL) {
            len += strlen(*strpp);         
        }
        if (--i <= 0)
            break;
        len += kDelimLen;
    }
    
	cp = *data = (char*) malloc(len + 1);
	if (cp == NULL)
		return;
	
	*outLen = len;
	
    for (i = arr->nelts * 2, strpp = (char **) arr->elts; ; ++strpp) {
        if (strpp && *strpp != NULL) {
            len = strlen(*strpp);
            memcpy(cp, *strpp, len);
            cp += len;
        }
        if (--i <= 0)
            break;
        
        /* copy delimiter */
        memcpy( cp, (i & 0x1) ? kFieldDelim : kHeaderDelim, kDelimLen );
		cp += kDelimLen;
    }

    *cp = '\0';
}

int get_param( request_rec * r, RequestParamKeyword word, const char * param, char ** data, int * outLen )
{
	int good = 1;
	switch( word )
	{
		case rpPathArgKeyword:
			copy_string(r->path_info, data, outLen);
			break;
		case rpSearchArgKeyword:
			copy_string(r->args, data, outLen);
			break;
		case rpAuthorizationKeyword:
			{
				const char * str = ap_table_get(r->headers_in, "Authorization");
				if (str)
					copy_string(str, data, outLen);
				else
					copy_string("", data, outLen);
			}
			break;
		case rpUserKeyword:
		  {
		    const char * str = ap_table_get(r->headers_in, "Authorization");
		    if ( str != NULL && strncasecmp("basic ", str, 6) == 0 )
		      {
			/* move past the first space */
			while(*str && !isspace(*str))
			  ++str;
			if ( *str == ' ' )
			  { 
			    ++str;
			    char * res = ap_uudecode(r->pool, str);
			    char * res2 = res;
			    /* terminate at the colon */
			    while( res && *res && *res != ':' )
			      ++res;
			    if ( *res == ':' ) *res = '\0';
			    copy_string(res2, data, outLen);
			  }
			break;
		      }
		   
		    copy_string("", data, outLen);
		  }
		  break;
		case rpPasswordKeyword:
		  {
		    const char * str = ap_table_get(r->headers_in, "Authorization");
		    if ( str != NULL && strncasecmp("basic ", str, 6) == 0 )
		      {
			/* move past the first space */
			while(*str && !isspace(*str))
			  ++str;
			if ( *str == ' ' )
			  { 
			    ++str;
			    char * res = ap_uudecode(r->pool, str);
			    /* use what's after the colon */
			    while( res && *res && *res != ':' )
			      ++res;
			    if ( *res == ':' ) ++res;
			    copy_string(res, data, outLen);
			  }
			break;
		      }
		   
		    copy_string("", data, outLen);
		  }
		  break;
		case rpFromUser:
			break;
		case rpAddressKeyword:
			copy_string(r->connection->remote_ip, data, outLen);
			break;
		case rpMethodKeyword:
			copy_string(r->method, data, outLen);
			break;
		case rpServerName:
			copy_string(ap_get_server_name(r), data, outLen);
			break;
		case rpServerPort:
			*data = (char*)malloc(20);
			sprintf(*data, "%d", (int)ap_get_server_port(r));
			*outLen = strlen(*data);
			break;
		case rpScriptName:
			copy_string(r->uri, data, outLen);
			break;
		case rpContentType:
			copy_string(ap_table_get(r->headers_in, "Content-Type"), data, outLen);
			break;
		case rpContentLength:
			copy_string(ap_table_get(r->headers_in, "Content-Length"), data, outLen);
			break;
		case rpReferrerKeyword:
			{
				const char * res = ap_table_get(r->headers_in, "Referer");
				if ( res == NULL )
					res = ap_table_get(r->headers_in, "Referrer");
				copy_string(res, data, outLen);
			}
			break;
		case rpUserAgentKeyword:
			copy_string(ap_table_get(r->headers_in, "User-Agent"), data, outLen);
			break;
		case rpActionKeyword:
		case rpActionPathKeyword:
			break;
		case rpClientIPAddress:
			copy_string(r->connection->remote_ip, data, outLen);
			break;
		case rpFullRequestKeyword:
			concat_headers( ap_table_elts(r->headers_in), data, outLen );
			break;
		case rpRealmsKeyword:
		case rpCurrentRealmKeyword:
			break;
		case rpCookiesKeyword:
			copy_string(ap_table_get(r->headers_in, "Cookie"), data, outLen);
			break;
		
		case rpSessionRootPath:
			{
				const char * fn = r->uri;//filename;//uri
				
				char * found = strrchr(fn, '/');
				if ( found != NULL )
				{
					int num = (found-fn)+1;
					*data = (char*)malloc(num+1);
					strncpy(*data, fn, num);
					*outLen = num;
				}
			}
			break;
		case rpResolveFilePath:
			{
				request_rec * subrec = ap_sub_req_lookup_uri(param != NULL ? param : "/", r);
				bool isDir = false;
				struct stat st;
				    
				*data = (char*)malloc( strlen(subrec->filename) + 2 + (subrec->path_info ? strlen(subrec->path_info) : 0) );
				strcpy(*data, subrec->filename);
				if ( subrec->path_info )
					strcat(*data, subrec->path_info);

				if ( (*data)[strlen(*data)-1] != '/' )
				{
					isDir = ::stat(*data, &st) == 0 ? S_ISDIR(st.st_mode) : false;
					if ( !isDir )
						isDir = param[strlen(param)-1] == '/';

					if ( isDir )
						strcat(*data, "/");
				}
				
				*outLen = strlen(*data);
				ap_destroy_sub_req(subrec);
			}
			break;
		
		case rpWebFileData:
			if ( param != NULL )
			{
				char path[1024] = "";
				if ( param[0] != '/' ) // relative from session root
				{
					const char * fn = r->filename;
					char * found = strrchr(fn, '/');
					if ( found != NULL )
					{
						strncat(path, fn, (found-fn)+1);
					}
					strcat(path, param);
				}
				else
				{
					request_rec * subrec = ap_sub_req_lookup_uri(param, r);
					strcat(path, subrec->filename);
					ap_destroy_sub_req(subrec);
				}
                                
				int f = open(path, O_RDONLY, 0);
				if ( f != -1 )
				{
					struct stat st;
				    
					::stat(path, &st);
					int filelen = st.st_size;
                                        
					*data = (char*)malloc(filelen+1);
					int res = read(f, *data, filelen);
					if ( res > -1 )
					{
						(*data)[res] = '\0';
						*outLen = filelen;
					}
					else
					{
						(*data)[0] = '\0';
						*outLen = 0;
					}
					close(f);
				}
				else
				{
					good = osErrFileNotFound;
				}
			}
			break;
		case rpIndexFile:
		default:
			break;		
	}
//	if ( *data == NULL )
//	{
//		*data = (char*)malloc(1);
//		(*data)[0] = '\0';
//		*outLen = 1;
//	}
	return good;
}

int get_post( int sock, request_rec * r, LP8POSTReader * reader )
{
	int toRead = 0;
    if ( ap_setup_client_block(r, REQUEST_CHUNKED_ERROR) == OK )
    {
		if ( ap_should_client_block(r) == 1)
		{
			char * buffer = new char[DEFAULT_BUFFER+1];
			int readLen = 0;
			toRead = r->remaining;
			int readSize = DEFAULT_BUFFER;
			ap_hard_timeout("rpPostKeyword", r);
			
			while ( (readLen = ap_get_client_block(r, buffer, readSize)) > 0 )
			{
			    buffer[readLen] = '\0';
			    reader->AddToBuffer(buffer, readLen);
			    ap_reset_timeout(r);
			    toRead -= readLen;
			    readSize = toRead < DEFAULT_BUFFER ? toRead : DEFAULT_BUFFER;
			}
			ap_kill_timeout(r);	  
			
			delete [] buffer; 
		}
    }
    return toRead == 0 ? 1 : 0;
}

void init_module(server_rec*, pool*)
{
	srandom(time(NULL));
}

int open_connection(pool * p, server_rec * sr)
{
    /*
    open a socket and connect
    */
    int res = 0;
    int s = ap_psocket(p, AF_INET, SOCK_STREAM, 0);
    sockaddr_in addr;
	
    if ( !gHasInitted )
    {
		char buff[256];
		char port[128] = "14552";
		char * tmp = NULL;
		FILE * conf = fopen(kConfFile, "r");
		
		if ( conf != NULL )
		{
			while( (tmp = fgets(buff, 255, conf)) != NULL )
			{
			if ( *tmp == '#' )
				continue;
			
			while(*tmp != '\0' && isspace(*tmp))
				++tmp;
			if ( *tmp != '\0' )
			{
				if ( strncasecmp(tmp, kServer, strlen(kServer)) == 0 )
				{
				int count = 0;
				tmp += strlen(kServer);
				while(*tmp != '\0' && isspace(*tmp))
					++tmp;
				while(*tmp != '\0')
					gAddress[count++] = *tmp++;
				gAddress[count] = '\0';
				}
				else if ( strncasecmp(tmp, kPort, strlen(kPort)) == 0 )
				{
				int count = 0;
				tmp += strlen(kPort);
				while(*tmp != '\0' && isspace(*tmp))
					++tmp;
				while(*tmp != '\0')
					port[count++] = *tmp++;
				port[count] = '\0';	
				gPort = (short)atoi(port);
				}
			}
			}
			fclose(conf);
			gHasInitted = true;
		}
    }
    addr.sin_family = AF_INET;
    addr.sin_port = htons(gPort);
    addr.sin_addr.s_addr = inet_addr(gAddress);
    addr.sin_zero[0] = addr.sin_zero[1] = addr.sin_zero[2] = addr.sin_zero[3] = 0;
	
    res = connect(s, (sockaddr*)&addr, sizeof(sockaddr_in));	
    if ( res == 0 )
    {		
		return s;
    }
    ap_log_error(APLOG_MARK, APLOG_EMERG, sr, "Lasso Connector failed connect: connect=%d socket=%d port=%d address=%s", res, s, gPort, gAddress);
    if ( s != -1 )
		ap_pclosesocket(p, s), s = -1;
	
    return -1;	 
}

void child_term_handler(server_rec*s, pool*p)
{
}

void child_init_handler(server_rec*, pool*p)
{
    gChildPool = p;
}

