
#include "Base64.h"


// MACROS
#define ISBASE64(a) ((('A' <= a && a <= 'Z') || \
					('a' <= a && a <= 'z')   || \
					('0' <= a && a <= '9')   || \
					( a == '+')	|| ( a == '/')) \
					? TRUE : FALSE )

// !FIX - This code needs to be seperated into base 64 decoding and encoding,
// then wrapped into the specific username and password retrieval
///////////////////////////////////////////////////////////////////////////////
//								Base64 Decoding
//
///////////////////////////////////////////////////////////////////////////////

static char base64tab[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    "abcdefghijklmnopqrstuvwxyz0123456789+/";

static char base64idx[128] = {
    '\377','\377','\377','\377','\377','\377','\377','\377',
    '\377','\377','\377','\377','\377','\377','\377','\377',
    '\377','\377','\377','\377','\377','\377','\377','\377',
    '\377','\377','\377','\377','\377','\377','\377','\377',
    '\377','\377','\377','\377','\377','\377','\377','\377',
    '\377','\377','\377',    62,'\377','\377','\377',    63,
        52,    53,    54,    55,    56,    57,    58,    59,
        60,    61,'\377','\377','\377','\377','\377','\377',
    '\377',     0,     1,     2,     3,     4,     5,     6,
         7,     8,     9,    10,    11,    12,    13,    14,
        15,    16,    17,    18,    19,    20,    21,    22,
        23,    24,    25,'\377','\377','\377','\377','\377',
    '\377',    26,    27,    28,    29,    30,    31,    32,
        33,    34,    35,    36,    37,    38,    39,    40,
        41,    42,    43,    44,    45,    46,    47,    48,
        49,    50,    51,'\377','\377','\377','\377','\377'
};

/*-----------------------------------------------------------------------------
 Function:		bool GetUsername(unsigned char* dst,unsigned char* encodedText)
 Description:	this decodes the base64 username and password, and stores the 
				decoded username in dst
-----------------------------------------------------------------------------*/
BOOL GetUsername( unsigned char* dst, unsigned char* encodedText )
{
	int b1, b2, b3;
	int a1, a2, a3, a4;
	while ( *encodedText )
	{
		// see if we've reached the end token
		if ( *encodedText == '?' && *(encodedText+1) == '=' )
		{
			// if we got here, the ':' was never found, so return an error
			*dst = '\0';
			return FALSE;
		}
		
		// Get four input chars at a time and decode them. Ignore white space
		// chars (CR, LF, SP, HT). If '=' is encountered, terminate input. If
		// a char other than white space, base64 char, or '=' is encountered,
		// bail out and just return the encoded text.
	 
		a1 = a2 = a3 = a4 = 0;
		
		// get the first encoded character
		a1 = *encodedText & 0xFF;
		if ( !( ISBASE64(a1) || a1 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the second encoded character
		a2 = *encodedText & 0xFF;
		if ( !( ISBASE64(a2) || a2 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the third encoded character
		a3 = *encodedText & 0xFF;
		if ( !( ISBASE64(a3) || a3 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the fourth encoded character
		a4 = *encodedText & 0xFF;
		if ( !( ISBASE64(a4) || a4 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		if ( ISBASE64(a1) && ISBASE64(a2) && ISBASE64(a3) && ISBASE64(a4) )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			a3 = base64idx[a3] & 0xFF;
			a4 = base64idx[a4] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);
			b2 = ((a2 << 4) & 0xF0) | ((a3 >> 2) & 0x0F);
			b3 = ((a3 << 6) & 0xC0) | ( a4	   & 0x3F);

			if ( (unsigned char)b1 == ':' )
				break;
			*dst++ = (unsigned char)b1;

			if ( (unsigned char)b2 == ':' )
				break;
			*dst++ = (unsigned char)b2;

			if ( (unsigned char)b3 == ':' )
				break;
			*dst++ = (unsigned char)b3;
		}
		else if ( ISBASE64(a1) && ISBASE64(a2) && ISBASE64(a3) && a4 == '=' )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			a3 = base64idx[a3] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);
			b2 = ((a2 << 4) & 0xF0) | ((a3 >> 2) & 0x0F);

			if ( (unsigned char)b1 == ':' )
				break;
			*dst++ = (unsigned char)b1;

			if ( (unsigned char)b2 == ':' )
				break;
			*dst++ = (unsigned char)b2;
		}
		else if ( ISBASE64(a1) && ISBASE64(a2) && a3 == '=' && a4 == '=' )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);

			if ( (unsigned char)b1 == ':' )
				break;
			*dst++ = (unsigned char)b1;
		}
	}

	*dst = '\0';
	return TRUE;
}

/*-----------------------------------------------------------------------------
 Function:		bool GetPassword(unsigned char* dst,unsigned char* encodedText)
 Description:	this decodes the base64 username and password, and stores the 
				decoded password in dst
-----------------------------------------------------------------------------*/

BOOL GetPassword( unsigned char* dst, unsigned char* encodedText )
{
	int b1, b2, b3;
	int a1, a2, a3, a4;
	BOOL decodeToDst = FALSE;
	while ( *encodedText )
	{
		// see if we've reached the end token
		if ( *encodedText == '?' && *(encodedText+1) == '=' )
		{
			// we're done, return
			*dst = '\0';
			return TRUE;
		}
		
		// Get four input chars at a time and decode them. Ignore white space
		// chars (CR, LF, SP, HT). If '=' is encountered, terminate input. If
		// a char other than white space, base64 char, or '=' is encountered,
		// bail out and just return the encoded text.
	 
		a1 = a2 = a3 = a4 = 0;
		
		// get the first encoded character
		a1 = *encodedText & 0xFF;
		if ( !( ISBASE64(a1) || a1 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the second encoded character
		a2 = *encodedText & 0xFF;
		if ( !( ISBASE64(a2) || a2 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the third encoded character
		a3 = *encodedText & 0xFF;
		if ( !( ISBASE64(a3) || a3 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		// get the fourth encoded character
		a4 = *encodedText & 0xFF;
		if ( !( ISBASE64(a4) || a4 == '=' ) )
			return FALSE;	// this base64 word is hosed, so bail
		encodedText++;
		
		if ( ISBASE64(a1) && ISBASE64(a2) && ISBASE64(a3) && ISBASE64(a4) )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			a3 = base64idx[a3] & 0xFF;
			a4 = base64idx[a4] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);
			b2 = ((a2 << 4) & 0xF0) | ((a3 >> 2) & 0x0F);
			b3 = ((a3 << 6) & 0xC0) | ( a4	   & 0x3F);

			if ( decodeToDst )
				*dst++ = (unsigned char)b1;
			else if ( (unsigned char)b1 == ':' )
				decodeToDst = TRUE;

			if ( decodeToDst )
				*dst++ = (unsigned char)b2;
			else if ( (unsigned char)b2 == ':' )
				decodeToDst = TRUE;

			if ( decodeToDst )
				*dst++ = (unsigned char)b3;
			else if ( (unsigned char)b3 == ':' )
				decodeToDst = TRUE;
		}
		else if ( ISBASE64(a1) && ISBASE64(a2) && ISBASE64(a3) && a4 == '=' )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			a3 = base64idx[a3] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);
			b2 = ((a2 << 4) & 0xF0) | ((a3 >> 2) & 0x0F);

			if ( decodeToDst )
				*dst++ = (unsigned char)b1;
			else if ( (unsigned char)b1 == ':' )
				decodeToDst = TRUE;

			if ( decodeToDst )
				*dst++ = (unsigned char)b2;
			else if ( (unsigned char)b2 == ':' )
				decodeToDst = TRUE;
		}
		else if ( ISBASE64(a1) && ISBASE64(a2) && a3 == '=' && a4 == '=' )
		{
			a1 = base64idx[a1] & 0xFF;
			a2 = base64idx[a2] & 0xFF;
			b1 = ((a1 << 2) & 0xFC) | ((a2 >> 4) & 0x03);

			if ( decodeToDst )
				*dst++ = (unsigned char)b1;
			else if ( (unsigned char)b1 == ':' )
				decodeToDst = TRUE;
		}
	}

	*dst = '\0';
	return TRUE;
}
