#include "Lasso4Filter.h"
#include "Base64.h"

// Version info for Filter
#define LASSO_MAJOR_VER 8
#define LASSO_MINOR_VER 0

// Filter description
static const char* kLassoFilterDesc = "Lasso Filter";


///////////////////////////////////////////////////////////////////////////////
//							Lasso Filter Functions
//
///////////////////////////////////////////////////////////////////////////////

BOOL WINAPI GetFilterVersion( HTTP_FILTER_VERSION* version )
{
	version->dwFilterVersion = MAKELONG( LASSO_MINOR_VER, LASSO_MAJOR_VER );

	strcpy( version->lpszFilterDesc, kLassoFilterDesc );

	version->dwFlags = ( SF_NOTIFY_ORDER_HIGH		|
						 SF_NOTIFY_PREPROC_HEADERS | SF_NOTIFY_SEND_RESPONSE);//	| 
//						 SF_NOTIFY_AUTHENTICATION );

	return TRUE;
}

DWORD WINAPI HttpFilterProc( PHTTP_FILTER_CONTEXT pfc, DWORD dwNotifyType,
						   LPVOID* lpData )
{
	DWORD dwRet = SF_STATUS_REQ_NEXT_NOTIFICATION;

	switch( dwNotifyType )
	{
		case SF_NOTIFY_PREPROC_HEADERS:
		{
			dwRet = HandlePreprocHeaders( pfc, (PHTTP_FILTER_PREPROC_HEADERS)lpData );
			break;
		}
		case SF_NOTIFY_AUTHENTICATION:
		{
			dwRet = HandleAuthentication( pfc, (PHTTP_FILTER_AUTHENT)lpData );
			break;
		}
		case SF_NOTIFY_SEND_RESPONSE:
		{
            // must remove the NTLM authentication
			HTTP_FILTER_SEND_RESPONSE * resp = (HTTP_FILTER_SEND_RESPONSE*)lpData;
			char headerVal[1024];
            char * t = NULL, *p = NULL;
			DWORD size = 1024;
			if (resp->GetHeader(pfc, "WWW-Authenticate:", headerVal, &size) == TRUE)
			{
				resp->SetHeader(pfc, "WWW-Authenticate:", 0);
				t = p = headerVal;
				while(1)
				{
					if (*p == ',' || *p == '\0')
					{
						int tmpBrk = *p == '\0';
						*p = '\0';
						if (stricmp(t, "NTLM") != 0 && stricmp(t, "Negotiate") != 0)
						{
							resp->AddHeader(pfc, "WWW-Authenticate:", t);
						}
						t = p + 1;
						if (tmpBrk)
							break;
					}
					++p;
				}
			}
			break;
		}
	}
	
	return dwRet;
}

DWORD HandlePreprocHeaders( HTTP_FILTER_CONTEXT* pfc, PHTTP_FILTER_PREPROC_HEADERS lpHeaders )
{
	// !FIX - When COM wrapper is done will do extension checking so as to not
	// modify requests we shouldn't
	char authorization[1024];
	unsigned long authorizationSize = sizeof( authorization );

	if ( lpHeaders->GetHeader( pfc, "Authorization:", authorization, &authorizationSize ) )
	{
		char* usernamePassword = authorization;
		if( !lpHeaders->AddHeader( pfc, "LassoAuthorization:", usernamePassword) )
			return SF_STATUS_REQ_ERROR;
		if ( *usernamePassword++ == 'B' && 
			 *usernamePassword++ == 'a' && 
			 *usernamePassword++ == 's' && 
			 *usernamePassword++ == 'i' && 
			 *usernamePassword++ == 'c' && 
			 *usernamePassword++ == ' '    )
		{
			char newHeader[512];

			if( GetUsername( (unsigned char*)newHeader, (unsigned char*)usernamePassword ) )
			{
				if( !lpHeaders->AddHeader( pfc, "LassoUsername:", newHeader) )
					return SF_STATUS_REQ_ERROR;
			}

			if( GetPassword( (unsigned char*)newHeader, (unsigned char*)usernamePassword ) )
			{
				if ( !lpHeaders->AddHeader( pfc, "LassoPassword:", newHeader ) )
					return SF_STATUS_REQ_ERROR;
			}
		}
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


DWORD HandleAuthentication( HTTP_FILTER_CONTEXT* pfc, PHTTP_FILTER_AUTHENT lpFilterAuth )
{
	*(lpFilterAuth->pszUser) = '\0';
	*(lpFilterAuth->pszPassword) = '\0';

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}