/*

*/
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <winsock2.h>
#include <vector>
#include "Lasso4Isapi.h"
#include "LassoProto.h"
#include "LassoProtoUtils.h"
#include "osErrors.h"

#include "POSTReader.h"

// Version info for ISAPI
#define LASSO_MAJOR_VER 5
#define LASSO_MINOR_VER 0

// Version info for Winsock .dll
#define WINSOCK_MAJOR_VER 2
#define WINSOCK_MINOR_VER 2

// Might need to be bigger
#define DEFAULT_BUFF 4096 
#define PATH_LEN 2048  // Believe is NT/Win2K path len

//#define DUMP_POST 1

#ifdef DUMP_POST
HANDLE gDumpFile = INVALID_HANDLE_VALUE;
#endif

// ISAPI description
static const char* kLassoIsapiDesc = "Lasso ISAPI, v8.0";

static const char* kTMPLasso = "TMP_LASSO";

unsigned int gtime_out = 1000;
unsigned short gLassoPort = 14552;
struct sockaddr_in gLassoAddr;
unsigned int gCacheSize = 0;//20; // configurable
std::vector<int> gSocketCache;
CRITICAL_SECTION gSocketCacheLock;

static unsigned int gOpenCons = 0;
 
BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpReserved )
{
	switch( dwReason )
	{ 
		case DLL_PROCESS_ATTACH:
			{
				unsigned int x = 0;
				char szport[128] = "14552";
				char ip_add[128] = "127.0.0.1";
				char cache_size[128] = "0";
				char time_out[128] = "90";

				InitializeCriticalSection(&gSocketCacheLock);

				get_conf_value( "port", szport );
				gLassoPort = (unsigned short)atoi(szport);

				get_conf_value( "server", ip_add );

				gLassoAddr.sin_family = AF_INET;
				gLassoAddr.sin_port = htons(gLassoPort);
				gLassoAddr.sin_addr.s_addr = inet_addr( ip_add );
				gLassoAddr.sin_zero[0] = gLassoAddr.sin_zero[1] = gLassoAddr.sin_zero[2] = gLassoAddr.sin_zero[3] = 0;

				get_conf_value("concachesize", cache_size);
				gCacheSize = atoi(cache_size);
 
				get_conf_value( "timeout", time_out );
				gtime_out = atoi( time_out );

				if ( gCacheSize > 0 )
				{
					gSocketCache.reserve(gCacheSize);
				}
			}
			break; 
		case DLL_THREAD_ATTACH:
			//break;
		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			DeleteCriticalSection(&gSocketCacheLock);
			break;
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//							Lasso ISAPI Functions
//
///////////////////////////////////////////////////////////////////////////////

BOOL WINAPI GetExtensionVersion( HSE_VERSION_INFO* pVer )
{
	WORD wVerRequested = MAKEWORD( WINSOCK_MINOR_VER, WINSOCK_MAJOR_VER );
	WSADATA wsaData;
	int nErr;

	pVer->dwExtensionVersion = MAKELONG( LASSO_MINOR_VER, LASSO_MAJOR_VER );
	strncpy_s(pVer->lpszExtensionDesc, HSE_MAX_EXT_DLL_NAME_LEN, kLassoIsapiDesc, _TRUNCATE);

	// Ensure ws2_32.dll loaded
	// Possibly move to DllMain
	nErr = WSAStartup( wVerRequested, &wsaData );

	return TRUE;
}

BOOL WINAPI TerminateExtension( DWORD dwFlags )
{
	WSACleanup( );

	return TRUE;
}

DWORD WINAPI HttpExtensionProc( EXTENSION_CONTROL_BLOCK* pECB )
{

	LPRequestID reqID = -1;
	READ_RESULT res = READ_OK;
	int sock = -1;
	int reqOpen = 1;
	int read = 0;
	char* data = NULL;
	char* contentType = NULL;
	int contentTypeLen = 0;
	int openTries = 0;
	bool sentHeader = false;
	LPCommandBlock block;
	
	GetParam( sock, pECB, rpContentType, NULL, &contentType, &contentTypeLen, NULL );
#ifdef DUMP_POST
	gDumpFile = CreateFile("C:\\POSTDUMP.txt", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL); 
	DWORD s = strlen(contentType)+1;
	DWORD scratch;
	WriteFile(gDumpFile, &s, sizeof(s), &scratch, NULL);
	WriteFile(gDumpFile, contentType, s, &s, NULL);
#endif

	LP8POSTReader postReader(contentType);
	free(contentType);

	GetPOST( sock, pECB, &postReader );
open_con:
	sock = open_connection();

	if ( sock == INVALID_SOCKET )
	{
		send_error( pECB, "Invalid sock on initial open." );
		
		return HSE_STATUS_ERROR;
	}
	
	/* start reading command blocks from Lasso */
	while ( reqOpen == 1 && (read = read_block(sock, &block, &data, &gtime_out)) == READ_OK )
	{
		switch( block.fCmd )
		{
			case cmdGetParam:
			{
				unsigned int pos;
				char* myData = data;
				char* pData = NULL;
				char* ppData = NULL;
				int numParams = 0;
				int x = 0;

				struct ptr_size
				{
					char* data;
					int size;
				};
				
				struct ptr_size* rec;

				// scan to see how many params
				for( pos = 0; pos < block.fDataSize; )
				{
					int dSize = 0;

					++numParams;
					pos += sizeof(RequestParamKeyword);
					dSize = *(int*)(data+pos);
					pos += (ntohl(dSize)+sizeof(int));
				}

				block.fCmd = cmdGetParamRep;
				block.fDataSize = 0;
				
				rec = (struct ptr_size*)malloc(numParams*sizeof(struct ptr_size));
								
				for ( x = 0; (x < numParams); ++x )
				{
					int dLen = 0;
					int dSize = 0;
 					RequestParamKeyword word = *(RequestParamKeyword*)myData;

					word = (RequestParamKeyword)ntohl(word);
					dLen = ntohl(*(int*)(myData+sizeof(RequestParamKeyword)));
					
					myData += (sizeof(RequestParamKeyword) + sizeof(int));

					if ( word == rpPostKeyword )
					{
						const std::string * str = postReader.GetPostArgs( );
						if ( str != NULL ) 
						{
							pData = (char*)malloc(str->size()+1);
							dSize = (int)str->size();
							memcpy(pData, str->data(), dSize+1);
							pData[dSize] = '\0';
						}
						else
						{
							pData = (char*)malloc(1);
							dSize = 0;
							*pData = '\0';
						}

						block.fResultCode = osErrNoErr;
					}
					else
					{
						block.fResultCode = (int)GetParam( sock, pECB, word, dLen > 0 ? myData : NULL, &pData, &dSize, &postReader );
					}

					rec[x].data = pData;
					rec[x].size = dSize;
					
					block.fDataSize += (dSize + sizeof(int));
				}
				
				/* alloc a buffer for the data portion of the block*/
				pData = (char*)malloc(block.fDataSize);
				ppData = pData; 

				for ( x = 0; (x < numParams); ++x )
				{
					int mSize;
						
					mSize = htonl( rec[x].size );
					memcpy(ppData, &mSize, sizeof(int));
					memcpy(ppData+sizeof(int), rec[x].data, rec[x].size);
					ppData += (sizeof(int) + rec[x].size);
					
					free( rec[x].data );
				}
										
				send_block(sock, &block, pData);
				
				// dispose rec and each pointer within!
				free(rec);
				free(pData);

				break;
			}
			case cmdPushData:
				if (block.fDataSize > 0)
				{
					DWORD dwBytes = block.fDataSize;  //Probably do not need
					char * mData = data;
					if (!sentHeader)
					{
	#define ATEND(VAL) (mData + (VAL) >= data + block.fDataSize)
	
						HSE_SEND_HEADER_EX_INFO headerInfo;
						mData += 9;
						headerInfo.pszStatus = mData;
						headerInfo.cchStatus = 0;
						while(!ATEND(headerInfo.cchStatus) && mData[headerInfo.cchStatus] != '\r')
							++headerInfo.cchStatus;
						if (ATEND(headerInfo.cchStatus))
						{
							send_error( pECB, "Server returned malformed header." );
							free(data);
							release_connection(sock, true);
							return HSE_STATUS_ERROR;
						}
						((char*)headerInfo.pszStatus)[headerInfo.cchStatus] = '\0';
	
	//					pECB->dwHttpStatusCode = atoi(mData);
						mData += headerInfo.cchStatus + 2;
						headerInfo.pszHeader = mData;
						headerInfo.cchHeader = 0;
						while(!ATEND(headerInfo.cchHeader))
						{
							if (mData[headerInfo.cchHeader] == '\r')
							{	
								if (mData[++headerInfo.cchHeader] == '\n' && mData[++headerInfo.cchHeader] == '\r' && mData[++headerInfo.cchHeader] == '\n')
								{
									++headerInfo.cchHeader;
									break;
								}
							}
							else
							{
								++headerInfo.cchHeader;
							}
						}
						// now we have to duplicate it since even though we pass in the lengths of these fields, IIS still wants them null terminated (which is pointless)
						char * dupHeaders = new char[headerInfo.cchHeader+1];
						strncpy(dupHeaders, headerInfo.pszHeader, headerInfo.cchHeader);
						headerInfo.pszHeader = dupHeaders;
						((char*)headerInfo.pszHeader)[headerInfo.cchHeader] = '\0';
	
						mData += headerInfo.cchHeader;
						headerInfo.fKeepConn = false;//true;
						if (!pECB->ServerSupportFunction(pECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER_EX, &headerInfo, NULL, 0))
						{
							DWORD err = GetLastError();
							send_error( pECB, "Server returned malformed header." );
							delete [] headerInfo.pszHeader;
							free(data);
							release_connection(sock, true);
							return HSE_STATUS_ERROR;
						}
						delete [] headerInfo.pszHeader;
						dwBytes -= (mData - data);
						sentHeader = true;
					}
					if (dwBytes > 0)
						pECB->WriteClient( pECB->ConnID, mData, &dwBytes, HSE_IO_SYNC | HSE_IO_NODELAY );
					break;
				}
			case cmdCloseReq:
				reqOpen = 0;
				release_connection(sock);
				sock = INVALID_SOCKET;
				break;
			default:
				break;
		}
		free(data);
		data = NULL;
	}
 
	if ( sock != INVALID_SOCKET && read != READ_OK ) // error from the read; close this connection
	{
		release_connection(sock, true);
		sock = INVALID_SOCKET;

		if ( read == READ_ERROR && openTries++ < (int)gCacheSize )
		{
			printf("Retrying connection\n");
			goto open_con;
		}
		if ( read == READ_TIMEOUT )
			send_error( pECB, "Connection timed out." );
		else
			send_error( pECB, "Failed to open connection." );
		return HSE_STATUS_ERROR;
	}
	if ( sock != INVALID_SOCKET )
		release_connection(sock);
	DWORD dwState = HSE_STATUS_SUCCESS;
	pECB->ServerSupportFunction(pECB->ConnID, HSE_REQ_DONE_WITH_SESSION, &dwState, NULL, 0);
	return HSE_STATUS_SUCCESS;
}

void send_error( EXTENSION_CONTROL_BLOCK* pECB, const char * msg )
{
	const char * kClosePage = "</html><body>";
	DWORD dwBytes = strlen( kLassoCommError );
	char buffer[256];

	sprintf( buffer, "Content-Type: text/html\r\nContent-Length: %d\r\n\r\n", strlen( kLassoCommError ) + strlen(msg) + strlen(kClosePage) );

	dwBytes = strlen(buffer);

	pECB->ServerSupportFunction( pECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER, NULL, &dwBytes, (unsigned long*)buffer );

	dwBytes = strlen( kLassoCommError );

	pECB->WriteClient( pECB->ConnID, (void*)kLassoCommError, &dwBytes, HSE_IO_SYNC );
	
	dwBytes = strlen( msg );

	pECB->WriteClient( pECB->ConnID, (void*)msg, &dwBytes, HSE_IO_SYNC );
	
	dwBytes = strlen( kClosePage );

	pECB->WriteClient( pECB->ConnID, (void*)kClosePage, &dwBytes, HSE_IO_SYNC );
}

int GetPOST( int sock, EXTENSION_CONTROL_BLOCK* pECB, LP8POSTReader * postReader )
{
	try
	{
		unsigned long nBytes = pECB->cbTotalBytes - pECB->cbAvailable; // total bytes to read

		if ( pECB->cbAvailable > 0 )
		{
			postReader->AddToBuffer( (const char*)pECB->lpbData, pECB->cbAvailable );
#ifdef DUMP_POST
			DWORD s = pECB->cbAvailable;
			DWORD scratch;
			WriteFile(gDumpFile, &s, sizeof(s), &scratch, NULL);
			WriteFile(gDumpFile, (const char*)pECB->lpbData, pECB->cbAvailable, &s, NULL);
#endif
		}

		if ( nBytes > 0 )
		{
			char * buffer = new char[nBytes+1];
			while( nBytes > 0 )
			{
				unsigned long buffSize = nBytes;

				if ( pECB->ReadClient( pECB->ConnID, buffer, &buffSize ) )
				{
					nBytes -= buffSize;

					try
					{
					postReader->AddToBuffer( buffer, buffSize );
					}
					catch(...)
					{
						//log_message(sock, "EXCEPTION!!!!!");
						// I was not able to track why I get an exception here sometimes
						// retrying usually succeeds
						// !FIX!
					}
#ifdef DUMP_POST
					DWORD s;
					WriteFile(gDumpFile, &buffSize, sizeof(buffSize), &s, NULL);
					WriteFile(gDumpFile, buffer, buffSize, &s, NULL);
#endif
				}
				else
				{
#ifdef DUMP_POST
					DWORD s;
					char tmp[256];
					sprintf(tmp, "\r\nFAILED: %d\r\n", GetLastError());
					WriteFile(gDumpFile, tmp, strlen(tmp), &s, NULL);
#endif
					postReader->Reset( );
					return 0; // failed
				}
			}
			delete [] buffer;
		}
	}
	catch(...)
	{
		//log_message(sock, "EXCEPTION!!!!!");
		// I was not able to track why I get an exception here sometimes
		// retrying usually succeeds
		// !FIX!
	}
#ifdef DUMP_POST
	CloseHandle(gDumpFile);
#endif
	return 1;
}

int GetParamSize( EXTENSION_CONTROL_BLOCK* pECB, char* szVarName )
{
	int nLen = 0;
	char* temp = NULL;
	DWORD dwErr = 0;

	pECB->GetServerVariable( pECB->ConnID, szVarName, &temp, (unsigned long*)&nLen );

	dwErr = GetLastError();

	switch( dwErr )
	{
		case ERROR_INSUFFICIENT_BUFFER:
			SetLastError( 0 );
			return nLen;
		case ERROR_MORE_DATA:
			SetLastError( 0 );
			return DEFAULT_BUFF;
		default:
			return 0;
	}
}

DWORD GetParam( int sock, EXTENSION_CONTROL_BLOCK* pECB, RequestParamKeyword word, const char* param, char** data, int* outLen, const LP8POSTReader * reader )
{
	*data = NULL;

	switch( word )
	{
		case rpPathArgKeyword:
		{
			// First call to get length for proper size allocation
			if( (*outLen = GetParamSize( pECB, "PATH_INFO" ) ) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "PATH_INFO", *data, (unsigned long*)&outLen );
				--(*outLen); // no NULL
			}
			break;
		}
		case rpSearchArgKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "QUERY_STRING" ) ) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "QUERY_STRING", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpAuthorizationKeyword:
		{
			char* temp		= NULL;
			char* ptrpos	= NULL;
			int tempSize	= 0;
			
			if( (tempSize = GetParamSize( pECB, "ALL_HTTP" )) > 0 )
			{
				temp = (char*)malloc( tempSize );
				pECB->GetServerVariable( pECB->ConnID, "ALL_HTTP", temp, (unsigned long*)&tempSize );

				if( (ptrpos = strstr( temp, "LASSOAUTHORIZATION" )) != NULL )
				{
					char* ptrlinefeed = NULL;

					ptrpos = ( strchr( ptrpos, ':' ) + 1 );
					ptrlinefeed = strchr( ptrpos, '\n' );
					*data = (char*)malloc( (ptrlinefeed - ptrpos) + 1 );
					memcpy( *data, ptrpos, (ptrlinefeed - ptrpos) );
					(*data)[ptrlinefeed - ptrpos] = '\0';
					
					*outLen = strlen( *data ) + 1;
				}
				free( temp );
			}			
			break;
		}
		case rpUserKeyword:
		{
			char* temp		= NULL;
			char* ptrpos	= NULL;
			int tempSize	= 0;
			
			if( (tempSize = GetParamSize( pECB, "ALL_HTTP" )) > 0 )
			{
				temp = (char*)malloc( tempSize );
				pECB->GetServerVariable( pECB->ConnID, "ALL_HTTP", temp, (unsigned long*)&tempSize );

				if( (ptrpos = strstr( temp, "LASSOUSERNAME" )) != NULL )
				{
					char* ptrlinefeed = NULL;

					ptrpos = ( strchr( ptrpos, ':' ) + 1 );
					ptrlinefeed = strchr( ptrpos, '\n' );
					*data = (char*)malloc( (ptrlinefeed - ptrpos) + 1 );
					memcpy( *data, ptrpos, (ptrlinefeed - ptrpos) );
					(*data)[ptrlinefeed - ptrpos] = '\0';

					*outLen = strlen( *data ) + 1;
				}
				
				free( temp );
			}

			break;
		}
		case rpPasswordKeyword:
		{
			char* temp		= NULL;
			char* ptrpos	= NULL;
			int tempSize	= 0;
			
			if( (tempSize = GetParamSize( pECB, "ALL_HTTP" )) > 0 )
			{
				temp = (char*)malloc( tempSize );
				pECB->GetServerVariable( pECB->ConnID, "ALL_HTTP", temp, (unsigned long*)&tempSize );

				if( (ptrpos = strstr( temp, "LASSOPASSWORD" )) != NULL )
				{
					char* ptrlinefeed = NULL;

					ptrpos = ( strchr( ptrpos, ':' ) + 1 );
					ptrlinefeed = strchr( ptrpos, '\n' );
					*data = (char*)malloc( (ptrlinefeed - ptrpos) + 1 );
					memcpy( *data, ptrpos, (ptrlinefeed - ptrpos) );
					(*data)[ptrlinefeed - ptrpos] = '\0';
					
					*outLen = strlen( *data ) + 1;
				}

				free( temp );
			}

			break;
		}
		case rpFromUser:
			break;
		case rpAddressKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "REMOTE_HOST" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "REMOTE_HOST", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpMethodKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "REQUEST_METHOD" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "REQUEST_METHOD", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpServerName:
		{
			if( (*outLen = GetParamSize( pECB, "SERVER_NAME" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "SERVER_NAME", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpServerPort:
		{
			if( (*outLen = GetParamSize( pECB, "SERVER_PORT" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "SERVER_PORT", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpScriptName:
		{
			if( (*outLen = GetParamSize( pECB, "SCRIPT_NAME" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "SCRIPT_NAME", *data, (unsigned long*)outLen ); //Might need to change
				--(*outLen);
			}
			break;
		}
		case rpContentType:
		{
			if( (*outLen = GetParamSize( pECB, "CONTENT_TYPE" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "CONTENT_TYPE", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpContentLength:
		{
			if( (*outLen = GetParamSize( pECB, "CONTENT_LENGTH" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "CONTENT_LENGTH", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpReferrerKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "HTTP_REFERER" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "HTTP_REFERER", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpUserAgentKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "HTTP_USER_AGENT" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "HTTP_USER_AGENT", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpActionKeyword:
		case rpActionPathKeyword:
			break;
		case rpClientIPAddress:
		{
			if( (*outLen = GetParamSize( pECB, "REMOTE_ADDR" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "REMOTE_ADDR", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpFullRequestKeyword:
		{
			if( (*outLen = GetParamSize( pECB, "ALL_HTTP" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "ALL_HTTP", *data, (unsigned long*)outLen );
				--(*outLen);
			}
			break;
		}
		case rpRealmsKeyword:
		case rpCurrentRealmKeyword:
			break;
		case rpCookiesKeyword:
		{			
			if( (*outLen = GetParamSize( pECB, "HTTP_COOKIE" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "HTTP_COOKIE", *data, (unsigned long*)outLen);
				--(*outLen);
			}

			break;
		}
		case rpSessionRootPath:
		{
			if( (*outLen = GetParamSize( pECB, "PATH_INFO" )) > 0 )
			{
				*data = (char*)malloc( *outLen );
				pECB->GetServerVariable( pECB->ConnID, "PATH_INFO", *data, (unsigned long*)outLen );
				char * found = strrchr(*data, '/');
				if ( found != NULL )
				{
					int num = (found-(*data))+1;
					*outLen = num;
				}
			}
			break;
		}
		case rpResolveFilePath:
		{
			*outLen = strlen(param)+1;			

			HSE_URL_MAPEX_INFO info;
			BOOL b = pECB->ServerSupportFunction( pECB->ConnID, HSE_REQ_MAP_URL_TO_PATH_EX, (void*)param, (unsigned long*)outLen, (unsigned long*)&info );
			if ( b )
			{
				*outLen = strlen(info.lpszPath);
				*data = (char*)malloc( *outLen + 1 );
				strcpy(*data, info.lpszPath);
			}
			else
			{
				*data = (char*)malloc(1);
				(*data)[0] = '\0';
				*outLen = 1;		
				return osErrFileNotFound;
			}			
			break;
		}
		case rpWebFileData:
		{
			if ( param != NULL )
			{
				if( strlen( param ) < PATH_LEN )
				{
					int fWebFile = 0;
					HSE_URL_MAPEX_INFO urlInfo;
					char szPath[PATH_LEN] = "";

					if( reader && reader->IsUploadedFile(param) )
					{
						strcpy( urlInfo.lpszPath, param );
					}
					else
					{
						if( param[0] != '/' )
						{
							int len = PATH_LEN-1;
							char* found = NULL;

							pECB->GetServerVariable( pECB->ConnID, "PATH_INFO", &szPath[0], (unsigned long*)&len );
							szPath[len] = '\0';
							found = strrchr( szPath, '/' );

							if( found )
								*(found + 1) = '\0';
						}

						strcat( szPath, param );
						*outLen = strlen( szPath );

						pECB->ServerSupportFunction( pECB->ConnID, HSE_REQ_MAP_URL_TO_PATH_EX, szPath, (unsigned long*)outLen, (unsigned long*)&urlInfo );			
					}

					fWebFile = open( urlInfo.lpszPath, _O_RDONLY | O_BINARY );

					if( fWebFile != -1 )
					{
						struct _stat st;
						int res = 0;

						_fstat( fWebFile, &st );
						*outLen = st.st_size;
						*data = (char*)malloc( (*outLen)+1 );
						res = read( fWebFile, *data, st.st_size );

						// Need to ensure not bigger than *data
						if( res > -1 )
							(*data)[res] = '\0';
						else
							(*data)[0] = '\0';

						close( fWebFile );
					}
					else
					{
						*data = (char*)malloc(1);
						(*data)[0] = '\0';
						*outLen = 1;		
						return osErrFileNotFound;
					}

				}
			}
			break;
		}
		case rpIndexFile:
		default:
			break;		
	}

	if ( *data == NULL )
	{
		*data = (char*)malloc(1);
		(*data)[0] = '\0';
		*outLen = 1;
	}
	
	return GetLastError();
}



void release_connection( int sock, bool justKillIt )
{
	if ( !justKillIt && gCacheSize > 0 )
	{
		EnterCriticalSection(&gSocketCacheLock);
		if ( gSocketCache.size() < gCacheSize )
		{
			gSocketCache.push_back(sock);
			sock = INVALID_SOCKET;
		}
		LeaveCriticalSection(&gSocketCacheLock);
	}
	if ( sock != INVALID_SOCKET ) // just close it
	{
		shutdown(sock, SD_BOTH);
		closesocket(sock);
	}
}

int open_connection()
{
	int s = INVALID_SOCKET;
	
	// can we find a connection in the cache?
	if ( gCacheSize > 0 )
	{
		EnterCriticalSection(&gSocketCacheLock);
		if ( gSocketCache.size() > 0 )
		{
			s = gSocketCache.back();
			gSocketCache.pop_back();
		} 
		LeaveCriticalSection(&gSocketCacheLock);
	}
	/*
	open a socket and connect
	need to add support for resolving hostnames
	*/	
	if ( s != INVALID_SOCKET )
		return s;
	if( (s = socket(AF_INET, SOCK_STREAM, 0)) != INVALID_SOCKET )
	{		
		if( connect(s, (struct sockaddr*)&gLassoAddr, sizeof(struct sockaddr_in)) == 0 )
		{
			return s;
		}
		release_connection(s, true);
	}

	s = INVALID_SOCKET;

	return s;	
}

// Hard coded conf. path for the moment.  may use registry to store this value
#define LASSOISAPICONF "Lasso8ISAPI.conf"

///////////////////////////////////////////////////////////////////////////////
//							Configuration File Functions
//
///////////////////////////////////////////////////////////////////////////////

int get_conf_value( char* szKey, char* ptrValue )
{
	int fConf = 0;
	char* data = NULL;
	char key[256];

	if( (fConf = _open( LASSOISAPICONF, _O_RDONLY )) != -1 )
	{
		int x = 0;
		int red = 0;
		struct _stat st;
		_fstat( fConf, &st );
		
		data = (char*)malloc( st.st_size + 1 );
		red = read( fConf, data, st.st_size );
		data[red] = '\0';

		while( data[x] != '\0' )
		{
			if( data[x] == '#' )
			{	
				while( data[x] != '\n' && data[x] != '\0' )
					x++;
			}
			else
			{
				int i = 0;

				for( ; data[x] != ' '; x++, i++ )
					key[i] = data[x];

				key[i] = '\0';
				x++;

				if( !strcmp( key, szKey ) )
				{
					for( i = 0; data[x] != '\n' && data[x] != '\0'; x++, i++ )
					{
						ptrValue[i] = data[x];
					}

					ptrValue[i] = '\0';
					_close(fConf);
					free(data);

					return 1;
				}
				else
				{
					while( data[x] != '\n' && data[x] != '\0' )
						x++;
				}
			}

			x++;
		}

		free(data);
		_close(fConf);
	}

	return 0;
}
