#include "LassoWebSTAR.h"

#include "Parameters.h"
#include "StarNine.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <syslog.h>
#include <stdio.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include "Base64.h"


#include "../../LassoService.8/LassoProto.h"
#include "../../LassoService.8/LassoProtoUtils.h"
#include <pthread.h>
#include <vector.h>

char * kPluginName = "Lasso 8.5 Connector";
char * kConfFile = "/etc/lasso/lasso8webstar.conf";
char * kServer = "server";
char * kPort = "port";
char * kCacheSize = "concachesize";

static char * gServerPath = NULL;

static short gPort = 14552;
static char gAddress[128] = "127.0.0.1";

unsigned int gCacheSize = 20; // configurable
std::vector<int> gSocketCache;
pthread_mutex_t gSocketCacheLock;


#define DEFAULT_BUFFER 4096
#define LITTLE_BUFF_SIZE 1024
#define WS_MAX_POSTLEN	4096 //24576
#define INVALID_SOCKET (-1)

//#define DO_SYSLOG 1

WSAPI_ErrorCode WSAPI_Dispatch (WSAPI_CommandPBPtr pb) {
	WSAPI_ErrorCode retval = WSAPI_I_NoErr ;
	switch(pb->command) {
		case WSAPI_Run:
			retval = HandleRun(pb) ;
			break;
		case WSAPI_Register:
			HandleRegister(pb) ;
			break;
		case WSAPI_Init:
			HandleInit(pb) ;
			break;
		case WSAPI_Shutdown:
			HandleShutdown(pb) ;
			break;
		case WSAPI_AccessControl :
		case WSAPI_Authenticate :
		case WSAPI_Idle:
		case WSAPI_Emergency:
		case WSAPI_ServerStateChanged:
		case WSAPI_Filter:
		case WSAPI_Event:
		case WSAPI_UIActivate:
		case WSAPI_ListenComplete:
		case WSAPI_Logging:
		case WSAPI_Route:
		case WSAPI_FileInfo:
		case WSAPI_OutOfContext:
		default:
			retval = WSAPI_E_MessageNotHandled ;
			break;
	}

	return retval ;
}


static WSAPI_ErrorCode HandleRegister(WSAPI_CommandPBPtr pb) {
	WSAPI_ErrorCode werr = WSAPI_I_NoErr;
	
	pb->param.init.capabilities = capCGI | capChunkedData | capPreemptive ;
	strcpy(pb->param.init.pluginName, kPluginName);
	strcpy(pb->param.init.version, "5.8.5.0");
	werr = WSAPI_RegisterAction(pb, "LASSO8", kPluginName);
	
	if (!werr)
	{
		werr = WSAPI_RegisterSuffix(pb, "LASSO8", ".lasso", '*   ', '*   ', "text/html");
		if (!werr)
			werr = WSAPI_RegisterSuffix(pb, "LASSO8", ".LassoApp", '*   ', '*   ', "text/html");
	}
	
	return werr;
}


static WSAPI_ErrorCode HandleInit(WSAPI_CommandPBPtr pb) {
	srandom(time(NULL));
#ifdef DO_SYSLOG
	openlog("lassowebstar", LOG_CONS | LOG_NDELAY | LOG_PID, LOG_LOCAL0);
#endif
	size_t tlen;
	WSAPI_ErrorCode err = NewWSAPIParameter(pb, piServerDirectoryPath, NULL, (void**)&gServerPath, &tlen, 1);
	if (gServerPath != NULL)
		gServerPath[tlen] = '\0';
		
	pthread_mutex_init(&gSocketCacheLock, NULL);

	char buff[256];
	char port[128] = "14552";
	char cachesize[128] = "20";
	char * tmp = NULL;
	FILE * conf = fopen(kConfFile, "r");
	
	if ( conf != NULL )
	{
		while( (tmp = fgets(buff, 255, conf)) != NULL )
		{
			if ( *tmp == '#' )
				continue;
			
			while(*tmp != '\0' && isspace(*tmp))
				++tmp;
			if ( *tmp != '\0' )
			{
				if ( strncasecmp(tmp, kServer, strlen(kServer)) == 0 )
				{
					int count = 0;
					tmp += strlen(kServer);
					while(*tmp != '\0' && isspace(*tmp))
						++tmp;
					while(*tmp != '\0')
						gAddress[count++] = *tmp++;
					gAddress[count] = '\0';
				}
				else if ( strncasecmp(tmp, kPort, strlen(kPort)) == 0 )
				{
					int count = 0;
					tmp += strlen(kPort);
					while(*tmp != '\0' && isspace(*tmp))
						++tmp;
					while(*tmp != '\0')
						port[count++] = *tmp++;
					port[count] = '\0';	
					gPort = (short)atoi(port);
				}
				else if ( strncasecmp(tmp, kCacheSize, strlen(kCacheSize)) == 0 )
				{
					int count = 0;
					tmp += strlen(kCacheSize);
					while(*tmp != '\0' && isspace(*tmp))
						++tmp;
					while(*tmp != '\0')
						cachesize[count++] = *tmp++;
					cachesize[count] = '\0';	
					gCacheSize = (short)atoi(cachesize);
				}
			}
		}
		fclose(conf);
	}
	
	return err;
}


							
static WSAPI_ErrorCode HandleRun(WSAPI_CommandPBPtr pb) {
	WSAPI_ErrorCode werr = WSAPI_I_NoErr; //WSAPI_E_MessageNotHandled ;

	int res = 0;
	int openTries = 0;
	char * data = NULL;
	char * fullReq = NULL;
	LPCommandBlock block;
	int reqOpen = 1;
	unsigned int timeout = 300;
	int read = 0;
	int sock = INVALID_SOCKET;
	char * contentType = NULL;
	size_t tempLen = 0;
	

	NewWSAPIParameter(pb, piFullRequestKeyword, NULL, (void**)&fullReq, &tempLen, 1);
	if ( fullReq != NULL )
	{
		//syslog(LOG_INFO, "Full Request (len=%d): %s", tempLen, fullReq);
		fullReq[ (int)tempLen ] = '\0';
		if ( fullReq == icstrstr( fullReq, "OPTIONS" ) )
		{
			WSAPI_FreeMemory (pb, fullReq);
			return WSAPI_E_MessageNotHandled;
		}
		WSAPI_FreeMemory (pb, fullReq);
	}
	
	tempLen = 0;
	NewWSAPIParameter(pb, piContentType, NULL, (void**)&contentType, &tempLen, 0);
	LP8POSTReader reader( contentType );			
	WSAPI_FreeMemory (pb, contentType);

	long int contentLen = 0;
	werr = Get1WSAPIParameter(pb, piContentLength, (void*)&contentLen, &tempLen);
	
	get_post(sock, pb, &reader, contentLen);
	
openCon:
	sock = open_connection(pb);
		
	if ( sock == INVALID_SOCKET )
	{
		return WSAPI_E_MessageNotHandled;
	}
						
	/* start reading command blocks from Lasso */
	while ( reqOpen == 1 && (read = read_block(sock, &block, &data, &timeout)) == 1 )
	{
		switch( block.fCmd )
		{
			case cmdGetParam:
				{
					char * myData = data;
					int numParams = 0;
					
					// scan to see how many params
					for (int pos = 0; pos < block.fDataSize;)
					{
						++numParams;
						pos += sizeof(RequestParamKeyword);
						
						int dSize = *(int*)(data+pos);
						pos += (ntohl(dSize)+sizeof(int));
					}

					block.fCmd = cmdGetParamRep;
					block.fDataSize = 0;
					block.fResultCode = 0;
					
					struct ptr_size
					{
						char * data;
						int size;
					};

					ptr_size * rec = (ptr_size *)WSAPI_AllocatePoolMemory (pb, numParams*sizeof(ptr_size));

					for (int x = 0; x < numParams; ++x )
					{
						RequestParamKeyword word = (RequestParamKeyword)ntohl(*(RequestParamKeyword*)myData);
						int dLen = ntohl(*(int*)(myData+sizeof(RequestParamKeyword)));

						char * pData = NULL;
						size_t dSize = 0;
						
						myData += (sizeof(RequestParamKeyword) + sizeof(int));
						
						if ( word == rpPostKeyword )
						{
							const std::string * str = reader.GetPostArgs();

							if ( str != NULL )
							{
								pData = (char*)WSAPI_AllocatePoolMemory(pb, str->size());
								dSize = str->size();
								memcpy(pData, str->c_str(), str->size());
							}
						}
						else
						{
							int res = get_param(pb, word, dLen > 0 ? myData : NULL, (void**)&pData, &dSize);
							if ( res < 0 )
								block.fResultCode = res;
						}
						rec[x].data = pData;
						rec[x].size = dSize;
						
						block.fDataSize += (dSize + sizeof(int));
					}
					
					/* alloc a buffer for the data portion of the block*/
					char * pData = (char*)WSAPI_AllocatePoolMemory(pb, block.fDataSize);
					char * ppData = pData; 
					for ( int x = 0; x < numParams; ++x )
					{
						int mSize = htonl(rec[x].size);
						memcpy(ppData, &mSize, sizeof(int));
						memcpy(ppData+sizeof(int), rec[x].data, rec[x].size);
						ppData += (sizeof(int) + rec[x].size);
						
						WSAPI_FreePoolMemory(pb, rec[x].data);
					}
											
					res = send_block(sock, &block, pData);
						
					// dispose rec and each pointer within!
					WSAPI_FreePoolMemory(pb, rec);
					WSAPI_FreePoolMemory(pb, pData);
				}
				break;
			case cmdPushData:
				WSAPI_SendHTTPData(pb, data, block.fDataSize);
				break;
			case cmdCloseReq:
				reqOpen = 0;
				break;

			default:
				break;
		}
		free(data);
		data = NULL;
	}
    if ( sock != INVALID_SOCKET && read != 1 ) // some error
    {   
		release_connection(sock, true);
		sock = INVALID_SOCKET;
	    
		if ( ++openTries < (gCacheSize+5) )
		    goto openCon;
			
	    return WSAPI_E_SocketClosed; //SERVER_ERROR;
    } 
	if ( sock != INVALID_SOCKET )
		release_connection(sock, false);

	if (reader.GotFileUpload())
	{
		short ok = 1; 
		werr = SetWSAPIParameter(pb, piCacheFlush, typeSMInt, NULL, &ok, 2);
		//WSAPI_WriteLogEntry (pb, "Lasso", "Flushing WebStar cache.");
	}
	
	return werr; //OK;
}


static WSAPI_ErrorCode HandleShutdown(WSAPI_CommandPBPtr pb) {
	if (gServerPath != NULL)
		WSAPI_FreeMemory(pb, gServerPath);
	pthread_mutex_destroy(&gSocketCacheLock);
	return WSAPI_I_NoErr ;
}

void release_connection( int sock, bool justCloseIt )
{
#ifdef DO_SYSLOG
	syslog(LOG_INFO, "release: %d %d", sock, (int)justCloseIt);
#endif
	if ( !justCloseIt && gCacheSize > 0 )
	{
		pthread_mutex_lock(&gSocketCacheLock);
		if ( gSocketCache.size() < gCacheSize )
		{
			gSocketCache.push_back(sock);
			sock = INVALID_SOCKET;
		}
		pthread_mutex_unlock(&gSocketCacheLock);
	}
	if ( sock != INVALID_SOCKET)
	{
		close(sock);
	}
}

int open_connection(WSAPI_CommandPBPtr pb)
{
#ifdef DO_SYSLOG
	syslog(LOG_INFO, "openconn");
#endif
	int s = INVALID_SOCKET;
	if ( gCacheSize > 0 )
	{
		pthread_mutex_lock(&gSocketCacheLock);
		if ( gSocketCache.size() > 0 )
		{
			s = gSocketCache.back();
			gSocketCache.pop_back();
		}			
		pthread_mutex_unlock(&gSocketCacheLock);
	}
	if ( s != INVALID_SOCKET )
		return s;

    sockaddr_in addr;
	addr.sin_family = AF_INET;
    addr.sin_port = htons(gPort);
    addr.sin_addr.s_addr = inet_addr(gAddress);
    addr.sin_zero[0] = addr.sin_zero[1] = addr.sin_zero[2] = addr.sin_zero[3] = 0;
	
    if ( (s = socket(AF_INET, SOCK_STREAM, 0)) != INVALID_SOCKET )
	{
		if ( connect(s, (sockaddr*)&addr, sizeof(sockaddr_in)) == 0 )
			return s;
		release_connection(s, true);
    }
    return INVALID_SOCKET;	 
}


WSAPI_ErrorCode get_param( WSAPI_CommandPBPtr pb, RequestParamKeyword word, const char * param, void ** data, size_t * outLen )
{
	WSAPI_ErrorCode werr = WSAPI_I_NoErr;
	switch( word )
	{
		case rpPathArgKeyword:
			NewWSAPIParameter(pb, piPathArgKeyword, NULL, data, outLen, 0);
			break;
		case rpSearchArgKeyword:
			NewWSAPIParameter(pb, piSearchArgKeyword, NULL, data, outLen, 0);
			break;
		case rpAuthorizationKeyword:
			{
				char * ptrpos, *mainPtr = NULL;
				NewWSAPIParameter(pb, piFullRequestKeyword, NULL, (void**)&mainPtr, outLen, 0);
				if( mainPtr != NULL )
				{
					if ((ptrpos = icstrstr( mainPtr, "Authorization: " )) != NULL )
					{
						ptrpos += 15;
						char * term = strstr(ptrpos, "\r\n");
						if ( term != NULL )
							*term = '\0';
						*outLen = strlen(ptrpos);
						*data = (char*)WSAPI_AllocateMemory(pb, (*outLen)+1);
						strcpy((char*)*data, ptrpos);
					}
					else
					{
						*outLen = 0;
					}
					WSAPI_FreeMemory(pb, mainPtr);
				}
			}
			break;
		case rpUserKeyword:
			//NewWSAPIParameter(pb, piUserKeyword, NULL, data, outLen, 0);
			{
				char* temp		= NULL;
				char* ptrpos	= NULL;
				int tempSize	= 0;
				
				if ( (ptrpos = get_auth(pb, &temp)) != NULL)
				{
					if ( GetUsername((unsigned char*)ptrpos, (unsigned char*)ptrpos) )
					{
						tempSize = strlen(ptrpos);
						*data = (char*)WSAPI_AllocateMemory(pb, tempSize + 1 );
						if ( *data != NULL )
						{
							strncpy((char*)*data, ptrpos, tempSize);
							((char*)*data)[tempSize] = '\0';
							*outLen = tempSize;
						}
					}
				}
				WSAPI_FreeMemory( pb, temp );
			}
			break;
		case rpPasswordKeyword:
			//NewWSAPIParameter(pb, piPasswordKeyword, NULL, data, outLen, 0);
			{
				char* temp		= NULL;
				char* ptrpos	= NULL;
				int tempSize	= 0;
				
				if ( (ptrpos = get_auth(pb, &temp)) != NULL)
				{
					if ( GetPassword((unsigned char*)ptrpos, (unsigned char*)ptrpos) )
					{
						tempSize = strlen(ptrpos);
						*data = (char*)WSAPI_AllocateMemory(pb, tempSize + 1 );
						if ( *data != NULL )
						{
							strncpy((char*)*data, ptrpos, tempSize);
							((char*)*data)[tempSize] = '\0';
							*outLen = tempSize;
						}
					}
				}
				WSAPI_FreeMemory( pb, temp );
			}
			break;
		case rpFromUser:
			NewWSAPIParameter(pb, piFromUser, NULL, data, outLen, 0);
			break;
		case rpAddressKeyword:
			NewWSAPIParameter(pb, piAddressKeyword, NULL, data, outLen, 0);
			break;
		case rpPostKeyword:
			//NewWSAPIParameter(pb, piPostKeyword, NULL, data, outLen, 0);
			break;
		case rpMethodKeyword:
			NewWSAPIParameter(pb, piMethodKeyword, NULL, data, outLen, 0);
			break;
		case rpServerName:
			NewWSAPIParameter(pb, piServerName, NULL, data, outLen, 0);
			break;
		case rpServerPort:
			{
				short port = 0;
				size_t tLen = sizeof(short);
				werr = Get1WSAPIParameter(pb, piServerPort, (void*)&port, &tLen);
				if (werr == WSAPI_I_NoErr && tLen > 0)
				{
					*data = WSAPI_AllocateMemory(pb, 20);
					if (*data != NULL) {
						sprintf((char*)*data, "%d", (int)port);
						*outLen = strlen((char*)*data);
					}
				}
				break;
			}
		case rpScriptName:
			NewWSAPIParameter(pb, piScriptName, NULL, data, outLen, 0);
			
			// default file has been mapped by WS?
			if ( ((char*)*data)[*outLen-1] == '/' ) 
			{
				char * temp = NULL;
				size_t tLen;
						
				werr = NewWSAPIParameter(pb, piURLPhysicalPath, NULL, (void**)&temp, &tLen, 0);
				if ( werr == WSAPI_I_NoErr && temp != NULL )
				{
					char * found = strrchr(temp, '/');
					if ( found != NULL)
					{
						++found;
						tLen -= (found - temp);
					}
					else
					{
						found = temp;
					}
					char * buf = (char*)WSAPI_ReallocateMemory(pb, *data, (long int)(*outLen + tLen));
					if ( buf != NULL )
					{
						strncpy( (char*)(buf + *outLen), found, tLen );
						*data = buf;
						*outLen += tLen;
					}
					WSAPI_FreeMemory(pb, temp);	
				}
			}
			
#ifdef DO_SYSLOG
			syslog(LOG_INFO, "scriptname: %s", *data);
#endif			
			break;
		case rpContentType:
			NewWSAPIParameter(pb, piContentType, NULL, data, outLen, 0);
			break;
		case rpContentLength:
			{
				int cLen = 0;
				size_t tLen = sizeof(int);
				Get1WSAPIParameter(pb, piContentLength, (void*)&cLen, &tLen);
				*data = (char*)WSAPI_AllocateMemory(pb, 20);
				if (*data != NULL) {
					sprintf((char*)*data, "%d", (int)cLen);
					*outLen = strlen((char*)*data);
				}
			}
			break;
		case rpReferrerKeyword:
			NewWSAPIParameter(pb, piRefererKeyword, NULL, data, outLen, 0);
			break;
		case rpUserAgentKeyword:
			NewWSAPIParameter(pb, piUserAgentKeyword, NULL, data, outLen, 0);
			break;
		case rpActionKeyword:
			NewWSAPIParameter(pb, piActionKeyword, NULL, data, outLen, 0);
			break;
		case rpActionPathKeyword:
			NewWSAPIParameter(pb, piActionPathKeyword, NULL, data, outLen, 0);
			break;
		case rpClientIPAddress:
			{
				in_addr temp;
				size_t	tLen = 4;
				char * ip;
				
				WSAPI_ErrorCode err = Get1WSAPIParameter(pb, piClientIPAddress, (void*)&temp, &tLen);
				if (err == WSAPI_I_NoErr && tLen == 4)
				{
					ip = inet_ntoa(temp);
					*outLen = strlen(ip);
					*data = (char*)WSAPI_AllocateMemory(pb, *outLen);
					if (*data != NULL) {
						strncpy((char*)*data, ip, *outLen);
					}
				}
			}
			break;
		case rpFullRequestKeyword:
			NewWSAPIParameter(pb, piFullRequestKeyword, NULL, data, outLen, 0);
			break;
		case rpRealmsKeyword:
			NewWSAPIParameter(pb, piCurrentRealm, NULL, data, outLen, 0);
			break;
		case rpCurrentRealmKeyword:
			NewWSAPIParameter(pb, piCurrentRealm, NULL, data, outLen, 0);
			break;
		case rpCookiesKeyword:
			{
				char * temp;
				size_t tlen;

				NewWSAPIParameter(pb, piFullRequestKeyword, NULL, (void**)&temp, &tlen, 0);
				char * found = icstrstr(temp, "Cookie: ");
				if ( found != NULL )
				{
					found += 8;
					char * del = strchr(found, '\r');
					if (del == NULL)
						del = (temp + tlen);
					int num = (del-found);
					*data = (char*)WSAPI_AllocateMemory(pb, num+1);
					*outLen = num;
					strncpy((char*)*data, found, num);
					((char*)*data)[num] = '\0';
				}
				WSAPI_FreeMemory(pb, temp);
			}
			break;		
		case rpSessionRootPath:
			{
				NewWSAPIParameter(pb, piScriptName, NULL, data, outLen, 0);
				
				char * path = (char*)*data;
				if ( path[*outLen-1] != '/' )
				{
					char * found = strrchr(path, '/');
					if (found != NULL)
					{
						*outLen = found - path + 1;
					}
				}
			}
#ifdef DO_SYSLOG
			syslog(LOG_INFO, "session root: %s", *data);
#endif	
			break;
		case rpResolveFilePath:
			if ( param != NULL )
			{				
				NewWSAPIParameter(pb, (WSAPI_ParamKeywords)piDestinationPhysicalPath, 
														(char*)param, data, outLen, 0);
				if (*data == NULL)
					break;
					
				if ( (*(char**)data)[0] != '/' ) // not complete
				{
					char path2[1024];
					getcwd(path2, sizeof(path2));
					char * destroy = *(char**)data;
					*data = WSAPI_AllocateMemory(pb, strlen(path2) + 2 + *outLen);
					strcpy(*(char**)data, path2);
					strcat(*(char**)data, "/");
					strcat(*(char**)data, destroy);
					*outLen = strlen(*(char**)data);
					WSAPI_FreeMemory(pb, destroy);
				}
				if ( strcmp(param, "/") == 0 && pb->param.run.role != WSAPI_Index_Role )
				{
					char * tmp234 = strrchr(*(char**)data, (int)'/');
					if ( tmp234 )
					{
						++tmp234;
						*tmp234 = '\0';
						*outLen = strlen(*(char**)data);
					}
				}
			}
#ifdef DO_SYSLOG
			syslog(LOG_INFO, "resolve path: %s %s", *data, param);
#endif	
			break;
		
		case rpWebFileData:
			if ( param != NULL )
			{
				char * path = NULL;
				size_t pathLen;
				
				if ( param[0] != '/' )
				{
					char * rootPath;
					get_param(pb, rpSessionRootPath, NULL, (void**)&rootPath, &pathLen);
					char * tmp = new char[strlen(param) + pathLen + 1];
					strncpy(tmp, rootPath, pathLen);
					strcat(tmp, param);
					NewWSAPIParameter(pb, (WSAPI_ParamKeywords)piDestinationPhysicalPath, 
														tmp, (void**)&path, &pathLen, 0);					
					delete [] tmp;
					WSAPI_FreeMemory(pb, rootPath);
				}
				else
				{
					NewWSAPIParameter(pb, (WSAPI_ParamKeywords)piDestinationPhysicalPath, 
														(char*)param, (void**)&path, &pathLen, 0);
				}
				int f = -1;
				if ( path != NULL )	   	   
					f = open(path, O_RDONLY, 0);
				if ( f != -1 )
				{
					struct stat st;
					
				   ::stat(path, &st);
				   int filelen = st.st_size;

					*data = WSAPI_AllocateMemory(pb, filelen+1);
					int res = read(f, *data, filelen);
					if ( res > -1 )
					{
						(*(char**)data)[res] = '\0';
						*outLen = filelen;
					}
					else
					{
						(*(char**)data)[0] = '\0';
						*outLen = 0;
						werr = WSAPI_E_MessageNotHandled;
					}
					close(f);
				}
				else
				{
					syslog(LOG_INFO, "ERROR OPENING FILE: %s, %s, %d, %s", param, path, errno, strerror(errno));
					werr = WSAPI_E_MessageNotHandled;
				}
				WSAPI_FreeMemory(pb, path);
			}
			break;
		case rpIndexFile:
			NewWSAPIParameter(pb, piIndexFile, NULL, data, outLen, 0);
			break;
		default:
			break;		
	}
/*
	if ( *data == NULL )
	{
		*data = (char*)WSAPI_AllocateMemory(pb, 1);
		(*(char**)data)[0] = '\0';
		*outLen = 1;
	}
*/	return werr;
}



int get_post( int sock, WSAPI_CommandPBPtr pb, LP8POSTReader * reader, int toRead )
{
	WSAPI_ConnectionStatus status;
	WSAPI_ErrorCode werr = WSAPI_HTTPConnectionStatus(pb, &status);

	if ( werr == WSAPI_I_NoErr && (status == WSAPI_IP_Data || status == WSAPI_IP_Open) )
	{
			long int readSize = 0;
/*			 
			if ( toRead < WS_MAX_POSTLEN )
			{
				char * buffer = NULL;
				werr = NewWSAPIParameter(pb, piPostKeyword, NULL, (void**)&buffer, (size_t*)&readSize, 1);
							
				if (werr == WSAPI_I_NoErr)
				{
					buffer[readSize] = '\0';
					reader->AddToBuffer(buffer, readSize);
					WSAPI_FreeMemory(pb, buffer);
				}
				return (werr == WSAPI_I_NoErr);
			}
*/	
			readSize = DEFAULT_BUFFER;
			char * buffer = new char[readSize+1];
			
			while ( (werr = WSAPI_ReadHTTPData(pb, buffer, &readSize)) == WSAPI_I_NoErr && readSize > 0)
			{
				buffer[readSize] = '\0';		
				//if ( readSize > 0 ) 
				//{
					reader->AddToBuffer(buffer, readSize);
					toRead -= readSize;
				//}
				if ( toRead <= 0)
				{
					break;
				}
				readSize = DEFAULT_BUFFER;		
			}
			delete [] buffer;
	}
	return 1;
}

char * get_auth(WSAPI_CommandPBPtr pb, char ** temp)
{
	size_t tempSize = 0;
	char * ptrpos = NULL;
	
	NewWSAPIParameter(pb, piFullRequestKeyword, NULL, (void**)temp, &tempSize, 0);
	if( tempSize > 0 && (ptrpos = icstrstr( *temp, "Authorization: " )) != NULL )
	{
		ptrpos += 15;
		if ( strncasecmp("basic ", ptrpos, 6) == 0 )
		{
			while(*ptrpos && !isspace(*ptrpos))
				++ptrpos;
			if ( *ptrpos == ' ' )
			{ 
				++ptrpos;
				char * term = strstr(ptrpos, "\r\n");
				if ( term != NULL )
					*term = '\0';
				return ptrpos;
			}
		}
	}
	return NULL;
}

char *icstrstr(const char *str, const char *substring)
{
	long	i = 0, j = 0, str_l = strlen(str), substring_l = strlen(substring);
	char	*firstMatch;
	
		firstMatch = NULL;
		for(i = 0; i <= str_l - substring_l && firstMatch == NULL; i++)
		{
			firstMatch = (char *)str + i;
			for(j = 0; j < substring_l; j++)
				if(tolower(substring[j]) != tolower(str[i+j]))
				{
					firstMatch = NULL;
					break;
				}
		}
	
		return firstMatch;
}
