/*	Parameters.c
	
	Tools for setting and retrieving WSAPI parameters.
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "WSAPI.h"
#include "Parameters.h"

#define BIG_BUFFER_SIZE		1024

char	get_buffer[BIG_BUFFER_SIZE];	// we desperately need to get rid of this.  Not preemption safe!!!

//------------------------------------------------------------------------------------
//  NewWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to retrieve W*API parameters, indexed and otherwise. This
// routine allocates the required memory. The caller must de-allocate it.

WSAPI_ErrorCode NewWSAPIParameter (	WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords	which,
									char			  * index,
									void			  ** buffer,
									size_t			  * length,
									size_t				extra )
{
	WSAPI_ErrorCode	err = WSAPI_I_NoErr;
	WSAPI_ErrorCode err2;
	WSAPI_DescPtr	d;
	OSType			dtype;
	unsigned long	dsize;
	int				nullspot = 0 ;
	
	//Load in the parameter descriptor
	if (index == NULL)
		err = WSAPI_GetParameter (pb, which, &d);
	else
		err = WSAPI_GetIndexedParameter (pb, which, (WSAPI_IndexStringPtr) index, &d);
	if (err == WSAPI_I_NoErr) {
		//Get the data size from the descriptor
		err = WSAPI_GetDescriptor (pb, d, &dtype, NULL, &dsize);
		if (err == WSAPI_I_NoErr) {
			//Got the size, now allocate memory
			if(dtype==typeChar)
				nullspot = 1 ;
			*buffer = WSAPI_AllocateMemory(pb, dsize + extra + nullspot);
			if ((err == WSAPI_I_NoErr) && (*buffer != NULL)) {
				//Now load the data into the buffer
				err = WSAPI_GetDescriptor(pb, d, &dtype, *buffer, &dsize);
				//Null terminate char data
				if ((err == WSAPI_I_NoErr) && (dtype == typeChar))
					((char *) *buffer)[dsize] = '\0';
				*length = dsize ;
			} else
				err = WSAPI_E_InsufficientMemory;
		}
		//Dispose of the descriptor
		err2 = WSAPI_DisposeDescriptor (pb, &d);
		if (err == WSAPI_I_NoErr)
			err = err2;
	}
	
	return err;
}

//------------------------------------------------------------------------------------
//  GetWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to retrieve W*API parameters, indexed and otherwise. 
// Up to 'length' bytes of data is copied to the specified buffer. 'length'
// is updated with the actual length of the data returned. 

WSAPI_ErrorCode GetWSAPIParameter (	WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords				which,
									char			  * index,
									void			  * buffer,
									size_t			  * length )
{
	WSAPI_ErrorCode	err = WSAPI_I_NoErr;
	WSAPI_ErrorCode err2;
	WSAPI_DescPtr	d;
	OSType			dtype;
	unsigned long	dsize;
	int			isChar = 0;
	
	//Load in the parameter descriptor
	if (index == NULL)
		err = WSAPI_GetParameter (pb, which, &d);
	else
		err = WSAPI_GetIndexedParameter (pb, which, (WSAPI_IndexStringPtr) index, &d);
	if (err == WSAPI_I_NoErr) {
		//Get the data size from the descriptor
		err = WSAPI_GetDescriptor (pb, d, &dtype, NULL, &dsize);
		if (err == WSAPI_I_NoErr) {
			isChar = (dtype == typeChar) ? 1 : 0;
			//Got the size, check it to see if it fits in buffer
			if (*length >= dsize + isChar) {
				//Now load the data into the buffer
				err = WSAPI_GetDescriptor(pb, d, &dtype, buffer, &dsize);
				//Null terminate char data
				if ((err == WSAPI_I_NoErr) && isChar) {
					((char *) buffer)[dsize] = '\0';
					*length = strlen((const char*)buffer);
				} else
					*length = dsize;
			} else if (isChar && (BIG_BUFFER_SIZE >= (dsize + 1))) {
				err = WSAPI_GetDescriptor(pb, d, &dtype, get_buffer, &dsize);
				if (err == WSAPI_I_NoErr) {
					strncpy((char*)buffer, get_buffer, *length-1);
					((char *) buffer)[*length-1] = '\0';
					*length = strlen((const char*)buffer);
				} else
					*length = dsize;
			} else 
				err = WSAPI_E_InsufficientMemory;
		}
		//Dispose of the descriptor
		err2 = WSAPI_DisposeDescriptor (pb, &d);
		if (err == WSAPI_I_NoErr)
			err = err2;
	}
	
	return err;
}

//------------------------------------------------------------------------------------
//  SetWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to set W*API parameters, indexed and otherwise. The caller 
// must de-allocate the buffer.

WSAPI_ErrorCode SetWSAPIParameter ( WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords				which,
									OSType				dtype,
									char			  * index,
									void			  * buffer,
									size_t				length )
{
	WSAPI_ErrorCode	err = WSAPI_I_NoErr;
	WSAPI_ErrorCode err2;
	WSAPI_DescPtr	d;
	unsigned long	dsize = length;
	
	//Get a new descriptor to hold the value
	err = WSAPI_NewDescriptor (pb, &d, dtype, NULL, dsize);
	if (err == WSAPI_I_NoErr) {
		//Fill in the descriptor with the data
		err = WSAPI_SetDescriptor (pb, d, buffer, dsize);
		if (err == WSAPI_I_NoErr) {
			//Send the parameter to WebSTAR
			if (index == NULL)
				err = WSAPI_SetParameter (pb, which, d);
			else
				err = WSAPI_SetIndexedParameter (pb, which, (WSAPI_IndexStringPtr) index, d);
		}
		//Dispose of the descriptor
		err2 = WSAPI_DisposeDescriptor (pb, &d);
		if (err == WSAPI_I_NoErr)
			err = err2;
	}
	
	return err;
}

//------------------------------------------------------------------------------------
//  New1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed W*API parameters. 
// Allocates new buffer. The caller must de-allocate the buffer.

WSAPI_ErrorCode New1WSAPIParameter (WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords				which,
									void			 ** buffer,
									size_t			  * length )
{
	return NewWSAPIParameter (pb, which, NULL, buffer, length, 0);
}

//------------------------------------------------------------------------------------
//  Get1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed W*API parameters. 
// Up to 'length' bytes of data is copied to the specified buffer. 'length'
// is updated with the actual length of the data returned. 

WSAPI_ErrorCode Get1WSAPIParameter (WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords				which,
									void			  * buffer,
									size_t			  * length )
{
	return GetWSAPIParameter (pb, which, NULL, buffer, length);
}

//------------------------------------------------------------------------------------
//  Set1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to set fixed-length, non-indexed W*API parameters. The caller 
// must de-allocate the buffer.

WSAPI_ErrorCode Set1WSAPIParameter (WSAPI_CommandPBPtr	pb,
									WSAPI_ParamKeywords				which,
									OSType				dtype,
									void			  * buffer,
									size_t				length )
{
	return SetWSAPIParameter (pb, which, dtype, NULL, buffer, length);
}


//------------------------------------------------------------------------------------
//  New1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed, W*API parameters of type char. 
// A new buffer is allocated. The caller must de-allocate the buffer.

WSAPI_ErrorCode New1WSAPICharParameter (WSAPI_CommandPBPtr	pb,
										WSAPI_ParamKeywords				which,
										char			 ** buffer )
{
	size_t buflen;
	
	return New1WSAPIParameter (pb, which, (void**)buffer, &buflen);
}


//------------------------------------------------------------------------------------
//  Get1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed, W*API parameters of type char. 
// Buffer MUST be large enough to hold result. No length checking is performed!!!

WSAPI_ErrorCode Get1WSAPICharParameter (WSAPI_CommandPBPtr	pb,
										WSAPI_ParamKeywords				which,
										char			  * buffer )
{
	size_t buflen = 32768;
	
	return Get1WSAPIParameter (pb, which, buffer, &buflen);
}


//------------------------------------------------------------------------------------
//  Set1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to set fixed-length, non-indexed, W*API parameters of type char. 
// The caller must de-allocate the buffer.
// Notice the trailing NULL is sent as part of the parametr

WSAPI_ErrorCode Set1WSAPICharParameter (WSAPI_CommandPBPtr	pb,
										WSAPI_ParamKeywords				which,
										char			  * buffer )
{
	return Set1WSAPIParameter (pb, which, typeChar, buffer, strlen(buffer)+1);
}


