/*
	File:		W*APIParams.h

	Contains:	xxx put contents here xxx

	Written by:	Mike Haas

	Copyright:	 1998 by StarNine Technologies, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 2/11/98	mdh		merge with Houston library sources

	To Do:
*/

//====================================================================================
// WSAPIParams.h	Copyright  1996, Quarterdeck Corp. Written by Louis Slothouber
//====================================================================================
//
// This library contains a variety of routines that facilitate getting and setting
// W*API parameters.

#pragma once

#include "WSAPI.h"
#include <stdlib.h>

//------------------------------------------------------------------------------------
//  NewWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to retrieve W*API parameters, indexed and otherwise. This
// routine allocates the required memory. The caller must de-allocate it.

WSAPI_ErrorCode NewWSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords		which,	//paramerter ID
											char	  * index,	//index or NULL
											void	 ** buffer,	//address of (char *)
											size_t	  * length,	//length of allocation
											size_t		extra	//extra bytes to allocate
										 );

//------------------------------------------------------------------------------------
//  GetWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to retrieve W*API parameters, indexed and otherwise. 
// Up to 'length' bytes of data is copied to the specified buffer. 'length'
// is updated with the actual length of the data returned. 

WSAPI_ErrorCode GetWSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords		which,
											char	  * index,
											void	  * buffer,
											size_t	  * length
										 );

//------------------------------------------------------------------------------------
//  SetWSAPIParameter
//------------------------------------------------------------------------------------
// General purpose routine to set W*API parameters, indexed and otherwise. The caller 
// must de-allocate the buffer.

WSAPI_ErrorCode SetWSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords		which,
											OSType		dtype,
											char	  * index,
											void	  * buffer,
											size_t		length 
										 );

//------------------------------------------------------------------------------------
//  New1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed W*API parameters. 
// Allocates new buffer. The caller must de-allocate the buffer.

WSAPI_ErrorCode New1WSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords			which,
											void		 ** buffer,
											size_t		  * length
										 );

//------------------------------------------------------------------------------------
//  Get1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed W*API parameters. 
// Up to 'length' bytes of data is copied to the specified buffer. 'length'
// is updated with the actual length of the data returned. 

WSAPI_ErrorCode Get1WSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords			which,
											void		  * buffer,
											size_t		  * length
										 );

//------------------------------------------------------------------------------------
//  Set1WSAPIParameter
//------------------------------------------------------------------------------------
// Specialized routine to set fixed-length, non-indexed W*API parameters. The caller 
// must de-allocate the buffer.

WSAPI_ErrorCode Set1WSAPIParameter (	WSAPI_CommandPBPtr	pb,
											WSAPI_ParamKeywords			which,
											OSType			dtype,
											void		  * buffer,
											size_t			length 
										 );

//------------------------------------------------------------------------------------
//  New1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed, W*API parameters of type char. 
// A new buffer is allocated. The caller must de-allocate the buffer.

WSAPI_ErrorCode New1WSAPICharParameter (	WSAPI_CommandPBPtr	pb,
												WSAPI_ParamKeywords			which,
												char		 ** buffer
										 	);

//------------------------------------------------------------------------------------
//  Get1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to get fixed-length, non-indexed, W*API parameters of type char. 
// Buffer MUST be large enough to hold result. No length checking is performed!!!

WSAPI_ErrorCode Get1WSAPICharParameter (	WSAPI_CommandPBPtr	pb,
												WSAPI_ParamKeywords			which,
												char		  * buffer
										 	);

//------------------------------------------------------------------------------------
//  Set1WSAPICharParameter
//------------------------------------------------------------------------------------
// Specialized routine to set fixed-length, non-indexed, W*API parameters of type char. 
// The caller must de-allocate the buffer.
// Notice the trailing NULL is sent as part of the parametr

WSAPI_ErrorCode Set1WSAPICharParameter (	WSAPI_CommandPBPtr	pb,
												WSAPI_ParamKeywords			which,
												char		  * buffer
											 );

