/*
	File:		StarNine.h

	Contains:	StarNine specific parameters for WSAPI

	Written by:	Joe Kalash

	Copyright:	 1999 by StarNine Technologies, Inc., all rights reserved.

	Change History (most recent first):

	   <1.9>	 3/17/00	mdh		added piRawLogFormat,	piGMTDelta,	piURLFileName and
									piHTTPResponseCode for logging plugin
       <1.8>     11/4/99    cjh     add piIsLogging
       <1.7>     8/20/99    cjh     Added piPeerCertDER; changed PF_AuthClientCert to
                                    PF_RequireClientCert; added comments
       <1.6>     7/30/99    jtk     Add a new parameter to get the plug-in file name. Useful for the
                                    Debug plug-in.
       <1.5>      5/3/99    cjh     Change FunctionPtr to FragFunctionPtr
       <1.4>     4/26/99    jtk     Allow plug-ins to get access to the list of all loaded
                                    fragments.
       <1.3>      3/8/99    cjh     Add PF_IsOn; add PF_3DES_EDE to the PF_Default mask
       <1.2>      3/5/99    jtk     Add the piSSLAvailable parameter.
       <1.1>     2/22/99    jtk     Add the piWNEThreshold parameter.
	   <1.0>	 1/27/99	jtk		first checked in

	To Do:
*/
# ifndef	_STARNINE_H
# define	_STARNINE_H	1

# include	"WSAPI.h"

/*
** StarNine specific extensions to the WebSTAR API.
*/
typedef enum
{
	piSSLInterface			= 'rosl',	// for indexed use only.  The index is a WSAPI_Interface pointer (from piInterfaceList).
										// Used to get and set SSL data
	piBrowserBufferBug		= 'PiBB',	//typeLong - W/O, The server normally looks at the first 256 bytes sent by
										//	the plug-in, to deal with a browser bug that happens when the EOH is on
										//	a 256, 257 or 258 byte boundary. Setting this parameter tells the server
										//	NOT to look at the data for the current connection.
	piDefaultAction			= 'Pdft',	//typeChar - R/W the default action the server will use, rather then serving the file internally.
										// The action must already be defined.
	piByteServerAction		= 'Pbyt',	//typeChar - R/W Action to use if there is a Range header, and the server would normally serve the file.
										// This is used even if piDefaultAction is set. The action must already be defined.
	piCGIPersistent			= 'PCGI',	//typeLong - R/W turn persistent connections on/off for a CGI. This is server wide, not
										// connection based. 
	piSSLPort				= 'Slpt',	//typeShortInteger - R/W IP port for the SSL server. Can not be set to piPort.
	piWNEThreshold			= 'PWNE',	//typeLong - R/W how often to call WaitNextEvent. This isn't in any normal units,
										// it is a count of the number of times to skip WNE calls. The time between WNE calls
										// can be random. Setting this to 0 will mean lots of WNE calls. This can not be set < 0, or > 100.
										// Current default for the server is 30 (for what that is worth).
	piSSLAvailable			= 'pSSL',	// typeLong - R/O What SSL is available. 0 -> NO SSL. 1 -> US SSL. 2 -> Export SSL.
	piFragList				= 'pFRG',	// typeLong - R/O Linked list of loaded fragments (plug-ins)
	piFragVers				= 'pFRv',	// typeLong - R/O Version of the piFragList
	piPluginName			= 'pFil',	// typeChar - R/O file name of the plug-in that WebSTR called. Normally only useful for the Debug plug-in.
										// will return WSAPI_E_RequestFailed if it can't find the name in the WSAPI_CommandPBPtr.
	piPeerCertDER			= 'peer',	// returns DER form of peer certificate.  typeChar, but composed of binary data.
										// if you don't have DER encoding skills/tools, this is probably of no use to you.
	piIsLogging				= 'Ilog',	// returns whether or not logging is actually happening, which may be a separate issue from
										// whether or not logging is turned on as a matter of preference.  Set to 0 to temporarily
										// suspend logging, then set to any other value to turn logging back on.  Get returns
										// 1 for active logging, 0 for suspended logging.
	piRawLogFormat			= 'rlft',	// typeChar - R/O returns the raw log format for logging plugin
	piGMTDelta				= 'gmtd',	// typeLong - R/O returns the gmtDelta global for logging plugin
	piURLFileName			= 'urlf',	// typeLong - R/O returns the gmtDelta global for logging plugin
	piHTTPResponseCode		= 'resp',	// typeShortInteger - R/O returns the httpResponseCode
	
	piPAppResRef			= 'ARes',	// typeShortInteger - the once-secret parameter containing the reference to the server's settings resource file

	// VHost and settings related params
	piVHostID				= 'vhid',	// typeLongInteger - ID of the current virtual host
	piVHostName				= 'vhui',	// typeChar - the current virtual host's UI name
	piVHostList				= 'vhlt',	// typeChar - list of all vhost names and IDs *( VHostName TAB VHostID CR )
										// eg: "Joe's Bar\t5\rCool Store Inc.\t66\r"
	piSettingsData			= 'pref',	// typeChar - the settings data for the current plug-in from the server's settings file.
	piAdminInterface		= 'admn',	// void (*AMP_EntryPoint)(void*) - entry point for the admin interface

	// new miscellaneous parameters
	piDestinationPhysicalPath = 'dest',	// indexed only - returns the physical location of the file that matches the
										// URI indicated by the index.  (eg: use this if you want to find out where
										// the file for /somefolder/somefile.txt would exist.)
	piRuntimeUserID			= 'rusr',	// The runtime user id
	piRuntimeGroupID		= 'rgrp',	// The runtime group id
	piFinalData 			= 'dfin'	// the next call to WSAPI_WriteHTTPData will contain the last piece of data that
										// will be sent for this request.  You can use any non-NULL value for the descriptor
										// since it is never dereferenced.  Using this allows the server to optimize the number
										// of network writes required, espeically if you are sending all of your data at once.
} StarNine_WSAPI_Extensions;

/*
** Policy flags for an SSL connection.
*/
typedef unsigned long PolicyFlags ;

#define PF_RequestClientCert		0x00000001	// ask the client for its certificate
#define PF_RequireClientCert		0x00000002	// fail the SSL handshake if the certificate is not forthcoming.
												// this is useless if PF_RequestClientCert is off.
#define PF_AllowSSL2				0x00000004
#define PF_AllowSSL3				0x00000008
#define PF_AllowTLS					0x00000010	// for future use
#define PF_MAC_Only					0x00000100	// SSL_RSA_WITH_NULL_SHA
												// SSL_RSA_WITH_NULL_MD5
#define PF_RC4_40					0x00000200	// SSL_RSA_EXPORT_WITH_RC4_40_MD5
#define PF_DES_40					0x00000400	// SSL_RSA_EXPORT_WITH_DES40_CBC_SHA
#define PF_DES_56					0x00010000	// SSL_RSA_WITH_DES_CBC_SHA
#define PF_3DES_EDE					0x00020000	// SSL_RSA_WITH_3DES_EDE_CBC_SHA
#define PF_RC4_128					0x00040000	// SSL_RSA_WITH_RC4_128_SHA
												// SSL_RSA_WITH_RC4_128_MD5
#define PF_IDEA_128					0x00080000	// SSL_RSA_WITH_IDEA_CBC_SHA, not yet widely supported in clients


#define PF_ExportMask				0x0000FFFF	// used to eliminate all non-exportable ciphers
#define PF_IsOn			(PF_AllowSSL2|PF_AllowSSL3)
#define PF_Default		(PF_AllowSSL2|PF_AllowSSL3|PF_RC4_40|PF_DES_40|PF_DES_56|PF_3DES_EDE|PF_RC4_128)

/*
** WSAPI_MultiHomeSslStruct
**
**	A pointer to this structure is what is passed to/from the server using the piSSLInterface parameter.
*/
typedef struct
{
	long			isSSL;					// is SSL turned on for this interface ?
	char			certFile[256];			// the name of the certificate file
	char			keyFile[256];			// the name of the pub/priv key file
	char			password[128];			// the password to the key file
	PolicyFlags		policy;					// the policy flags (see above)
} WSAPI_MultiHomeSslStruct, *WSAPI_MultiHomeSslPtr, **WSAPI_MultiHomeSslHandle ;

/*
** Fragment stuff.
*/
#if TARGET_API_MACOS_8

#define	FRAGINFO_VERSION	1		// Current version of the fragment info. 

typedef	long (*FragFunctionPtr)(void *value);	// Should be a form of UPP, but I am lazy...

/*
** WSAPI_FragInfoBlk
**
**	Information specific to the fragment manager.
*/
typedef struct
{
	FragFunctionPtr	fragFunc;			// Resolved function (post GetDiskFragment)
	FSSpec		fragFile;			// The FSSpec for the file
	short		fragResource;		// The resource file associated with the fragment
	CFragConnectionID fragConnID;	// Connection ID for the fragment
	Str63		fragName;			// The name of the fragment
} WSAPI_FragInfoBlk, *WSAPI_FragInfoPtr, **WSAPI_FragInfoHndl;

/*
** WSAPI_PluginFragInfo
**
** How WebSTAR stores fragment (plug-in) information internally.
** Be VERY careful, this information is subject to change without notice. Always
** check that FRAGINFO_VERSION matches before trying to get/use this data.
*/
typedef struct WSAPI_frag_struct {
	WSAPI_FragInfoPtr			fragInfo;
	long						id;
	unsigned long				nextIdleTime;		//future tickcount at which an idle message will be sent
	struct WSAPI_frag_struct	*next;
	char						name[WSAPI_NAME_SIZE];
	unsigned long				api_version;		//version # of the API
	short						valid;				//was the plug-in initialized OK and ready to go?
	short						accessControl;		//if true, call with access control messages
	short						filter;				//if true, call with filter messages
	short						logging;			//if true, call with logging messages
	short						routing;			//if true, call before normal filters, as a routing thing
	short						menuId;				// MenuID
	unsigned long				capabilities;		// flags that indicate functions the plug-in can perform or services it wants
	WSAPI_IPC_ResultPtr			ipcResult;
	char						version [12];
	char						adminURL[WSAPI_FILE_NAME_SIZE];
} WSAPI_PluginFragInfo, *WSAPI_PluginFragInfoPtr;

#endif	// TARGET_API_MACOS_8

# endif	// _STARNINE_H
