/*
	File:		WSAPI.h

	Contains:	xxx put contents here xxx

	Written by:	Chuck Shotton

	Copyright:	 1996 by StarNine Technologies, Inc., all rights reserved.

	Change History (most recent first):

        <22>      5/9/00    cjh     carbonize
        <21>      5/7/99    jtk     Note that piServerPort is now a perconnection thing.
        <20>      2/3/99    jtk     Fix the comment for WSAPI_CloseStreamWithTimeout().
		<19>	 1/27/99	jtk		Pull some parameters out of WSAPI.h, and put them into
									StarNine.h.
		<18>	 1/25/99	jtk		Add a new parameter so a plug-in can tell the server not to look
									at headers.
        <17>    12/23/98    jtk     Add new persistent connection support in.
        <16>    12/10/98    jtk     Add a piDefaultAction. This is used to allow a plug-in to define
                                    a default action that the server will use. Update with server
                                    version of WSAPI.h. Added piByteServerAction.
		<15>	 6/26/98	mdh		add piConnectionIDKeyword
		<14>	  3/6/98	jtk		Add WSAPI_OpenStreamOnSocket. New error code
									WSAPI_E_SocketClosed. Fixed up some comment errors.
		<13>	 1/30/98	jtk		Added new error WSAPI_E_SocketAlreadyOpen, changed
									WSAPI_ErrorCode back to a byte for backward compatibility with
									2.1.
		<12>	 1/21/98	cjh		Updated to latest distributed version of WSAPI.h
		<11>	12/17/97	jtk		Add capChunkedData capability flag (indicates that the plug-in
									understands chunked data). Added serverIPAddress to the filter
									param. Added a bunch of pi params to access the realm/user
									database. Added support for piNoGUIInterface.
		<10>	 11/7/97	jtk		Add a list of old versions.
		 <9>	10/27/97	jtk		Update to 1.3 (changes for multihoming).
		 <8>	 7/17/97	cjh		Update header to v1.2
		 <7>	  7/1/97	cjh		Updating header to v1.2 b
		 <6>	 11/8/96	cac		Add new header.
		 <5>	10/21/96	cac		New header
		 <4>	10/11/96	cmh		add new WSAPI.h 2.0d8to database.
		 <3>	 10/9/96	cmh		pasted in new admin api
		 <2>	 10/1/96	cac		Check in the latest beta of the header.

	To Do:
*/

//WSAPI.h

/*
WebSTAR API Interface, v1.3 FC
Copyright 1996-1998, StarNine Inc. All rights reserved.

This file defines the public interface between the WebSTAR plug-in framework and
user-defined plug-in routines.

*/

#ifndef WSAPI_Included
#define WSAPI_Included 1

#if __POWERPC__
#pragma options align=mac68k
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define WSAPI_VERSION	(unsigned long) 0x02000001	// 2.0b1 Do NOT modify this value
#define WSAPI_Version_10 0x01000000
#define WSAPI_Version_11 0x01010000
#define WSAPI_Version_12 0x01020000
#define WSAPI_Version_13 0x01030000		// Added WSAPI_RegisterRouter
#define WSAPI_Version_20 0x02000000
#define WSAPI_MAC	1

extern unsigned long wsapi_lib_version;	// the version of the WSAPI library code this plug-in is linked with. Useful
										// for making sure your plug-in was built with libs that match the interfaces, etc.


// enums that identify what type of server is "glued" to the WSAPI plug-in. Useful for determining the callback
// and parameter feature set that will be available at run-time. See wsapi_glue_type variable below.
typedef enum {
	WSAPI_Mac_Glue,		//plug-in is built as a WebSTAR Mac plug-in
	WSAPI_PC_Glue,		//plug-in is built as a WebSTAR 95 or NT plug-in
	NSAPI_Glue,			//plug-in is built as a Netscape plug-in
	ISAPI_Glue			//plug-in is built as a Microsoft plug-in
} WSAPI_Server_Glue_Type;

extern WSAPI_Server_Glue_Type wsapi_glue_type;	//what type of glue libraries are being used by the plug-in?

#define WSAPI_NAME_SIZE	32
#define WSAPI_FILE_NAME_SIZE 	256			/*size of text representations of file paths*/
#define WSAPI_VERSION_STR_SIZE	12			/*size of the version string returned in the init message pb*/
#define WSAPI_INDEX_SIZE 64					//the maximum length of index selectors used with Get and SetIndexedParameter

#define WSAPI_IPC_CONNECTION_MASK 0x00003FFF //See WSAPI_SendIPCMessage below

//  Not completely implemented yet. You may set these flags, but WebSTAR ignores them for now.
//  Flags that define capabilities that the plug-in can perform or services it wants to take advantage of
//  Used in the "capabilities" field of the Init message's PB record. Your plug-in should OR the appropriate
//  flags together and assign them to the capabilities field which is returned to the server at the completion of the
//	Init message handling. The server will use these flags to determine if a user has misconfigured your plug-in for
//	a role that it cannot support.

#define capISAPISubset		(unsigned long) 0x00000001		//restrict plug-in to ISAPI compatible calls - not implemented yet
#define capCGI				(unsigned long) 0x00000002		//plug-in is willing to operate as a normal CGI plug-in
#define capPreProcessor		(unsigned long) 0x00000004		//p.i. will work as a preprocessor
#define capPostProcessor	(unsigned long) 0x00000008		//p.i. will work as a postprocessor
#define capLogging			(unsigned long) 0x00000010		//p.i. will receive logging and status messages - not implemented yet
#define capSecurity			(unsigned long) 0x00000020		//p.i. will perform security checks - not implemented yet
#define capNSAPISubset		(unsigned long) 0x00000040		//restrict plug-in to NSAPI compatible calls - not implemented yet
#define capChunkedData		(unsigned long) 0x00000080		//This plug-in understands "chunked" data requests
#define capPreemptive		(unsigned long) 0x00000100		//For servers that care, this plug-in is preemption-safe
#define capSmartHeaders		(unsigned long) 0x00000200		//plug-in wants to use full headers

/*	Multihoming Capability Flags - lists the things you can do with MH addresses and ports

Allowed - if this is off, multihoming isn't even supported.  A routing
plug-in may still be used, but the IP addresses can't be managed

ToggleInterface - A plug-in may toggle individual interfaces, as listed via
the piInterfaceList keyword, on and off.  If this flag is off, you cannot
manage the IP interfaces other than to get a list of them via the
piInterfaceList keyword.

ChangePort - A plug-in may change the port on which a given IP address
listens.  A call to WSAPI_SetIndexedParameter with a given IP address and
port number and a non-zero value will cause the server to *change* the port
on which it listens for that IP address.

MultiPort - A plug-in may instruct the server to listen on multiple ports
for the same IP address.  A call to WSAPI_SetIndexedParameter with a gien
IP address and port number and a non-zero value will cause the server to
*add* a listen for that IP address on the given port.

Note that the MultiPort and ChangePort flag are mutually exclusive, as they
imply different semantics for the same calls to WSAPI_SetIndexedParameter.

*/
#define WSAPI_MHCAP_Allowed			(unsigned short) 0x0001		// Is MH supported?
#define WSAPI_MHCAP_ToggleInterface	(unsigned short) 0x0002		// Can turn on/off individual interfaces
#define WSAPI_MHCAP_ChangePort		(unsigned short) 0x0004		// Can change the socket an interface is listening on
#define WSAPI_MHCAP_MultiPort		(unsigned short) 0x0008		// Can listen on multiple sockets on a single IP address

/*
** Keep alive states for a connection. If the server supports piKeepAliveRequest, this is what it will return.
*/
typedef enum {
	WSAPI_KeepAliveNotSupported,	// Server does not allow keep alive at all.
									// Usually means off in the configuration.
									// Keep alive can not be turned on.
	WSAPI_KeepAliveNoClientSupport,	// Client doesn't support keep alive
									// (a "Connection: closed" header, 1.0 request, ...).
									// Keep alive can not be turned on.
	WSAPI_KeepAliveOffByPlugin,		// A plug-in has turned off keep alive.
									// Keep alive can not be turned on.
	WSAPI_KeepAliveUnknown,			// Keep alive is in an unknown state. The server only reads headers after filters
									// have run. So in a filter/route role, the server doesn't know if the client
									// supports keep alive. Keep alive can be turned off.
	WSAPI_KeepAliveOffByServer,		// The server has turned off keep alive.
									// Keep alive can be turned on.
	WSAPI_KeepAliveOn				// Keep alive is active (can be turned off)
} WSAPI_KeepAliveStatus;

/*	Permissions Flags	*/
#define	WSAPI_Perm_ReadFile			0x00000001	// can read files
#define WSAPI_Perm_ModifyFile		0x00000002	// can modify existing files
#define WSAPI_Perm_CreateFile		0x00000004	// can create new files
#define WSAPI_Perm_DeleteFile		0x00000008	// can delete files
#define WSAPI_Perm_ReadDir			0x00000010	// can list the contents of directories
#define WSAPI_Perm_CreateDir		0x00000020	// can make new directories
#define WSAPI_Perm_DeleteDir		0x00000040	// can delete directories
#define WSAPI_Perm_Lock				0x00000080	// can lock items (WebDAV-specific)
#define WSAPI_Perm_WebDAV			0x00010000	// can do WebDAV actions (actually not a permission)

#define WSAPI_Perm_CustomMask		0xffff0000	// bits available for vendor-specific use
#define WSAPI_Perm_Mask				0x0000ffff	// bits for standard permissions

/*****************************************************************************/
/* 	Important note to developers: Not all servers will implement all of these parameters.
		It is up to individual server implementors to decide which parameters will be supported,
		since some of these are WebSTAR-specific. It is possible that other server implementors
		will add additional parameters beyond the ones supported by WebSTAR as well.
 */

/** Keywords used to get/set WebSTAR parameters with WSAPI_Get/SetParameter **/
/** Note that these have the same definitions and formats as AppleEvent parameters
	of the same value (except for piConnectionID and piConnectionIDKeyword). 
	With the exceptions noted below, all parameters are of type
	"typeChar." The two numeric values below should be noted.
**/
typedef enum {
	piPathArgKeyword		= '----',
	piSearchArgKeyword		= 'kfor',
	piUserKeyword			= 'user',
	piPasswordKeyword		= 'pass',
	piFromUser				= 'frmu',
	piAddressKeyword		= 'addr',
	piPostKeyword			= 'post',
	piMethodKeyword			= 'meth',
	piServerName			= 'svnm',
	piServerPort			= 'svpt',	//typeShortInteger - R/O IP port that this connection came in on.
										// When used in a role that is not connection-oriented,
										// this will be the 'default' main port for the server (same thing as piPort).
	piScriptName			= 'scnm',
	piContentType			= 'ctyp',
	piContentLength			= 'CLen',	//typeLongInteger - the content-length header field (if any) from HTTP request
	piRefererKeyword		= 'refr',
	piUserAgentKeyword		= 'Agnt',
	piActionKeyword			= 'Kact',
	piActionPathKeyword		= 'Kapt',
	piClientIPAddress		= 'Kcip',	//typeLongInteger - 4 byte IP address
	piFullRequestKeyword	= 'Kfrq',
	piConnectionIDKeyword	= 'Kcid',	//long - unique # for each connection for this run of the server
	
	//new for plug-ins only
	piFileMIMEType			= 'Mime',	// The MIME type of the URL in piScriptName
	piServerField			= 'SvFd',	// The HTTP header field for this server. e.g., "Server: WebSTAR/1.3 ID/22"
	piServerDirectoryPath	= 'SPth',	// A C string representing the complete (machine-specific) path name of the server's working directory
	piURLPhysicalPath		= 'UPth',	// If the URL represents an actual file in the file system, this parameter contains its name relative to the server's root.
	piIfModifiedSince		= 'IfMS',	// short. True if the file has been modified since last retrieval or if modification cannot be determined. False if unchanged.
	piIfModifiedDate		= 'IfMD',	// typeChar, R/O, if-modified-since date string from HTTP request header
	piCurrentRealm			= 'CRlm',	// typeChar, 256 max. The realm (if any) that the requested URL belongs to.
										// WSAPI_E_RequestFailed will be returned by GetParameter if this value isn't defined for the current request.
	piURLFileRef			= 'Pufr',	// typeFSS, R/0, Server-specific reference to file in the native O/S. On the
										//  Mac with WebSTAR, this is a FSSpec to actual file being requested if it
										//  really exists on disk. Nonexistent files will have a FSSpec that has all
										//  fields set to zero and the name field will be empty.
	piRequestHeaderField	= 'Prhf',	//## typeChar - INDEXED! index is the request header field to query, 
										//##   value of header field is returned if present in the request.
	piResponseHeaderField	= 'Prpf',	//	typeChar - INDEXED.  Index is the header field to add to the
										//	outgoing response.
	//******************************
	//log- and status-related values
	piConnectionID			= 'LCID',	//long - unique # for each connection for this run of the server
	piServerUniqueID		= 'LSID',	//long - unique # across all current server invocations on this machine
	// note: the value of the WebSTAR message-id: field can be generated with the following C printf format,
	//   using the values of the preceding two properties:
	//  printf ("<%0lx.%ld@%s>", serverUniqueIDVar, connectionIDVar, hostnameVar);
	
	//*****************************
	// The following parameters are read-only and represent information present in the server's 
	// status display and/or the information broadcast to remote logging processes.
	piTotalConnections		= 'STCn',	// long. Total connections the server has received since it started up
	piCurrentUserLevel		= 'SCLd',	// short. Current number of users connected to the server at this time
	piHighestUserLevel		= 'SMLd',	// short. Max number of users ever connected to the server at once
	piCurrentFreeMemory		= 'SFMm',	// long. Amount of free memory available to the server process
	piMinimumFreeMemory		= 'SMFM',	// long. Minimum amount of free memory that was ever available to the server process
	piTotalConTimeouts		= 'STTm',	// short. Total number of client connections that timed out before completion
	piTotalConBusies		= 'STBz',	// short. Total number of client connections refused because the server was too busy
	piTotalConDenied		= 'STDn',	// short. Total number of client connections denied by domain name or IP address
	piTotalBytesSent		= 'TByt',	// long. Total number of bytes sent by the server
	piUpSinceDate			= 'SUpS',	// C string. Date & time the server was launched; Format, MM/DD/YY:HH:MM
	piPluginAdminInfo		= 'Padm',	//typeChar, R/O, text block containing info on each plug-in currently installed
										//    tab separated syntax of each line is: <name> <version> <Admin URL path><cr>
	
	//******************************
	// These four parameters describe the plug-in environment, including the physical location of plug-ins
	// and information about each parameter that can be manipulated at runtime to determine range checks, types,
	// string representations, etc. See the W*API docs for details on the format of the piPluginParamInfo data structure.
	
	piPluginParamInfo		= 'Pipi',	//typeLong, R/O, pointer to a W*API-specific data structure describing plug-in parameter semantics
	piPluginParamCount		= 'Pipc',	//typeSMInt, R/O, count of the number of W*API parameters in piPluginParamInfo
	piPluginDirPath			= 'Pdph',	//typeChar, R/O, full path to the plug-ins folder
	piPluginDirFSSpec		= 'Pdfs',	//typeFSS, R/O, platform-specific file system reference to plug-ins folder
	
	//******************************
	//the following 3 values are only guaranteed to be set AFTER a connection has been completed
	//  and logged. (i.e., only access them from a post-processor plug-in.)
	piTransferResult		= 'LTRs',	//OK, PRIV, or ERR! string, depending on transfer result
	piBytesSent				= 'LByt',	//number of bytes written to the HTTP client
	piTransferTime			= 'LTrT',	//number of ticks it took from connection open to close
	
	//******************************
	// WebSTAR properties below here.
	//  These match the definitions in the WebSTAR AETE resource.
	
	// The following values are all global WebSTAR properties.
	
	piDumpBufSize			= 'Dbuf',	//typeLongInteger
	piPigDelay				= 'PigD',	//typeSMInt
	piMaxUsers				= 'MaxU',	//typeSMInt
	piNoDNS					= 'Ndns',	//typeSMInt
	piTimeOut				= 'TimO',	//typeSMInt
	piMaxListens			= 'MaxL',	//typeSMInt
	piLogging 				= 'Plog',	//typeSMInt
	piVerboseMessages		= 'Pvrb',	//typeSMInt
	piRefuseConnections		= 'Pcon',	//typeSMInt
	piHideWindow			= 'Phid',	//typeSMInt
	piDefaultMIMEType		= 'Pmim',	//typeChar
	piPort					= 'Pprt',	//typeSMInt, R/W main IP port for this server. For WebSTAR >= 4.0, use piSSLPort to get
										// or set the SSL port on the combined server.
	piIndexFile				= 'Pixf',	//typeChar
	piErrorFile				= 'Perf',	//typeChar
	piErrorDir				= 'Perd', 	//typeChar, the URI to the error file directory.
	piLogFile				= 'Plgf',	//typeChar
	piNoAccessFile			= 'Pnaf',	//typeChar
	piSuffixMappings		= 'Psuf',	//typeChar
	piAllowDeny				= 'Pa/d',	//typeChar
	piAllowDenyByRealm		= 'PADr',	//typeChar - R/W, NOTE!! This is an indexed value and you must use the
										// WSAPI_GetIndexedParameter or WSAPI_SetIndexedParameter call to manipulate it. The Index is
										// the name of the realm to get or set allow/deny info for.
	piRealms				= 'Prlm',	//typeChar
	piActions				= 'Pact',	//typeChar
	piLogFormat				= 'Plfm',	//typeChar
	piPreProcessor			= 'Ppre',	//typeChar
	piPostProcessor			= 'Ppst',	//typeChar
	piVersionNumber			= 'Pvrs',	//typeChar
	piReportDelay			= 'Prdl',	//typeSMInt - R/W, amount of time in seconds between status report events
	piCacheFlush			= 'Pcfl',	//typeSMInt - W/O, setting this flag indicates the server should flush any
										// cached data. A plug-in receiving this parameter to a WSAPI_ServerStateChanged
										// message should interpret it as a signal for the plug-in to flush any cached data
										// it retains from the local file system.
	piFileInfoCacheSize		= 'Pfic',	//typeLong - R/W, amount of memory to allocate to file info caching
	piKeepAliveConnections	= 'Pkac',	//typeSMInt- R/W, number of connections per client to keep alive
	piKeepAliveTime			= 'Pkat',	//typeSMInt- R/W, seconds to keep alive idle client connections
	piKeepAliveRequest		= 'Pkar',	//typeLong- R/W, setting for this connections keep-alive status.
										// Setting this to 1 (from a CGI/INDEX role) can turn on keep-alive, setting it to 0 will turn it off.
										// See WSAPI_KeepAliveStatus above for details of what a "Get" can return.
	piServerCName			= 'Pscn',	//typeChar - R/W, canonical name of server to be used in HTTP responses, regardless of DNS name
	piCGIBinOnly			= 'Pcbo',	//typeSMInt - R/W, flag determining if CGIs will be executed from the /cgi-bin folder only
	piServiceAddress		= 'Pisa',	//typeLong - R/W, INDEXED VALUE, table of registered service addresses
										// This is a complicated parameter. You should refer to the W*API documentation for details.
										// The index is the name of the service whose address you wish to register or retrieve.
	piMHCapabilities		= 'Mcap',	// typeUShort - R/O, WSAPI_MHCAP flags
	piIPAddress				= 'roip',	// typeLong - R/O the address that this connection arrived on.
										// When used in a role that is not connection-oriented,
										// this will be the 'default' IP address of the server
	piInterfaceList			= 'roil',	// R/O - returns a 
										// variable-length block of data (length is divisible by sizeof(WSAPI_Interface)) 
										// containing an array of WSAPI_Interface structures.
	piInterface				= 'roon',	// for indexed use only.  The index is a WSAPI_Interface pointer (from piInterfaceList).
										// Used to turn specific interfaces on or off.
	piRouteParameters		= 'rorp',	// R/O - returns a variable-length block of data
										// (length is divisible by four) containing an array of the parameters
										// that can be changed during the WSAPI_Route command.  At a minimum,
										// piServerDirectoryPath and piServerCName must appear in this list.
										// Some server vendors may want to expand this list.
	piServerDefault			= 'rosd',	// for indexed use only.  Used to access the default server
										// settings for parameters that can be changed on a per-connection basis.
	piNoGUIPresent			= 'gui?',	//typeSMInt - R/O Does the server have a GUI (are QD calls safe?)
										// 1 -> NO GUI
										// 0 -> Has a GUI

	piAdminWindow			= 'Awin',	// typeChar - W, C string for the monitor window
	piUserCount				= 'Ucnt',	//typeLong - R/O number of users in the database
	piUserCookie			= 'Ucky',	//typeLong - Magic cookie thing for the database
										// Use this as index to piFindUserByCookie, when finished do a DisposePtr()!
	piFindUserByCookie		= 'Ufnd',	//typeChar - R/O get the next user via cookie. Returns two C strings
	piAddUser				= 'Uadd',	//typeChar - R/O Add a user to the database (pass 2 C strings in the index)
	piDeleteUser			= 'Udel',	//typeChar - R/O Delete a user in the database (pass 2 C strings in the index)
	piExportUsers			= 'EUsr',	//typeChar - R/O Export users, pass a FSSpec
	piImportUsers			= 'IUsr',	//typeChar - R/O import users, pass a FSSpec
	
	// These go with the WSAPI/2 security structure
	piAuthMethodList		= 'AutL',	/* the list of all installed authorization methods, delimited by CR */
	piAuthMethod			= 'Auth',	/* the auth method used to authenticate the user */
	piPermissions			= 'Perm',	/* indexed, NULL index indicates the piScriptName value */
	piSSLCryptBits			= 'SSLc',	/* typeLong - number of unknown bits in the symetric cipher (eg: 40, 56, 128) */
	
	// also new with WSAPI/2
	piProtocolVersion		= 'Pver',	/* the protocol version of the request (eg:  "HTTP/1.1") */
	piOptionalMethod		= 'optm',	/* Add a method to what is supported by the web server */
	piNote					= 'note',	/* Add a note to this connection.  The index is the key, and the data must 
										   be a pointer allocated with WSAPI_AllocateMemory */
	piURI					= 'cURI',	/* Full unparsed URI of the request, including search and path args. */
	piOriginalScriptName	= 'oSCN'	/* If requested during a Run or FileInfo commands during Error processing, 
											returns the original script name of the current request.  Otherwise, behaves
											the same as piScriptName. */

} WSAPI_ParamKeywords;
			
/*****************************************************/
//    Parameter block information

typedef unsigned long WSAPI_Stream, *WSAPI_StreamPtr; //type for IP callbacks in network stuff below
typedef unsigned long WSAPI_ListenerRef;

/** TCP/IP conection status codes **/

typedef enum {
	WSAPI_IP_Closed, 	//connection is closed or terminated. plug-in should stop processing
	WSAPI_IP_Open, 		//connection is open
	WSAPI_IP_Data,		//connection is open and has unread data on it
	WSAPI_IP_Unknown	//an unknown or unspecified error occurred
} WSAPI_ConnectionStatus;

//messages sent to plug-in dispatcher
typedef enum {
	WSAPI_Init, 
	WSAPI_Shutdown, 
	WSAPI_Idle, 
	WSAPI_Run, 
	WSAPI_Emergency, 
	WSAPI_ServerStateChanged, 
	WSAPI_Register,
	WSAPI_AccessControl,
	WSAPI_Filter,
	WSAPI_Event,			//## sent to plugins with event info
	WSAPI_UIActivate,		//## sent to plugins signaling it's time to open windows, display version info, etc.
	WSAPI_ListenComplete, 	//## sent when a pending listen completes and a client is available for the plug-in
	WSAPI_Logging,			//## sent after post-processing is complete to plug-ins that request it
	WSAPI_Route,			// sent before filters, allows one plugin to route a request
	WSAPI_Authenticate,		// called through the WSAPI_AccessControl command
	WSAPI_OutOfContext,		// the command given to PBs created through WSAPI_CreateCustomPB
	WSAPI_FileInfo			// takes the place of most uses of the WSAPI_Filter command
} WSAPI_Command;

//sub-messages sent to plug-in dispatcher in the "role" field when message is "WSAPI_Run"
typedef enum {
	WSAPI_CGI_Role,				//running as a normal CGI replacement
	WSAPI_PreProcessor_Role,	//acting as a pre-processor
	WSAPI_PostProcessor_Role,	//acting as a post-processor
	WSAPI_Logging_Role,			//UNUSED - receiving log info
	WSAPI_Security_Role,		//UNUSED - performing an access authorization
	WSAPI_Error_Role,			//handling bad URL requests
	WSAPI_NoAccess_Role,		//handling requests denied by IP or domain
	WSAPI_Index_Role			//handing a request for the default file
} WSAPI_Run_Role;

//descriptor type used to extract various named parameters available to plug_ins
typedef void *WSAPI_DescPtr;

/** structure types making up the CommandPB parameter block **/

typedef struct {		// now used by both init and register messages
	short resRef;		//in - reference number of plug-in's resource file
	char pluginName [WSAPI_NAME_SIZE]; //in/out - the unique name that identifies this plug-in. Initialized to file name.
	unsigned long capabilities; // out - flags that indicate functions the plug-in can perform or services it wants
	// these fields should be filled in during the Register message
	char version [WSAPI_VERSION_STR_SIZE]; //out - string representation of the plug-in's version, e.g., "1.0b1"
	char adminURL [WSAPI_FILE_NAME_SIZE]; //out - path portion ONLY of URL to configure plug-in, e.g. "/admin.plugsuffix"
	// new with WSAPI/2
	void *piRef ;		// in - a platform specific reference to the plug-in.  On MacOS X this is your CFBundleRef.
} WSAPI_InitPB;

typedef struct {
	short unused;
} WSAPI_ShutdownPB;

typedef struct {
	unsigned long ticksToSleep;	//out - how long to sleep before WebSTAR's idle handler calls again
} WSAPI_IdlePB;

typedef struct {
	WSAPI_Run_Role role;	//in - what is this invocation supposed to do? be a CGI, preprocessor, etc?
	unsigned long result;	//## out - the HTTP status code returned by the plug-in in its HTTP header. e.g. 200, 401, etc.
							//  all plug-ins returning data to clients should fill in this field. Servers may use
							//  this value when writing log information, etc.
} WSAPI_RunPB;

typedef struct {
	short unused;
} WSAPI_EmergencyPB;

typedef struct {
	WSAPI_ParamKeywords	whichParam;		//in - contains the WSAPI_ParamKeywords (parameter) value that has changed
} WSAPI_StateChangedPB;

typedef struct {
	short	grantAccess;		// out - boolean flag. If true, the server will process the URL as normal
	short	authRequired;		// out - boolean flag. If true, the server will require password access
	// only available on WSAPI/2 and later
	long	priority ;			// int - the priority level that this plug-in registered with
	char	*uri ;				// in - the URI whose permissions are to be returned
	long	accessPerms ;		// out - the permission flags
	long	responseCode ;		// out - if set, indicates the error type to be generated for the client if no permissions
								// 	are granted.  Default is 401.  
} WSAPI_AccessControlPB;


typedef struct {					// server sends this with the WSAPI_Authenticate message
	char *method ;					// in - the name provided by WSAPI_RegisterAuthorizer
	char *realm ;					// in - the realm of the request
	char *credentials ;				// in - NULL or the data from the "Authorization:" request header
} WSAPI_AuthenticatePB ;

typedef struct {				// server sends this structure with the WSAPI_Filter message
	char *request;				// in - request contains a pointer to the read-only HTTP request data (not including object body)
	unsigned short requestLength;// in - length in characters of the request. Data beyond length bytes should not be modified!
	char *newRequest;			// out - null initially. setting pointer to new request data (allocated by you with WSAPI_AllocateMemory ONLY!)
								//   forces server to replace original request with data you generate for the request instead
	unsigned short newRequestLength;// out - zero initially. length in characters of the new request you create.
	unsigned long serverIPAddress;	// IP Address of the server. Only defined if WSAPI_MHCAP_Allowed is true.
} WSAPI_FilterPB;

typedef struct {				//## sent along with WSAPI_Event message
	void *event;				// in - on the Mac, this can be typecast as EventRecord *
	void *window;				// in - on the Mac, this can be typecast as WindowPtr *
	void *environment;			// in - on the Mac, typecast as a QDGlobalsPtr to get to Quickdraw globals
} WSAPI_EventPB;			

typedef struct {				//## sent along with WSAPI_UIActivate message
	unsigned long unused;
} WSAPI_UIActivatePB;

typedef struct {				//## sent with a WSAPI_ListenComplete Message
	WSAPI_ListenerRef refCon;	// in - returned to WSAPI_ListenCancel -- do not modify
	WSAPI_StreamPtr stream;		// in - stream pointer for incoming connection
	unsigned short port;		// in - port that the listen was accepted on
	unsigned long remoteAddr;	// in - address of the client just connected, as a 4 byte value
	WSAPI_ConnectionStatus status;	// in - status of the listener, closed means it's gone for good
} WSAPI_ListenCompletePB;
		
typedef struct {				//## sent with a WSAPI_Logging message
	unsigned long result;		//## in - HTTP status code (if non-zero) sent to client. 0 indicates a failed request attempt.
								//  (same as the result returned in the WSAPI_RunPB structure)
} WSAPI_LoggingPB;

typedef struct {
	long isError ;				// set to non-zero if the request is in error processing
} WSAPI_FileInfoPB ;

typedef union {
	WSAPI_InitPB 		init;			// used by both register and init messages
	WSAPI_ShutdownPB 	shutdown;
	WSAPI_IdlePB		idle;
	WSAPI_RunPB			run;
	WSAPI_EmergencyPB	emergency;
	WSAPI_StateChangedPB stateChanged;
	WSAPI_AccessControlPB accessControl;
	WSAPI_AuthenticatePB auth ;
	WSAPI_FilterPB		filter;
	WSAPI_EventPB		event;			//##
	WSAPI_UIActivatePB	uiActivate;		//##
	WSAPI_ListenCompletePB listen;		//##
	WSAPI_LoggingPB		logging;		//##
	WSAPI_FileInfoPB	fileInfo;
} WSAPI_PB_Union;

typedef struct {
	long api_version;			//in - what version of API does the server support?
	void *api_data;				//in - private data. DON'T TOUCH!
	WSAPI_Command command;		//in - what operation is the plug-in to perform?
	WSAPI_PB_Union param;
} WSAPI_CommandPB, *WSAPI_CommandPBPtr;

// record for holding pending IPC stuff
#if WSAPI_MAC

#include <Carbon/Carbon.h>

typedef struct {
	AppleEvent theAppleEvent;
	AppleEvent reply;
	long handlerRefcon;
} WSAPI_IPC_Result, *WSAPI_IPC_ResultPtr;

#else

typedef struct {
	void *theIPCMsg;
} WSAPI_IPC_Result, *WSAPI_IPC_ResultPtr;

#endif

//*******************************
//Definitions for the piPluginParamInfo data structure

#define WSAPI_PARAM_SIZE 32

typedef struct piparam_table_struct {
	WSAPI_ParamKeywords param;		//one of the enums from above
	char name[WSAPI_PARAM_SIZE];	//case-sensitive string representation of the parameter's name
	char index[WSAPI_PARAM_SIZE];	//"name" of the index field. usually maps to a HTML form field name
	OSType type;					//4 character (long) parameter type
	short mode;						//bit field describing read/write access. masks: read = 2, write = 1
	long lower;						//lower bound for range checking integer parameters
	long upper;						//upper bound for range checking. If less than lower bound, no range is specified
	void *extra_info;				//reserved for future expansion
} WSAPI_ParamInfo, *WSAPI_ParamInfoPtr;


//*******************************
/** Error codes returned by various WSAPI calls **/

typedef enum {
	WSAPI_I_NoErr,				//normal completion code, synonymous with noErr
	WSAPI_E_MessageNotHandled,	//return value indicates the command message wasn't processed
	WSAPI_E_ParameterNotFound,	//requested parameter descriptor did not exist
	WSAPI_E_InsufficientMemory,	//not enough memory to perform requested operation
	WSAPI_E_Unimplemented,		//requested function is not yet implemented or isn't available for this version of WSAPI
	WSAPI_E_HTTPConnectionGone,	//the client connected to this thread has closed the connection
	WSAPI_E_BadCommandPtr,		//commandPtr that was passed was null or invalid
	WSAPI_E_CantOpenStream,		//TCP/IP stream couldn't be created or opened
	WSAPI_E_BufferTooBig,		//data buffer was too large for the callback to process
	WSAPI_E_NameLookupFailed,	//the domain name resolver failed to resolve the requested name
	WSAPI_E_NullDescriptor,		//10, a null descriptor, or one with no data pointer was passed when real data was required
	WSAPI_E_UnspecifiedError,	//some unknown error happened
	WSAPI_E_BadArgument,		//one of the arguments (usually a pointer) passed to a callback was invalid
	WSAPI_E_CantModifyData,		//an attempt was made to modify read-only data
	WSAPI_E_DescriptorTooSmall,	//user-supplied storage for a descriptor is too small for the data being assigned to it
	WSAPI_E_HTTPConnectionErr,	//plug-in attempted to read from a closed or non-existent HTTP stream, or some other IP-related error occurred.
	WSAPI_E_WrongRoleForCall,	// plug-in tried to make a callback that is inappropriate for the current WSAPI_Command
	WSAPI_E_RequestFailed,		// whatever request the plug-in was making of the server failed to complete successfully
	WSAPI_E_IPCMsgPending,		// an attempt was made to send an IPC message while one was already in progress
	WSAPI_E_IPCNoMsgPending,	// an attempt was made to receive IPC message results when no message was in progress
	WSAPI_E_Timeout,			//20, requested operation took too long and timed out before being completed
	WSAPI_I_NoPermission,		// current user doesn't have permissions for realm; other security violation
	WSAPI_E_ParameterIndexedOnly,//the requested parameter can only be accessed as an indexed parameter
	WSAPI_E_IndexedValueNotFound,//the item specified by the index was not present in the requested object
	WSAPI_E_TooManyRouters,		//24, an attempt to register more then one routing module (WSAPI_RegisterRouter)
	WSAPI_E_SocketAlreadyOpen,	// 25 A call to WSAPI_ListenStream on a socket that is already open.
	WSAPI_E_SocketClosed,		// 26 Attempt to read/write from a socket that the remote site has closed.
	WSAPI_E_DuplicateName,		// 27 Attempt to register a duplicate service name
	WSAPI_E_LASTERROR			// place holder for the last error
} WSAPI_ErrorCode;

//*******************************



/** Misc. W*API type definitions **/

/* An array of these is returned by piInterfaceList */
typedef struct {
	unsigned long ipAddress;	// IP address
	unsigned short port;		// Port to listen on
} WSAPI_Interface, *WSAPI_InterfacePtr;

typedef char WSAPI_IndexString [WSAPI_INDEX_SIZE];	//index selector used by WSAPI_Get and SetIndexedParameter
typedef WSAPI_IndexString *WSAPI_IndexStringPtr;

/********************************************************************/
//WSAPI Framework Function prototypes

// You may call any of the following functions from inside your plug-in to 
// have WebSTAR perform the requested function.

/*************************/
/**  Memory Management  **/

void *WSAPI_AllocateMemory (WSAPI_CommandPBPtr pb, long size);
/* 	Allocates and returns a pointer to "size" bytes of data. Returns NULL if
	memory allocation fails.
	Do NOT use any other techniques to allocate memory within your plug-in code.
	All memory should be allocated using this call and disposed of using the 
	corresponding WSAPI_FreeMemory call.
*/

void *WSAPI_ReallocateMemory(WSAPI_CommandPBPtr pb, void *data, long size) ;
/*	Attempts to expand the memory allocated to data, or finds a new location for it.
	If NULL is returned, then there wasn't enough memory to satisfy the request.  Otherwise,
	use the returned pointer instead of the one you passed in.
*/

void WSAPI_FreeMemory (WSAPI_CommandPBPtr pb, void *data);
/*	Deallocates a block of memory originally allocated with WSAPI_AllocateMemory. Do NOT mix
	and match WSAPI_AllocateOSMemory and WSAPI_FreeMemory or vice versa!!!
*/

/*	Pooled Memory - works the same as above, but allocates from a pool associated
	with the pb, avoiding many heap synchronizations.  It is very important that you:
		- use a valid PB
		- use the same pb for allocate/reallocate/free calls on the pointer
		- only use the the pb from one thread at a time (this is normally not an issue)
		- not attempt to save these pointers between requests.  If you need more persistent data,
			use the normal memory calls.
*/
void *WSAPI_AllocatePoolMemory (WSAPI_CommandPBPtr pb, long size);
void *WSAPI_ReallocatePoolMemory(WSAPI_CommandPBPtr pb, void *data, long size) ;
void WSAPI_FreePoolMemory (WSAPI_CommandPBPtr pb, void *data);


void *WSAPI_AllocateOSMemory (WSAPI_CommandPBPtr pb, long size);
/* 	 Allocates and returns an O/S-specific memory accessor (e.g., a Handle under Mac O/S)
	to "size" bytes of data. Returns NULL if memory allocation fails.
	Do NOT use any other techniques to allocate memory within your plug-in code.
	All memory should be allocated using this call and disposed of using the 
	corresponding WSAPI_FreeOSMemory call.
*/

void WSAPI_FreeOSMemory (WSAPI_CommandPBPtr pb, void *data);
/*	 Deallocates a block of memory originally allocated with WSAPI_AllocateOSMemory. Do NOT mix
	and match WSAPI_AllocateMemory and WSAPI_FreeOSMemory or vice versa!!! On Macs, "data" must be
	a pointer to a Handle, as this call will null out the handle to ensure it isn't reused.
*/


/*************************/
/** Process Management  **/

WSAPI_ErrorCode WSAPI_YieldTime (WSAPI_CommandPBPtr pb, long ticksToYield);
/*	Yields time to WebSTAR, allowing other threads, plug-ins, and processes to
	have access to the CPU. ticksToYield is only a suggestion to WebSTAR about how
	long to wait (in ticks) before resuming this thread. Control may return sooner or later
	than the time specified. This routine MUST be called if you are performing any
	processing that takes more than a few milliseconds, involves I/O of any significant
	amount, or would otherwise prevent normal operation of other apps in the
	MultiFinder environment. This call is essentially equivalent to the Thread Manager's
	YieldToAnyThread call.
*/
 
WSAPI_ErrorCode WSAPI_RequestIdleTime (WSAPI_CommandPBPtr pb, unsigned long ticksToSleep);
/*	This routine registers your plug-in with WebSTAR's idle time dispatcher. Plug-ins
	registered this way will have the WSAPI_Idle command sent to them at an interval
	approximating ticksToSleep. Plug-ins are called in the context of the idle dispatcher
	and will not have access to an active HTTP connection or associated data.
	This routine is useful for performing periodic global tasks within your plug-in.
*/

/*************************/
/** Networking Services **/

WSAPI_ErrorCode WSAPI_SendHTTPData (WSAPI_CommandPBPtr pb, void *data, long dataLen);
/*	Sends an arbitrarily large buffer of data to a HTTP client over an open HTTP connection.
	This implements a raw data stream to the client, allowing you to generate your own HTTP 
	header and content, similar to what current CGI applications do now. The difference is
	that your data is going back in real time and you can send as much as you'd like.
	"data" is a pointer to a buffer containing any type of data. dataLen is the number of 
	bytes within this buffer to transmit. No translation is performed by WebSTAR on data. 
	You are responsible for disposing of the data pointer after this call returns, if 
	necessary. Avoid sending many, small buffers of data with this routine if possible.
	
	NOTE: If this call ever returns a non-zero return value, you should terminate any
	processing and return control to WebSTAR immediately. This is an indication 
	that the client has disconnected and WebSTAR needs to clean up the connection.
*/

WSAPI_ErrorCode WSAPI_ReadHTTPData (WSAPI_CommandPBPtr pb, void *data, long *dataLen);
/*	Reads data from the HTTP stream. Useful for reading and processing large POST args,
	etc., like file uploads. You must supply storage for data and pass in the maximum amount of
	data to be returned in the dataLen parameter. WSAPI will return up to dataLen bytes in data
	and will set dataLen to the actual amount of data returned. 
	
	Use the WSAPI_HTTPConnectionStatus below to determine if data is pending on the HTTP connection, etc.
	
	IMPORTANT! If a pre-processor reads the HTTP object body data from a connection, that data
	will NOT be available to downstream plug-ins or CGIs should the pre-processor decide not to
	handle the URL. That means no post args for CGIs, etc. In general, it's bad form for any
	role other than CGI, Error, NoAccess, or Index to make this call. Specific cases excepted.
	
	Additional note, the piPostKeyword parameter can be used to repeatedly access the first 24k bytes
	of this data without any side effects to downstream consumers or the IP data stream.
*/

WSAPI_ErrorCode WSAPI_HTTPConnectionStatus (WSAPI_CommandPBPtr pb, WSAPI_ConnectionStatus *status);
/*	Returns the status of the HTTP connection associated with the thread that this call is made
	from. Obviously, this is an invalid call to make from any message handler except the RUN 
	message. Returns the appropriate WSAPI_ConnectionStatus in the status parameter.
*/
	
//Generic Open, Close, Send, and Receive

WSAPI_ErrorCode WSAPI_OpenStream (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, unsigned long ip_addr, unsigned short ip_port);
/*
	Returns a Stream pointer to an open connection to the host and port specified. ip_addr is a
	long integer representation of the host address, as returned from the DNS routines. If an error is returned,
	the stream was not successfully created.
*/
WSAPI_ErrorCode WSAPI_OpenStreamOnSocket (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, unsigned long ip_addr, unsigned short ip_port, unsigned short localPort);
/*
	Returns a Stream pointer to an open connection to the host and port specified. Try and bind to the socket specified. ip_addr is a
	long integer representation of the host address, as returned from the DNS routines. If an error is returned,
	the stream was not successfully created.
*/


WSAPI_ErrorCode WSAPI_CloseStream (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream);
/*
	Gracefully closes a connection opened using OpenStream.
*/

WSAPI_ErrorCode WSAPI_CloseStreamWithTimeOut (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, unsigned long timeout);
/*
	Gracefully closes a connection opened using OpenStream, but the plugin controls how long to wait for an orderly disconnect.
	The timeout is in seconds.
*/

WSAPI_ErrorCode WSAPI_ReadStream (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, void *data, long *dataLen);
/*
	Reads up to *dataLen bytes from the specified stream. *dataLen contains the actual number of bytes read,
	which may be less than the number requested.
*/

WSAPI_ErrorCode WSAPI_WriteStream (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, void *data, long dataLen);
/*
	Writes dataLen bytes of data to the specified stream. If dataLen is too large, an error will be returned.
	Generally, you should never try to write more data than the maximum dump_buf_size for WebSTAR in one operation.
*/

WSAPI_ErrorCode WSAPI_StatusStream (WSAPI_CommandPBPtr pb, WSAPI_Stream *stream, WSAPI_ConnectionStatus *status, long *pending);
/*
	Returns the status of the specified stream (open, closed, data pending) and the amount of unread data. Under
	OpenTransport, the pending flag will simply be positive if data exists to be read.
*/

WSAPI_ErrorCode WSAPI_IPNameToAddr (WSAPI_CommandPBPtr pb, char *name, unsigned long *addr);
/*
	Converts the domain name or dotted IP address in "name" into a 4 byte IP address for use in the OpenStream call.
*/

WSAPI_ErrorCode WSAPI_IPAddrToName (WSAPI_CommandPBPtr pb, unsigned long addr, char *name);
/*##
	Converts the 4 byte IP address in "addr" into its associated domain name. "name" must point to a
	256 byte buffer to accept the returned name. If DNS look-ups are disabled or not functioning, a
	string representation of the dotted decimal address will return.
*/

WSAPI_ErrorCode WSAPI_ListenStream (WSAPI_CommandPBPtr pb, unsigned short port, unsigned short maxConnections,
									unsigned long suggestedSize, WSAPI_ListenerRef *refCon);
/*##
	Requests the server to start accepting incoming connections on the specified TCP port. No more
	than maxConnections instances of the plug-in will be active at any one time. suggestedSize is 
	an O/S-specific value that represents how much thread stack space a plug-in would like for each 
	listening thread. Setting this value to 0 uses the default value for the server and/or O/S. 0 is 
	the recommended value. refCon is a private value returned to the plug-in for use in a subsequent 
	call to WSAPI_ListenCancel.
**/

WSAPI_ErrorCode WSAPI_ListenCancel (WSAPI_CommandPBPtr pb, WSAPI_ListenerRef refCon);
/*##
	Causes the server to cancel a previous request to listen for incoming connections. refCon is the value
	returned from the WSAPI_ListenStream call in its refCon parameter. All threads will be killed 
	and the listening port will be closed before this routine returns.
**/

/*************************/
/**  Utility Functions  **/

WSAPI_ErrorCode WSAPI_GetParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_DescPtr *descriptor);
/*	Use this routine to obtain descriptors that reference named WebSTAR data elements. These
	elements are generally equivalent to the AppleEvent parameters passed to CGIs. However,
	certain WebSTAR properties will also be made available via this interface. See the list
	of available parameters and properties above (WSAPI_ParamKeywords).
	
	You must always dispose of any descriptors returned by WSAPI_GetParameter.
	
	If the value for a parameter is undefined at the time the call is made, but the paramter is a legal
	one to request, WSAPI_GetParamter will return WSAPI_E_RequestFailed. If the parameter cannot be requested
	at the time the call is made (because of role or message conflicts), the WSAPI_E_WrongRoleForCall error
	will be returned.
	
*/

WSAPI_ErrorCode WSAPI_SetParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_DescPtr descriptor);
/*	 Use this routine to set server parameters to the value specified in the descriptor.
	You need to create the descriptor first, using WSAPI_NewDescriptor. Then use WSAPI_SetDescriptor
	to insert values into the new descriptor before passing it to WSAPI_SetParameter.
*/

WSAPI_ErrorCode WSAPI_GetIndexedParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_IndexStringPtr index, WSAPI_DescPtr *descriptor);
/*	 Use this routine to obtain descriptors that reference indexed items in a named server data element. 
	These elements consist of values with repeating items that are specified by a unique index value
	for each item. Attempting to use this routine on a non-indexed parameter will simply return the
	value of the scalar parameter. The index selector is case-insensitive. pb may be null when appropriate.
	
	You must always dispose of any descriptors returned by WSAPI_GetIndexedParameter.
	
	If the value for a parameter is undefined at the time the call is made, but the parameter is a legal
	one to request, WSAPI_GetIndexedParameter will return WSAPI_E_RequestFailed. If the parameter cannot be 
	requested at the time the call is made (because of role or message conflicts), the WSAPI_E_WrongRoleForCall 
	error will be returned.
	
*/


WSAPI_ErrorCode WSAPI_SetIndexedParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_IndexStringPtr index, WSAPI_DescPtr descriptor);
/*	 Use this routine to set indexed items of server elements to the value specified in the descriptor.
	You need to create the descriptor first, using WSAPI_NewDescriptor. Then use WSAPI_SetDescriptor
	to insert values into the new descriptor before passing it to WSAPI_SetIndexedParameter.
	Attempting to use this routine on a non-indexed parameter will simply set the
	value of the scalar parameter. The index selector is case-insensitive. pb may be null when appropriate.
*/

WSAPI_ErrorCode WSAPI_NewDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr *descriptor, 
										OSType dataType, void *data, unsigned long dataSize);

/*  Pass a pointer to a descriptor pointer and WSAPI will allocate it, defining the data type, size and content. 
	If the data pointer is null when passed, a data block of dataSize will be allocated anyway and you
	can assign values to the descriptor later using WSAPI_SetDescriptor. If you supply data storage and pass
	it in the data parameter, you are responsible for deallocating the storage later. If you pass NULL, the
	data is allocated and deallocated by WSAPI automatically.
*/

WSAPI_ErrorCode WSAPI_DisposeDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr *descriptor);

/*  Pass a pointer to a descriptor pointer that needs deallocation and WSAPI will deallocate it.
*/

WSAPI_ErrorCode WSAPI_GetDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr descriptor, 
										OSType *dataType, void *data, unsigned long *dataSize);
/*  Pass a descriptor pointer and WSAPI will return specific information about the descriptor. You 
	must provide storage for all pointers. The data pointer must be large enough to hold the number of
	bytes returned in the dataSize parameter. If the data pointer is NULL, WSAPI still returns the type
	and size information, allowing you to then allocate an appropriately sized data container and 
	call WSAPI_GetDescriptor again with a non-null data pointer. When called, dataSize should be initialized
	to the maximum size of your data buffer. On return, it will contain the actual amount of data stored
	in the data buffer.
*/


WSAPI_ErrorCode WSAPI_SetDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr descriptor, 
									void *data, unsigned long dataSize);
/*  Sets the data and size portions of an existing descriptor pointed to by "descriptor".
	Data is copied from the "data" parameter into the storage allocated inside the descriptor.
	If the existing data area allocated inside the descriptor is too small, the server will attempt
	to resize it to be large enough for dataSize bytes of data. But your plug-in should be prepared
	to handle a failure if the storage cannot be resized.
*/

WSAPI_ErrorCode WSAPI_RegisterAction (WSAPI_CommandPBPtr pb, char *actionTag, char *pluginName);
/*	 Your plug-in must call this routine in response to the WSAPI_Register message in order to 
	define an action that allows WebSTAR to call your plug-in. These actions are temporary and
	only exist while your plug-in is active. actionTag is the name of the action that will
	show up in Admin's transfer type pop-up. pluginName is a string that names your code
	fragment. Both values should be unique among all plug-ins and may be the same value.
*/

WSAPI_ErrorCode WSAPI_RegisterSuffix (WSAPI_CommandPBPtr pb, char *actionTag, char *suffix,
										OSType fileType, OSType creatorType, char *mimeType);
/*	Your plug-in can call this routine in response to the WSAPI_Register message in order to 
	define suffix mappings that allows the server will use with your plug-in. 
	These suffix mappings are temporary and only exist while your plug-in is active. 
	actionTag is the name of the action that will be invoked when a given suffix mapping
	show up in Admin's transfer type pop-up. The suffix parameter is the string that the server
	will match (e.g., ".html") to invoke the specified action.
	The next two fields are machine-specific and make sense on Macs only. The last parameter is
	the MIME type to send with this transfer.
*/

WSAPI_ErrorCode WSAPI_RegisterAccessControl (WSAPI_CommandPBPtr pb, long priority);
/*  By making this call, your plug-in is registered as a handler for access control checks. There
	may be more than one access control handler. Your plug-in will receive a WSAPI_AccessControl message
	when appropriate. You should evaluate the URL and any other information necessary to determine whether or 
	not access should be granted. Fill in the accessControl parameter block fields accordingly. If accessGranted 
	is false, the server will refuse to allow access to that URL and if authRequired is true, it will return an
	appropriate HTTP response to cause the client to prompt for a username and password.
	Note that you are running as a "pre-" preprocessor and should not attempt to access data from the HTTP stream
	or otherwise manipulate information that may affect subsequent processing of the URL. Also, the first
	access control plug-in to refuse access terminates the access control checks, so it's not always the case
	that you will receive every WSAPI_AccessControl message, especially if there are other access control
	handlers installed before yours.
	
	The priority parameter is new in WSAPI/2, and allows some additional flexibility in arranging what order
	the access control plug-ins are called in.  The new frameworks use the older version of the call if the server 
	does not support WSAPI/2.  WSAPI/2 servers should support the old form of the call.
	
	WSAPI/2 plug-ins acting in the AccessControl command should set the name of the authenticator they wish to
	use, and the name of the applicable realm _before_ obtaining piUsernameKeyword.
	
	The priority should be between 1 and 1024.  Lower numbered plug-ins will be executed first.
	
	The return value can be one of three values:  WSAPI_E_MessageNotHandled if the URL does not fall within any
	of the protection spaces defined by this pluig-in.  WSAPI_E_NoPermission if the URL is protected by this 
	plug-in, but the user either doesn't have credentials or the credentials are invalid.  WSAPI_E_NoError if 
	the URL is protected and the user is granted some kind of access.
*/

WSAPI_ErrorCode WSAPI_RegisterAuthenticator(WSAPI_CommandPBPtr pb, char *authName, char *vhostName) ;
/*	By making this call, your plug-in is registered to receive the WSAPI_Authenticate messages.  
	You must provide a name to identify your authenticator, and this name will be used by access control
	plug-ins to indicate when they want to use your authentication.  A plug-in may register more than
	one authenticator.
	
	When called, your plug-in must obtain the username from whatever part of the HTTP request it comes from
	and verify its credentials.  If the credentials are good, then the handler sets piUsernameKeyword to the
	name of the verified user.  If the credentials are bad or non-existent, then the handler sets the 
	appropriate challenge using the indexed piResponseHeaderField keyword.  piRealmName should be available
	for this purpose.
	
	The name and vhost name should be no longer than WSAPI_NAME_SIZE-1 plus the terminating 0 character.  
	Longer names will be truncated.  vhostName can be NULL, if the authenticator is to be available to
	all virtual hosts.
*/

WSAPI_ErrorCode WSAPI_RegisterFilter (WSAPI_CommandPBPtr pb);
/*  By making this call, your plug-in is registered as a filter for incoming HTTP requests. You will be
	sent the WSAPI_Filter message and a PB structure (WSAPI_FilterPB) that contains a pointer to the complete
	HTTP request and its length. This includes all header info up to and including the CRLF CRLF separator between
	the HTTP request and the HTTP object body (if any).
	
	You will receive this message BEFORE the server has interpreted the contents of the HTTP request. 
	You may read any info you'd like from this request buffer, but may not modify its contents. 
	You may also create a new buffer using WSAPI_AllocateMemory and populate it with a new HTTP
	request to be interpeted by the server. Since multiple filters may be installed, there is no guarantee what
	order you will be called in or whether or not subsequent modifications will be made to the request you generate.
	By setting the newRequest field to a value other than null, you inform the server that it should use your
	request instead of the one received from the client. You MUST create a legal HTTP 1.0 request header and
	include the CRLF CRLF separator at the end of the request.
	The server is responsible for deallocating this new request buffer when done.
*/

WSAPI_ErrorCode WSAPI_RegisterFileInfo (WSAPI_CommandPBPtr pb, long priority);
/*	By making this call, your plug-in is registering to receive WSAPI_FileInfo commands for each request.
	The WSAPI_FileInfo command is issued after the WSAPI_Filter comand and after the request has been parsed.
	The purpose of this command is to allow plug-ins to change how the server will handle the request without 
	having to rewrite the entire request.  At a mimimum, a server should allow plug-ins in the WSAPI_FileInfo
	command to set the following parameters for a request:
	
		piURLPhysicalPath - set/change the physical file path.  By default the file path is 
			piServerDirectoryPath + piScriptName, but by changing this for certain URIs a plug-in
			could implement more interesting file mappings (eg: /~username style URIs)
		piActionKeyword - change the Action for the request, and thereby the plug-in or CGI
			that is responsible for it.  By default, the action is "DEFAULT" or the action that
			corresponds to the suffix of the URI.  Normally, a plug-in would use this to give itself
			ownership of a request.
		piFileMIMEType - set the MIME type for the file.  In WebSTAR the default plug-in now keeps the 
			table of suffix mappings, and uses the WSAPI_FileInfo command for setting the MIME type
			of a file.

	The WSAPI_FileInfo command was added because a lot of plug-ins were going to a lot of trouble to rewrite requests 
	simply to gain ownership of them later, or to change other simple aspects of how the request would be handled.  It
	is hoped that this will obsolete most uses of the WSAPI_Filter command.
	
	The priority should be between 1 and 1024.  Lower numbered plug-ins will be executed first.
*/

WSAPI_ErrorCode WSAPI_RegisterLogging (WSAPI_CommandPBPtr pb);
/*##
	By making this call, your plug-in is registered to receive logging messages. Logging plug-ins can perform
	the same functions as post-processors and are invoked immediately after any post-processor function.
	There can be multiple logging plug-ins. 
	
	When a plug-in is invoked with a logging message, the HTTP connection to the client is already closed.
	All connection-related parameters/fields are available to the logging plug-in, but no HTTP communications
	functions will be allowed.
*/

WSAPI_ErrorCode WSAPI_RegisterRouter (WSAPI_CommandPBPtr pb);
/* 
	Registers the plug-in to receive the WSAPI_Route command.
	Returns an error code (WSAPI_TooManyRouters) if a router 
	is already registered.
*/

WSAPI_ErrorCode WSAPI_NewCommandPB(WSAPI_CommandPBPtr pb, WSAPI_CommandPBPtr *customPB) ;
WSAPI_ErrorCode WSAPI_DisposeCommandPB(WSAPI_CommandPBPtr customPB) ;
/*	Allows a plug-in to create an out-of-context pb.  This is useful when a plug-in creates its
	own thread to run outside of the context of the normall commands, but still needs to have
	access to the server callbacks.  Since a valid pb is required for most callbacks, and 
	faking/reusing/copying PBs is extreemly dangerous, you should ask the server to create
	one for you.
	
	Any given PB should only be accessed from one thread at a time.  When a PB is disposed of,
	all of its pooled memory is free'd, too.  So don't plan on using any pooled memory beyond 
	the lifetime of its PB.
*/

/*********************/
/* security services */

WSAPI_ErrorCode WSAPI_ValidateUser (char *name, char *passwd, char *realm);
/*	Using the piUserKeyword, piPasswordKeyword, and piCurrentRealm fields, you can validate whether or not a specific
	user should be allowed access. You may substitute your own values in place of any of the predefined parameters
	if desired. (e.g., pass your own realm string instead of checking only for the current realm.)
	
	Returns WSAPI_I_NoErr if the user, password, and realm are OK. Returns WSAPI_I_NoPermission if the validation
	fails for any reason. Note that this call will return WSAPI_E_MessageNotHandled if an external function is
	providing security controls for the server (e.g., a security plug-in).
*/

/*************************/
/** AppleEvent Services **/

// routines for registering event handlers, sending events, etc.

WSAPI_ErrorCode WSAPI_SendIPCMessage (WSAPI_CommandPBPtr pb, void *msg);
/*	Sends an async, platform-specific interprocess communication message (e.g., AppleEvent on a Mac) that your
	plug-in has already constructed. For Mac O/S, pass a pointer to the AppleEvent structure you wish to send.

	IMPORTANT for WebSTAR Mac plug-ins!!!! 
	For WebSTAR plug-ins, the server generates the appropriate return ID for your AppleEvent, so the value you
	supply to the AECreateAppleEvent call will be overwritten. If you are making this call as part of processing a
	message other than "Run" (i.e., that is NOT associated with a HTTP connection), you should note that your
	plug-in can have at most one pending event because ALL of your other message handlers run in a separate context 
	from a HTTP connection and share a single return ID that WebSTAR uses to connect the asynch event reply to your
	plug-in.
	
	You are responsible for disposing any platform-specific data associated with your message after it
	has been sent (i.e., the event and address descriptors on Macs).
*/

WSAPI_ErrorCode WSAPI_WaitForIPCReply (WSAPI_CommandPBPtr pb, WSAPI_IPC_ResultPtr *msg);
/*	If you send an interprocess communication message (e.g., an AppleEvent under Mac O/S) and expect
	and asynchronous reply received by and delivered through the server to your thread of execution, you
	should call this routine once you expect the reply (i.e., after you've sent the IPC message). If the
	server receives the message reply before the timeout period defined by the server, your reply will be
	returned in the msg parameter. If not, msg will be null and an error code will be returned by the function.
	
	IMPORTANT for WebSTAR Mac plug-ins!!!! 
	When the AppleEvent is returned to you, you MUST resume the AppleEvent with a call to AEResumeTheCurrentEvent.
	(See IM VI, 6-84) This will pass control to the handler you specify (should be a UPP to a procedure in your 
	plug-in.) The handler must process the AE to completion. It is VERY important that your plug-in do this in
	a timely fashion and NOT yield time to other threads. (Well, yielding seems like a bad thing at this point
	because it might allow another plug-in to begin processing an event in parallel, and the AE manager is NOT
	reentrant and cannot process 2 events at once!) This means no callbacks until the event is handled!
	
	If msg is not null, you are responsible for disposing the msg data structure, plus any data passed to 
	you in it if necessary.
*/

/*************************/
/**  Logging Services   **/

//TBS - routines for making log entries, processing log data, etc.

WSAPI_ErrorCode WSAPI_WriteLogEntry (WSAPI_CommandPBPtr pb, char *pluginName, char *msg);
/*	Writes the contents of msg to the server's log file. msg must be a null terminated
	C string. pluginName is a C string (generally your plug-in's name) that will prefix the log entry.
	The combined length of pluginName and msg should be less than 510 characters.
*/


/*****************************/
/** User Interface Services **/

WSAPI_ErrorCode WSAPI_DisplayMessage (WSAPI_CommandPBPtr pb, char *msg);
/*	
	Writes the contents of msg on the server's status window. msg must be a null terminated
	C string.
*/

WSAPI_ErrorCode WSAPI_CreateWindow (WSAPI_CommandPBPtr pb, void *window, char *title, 
			short left, short top, short right, short bottom);
/* ##
	Creates an O/S-specific window, returning a pointer to the appropriate data type (a WindowPtr on Macs)
	if successful or NULL if the call fails. On Macs, do NOT modify the refcon field stored inside the
	WindowPtr. Servers will need to use this info to identify which plug-in to deliver events to.
	Pass a pointer to a WindowPtr in the window parameter.
	
	Events destined for the window will be returned in an EventPB structure sent to your plug-in in a
	WSAPI_Event message.
	
	You must use WSAPI_HideWindow or WSAPI_ActivateWindow to show a newly created window as the window will
	be invisible when created.
*/

WSAPI_ErrorCode WSAPI_DisposeWindow (WSAPI_CommandPBPtr pb, void *window);
/* ##
	Disposes a plug-in window. Any controls or other UI elements associated with the window will NOT
	be deallocated. That is the responsibility of the plug-in.
*/

WSAPI_ErrorCode WSAPI_HideWindow (WSAPI_CommandPBPtr pb, void *window, short hide);
/* ##
	Shows or hides a plug-in window based on the boolean value of the hide parameter.
*/

WSAPI_ErrorCode WSAPI_ActivateWindow (WSAPI_CommandPBPtr pb, void *window);
/* ##
	Brings a plug-in window to the front. If it's an inappropriate time for a window to be
	activated, WSAPI_E_RequestFailed will be returned as an error code. (i.e., a modal dialog is being
	currently being displayed).
*/


/*****************************/
/* Plug-in To Plug-in Services */

typedef unsigned long (*WSAPI_ServiceFunc)(WSAPI_CommandPBPtr pb, void *arg) ;

WSAPI_ErrorCode WSAPI_RegisterService(WSAPI_CommandPBPtr pb, char *name, WSAPI_ServiceFunc func) ;
/*	Use this function to register a function that other plug-ins may use.  If another plug-in has registered
	a service of the same name, you will get a WSAPI_E_DuplicateName error will be returned. Services must 
	be registered during the WSAPI_Register command.  Plug-ins should not attempt to call services until 
	the WSAPI_Init command.
	
	The name must not be longer than WSAPI_INDEX_SIZE.
*/


WSAPI_ErrorCode WSAPI_CallService(WSAPI_CommandPBPtr pb, char *name, void *arg, unsigned long *result) ;
/*	Use this function to call a service.  If the requested service does not exist, then WSAPI_E_BadArgument will
	be returned.  The service will deposit its return code in result.  Calling services in this manner allows
	the server to patch up differences in threading model and calling convention between plug-ins.
*/


/**************************************/
/** User-supplied dispatcher routine **/

WSAPI_ErrorCode WSAPI_Dispatch (WSAPI_CommandPBPtr command);
/*	You must define this routine, which will be called from the WebSTAR Plug-in Framework
	library with a complete WSAPI_CommandPB record passed to your routine.
*/

#ifdef __cplusplus
}
#endif

#if __POWERPC__
#pragma options align=reset
#endif

#endif
