/*
	File:		WSAPI_Main.c

	Contains:	xxx put contents here xxx

	Written by:	Joe Kalash

	Copyright:	 1997 by StarNine Technologies, Inc., all rights reserved.

	Change History (most recent first):

         <4>     4/29/99    cjh     fix WSAPI_CreateWindow return value
		 <3>	  3/6/98	jtk		Add WSAPI_OpenStreamOnSocket.
		 <2>	10/27/97	jtk		Added WSAPI_CloseStreamWithTimeOut(), and
									WSAPI_RegisterRouter().

	To Do:
*/

//WSAPI_Main.c

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "WSAPI.h"
#include "WSAPI_Main.h"

#ifdef __cplusplus
extern "C" {
#endif

int main(WSAPI_ArgsPtr args) ;

#ifdef __cplusplus
}
#endif

WSAPI_CallbacksPtr callbacks = NULL; //container for private callback routine addresses

unsigned long server_impl_version = 0 ;	// the server's WSAPI version.  Important for making sure we don't send callbacks it can't handle
unsigned long wsapi_lib_version = WSAPI_VERSION;
WSAPI_Server_Glue_Type wsapi_glue_type = WSAPI_Mac_Glue;	//what type of glue libraries are being used by the plug-in?

/***************************************************/

int main(WSAPI_ArgsPtr args) {
	int		returnCode;
	
	
	callbacks = args->callbacks;

	if ((args->commandPtr)->command == WSAPI_PrivateInit) {
		//set up private API data
		//need to do much more work here, including verifying the plug-in framework version,
		// crypto handshake, etc.
		
		// remember the server's WSAPI version
		server_impl_version = (args->commandPtr)->api_version ;
		
		//send the plug-in's version info back to the server, only on private init
		(args->commandPtr)->api_version = WSAPI_VERSION;
		returnCode = 0;
	}
	else {
		//set up the appropriate command structure and dispatch the right framework routine
		(args->commandPtr)->api_data = args->cookie;
		returnCode =  WSAPI_Dispatch (args->commandPtr);
	}
	
	return (long)returnCode;
}

/***************************************************/

/** Callback handlers **/

/*************************/
/**  Memory Management  **/

void *WSAPI_AllocateMemory (WSAPI_CommandPBPtr pb, long size)
{
	return (void *) ((WSAPI_CallbackPtrProcPtr) callbacks->addr [cAllocateMemory]) (pb, size);
}

/***************************************************/

void WSAPI_FreeMemory (WSAPI_CommandPBPtr pb, void *data)
{
	(callbacks->addr [cFreeMemory]) (pb, data);
}

/***************************************************/

void *WSAPI_AllocateOSMemory (WSAPI_CommandPBPtr pb, long size)
{
	return (void *) ((WSAPI_CallbackPtrProcPtr) callbacks->addr [cAllocateOSMemory]) (pb, size);
}

/***************************************************/

void WSAPI_FreeOSMemory (WSAPI_CommandPBPtr pb, void *data)
{
	(callbacks->addr [cFreeOSMemory]) (pb, data);
}

/*************************/
/** Process Management  **/

WSAPI_ErrorCode WSAPI_YieldTime (WSAPI_CommandPBPtr pb, long ticksToYield)
{
	return (callbacks->addr [cYieldTime]) (pb, ticksToYield);
}
 
/***************************************************/

WSAPI_ErrorCode WSAPI_RequestIdleTime (WSAPI_CommandPBPtr pb, unsigned long ticksToSleep)
{
	return (callbacks->addr [cRequestIdleTime]) (pb, ticksToSleep);
}

/*************************/
/** Networking Services **/

WSAPI_ErrorCode WSAPI_SendHTTPData (WSAPI_CommandPBPtr pb, void *data, long dataLen)
{
	return (callbacks->addr [cSendHTTPData]) (pb, data, dataLen);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_ReadHTTPData (WSAPI_CommandPBPtr pb, void *data, long *dataLen)
{
	return (callbacks->addr [cReadHTTPData]) (pb, data, dataLen);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_HTTPConnectionStatus (WSAPI_CommandPBPtr pb, WSAPI_ConnectionStatus *status)
{
	return (callbacks->addr [cHTTPConnectionStatus]) (pb, status);
}

//Generic Open, Close, Send, and Receive

/***************************************************/

WSAPI_ErrorCode WSAPI_OpenStream (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, unsigned long ip_addr, unsigned short ip_port)
{
	return (callbacks->addr [cOpenStream]) (pb, stream, ip_addr, ip_port);
}

WSAPI_ErrorCode WSAPI_OpenStreamOnSocket (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, unsigned long ip_addr, unsigned short ip_port, unsigned short localPort)
{
	return (callbacks->addr [cOpenStreamOnSocket]) (pb, stream, ip_addr, ip_port, localPort);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_CloseStream (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream)
{
	return (callbacks->addr [cCloseStream]) (pb, stream);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_CloseStreamWithTimeOut (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, unsigned long timeout)
{
	if (pb->api_version <= WSAPI_Version_12)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cCloseStreamWithTimeout]) (pb, stream, timeout);
	}
}

/***************************************************/

WSAPI_ErrorCode WSAPI_ReadStream (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, void *data, long *dataLen)
{
	return (callbacks->addr [cReadStream]) (pb, stream, data, dataLen);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_WriteStream (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, void *data, long dataLen)
{
	return (callbacks->addr [cWriteStream]) (pb, stream, data, dataLen);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_StatusStream (WSAPI_CommandPBPtr pb, WSAPI_StreamPtr stream, WSAPI_ConnectionStatus *status, long *pending)
{
	return (callbacks->addr [cStatusStream]) (pb, stream, status, pending);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_IPNameToAddr (WSAPI_CommandPBPtr pb, char *name, unsigned long *addr)
{
	return (callbacks->addr [cIPNameToAddr]) (pb, name, addr);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_IPAddrToName (WSAPI_CommandPBPtr pb, unsigned long addr, char *name)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else 
		return (callbacks->addr [cIPAddrToName]) (pb, addr, name);
}

/*************************/
/**  Utility Functions  **/

WSAPI_ErrorCode WSAPI_GetParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords kind, WSAPI_DescPtr *descriptor)
{
	return (callbacks->addr [cGetParameter]) (pb, kind, descriptor);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_SetParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_DescPtr descriptor)
{
	return (callbacks->addr [cSetParameter]) (pb, whichParam, descriptor);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_GetIndexedParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords kind, 
									WSAPI_IndexStringPtr index, WSAPI_DescPtr *descriptor)
{
	if (pb->api_version <= WSAPI_Version_10)
		return WSAPI_E_Unimplemented;
	else 
		return (callbacks->addr [cGetIndexedParameter]) (pb, kind, index, descriptor);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_SetIndexedParameter (WSAPI_CommandPBPtr pb, WSAPI_ParamKeywords whichParam, 
									WSAPI_IndexStringPtr index, WSAPI_DescPtr descriptor)
{
	if (pb->api_version <= WSAPI_Version_10)
		return WSAPI_E_Unimplemented;
	else 
		return (callbacks->addr [cSetIndexedParameter]) (pb, whichParam, index, descriptor);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_NewDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr *descriptor, 
										OSType dataType, void *data, unsigned long dataSize)
{
	return (callbacks->addr [cNewDescriptor]) (pb, descriptor, dataType, data, dataSize);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_DisposeDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr *descriptor)
{
	return (callbacks->addr [cDisposeDescriptor]) (pb, descriptor);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_GetDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr descriptor, 
										OSType *dataType, void *data, unsigned long *dataSize)
{
	return (callbacks->addr [cGetDescriptor]) (pb, descriptor, dataType, data, dataSize);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_SetDescriptor (WSAPI_CommandPBPtr pb, WSAPI_DescPtr descriptor, 
									void *data, unsigned long dataSize)
{
	return (callbacks->addr [cSetDescriptor]) (pb, descriptor, data, dataSize);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterAction (WSAPI_CommandPBPtr pb, char *actionTag, char *pluginName)
{
	return (callbacks->addr [cRegisterAction]) (pb, actionTag, pluginName);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterSuffix (WSAPI_CommandPBPtr pb, char *actionTag, char *suffix,
										OSType fileType, OSType creatorType, char *mimeType)
{
	return (callbacks->addr [cRegisterSuffix]) (pb, actionTag, suffix, fileType, creatorType, mimeType);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterAccessControl (WSAPI_CommandPBPtr pb, long priority)
{
	if (pb->api_version <= WSAPI_Version_10)
		return WSAPI_E_Unimplemented;
	else if(pb->api_version < WSAPI_Version_20) {
		return (callbacks->addr [cRegisterAccessControl]) (pb);
	} else {
		return (callbacks->addr[cRegisterAccessControl_2]) (pb, priority) ;
	}
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterFilter (WSAPI_CommandPBPtr pb)
{
	if (pb->api_version <= WSAPI_Version_10)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cRegisterFilter]) (pb);
	}
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterLogging (WSAPI_CommandPBPtr pb)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cRegisterLogging]) (pb);
	}
}

/***************************************************/

WSAPI_ErrorCode WSAPI_RegisterRouter(WSAPI_CommandPBPtr pb)
{
	if (pb->api_version <= WSAPI_Version_12)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cRegisterRoutingFilter]) (pb);
	}
}

/*********************/
/* Security Services */

WSAPI_ErrorCode WSAPI_ValidateUser (char *name, char *passwd, char *realm)
{
	return (callbacks->addr [cValidateUser]) ((WSAPI_CommandPBPtr)name, passwd, realm);
}


/*************************/
/** AppleEvent Services **/

WSAPI_ErrorCode WSAPI_SendIPCMessage (WSAPI_CommandPBPtr pb, void *msg)
{
	return (callbacks->addr [cSendIPCMessage]) (pb, msg);
}

/***************************************************/

WSAPI_ErrorCode WSAPI_WaitForIPCReply (WSAPI_CommandPBPtr pb, WSAPI_IPC_ResultPtr *msg)
{
	return (callbacks->addr [cWaitForIPCMessage]) (pb, msg);
}

/*************************/
/**  Logging Services   **/

WSAPI_ErrorCode WSAPI_WriteLogEntry (WSAPI_CommandPBPtr pb, char *pluginName, char *msg)
{
	return (callbacks->addr [cWriteLogEntry]) (pb, pluginName, msg);
}

/*************************/
/** Status Msg Services **/

WSAPI_ErrorCode WSAPI_DisplayMessage (WSAPI_CommandPBPtr pb, char *msg)
{
	return (callbacks->addr [cDisplayMessage]) (pb, msg);
}

WSAPI_ErrorCode WSAPI_CreateWindow (WSAPI_CommandPBPtr pb, void *window, char *title, 
			short left, short top, short right, short bottom)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return  (callbacks->addr [cCreateWindow]) (pb, window, title, left, top, right, bottom);
	}
}

WSAPI_ErrorCode WSAPI_DisposeWindow (WSAPI_CommandPBPtr pb, void *window)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cDisposeWindow]) (pb, window);
	}
}

WSAPI_ErrorCode WSAPI_HideWindow (WSAPI_CommandPBPtr pb, void *window, short hide)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cHideWindow]) (pb, window, hide);
	}
}


WSAPI_ErrorCode WSAPI_ActivateWindow (WSAPI_CommandPBPtr pb, void *window)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cActivateWindow]) (pb, window);
	}
}


WSAPI_ErrorCode WSAPI_ListenStream (WSAPI_CommandPBPtr pb, unsigned short port, unsigned short maxConnections,
									unsigned long suggestedSize, WSAPI_ListenerRef *refCon)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cListenStream]) (pb, port, maxConnections, suggestedSize, refCon);
	}
}


WSAPI_ErrorCode WSAPI_ListenCancel (WSAPI_CommandPBPtr pb, WSAPI_ListenerRef refCon)
{
	if (pb->api_version <= WSAPI_Version_11)
		return WSAPI_E_Unimplemented;
	else {
		return (callbacks->addr [cListenCancel]) (pb, refCon);
	}
}


/*	WSAPI/2 Callbacks */

void *WSAPI_ReallocateMemory(WSAPI_CommandPBPtr pb, void *data, long size) {
	if(server_impl_version < WSAPI_Version_20)
		return NULL ;
	else
		return (void*)(callbacks->addr[cReallocateMemory]) (pb, data, size) ;
}


void *WSAPI_AllocatePoolMemory (WSAPI_CommandPBPtr pb, long size) {
	if(server_impl_version < WSAPI_Version_20)
		return NULL ;
	else
		return (void*)(callbacks->addr[cAllocatePoolMemory]) (pb, size) ;
}


void *WSAPI_ReallocatePoolMemory(WSAPI_CommandPBPtr pb, void *data, long size) {
	if(server_impl_version < WSAPI_Version_20)
		return NULL ;
	else
		return (void*)(callbacks->addr[cReallocatePoolMemory]) (pb, data, size) ;
}

void WSAPI_FreePoolMemory (WSAPI_CommandPBPtr pb, void *data) {
	if(server_impl_version < WSAPI_Version_20)
		 ;
	else
		(callbacks->addr[cFreePoolMemory]) (pb, data) ;
}


WSAPI_ErrorCode WSAPI_RegisterAuthenticator(WSAPI_CommandPBPtr pb, char *name, char *vhostName) {
	if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cRegisterAuthenticator]) (pb, name, vhostName) ;
}


WSAPI_ErrorCode WSAPI_NewCommandPB(WSAPI_CommandPBPtr pb, WSAPI_CommandPBPtr *customPB) {
	if(!customPB)
		return WSAPI_E_BadArgument ;
	else if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cNewCommandPB]) (pb, customPB) ;
}


WSAPI_ErrorCode WSAPI_DisposeCommandPB(WSAPI_CommandPBPtr customPB) {
	if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cDisposeCommandPB]) (customPB) ;
}

WSAPI_ErrorCode WSAPI_RegisterService(WSAPI_CommandPBPtr pb, char *name, WSAPI_ServiceFunc func) {
	if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cRegisterService]) (pb, name,func) ;
}

WSAPI_ErrorCode WSAPI_CallService(WSAPI_CommandPBPtr pb, char *name, void *arg, unsigned long *result) {
	if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cCallService]) (pb, name, arg, result) ;
}

WSAPI_ErrorCode WSAPI_RegisterFileInfo (WSAPI_CommandPBPtr pb, long priority) {
	if(server_impl_version < WSAPI_Version_20)
		return WSAPI_E_Unimplemented ;
	else
		return (callbacks->addr[cRegisterFileInfo]) (pb, priority) ;

}

/*
	if(pb->api_version < WSAPI_Version_20)
		return NULL ;
	else
		return (callbacks->addr[]) (pb) ;
*/
