// This is the main DLL file.

#include "stdafx.h"
#include "ADO_DS.NET.h"
#include <cstring>

using System::Runtime::InteropServices::GCHandle;
using namespace System::Data::OleDb;
using System::Runtime::InteropServices::Marshal;
using System::String;
using System::TypeCode;
using System::Reflection::BindingFlags;

void registerLassoModule()
{
	lasso_registerDSModule( "Lasso Connector for ADO.NET", ADO_DS_handler, 0 );
}

IDbConnection * doConnect(lasso_request_t token)
{
	lasso_dsconnection_t test = NULL;
	if (lasso_getDSConnection(token, &test)==osErrNoErr && test != NULL)
	{		
		return __try_cast<IDbConnection*>(GCHandle::op_Explicit((int)test).Target);
	}
	IDbConnection * conn = NULL;

	auto_lasso_value_t host, auth;
	lasso_getDataHost(token, &host, &auth);

	if (host.dataSize) // connection class name
	{
		System::Type * type = System::Type::GetType(new System::String(host.data), true, true);
		if (type)
		{
			System::Object* obj = System::Activator::CreateInstance(type);
			conn = __try_cast<IDbConnection*>(obj);
		}
	}
	else
	{
		conn = new OleDbConnection(new System::String(host.name)); // default
	}
	if (conn != NULL)
	{
		lasso_setDSConnection(token, (lasso_dsconnection_t)GCHandle::op_Explicit(GCHandle::Alloc(conn)).ToInt32());
	}
	return conn;	
}

osError ADO_DS_handler(lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param)
{
	osError err = osErrNoErr;
	try
	{
		switch(action)
		{
			case datasourceInit:
				// nothing!
				break;
			case datasourceTerm:
				// nothing!
				break;
			case datasourceCloseConnection: // connections only get closed through here
				{
					lasso_dsconnection_t conn = NULL;
					if ( lasso_getDSConnection(token, &conn) == osErrNoErr && conn )
					{
						GCHandle::op_Explicit((int)conn).Free();
					}
				}
				break;
			case datasourceTickle:
				{
					doConnect(token);
				}
				break;	
			case datasourceNames:
				{
					IDbConnection * conn = doConnect(token);
					conn->Open();

					OleDbConnection * oleConn = __try_cast<OleDbConnection*>(conn);
					if (oleConn)
					{
						try
						{
							System::Object * restrictions[] = {NULL, NULL, NULL};
							DataTable * schemaTbl = oleConn->GetOleDbSchemaTable(OleDbSchemaGuid::Schemata, restrictions);
							System::Collections::IEnumerator* myEnum1 = schemaTbl->Rows->GetEnumerator();
							while (myEnum1->MoveNext())
							{
								DataRow* myRow = __try_cast<DataRow*>(myEnum1->Current);
								String * s = __try_cast<String*>(myRow->Item[S"CATALOG_NAME"]);
								char* str2 = (char*)(void*)Marshal::StringToHGlobalAnsi(s);
								lasso_addDataSourceResult(token, str2);				
								Marshal::FreeHGlobal(str2);
							}
						}
						catch(System::ArgumentException *)
						{
							String * tst = conn->Database;
							if (tst == NULL || tst->Length == 0)
								tst = oleConn->DataSource;
							char* str2 = (char*)(void*)Marshal::StringToHGlobalAnsi(tst);
							lasso_addDataSourceResult(token, str2);				
							Marshal::FreeHGlobal(str2);
						}
					}
					conn->Close();
				}
				break;
			case datasourceTableNames:
				{
					IDbConnection * conn = doConnect(token);
					conn->Open();
					conn->ChangeDatabase(new System::String(param->name));

					OleDbConnection * oleConn = dynamic_cast<OleDbConnection*>(conn);
					if (oleConn)
					{
						try
						{
							System::Object * restrictions[] = {0, 0, 0, S"TABLE"};
							DataTable * schemaTbl = oleConn->GetOleDbSchemaTable(OleDbSchemaGuid::Tables, restrictions);
							System::Collections::IEnumerator* myEnum1 = schemaTbl->Rows->GetEnumerator();
							while (myEnum1->MoveNext())
							{
								DataRow* myRow = __try_cast<DataRow*>(myEnum1->Current);
								String * s = __try_cast<String*>(myRow->Item[S"TABLE_NAME"]);
								char* str2 = (char*)(void*)Marshal::StringToHGlobalAnsi(s);
								lasso_addDataSourceResult(token, str2);				
								Marshal::FreeHGlobal(str2);
							}
						}
						catch(System::ArgumentException *)
						{
							
						}
					}

					conn->Close();
				}
				break;
			case datasourceInfo:
				{
					auto_lasso_value_t dbName;
					auto_lasso_value_t tableName;

					lasso_getDataSourceName( token, &dbName, NULL, NULL );
					lasso_getTableName( token, &tableName );
					if ( tableName.data[0] != '\0' )
					{
						IDbConnection * conn = doConnect(token);
						conn->Open();
						conn->ChangeDatabase(new System::String(dbName.name));
						OleDbConnection * oleConn = dynamic_cast<OleDbConnection*>(conn);
						if (oleConn)
						{
							try
							{
								System::Object * restrictions[] = {NULL, NULL, new System::String(tableName.name), NULL};
								DataTable * schemaTbl = oleConn->GetOleDbSchemaTable(OleDbSchemaGuid::Columns, restrictions);
								System::Collections::IEnumerator* myEnum1 = schemaTbl->Rows->GetEnumerator();
								while (myEnum1->MoveNext())
								{
									DataRow* myRow = __try_cast<DataRow*>(myEnum1->Current);
									String * s = __try_cast<String*>(myRow->Item[S"COLUMN_NAME"]);
									char* str2 = (char*)(void*)Marshal::StringToHGlobalAnsi(s);
									lasso_addColumnInfo(token, str2, true, lpTypeString, kProtectionNone);
									Marshal::FreeHGlobal(str2);
								}
							}
							catch(System::ArgumentException *)
							{

							}
						}
						conn->Close();                    
					}
				}
				break;
			case datasourceExecSQL:
				if ( param != NULL && param->data != NULL )
				{
					IDbConnection * conn = doConnect(token);
					auto_lasso_value_t dbName;
					lasso_getDataSourceName( token, &dbName, NULL, NULL );
					conn->Open();
					conn->ChangeDatabase(new System::String(dbName.name));

					IDbCommand * cmd = conn->CreateCommand();
					cmd->CommandText = new System::String(param->data);
					IDataReader * reader = cmd->ExecuteReader();
					int cols = reader->FieldCount;

					LP_TypeDesc * types = new LP_TypeDesc[cols];
					char ** rows = new char*[cols];
					unsigned long * sizes = new unsigned long[cols];
					int found = 0;
					addColumnInfo(token, reader->GetSchemaTable(), types);
					while(reader->Read())
					{
						++found;
						for (int x = 0; x < cols; ++x)
						{
							switch(types[x])
							{
							case lpTypeBytes:
								{
									sizes[x] = (unsigned int)reader->GetBytes(x, 0, NULL, 0, 0);
									rows[x] = new char[sizes[x]];
									unsigned char tmp __gc[] =  new unsigned char __gc[sizes[x]];
									reader->GetBytes(x, 0, tmp, 0, sizes[x]);
									GCHandle handle = GCHandle::Alloc(tmp);
									System::IntPtr ptr = Marshal::UnsafeAddrOfPinnedArrayElement(tmp, 0);
                                    memcpy(rows[x], (const void*)ptr, sizes[x]);
									handle.Free();
								}
								break;
							case lpTypeString:
							case lpTypeInteger:
							case lpTypeDecimal:
							case lpTypeBoolean:
							case kDateDataType:
							default:
								{
									rows[x] = (char*)(void*)Marshal::StringToHGlobalAnsi(reader->GetValue(x)->ToString());	
									sizes[x] = strlen(rows[x]);
								}
								break;
							}
						}
						lasso_addResultRow(token, (const char**)rows, sizes, cols);
						for (int x = 0; x < cols; ++x)
						{
							switch(types[x])
							{
							case lpTypeBytes:
								{
									delete [] rows[x];
								}
								break;
							case lpTypeString:
							case lpTypeInteger:
							case lpTypeDecimal:
							case lpTypeBoolean:
							case kDateDataType:
							default:
								{
									Marshal::FreeHGlobal(rows[x]);
								}
								break;
							}
						}
					}

					lasso_setNumRowsFound( token, found );
					delete [] sizes;
					delete [] rows;
					delete [] types;

					reader->Close();				
					conn->Close();
				}
				break;
		}
	}
	catch(System::Exception * e)
	{
		char* str2 = (char*)(void*)Marshal::StringToHGlobalAnsi(e->ToString());
		lasso_setResultMessage(token, str2);
		Marshal::FreeHGlobal(str2);
		return osErrDatasourceError;
	}
	return err;
}

void addColumnInfo(lasso_request_t token, DataTable * table, LP_TypeDesc * types)
{
	System::Collections::IEnumerator* myEnum1 = table->Rows->GetEnumerator();
	int count = 0;
	while (myEnum1->MoveNext())
	{
		DataRow* myRow = __try_cast<DataRow*>(myEnum1->Current);
		
		LP_TypeDesc type = lpTypeString;
		System::Type * o = __try_cast<System::Type*>(myRow->Item[S"DataType"]);
				
		switch(System::Type::GetTypeCode(o))
		{
			case TypeCode::Boolean:
				type = lpTypeBoolean;
				break;
			case TypeCode::Byte:
			case TypeCode::Int16:
			case TypeCode::Int32:
			case TypeCode::Int64:
			case TypeCode::UInt16:
			case TypeCode::UInt32:
			case TypeCode::UInt64:
			case TypeCode::SByte:
			case TypeCode::Char:
				type = lpTypeInteger;
				break;
			case TypeCode::DateTime:
				type = kDateDataType;
				break;
			case TypeCode::Decimal:
			case TypeCode::Double:
			case TypeCode::Single:
				type = lpTypeDecimal;
				break;
			case TypeCode::DBNull:
			case TypeCode::Empty:
				type = lpTypeNull;
			case TypeCode::String:
			case TypeCode::Object:
			default:
				type = lpTypeString;
				break;			
		}
		
		types[count] = type;
		char* name = (char*)(void*)Marshal::StringToHGlobalAnsi(__try_cast<System::String*>(myRow->Item[S"ColumnName"]));
		lasso_addColumnInfo(token, name, System::Convert::ToBoolean(myRow->Item[S"AllowDBNull"]), type, kProtectionNone);
		Marshal::FreeHGlobal(name);	
		++count;
	}
}
