#include "MySQLDS.h"
#include <stdio.h>
#include <stack>
#include <vector>
#include <list>
#include <ctype.h>
#include <utility>
//#if MACHINE_LINUX
#include <memory>
//#endif

static const char * kRowCountVar = "__updated_count__";
static const char * kGoofyUTF8 = "utf8";
static const char * kGoofyISO1 = "latin1";

#define MYSQL_VERSION_4 40000
#define MYSQL_VERSION_41 40100

#ifdef WIN32
#define strcasecmp stricmp
#endif

struct ValStore
{
	LP_TypeDesc type;
	unsigned long len;
	union
	{
		double d;
		osInt64 i;
		void * b;
		const char * c;
	} u;
};

MYSQL * local_doConnect( lasso_request_t token );

void local_addReturnFields( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to );
bool local_addSearchCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to, LP_TypeDesc defaultOp, datasource_action_t action, bool useLimit );
void local_addUpdateCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to );
void local_addSortOrder( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to );
void local_addSearchValue( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to, const char * value, unsigned int size, LP_TypeDesc op, bool doWildcards );
//bool local_addPrimaryKeySearch( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to );
void local_addColumnInfo( lasso_request_t token, MYSQL_RES * res, MYSQL * mysql, const char * db, const char * table );
void local_addOperator( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to, LP_TypeDesc op, const char * value, unsigned int size );
void local_addInsertCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & query );

static void threadDone(void*)
{
//	mysql_thread_init();
	mysql_thread_end();
}

void registerLassoModule()
{
	lasso_registerDSModule( "MySQLDS", local_mysqlds_func, 0 );
	lasso_log(LOG_LEVEL_ALWAYS, "MySQLConnector using MySQL client: %s",  mysql_get_client_info());
	lasso_addThreadDoneFunc(threadDone, NULL);

	// fudge initializing MySQL. it does some funky thread stuff which breaks on 10.3.9
	// if you call mysql_thread_end without ever having called mysql_init
	mysql_close(mysql_init(NULL));
}

inline osError CheckError(lasso_request_t token, MYSQL * m)
{
	if ( mysql_errno(m))
	{
		lasso_log(LOG_LEVEL_WARNING, "MySQLDS: error from mysql: %d, %s",  mysql_errno(m),  mysql_error(m));
		lasso_setResultMessage(token,  mysql_error(m));
		return (osError)mysql_errno(m);
	}
	return osErrNoErr;
}

inline const char * convertFromTableEncToISO(lasso_request_t token, MYSQL * mysql, const char * theText)
{
	const char * connEnc = mysql_character_set_name(mysql);
	if (strcasecmp(connEnc, "latin1") == 0 || strcasecmp(connEnc, "latin2") == 0)
		return theText;
	lasso_type_t theUnicodeStr = NULL;
	lasso_typeAllocStringConv(token, &theUnicodeStr, theText, strlen(theText), connEnc);
	auto_lasso_value_t converted;
	INITVAL(&converted);
	lasso_typeGetStringConv(token, theUnicodeStr, &converted, kLPNameEncodingType);
	return converted.name;
}

inline const char * convertFromISOToTableEnc(lasso_request_t token, MYSQL * mysql, const char * theText)
{
	const char * connEnc = mysql_character_set_name(mysql);
	if (strcasecmp(connEnc, "latin1") == 0 || strcasecmp(connEnc, "latin2") == 0)
		return theText;
	lasso_type_t theUnicodeStr = NULL;
	lasso_typeAllocStringConv(token, &theUnicodeStr, theText, strlen(theText), kLPNameEncodingType);
	auto_lasso_value_t converted;
	INITVAL(&converted);
	lasso_typeGetStringConv(token, theUnicodeStr, &converted, connEnc);
	return converted.name;
}
#define AddColumnName AddTickedName
inline STRING_TYPE & AddTickedName(lasso_request_t token, MYSQL * mysql, const char * in, STRING_TYPE & out)
{
	if (in)
	{
		in = convertFromISOToTableEnc(token, mysql, in);
		int x = 0;
		out.APND("`");
		if (in[x] == '`')
			++x;
		while(in[x])
		{
			if (in[x] == '`' || in[x] == '#' || (in[x] == '-' && in[x+1] == '-') || (in[x] == '/' && in[x] == '*'))
				break;
			out.APND(in+x, 1);
			++x;
		}
		out.APND("`");
	}
	return out;
}

inline void setConnectionCharset(lasso_request_t token, MYSQL * mysql)
{
	auto_lasso_value_t tableEncoding, extra;
	INITVAL(&tableEncoding);
	//INITVAL(&extra);
	lasso_getDataHostExtra(token, &extra);

	/*if (extra.name && tolower(*extra.name) == 'y')
	{*/
		lasso_getTableEncoding(token, &tableEncoding);
		const char * to = tableEncoding.name;
		if (to && *to)
		{
			if (strcasecmp(to, kLPUTF8EncodingType) == 0)
				to = kGoofyUTF8;
			else if (strcasecmp(to, kLPNameEncodingType) == 0)
				to = kGoofyISO1;
			else if (strcasecmp(to, "iso8859-1") == 0)
				to = kGoofyISO1;
			mysql_set_character_set(mysql, (char*)to);
		}
	//}
}

MYSQL * local_doConnect( lasso_request_t token )
{
	// updated to work with LP7 datasource connection caching
	lasso_dsconnection_t test = NULL;
	if (lasso_getDSConnection(token, &test)==osErrNoErr && test != NULL)
	{
//		lasso_log(LOG_LEVEL_DETAIL, "MySQLDS using cached connection %d\n", (int)test);
		auto_lasso_value_t db;
		INITVAL(&db);
		lasso_getDataSourceName(token, &db, NULL, NULL);
		mysql_select_db(reinterpret_cast<MYSQL*>(test), db.data);
		setConnectionCharset(token, reinterpret_cast<MYSQL*>(test));
		return reinterpret_cast<MYSQL*>(test);
	}
	auto_lasso_value_t host, auth;
	lasso_getDataHost(token, &host, &auth);

	MYSQL * worked = NULL;

	worked =  mysql_init(NULL);
	my_bool  my_true = true;
	mysql_options(worked, MYSQL_OPT_RECONNECT, &my_true);
	mysql_options(worked, MYSQL_OPT_LOCAL_INFILE, NULL);
	
	auto_lasso_value_t timeoutVal;
	INITVAL(&timeoutVal);
	if (lasso_findInputColumn(token, "-timeout", &timeoutVal) == osErrNoErr && timeoutVal.data != NULL)
	{
		unsigned int timeout = atoi(timeoutVal.data);
		mysql_options(worked, MYSQL_OPT_CONNECT_TIMEOUT, (char*)&timeout);
		mysql_options(worked, MYSQL_OPT_READ_TIMEOUT, (char*)&timeout);
		mysql_options(worked, MYSQL_OPT_WRITE_TIMEOUT, (char*)&timeout);
	}
	else
	{
		unsigned int timeout = 5;
		mysql_options(worked, MYSQL_OPT_CONNECT_TIMEOUT, (char*)&timeout);
	}
	
	if ( mysql_real_connect(worked, host.name, auth.name, auth.data, NULL, atoi(host.data), NULL, CLIENT_FOUND_ROWS|CLIENT_MULTI_STATEMENTS) == NULL )
	{
		lasso_log(LOG_LEVEL_CRITICAL, "MySQLDS failed connecting to MySQL. errno=%d \"%s\"",  mysql_errno(worked),  mysql_error(worked));
		CheckError(token, worked);
		 mysql_close(worked);
		worked = NULL;
	}
	else
	{
//		printf("MYSQL VERSION %d\n", mysql_get_server_version(worked));
//		lasso_log(LOG_LEVEL_DETAIL, "MySQLDS caching new connection %d\n", (int)worked);
		auto_lasso_value_t db;
		INITVAL(&db);
		lasso_getDataSourceName(token, &db, NULL, NULL);
		mysql_select_db(worked, db.data);
		setConnectionCharset(token, worked);
		lasso_setDSConnection(token, reinterpret_cast<lasso_dsconnection_t>(worked));
	}
	return worked;
}

osError local_mysqlds_func( lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param )
{
	bool statOnly = false;
	lasso_getIsStatementOnly(token, &statOnly);

	osError err = osErrNoErr;
	switch( action )
	{
		case datasourceMatchesName:
			{
				if (strcasecmp(param->name, "MySQLDS") == 0 ||
					strcasecmp(param->name, "MySQL") == 0 ||
					strcasecmp(param->name, "MySQLConnector") == 0||
					strcasecmp(param->name, "Lasso Connector for MySQL") == 0)
				{
					SET_MATCHED_NAME(param);
					return osErrNoErr;
				}
				return osErrInvalidParameter;
			}
			break;
		case datasourceInit:
			// nothing!
			break;
		case datasourceTerm:
			// nothing!
			break;
		case datasourceCloseConnection: // connections only get closed through here
			{
				lasso_dsconnection_t conn = NULL;
				if ( lasso_getDSConnection(token, &conn) == osErrNoErr && conn )
				{
//					lasso_log(LOG_LEVEL_DETAIL, "MySQLDS closing cached connection %d\n", (int)conn);
					 mysql_close(reinterpret_cast<MYSQL*>(conn));
				}
			}
			break;
		case datasourceTickle:
			{
				MYSQL * conn = local_doConnect(token);
			}
			break;
		case datasourceNames:
			{
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				MYSQL_RES * res = NULL;
				MYSQL_ROW row;

				if (  mysql_query(mysql, "show databases") == 0 )
				{
					res =  mysql_store_result(mysql);
					if ( res != NULL )
					{
						while( (row =  mysql_fetch_row(res)) != NULL )
						{
							lasso_addDataSourceResult( token, convertFromTableEncToISO(token, mysql, row[0]) );
						}
						 mysql_free_result(res);
					}
				}
//				mysql_close(mysql);
			}
			break;
		case datasourceTableNames:
			{
				MYSQL_RES * res = NULL;

				auto_lasso_value_t db;

				lasso_getDataSourceName(token, &db, NULL, NULL);

				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				res =  mysql_list_tables(mysql, NULL);
				if ( res != NULL )
				{
					MYSQL_ROW row;
					while( (row =  mysql_fetch_row(res)) != NULL )
					{
						lasso_addDataSourceResult(token, convertFromTableEncToISO(token, mysql, row[0]));
					}
					 mysql_free_result(res);
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceSearch:
		case datasourceFindAll:
			{
				auto_lasso_value_t dbName, tableName, test;

				STRING_TYPE query;
				bool useLimit = lasso_findInputColumn( token, "-uselimit", &test ) == osErrNoErr;

				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );

				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				bool mysql4OrLater = mysql_get_server_version(mysql) >= MYSQL_VERSION_4;
				query.APND("SELECT ");

				/* SELECT (<result_fields>) FROM <databasename>.<table_name> WHERE <search_fields> ORDER BY <sort_fields> */
				if (!useLimit && mysql4OrLater)
					query.APND("SQL_CALC_FOUND_ROWS ");

				if ( lasso_findInputColumn( token, "-distinct", &test ) == osErrNoErr )
				{
					query.APND("DISTINCT ");
				}

				local_addReturnFields( token, mysql, query );

			query.APND(" FROM ");
			AddTickedName(token, mysql, dbName.data, query);
			query.APND(".");
			AddTickedName(token, mysql, tableName.data, query);

				osInt64 tmp;
				lasso_getRowID(token, &tmp);
				if ( tmp != kLassoAny )
				{
					local_addSearchCriteria( token, mysql, query, lpTypeNull, action, useLimit || mysql4OrLater );

					lasso_setActionStatement(token, query.CHAR_PTR());
					if (statOnly)
						break;
					lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

					if (  mysql_query(mysql, query.CHAR_PTR()) == 0 )
					{
						MYSQL_ROW row;
						MYSQL_RES * res =  mysql_store_result(mysql);

						if ( res != NULL )
						{
							int num =  mysql_num_fields(res), skip=0, max=0, found;

							if (!useLimit && mysql4OrLater && mysql_query(mysql, "SELECT FOUND_ROWS()") == 0)
							{
								MYSQL_ROW row2;
								MYSQL_RES * res2 =  mysql_store_result(mysql);
								if ( (row2 = mysql_fetch_row(res2)) != NULL)
									found = atoi(row2[0]);
								mysql_free_result(res2);
							}
							else
							{
								found = (int) mysql_num_rows(res);
							}
							lasso_setNumRowsFound( token, found );

							// we have to set the column info
							local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );

							// if the query used LIMIT then don't do skip and max. just get them all
							if ( useLimit || mysql4OrLater )
							{
								max = -1;
							}
							else
							{
								lasso_getSkipRows( token, &skip );
								lasso_getMaxRows( token, &max );
							}

							if ( max > -1 && skip > 0 )
								 mysql_data_seek( res, skip );

							while( (row = mysql_fetch_row(res)) != NULL && (max == -1 || max--) )
							{
								unsigned long * lengths =  mysql_fetch_lengths(res);
								lasso_addResultRow( token, (const char**)row, lengths, num );
							}
							mysql_free_result(res);
						}
					}
				}
				else
				{
					query.APND(" ORDER BY RAND() LIMIT 1");

					lasso_setActionStatement(token, query.CHAR_PTR());
					if (statOnly)
						break;
					lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

					MYSQL_ROW row;
					MYSQL_RES * res = NULL;

					int result =  mysql_query(mysql, query.CHAR_PTR());
					if ( result == 0 )
					{
						res =  mysql_store_result(mysql);
					}

					if ( res != NULL )
					{
						int num =  mysql_num_fields(res);
						// we have to set the column info
						local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
						int found = (int) mysql_num_rows(res);

						lasso_setNumRowsFound( token, found );

						row =  mysql_fetch_row(res);
						if ( row != NULL )
						{
							unsigned long * lengths =  mysql_fetch_lengths(res);
							lasso_addResultRow( token, (const char**)row, lengths, num );
						}
						 mysql_free_result(res);
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceAdd:
			 {
				auto_lasso_value_t dbName, tableName;

				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );

				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				STRING_TYPE query;
			query.APND("INSERT INTO ");
			AddTickedName(token, mysql, dbName.data, query);
			query.APND(".");
			AddTickedName(token, mysql, tableName.data, query);

				local_addInsertCriteria(token, mysql, query);

				lasso_setActionStatement(token, query.CHAR_PTR());
				if (statOnly)
					break;
				lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

				if ( mysql_query( mysql, query.CHAR_PTR() ) == 0)
				{
					if (  mysql_affected_rows(mysql) == 0 )
					{
						err = osErrWebAddError;
						lasso_setVariable( token, kRowCountVar, "0", 0 );
					}
					else
					{
						char rowcount[16] = "";
						sprintf( rowcount, "%lu", (unsigned long) mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}
					if( err == osErrNoErr )
					{
						lasso_setRowID( token, (unsigned long)  mysql_insert_id( mysql ) );
						int c;
						auto_lasso_value_t key;
						lasso_getMaxRows(token, &c);
						if ( c > 0 && lasso_getPrimaryKeyColumn( token, &key ) == osErrNoErr && key.nameSize > 0 )
						{
							query = "SELECT ";
							local_addReturnFields(token, mysql, query);
						query.APND(" FROM ");
						AddTickedName(token, mysql, dbName.data, query);
						query.APND(".");
						AddTickedName(token, mysql, tableName.data, query);
						query.APND(" WHERE ");
						AddTickedName(token, mysql, key.name, query);
							query.APND(" = LAST_INSERT_ID()");
							int result =  mysql_query(mysql, query.CHAR_PTR());
							if ( result == 0 )
							{
								MYSQL_RES * res =  mysql_store_result(mysql);
								int num =  mysql_num_fields(res);
								local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
								MYSQL_ROW row =  mysql_fetch_row(res);
								if ( row != NULL )
								{
									lasso_setNumRowsFound( token, mysql_num_rows(res) );
									unsigned long * lengths =  mysql_fetch_lengths(res);
									lasso_addResultRow( token, (const char**)row, lengths, num );
									mysql_free_result(res);
								}
							}
						}
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceUpdate:
			 {
				auto_lasso_value_t db, table;
				lasso_getDataSourceName(token, &db, NULL, NULL);
				lasso_getTableName(token, &table);

				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				STRING_TYPE query;
			query.APND("UPDATE ");
			AddTickedName(token, mysql, db.data, query);
			query.APND(".");
			AddTickedName(token, mysql, table.data, query);

				local_addUpdateCriteria( token, mysql, query );
				bool useLimit = false;
				if ( !local_addSearchCriteria( token, mysql, query, lpTypeNull, action, useLimit) )
				{
					return osErrWebUpdateError;
				}

				lasso_setActionStatement(token, query.CHAR_PTR());
				if (statOnly)
					break;
				lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

				if ( mysql_query(mysql, query.CHAR_PTR()) == 0)
				{
					if (  mysql_affected_rows(mysql) > 0 )
					{
						lasso_setVariable( token, kRowCountVar, "0", 0 );
						int c;
						lasso_getMaxRows(token, &c);
						if ( c > 0 )
						{
							query = "SELECT ";
							local_addReturnFields(token, mysql, query);
						query.APND(" FROM ");
						AddTickedName(token, mysql, db.data, query);
						query.APND(".");
						AddTickedName(token, mysql, table.data, query);
							bool useLimit = false;
							local_addSearchCriteria( token, mysql, query, lpTypeNull, action, useLimit);

							int result =  mysql_query(mysql, query.CHAR_PTR());
							if ( result == 0 )
							{
								MYSQL_RES * res =  mysql_store_result(mysql);
								int num =  mysql_num_fields(res);
								local_addColumnInfo( token, res, mysql, db.data, table.data );
								MYSQL_ROW row =  mysql_fetch_row(res);
								if ( row != NULL )
								{
									lasso_setNumRowsFound( token, mysql_num_rows(res) );
									unsigned long * lengths =  mysql_fetch_lengths(res);
									lasso_addResultRow( token, (const char**)row, lengths, num );
									mysql_free_result(res);
								}
							}
						}
					}
					else
					{
						char rowcount[16] = "";
						sprintf( rowcount, "%lu", (unsigned long)  mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceDelete:
			 {
				auto_lasso_value_t db, table;

				lasso_getDataSourceName( token, &db, NULL, NULL );
				lasso_getTableName( token, &table );

				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				STRING_TYPE query;
			query.APND("DELETE FROM ");
			AddTickedName(token, mysql, db.data, query);
			query.APND(".");
			AddTickedName(token, mysql, table.data, query);
				bool useLimit = false;
				if ( !local_addSearchCriteria( token, mysql, query, lpTypeNull, action, useLimit) )
				{
					return osErrWebDeleteError;
				}

				lasso_setActionStatement(token, query.CHAR_PTR());
				if (statOnly)
					break;
				lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

				if ( mysql_query(mysql, query.CHAR_PTR()) == 0)
				{
					if (  mysql_affected_rows(mysql) == 0 )
					{
						err = osErrWebDeleteError;
						lasso_setVariable( token, kRowCountVar, "0", 0 );
					}
					else
					{
						char rowcount[16] = "";
						sprintf( rowcount, "%lu", (unsigned long) mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceInfo:
			 {
				auto_lasso_value_t dbName;
				auto_lasso_value_t tableName;

				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );
				if ( tableName.data[0] != '\0' )
				{
					MYSQL * mysql = local_doConnect(token);
					if ( mysql == NULL )
						return osErrConnectionInvalid;

				STRING_TYPE query;
					/* no result set but we get the column info */
				query.APND("SELECT * FROM ");
				AddTickedName(token, mysql, dbName.data, query);
				query.APND(".");
				AddTickedName(token, mysql, tableName.data, query);
				query.APND(" WHERE 1=0");

				lasso_setActionStatement(token, query.CHAR_PTR());
					if (statOnly)
						break;
				lasso_log(LOG_LEVEL_SQL, "MySQLDS made query: %s", query.CHAR_PTR());

				if (  mysql_query(mysql, query.CHAR_PTR()) == 0 )
					{
						MYSQL_RES * res =  mysql_store_result(mysql);

						local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );

						 mysql_free_result(res);
					}
					err = CheckError(token, mysql);
				}
			}
			break;
		case datasourcePrepareSQL:
			if ( param != NULL && param->data != NULL )
			{
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				lasso_log(LOG_LEVEL_SQL, "MySQLDS preparing user query: %s", param->data);

				MYSQL_STMT * statPtr = mysql_stmt_init(mysql);
				if (!statPtr)
				{
					lasso_setResultMessage(token, "Could not allocate MYSQL_STMT.");
					err = osErrOutOfMemory;
				}
				else
				{
					int res = mysql_stmt_prepare(statPtr, param->data, param->dataSize);
					if (res != 0)
					{
                        lasso_setResultMessage(token, mysql_stmt_error(statPtr));
						err = (osError)res;
					}
					else
					{
						lasso_setDSPreparedPtr(token, statPtr);
					}
				}
			}
			break;
		case datasourceUnprepareSQL:
			{
				void * statPtr = NULL;
				lasso_getDSPreparedPtr(token, &statPtr);
				if (statPtr)
					mysql_stmt_close((MYSQL_STMT*)statPtr);
			}
			break;
		case datasourceExecSQL:
			if ( param != NULL && (param->data == NULL || param->data[0] == '\0') ) // executing a prepared statement. probably
			{
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				MYSQL_STMT * statPtr = NULL;
				lasso_getDSPreparedPtr(token, (void**)&statPtr);
				if (statPtr)
				{
					unsigned long paramCount = mysql_stmt_param_count(statPtr);
					MYSQL_BIND * binds = NULL;
					ValStore * vals = NULL;
					if (paramCount)
					{
						int count = 0;
						lasso_getInputColumnCount(token, &count);
						if (paramCount != (unsigned long)count)
						{
							lasso_setResultMessage(token, "The number of placeholders did not match the number of input values.");
                            return osErrInvalidParameter;
						}

						binds = new MYSQL_BIND[paramCount];
						vals = new ValStore[paramCount];

						memset(binds, 0, sizeof(MYSQL_BIND)*paramCount);
						memset(vals, 0, sizeof(ValStore)*paramCount);

						for (int x = 0; err == osErrNoErr && x < count; ++x)
						{
							auto_lasso_value_t hardlyused;
							INITVAL(&hardlyused);
							lasso_type_t type = NULL;
							if (lasso_getInputColumn3(token, x, &hardlyused, &type) == osErrNoErr && type != NULL)
							{
								INITVAL(&hardlyused);
								if (lasso_typeIsA(token, type, lpTypeDecimal) == osErrNoErr)
								{
									lasso_typeGetDecimal(token, type, &vals[x].u.d);
									vals[x].len = binds[x].buffer_length = sizeof(double);
									binds[x].buffer_type = MYSQL_TYPE_DECIMAL;
									binds[x].buffer = &vals[x].u.d;
									binds[x].length = &vals[x].len;
								}
								else if (lasso_typeIsA(token, type, lpTypeInteger) == osErrNoErr)
								{
									lasso_typeGetInteger(token, type, &vals[x].u.i);
									vals[x].len = binds[x].buffer_length = sizeof(osInt64);
									binds[x].buffer_type = MYSQL_TYPE_LONGLONG;
									binds[x].buffer = &vals[x].u.i;
									binds[x].length = &vals[x].len;
								}
								else if (lasso_typeIsA2W(token, type, kLPBytesTypeName) == osErrNoErr)
								{
									lasso_typeGetString(token, type, &hardlyused);
									vals[x].len = binds[x].buffer_length = hardlyused.nameSize;
									binds[x].buffer_type = MYSQL_TYPE_LONG_BLOB;
									binds[x].buffer = (char*)hardlyused.name;
									binds[x].length = &vals[x].len;
								}
								else if (lasso_typeGetName(token,type, &hardlyused) == osErrNoErr && strcasecmp(hardlyused.name, "null") == 0)
								{
									binds[x].buffer_type = MYSQL_TYPE_NULL;
								}
								else
								{
									lasso_typeGetString(token, type, &hardlyused);
									vals[x].len = binds[x].buffer_length = hardlyused.nameSize;
									binds[x].buffer_type = MYSQL_TYPE_VAR_STRING;
									binds[x].buffer = (char*)hardlyused.name;
									binds[x].length = &vals[x].len;
								}
							}
						}
						mysql_stmt_bind_param(statPtr, binds);
					}
					int res = mysql_stmt_execute(statPtr);
					if (res != 0)
					{
						lasso_setResultMessage(token, mysql_stmt_error(statPtr));
						err = (osError)res;
					}
					else
					{
						MYSQL_RES * resPtr = mysql_stmt_result_metadata(statPtr);
						lasso_setRowID( token, (unsigned long)  mysql_stmt_insert_id(statPtr) );

						char rowcount[16] = "";
						sprintf( rowcount, "%lu", (unsigned long) mysql_stmt_affected_rows(statPtr) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );

						if ( resPtr != NULL )
						{
							local_addColumnInfo(token, resPtr, mysql, "", "");

							int num =  mysql_stmt_field_count(statPtr);

							MYSQL_BIND * rowBinds = new MYSQL_BIND[num];
							char ** row = new char*[num];
							unsigned long * lengths = new unsigned long[num];
							my_bool * isNulls = new my_bool[num];

							memset(rowBinds, 0, sizeof(MYSQL_BIND)*num);
							memset(row, 0, sizeof(char*)*num);
							memset(lengths, 0, sizeof(unsigned int*)*num);
							memset(isNulls, 0, sizeof(my_bool)*num);

							// set up the binds so that they get the data lengths
							// but don't plug in a buffer
							for (int x = 0; x < num; ++x)
							{
								rowBinds[x].buffer_type = MYSQL_TYPE_VAR_STRING;
								rowBinds[x].length = &lengths[x];
								rowBinds[x].is_null = &isNulls[x];
							}

							mysql_stmt_bind_result(statPtr, rowBinds);

							res = mysql_stmt_store_result(statPtr);
							int found = (int) mysql_stmt_num_rows(statPtr);
							int skip, max;

							lasso_getSkipRows( token, &skip );
							lasso_getMaxRows( token, &max );
							lasso_setNumRowsFound( token, found );

							if ( skip > 0 )
								mysql_stmt_data_seek( statPtr, skip );

							while(mysql_stmt_fetch(statPtr) == 0 && max--)
							{
								for (int x = 0; x < num; ++x)
								{
									MYSQL_BIND bind = rowBinds[x];
									if (isNulls[x])
									{
										row[x] = NULL;
									}
									else if (lengths[x] == 0)
									{
										row[x] = new char[1];
										*row[x] = '\0';
									}
									else
									{
										row[x] = new char[lengths[x]+1];
									}
									bind.buffer = row[x];
									bind.buffer_length = lengths[x];
                                    mysql_stmt_fetch_column(statPtr, &bind, x, 0);
									bind.buffer = NULL;
								}
								lasso_addResultRow( token, (const char**)row, lengths, num );
								for (int x = 0; x < num; ++x)
								{
									delete [] row[x];
									row[x] = NULL;
									lengths[x] = 0;
									isNulls[x] = (my_bool)0;
								}
							}

							delete [] isNulls;
							delete [] lengths;
							delete [] row;
							delete [] rowBinds;

							mysql_free_result(resPtr);
							mysql_stmt_free_result(statPtr);
						}
					}
					mysql_stmt_reset(statPtr);
					if (binds)
						delete [] binds;
					if (vals)
						delete [] vals;
				}
				else
				{
					lasso_setResultMessage(token, "Could not find prepared statement.");
					err = osErrInvalidParameter;
				}
			}
			else if ( param != NULL && param->data != NULL )
			{
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				if (statOnly)
					break;
				lasso_log(LOG_LEVEL_SQL, "MySQLDS user query: %s", param->data);
				if (mysql_get_server_version(mysql) < MYSQL_VERSION_41)
				{
			// **** MySQL 4.0 compatability
					int max = param->dataSize, start=-1, finish=-1;
					int count = 0;
					while( err == osErrNoErr && lasso_sqlFindStat(param->data, &start, &finish, max) )
					{
						MYSQL_RES * res = NULL;
						if (count > 0)
							lasso_addResultSet(token);
						if ( mysql_real_query( mysql, param->data + start, finish-start ) == 0)
						{	// this code and the bottom code should be unified
							res =  mysql_store_result(mysql);
							lasso_setRowID( token, (unsigned long)  mysql_insert_id( mysql ) );

							char rowcount[16] = "";
							sprintf( rowcount, "%lu", (unsigned long) mysql_affected_rows(mysql) );
							lasso_setVariable( token, kRowCountVar, rowcount, 0 );

							if ( res != NULL )
							{
								local_addColumnInfo(token, res, mysql, "", "");

								int found = (int) mysql_num_rows(res);
								int num =  mysql_num_fields(res);
								int skip, max;
								MYSQL_ROW row;

								lasso_getSkipRows( token, &skip );
								lasso_getMaxRows( token, &max );
								lasso_setNumRowsFound( token, found );

								if ( skip > 0 )
									 mysql_data_seek( res, skip );

								while( (row =  mysql_fetch_row(res)) != NULL && max--)
								{
									unsigned long * lengths =  mysql_fetch_lengths(res);
									lasso_addResultRow( token, (const char**)row, lengths, num );
								}
								mysql_free_result(res);
							}
							++count;
						}
						else
							err =  (osError)mysql_errno(mysql);
					}
			// ****
				}
				else if ( mysql_real_query(mysql, param->data, param->dataSize) == 0 )
				{
					MYSQL_RES * res = NULL;
					while(true)
					{
						res =  mysql_store_result(mysql);
						lasso_setRowID( token, (unsigned long)  mysql_insert_id( mysql ) );

						char rowcount[16] = "";
						sprintf( rowcount, "%lu", (unsigned long) mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );

						if ( res != NULL )
						{
							local_addColumnInfo(token, res, mysql, "", "");

							int found = (int) mysql_num_rows(res);
							int num =  mysql_num_fields(res);
							int skip, max;
							MYSQL_ROW row;

							lasso_getSkipRows( token, &skip );
							lasso_getMaxRows( token, &max );
							lasso_setNumRowsFound( token, found );

							if ( skip > 0 )
								 mysql_data_seek( res, skip );

							while( (row =  mysql_fetch_row(res)) != NULL && max--)
							{
								unsigned long * lengths =  mysql_fetch_lengths(res);
								lasso_addResultRow( token, (const char**)row, lengths, num );
							}
							mysql_free_result(res);
						}
						if (mysql_next_result(mysql) == 0)
						{
							lasso_addResultSet(token);
						}
						else
							break;
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		default:
			break;

	}
	return err;
}

struct mysql_string_ptr_pair_t
{
	mysql_string_ptr_pair_t( const STRING_TYPE & str, const std::auto_ptr<STRING_TYPE> & ptr)
	{
		first.CSET(str);
		second = const_cast<std::auto_ptr<STRING_TYPE>&>(ptr);
	}
	mysql_string_ptr_pair_t( const mysql_string_ptr_pair_t & pr )
	{
		first.CSET(pr.first);
		second = const_cast<mysql_string_ptr_pair_t&>(pr).second;
	}
	mysql_string_ptr_pair_t(){}

	STRING_TYPE first;
	std::auto_ptr<STRING_TYPE> second;
};
typedef std::list<mysql_string_ptr_pair_t> mysqlconsolidated_input_t;

inline void AddOrCat( const char * name, const char * value, int valueLen, mysqlconsolidated_input_t * lst )
{
	for ( mysqlconsolidated_input_t::iterator it = lst->begin(), end = lst->end(); it != end; ++it )
	{
		if ( (*it).first.CEQUALS(name) ) // it's in there
		{
			if ( value != NULL )
			{
				if ( (*it).second.get() == NULL )
					(*it).second.reset(new STRING_TYPE);
				else
					(*it).second->APND(",");
				(*it).second->APND(value, valueLen);
			}
			return;
		}
	}
	// add it anew
	std::auto_ptr<STRING_TYPE> tmp(NULL);
	lst->push_back(mysql_string_ptr_pair_t(STRING_TYPE(name), tmp));
	if ( value != NULL )
		lst->back().second.reset(new STRING_TYPE(value, valueLen));
}

inline void
mysqlConsolidateInput( lasso_request_t token, mysqlconsolidated_input_t * lst )
{
	int num,i;

	lasso_getInputColumnCount( token, &num );
	for ( i = 0; i < num; ++i )
	{
		auto_lasso_value_t val;
		if ( lasso_getInputColumn( token, i, &val ) == osErrNoErr && val.name && val.name[0] != '-' )
		{
			if (val.name && val.name[0])
				AddOrCat( val.name, val.data, val.dataSize, lst);
		}
	}
}

void local_addInsertCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & query )
{
	int num, i;
	mysqlconsolidated_input_t lst;
	mysqlConsolidateInput(token, &lst);

	num = lst.size();
	if ( num > 0 )
	{
		query.APND(" (");
		i = 0;
		{for ( mysqlconsolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			AddColumnName(token, mysql, (*it).first.CHAR_PTR(), query);
			if ( i + 1 < num )
				query.APND(",");
			++i;
		}}
		query.APND(") VALUES (");
		i = 0;
		{for ( mysqlconsolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			const char * str = (*it).second.get() ? (*it).second->CHAR_PTR() : NULL;
			int len = (*it).second.get() ? (*it).second->CLENGTH() : 0;
			local_addSearchValue( token, mysql, query, str, len, lpTypeNull, false);

			if ( i + 1 < num )
				query.APND(",");
			++i;
		}}
		query.APND(")");
	}
}

void local_addUpdateCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to )
{
	mysqlconsolidated_input_t lst;
	bool addComma = false;

	mysqlConsolidateInput( token, &lst );

	if ( lst.size() > 0 )
	{
		to.APND(" SET ");
		for ( mysqlconsolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			if ( addComma )
				to.APND(",");
			else
				addComma = true;

			AddColumnName(token, mysql, (*it).first.CHAR_PTR(), to);
			to.APND(" = ");
			const char * str = (*it).second.get() ? (*it).second->CHAR_PTR() : NULL;
			int len = (*it).second.get() ? (*it).second->CLENGTH() : 0;
			local_addSearchValue( token, mysql, to, str, len, lpTypeNull, false );
		}
	}
}
/*
bool local_addPrimaryKeySearch( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to )
{
	auto_lasso_value_t key;

	if ( lasso_getPrimaryKeyColumn( token, &key ) == osErrNoErr )
	{
		if ( key.name != NULL && strlen(key.name) > 0 )
		{
			to.APND(" WHERE ");
			AddColumnName(token, mysql, key.name, to);
			to.APND(" = ");
			if ( key.data != NULL && strlen(key.data) > 0 )
			{
				local_addSearchValue( token, mysql, to, key.data, key.dataSize, lpTypeNull );
				return true;
			}
		}
	}
	return false;
}
*/
/*
+-------+--------------------------------+------+-----+---------+-------+
| Field | Type                           | Null | Key | Default | Extra |
+-------+--------------------------------+------+-----+---------+-------+
| Title | enum('Mr.','Mrs.','Ms.','Dr.') | YES  | MUL | NULL    |       |
+-------+--------------------------------+------+-----+---------+-------+
*/
void grabValueList( MYSQL * mysql, const char * col, const char * db, const char * table, char *** vals, int * count )
{
	STRING_TYPE query;
	query.APND("SHOW COLUMNS FROM `").APND(table).APND("` FROM `").APND(db).APND("` LIKE '").APND(col).APND("'");

	int result =  mysql_query(mysql, query.CHAR_PTR());
	if ( result == 0 )
	{
		MYSQL_RES * res =  mysql_store_result(mysql);
		if ( res )
		{
			int num =  mysql_num_fields(res);
			MYSQL_ROW row =  mysql_fetch_row(res);
			if ( row != NULL && num >= 2 )
			{
				const char * def = row ? row[1] : NULL;

				if ( def && (strncmp("enum", def, 4) == 0 || strncmp("set", def, 3) == 0) )
				{
					if ( def[0] == 'e' )
						def += 5;
					else
						def += 4;

					*count = 1;
					for ( int tmp = 0; def[tmp] != '\0'; ++tmp )
					{
						if ( def[tmp] == ',' )
							++*count;
					}

					*vals = new char*[*count];

					for ( int x = 0; x < *count; ++x )
					{
						if ( *def == '\'' )
							++def;
						STRING_TYPE str;
						while( *def != ',' && *def != '\'' )
						{
							str.APND(def, 0, 1);
							++def;
						}
						if ( *def == '\'' )
							def += 2;
						else
							++def;

						(*vals)[x] = new char[str.CLENGTH()+1];
						strcpy((*vals)[x], str.CHAR_PTR());
					}
				}
			}
			 mysql_free_result(res);
		}
	}
}

void local_addColumnInfo( lasso_request_t token, MYSQL_RES * res, MYSQL * mysql, const char * db, const char * table )
{
	int numFields =  mysql_num_fields(res), i;
	MYSQL_FIELD * field =  mysql_fetch_fields(res);

	for( i = 0; i < numFields; ++i )
	{
		LP_TypeDesc type;
		switch( field[i].type )
		{
			case FIELD_TYPE_TINY:
			case FIELD_TYPE_SHORT:
			case FIELD_TYPE_LONG:
			case FIELD_TYPE_INT24:
			case FIELD_TYPE_LONGLONG:
				type = lpTypeInteger;
				break;
			case FIELD_TYPE_DECIMAL:
			case FIELD_TYPE_FLOAT:
			case FIELD_TYPE_DOUBLE:
				type = lpTypeDecimal;
				break;
			case FIELD_TYPE_TIMESTAMP:
			case FIELD_TYPE_DATE:
			case FIELD_TYPE_TIME:
			case FIELD_TYPE_DATETIME:
			case FIELD_TYPE_YEAR:
				type = lpTypeDateTime; // ???
				break;
			case FIELD_TYPE_TINY_BLOB:
			case FIELD_TYPE_MEDIUM_BLOB:
			case FIELD_TYPE_LONG_BLOB:
			case FIELD_TYPE_BLOB:
				if ( (field[i].flags & BINARY_FLAG) != 0)
				{
					type = lpTypeBytes;
					break;
				}
			default:
				type = lpTypeString;
				break;
		}
		if ( (db != NULL && table != NULL && db[0] != '\0' && table[0] != '\0') && field[i].name && ((field[i].flags & ENUM_FLAG) != 0 || (field[i].flags & SET_FLAG) != 0) )
		{
			char ** vals = NULL;
			int count = 0;

			grabValueList(mysql, field[i].name, db, table, &vals, &count);
			lasso_addColumnInfo2( token, convertFromTableEncToISO(token, mysql, field[i].name), IS_NOT_NULL(field[i].flags) ? 0 : 1, type, kProtectionNone, (const char**)vals, count );

			for ( int o = 0; o < count; ++o )
				delete [] vals[o];
			delete [] vals;
		}
		else
			lasso_addColumnInfo( token, convertFromTableEncToISO(token, mysql, field[i].name), IS_NOT_NULL(field[i].flags) ? 0 : 1, type, kProtectionNone );
	}
}

void local_addReturnFields( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to )
{
	int num, i;
	lasso_getReturnColumnCount( token, &num );

	if ( num == 0 )
	{
		to.APND("*");
	}
	else
	{
		for ( i = 0; i < num; ++i )
		{
			auto_lasso_value_t val;
			if ( lasso_getReturnColumn( token, i, &val ) == osErrNoErr )
			{
				if ( i > 0 )
					to.APND(",");
				AddTickedName(token, mysql, val.data, to);
			}
			else
				break;
		}
	}
}

#define ISOPBEGIN(X) (strcasecmp(X, kLPOpBegin) == 0 || strcasecmp(X, kLPOperatorBegin) == 0)
#define ISOPEND(X) (strcasecmp(X, kLPOpEnd) == 0 || strcasecmp(X, kLPOperatorEnd) == 0)
#define ISGROUPBY(X) (strcasecmp(X, "-groupby") == 0)


// gets the key values from the -key/-keyField or from the input fields, if provided
void getKeyValues( lasso_request_t token, std::vector<std::pair<auto_lasso_value_t, LP_TypeDesc> > & validValues,
											std::vector<auto_lasso_value_t> & groupByValues, bool keyOnly )
{
	int keyNum = 0, inNum = 0, i = 0;
	LP_TypeDesc op;
	lasso_getLogicalOp( token, &op );
	auto_lasso_value_t tmpVal;
	tmpVal.name = "opbegin";

	if ( op == kLassoOR )
	{
		tmpVal.data = "or";
	}
	else
	{
		tmpVal.data = "and";
	}
	validValues.push_back(std::make_pair(tmpVal, lpTypeString));

	lasso_getPrimaryKeyColumnCount( token, &keyNum );
	lasso_getInputColumnCount( token, &inNum );

	// if there are keys provided, use the keys only
	if (!keyOnly)
		keyOnly = keyNum != 0;
	if (!keyOnly)
		keyNum = inNum;

	for ( i = 0; i < keyNum; ++i )
	{
		LP_TypeDesc type;
		auto_lasso_value_t val;
		INITVAL(&val);
		if (keyOnly)
		{
			if ( lasso_getPrimaryKeyColumn2(token, i, &val, &type) != osErrNoErr )
				return;
		}
		else
		{
			if ( lasso_getInputColumn2( token, i, &val, &type ) != osErrNoErr )
				return;
		}
		if (val.name == NULL)
			continue;
		if ( ISOPEND(val.name) )
			val.data = "something";
		else if (val.name[0] == '-' && ISGROUPBY(val.name) && val.data && *val.data)
		{
			groupByValues.push_back(val);
			continue;
		}
		else if ( val.name[0] == '-' && !ISOPBEGIN(val.name) )
		{
			continue;
		}
		// allow blank values
		if ( (val.data != NULL && strlen(val.data) > 0) || val.type == kLassoEquals || val.type == kLassoNOT )
		{
			if ( i != 0 && ISOPEND(val.name) && ISOPBEGIN(validValues.back().first.name) )
			{
				validValues.pop_back();
			}
			else
				validValues.push_back(std::make_pair(val, type));
		}
	}
}

bool local_addSearchCriteria( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to, LP_TypeDesc defaultOp, datasource_action_t action, bool useLimit )
{
	bool hadValues = false;
	if (action != datasourceFindAll)
	{
		std::vector<std::pair<auto_lasso_value_t, LP_TypeDesc> > validValues;
		std::vector<auto_lasso_value_t> groupByValues;
		getKeyValues(token, validValues, groupByValues, action != datasourceSearch);
		LP_TypeDesc op;
		lasso_getLogicalOp( token, &op );
		int i = 0, num = (int)validValues.size();
		if ( num > 1 )
		{
			hadValues = true;
			to.APND(" WHERE ");

			std::vector<LP_TypeDesc> opStack;

			for ( i = 0; i < num; ++i )
			{
				auto_lasso_value_t curr = validValues[i].first;
				LP_TypeDesc type = validValues[i].second;
				if ( ISOPBEGIN(curr.name) )
				{
					if ( strcasecmp(curr.data, "not") == 0 )
					{
						to.APND("NOT (");
						opStack.push_back(kLassoNOT);
					}
					else
					{
						if ( strcasecmp(curr.data, "and") == 0 )
							opStack.push_back(kLassoAND);
						else if ( strcasecmp(curr.data, "or") == 0 )
							opStack.push_back(kLassoOR);
						else
							opStack.push_back(op);

						to.APND("(");
					}
					continue;
				}
				else if ( ISOPEND(curr.name) )
				{
					to.APND(")");
					opStack.pop_back();
				}
				else
				{
					if ( curr.type == kLassoInFullText )
					{
						to.APND("MATCH(");
						AddColumnName(token, mysql, curr.name, to);
						to.APND(")");
					}
					else
					{
						AddColumnName(token, mysql, curr.name, to);
					}

					if ( defaultOp != lpTypeNull ) // no default operator
					{
						curr.type = defaultOp;
					}

					local_addOperator( token, mysql, to, curr.type, curr.data, curr.dataSize );
					local_addSearchValue( token, mysql, to, curr.data, curr.dataSize, curr.type, true );
				}
				if ( (i + 1) < num && (!ISOPEND(validValues[i+1].first.name)) )
				{
					int num = opStack.size()-1;
					LP_TypeDesc lastOp = opStack[num--];

					while( num >= 0 && lastOp == kLassoNOT )
					{
						lastOp = opStack[num--];
					}

					if ( lastOp != kLassoAND )
						to.APND(" OR ");
					else
						to.APND(" AND ");
				}
			}
			while ( opStack.size() > 0 )
			{
				to.APND(")");
				opStack.pop_back();
			}
		}
		if (action == datasourceSearch)
		{
			num = (int)groupByValues.size();
			if (num > 0)
			{
				to.APND(" GROUP BY ");
				for (int i = 0; i < num; ++i)
				{
					if (i > 0) // not the first
						to.APND(",");
					to.APND(groupByValues[i].data);
				}
			}
		}
	}
	if (action == datasourceSearch || action == datasourceFindAll)
	{
		local_addSortOrder( token, mysql, to );
		if ( useLimit )
		{
			int max, skip;
			lasso_getSkipRows( token, &skip );
			lasso_getMaxRows( token, &max );

			if ( max == -1 )
				max = 2147483647;
			char tmp[256];
			sprintf(tmp, " LIMIT %d,%d", skip, max);
			to.APND(tmp);
		}
	}
	return hadValues;
}

void local_addSortOrder( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to )
{
	int num, i;
	std::vector<auto_lasso_value_t> sorts;
	auto_lasso_value_t val;

	lasso_getSortColumnCount( token, &num );
	for ( i = 0; i < num; ++i )
	{
		if ( lasso_getSortColumn( token, i, &val ) == osErrNoErr )
		{
			if ( val.data != NULL && strlen(val.data) > 0 )
				sorts.push_back(val);
		}
		else
			break;
	}
	num = sorts.size();
	if ( num > 0 )
	{
		to.APND(" ORDER BY ");
		for ( i = 0; i < num; ++i )
		{
			AddColumnName(token, mysql, sorts[i].data, to);
			if ( sorts[i].type == orderDescending )
				to.APND(" DESC ");
			if ( i + 1 < num )
				to.APND(",");
		}
	}
	auto_lasso_value_t test;
	if ( lasso_findInputColumn(token, "-sortrandom", &test) == osErrNoErr )
	{
		to.APND(" ORDER BY RAND()");
	}
}

void local_addOperator( lasso_request_t, MYSQL*, STRING_TYPE & to, LP_TypeDesc op, const char * value, unsigned int size )
{
	switch( op )
	{
	case kLassoEquals:
		if ( value == NULL )
			to.APND(" IS ");
		else
			to.APND(" = ");
		break;
	case kLassoGreaterThan:
		to.APND(" > ");
		break;
	case kLassoGreaterThanEquals:
		to.APND(" >= ");
		break;
	case kLassoLessThan:
		to.APND(" < ");
		break;
	case kLassoLessThanEquals:
		to.APND(" <= ");
		break;
	case kLassoNOT:
		if ( value == NULL )
			to.APND(" IS NOT ");
		else
			to.APND(" <> ");
		break;
	case kLassoContains:
	case kLassoBeginsWith:
	case kLassoEndsWith:
		to.APND(" LIKE ");
		break;
	case kLassoNotContains:
	case kLassoNotBeginsWith:
	case kLassoNotEndsWith:
		to.APND(" NOT LIKE ");
		break;
	case kLassoInFullText:
		to.APND(" AGAINST ");
		break;
	case kLassoInRegExp:
		to.APND(" REGEXP ");
		break;
	case kLassoNotInRegExp:
		to.APND(" NOT REGEXP ");
		break;
	case kLassoInList:
		to.APND(" IN ");
		break;
	case kLassoNotInList:
		to.APND(" NOT IN ");
		break;
	default:
		to.APND(" = ");
		break;
	}
}

void local_addSearchValue( lasso_request_t token, MYSQL * mysql, STRING_TYPE & to, const char * value, unsigned int size, LP_TypeDesc op, bool doWildcards )
{
	if ( value == NULL )
	{
		to.APND("NULL");
	}
	else if ( size == 0 )//value[0] == '\0' )
	{
		to.APND("''");
	}
	else
	{
		bool isLIKE = op == kLassoEndsWith || op == kLassoContains || op == kLassoNotEndsWith || op == kLassoNotContains ||
				  op == kLassoBeginsWith || op == kLassoNotBeginsWith;

		// values need to be encoded
		char * encoded = new char[size*2+1];
		// mysql_real_escape_string didn't seem to be exported in the mysqlk client I am using
		mysql_escape_string(encoded, value, size);
		if (!isLIKE)
			doWildcards = false;
		if (doWildcards)
		{
			char * enc2 = new char[size*2+1];

			char * pos = encoded, *pos2 = enc2;
			for ( ; *pos; ++pos, ++pos2)
			{
				if (*pos == '%' || *pos == '_')
				{
					*pos2 = '\\';
					++pos2;
				}
				*pos2 = *pos;
			}
			*pos2 = '\0';
			strcpy(encoded, enc2);
			delete [] enc2;
		}
		if ( isLIKE )
		{
			to.APND("'");
			if (op == kLassoEndsWith || op == kLassoContains || op == kLassoNotEndsWith || op == kLassoNotContains)
			{
				to.APND("%");
			}
			to.APND(encoded);
			if (op == kLassoBeginsWith || op == kLassoContains || op == kLassoNotBeginsWith || op == kLassoNotContains)
			{
				to.APND("%");
			}
			to.APND("'");
		}
		else if (op == kLassoInList || op == kLassoNotInList)
		{
		  to.APND("(").APND(encoded).APND(")");
		}
		else
		{
			if ( op == kLassoInFullText )
				to.APND("(");
			to.APND("'").APND(encoded).APND("'");
			if ( op == kLassoInFullText )
				to.APND(")");
		}

		delete [] encoded;
	}
}
