#ifndef _MYSQLDS_H_
#define _MYSQLDS_H_

#if MACHINE_MACOSX
#include <Lasso/LassoCAPI.h>
#else
#include "LassoCAPI.h"
#endif
#ifdef WIN32
#include <winsock.h>
#endif
#include "mysql.h"
#include <string.h>

#if 0
#include <string>
#define STRING_TYPE std::string
#define APND append
#define CHAR_PTR c_str
#define CLENGTH size
#define CSET operator=
#define CEQUALS operator==
#endif

#define STRING_TYPE CustomBuffer
#define APND append
#define CHAR_PTR c_str
#define CLENGTH size
#define CSET operator=
#define CEQUALS equalsNoCase


#define gPadSize 200
class CustomBuffer
{
public:
	inline bool operator==(const char * p) const
	{
		return equalsNoCase(p);
	}
	inline bool equalsNoCase( const char * p ) const
	{
		if ( (p == NULL || *p == 0) && (fBuff == NULL || *fBuff == 0) )
			return true;
		if (p == NULL || fBuff == NULL)
			return false;
        return strcasecmp(fBuff, p) == 0;
	}
	inline const char * c_str() const { return fBuff ? fBuff : ""; }
	inline int size() const { return fCLen; }
	inline CustomBuffer & append(const CustomBuffer & rhs)
	{
		return append(rhs.fBuff, rhs.fCLen);
	}
	inline CustomBuffer & append(const char * p, int offset, int num)
	{
		return append(p+offset, num);
	}
	inline CustomBuffer & append(const char * p, int len = -1)
	{
		if (p)
		{
			if (len == -1)
				len = strlen(p);
            if (fCLen + len + 1 > fBuffLen)
			{
				char * oldB = fBuff;
				fBuffLen = fCLen + len + gPadSize;
				fBuff = new char[fBuffLen];
				if (oldB)
					memcpy(fBuff, oldB, fCLen);
				memcpy(fBuff + fCLen, p, len);
				if (oldB)
					delete [] oldB;
				fCLen += len;
				fBuff[fCLen] = '\0';
			}
			else
			{
				memcpy(fBuff + fCLen, p, len);
				fCLen += len;
				fBuff[fCLen] = '\0';
			}
		}
		return *this;
	}

	CustomBuffer() :fBuff(NULL), fCLen(0), fBuffLen(0) {}
	CustomBuffer( const CustomBuffer & rhs) :fBuff(NULL), fCLen(0), fBuffLen(0)
	{
		this->operator =(rhs);
	}
	CustomBuffer(const char * b, int len = -1)
	{
		if (b)
		{
			if (len == -1)
				len = strlen(b);
			fBuff = new char[len+gPadSize];
			memcpy(fBuff, b, len);
			fBuff[len] = '\0';
			fCLen = len;
			fBuffLen = len+gPadSize;
		}
	}
	~CustomBuffer()
	{
		delete [] fBuff;
	}
	CustomBuffer & operator=(const char * p)
	{
		fCLen = 0;
		return this->append(p);
	}
	CustomBuffer & operator=(const CustomBuffer & rhs)
	{
		if (rhs.fCLen > fBuffLen)
		{
			delete [] fBuff;
			fBuff = new char[rhs.fBuffLen];
			memcpy(fBuff, rhs.fBuff, rhs.fCLen);
			fBuff[rhs.fCLen] = '\0';
			fCLen = rhs.fCLen;
			fBuffLen = rhs.fBuffLen;
		}
		else if (rhs.fCLen)
		{
			memcpy(fBuff, rhs.fBuff, rhs.fCLen);
			fBuff[rhs.fCLen] = '\0';
			fCLen = rhs.fCLen;
		}
		else
		{
			fCLen = 0;
		}
		return *this;
	}
protected:
	char * fBuff;
	unsigned int fCLen;
	unsigned int fBuffLen;

};

#ifdef WIN32
#define MYSQLDS_EXPORT __declspec(dllexport)
#define MYSQLDS_IMPORT __declspec(dllimport)
#else
#define MYSQLDS_EXPORT
#define MYSQLDS_IMPORT
#endif

#ifdef __cplusplus
extern "C"
{
#endif

MYSQLDS_EXPORT void registerLassoModule(void);

#ifdef __cplusplus
}
#endif
static osError local_mysqlds_func(lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param);

#endif
