#include "IndexService.h"
#include <stdio.h>
#include <sys/stat.h>
#if MACHINE_LINUX
#include <errno.h>
#endif
#ifdef WIN32
#include <io.h>
#endif
#define _POSIX_
#include <fcntl.h>
#include <string>

void registerLassoModule()
{
	lasso_registerTagModule("indexservice", "searchfor", indexservice_search, REG_FLAGS_TAG_DEFAULT, "Performs a search using the index service.");
}

osError indexservice_search(lasso_request_t token, tag_action_t)
{
	osError err = osErrInvalidParameter;

	lasso_type_t in = NULL;
	auto_lasso_value_t term;
	FSSpec * specs = NULL;
	UInt32 numSpecs = 0;
	INITVAL(&term);
	
	err = lasso_getTagParam(token, 0, &term);
	if (err != osErrNoErr)
		return err;
	err = lasso_getTagParam2(token, 1, &in);
	if (err != osErrNoErr)
		return err;
		
	if (lasso_typeIsA(token, in, lpTypePair) == osErrNoErr)
		lasso_pairGetSecond(token, in, &in);
	if (lasso_typeIsA(token, in, lpTypeArray) == osErrNoErr)
	{
		int count = 0;
		lasso_arrayGetSize(token, in, &count);
		numSpecs = count;
		specs = new FSSpec[count];
		for (int x = 0; x < count; ++x)
		{
			lasso_type_t t = NULL;
			lasso_arrayGetElement(token, in, x, &t);
			auto_lasso_value_t path;
			lasso_typeGetString(token, t, &path);
			osPathname xform;
			lasso_internalToFullPlatformPath(token, path.name, xform);
			if (xform[strlen(xform)-1] == '/')
				xform[strlen(xform)-1] = '\0';
			FSRef theRef;
			::FSPathMakeRef((const UInt8*)xform, &theRef, NULL);
			::FSGetCatalogInfo(&theRef, kFSCatInfoNone, NULL, NULL, &specs[x], NULL);			
		}
	}
	else
	{
		auto_lasso_value_t path;
		lasso_typeGetString(token, in, &path);
		osPathname xform;
		lasso_internalToFullPlatformPath(token, path.name, xform);
		if (xform[strlen(xform)-1] == '/')
			xform[strlen(xform)-1] = '\0';
		FSRef theRef;
		::FSPathMakeRef((const UInt8*)xform, &theRef, NULL);
		
		specs = new FSSpec[1];
		numSpecs = 1;
		FSGetCatalogInfo(&theRef, kFSCatInfoNone, NULL, NULL, &specs[0], NULL);
	}
	
	SERVICEPTR ptr = NULL;
	OSErr macerr = ::FBCCreateSearchSession(&ptr);
	CFStringRef stringRef = CFStringCreateWithCString(kCFAllocatorDefault, term.name, kCFStringEncodingUTF8);
	macerr = ::FBCDoCFStringSearch(ptr, stringRef, specs, numSpecs, 500, 0);
	if (macerr == 0)
	{
		UInt32 count = 0;
		macerr = ::FBCGetHitCount(ptr, &count);
		if (macerr == 0)
		{
			lasso_type_t ary = NULL;
			lasso_typeAllocArray(token, &ary, 0, NULL);
			for (UInt32 x = 0; x < count; ++x)
			{
				FSRef ref;
				if (::FBCGetHitDocumentRef(ptr, x, &ref) == 0)
				{
					float score = 0;
					::FBCGetHitScore(ptr, x, &score);
					UInt8 path[2048];
					FSRefMakePath(&ref, path, 2048);
					osPathname internal;
					lasso_getInternalPath((const char*)path, internal);
					lasso_type_t str = NULL, scr = NULL, pair = NULL;
					lasso_typeAllocString(token, &str, (const char*)internal, strlen((const char*)internal));
					lasso_typeAllocDecimal(token, &scr, (double)score);
					lasso_typeAllocPair(token, &pair, str, scr);
					lasso_arraySetElement(token, ary, x, pair);
				}
			}
			lasso_returnTagValue(token, ary);
		}
		::FBCReleaseSessionHits(ptr);
	}
	CFRelease(stringRef);	
	delete [] specs;	
	::FBCDestroySearchSession(ptr);
	return (osError)macerr;
}
