
#include "main.h"

// this is the namespace we will place our tag in
static const char * kAudioNamespaceName = "audio";
// this is the name of the tag we are registering
static const char * kPlaySound = "playsound";
// here is the description of the tag we are registering
static const char * kDescription = "audio_playsound(<path to sound file>) - Plays the specified sound file. \
							Supports AIFF, WAV, and NeXT \".snd\" files. Allows 16-bit, mono and stereo, 44.1KHz and 22.05KHz data.";

// this is the standard entry point which Lasso looks for in every LCAPI module.
void registerLassoModule()
{
	lasso_registerTagModule(kAudioNamespaceName, kPlaySound, audio_playsound, REG_FLAGS_TAG_DEFAULT, kDescription);
}

// here is the prototype for the tag we will implement
osError audio_playsound(lasso_request_t token, tag_action_t)
{
	// this will hold the provided path name
	auto_lasso_value_t thePath;
	INITVAL(&thePath);
	
	// if a parameter was not provided...
	osError err = lasso_getTagParam(token, 0, &thePath);
	if (err != osErrNoErr)
		return err; // bail
		
	// make sure that this file path is permitted for read access
	err = lasso_operationAllowed(token, kReadFiles, (void*)thePath.name);
	if (err != osErrNoErr)
		return err; // bail
		
	// now convert the path to a full platform specific path
	osPathname dst = "";
	lasso_getPlatformSpecificPath(thePath.name, dst);
	
	// to do Objective-C stuff in a Lasso tag, we have to set up the Cocoa garbage collection
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	// now make the NSSound which we will use to play the file
	NSSound * sound = [[NSSound alloc] initWithContentsOfFile:[NSString stringWithUTF8String:dst] byReference:NO];
	[sound autorelease]; // this insures that sound goes away when we call the RELEASE macro
	
	if (![sound play]) // some undisclosed error
	{
		err = osErrInvalidParameter;
		lasso_setResultMessage(token, "There was a problem playing the specified sound.");
	}	
	[pool release];
	return err;
}



