import com.omnipilot.lassopro.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class NNTP_DS extends LassoDSModule
{
	Socket sock;
	PrintStream printer;
	BufferedReader reader;
	String host = null;
	int port = 0;
	String user = null, pass = null;
	String hostInfo = "";
	Vector headers = new Vector(10);	
	int refsIdx = -1;
	int xrefIdx = -1;
	int bytesIdx = -1;
	boolean useXpat = false;
	String groupFilter = "";
	String group = "";
	String article = "";
	int groupCount = -1;
	int articleCount = -1;
	
	public void registerLassoModule () {	
		registerDSModule("NNTP", "dsFunc", 0, "Lasso Connector for NNTP", "Simple Usenet client");
	}
	
	public int dsFunc (LassoCall lasso, int cmd, LassoValue value) {
		int err = ERR_NOERR;
		switch (cmd) {
			case ACTION_INIT:
				err = doInit(lasso);
				break;	
			case ACTION_TERM:
				err = doTerm(lasso);
				break;	
			case ACTION_EXISTS:
				if (!value.data().equalsIgnoreCase("News"))
					err = -1;
				break;					
			case ACTION_DB_NAMES:
				err = doDBNames(lasso);
				break;	
			case ACTION_TABLE_NAMES:
				err = doTableNames(lasso, value.data());
				break;			
			case ACTION_INFO:
				err = doInfo(lasso, true);
				break;
			case ACTION_SEARCH:
				err = doSearch(lasso);
				break;
		}
		return err;
	}	
	
	int doInit(LassoCall lasso) {
		return ERR_NOERR;
	}
	
	int doTerm(LassoCall lasso) {
		close();			
		return ERR_NOERR;
	}
	
	int doDBNames(LassoCall lasso) {
		return lasso.addDataSourceResult("News");
	}
	
	int doTableNames(LassoCall lasso, String db) {	
		if (!db.equalsIgnoreCase("News"))
			return -1;
		lasso.addDataSourceResult("Groups");
		lasso.addDataSourceResult("Articles");		
		return ERR_NOERR;
	}
	int doInfo(LassoCall lasso, boolean listAllCols) {
		int err = ERR_NOERR;
		LassoValue tbl = new LassoValue();
		err = lasso.getTableName(tbl);
		if (err != ERR_NOERR || tbl.data().length() == 0)
			return LassoErrors.InvalidParameter;
     	if (!connect(lasso))
			return LassoErrors.Network;
		if (tbl.data().equalsIgnoreCase("Groups")) {
			lasso.addColumnInfo("Group", 0, LassoValue.TYPE_CHAR, PROTECTION_READ_ONLY);
			lasso.addColumnInfo("Last", 0, LassoValue.TYPE_INT, PROTECTION_READ_ONLY);
			lasso.addColumnInfo("First", 0, LassoValue.TYPE_INT, PROTECTION_READ_ONLY);
			lasso.addColumnInfo("AllowPost", 0, LassoValue.TYPE_CHAR, PROTECTION_READ_ONLY);	
		} else if (tbl.data().equalsIgnoreCase("Articles")) {		
			if (!this.headers.isEmpty()) {	
				String str;
				int type, count = headers.size();
				lasso.addColumnInfo("Number", 0, LassoValue.TYPE_INT, PROTECTION_READ_ONLY);
				for (int i = 0; i < count; ++i) {
					str = (String)this.headers.elementAt(i);
					if (str.equalsIgnoreCase("Lines") || str.equalsIgnoreCase("Bytes"))
						type = LassoValue.TYPE_INT;
					else if (str.equalsIgnoreCase("Date"))
						type = LassoValue.TYPE_DATETIME;
					else
						type = LassoValue.TYPE_CHAR;
					err = lasso.addColumnInfo((String)headers.elementAt(i), 0, type, PROTECTION_READ_ONLY);		
				}	
				if (listAllCols) {
					lasso.addColumnInfo("Headers", 0, LassoValue.TYPE_CHAR, PROTECTION_READ_ONLY);
					lasso.addColumnInfo("Body", 0, LassoValue.TYPE_CHAR, PROTECTION_READ_ONLY);
				}
			}
		}
		return err;
	}
	
	int doSearch(LassoCall lasso) {
		int err = ERR_NOERR;
		int skip = 0;
		int max = 50;
		int totalcount = 0;
		String filter = "", reply = "";
		LassoValue tbl = new LassoValue();
		LassoValue val = new LassoValue();
		IntValue ival = new IntValue();	
		if (lasso.getSkipRows(ival) == ERR_NOERR)
			skip = ival.intValue();
		if (lasso.getMaxRows(ival) == ERR_NOERR)
			max = ival.intValue();	
		lasso.getTableName(tbl);
		lasso.getInputColumnCount(ival);	
		if (!connect(lasso))
			return LassoErrors.Network;
		if ((err = doInfo(lasso, max == 1)) != ERR_NOERR)
			return err;
		try {			
			if (tbl.data().equalsIgnoreCase("GROUPS")) {			
				if (lasso.findInputColumn("group", val) == ERR_NOERR) {
					if (val.type() == LassoOperators.OP_ENDS_WITH)
						filter = '*' + val.data();
					else if (val.type() == LassoOperators.OP_CONTAINS)
						filter = '*' + val.data() + '*';
					else if (val.type() == LassoOperators.OP_EQUALS)
						filter = val.data();
					else
						filter = val.data() + '*';		
				}
				this.printer.print("LIST ACTIVE " + filter + "\r\n");			
				reply=reader.readLine();
				if (!reply.startsWith("2"))
					return setError(lasso, reply);
				if (!this.groupFilter.equalsIgnoreCase(filter)) {					
					this.groupFilter = filter;
					this.groupCount = -1;
				}
				err = addGroups(lasso, skip, max);	
			} else if (tbl.data().equalsIgnoreCase("ARTICLES")) {
				if (lasso.findInputColumn("-group", val) == ERR_NOERR ||
					lasso.findInputColumn("group", val) == ERR_NOERR) {
					if (val.data().length() > 0) {
						if (!val.data().equalsIgnoreCase(this.group))
							this.articleCount = -1;
						this.group = val.data();
					}
				}
				if (this.group == null || this.group.length() < 1) {
					lasso.setResultMessage("Missing group parameter.");
					return LassoErrors.InvalidParameter;
				}
				String id = null;
				ival.setInt(0);						
				if (lasso.getRowID(ival) == ERR_NOERR && ival.intValue() != -1)
					id = Integer.toString(ival.intValue());
				else if (lasso.getPrimaryKeyColumn(val) == ERR_NOERR &&
						 (val.name().equalsIgnoreCase("message-id") || 
						  val.name().equalsIgnoreCase("number"))) 
					filter = val.data();
				else if (lasso.findInputColumn("message-id", val) == ERR_NOERR || 
						  lasso.findInputColumn("number", val) == ERR_NOERR) 
					filter = val.data();
				if (this.articleCount == -1) {
					err = selectGroup(lasso);
					if (err != ERR_NOERR)
						return err;
				}	
				if (max == 1 && (filter == null || filter.length() < 1))
					id = getRange(lasso, skip, 1);
				if (filter.startsWith("<") || filter.indexOf('-') == -1)
					id = filter;		
				if (id != null && id.length() > 1) { // detail
					this.printer.print("ARTICLE " + id + "\r\n");
					reply=reader.readLine();
					if (!reply.startsWith("2"))
						return setError(lasso, reply);
					int idx=0, i=0, bytes=0;
					String str;
					String[] data = new String[headers.size()+3];
					data[0] = reply.substring(4, reply.indexOf(' ', 4));
					data[data.length-1] = ""; // body
					data[data.length-2] = ""; // headers
					while(!(reply=reader.readLine()).startsWith(".")) {	
						bytes += reply.length() + 2;
						i = reply.indexOf(": ");
						if (i != -1) { // header
							str = reply.substring(0,i);
							idx = this.headers.indexOf(str);
							if (idx != -1) // known header
								data[idx+1] = reply.substring(i+2);
							else
								data[data.length-2] += reply + '\r';
						} else { // body
							StringBuffer buf = new StringBuffer();
							while (!(reply=reader.readLine()).startsWith(".")) {
								bytes += reply.length() + 2;
								buf.append(reply).append('\r');
							}
							data[data.length-1] = buf.toString();
							data[this.bytesIdx] = Integer.toString(bytes + 2);
							break;
						}
					}
					if (data[0].equals("0") && this.refsIdx != -1) {
						idx = data[this.xrefIdx].lastIndexOf(group);
						if (idx != -1) {
							str = data[this.xrefIdx].substring(idx + group.length() + 1);
							if ((idx=str.indexOf(' ')) != -1)
								str = str.substring(0, idx);
							data[0] = str;
						}
					}
					err = lasso.addResultRow(data);
				} else { // GET LIST
					if (filter == null || filter.length() == 0)
						filter = getRange(lasso, skip, max);
					this.printer.print("XOVER " + filter + "\r\n");
					reply=reader.readLine();
					if (!reply.startsWith("2"))
						return setError(lasso, reply);
					while(err == ERR_NOERR && !(reply=reader.readLine()).startsWith("."))
						err = lasso.addResultRow(split(reply, "\t"));
				}						
				lasso.setNumRowsFound(this.articleCount);
			}	
		} catch (Exception e) {
			System.err.println(e.toString());
			lasso.setResultMessage(e.getMessage());
			err = LassoErrors.Network;
			try { this.sock.close(); } 
				catch (Exception e2) {}
			this.sock = null;
		}
		return err;
	}
	
	int setError(LassoCall lasso, String reply) {
		int err = -1;
		try { 
			err = Integer.parseInt(reply.substring(0, 3));
			lasso.setResultMessage(reply.substring(4));
		} catch (Exception e) {};
		lasso.setResultCode(err);
		return err;
	}
	
	String[] split (String str, String ch) {	
		int i = 0;
		int numcols = headers.size() + 1;
		String cols[] = new String[ numcols ];
		StringTokenizer tok = new StringTokenizer(str, ch);
		int count = tok.countTokens();
		while (tok.hasMoreTokens()) {
			if (i == this.refsIdx && numcols > count)
				cols[i++] = ""; // empty References field
			cols[i++] = tok.nextToken();
		}	
		return cols;
	}
	
	boolean connect(LassoCall lasso) {
		if (getHostInfo(lasso) != ERR_NOERR)
			return false;
		try 
		{
			String reply;
			if (this.sock != null) {
				this.printer.print("MODE READER\r\n"); // probe the connnection
				reply = reader.readLine();
				if (!reply.startsWith("2")) {
					this.sock.close();
					this.sock = null;
				}
			}
			if (this.sock == null) {
				this.sock=new Socket(this.host,this.port);
				this.reader=new BufferedReader(new InputStreamReader(this.sock.getInputStream()), 2500);
				this.printer=new PrintStream(new BufferedOutputStream(this.sock.getOutputStream(),2500),true);
				this.hostInfo = this.reader.readLine();
				login();
				this.printer.print("MODE READER\r\n");
				reader.readLine();
				if (this.headers.isEmpty()) {
					printer.print("LIST OVERVIEW.FMT\r\n");
					reply = reader.readLine();
					if (reply.startsWith("2")) {	
						int idx, i = 1;
						while(!(reply=reader.readLine()).startsWith(".")) {
							idx = reply.indexOf(':');
							if (idx != -1)
								reply = reply.substring(0, idx);
							this.headers.addElement(reply);
							if (reply.equalsIgnoreCase("References"))
								this.refsIdx = i;
							else if (reply.equalsIgnoreCase("Bytes"))
								this.bytesIdx = i;
							else if (reply.equalsIgnoreCase("Xref"))
								this.xrefIdx = i;					
							++i;
						}
					}	
				}
			}
		} catch (Exception e) {
			lasso.setResultMessage(e.getMessage());
			this.sock = null;
			return false;
		}
		return true;
	}
	
	void close() {
		this.host = null;
		this.headers.clear();
		this.groupCount = this.articleCount = -1;
		this.refsIdx = this.xrefIdx = this.bytesIdx = -1;
		try
		{
			this.printer.print("QUIT\r\n");
			this.reader.close();
			this.printer.close();
			this.sock.close();
			this.sock = null;
		} catch (Exception e) {}
	}
	
	boolean login()
	{
		String reply = "";
		if (user != null && user.length() > 0) {
			try
			{
				this.printer.print("AUTHINFO USER " + this.user + "\r\n");
				this.printer.print("AUTHINFO PASS " + this.pass + "\r\n");
				return ((reply=reader.readLine()).startsWith("281"));
			} catch (Exception e) {}
		}
		return false;
	}
	
	int getHostInfo(LassoCall lasso) {
		int err = ERR_NOERR;
		LassoValue hostPort = new LassoValue();
		LassoValue userPass = new LassoValue();
		err = lasso.getDataHost(hostPort, userPass);
		if (err != ERR_NOERR || 
			 hostPort.name() == null || 
			 hostPort.name().length() == 0)
			return err;	
		if (!hostPort.name().equalsIgnoreCase(this.host))
			close();	
		this.host = hostPort.name();
		this.user = userPass.name();
		this.pass = userPass.data();
		try {
			this.port = Integer.parseInt(hostPort.data());
		} catch (Exception e) {}	
		
		if (this.port == 0)
			this.port = 119; // default NNTP port
		return ERR_NOERR;
	}
	
	int addGroups(LassoCall lasso, int skip, int max) {			
		int err = ERR_NOERR;
		int count = 0;
		boolean getFirst = (max == 1 || this.group == null || this.group.length() < 1);
		String reply = "";		
		try {
			while ((skip-- > 0) && !(reply=reader.readLine()).startsWith("."))
				count++;
			if (!reply.startsWith(".")) {
				String row[];
				while (err == ERR_NOERR && !(reply=reader.readLine()).startsWith(".") && (max-- > 0)) {
					count++;
					row = split(reply, " ");
					if (getFirst) {
						this.group = row[0];
						err = lasso.addResultRow(row);
						getFirst = false;
					} else
						err = lasso.addResultRow(row);	
				}
				if (this.groupCount != -1) {
					count = this.groupCount;
					this.sock.close();
					this.sock = null;
				} else if (!reply.startsWith(".")) {
					while (!(reader.readLine()).startsWith("."))
						count++;
					this.groupCount = count;
				}
			}
		} catch (Exception e) { System.err.println(e.toString()); }		
		err = lasso.setNumRowsFound(count);
		return err;
	}
	
	int selectGroup(LassoCall lasso) throws java.io.IOException {
		this.printer.print("GROUP " + this.group + "\r\n"); 
		String reply=reader.readLine();
		if (!reply.startsWith("2"))
			return setError(lasso, reply);
		else
			return ERR_NOERR;
	}
	
	String getRange(LassoCall lasso, int skip, int max) {
		this.printer.print("LISTGROUP " + this.group + "\r\n");
		StringBuffer result = new StringBuffer();
		int count = 0;
		try {
			String reply=reader.readLine();	
			if (reply.startsWith("2")) {
				String last = "";
				while (!(reply=reader.readLine()).startsWith(".") && (skip-- > 0))
					count++;
				if (!reply.startsWith(".")) {
					result.append(reply);
					if (max != 1) 
						result.append("-");
					while (!(reply=reader.readLine()).startsWith(".") && (--max > 0)) {
						count++;
						last = reply;
					}
					if (this.articleCount > -1) {
						this.printer.println("QUIT\r\n");
						this.sock.close();
						this.sock = null;
						count = this.articleCount;
						if (connect(lasso))
							selectGroup(lasso);
					} else if (!reply.startsWith(".")) {
						while (!(reader.readLine()).startsWith("."))
							count++;
						result.append(last);
						this.articleCount = count;
					}
				}
			}
		} catch (Exception e) {} ;
		return result.toString();
	}
}