import com.omnipilot.lassopro.*;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.PdfWriter;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.io.*;
import java.awt.Color;

/**
 * This class holds all the implementations of the member tags and the Accessor functions of the
 * PDF_-BarCode tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFBarCode
{

	private Barcode myBarCode = null;
	private Color barColor = null;
	private Color textColor = null;

	private String barType = null;

	// following two added Aug 03
	private String myAlign = null;
	private boolean myWrap = false;

	/**
	 * Constructor of the class.
	 * The long key value is generated here.
	 */

	public PDFBarCode()
	{
	}


	public Barcode getBarcode()
	{
		return myBarCode;
	}

	public Color getBarcodeColor()
	{
		return barColor;
	}

	public Color getBarTextColor()
	{
		return textColor;
	}

	public String getAlignment()
	{
		return myAlign;
	}

	public void setAlignment(String pAlign)
	{
		myAlign = pAlign;
	}

	public boolean getWrap()
	{
		return myWrap;
	}

	public void setWrap(boolean pWrap)
	{
		myWrap = pWrap;
	}

	/**
	 * This method creates a font
	 * @param face - the font face name.
	 * @param filePath - the name of the file holding the font face.
	 * @param fontSize - the size of the font.
	 * @param fColor - the font color.
	 * @param encoding - the font encoding value.
	 * @param embed - whether the font is to be embedded or not.
	 */

	public void createBarCode(	String filePath, String type, Color pColor, String code,
								float supplemental, boolean generateCheckSum, boolean Code39StartStop,
								boolean EANGaurdBars,float fBarHeight, float fBarWidth,
								float fBaseLine, boolean bShowCheckSum, Font pFont,
								float fBarMultiplier, float fTextSize,
								String sTextAlignment)
	{
		try
		{
			myBarCode = null; //safeguard

			if(type == null) type = "code39";

			barType = type;

			if(type.equalsIgnoreCase("code39"))
			{
				myBarCode = new Barcode39();
		        myBarCode.setCode(code);
			}
			if(type.equalsIgnoreCase("code39_ex"))
			{
				myBarCode = new Barcode39();
		        myBarCode.setCode(code);
		        myBarCode.setExtended(true);
			}
			else if(type.equalsIgnoreCase("code128"))
			{
				myBarCode = new Barcode128();
		        myBarCode.setCode(code);
			}
			else if(type.equalsIgnoreCase("code128_UCC"))
			{
				myBarCode = new Barcode128();
				myBarCode.setCodeType(myBarCode.CODE128_UCC);
		        myBarCode.setCode(code);
			}
			else if(type.equalsIgnoreCase("code128_RAW"))
			{
				myBarCode = new Barcode128();
				myBarCode.setCodeType(myBarCode.CODE128_RAW);
		        myBarCode.setCode(code);
			}
			else if(type.equalsIgnoreCase("EAN13"))
			{
				if(supplemental > 0) {
					int supplementalInt = (int)supplemental;
					BarcodeEAN codeEAN = new BarcodeEAN();
					codeEAN.setCodeType(codeEAN.EAN13);
					codeEAN.setCode(code);

					if(supplementalInt < 100) {
						BarcodeEAN codeSUPP = new BarcodeEAN();
						codeSUPP.setCodeType(codeEAN.SUPP2);
						codeSUPP.setCode(Integer.toString(supplementalInt));
						myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
					} else {
						BarcodeEAN codeSUPP = new BarcodeEAN();
						codeSUPP.setCodeType(codeEAN.SUPP5);
						codeSUPP.setCode(Integer.toString(supplementalInt));
						myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
					}
				} else {
					myBarCode = new BarcodeEAN();
					myBarCode.setCodeType(myBarCode.EAN13);
					myBarCode.setCode(code);
				}
			}
			else if(type.equalsIgnoreCase("POSTNET"))
			{
				myBarCode = new BarcodePostnet();
		        myBarCode.setCode(code);
			}
			else if(type.equalsIgnoreCase("PLANET"))
			{
				myBarCode = new BarcodePostnet();
				myBarCode.setCodeType(myBarCode.PLANET);
		        myBarCode.setCode(code);
			}
			else if(type.equalsIgnoreCase("EAN8"))
			{
				if(supplemental > 0) {
					int supplementalInt = (int)supplemental;
					BarcodeEAN codeEAN = new BarcodeEAN();
					codeEAN.setCodeType(codeEAN.EAN8);
					codeEAN.setCode(code);

					if(supplementalInt < 100) {
						BarcodeEAN codeSUPP = new BarcodeEAN();
						codeSUPP.setCodeType(codeEAN.SUPP2);
						codeSUPP.setCode(Integer.toString(supplementalInt));
						myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
					} else {
						BarcodeEAN codeSUPP = new BarcodeEAN();
						codeSUPP.setCodeType(codeEAN.SUPP5);
						codeSUPP.setCode(Integer.toString(supplementalInt));
						myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
					}
				} else {
					myBarCode = new BarcodeEAN();
					myBarCode.setCodeType(myBarCode.EAN8);
					myBarCode.setCode(code);
				}
			}
			else if(type.equalsIgnoreCase("SUPP2"))
			{
				BarcodeEAN codeEAN = new BarcodeEAN();
				codeEAN.setCodeType(codeEAN.EAN8);
				codeEAN.setCode("9780201615883");

				BarcodeEAN codeSUPP = new BarcodeEAN();
				codeSUPP.setCodeType(codeEAN.SUPP2);
				codeSUPP.setCode(code);
				myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
			}
			else if(type.equalsIgnoreCase("SUPP5"))
			{
				BarcodeEAN codeEAN = new BarcodeEAN();
				codeEAN.setCodeType(codeEAN.EAN8);
				codeEAN.setCode("9780201615883");

				BarcodeEAN codeSUPP = new BarcodeEAN();
				codeSUPP.setCodeType(codeEAN.SUPP5);
				codeSUPP.setCode(code);
				myBarCode = new BarcodeEANSUPP(codeEAN, codeSUPP);
			}

			if(myBarCode != null)
			{
		        myBarCode.setStartStopText(Code39StartStop);
		        myBarCode.setGenerateChecksum(generateCheckSum);
		        myBarCode.setGuardBars(EANGaurdBars);
		        if(pColor != null)
		        	barColor = pColor;

				if(filePath != null)
				{
					//Not implemented as of now
					//there is no method in iText image class to save it to disk

					//PdfContentByte cb = PdfWriter.getInstance(new Document(), new ByteArrayOutputStream()).getDirectContent();
					//Image img = myBarCode.createImageWithBarcode(cb, barColor, null);
					//FileOutputStream fos = new FileOutputStream(filePath);
					//byte[] bytes = img.rawData();
					//fos.write(bytes);
					//fos.close();
				}

				if(fBarHeight>0)
					setBarHeight(fBarHeight);
				if(fBarWidth>0)
					setBarWidth(fBarWidth);
				if(fBaseLine>0)
					setBaseline(fBaseLine);
				setShowChecksum(bShowCheckSum);
				if(pFont!=null)
					setFont(pFont);
				if(fBarMultiplier>0)
					setBarMultiplier(fBarMultiplier);
				if(fTextSize>0)
					setTextSize(fTextSize);

				setTextAlignment(sTextAlignment);

			}
		}
		catch(Exception de) {
			de.printStackTrace();
		}
	}

	/**
	 * This method sets the BarHeight.
	 * @param x1 - the  x height.
	 */

	public int setBarHeight(float x1)
	{
			myBarCode.setBarHeight(x1);
			return 0;

	}

	/** Sets the minimum bar width.
     * @param x the minimum bar width
     */
    public int setBarWidth(float x) {
        myBarCode.setX(x);
        return 0;
    }

	/** Sets the text baseline.
     * If positive, the text distance under the bars. If zero or negative,
     * the text distance above the bars.
     * @param baseline the baseline.
     */

    public int setBaseline(float baseline) {
        myBarCode.setBaseline(baseline);
        return 0;
    }

	/** Setter for property generateChecksum.
     * @param generateChecksum New value of property generateChecksum.
     */
    public int setShowChecksum(boolean Checksum) {
        myBarCode.setChecksumText(Checksum);
        return 0;
    }

    /** Sets the code to generate.
     * @param code the code to generate
     */
    public int setCode(String code) {
        if(code != null)
        {
	        myBarCode.setCode(code);
    	    return 1;
    	}
    	return 0;
    }

	/** Sets the text font. <CODE>null</CODE> if no text.
     * @param font the text font. <CODE>null</CODE> if no text
     */
    public int setFont(Font font) {
		if(font != null)
		{
			BaseFont f = font.getBaseFont();
			myBarCode.setFont(f);
			textColor = font.color();
			myBarCode.setSize(font.size());
		}

        return 0;
    }

	/** Setter for property generateChecksum.
     * @param generateChecksum New value of property generateChecksum.
     */
    public int setGenerateChecksum(boolean generateChecksum) {
        myBarCode.setGenerateChecksum(generateChecksum);
        return 0;

    }

    /** Sets the bar multiplier for wide bars.
     * @param n the bar multiplier for wide bars
     */
    public int setBarMultiplier(float n) {
        myBarCode.setN(n);
        return 0;
    }

    /** Sets the size of the text.
     * @param size the size of the text
     */
    public int setTextSize(float size) {
        myBarCode.setSize(size);
        return 0;
    }

    /** Sets the text alignment. Can be <CODE>Element.ALIGN_LEFT</CODE>,
     * <CODE>Element.ALIGN_CENTER</CODE> or <CODE>Element.ALIGN_RIGHT</CODE>.
     * @param textAlignment the text alignment
     */
    public int setTextAlignment(String textAlignment) {

		if(textAlignment.equalsIgnoreCase("left"))
	        myBarCode.setTextAlignment(Element.ALIGN_LEFT);
		else if(textAlignment.equalsIgnoreCase("center"))
	        myBarCode.setTextAlignment(Element.ALIGN_CENTER);
		else if(textAlignment.equalsIgnoreCase("right"))
	        myBarCode.setTextAlignment(Element.ALIGN_RIGHT);

        return 0;
    }

	/** Gets the maximum area that the barcode and the text, if
     * any, will occupy. The lower left corner is always (0, 0).
     * @return the size the barcode occupies.
     */

	public Rectangle getSize() {
	return myBarCode.getBarcodeSize();
	}

	/** Gets the height of the bars.
     * @return the height of the bars
     */
    public float getBarHeight() {
        return myBarCode.getBarHeight();
    }

    /** Gets the minimum bar width.
     * @return the minimum bar width
     */
    public float getBarWidth() {
        return myBarCode.getX();
    }

    /** Gets the text baseline.
     * If positive, the text distance under the bars. If zero or negative,
     * the text distance above the bars.
     * @return the baseline.
     */
    public float getBaseline() {
        return myBarCode.getBaseline();
    }

	/** Gets the code to generate.
     * @return the code to generate
     */
    public String getCode() {
        return barType.toUpperCase();
    }

    /** Gets the code type.
     * @return the code type
     */
    public int getType() {
        return myBarCode.getCodeType();
    }

	/** Gets the text font. <CODE>null</CODE> if no text.
     * @return the text font. <CODE>null</CODE> if no text
     */
     //should return PDF_FONT type data.

    public BaseFont getFont() {
        return myBarCode.getFont();

    }

	/** Gets the bar multiplier for wide bars.
     * @return the bar multiplier for wide bars
     */
    public float getBarMultiplier() {
        return myBarCode.getN();
    }

    /** Gets the size of the text.
     * @return the size of the text
     */
    public float getTextSize() {
        return myBarCode.getSize();
    }


	/** Gets the text alignment. Can be <CODE>Element.ALIGN_LEFT</CODE>,
     * <CODE>Element.ALIGN_CENTER</CODE> or <CODE>Element.ALIGN_RIGHT</CODE>.
     * @return the text alignment
     */
    public int getTextAlignment() {
        return myBarCode.getTextAlignment();
    }

    /** Gets the property to show the generated checksum in the the text.
     * @return value of property checksumText
     */
    public boolean showChecksum() {
        return myBarCode.isChecksumText();
    }


	/** Gets the optional checksum generation.
     * @return the optional checksum generation
     */
    public boolean generateChecksum() {
        return myBarCode.isGenerateChecksum();
    }

	/** Gets the property to show the guard bars for barcode EAN.
     * @return value of property guardBars
     */
    public boolean showEANGuardBars() {
        return myBarCode.isGuardBars();
    }

    /** Sets the property to show the start and stop character '*' in the text for
     * the barcode 39.
     * @return value of property startStopText
     */
    public boolean showCode39StartStop() {
        return myBarCode.isStartStopText();
    }
}
