import com.omnipilot.lassopro.*;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.rtf.*;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.Color;


/**
 * This class holds all the implementations of the member tags and the Accessor functions of the
 * PDF_Doc tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */
public class PDFDoc
{

	Document myDocument 		= null;
	Header myHeader				= null;
	Paragraph myParagraph		= null;
	Section mySection 			= null;
	Chapter myChapter			= null;
	Graphic myGraphics			= null;
	Font mygFont				= null;
	ByteArrayOutputStream bos	= null;
	String strFileName 			= null;
	PdfAcroForm acroForm		= null;
	Hashtable htRg				= null;
	PdfContentByte cb			= null;
	PdfWriter pdfwriter			= null;
	RtfWriter rtfwriter			= null;
	PdfCopy copyWriter			= null;

	String author = null;
	String title = null;
	String subject = null;
	String keywords = null;
	String creator = null;

	boolean	  bRtf				= false;

	/**
	 * Constructor of the class.
	 * The long key value is generated here.
	 */
	public PDFDoc()
	{
		myDocument 	= new Document();
		myGraphics 	= new Graphic();
		mygFont 	= new Font();
		bos 		= new ByteArrayOutputStream();
		htRg		= new Hashtable();
		bRtf 		= false;
	}

	public Document getDocument()
	{
		return myDocument;
	}

	/**
	 * This method sets the font of the Document.
	 */
	public int setFont(Font f) throws Exception
	{
		if(f!=null)
			mygFont = f;
		return 0;
	}

	/**
	 * This method sets the color of the font set above.
	 * @param fColor- The color to be set for the document.
	 */
	public int setDocColor(String  fColor)
	{
		if(fColor==null || fColor.trim().equals(""))
		{
			if(mygFont.color()==null)
				mygFont.setColor(new Color(0, 0, 0));
		}
		else
			mygFont.setColor(Color.decode(fColor));
		return 0;
	}

	/**
	 * This method returns the color of the font set above.
	 * @return      the color of the Document.
	 */
	public String getDocColor()
	{
		Color clr=null ;
		if(mygFont != null)
			clr = mygFont.color();  // new Color(200,200,200);
		String retStr = null;
		if(clr != null)
		{
			retStr = this.toHexStr(clr);
		}
		return retStr;
	}

	/**
	 * This method converts the color of the font to HexString format.
	 * @return      the String containing the color.
	 */
	public String toHexStr( Color c )
	{
		String red = Integer.toHexString(c.getRed());
		String green = Integer.toHexString(c.getGreen());
		String blue = Integer.toHexString(c.getBlue());

		if (red.length() != 2 )
			red = "0"+red;
		if (green.length() != 2 )
			green = "0"+green;
		if (blue.length() != 2 )
			blue = "0"+blue;

		return new String( red+green+blue );
	}

	/**
	 * This method creates the document with given filename.
	 * @param fileName - the name of the PDFFile to be created.
	 * @param strWaterMarkImage - the WaterMark Image to be added to the document.
	 * @param leftWaterMark - the left position.
	 * @param topWaterMark - the top position.
	 */
	public int createDoc(String fileName, String strWaterMarkImage, float leftWaterMark, float topWaterMark, String format)
	{
		try
		{	//the getInstance() is a static method of class PdfWriter.

			if(mygFont.color()==null)
				mygFont.setColor(new Color(0, 0, 0));

			if(fileName != null)
				strFileName = fileName;

			if(format != null && format.equalsIgnoreCase("RTF"))
			{
				rtfwriter = RtfWriter.getInstance(myDocument, bos);
	            acroForm = null;
	            bRtf = true;
			}
			else
			{
				pdfwriter = PdfWriter.getInstance(myDocument, bos);
	            acroForm = pdfwriter.getAcroForm();
	            /* this would cause problem as it opens another pdf writer internally
	            to this doc  */
	           // copyWriter = new PdfCopy(myDocument, bos);
			}

			if(!bRtf) {
			/*	pdfwriter.open();
				try{
				cb = pdfwriter.getDirectContent();
				}catch(RuntimeException rte){
				}
			*/
				if(strWaterMarkImage != null)
					docAddWatermark(strWaterMarkImage, leftWaterMark, topWaterMark);
			}
           	return 0;
		}
		catch(DocumentException de)
		{
			System.err.println(de.getMessage());
			return 1;
		}
	}

	public String getPDFString()
	{
		myDocument.close();
		return bos.toString();
	}

	/**
	 * This method opens the document so that Texts and all could be added to it.
	 */
	public void openDoc()
	{
		myDocument.open();
		cb = pdfwriter.getDirectContent();
	}

	/**
	 * This method closes the document.This commits the document to the disk and saves it.
	 */
	public void closeDoc()
	{
		try
		{
			myDocument.close();
			if (strFileName != null)
			{
				FileOutputStream fos = new FileOutputStream(strFileName);
				bos.writeTo(fos);
				fos.close();
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public String pdfToString()
	{
		try
		{
			byte[] bytearray = cb.toPdf(null);
			return new String(bytearray, "UTF8");
		}
		catch(UnsupportedEncodingException e)
		{
			e.printStackTrace();
			return null;
		}
	}


	/**
	 * Sets the pagesize.
	 * @param	rec	- the new pagesize
	 */

	public int setPageSize(Rectangle rec)
	{
		myDocument.setPageSize(rec);
		return 0;
	}

	/**
	 * Sets the pagesize.
	 * @param	ps	- the string value (A4,A5,A6....etc)
	 */
	public int setPageSize(String ps)
	{

		Rectangle pageSize = PageSize.A4;

	    if(ps.equalsIgnoreCase("LETTER"))
	    	pageSize = PageSize.LETTER;
	    else if(ps.equalsIgnoreCase("NOTE"))
	    	pageSize = PageSize.NOTE;
	    else if(ps.equalsIgnoreCase("LEGAL"))
	    	pageSize = PageSize.LEGAL;
	    else if(ps.equalsIgnoreCase("A0"))
	    	pageSize = PageSize.A0;
	    else if(ps.equalsIgnoreCase("A1"))
	    	pageSize = PageSize.A1;
	    else if(ps.equalsIgnoreCase("A2"))
	    	pageSize = PageSize.A2;
	    else if(ps.equalsIgnoreCase("A3"))
	    	pageSize = PageSize.A3;
	    else if(ps.equalsIgnoreCase("A4"))
	    	pageSize = PageSize.A4;
	    else if(ps.equalsIgnoreCase("A5"))
	    	pageSize = PageSize.A5;
	    else if(ps.equalsIgnoreCase("A6"))
	    	pageSize = PageSize.A6;
	    else if(ps.equalsIgnoreCase("A7"))
	    	pageSize = PageSize.A7;
	    else if(ps.equalsIgnoreCase("A8"))
	    	pageSize = PageSize.A8;
	    else if(ps.equalsIgnoreCase("A9"))
	    	pageSize = PageSize.A9;
			else if(ps.equalsIgnoreCase("A10"))
	    	pageSize = PageSize.A10;
	    else if(ps.equalsIgnoreCase("B0"))
	    	pageSize = PageSize.B0;
	    else if(ps.equalsIgnoreCase("B1"))
	    	pageSize = PageSize.B1;
	    else if(ps.equalsIgnoreCase("B2"))
	    	pageSize = PageSize.B2;
	    else if(ps.equalsIgnoreCase("B3"))
	    	pageSize = PageSize.B3;
	    else if(ps.equalsIgnoreCase("B4"))
	    	pageSize = PageSize.B4;
	    else if(ps.equalsIgnoreCase("B5"))
	    	pageSize = PageSize.B5;
	    else if(ps.equalsIgnoreCase("ARCH_E"))
	    	pageSize = PageSize.ARCH_E;
	    else if(ps.equalsIgnoreCase("ARCH_D"))
	    	pageSize = PageSize.ARCH_D;
	    else if(ps.equalsIgnoreCase("ARCH_C"))
	    	pageSize = PageSize.ARCH_C;
	    else if(ps.equalsIgnoreCase("ARCH_B"))
	    	pageSize = PageSize.ARCH_B;
	    else if(ps.equalsIgnoreCase("ARCH_A"))
	    	pageSize = PageSize.ARCH_A;
	    else if(ps.equalsIgnoreCase("FLSA"))
	    	pageSize = PageSize.FLSA;
	    else if(ps.equalsIgnoreCase("FLSE"))
	    	pageSize = PageSize.FLSE;
	    else if(ps.equalsIgnoreCase("HALFLETTER"))
	    	pageSize = PageSize.HALFLETTER;
	    else if(ps.equalsIgnoreCase("_11X17"))
	    	pageSize = PageSize._11X17;
	    else if(ps.equalsIgnoreCase("TABLOID"))
	    	pageSize = PageSize._11X17;
	    else if(ps.equalsIgnoreCase("LEDGER"))
	    	pageSize = PageSize.LEDGER;


		myDocument.setPageSize(pageSize);
		return 0;

	}

	/**
	 * This method sets  the documents margins.
	 * @param marginLeft - the left side margin value.
	 * @param marginRight - the right side margin value.
	 * @param marginTop - the top margin value.
	 * @param marginBottom - the bottom margin value.
	 */

	public int setMargins(float marginLeft, float marginRight, float marginTop, float marginBottom)
	{
		myDocument.setMargins(marginLeft, marginRight, marginTop, marginBottom);
		return 0;
	}


	/**
	 * This method sets  the document's header.
	 * @param header - the header value to be set.
	 */

	public int setHeader(String header)
	{

		Phrase ph = new Phrase(header);
		HeaderFooter hf = new HeaderFooter(ph,false);
		myDocument.setHeader(hf);
		return 0;

	}

	/**
	 * This method sets  the document's creation date to the current date.
	 */

	public int setUseDate()
	{
		try
		{
			myDocument.addCreationDate();
			return 0;
		}
		catch(Exception e)
		{
			return 1;
		}

	}

	/**
	 * This method sets  the document's compress variable to true.
	 * Allows the pdf documents to be produced without compression for debugging purposes.
	 * @param compState - the true/false  value to be set.
	 */

	public int setCompress(boolean bState)
	{
		myDocument.compress = bState;
		return 0;
	}

	/**
	 * This method sets  the document's title.
	 * @param strTitle - the title to be set to the document.
	 */

	public int setDocTitle(String strTitle)
	{
		if(strTitle != null)
		{
		title = strTitle;
		if(myDocument.addTitle(strTitle)== true)
			return 0;
		else
			return 1;
		}
		return 1;
	}

	/**
	 * This method sets  the document's subject to the specified string.
	 * @param strSubject - the subject of the document to be set.
	 */

	public int setDocSubject(String strSubject)
	{
		if(strSubject != null)
		{
		subject = strSubject;
		if(myDocument.addSubject(strSubject)== true)
			return 0;
		else
			return 1;
		}
		return 1;
	}

	/**
	 * This method sets  the document's author to the specified name.
	 * @param strAuthor - the author name of the document to be set.
	 */

	public int setDocAuthor(String strAuthor)
	{
		if(strAuthor != null)
		{
		author = strAuthor;
		if(myDocument.addAuthor(strAuthor)== true)
			return 0;
		else
			return 1;
		}
		return 1;
	}

	/**
	 * This method sets  the document's keywords to the specified string.
	 * @param strKeyWords - the keywords for the document to be set.
	 */

	public int setDocKeywords(String strKeywords)
	{
		if(strKeywords != null)
		{
		keywords = strKeywords;
		if(myDocument.addKeywords(strKeywords)== true)
			return 0;
		else
			return 1;
		}
		return 1;
	}

	/**
	 * This method sets  the document's Creator to the specified string.
	 * @param strCreator - the creator of the document to be set.
	 */

	public int setDocCreator(String strCreator)
	{
		if(strCreator != null)
		{
		creator = strCreator;
		if(myDocument.addCreator(strCreator)== true)
			return 0;
		else
			return 1;
		}
		return 1;
	}

	public String getDocCreator()
	{
		return creator;
	}

	public String getDocAuthor()
	{
		return author;
	}

	public String getDocTitle()
	{
		return title;
	}

	public String getDocKeywords()
	{
		return keywords;
	}

	public String getDocSubject()
	{
		return subject;
	}




	/**
	 * This method sets  the document's creation date.
	 */

	public int setDocCreationDate()
	{
		if(myDocument.addCreationDate()== true)
			return 0;
		else
			return 1;
	}


	/**
	 * This method returns the  margins of the document.
	 * @return - the array containing the margins.
	 */

	public  float[] getDocMargins()
	{
		float[] tempArr = new float[4];

    	tempArr[0] = myDocument.leftMargin();
    	tempArr[1] = myDocument.rightMargin();
    	tempArr[2] = myDocument.topMargin();
    	tempArr[3] = myDocument.bottomMargin();
        return tempArr;

	}

	/**
	 * This method returns the  size of the document.
	 * @return - the array containing the height & the width.
	 */

    public int[] getDocSize()
    {
    	int[] sizeArr = new int[2];
    	Rectangle rect = myDocument.getPageSize();
    	if(rect != null)
    	{
    		sizeArr[0] = (int)rect.width();
    		sizeArr[1] = (int)rect.height();
    		return sizeArr;
    	}
    	else
    		return null;

	}

	/**
	 * Returns the document format.
	 * Currently it is returning "PDF".
	 * @return - the String holding the format.
	 */
    public String getDocFormat()
    {
		if(bRtf) return "RTF";
		return "PDF";
	}

	/**
	 * Returns the documentHeaders.
	 * @return - the array containing the formats value.
	 */
    public String [] getDocHeaders()
    {

		String[] headers = {
		getDocCreator(),
		getDocAuthor(),
		getDocTitle(),
		getDocSubject(),
		getDocKeywords()
		};


		return headers;
		//TODO
		//return myDocument.getHeaders();

	}

	/**
	 * Returns the document header.
	 * @return - the String holding the Header.
	 */
    public String getDocHeader(String strHeader)
    {

		return strHeader;

	}

   	/**
	 * Returns the document current page number.
	 * @return - the Current page number.
	 */
    public int getDocPageNumber()
    {
    	return myDocument.getPageNumber();
	}

	/**
	 * Sets the  document page number.
	 * @param pNum - the page number to be set.
	 */

    public int setDocPageNumber(int pNum)
    {
		    myDocument.setPageCount(pNum);
    		return 0;
    }


	/**
	 * Adds chunk to the document.
	 * @param strText - the chunk text.
	 * @param fName - the font name.
	 * @param strUrl - the URL name.
	 * @param anchorName - the anchor name.
	 * @param strGoTo - the local destination.
	 * @param strFileName - the file name.
	 */

	public int docAddText(String strText,Font fName,String strUrl,String anchorName,String strGoTo,String strFileName)
	{

	try	{
		Font f=null;

		if(fName != null) f = fName;
		else if(mygFont != null) f = mygFont;

		Chunk myChunk = new Chunk(strText);
		myChunk.setFont(f);
		if(strUrl != null && (strUrl.length() > 0))
			myChunk.setAnchor(strUrl);
		else if(strGoTo != null && (strFileName == null || strFileName.length() <= 0))
			myChunk.setLocalGoto(strGoTo);
		else if(strGoTo != null && (strFileName != null && strFileName.length() > 0))
			myChunk.setRemoteGoto(strFileName,strGoTo);

		if(anchorName != null)
			myChunk.setLocalDestination(anchorName);

		myDocument.add(new Phrase(myChunk));
		return 0;
	}
	catch(Exception e)	{
		e.printStackTrace();
		return 1;
	}

	}


	/**
	 * Adds a phrase to the document.
	 * @param strPhraseText - the phrase text.
	 * @param fLeading - the Leading value.
	 * @param fName - the font name.
	 * @param strUrl - the URL name.
	 * @param anchorName - the anchor name.
	 * @param strGoTo - the local destination.
	 * @param strFileName - the file name.
	 */

	public int docAddPhrase(String strPhraseText, float fLeading, Font fontName, String strUrl, String anchorName, String strGoTo, String strFileName)
	{
	try	{

		Font f=null;

		if(fontName != null) f = fontName;
		else if(mygFont != null) f = mygFont;

		Phrase myPhrase = null;
		if(f!=null)
			myPhrase = new Phrase(strPhraseText,f);
		else
			myPhrase = new Phrase(strPhraseText);

		myPhrase.setLeading(fLeading);
		myDocument.add(myPhrase);
		return 0;
	}
	catch(Exception e)	{
		e.printStackTrace();
		return 1;
	}

	}

	/**
	 * Adds a phrase to the document.
	 * @param strParagraphText - the paragraph text.
	 * @param fLeading - the Leading value.
	 * @param fName - the font name.
	 * @param strUrl - the URL name.
	 * @param anchorName - the anchor name.
	 * @param strGoTo - the local destination.
	 * @param strFileName - the file name.
	 */

	//Leading,Alignment parameter added on 12-09-2002
	public int docAddParagraph(String strParagraphText, float fLeading,float fMarginRight,float fMarginLeft,String strAlignment,Font fontName, String strUrl, String anchorName, String strGoTo, String strFileName)
	{
	try	{

		Font f=null;

		if(fontName != null) f = fontName;
		else if(mygFont != null) f = mygFont;


		if(f!=null)
			myParagraph = new Paragraph(fLeading,strParagraphText,f); //Leading Added
		else
			myParagraph = new Paragraph(fLeading,strParagraphText); //Leading added

		if(strAlignment != "")
			myParagraph.setAlignment(strAlignment); //Alignment set

		if( fMarginLeft > 0)
			myParagraph.setIndentationLeft(fMarginLeft); //Left Margin set

		if(fMarginRight > 0)
			myParagraph.setIndentationRight(fMarginRight); //Right Margin set

		myDocument.add(myParagraph);
		return 0;

	}
	catch(Exception e)	 {
		e.printStackTrace();
		return 1;
	}
	}


	public int docDrawText(String strText,Font fontName, float fTop,float fLeft,float fWidth,float fHeight,float fLeading,String strAlignment)
	{
	try	{

	//	if(fontName != null) myDocument.setFont(fontName);


	  // we draw some text on a certain position
		float s = 0;
		BaseFont bf = null;


		if(fontName == null) fontName = mygFont;

		if(fontName != null)
		{
			bf = fontName.getBaseFont();
			s = fontName.size();
			if(bf == null)
				bf = BaseFont.createFont("Helvetica", BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
			if(s<=0)
				s = 12;
		}

		if(fLeading > 0)
			cb.setLeading(fLeading);

		if(fLeft > 0 && fTop > 0)
		{
			//Added by ansuman Nov17 2003
			fTop = getConventionalTop(fTop);
			cb.beginText();
			cb.setFontAndSize(bf, s);
			cb.setTextMatrix(fLeft, fTop);
			cb.showText(strText);
			cb.endText();
		}
	//	myDocument.add(myParagraph);
		return 0;

	}
	catch(Exception e)	 {
		e.printStackTrace();
		return 1;
	}
	}

	/**
	 * Adds a section to the document.
	 * @param pText - PDFText object
	 * @param fIndent - section indent
	 * @param bHideNo - hide number or not.
	 */
	public int docAddSection(PDFText pText, float fIndent, boolean bHideNo)
	{
	try	{
		Phrase sTitle = pText.getNestedText(cb);
		mySection = myChapter.addSection(new Paragraph(sTitle),1);

		if(fIndent!=0)
			mySection.setIndentation(fIndent);

		mySection.add(myParagraph);
		if(bHideNo)
			mySection.setNumberDepth(0);
		else
			mySection.setNumberDepth(1);

		return 0;
	}
	catch(Exception e)	{
		e.printStackTrace();
		return 1;
	}
	}


	/**
	 * Adds a Chapter to the document.
	 * @param pText - PDFText object
	 * @param iNumber - the Chapter number.
	 * @param bHideNo - hide number or not.
	 */

	public int docAddChapter(PDFText pText, int iNumber, boolean bHideNo)
	{
	try	{
		Phrase sTitle = pText.getNestedText(cb);
		myChapter = new Chapter(new Paragraph(sTitle),iNumber);
		//myChapter.add(myParagraph);
		if(bHideNo)
			myChapter.setNumberDepth(0);
		else
			myChapter.setNumberDepth(1);
		myDocument.add(myChapter);
		return 0;
	}
	catch(Exception e)	{
		e.printStackTrace();
		return 1;
	}
	}


	/**
	 * Adds a List to the document.
	 * @param arr[] - the array containing the listItems.
	 * @param strFormat - the Format value.
	 * @param fontName - the font name.
	 * @param strUrl - the URL name.
	 * @param anchorName - the anchor name.
	 * @param strGoTo - the local destination.
	 * @param strFileName - the file name.
	 */

	public int docAddList(String arr[], String strFormat, Font fontName, String strUrl, String anchorName, String strGoTo, String strFileName)
	{
	try
	{

		com.lowagie.text.List myList = null;
		String [] listArr = new String[arr.length];

		if(strFormat == null)
			strFormat = "Number";

		if(strFormat.equalsIgnoreCase("Number"))
			myList = new com.lowagie.text.List(true, 10);
		else if(strFormat.equalsIgnoreCase("Letter"))
			myList = new com.lowagie.text.List(false, true, 10);
		else
			myList = new com.lowagie.text.List(true, 10);

		for(int count = 0; count < listArr.length; count ++)
		{
			listArr[count] = arr[count];

			myList.add(new ListItem(listArr[count]));

		}
		myDocument.add(myList);
		return 0;
	}
	catch(DocumentException de)	{
		de.printStackTrace();
		return 1;
	}

	}


	/**
	 * Adds a Image to the document.
	 * @param filPath - the image file.
	 * @param xOffSet - the X position.
	 * @param yOffSet - the Y position.
	 */

	public int docAddImage(String filePath, float xOffSet, float yOffSet)
	{
	try
	{

		Image img = Image.getInstance(filePath);
		img.setAbsolutePosition(xOffSet, yOffSet);
		myDocument.add(img);
		return 0;
	}
	catch(MalformedURLException mue)
	{
        mue.printStackTrace();
        return 1;
    }
    catch(DocumentException de)
    {
 		de.printStackTrace();
 		return 1;
    }
    catch(IOException ioe)
    {
        ioe.printStackTrace();
        return 1;
    }

	}

	/**
	 * Adds a WaterMark to the document.
	 * @param filPath - the image file.
	 * @param xOffSet - the X position.
	 * @param yOffSet - the Y position.
	 */


	public int docAddWatermark(String filePath,float xOffSet,float yOffSet)
	{
	try
	{
		Image img = Image.getInstance(filePath);
//		Watermark wm = new Watermark(img,xOffSet, yOffSet);
		Watermark wm = new Watermark(img,xOffSet, yOffSet);
		myDocument.add(wm);
		return 0;
	}
	catch(Exception e)
	{
		e.printStackTrace();
		return 1;
	}
	}


	/**
	 * Adds a new page to the document.
	 */

	public int docAddPage()
	{
	try
	{
		if(myDocument.newPage()) return 0;
		else return 1;
	}
	catch(Exception e)
	{
		e.printStackTrace();
		return 1;
	}

	}

	public int  getElementType()
	{

		return 0;

	}

	public int getElement()
	{

		return 0;

	}

	public String getElementCount()
	{

		return null;

	}


	/**
	 * Sets the line width for drawing operations.
	 * @param width - the width value.
	 */

	public int docSetLineWidth(float width)
	{

		myGraphics.setLineWidth(width);
		return 0;

	}


	/**
	 * Draws a straight line on the document.
	 * @param x1 - the X1 coordinate.
	 * @param y1 - the Y1 coordinate.
	 * @param x2 - the X2 coordinate.
	 * @param y2 - the Y2 coordinate.
	 */

//	public int docLine(int x1,int y1,int x2,int y2)
	public int docLine(float x1,float y1,float x2,float y2)
	{
	try	{
		myGraphics.moveTo(x1, y1);
		myGraphics.lineTo(x2, y2);
		myGraphics.stroke();
		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException de) {
		de.printStackTrace();
		return 1;
	}

	}

	/**
	 * Draws a line on the document.
	 * @param x1 - the X1 coordinate.
	 * @param y1 - the Y1 coordinate.
	 */

//	public int docLineTo(int x1,int y1)
	public int docLineTo(float x1,float y1)
	{
	try	{
		myGraphics.lineTo(x1, y1);
		myGraphics.stroke();
		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException de) {
		de.printStackTrace();
		return 1;
	}

	}


	/**
	 * Draws a Rectangle on the document.
	 * @param x - the X coordinate.
	 * @param y - the Y coordinate.
	 * @param w - the width value.
	 * @param h - the height value.
	 */

	public int docRect(float x, float y, float w, float h)
	{
	try	{
		myGraphics.rectangle(x, y, w, h);
		myGraphics.stroke();
		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException de) {
		de.printStackTrace();
		return 1;
	}
	}

	public int docMoveTo(float x, float y)
	{

		myGraphics.moveTo(x, y);
		return 0;

	}

	/**
	 * This method adds a graphic  object to the document.
	 * @param g - the graphic object.
	 */

	public int addGraphics(Graphic g) throws Exception
	{

		myDocument.add(g);
		return 0;

	}

	/**
	 * This method adds a table  object to the document.
	 * @param t - the table object.
	 */

	public int addTable(Table t) throws Exception
	{
		myDocument.add(t);
		return 0;
	}

	/**
	 * This method adds a barcode  object to the document.
	 * @param t - the barcode object.
	 */

	public int addBarcode(PDFBarCode pbc, float fLeft, float fTop) throws Exception
	{
		 Barcode bc = pbc.getBarcode();
		 if(bc != null)
		 {
			Image img = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());
			fTop = getConventionalTop(fTop);
			img.setAbsolutePosition(fLeft, fTop);
			pdfwriter.getDirectContent().addImage(img); //, img.plainWidth(), 0, 0, img.plainHeight(), fLeft, fTop-100);
			myDocument.add(img);
		}
		return 0;
	}

	/**
	 * Draws a curve on the document.
	 * @param x1 - the X1 coordinate.
	 * @param y1 - the Y1 coordinate.
	 * @param x2 - the X2 coordinate.
	 * @param y2 - the Y2 coordinate.
	 * @param x3 - the X3 coordinate.
	 * @param y3 - the Y3 coordinate.
	 */

	public int docCurveTo(float x1, float y1, float x2, float y2, float x3, float y3)
	{
	try	{
		myGraphics.moveTo(x1, y1);
		myGraphics.curveTo(x1, y1, x2, y2, x3, y3);
		myGraphics.stroke();

		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException e)	{
		e.printStackTrace();
		return 1;
	}

	}


	/**
	 * Draws a circle on the document.
	 * @param x - the X coordinate.
	 * @param y - the Y coordinate.
	 * @param r - the radius value.
	 */

	public int docCircle(float x, float y, float r)
	{
	try	{
		myGraphics.circle(x, y, r);
		myGraphics.stroke();
		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException de) {
		de.printStackTrace();
		return 1;
	}
	}

	/**
	 * Draws a arc on the document.
	 * @param x1 - the X1 coordinate.
	 * @param y1 - the Y1 coordinate.
	 * @param r - the radius value.
	 * @param startAng- the start angle.
	 * @param extent - the extent of the angle position.
	 */

	public int docArc(float x1, float y1, float r, float startAng, float extent)
	{
	try	{

		x1 = x1 -r;
		y1 = y1 - r;


		float x2 = 0;
		float y2 = 0;
		x2 = x1 + 2*r;
		y2 = y1 + 2*r;
		myGraphics.arc(x1, y1, x2, y2, startAng, extent);
		myGraphics.stroke();
		myDocument.add(myGraphics);
		return 0;
	}
	catch(DocumentException de) {
		de.printStackTrace();
		return 1;
	}

	}

	/**
     * Closes the path, fills it using the non-zero winding number rule to determine the region to fill and strokes it.
     */

	public int docClosePathFillStroke()
	{

		myGraphics.closePathFillStroke();
		myGraphics.stroke();
		return 0;

	}


	/**
     * Changes the currentgray tint for filling paths (device dependent colors!).
     * Sets the color space to <B>DeviceGray</B> (or the DefaultGray color space),
     * and sets the gray tint to use for filling paths.
     * @param	gray	a value between 0 (black) and 1 (white)
     */

	public int docSetGrayFill(float gray) {
		myGraphics.setGrayFill(gray);
		return 0;
	}

	/**
     * Changes the currentgray tint for stroking paths (device dependent colors!).
     * Sets the color space to <B>DeviceGray</B> (or the <B>DefaultGray</B> color space),
     * and sets the gray tint to use for stroking paths.
     * @param	gray	a value between 0 (black) and 1 (white)
     */

	public int docSetGrayStroke(float gray) {
		myGraphics.setGrayStroke(gray);
		return 0;
	}


	/**
     * Changes the current color for filling paths (device dependent colors!).
     * Sets the color space to <B>DeviceRGB</B> (or the <B>DefaultRGB</B> color space),
     * and sets the color to use for filling paths.</P>
     * Following the PDF manual, each operand must be a number between 0 (minimum intensity) and
     * 1 (maximum intensity).</P>
     * @param	red		the intensity of red. A value between 0 and 1
     * @param	green	the intensity of green. A value between 0 and 1
     * @param	blue	the intensity of blue. A value between 0 and 1
     */

	public int docSetRGBColorFillF(float red, float green, float blue) {
		myGraphics.setRGBColorFillF(red,green,blue);
		return 0;
	}


	/**
     * Changes the current color for stroking paths (device dependent colors!).
     * Sets the color space to <B>DeviceRGB</B> (or the <B>DefaultRGB</B> color space),
     * and sets the color to use for stroking paths.</P>
     * Following the PDF manual, each operand must be a number between 0 (miniumum intensity) and
     * 1 (maximum intensity).
     * @param	red		the intensity of red. A value between 0 and 1
     * @param	green	the intensity of green. A value between 0 and 1
     * @param	blue	the intensity of blue. A value between 0 and 1
     */

	public int docSetRGBColorStrokeF(float red, float green, float blue) {
        myGraphics.setRGBColorStrokeF(red,green,blue);
		return 0;

    }

    /**
     * Changes the current color for filling paths (device dependent colors!).
     * Sets the color space to <B>DeviceCMYK</B> (or the <B>DefaultCMYK</B> color space),
     * and sets the color to use for filling paths.</P>
     * Following the PDF manual, each operand must be a number between 0 (no ink) and
     * 1 (maximum ink).
     * @param	cyan	the intensity of cyan. A value between 0 and 1
     * @param	magenta	the intensity of magenta. A value between 0 and 1
     * @param	yellow	the intensity of yellow. A value between 0 and 1
     * @param	black	the intensity of black. A value between 0 and 1
     */


    public int docSetCMYKColorFillF(float cyan, float magenta, float yellow, float black) {
        myGraphics.setCMYKColorFillF(cyan, magenta, yellow, black );
        return 0;
    }


    /**
     * Changes the current color for stroking paths (device dependent colors!).
     * Sets the color space to <B>DeviceCMYK</B> (or the <B>DefaultCMYK</B> color space),
     * and sets the color to use for stroking paths.</P>
     * Following the PDF manual, each operand must be a number between 0 (miniumum intensity) and
     * 1 (maximum intensity).
     * @param	cyan	the intensity of cyan. A value between 0 and 1
     * @param	magenta	the intensity of magenta. A value between 0 and 1
     * @param	yellow	the intensity of yellow. A value between 0 and 1
     * @param	black	the intensity of black. A value between 0 and 1
     */

    public int docSetCMYKColorStrokeF(float cyan, float magenta, float yellow, float black) {
        myGraphics.setCMYKColorStrokeF(cyan, magenta, yellow, black );
        return 0;
    }

// Acro-Form 13th July, 2002

    public int addTextField(String name, String value, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}

            acroForm.addSingleLineTextField(name, value, f, fSize, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addPasswordField(String name, String value, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}

            acroForm.addSingleLinePasswordField(name, value, f, fSize, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addTextAreaField(String name, String value, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}

            acroForm.addMultiLineTextField(name, value, f, fSize, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addCheckBoxField(String name, String value, boolean bChecked, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
            acroForm.addCheckBox(name, value, bChecked, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addComboBoxField(String name, String[] options, String defaultValue, boolean editable, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}
            acroForm.addComboBox(name, options, defaultValue, editable, f, fSize,  llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addComboBoxField(String name, String[][] options, String defaultValue, boolean editable, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}
            acroForm.addComboBox(name, options, defaultValue, editable, f, fSize,  llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addSelectListField(String name, String[] options, String defaultValue, boolean editable, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}
            acroForm.addSelectList(name, options, defaultValue, f, fSize,  llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addSelectListField(String name, String[][] options, String defaultValue, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}
            acroForm.addSelectList(name, options, defaultValue, f, fSize, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }


    public int addHidenField(String name, String value) {
		if(acroForm == null) return 1;
		try
		{
            acroForm.addHiddenField(name, value);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

    public int addSubmitButton(String name, String caption, String value, String url, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}

            acroForm.addHtmlPostButton(name, caption, value, url, f, fSize, llx, lly, urx, ury);
            //acroForm.addResetButton("reset", "RESET", null, helv, fontSize, 430, 700, 495, 725);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }


    public int addRstButton(String name, String caption, String value, Font fName, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			BaseFont f = null;
			float fSize = 0;

			if(fName != null)
			{
				f = fName.getBaseFont();
				fSize = fName.size();
			}
			else
			{
				f = mygFont.getBaseFont();
				fSize = mygFont.size();
			}

            acroForm.addResetButton(name, caption, value, f, fSize, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }


    public int addRadButton(String group, String value, float llx, float lly, float urx, float ury) {
		if(acroForm == null) return 1;
		try
		{
			PdfFormField radiogroup = (PdfFormField)htRg.get(group);
			if(radiogroup == null)
			{
				radiogroup = acroForm.getRadioGroup(group, value, true);
				htRg.put(group, radiogroup);
			}

            acroForm.addRadioButton(radiogroup, value, llx, lly, urx, ury);
			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }


    public int addRadGroup(String group)	{
		if(acroForm == null) return 1;
		try
		{
			PdfFormField radiogroup = (PdfFormField)htRg.get(group);
			if(radiogroup == null)
			{
				//do nothing
				return 0;
			}

            acroForm.addRadioGroup(radiogroup);

			return 0;
		}
		catch(Exception e)	{
			e.printStackTrace();
			return 1;
		}
    }

// End Acro-Form

// ADD IMAGE object july 2003
	public int docAddImage(PDFImage pImage, String sAlign, boolean bWrap, float fLeft, float fTop, float fWidth, float fHeight)
	{
		try{
			if(pImage!=null)
			{
				Image iImage= pImage.getImage();
				int iAlign = Image.DEFAULT;
				if(sAlign.equalsIgnoreCase("left"))
					iAlign = Image.LEFT;
				else if(sAlign.equalsIgnoreCase("right"))
					iAlign = Image.RIGHT;
				if(sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("middle"))
					iAlign = Image.MIDDLE;

				if(bWrap)
					iImage.setAlignment(iAlign | Image.TEXTWRAP);
				else
					iImage.setAlignment(iAlign);

				if(fLeft>=0 && fTop>=0)
				{
					fTop = getConventionalTop(fTop);
					iImage.setAbsolutePosition(fLeft,fTop);
				}

				if(fWidth>0)
					iImage.scaleAbsoluteWidth(fWidth);

				if(fHeight>0)
					iImage.scaleAbsoluteHeight(fHeight);

				myDocument.add(iImage);
			}
			else
				return 1;
		}catch(DocumentException de){
			de.printStackTrace();
			return 1;
		}
		return 0;
	}

// ADD BarCode object july 2003
	public int docAddBarCode(PDFBarCode pbc, String sAlign, boolean bWrap, float fLeft, float fTop, float fWidth, float fHeight)
	{
		try{
			if(pbc!=null)
			{
				Barcode bc = pbc.getBarcode();
				Image iImage = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());

				int iAlign = Image.DEFAULT;
				if(sAlign.equalsIgnoreCase("left"))
					iAlign = Image.LEFT;
				else if(sAlign.equalsIgnoreCase("right"))
					iAlign = Image.RIGHT;
				if(sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("middle"))
					iAlign = Image.MIDDLE;

				if(bWrap)
					iImage.setAlignment(iAlign | Image.TEXTWRAP);
				else
					iImage.setAlignment(iAlign);

				if(fLeft>=0 && fTop>=0)
				{
					fTop = getConventionalTop(fTop);
					iImage.setAbsolutePosition(fLeft,fTop);
				}
				if(fWidth>0)
					iImage.scaleAbsoluteWidth(fWidth);
				if(fHeight>0)
					iImage.scaleAbsoluteHeight(fHeight);

				myDocument.add(iImage);
			}
			else
				return 1;
		}catch(DocumentException de){
			de.printStackTrace();
			return 1;
		}
		return 0;
	}

//THIS METHOD HANDLED HERE BECAUSE BARCODE NEEDS PdfContentByte to be inserted actually
//ALSO, A TABLE CAN ONLY BE ADDED TO A DOC and IT CAN'T BE ADDED AS A LISTITEM

// ADD LIST object july 2003
	 /**
	 * adds a PDFList to the doc
	 * @param pdfList list to be added
	 */
	 public int docAddList(PDFList pdfList)
	 {
		try{
			 if(pdfList!=null)
			 {
			 	com.lowagie.text.List iList = pdfList.getNestedList(mygFont,"#" + getDocColor(), cb);
				if(iList!=null)
					myDocument.add(iList);
			}
		}catch(DocumentException de){
			de.printStackTrace();
			return 1;
		}
		return 0;
	 }

// Added String object aug 03
	 /**
	 * adds a string to the doc
	 * @param sContent to be added
	 */
	 public int docAddString(String sContent)
	 {
		if(sContent!=null && !sContent.equals(""))
		{
			try{
				Chunk chunk = new Chunk(sContent);
				myDocument.add(new Phrase(chunk));
			}catch(DocumentException de){
				de.printStackTrace();
				return 1;
			}
		}

		return 0;
	 }

// ADD TEXT object Aug 2003
	 /**
	 * adds a PDFText to the doc
	 * @param pdfText text to be added
	 */
	 public int docAddText(PDFText pdfText)
	 {
		 return docAddText(pdfText, mygFont,"#" + getDocColor());
	 }


	protected int docAddText(PDFText pText, Font pFont, String pColor)
	{
		try{
			if(pText!=null)
			{
				Font textFont = pText.getFont();

				// set the inherited font if text's font is not defined
				if(textFont==null)
				{
					if(pFont==null)
						textFont = new Font();
					else
						textFont = new Font(pFont.getBaseFont(), pFont.size(), pFont.style(), pFont.color());
				}

				String textColor = pText.getColor();

				if(textColor==null || textColor.equals(""))
				{
					/* before taking doc color, take font's color */
					Color fontColor = textFont.color();
					if(fontColor==null)
						textFont.setColor(Color.decode(pColor));
				}
				else
					textFont.setColor(Color.decode(textColor));

				//set style ie. underline strikethru
				if(pText.myUnderLine)
					textFont.setStyle("underline");
				if(pText.myStrikeThru)
					textFont.setStyle("line-through");

				ArrayList textList = pText.getText();
				/* iterate to get the objects in the list and write them to the document */
				Iterator itr = textList.iterator();
				while(itr.hasNext())
				{
					Object o = itr.next();

					if(o instanceof Chunk)
					{
						Chunk c = (Chunk)o;
						c.setFont(textFont);
						myDocument.add(new Phrase(c));
					}
					else if (o instanceof Phrase)
					{
						Phrase p = (Phrase)o;
						ArrayList pChunks = p.getChunks();
						int pSize = 0;
						if(pChunks!=null)
							pSize = pChunks.size();
						for(int ctr=0; ctr<pSize; ctr++)
						{
							Chunk ck = (Chunk)pChunks.get(ctr);
							ck.setFont(textFont);
						}
						myDocument.add(p);
					}
					else if (o instanceof Paragraph)
					{
						Paragraph p = (Paragraph)o;
						ArrayList pChunks = p.getChunks();
						int pSize = 0;
						if(pChunks!=null)
							pSize = pChunks.size();
						for(int ctr=0; ctr<pSize; ctr++)
						{
							Chunk ck = (Chunk)pChunks.get(ctr);
							ck.setFont(textFont);
						}
						myDocument.add(p);
					}
					else if(o instanceof PDFBarCode)
					{
						PDFBarCode pbc = (PDFBarCode)o;
						Barcode bc = pbc.getBarcode();
						Image iImage = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());

						String sAlign = pbc.getAlignment();
						boolean wrap = pbc.getWrap();

						int iAlign = Image.DEFAULT;
						if(sAlign.equalsIgnoreCase("left"))
							iAlign = Image.LEFT;
						else if(sAlign.equalsIgnoreCase("right"))
							iAlign = Image.RIGHT;
						if(sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("middle"))
							iAlign = Image.MIDDLE;

						if(wrap)
							iImage.setAlignment(iAlign | Image.TEXTWRAP);
						else
							iImage.setAlignment(iAlign);

						myDocument.add(iImage);
					}
					else if(o instanceof PDFTable)
					{
						PDFTable pTable = (PDFTable)o;
						Table t = pTable.getNestedTable(cb);
						myDocument.add(t);
					}
					else if(o instanceof Image)
					{
						myDocument.add((Image)o);
					}
					else if(o instanceof PDFText)
					{
						docAddText((PDFText)o, textFont, textColor);
					}
				}
			}
			else
				return 1;
		}catch(DocumentException de){
			de.printStackTrace();
			return 1;
		}

		return 0;
	}

	/**
	* adds a PDFTable to the doc
	*/
	public int docAddTable(PDFTable pdfTable)
	{
		try{
			if(pdfTable!=null)
			{
				Table mainTable = pdfTable.getNestedTable(cb);
				myDocument.add(mainTable);
			}
			else
				return 1;

		}catch(DocumentException de){
			de.printStackTrace();
			return 1;
		}

		return 0;
	}

	/**
	* inserts a page from another document
	*/
	public int docInsertPage(PDFRead pRead, int pageNo, boolean newPage, float top, float left, float widthRatio, float heightRatio, boolean proportional, float rotate, boolean flipHor, boolean flipVer)
	{
		try{

			/* width = ratio of template's width to the doc's page width
			   height = ratio of template's height to the doc's page height */
			if(pRead!=null)
			{
				int n = pRead.getCount();
				if(pageNo>n)
				{
					System.out.println("Page No exceeds the count!");
					return 1;
				}

				PdfReader reader = pRead.getRead();

				if(widthRatio<=0)
					widthRatio = 1;
				if(heightRatio<=0)
					heightRatio = 1;

				if(top<0)
					top = 0;
				if(left<0)
					left = 0;

				/* get the conventional top value */
				//top = getConventionalTop(top);

				//if true  a new page is concatenated to the existing document
				if(newPage)
				{
					myDocument.newPage();
				}
				
				//if false then the page is drawn over the existing page.
				PdfImportedPage page = pdfwriter.getImportedPage(reader, pageNo);
				//cb.addTemplate(page, widthRatio, 0, 0, heightRatio, left, top);
				
				pdfwriter.getDirectContentUnder().addTemplate(page, widthRatio, 0, 0, heightRatio, left, top);
		
			}
		}catch(Exception e){
			e.printStackTrace();
			return 1;
		}

		return 0;
	}

	/**
	* returns the top position from the upper left
	*/
	public float getConventionalTop(float topFromBottom)
	{
		if(topFromBottom>=0 )
		{
			/* get the page height */
			int[] size = getDocSize();

			if(size!=null)
				return size[1] - topFromBottom;
			else
				return topFromBottom;
		}
		return topFromBottom;
	}

	/**
	* Added Nov 2003
	* @returns byte array
	*/
	public byte[] getBytes()
	{
		byte[] bytes = null;
		if(bos!=null)
			bytes = bos.toByteArray();

		return bytes;
	}
}
