import com.omnipilot.lassopro.*;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.PdfWriter;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.io.*;
import java.awt.Color;


/**
 * This class holds all the implementations of the member tags and the Accessor functions of the
 * PDF_Font tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFFont
{

	private Font myFont = null;
	private BaseFont myBF = null;

	/**
	 * This is the constructor of the class.
	 * The random key is initialized in the constructor.
	 */

	public PDFFont()
	{
	}

	/**
	 * This method returns the current font.
	 * @return      the Font.
	 */

	public Font getFont()
	{
		return this.myFont;
	}


	/**
	 * This method creates a font
	 * @param face - the font face name.
	 * @param filePath - the name of the file holding the font face.
	 * @param fontSize - the size of the font.
	 * @param fColor - the font color.
	 * @param encoding - the font encoding value.
	 * @param embed - whether the font is to be embedded or not.
	 */

	public void createFont(String face,String filePath,int fontSize,String fColor,String encoding,boolean embed)  throws Exception
	{
		boolean embedVal = false;
		if(embed)
			embedVal = BaseFont.EMBEDDED;
		else
			embedVal = BaseFont.NOT_EMBEDDED;
		String loc= null;
		if(face != null) loc = face;
		else loc = filePath;
		if(encoding == null) encoding = "";

		if(loc == null)
			loc = "Helvetica";

		String familyName = null;

		if(loc.equalsIgnoreCase("Courier"))
		{
			loc = "Courier";
			familyName = "Courier";
		}
		else if(loc.equalsIgnoreCase("Courier-Bold"))
		{
			loc = "Courier-Bold";
			familyName = "Courier";
		}
		else if(loc.equalsIgnoreCase("Courier-Oblique"))
		{
			loc = "Courier-Oblique";
			familyName = "Courier";
		}
		else if(loc.equalsIgnoreCase("Courier-BoldOblique"))
		{
			loc = "Courier-BoldOblique";
			familyName = "Courier";
		}
		else if(loc.equalsIgnoreCase("Helvetica"))
		{
			loc = "Helvetica";
			familyName = "Helvetica";
		}
		else if(loc.equalsIgnoreCase("Helvetica-Bold"))
		{
			loc = "Helvetica-Bold";
			familyName = "Helvetica";
		}
		else if(loc.equalsIgnoreCase("Helvetica-Oblique"))
		{
			loc = "Helvetica-Oblique";
			familyName = "Helvetica";
		}
		else if(loc.equalsIgnoreCase("Helvetica-BoldOblique"))
		{
			loc = "Helvetica-BoldOblique";
			familyName = "Helvetica";
		}
		else if(loc.equalsIgnoreCase("Symbol"))
		{
			loc = "Symbol";
			familyName = "Symbol";
		}
		else if(loc.equalsIgnoreCase("Times-Roman"))
		{
			loc = "Times-Roman";
			familyName = "Times New Roman";
		}
		else if(loc.equalsIgnoreCase("Times-Bold"))
		{
			loc = "Times-Bold";
			familyName = "Times New Roman";
		}
		else if(loc.equalsIgnoreCase("Times-Italic"))
		{
			loc = "Times-Italic";
			familyName = "Times New Roman";
		}
		else if(loc.equalsIgnoreCase("Times-BoldItalic"))
		{
			loc = "Times-BoldItalic";
			familyName = "Times New Roman";
		}
		else if(loc.equalsIgnoreCase("ZapfDingbats"))
		{
			loc = "ZapfDingbats";
			familyName = "ZapfDingbats";
		}


		myBF = BaseFont.createFont(loc, encoding, embedVal);

		if(myBF != null)
		{
			myFont = new Font(myBF, 12); //, Font.NORMAL);
			if(familyName != null) myFont.setFamily(familyName);
			if(fColor != null)
				this.setFontColor(fColor);
			if(fontSize > 0)
				this.setFontSize(fontSize);
			//this.setFontFace(face);
		}
	}

	public void resetFont(BaseFont bf, Color fColor, float fontSize)
	{
		try
		{
			if(bf == null) return;
			myBF = bf;
			myFont = new Font(myBF);
			if(fColor != null)
				myFont.setColor(fColor);
			if(fontSize > 0)
				this.setFontSize((int)fontSize);

		}
		catch(NumberFormatException e)
		{
			e.printStackTrace();
		}

	}

	/**
	 * Sets the  font size.
	 * @param fSize - the size of the font to be set.
	 */

	public int setFontSize(int fSize)
	{
		myFont.setSize(fSize);
		return 0;
	}


	public boolean isTrueType()
	{
		if(BaseFont.FONT_TYPE_TT == myBF.getFontType())
			return true;
		return false;
	}



	/**
	 * Sets the  font color.
	 * @param fColor - the color of the font to be set.
	 */

	public int setFontColor(String  fColor)
	{
	try
	{
		myFont.setColor(new Color(255, 255, 255).decode(fColor));
		return 0;
	}
	catch(NumberFormatException e)
	{
		e.printStackTrace();
		return 1;
	}

	}


	/**
	 * Sets the  font style to bold.
	 * @param fStyle - true\false value.
	 */

	public int setFontBold(boolean fStyle)
	{
		boolean b = myFont.isBold();
		boolean i = myFont.isItalic();
		boolean u = myFont.isUnderlined();
		myFont.setStyle("normal");
		if(fStyle) myFont.setStyle("bold");
		if(i) myFont.setStyle("italic");
		if(u) myFont.setStyle("underline");

		return 0;
	}

	/**
	 * Sets the  font style to Normal.
	 */

	public void setFontNormal()
	{
		myFont.setStyle("normal");
	}

	/**
	 * Sets the  font style to italic.
	 * @param fStyle - true\false value.
	 */

	public int setFontItalic(boolean fStyle)
	{

		boolean b = myFont.isBold();
		boolean i = myFont.isItalic();
		boolean u = myFont.isUnderlined();
		myFont.setStyle("normal");
		if(b) myFont.setStyle("bold");
		if(fStyle) myFont.setStyle("italic");
		if(u) myFont.setStyle("underline");

		return 0;
	}

	/**
	 * Sets the  font style to UnderLine.
	 * @param fStyle - true\false value.
	 */

	public int setFontUnderLine(boolean fStyle)
	{
		boolean b = myFont.isBold();
		boolean i = myFont.isItalic();
		boolean u = myFont.isUnderlined();
		myFont.setStyle("normal");
		if(b) myFont.setStyle("bold");
		if(i) myFont.setStyle("italic");
		if(fStyle) myFont.setStyle("underline");

		return 0;
	}

	/**
	 * Sets the  font Encoding value.
	 * @param fEncoding - the Encoding value.
	 */

	public int setFontEncoding(String fEncoding)
	{
	//	myBF.normalizeEncoding(fEncoding);
		return 0;
	}

	/**
	 * Sets the  font face to the given value.
	 * @param fName - the font face value.
	 */

	public int setFontFace(String fontName)
	{

		myFont.setFamily(fontName);
		return 0;

	}

	/**
	 * Returns the  font encoding value.
	 * @return - the font encoding value.
	 */

	public String getFontEncoding()
	{

		return myBF.getEncoding();

	}

	/**
	 * Returns the  font style.
	 * @return - the font style.
	 */

	public int getFontStyle()
	{

		return myFont.style();

	}

	/**
	 * Returns the  font face value.
	 * @return - the font fontface value as string.
	 */

	public String getFontFace()
	{

		return myFont.getFamilyname();

	}

	/**
	 * Converts the color to a HexString
	 * @return - the String containing the font color.
	 */

	public String toHexString( Color c )
	{
		String red = Integer.toHexString(c.getRed());
		String green = Integer.toHexString(c.getGreen());
		String blue = Integer.toHexString(c.getBlue());

		if (red.length() != 2 )
			red = "0"+red;
		if (green.length() != 2 )
			green = "0"+green;
		if (blue.length() != 2 )
			blue = "0"+blue;

		return new String( red+green+blue );
	}


	/**
	 * Returns the  font color value.
	 * @return - the font color as a String value.
	 */

	public String getFontColor()
	{
		Color clr=null ;
		if(myFont != null)
		clr =   myFont.color();  // new Color(200,200,200);
		String retStr = null;
		if(clr != null)
		{
			retStr = this.toHexString(clr);

		}
		return retStr;
	}

	/**
	 * Returns the  font Size value.
	 * @return - the font size value.
	 */

	public int getFontSize()
	{
		return (int)myFont.size();
	}

	/**
	 * Checks if the font style is bold or not.
	 * @return - returns true\false depending on whether the font is bold or not.
	 */

	public boolean getBold()
	{
		return myFont.isBold();
	}

	/**
	 * Checks if the font style is italic or not.
	 * @return - returns true\false depending on whether the font is italic or not.
	 */

	public boolean getItalic()
	{
		return myFont.isItalic();
	}

	/**
	 * Checks if the font style is UnderLine or not.
	 * @return - returns true\false depending on whether the font is UnderLine or not.
	 */

	public boolean getUnderScore()
	{
		return myFont.isUnderlined();
	}

	/**
	 * Returns the font name.
	 * @return - returns a String holding the font name.
	 */

	public String getPSFontName()
	{
		return myBF.getPostscriptFontName();
	}

	/**
	 * Returns the font supported encoding values.
	 * @return - returns a String [] holding the font supported encoding values.
	 */

	public String[] getSupportedEncodings()
	{
		return myBF.getCodePagesSupported();
	}

	/**
	 * Returns the font full name.
	 * @return - returns a String [][] holding the fontfullname  value.
	 */

	public String[][] getFontFullName()
	{
		return myBF.getFullFontName();
	}

	//Added Aug 03
	/**
	 * returns the width of the text with the given font
	 * @param sText - the text for which size is to be got.
	 * @return a float value indicating the width of the text with the given Font
	 */
	public float getTextWidth(String sText)
	{
		return myBF.getWidthPoint(sText,myFont.size());
	}

}
