import com.omnipilot.lassopro.*;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.PdfWriter;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.io.*;
import java.awt.Color;

/**
 * This class holds all the implementations of the member tags and the Accessor functions of the
 * PDF_Graphic tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */
 
public class PDFGraphics  
{

	public Graphic myGraphics;	
	
	/**
	 * Constructor of the class.
	 * The long key value is generated here.
	 */
	 
	public PDFGraphics()
	{
		myGraphics 	= new Graphic();
	}
	
	
	/**
	 * This method draws a straight line on the document.
	 * @param x1 - the  x coordinate of one corner.
	 * @param y1 - the  y coordinate of one corner.
	 * @param x2 - the  x2 coordinate of other corner.
	 * @param y2 - the  y2 coordinate of other corner.
	 * @param lWidth - the line width value.
	 * @param color - the color value for the drawing operation.
	 */
	 
	public int grDrawLine(int x1,int y1,int x2,int y2,int lWidth, String color)
	{
			myGraphics.moveTo(x1,y1);
			myGraphics.setLineWidth(lWidth);
			myGraphics.lineTo(x2,y2);
			myGraphics.setColorStroke(new Color(255, 255, 255).decode(color));
			myGraphics.stroke();
			return 0;
		
	}
	
	/**
	 * This method draws a rectangle on the document.
	 * @param x - the  x coordinate of one corner.
	 * @param y - the  y coordinate of one corner.
	 * @param w - the  width value of the rectangle.
	 * @param h - the  height value of the rectangle.
	 * @param color - the color value for the drawing operation.
	 */
	 
	public int grDrawRectangle(int x, int y, int w, int h, String color)
	{
		myGraphics.rectangle(x, y, w, h);
		myGraphics.setColorStroke(new Color(255, 255, 255).decode(color));
		myGraphics.stroke();
		return 0;
	}
	
	/**
	 * This method draws a ellipse on the document.
	 * @param x1 - the x coordinate of one corner.
	 * @param y1 - the y coordinate of one corner.
	 * @param x2 - the x cordinate of the other corner.
	 * @param y2 - the y cordinate of the other end.
	 */
	 
	public int grDrawEllipse(int x1, int y1, int x2, int y2) 
	{
		myGraphics.ellipse(x1, y1, x2, y2);
		myGraphics.stroke();
		return 0;
	}
	
	/**
     * Draws a partial ellipse inscribed within the rectangle x1,y1,x2,y2,
     * starting at startAng degrees and covering extent degrees. Angles
     * start with 0 to the right (+x) and increase counter-clockwise.
     *
     * @param x1 - a corner of the enclosing rectangle
     * @param y1 - a corner of the enclosing rectangle
     * @param x2 - a corner of the enclosing rectangle
     * @param y2 - a corner of the enclosing rectangle
     * @param startAng - starting angle in degrees
     * @param extent - angle extent in degrees
     */
     
	public int grDrawArc(int x1, int y1, int x2, int y2, int startAng, int extent)
	{
		myGraphics.arc(x1, y1, x2, y2, startAng, extent);
		myGraphics.stroke();
		return 0;
	
	}	
	
	/**
     * Ends the path without filling or stroking it.
     */
     
	public int grDrawNewPath()
	{
		myGraphics.newPath();
		return 0;
	}
	
	
	/**
     * Closes the current subpath by appending a straight line segment from the current point
     * to the starting point of the subpath.
     */
     
	public int grClosePath()
	{
		myGraphics.closePath();
		myGraphics.stroke();
		return 0;
	}
	
	 /**
     * Appends a Bzier curve to the path, starting from the current point.
     * @param		x1 - x-coordinate of the first control point	
     * @param		y1 - y-coordinate of the first control point
     * @param		x2 - x-coordinate of the second control point
     * @param		y2 - y-coordinate of the second control point
     * @param		x3 - x-coordinaat of the ending point (= new current point)
     * @param		y3 - y-coordinaat of the ending point (= new current point)
     */
     
	public int grDrawCurveTo(int x1, int y1, int x2, int y2, int x3, int y3) 
	{
	
		myGraphics.moveTo(x1,y1);
		myGraphics.curveTo(x1, y1, x2, y2, x3, y3);
		myGraphics.stroke();
		return 0;
	
	}
	
	/**
     * Appends a Bzier curve to the path, starting from the current point.
     *
     * @param		x2 - x-coordinate of the second control point
     * @param		y2 - y-coordinate of the second control point
     * @param		x3 - x-coordinaat of the ending point (= new current point)
     * @param		y3 - y-coordinaat of the ending point (= new current point)
     */
     
	public int grDrawCurveFromTo(int x2, int y2, int x3, int y3)
	{
	
		myGraphics.curveTo(x2,y2,x3,y3);
		myGraphics.stroke();
		return 0;
	
	}
	
	/**
     * Appends a straight line segment from the current point <I>(x, y)</I>. The new current
     * point is <I>(x, y)</I>.
     *
     * @param		x - new x-coordinate
     * @param		y - new y-coordinate
     */
		
	public int grDrawLineTo(int x, int y)
	{
	
		myGraphics.lineTo(x, y);
		myGraphics.stroke();
		return 0;
		
	}
	
	/**
     * Move the current point <I>(x, y)</I>, omitting any connecting line segment.
     *
     * @param		x - new x-coordinate
     * @param		y - new y-coordinate
     */
     
	public int grDrawMoveTo(int x, int y)
	{
	
		myGraphics.moveTo(x, y);
		return 0;
	
	}
	
}
