import java.util.ArrayList;
import java.awt.Color;
import com.omnipilot.lassopro.*;
import com.lowagie.text.*;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.Barcode;

/**
 * This class holds all the implementations of the member tags of the
 * PDF_List tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFList
{

	private ArrayList myList;
	private String myFormat;
	private Chunk myListSymbol;
	private float myIndent;
	private Font myFont;
	private String myAlign;
	private String myColor;
	private String myBGColor;
	private float myLeading;
	private float myIndentLeft;
	private float myIndentRight;

	/**
	 * This is the constructor of the class.
	 * The random number for the key is generated here.
	 */
	public PDFList()
	{
	}


	/**
	 * This method returns the current format.
	 * @return      the String.
	 */

	public String getFormat()
	{
		return this.myFormat;
	}

	/**
	 * This method returns the current list symbol.
	 * @return      the Chunk.
	 */
	public Chunk getListSymbol()
	{
		return this.myListSymbol;
	}


	/**
	 * This method returns the current indent.
	 * @return      the float.
	 */
	public float getIndent()
	{
		return this.myIndent;
	}

	/**
	 * This method returns the current font.
	 * @return      the Font.
	 */

	public Font getFont()
	{
		return this.myFont;
	}

	/**
	 * This method returns the current alignment
	 * @return      the String.
	 */

	public String getAlign()
	{
		return this.myAlign;
	}

	/**
	 * This method returns the current color
	 * @return      the String.
	 */

	public String getColor()
	{
		return this.myColor;
	}

	/**
	 * This method returns the current BGColor
	 * @return      the String.
	 */

	public String getBGColor()
	{
		return this.myBGColor;
	}

	/**
	 * This method returns the current leading.
	 * @return      the float.
	 */
	public float getLeading()
	{
		return this.myLeading;
	}

	/**
	 * This method returns the current left indent.
	 * @return      the float.
	 */
	public float getIndentLeft()
	{
		return this.myIndentLeft;
	}

	/**
	 * This method returns the current right indent.
	 * @return      the float.
	 */
	public float getIndentRight()
	{
		return this.myIndentRight;
	}

	/**
	* sets the font
	* @param pFont the Font
	*/
	public int setFont(Font pFont)
	{
		if(pFont!=null)
			myFont = pFont;
		return 0;
	}

	/**
	* sets the color
	* @param pColor the String
	*/
	public int setColor(String color)
	{
		if(color!=null && !color.equals(""))
		{
			myColor = color;
			if(myColor.indexOf("#")==-1)
				myColor = "#" + color;
		}
		return 0;
	}

	/**
	 * This method creates a list
	 */
	 public int CreateList(String sFormat,String sBullet,float fIndent,Font pFont,
	 						String sAlign, String sColor, String sBGColor, float fLeading,
	 						float fIndentLeft,float fIndentRight)
	 {
		myFormat = sFormat;

		if(sFormat.equalsIgnoreCase("bullet"))
		{
			if(sBullet!=null && !sBullet.equals(""))
				myListSymbol = new Chunk(sBullet);
			else
				myListSymbol = new Chunk("\u2022");
		}

		myIndent = fIndent;
		myFont = pFont;
		myAlign = sAlign;
		myColor = sColor;
		myBGColor = sBGColor;
		myLeading = fLeading;
		myIndentLeft = fIndentLeft;
		myIndentRight = fIndentRight;
		myList = new ArrayList();

		return 0;
	 }

	 /**
	 * adds a string to the list
	 * @param sContent to be added
	 */
	 public int listAddString(String sContent)
	 {
		if(myFont==null)
			myFont = new Font();
		if(myColor!=null && !myColor.equals(""))
			myFont.setColor(java.awt.Color.decode(myColor));
		Chunk chunk = new Chunk(sContent,myFont);
		if(myBGColor!=null && !myBGColor.equals(""))
			chunk = chunk.setBackground(java.awt.Color.decode(myBGColor));
		ListItem item = null;
		if(myLeading!=0)
			item = new ListItem(myLeading, chunk);
		else
			item = new ListItem(chunk);
		item.setAlignment(myAlign);
		myList.add(item);
		return 0;
	 }

	 /**
	 * adds an Image to the list
	 * @param pImage to be added
	 * @param sAlign alignment for the image
	 * @param bWrap wrap paramter for the image
	 */
	 public int listAddImage(PDFImage pImage, String sAlign, boolean bWrap)
	 {
		Image iImage = pImage.getImage();
		int iAlign = Image.DEFAULT;

		if(sAlign==null || sAlign.equals(""))
			sAlign = myAlign;

		if(sAlign==null)
			sAlign = "";
		if(sAlign.equalsIgnoreCase("left"))
			iAlign = Image.LEFT;
		else if(sAlign.equalsIgnoreCase("right"))
			iAlign = Image.RIGHT;
		else if(sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("centre") || sAlign.equalsIgnoreCase("middle"))
			iAlign = Image.MIDDLE;

		if(bWrap)
			iImage.setAlignment(iAlign | Image.TEXTWRAP);
		else
			iImage.setAlignment(iAlign);

		Chunk chunk = new Chunk(iImage,0,0);
		if(myBGColor!=null && !myBGColor.equals(""))
			chunk = chunk.setBackground(java.awt.Color.decode(myBGColor));
		ListItem item = null;
		if(myLeading!=0)
			item = new ListItem(myLeading, chunk);
		else
			item = new ListItem(chunk);

		setAlignment(item);
		myList.add(item);
		return 0;
	 }

	/**
	* sets the alignment of the ListItem in the List
	*/
	private void setAlignment(ListItem item)
	{
		if(myAlign!=null && !myAlign.equals(""))
		{
			if(myAlign.equalsIgnoreCase("left"))
				item.setAlignment(0);
			else if(myAlign.equalsIgnoreCase("center"))
				item.setAlignment(1);
			else if(myAlign.equalsIgnoreCase("right"))
				item.setAlignment(2);
		}
		else
			item.setAlignment(0);
	}

	 /**
	 * adds a PDFTable to the list
	 * @param pTable to be added
	 */
	 public int listAddTable(PDFTable pTable)
	 {
		String align = "left";

		if(myAlign==null || myAlign.equals(""))
			align = myAlign;

		pTable.getTable().setAlignment(align);
		myList.add(pTable);
		return 0;
	 }

	 /**
	 * adds a PDFBarCode to the list
	 * @param pBarCode to be added
	 */
	 public int listAddBarCode(PDFBarCode pBarCode)
	 {
		myList.add(pBarCode);
		return 0;
	 }

	 /**
	 * adds a PDFText to the list
	 * @param pText to be added
	 */
	 public int listAddText(PDFText pText)
	 {
		myList.add(pText);
		return 0;
	 }

	 /**
	 * adds a pdfList to the list
	 * @param pList to be added
	 */
	 public int listAddList(PDFList pList)
	 {
		myList.add(pList);
		return 0;
	 }

	/**
	 * This method returns the List.
	 * @return      the List.
	 */

	public ArrayList getList()
	{
		return this.myList;
	}

	/**
	* returns a com.lowagie.text.List object
	* @return com.lowagie.text.List object
	* @param pdfList PDFLIst object
	* @param pFont font of the immediate parent
	* @param pColor color of the immediate parent
	* @param cb PdfContentByte object needed for barcode
	*/
	public List getNestedList(Font pFont, String pColor, PdfContentByte cb)
	{
		return getNestedList(this, pFont, pColor, cb);
	}

	/**
	* returns a com.lowagie.text.List object
	* @return com.lowagie.text.List object
	* @param pdfList PDFLIst object
	* @param pFont font of the immediate parent
	* @param pColor color of the immediate parent
	* @param cb PdfContentByte object needed for barcode
	*/
	protected List getNestedList(PDFList pdfList, Font pFont, String pColor, PdfContentByte cb)
	{
		if(pdfList!=null)
		{
			Font listFont = pdfList.getFont();
			// set the inherited font if list's font is not defined
			if(listFont==null)
			{
				if(pFont==null)
					listFont = new Font();
				else
					listFont = new Font(pFont.getBaseFont(), pFont.size(), pFont.style(), pFont.color());
			}

			String listColor = pdfList.getColor();
			if(listColor==null || listColor.equals(""))
				listColor = pColor;

			listFont.setColor(Color.decode(listColor));

			ArrayList pList = pdfList.getList();
			int pdfListSize = pList.size();
			if(pList==null || pdfListSize==0)
				return null;

			//iText List
			List iList = null;

			float f = pdfList.getIndent();
			String sFormat = pdfList.getFormat();


			if(sFormat.equalsIgnoreCase("Number"))
				iList = new List(true, (int)f );
			else if(sFormat.equalsIgnoreCase("Letter"))
				iList = new List(false, true, (int)f);
			else
			{
				/* set default as bullet */
				iList = new List(false, (int)f);
				iList.setListSymbol(pdfList.getListSymbol());
			}

			f = pdfList.getIndentLeft();
			if(f!=0)
				iList.setIndentationLeft(f);
			f = pdfList.getIndentRight();
			if(f!=0)
				iList.setIndentationRight(f);
			for(int i=0; i<pdfListSize; i++)
			{
				Object o = pList.get(i);
				if(o instanceof ListItem) // if ListItem
				{
					iList.add((ListItem)o);
					//System.out.println("in list: listitem");
				}
				else if(o instanceof PDFTable)	//if table
				{
					//HANDLE TABLE HERE PROPERLY
					//myDocument.add(((PDFTable)o).getTable());
				}
				else if(o instanceof PDFBarCode)
				{
					if(cb!=null)
					{
						PDFBarCode pbc = (PDFBarCode)o;
						Barcode bc = pbc.getBarcode();
						Image iImage = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());

						Chunk chunk = new Chunk(iImage,0,0);
						ListItem item = null;
						float fLeading = pdfList.getLeading();
						if(fLeading!=0)
							item = new ListItem(fLeading, chunk);
						else
							item = new ListItem(chunk);

						iList.add(item);
						//System.out.println("in list: barcode");
					}
				}
				else if(o instanceof PDFText) // if PDFText
				{
					//Make it a single text i.e chunk/phrase/paragraph
					PDFText pText = (PDFText)o;
					Phrase textPhrase = pText.getNestedText(cb);

					//get text type
					String textType = pText.getType();

					if(textType==null || textType.equals(""))
						textType = "phrase";

					Font textFont = pText.getFont();
					if(textFont==null)
						textFont = new Font(listFont.getBaseFont(), listFont.size(), listFont.style(), listFont.color());

					String textColor = pText.getColor();
					if(textColor==null || textColor.equals(""))
						textColor = listColor;

					//set style i.e. underline strikethru
					if(pText.myUnderLine)
						textFont.setStyle("underline");

					if(pText.myStrikeThru)
						textFont.setStyle("line-through");

					if(textColor!=null && !textColor.equals(""))
						textFont.setColor(Color.decode(textColor));

					ArrayList chunkList = textPhrase.getChunks();
					if(chunkList==null)
						chunkList = new ArrayList();

					for(int x=0; x<chunkList.size();x++)
					{
						Chunk chunk = (Chunk)chunkList.get(x);
						if(chunk!=null)
						{
							String bgColor = pText.getBGColor();
							if(bgColor!=null && !bgColor.equals(""))
								chunk = chunk.setBackground(Color.decode(bgColor));

							if(pText.myTextRise!=0)
								chunk.setTextRise(pText.myTextRise);

							String sAnchor = pText.myAnchor;
							String sGoTo = pText.myGoTo;
							String sFileName = pText.myFileName;
							String sName = pText.myName;
							if(sAnchor!= null && (sAnchor.trim().length() > 0))
								chunk.setAnchor(sAnchor);
							else if(sGoTo != null && (sFileName== null || sFileName.trim().length() <= 0))
								chunk.setLocalGoto(sGoTo);
							else if(sGoTo != null && (sFileName!= null && sFileName.trim().length() > 0))
								chunk.setRemoteGoto(sFileName,sGoTo);
							if(sName != null)
								chunk.setLocalDestination(sName);
						}
					}

					if(textType.equalsIgnoreCase("phrase") || textType.equalsIgnoreCase("paragraph"))
					{
						//adding a paragraph doesn't work
						if(pText.myLeading!=0)
							textPhrase.setLeading(pText.myLeading);
						iList.add(new ListItem(textPhrase));
					}
					else
						iList.add(new ListItem(textPhrase));
				}
				else if(o instanceof PDFList) // if PDFList
				{
					//System.out.println("in list: PDFList");
					/* get all the nested lists in a single list */
					List nestedList = getNestedList((PDFList)o, listFont, listColor, cb);
					if(nestedList!=null)
						iList.add(nestedList);
				}
			}
			return iList;
		}
		else
			return null;
	}

}
