import com.omnipilot.lassopro.*;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.PdfWriter;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.io.*;
import java.awt.Color;
import java.awt.Point;

/**
 * This class holds all the implementations of the member tags and the Accessor functions of the
 * PDF_Table tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFTable
{

	public Table myTable;
	private ArrayList myList;

	/**
	 * This is the constructor of the class.
	 * The random number for the key is generated here.
	 */
	public PDFTable()
	{
	}

	/**
	 * This method creates a table.
	 * @param cols - the number of table coloumns.
	 * @param rows - the number of table rows.
	 * @param spacing - the cell spacing values.
	 * @param padding  - the cell padding values.
	 * @param iWidth  - the width value as percentage of the available space.
	 * @param iBWidth  - the border width value. (Added Aug-2003)
	 * @param fWidths  - the column widths value. (Added Aug-2003)
	 * @param sBrColor  - the Border color value. (Added Aug-2003)
	 * @param sBgColor  - the Background color value. (Added Aug-2003)
	 * @param sRotate  - the rotate value. (Added Aug-2003)
	 */

	 public int CreateTable(int cols,int rows,int spacing,int padding, int iWidth, int iBWidth, float []fWidths, String sBrColor, String sBgColor, String sRotate)
	 {

	 try
	 {
	 	myTable = new Table(cols,rows);
	 	myTable.setPadding(padding);
		myTable.setSpacing(spacing);

		if (iWidth != -1)
			myTable.setWidth(iWidth); //setting the width when it is specified

		if (iBWidth != -1)
			myTable.setBorderWidth(iBWidth);

		if (fWidths != null)
			myTable.setWidths(fWidths); //Sets the widths of the different columns (percentages).

		if (sBrColor != null)
			setTableBorderColor(sBrColor); //Sets the borderColor

		if (sBgColor != null)
			setBackGroundColor(sBgColor); //Sets the backGround Color

		myTable.setAutoFillEmptyCells(true);

		//Initialise myList
		myList = new ArrayList();

	 	return 0;
	 }
	 catch(DocumentException de) {
		System.err.println(de.getMessage());
		return 1;
	 }

	 }


	/**
	 * Sets the table border width value.
	 * @param width - the table width.
	 */
	public int setTableBorderWidth(float width)
	{
		myTable.setBorderWidth(width);
		return 0;

	}

	/**
	 * Sets the table border color.
	 * @param color - the table border color.
	 */
	public int setTableBorderColor(String color)
	{
		myTable.setBorderColor(new Color(255, 255, 255).decode(color));
		return 0;
	}

	/**
	 * Added Aug 2003
	 * Sets the table border color.
	 * @param color - the table border color.
	 */
	public int setBackGroundColor(String color)
	{
		myTable.setBackgroundColor(new Color(255, 255, 255).decode(color));
		return 0;
	}


	/**
	 * Sets the table padding value.
	 * @param tbpNum - the table padding value.
	 */
	public int setTablePadding(float tbpNum)
	{

		myTable.setPadding(tbpNum);
		return 0;

	}

	/**
	 * Sets the table spacing value.
	 * @param tbsNum - the table spacing value.
	 */
	public int setTableSpacing(float tbsNum)
	{

		myTable.setSpacing(tbsNum);
		return 0;

	}

	/**
	 * Sets the table alignment.
	 * @param tbaNum - the table alignment value.
	 */
	public int setTableAlignment(int tbaNum)
	{

		myTable.setAlignment(tbaNum);
		return 0;

	}

	/**
	 * Sets the table alignment.
	 * @param tbaNum - the table alignment value.
	 */
/*	public int setColoumnWidth(float[] widths)
	{
		try {
			myTable.setWidths(widths);
			return 0;
		}
		catch (Exception	 e)	{
			e.printStackTrace();
			return 1;
		}

	}
*/
	/**
	 * Gets the borderwidth.
	 * @return	a value
	 */
	public float getTableBorderWidth()
	{

		return myTable.borderWidth();

	}

	/**
	 * This method returns the  coloumn width.
	 * @return - the array containing width.
	 */

/*	public  float[] getColoumnWidth()
	{
		float[] tempArr = new float[1];
		tempArr[0] = myTable.getWidths();
        return tempArr;

	}
*/
	/**
	 * Gets the color of the border.
	 * @return	a value
	 */
	public String getTableBorderColor()
	{

		String tblClr = null;
		Color tblColor = myTable.borderColor();
		if(tblColor != null)
			tblClr = this.toHexString(tblColor);
		return tblClr;
	}

	/**
	 * Gets the cellpadding.
	 * @return  a value
	 */
	public float getTablePadding()
	{

		return myTable.cellpadding();

	}

	/**
	 * Gets the cellspacing.
	 * @return  a value
	 */
	public int getTableSpacing()
	{

		return (int)myTable.cellspacing();

	}

	/**
	 * Gets the horizontal alignment.
	 * @return  a value
	 */
	public int getTableAlignment()
	{

		return myTable.alignment();

	}
	/**
	 * Gets the number of columns.
	 * @return    a value
	 */
	public int getTableColoumnCount()
	{

		return myTable.columns();

	}

	/**
	 * Gets the number of rows in this Table.
	 * @return      the number of rows in this Table
	 */
	public int getTableRowCount()
	{

		return myTable.size();

	}

	/**
	 * Gets the table width (in pixels).
	 * @return  the table width
	 */
	public String getTableAbsWidth()
	{

		return myTable.absWidth();

	}

	/**
	 * This method inserts an Object into a existing table.
	 * @param obj - image/barcode/text/table/string to be put into the cell.
	 * @param tblColumn - the table coloumn value.
	 * @param tbRow  - the table row value.
	 * @param tbColSpan - the table column span value.
	 * @param tbRowSpan - the table row span value.
	 * @param bHeader - true\false value to set the Header of the table.
	 * @param bWrap - true\false value to set the Wrap for the table cell.
	 * @param xAbsPos - the X absolute position.
	 * @param yAbsPos - the Y absolute position.
	 */

	public int insert(Object obj,String strCols, String strRows, String strColSpan, String strRowSpan, String strHeader, String strNoWrap, String strXAbsPos, String strYAbsPos,String strVAlign,String strHAlign,String strBColor,String strBWidth)
	{
		try
		{
			Cell cell = null;

			if(obj instanceof String)
			{
				cell = new Cell((String)obj);
				setCellProperties(cell, strHeader, strNoWrap, strColSpan, strRowSpan, strVAlign, strHAlign, strBColor, strBWidth);
			}
			else if(obj instanceof PDFImage)
			{
				PDFImage pImage = (PDFImage)obj;
				Phrase p = new Phrase(new Chunk(pImage.getImage(),0,0));
				cell = new Cell(p);
				setCellProperties(cell, strHeader, strNoWrap, strColSpan, strRowSpan, strVAlign, strHAlign, strBColor, strBWidth);
			}
			else if(obj instanceof PDFTable || obj instanceof PDFBarCode || obj instanceof PDFText)
			{
				//System.out.println("Inside addtable::" + obj.getClass().getName());

				HashMap hashCell = new HashMap();
				hashCell.put("colNo",strCols);
				hashCell.put("rowNo",strRows);
				hashCell.put("header",strHeader);
				hashCell.put("noWrap",strNoWrap);
				hashCell.put("colSpan",strColSpan);
				hashCell.put("rowSpan",strRowSpan);
				hashCell.put("vAlign",strVAlign);
				hashCell.put("hAlign",strHAlign);
				hashCell.put("bColor",strBColor);
				hashCell.put("bWidth",strBWidth);

				ObjectCellPair ocp = new ObjectCellPair(obj,hashCell);
				myList.add(ocp);
			}

			if(strCols != null && strRows != null)
			{
				int iCols = Integer.parseInt(strCols);
				int iRows = Integer.parseInt(strRows);
				if(cell!=null)
					myTable.addCell(cell,iRows,iCols);
				return 0;
			}

			if(cell!=null)
				myTable.addCell(cell);
			return 0;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return 1;
		}
	}

	/**
	* sets the properties of the cell
	*/
	public int setCellProperties(Cell cell, String strHeader, String strNoWrap, String strColSpan, String strRowSpan, String strVAlign, String strHAlign, String strBColor, String strBWidth)
	{
		if(cell!=null)
		{
			if(strHeader != null)
			{
				boolean bHeader 	= false;

				if(strHeader.equalsIgnoreCase("true"))
					bHeader = true;
				else
					bHeader = false;

				cell.setHeader(bHeader);
			}

			if(strNoWrap != null)
			{
				boolean bNoWrap 	= false;
				if(strNoWrap.equalsIgnoreCase("true"))
					bNoWrap = true;
				else
					bNoWrap = false;

				cell.setNoWrap(bNoWrap);
			}

			if(strColSpan != null)
			{
				int iColSpan = Integer.parseInt(strColSpan);
				cell.setColspan(iColSpan);
			}

			if(strRowSpan != null)
			{
				int iRowSpan = Integer.parseInt(strRowSpan);
				cell.setRowspan(iRowSpan);
			}

			//code added 12-09-2002
			if(strVAlign != null)
			{
				if(strVAlign.equalsIgnoreCase("top"))
					cell.setVerticalAlignment(4);
				else if(strVAlign.equalsIgnoreCase("center"))
					cell.setVerticalAlignment(5);
				else if(strVAlign.equalsIgnoreCase("bottom"))
					cell.setVerticalAlignment(6);
			}

			if(strHAlign != null)
			{
				if(strHAlign.equalsIgnoreCase("left"))
					cell.setVerticalAlignment(0);
				else if(strHAlign.equalsIgnoreCase("Center"))
					cell.setVerticalAlignment(1);
				else if(strHAlign.equalsIgnoreCase("right"))
					cell.setVerticalAlignment(2);
			}

			if(strBColor != null)
			{
				cell.setBorderColor(new Color(255, 255, 255).decode(strBColor));

			}

			if(strBWidth != null)
			{
				int iBorderWidth = Integer.parseInt(strBWidth);
				cell.setBorderWidth(iBorderWidth);

			}
			else
				cell.setBorderWidth(0);
		}
		return 0;
	}
	/**
	 * returns the element at the position row, column
	 *          (Cast to Cell or Table)
	 *
	 * @return  dimension
	 */
	public Object getTableElement(int tbRow,int tbColoumn)
	{
		return myTable.getElement(tbRow,tbColoumn);
	}

	/**
	 * removes the row or the coloumn.
	 */

	public int remove(int tbRow,int tbColoumn)
	{
	try
	{
		if(tbRow == -1)
		{
			myTable.deleteColumn(tbColoumn);
			return 0;
		}
		else if(tbColoumn == -1)
		{
			myTable.deleteRow(tbRow);
			return 0;
		}
	}
	catch(Exception e)
	{
		e.printStackTrace();
		return 1;
	}
	return 1;
	}

	/**
	 * This method converts the color of the font to HexString format.
	 * @return      the String containing the color.
	 */
	public String toHexString( Color c )
	{
		String red = Integer.toHexString(c.getRed());
		String green = Integer.toHexString(c.getGreen());
		String blue = Integer.toHexString(c.getBlue());

		if (red.length() != 2 )
			red = "0"+red;
		if (green.length() != 2 )
			green = "0"+green;
		if (blue.length() != 2 )
			blue = "0"+blue;

		return new String( red+green+blue );
	}


	/**
	 * This method returns the Table.
	 * @return      the Table.
	 */

	public Table getTable()
	{
		return this.myTable;
	}

	/**
	* @return List containing all the objecta that are to be added to a table
	*/
	public ArrayList tableContentList()
	{
		return myList;
	}

	/**
	* @return a Table
	* @param pdfTable PDFTable object
	* @param cb PdfContentByte object to make a barcode
	*/
	protected Table getNestedTable(PdfContentByte cb)
	{
		//System.out.println("inside nestedTable");
		if(this!=null)
		{
			try
			{
				//iText Table
				Table iTable = this.getTable();
				if(iTable==null)
					return null;

				ArrayList cList = this.tableContentList();
				if(cList==null)
					return iTable;

				int pdfTableSize = cList.size();
				//System.out.println("table list size : " + pdfTableSize);

				if(pdfTableSize==0)
					return iTable;

				for(int i=0; i<pdfTableSize; i++)
				{
					ObjectCellPair ocp = (ObjectCellPair)cList.get(i);
					if(ocp==null)
						return iTable;

					Object o = ocp.getObject();
					HashMap cellProps = ocp.getCell();

					if(o instanceof PDFBarCode)
					{
						//System.out.println("inside nestedTable : barcode ");
						if(cb!=null)
						{
							PDFBarCode pbc = (PDFBarCode)o;
							Barcode bc = pbc.getBarcode();
							Image iImage = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());

							Phrase p = new Phrase(new Chunk(iImage,0,0));
							Cell cell = new Cell(p);
							setCellProperties(cell,(String)cellProps.get("header"),
													(String)cellProps.get("noWrap"),
													(String)cellProps.get("colSpan"),
													(String)cellProps.get("rowSpan"),
													(String)cellProps.get("vAlign"),
													(String)cellProps.get("hAlign"),
													(String)cellProps.get("bColor"),
													(String)cellProps.get("bWidth"));

							String strCols 	= (String)cellProps.get("colNo");
							String strRows 	= (String)cellProps.get("rowNo");

							if(strCols != null && strRows != null)
							{
								int iCols = Integer.parseInt(strCols);
								int iRows = Integer.parseInt(strRows);
								iTable.addCell(cell,iRows,iCols);
							}
							else
								iTable.addCell(cell);
						}
					}
					else if(o instanceof PDFText) // if PDFText
					{
						//Make it a single text i.e chunk/phrase/paragraph
						PDFText pText = (PDFText)o;
						Phrase p = pText.getNestedText(cb);
						Cell cell = new Cell(p);
						setCellProperties(cell,(String)cellProps.get("header"),
												(String)cellProps.get("noWrap"),
												(String)cellProps.get("colSpan"),
												(String)cellProps.get("rowSpan"),
												(String)cellProps.get("vAlign"),
												(String)cellProps.get("hAlign"),
												(String)cellProps.get("bColor"),
												(String)cellProps.get("bWidth"));

						String strCols 	= (String)cellProps.get("colNo");
						String strRows 	= (String)cellProps.get("rowNo");

						if(strCols != null && strRows != null)
						{
							int iCols = Integer.parseInt(strCols);
							int iRows = Integer.parseInt(strRows);
							iTable.addCell(cell,iRows,iCols);
						}
						else
							iTable.addCell(cell);
					}
					else if(o instanceof PDFTable) // if pdfTable
					{
						// get the nested table
						//System.out.println("found another table");

						Table nestedTable = ((PDFTable)o).getNestedTable(cb);
						if(nestedTable!=null)
						{
							Cell cell = new Cell(nestedTable);
							setCellProperties(cell,(String)cellProps.get("header"),
													(String)cellProps.get("noWrap"),
													(String)cellProps.get("colSpan"),
													(String)cellProps.get("rowSpan"),
													(String)cellProps.get("vAlign"),
													(String)cellProps.get("hAlign"),
													(String)cellProps.get("bColor"),
													(String)cellProps.get("bWidth"));

							String strCols 	= (String)cellProps.get("colNo");
							String strRows 	= (String)cellProps.get("rowNo");

							if(strCols != null && strRows != null)
							{
								int iCols = Integer.parseInt(strCols);
								int iRows = Integer.parseInt(strRows);
								iTable.addCell(cell, iRows, iCols);
							}
							else
								iTable.addCell(cell);
						}
					}
				}
				return iTable;
			}
			catch (BadElementException bee)
			{
				bee.printStackTrace();
				return null;
			}
		}
		else
			return null;
	}

}

/**
* class to handle object addition to a cell
*/
class ObjectCellPair
{
	Object myObject;
	HashMap myCell;

	public ObjectCellPair(Object pObject, HashMap pCell) {
		myObject = pObject;
		myCell = pCell;
	}

	public HashMap getCell()
	{
		return myCell;
	}

	public Object getObject() {
		return myObject;
	}
}
