import java.util.ArrayList;
import com.omnipilot.lassopro.*;
import com.lowagie.text.*;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.Barcode;

/**
 * This class holds all the implementations of the member tags of the
 * PDF_Text tag.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFText
{
	String myType;
	Font myFont;
	String myColor;
	String myBGColor;
	String myAlign;

	boolean myUnderLine;
	boolean myStrikeThru;
	float myTextRise;
	String myAnchor;
	String myName;
	String myGoTo;
	String myFileName;
	float myLeading;

	float myIndentLeft;
	float myIndentRight;

	ArrayList myList;


	/**
	 * This is the constructor of the class.
	 * The random number for the key is generated here.
	 */
	public PDFText()
	{
		myList = new ArrayList();
	}

	/**
	 * This method returns the current type
	 * @return      the String.
	 */

	public String getType()
	{
		return this.myType;
	}

	/**
	 * This method returns the current font.
	 * @return      the Font.
	 */

	public Font getFont()
	{
		return this.myFont;
	}

	/**
	 * This method returns the current color
	 * @return      the String.
	 */

	public String getColor()
	{
		return this.myColor;
	}

	/**
	 * This method returns the current BGColor
	 * @return      the String.
	 */

	public String getBGColor()
	{
		return this.myBGColor;
	}

	/**
	 * This method returns the current alignment
	 * @return      the String.
	 */

	public String getAlign()
	{
		return this.myAlign;
	}

	/**
	 * This method creates a text
	 */
	 public int CreateText(String content, String sType,Font pFont,String sColor,String sBGColor,
	 						boolean bUnderLine,boolean bStrikeThru,float fTextRise,String sAnchor,
	 						String sName,String sGoTo,String sFileName,float fLeading,String sAlign,
							float fIndentLeft,float fIndentRight)
	 {
		myType = sType;
		myFont = pFont;
		myColor = sColor;
		myBGColor = sBGColor;
		myUnderLine = bUnderLine;
		myStrikeThru = bStrikeThru;
		myTextRise = fTextRise;
		myAnchor = sAnchor;
		myName = sName;
		myGoTo = sGoTo;
		myFileName = sFileName;
		myLeading = fLeading;
		myAlign = sAlign;
		myIndentLeft = fIndentLeft;
		myIndentRight = fIndentRight;

		if(content!=null)
			return textAddString(content);

		return 0;
	 }

	 /**
	 * adds a string to the text
	 * @param sContent to be added
	 */
	 public int textAddString(String sContent)
	 {
		if(myType==null || myType.equals(""))
			myType = "paragraph";
		Chunk chunk = null;
		if(myFont!=null)
		{
			if(myColor!=null && !myColor.equals(""))
				myFont.setColor(java.awt.Color.decode(myColor));
			chunk = new Chunk(sContent,myFont);
		}
		else
			chunk = new Chunk(sContent);

		if(myBGColor!=null && !myBGColor.equals(""))
			chunk = chunk.setBackground(java.awt.Color.decode(myBGColor));
		if(myTextRise!=0)
			chunk.setTextRise(myTextRise);

		if(myAnchor!= null && (myAnchor.trim().length() > 0))
			chunk.setAnchor(myAnchor);
		else if(myGoTo != null && (myFileName == null || myFileName.trim().length() <= 0))
			chunk.setLocalGoto(myGoTo);
		else if(myGoTo != null && (myFileName!= null && myFileName.trim().length() > 0))
			chunk.setRemoteGoto(myFileName,myGoTo);
		if(myName != null)
			chunk.setLocalDestination(myName);

		if(myType.equalsIgnoreCase("phrase"))
		{
			Phrase phrase = null;
			if(myLeading==0)
				phrase = new Phrase(chunk);
			else
				phrase = new Phrase(myLeading, chunk);

			myList.add(phrase);
		}
		else if(myType.equalsIgnoreCase("paragraph"))
		{
			Paragraph paragraph = null;
			if(myLeading==0)
				paragraph = new Paragraph(chunk);
			else
				paragraph = new Paragraph(myLeading, chunk);
			paragraph.setAlignment(myAlign);
			if(myIndentLeft!=0)
				paragraph.setIndentationLeft(myIndentLeft);
			if(myIndentRight!=0)
				paragraph.setIndentationRight(myIndentRight);

			myList.add(paragraph);
		}
		else
			myList.add(chunk);

		return 0;
	 }

	 /**
	 * adds an Image to the text
	 * @param pImage to be added
	 * @param sAlign alignment for the image
	 * @param bWrap wrap paramter for the image
	 */
	 public int textAddImage(PDFImage pImage, String sAlign, boolean bWrap)
	 {
		Image iImage = pImage.getImage();
		int iAlign = Image.DEFAULT;

		if(sAlign==null || sAlign.equals(""))
			sAlign = myAlign;

		if(sAlign==null)
			sAlign = "";

		if(sAlign.equalsIgnoreCase("left"))
			iAlign = Image.LEFT;
		else if(sAlign.equalsIgnoreCase("right"))
			iAlign = Image.RIGHT;
		else if(sAlign.equalsIgnoreCase("center") || sAlign.equalsIgnoreCase("centre") || sAlign.equalsIgnoreCase("middle"))
			iAlign = Image.MIDDLE;

		if(bWrap)
			iImage.setAlignment(iAlign | Image.TEXTWRAP);
		else
			iImage.setAlignment(iAlign);

		myList.add(iImage);
		return 0;
	 }

	 /**
	 * adds a PDFBarCode to the text
	 * @param pBarCode to be added
	 * @param sAlign alignment for the barcode
	 * @param bWrap wrap paramter for the barcode
	 */
	 public int textAddBarCode(PDFBarCode pBarCode, String sAlign, boolean bWrap)
	 {
		/* TO ADD A BAR CODE TO DOC PdfContentByte IS NEEDED;so the adding of bar code
		to  the PDFText object is actually lazy addition */
		String align = "left";

		if(sAlign==null || sAlign.equals(""))
			sAlign = myAlign;

		if(sAlign!=null && !sAlign.equals(""))
		{
			if(sAlign.equalsIgnoreCase("right"))
				align = "right";
			else if(sAlign.equalsIgnoreCase("middle"))
				align = "middle";
			else if(sAlign.equalsIgnoreCase("left"))
				align = "left";
		}

		pBarCode.setAlignment(align);
		pBarCode.setWrap(bWrap);
		myList.add(pBarCode);
		return 0;
	 }

	 /**
	 * adds a PDFTable to the text
	 * @param pTable to be added
	 * @param sAlign alignment for the table
	 */
	 public int textAddTable(PDFTable pTable, String sAlign)
	 {
		String align = "left";

		if(sAlign==null || sAlign.equals(""))
			sAlign = myAlign;

		if(sAlign!=null && !sAlign.equals(""))
		{
			if(sAlign.equalsIgnoreCase("right"))
				align = "right";
			else if(sAlign.equalsIgnoreCase("middle"))
				align = "middle";
			else if(sAlign.equalsIgnoreCase("left"))
				align = "left";
		}

		pTable.getTable().setAlignment(align);
		myList.add(pTable);
		return 0;
	 }

	 /**
	 * adds a PDFText to the text
	 * @param pText to be added
	 */
	 public int textAddText(PDFText pText)
	 {
		myList.add(pText);
		return 0;
	 }

	 /**
	 * adds a chunk
	 */
	 public void addChunk(Chunk chunk)
	 {
		 if(chunk!=null)
		 	myList.add(chunk);
	 }

	/**
	* @return the ArrayList object
	*/
	public ArrayList getText()
	{
		return myList;
	}

	/**
	* this should not contain table
	*/
	public Phrase getNestedText(PdfContentByte cb)
	{
		return getNestedText(this, cb);
	}

	/**
	* should not contain table
	*/
	protected Phrase getNestedText(PDFText pText, PdfContentByte cb)
	{
		ArrayList aList = null;
		if(pText!=null)
			aList = pText.getText();

		if(aList!=null)
		{
			int length = aList.size();
			Phrase p = new Phrase("");
			Chunk separatorChunk = new Chunk(" ");
			for(int i=0; i<length; i++)
			{
				Object o = aList.get(i);
				if(o instanceof Chunk)
				{
					p.add((Chunk)o);
					//p.add(separatorChunk);
				}
				else if(o instanceof Phrase)
				{
					ArrayList chunkList = ((Phrase)o).getChunks();
					p.add(getPhraseFromChunks(chunkList));
					p.add(separatorChunk);
				}
				else if(o instanceof Paragraph)
				{
					ArrayList chunkList = ((Paragraph)o).getChunks();
					p.add(getPhraseFromChunks(chunkList));
					p.add(separatorChunk);
				}
				else if(o instanceof Image)
				{
					Image img = Image.getInstance((Image)o);
					Chunk chunk = new Chunk(img, 0, 0);
					p.add(chunk);
					p.add(separatorChunk);
				}
				else if(o instanceof PDFBarCode)
				{
					if(cb!=null)
					{
						PDFBarCode pbc = (PDFBarCode)o;
						Barcode bc = pbc.getBarcode();
						Image iImage = bc.createImageWithBarcode(cb, pbc.getBarcodeColor(), pbc.getBarTextColor());
						Chunk chunk = new Chunk(iImage, 0, 0);
						p.add(chunk);
						p.add(separatorChunk);
					}
				}
				else if(o instanceof PDFTable)
				{
					//handle here
					//Table t = ((PDFTable)o).getTable();
					//ArrayList chunkList= t.getChunks();
					//But this method in Table class simply returns a new Empty ArrayList
				}
				else if(o instanceof PDFText)
				{
					Phrase pNext = getNestedText((PDFText)o, cb);
					if(pNext!=null)
					{
						p.add(pNext);
						p.add(separatorChunk);
					}
				}
			}
			return p;
		}
		else
			return null;

	}

	/**
	* get a Phrase from an ArrayList of Chunks
	*/
	private Phrase getPhraseFromChunks(ArrayList chunks)
	{
		Phrase p = new Phrase("");
		int size = 0;
		if(chunks!=null)
			size = chunks.size();
		for(int i=0; i<size; i++)
		{
			Chunk chunk = (Chunk)chunks.get(i);
			if(chunk!=null)
				p.add(chunk);
		}
		return p;
	}




}
