import java.io.*;
import java.util.*;
import java.util.zip.*;
import com.omnipilot.lassopro.*;

public class ZipCountTag extends LassoTagModule
{
	public int myZipCountFunc(LassoCall lasso, int action)
	{
		int err = ERR_NOERR;
				
		try
		{		
			// define the storage for the tag param count
			IntValue count = new IntValue();
			
			// how many params this tag has?
			err = lasso.getTagParamCount( count );
			
			// proceed if the tag has at least one parameter
			if ( err == ERR_NOERR && count.intValue() > 0 )
			{
				boolean filesOnly = false;
				
				// define the variable to hold the tag param value
				LassoValue param = new LassoValue();
				
				// first, try to find the parameter by the name
				err = lasso.findTagParam( "-zipfile", param );
				
				// if -zipfile param doesn't exist, get the first tag param -
				// assuming that it is the name of a zip file
				if ( err != ERR_NOERR || param.name() == null )
					lasso.getTagParam( 0, param );
				else
					param.setName(param.data());
				
				// bail if we haven't got a valid filepath/name
				if ( param.name() == null || param.name().length() == 0 )
					return LassoErrors.InvalidParameter;
				
				// should we count directories, too?	
				if ( count.intValue() > 1 )
					filesOnly = ( lasso.tagParamIsDefined("-filesonly") == ERR_NOERR );

				// convert absolute or relative path into from-the-server-root path.
				String filePath = lasso.fullyQualifyPath( param.name() );
				
				// now convert a from-the-root path into a full internal path
				filePath = lasso.resolvePath( filePath );
				
				//finally, convert internal path to a platform-specific path
				filePath = lasso.getPlatformSpecificPath( filePath );
				
				// try to create new instance of ZipFile object														
				ZipFile zip = new ZipFile( filePath );
					
				Enumeration enumer = zip.entries();
				ZipEntry entry = null;
				int zipcount = 0;
				
				// iterate through zip archive items, incrementing the count
				while ( enumer.hasMoreElements() )
				{
					entry = (ZipEntry)enumer.nextElement();
					if ( !filesOnly || !entry.isDirectory() )
						++zipcount;
				}
				
				// output the result as string
				err = lasso.outputTagData( Integer.toString( zipcount ) );
				
				zip.close();
			}
		}
		catch (ZipException e)
		{
			System.err.println( e.toString() );
			lasso.setResultMessage( e.getMessage() );
			return LassoErrors.FileNotFound;
		}
		catch ( java.io.IOException e )
		{
			System.err.print( e.toString() );
			lasso.setResultMessage( e.getMessage() );			
			return LassoErrors.FileNotFound;
		}
		
		return err;
	}
	
	public void registerLassoModule()
	{
		registerTagModule( "ZipCountTag", "zip_count", "myZipCountFunc", FLAG_SUBSTITUTION, "Count items in a zip file" );
	}
}

