/** \file LassoDefines.h
	\brief Definitions of constant values
*/

#ifndef _LASSO_DEFINES_H_
#define _LASSO_DEFINES_H_

#define kLPOpBegin "opbegin"
#define kLPOperatorBegin "operatorbegin"
#define kLPOpEnd "opend"
#define kLPOperatorEnd "operatorend"

#define kLPOpBeginW LPW(L"opbegin")
#define kLPOperatorBeginW LPW(L"operatorbegin")
#define kLPOpEndW LPW(L"opend")
#define kLPOperatorEndW LPW(L"operatorend")

// some internal variable keys
#define kLPHTTPHeader "__http_header__"
#define kLPHTMLReply "__html_reply__"
#define kLPHTTPHeaderW LPW(L"__http_header__")
#define kLPHTMLReplyW LPW(L"__html_reply__")

#define kLPOutputEncoding LPW(L"__encoding__")
#define kLPInputEncoding LPW(L"__input_encoding__")
#define kLPInternalConverters LPW(L"__internal_converters__")
#define kLPExternalConverters LPW(L"__external_converters__")

// result code & message
#define kLPResultCode "__result_code__"
#define kLPResultMessage "__result_message__"
#define kLPResultCodeW LPW(L"__result_code__")
#define kLPResultMessageW LPW(L"__result_message__")

// formatting flags
#define kLPFormattingFlags "lp_formatting_flags"
// valie parameters for formatting flags
#define kLPFlagScientific LPW(L"-scientific")
#define kLPFlagPrecision LPW(L"-precision")
#define kLPFlagDecChar LPW(L"-decimalchar")
#define kLPFlagPadding LPW(L"-padding")
#define kLPFlagPadChar LPW(L"-padchar")
#define kLPFlagPadRight LPW(L"-padright")
#define kLPFlagGroupChar LPW(L"-groupchar")
#define kLPFlagHexidecimal LPW(L"-hexadecimal")

#define kLPLassoServiceHost "__lassoservice_host__"
#define kLPLassoServiceIP "__lassoservice_ip__"

// top level location of tag registry
//#define kLPTagRegistry "__tag_registry__"
// within that, location of substitution tags
#define kLPSubstitutionTags "__tags__"
// suffix for site specific dbs
#define kLPSiteDBPrefix "lasso_site_"
// the admins group id
#define kLPAdminsID "__admins_id__"
// the everybody id
#define kLPEverybodyID "__everybody_id__"

#define kLPAnonymousName "ANONYMOUS"
#define kLPAdministratorName "ADMINISTRATOR"

#define kLPPreCondition LPW(L"precondition")
#define kLPPostCondition LPW(L"postcondition")
#define kLPHandle LPW(L"handle")

// base type names
#define kLPNullTypeName LPW(L"null")
#define kLPIntegerTypeName LPW(L"integer")
#define kLPDecimalTypeName LPW(L"decimal")
#define kLPStringTypeName LPW(L"string")
#define kLPBoolTypeName LPW(L"boolean")
#define kLPArrayTypeName LPW(L"array")
#define kLPMapTypeName LPW(L"map")
#define kLPPairTypeName LPW(L"pair")
#define kLPReferenceTypeName LPW(L"reference")
#define kLPTagTypeName LPW(L"tag")
#define kLPCustomTypeName LPW(L"custom")
#define kLPDBInputTypeName LPW(L"dbinput")
#define kLPDBOutputTypeName LPW(L"dboutput")
#define kLPBytesTypeName LPW(L"bytes")

#define kLPAtBegin LPW(L"_at_begin")
#define kLPAtEnd LPW(L"_at_end")

// special tags for custom types
#define kLPOnCompare "oncompare"
#define kLPOnAssign "onassign"
#define kLPOnDestroy "ondestroy"
#define kLPOnCreate "oncreate"
#define kLPOnConvert "onconvert"

#define kLPOnCompareW LPW(L"oncompare")
#define kLPOnAssignW LPW(L"onassign")
#define kLPOnDestroyW LPW(L"ondestroy")
#define kLPOnCreateW LPW(L"oncreate")
#define kLPOnConvertW LPW(L"onconvert")

// the name & id for the current site
#define kLPSiteID "__site_id__"
#define kLPSiteName "__site_name__"

#define kLPLoopCountW LPW(L"__loop_count__")
#define kLPRecordCountW LPW(L"__record_count__")
#define kLPRelatedCountW LPW(L"__related_count__")
#define kLPRepeatCountW LPW(L"__repeat_count__")
#define kLPValueCountW LPW(L"__value_count__")

#define kLPNameEncodingType "ISO-8859-1"
#define kLPUTF8EncodingType "UTF-8"
#define kLPWindowsFileNameEncodingType "windows-1252"

#define kLPSiteRoot "LassoSites"
#define kLPDatabasesRoot "SQLiteDBs"
#define kLPStartupRoot "LassoStartup"
#define kLPModulesRoot "LassoModules"
#define kLPAdminRoot "LassoAdmin"
#define kLPLibrariesRoot "LassoLibraries"
#define kLPAppsRoot "LassoApps"
#define kSQLiteDBRoot "SQLiteDBs"

#define kLPErrorsLogFile "LassoErrors.txt"

#define kLassoInternalDB "lasso_internal"
#define kLassoGlobalDB "lasso_global"

#ifdef WIN32
#	define kChildPipeNamePrefix "\\\\.\\pipe\\"
#else
#	define kChildPipeNamePrefix "/tmp/"
#endif

#define kNamespaceLoader LPW(L"namespace_load")

#define kServerAdminLassoApp "ServerAdmin.LassoApp"

#define kLPDigestAuthStatus LPW(L"__digest_auth__")

enum SortOrder { orderAscending, orderDescending, orderCustom = 4 };

/*
	Operators for datasources
*/
enum
{
	kLassoGreaterThan = '>   ',
	kLassoGreaterThanEquals = '>=  ',
	kLassoEquals = '=   ',
	kLassoLessThan = '<   ',
	kLassoLessThanEquals = '<=  ',
	kLassoBeginsWith = 'bgwt',
	kLassoEndsWith = 'ends',
	kLassoContains = 'cont',
	kLassoNotContains = '!cts',
	kLassoNotBeginsWith = '!bgs',
	kLassoNotEndsWith = '!end',
	kLassoAND = 'AND ',
	kLassoOR = 'OR  ',
	kLassoNOT = 'NOT ',
	kLassoNo = 'no  ',
	kLassoAny = 'any ',
	kLassoInList = ' nlt',
	kLassoNotInList = '!nlt',
	kLassoInFullText = ' ftx',
	kLassoInRegExp = ' rxp',
	kLassoNotInRegExp = '!rxp'
};

typedef enum lpEncodingType 
{ 
	encodeURL,							// TODO: Comment encoding methods here
	encodeRaw, 							// 
	encodeSmart, 						// 
	encodeBreak, 						// 
	encodeDefault, 						// 
	encodeStrictURL, 					// 
	encodeXML,
	encodeNone,							// 
	encodeHTML
} encodingMethod;	// end encodingMethod

#endif
