#include "LassoProtoUtils.h"
#if defined(WIN32) || defined(WIN64)
#include <winsock.h>
#else
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <syslog.h>
#if MACHINE_LINUX
#include <netinet/in.h>
#endif
#endif

void convert_endian_h( LPCommandBlock* block )
{
	block->fCmd = (LPCommand)ntohl( block->fCmd );
	block->fSeqNum = ntohl( block->fSeqNum );
	block->fReqID = ntohl( block->fReqID );
	block->fResultCode = ntohl(block->fResultCode);
	block->fDataSize = ntohl( block->fDataSize );
}

void convert_endian_n( LPCommandBlock* block )
{
	block->fCmd = (LPCommand)htonl( block->fCmd );
	block->fSeqNum = htonl( block->fSeqNum );
	block->fReqID = htonl( block->fReqID );
	block->fResultCode = htonl(block->fResultCode);
	block->fDataSize = htonl( block->fDataSize );
}

int send_block( int sock, const LPCommandBlock* block, const char * data )
{
	int nReturn = 0;
	int res	 = 0;
	unsigned int sent = 0;
	LPCommandBlock local;
	
	if( sock > 0 )
	{
		char* fullBlock = (char*)malloc( sizeof(LPCommandBlock)+block->fDataSize );
		
		local = *block;
		convert_endian_n(&local);
		memcpy( fullBlock, &local, sizeof(LPCommandBlock) );

		if ( block->fDataSize > 0 )
			memcpy( (fullBlock+sizeof(LPCommandBlock)), data, block->fDataSize );
		
		do
		{
			res = send(sock, fullBlock+sent, (sizeof(LPCommandBlock)+block->fDataSize)-sent, 0);
			if ( res > 0 ) sent += res;
		}
		while( res > 0 && sent < sizeof(LPCommandBlock)+block->fDataSize );
		
		nReturn = (sent == sizeof(LPCommandBlock)+block->fDataSize);
		
		free(fullBlock);
	}

	return nReturn;	
}

READ_RESULT
read_block( int sock, LPCommandBlock * block, char ** data, unsigned int* time_out )
{
	int res = 0;
	struct timeval tv;
	fd_set rset, eset;
	
	FD_ZERO(&rset);
	FD_ZERO(&eset);
	FD_SET(sock, &rset);
	FD_SET(sock, &eset);

	tv.tv_usec = 0;
	tv.tv_sec = *time_out;
	
	res = select(FD_SETSIZE, &rset, NULL, &eset, &tv );

	// If 0 means select timed out,
	if ( res == 0 )
		return READ_TIMEOUT;
	if ( (res == 1 && FD_ISSET(sock, &eset)) || !FD_ISSET(sock, &rset) )
		return READ_ERROR;

	res = recv( sock, (char*)block, sizeof( LPCommandBlock ), 0 );
	while(res > 0 && res < (int)sizeof( LPCommandBlock )) // handle partial block read
	{
		FD_ZERO(&rset);
		FD_ZERO(&eset);
		FD_SET(sock, &rset);
		FD_SET(sock, &eset);
		
		tv.tv_usec = 0;
		tv.tv_sec = *time_out;
		
		int newRes = select(FD_SETSIZE, &rset, NULL, &eset, &tv );
		if ( newRes == 0 )
			return READ_TIMEOUT;
		if ( (newRes == 1 && FD_ISSET(sock, &eset)) || !FD_ISSET(sock, &rset) )
			return READ_ERROR;
		
		newRes = recv( sock, ((char*)block)+res, sizeof( LPCommandBlock )-res, 0 );
		if (newRes <= 0)
			return READ_ERROR;
		res += newRes;
	}
	if( res == sizeof( LPCommandBlock ) )
	{
		convert_endian_h(block);
		if ( block->fDataSize > 0 )
		{
			unsigned int read = 0;

			if ( block->fDataSize > 1048576 )
				return READ_ERROR;
			*data = (char*)malloc(block->fDataSize);

			do
			{
				res = recv(sock, (*data) + read, block->fDataSize - read, 0);
				if ( res > 0 )
					read += res;
			}
			while( res > 0 && read < block->fDataSize );
			
			return read == block->fDataSize ? READ_OK : READ_ERROR;
		}
		return READ_OK;
	}
	return READ_ERROR;
}
/*
int log_message(int sock, const char * msg)
{
	int res = 0;
	unsigned int size = strlen(msg)+1;
	LPCommandBlock block;
	block.fCmd = cmdLogMsg;
	block.fSeqNum = 1;
	block.fReqID = -1;
	block.fResultCode = 0;
	block.fDataSize = size;

	return send_block(sock, &block, msg);
}
*/