#ifndef _POSTREADER_H_
#define _POSTREADER_H_

// This class handles reading POST data for web server modules
// The class is initialized with the current submission's content type
// Data is fed using the AddData method
// After all the data has been read, information about the submission can be accessed and made available to Lasso
// The destructor will clean up any uploaded files

#ifndef LASSO_WEBSTAR
#include "osConfig.h"
#else
// osConfig.h includes several types defined in MacTypes.h
typedef	unsigned long long osUInt64;
#endif
#include <string>
#include <list>
#include <stdio.h>


#if MACHINE_UNIX
#	include <unistd.h>
#	define UNLINK(path) (remove(path))
#else
#	include <windows.h>
#	define UNLINK(path) (DeleteFile(path))
#endif

#define FCLOSE(file) (fclose(file))
#define MY_INVALID_FILE NULL

class LP8POSTReader
{
public:
	LP8POSTReader( const char * contentType );
	~LP8POSTReader();

	void Reset();

	bool IsMultiPart(){ return fMulti; }
	bool GotFileUpload(){ return fGotFile; }
	
	int AddToBuffer( const char* data, unsigned int length );
	const std::string* GetPostArgs( );
//	void GetTMPFilePaths( std::vector<std::string>& tmppaths );

	bool IsUploadedFile( const char * path ) const;

	void SetMaxFileSize( int s ) { fMaxFileSize = s; }

	enum 
	{
		stateNone,
		stateBoundary, // next thing to be read will be a boundry
		stateHeader, // read header lines until data starts
		stateNameValue, // read a simple value; name has already been set
		stateFile, // read file data until boundry
		stateDone
	};

	class BodySpec
	{
	public:
		BodySpec() : fFile(MY_INVALID_FILE), fFileSize(0), fState(stateNone) {}
		~BodySpec()
		{
			if ( fFile != MY_INVALID_FILE )
			{
				FCLOSE(fFile);
				fFile = MY_INVALID_FILE;
			}
			if ( fTmpFileName.size() > 0 )
			{
				UNLINK(fTmpFileName.c_str());
			}
		}

		BodySpec & operator=( const BodySpec & ); // no definition
		BodySpec( const BodySpec & ); // no definition

		std::string fFieldName;
		std::string fFieldValue;

		FILE* fFile;

		osUInt64 fFileSize;
		std::string fContentType;
		std::string fFileName;
		std::string fTmpFileName;

		int fState;
	};

protected:
	int fMaxFileSize;
	bool fMulti;
	int fState;
	bool fGotFile;
	
	std::list<BodySpec*> fBody;
	std::string fDirectory; //used in the future to store where tmp files go
	std::string fBuffer;  //used as scratch
	std::string fBoundary; //keep copy of boundary for parsing of mulitpart data

	bool EncodeURL( std::string* data );
};

#endif //_POSTREADER_H_
