/** \file RequestParams.h
	\brief Enumeration of request specific variables
	
	This file contains the names of all the request specific variables which can be retrieved using lasso_getRequestParam.
	Lasso Connector modules must support the retrieval of these variables.
*/

#ifndef _REQUESTPARAMS_H_
#define _REQUESTPARAMS_H_

#ifdef __cplusplus
extern "C"
{
#endif

/**
	Constants used for retrieving parameters from lasso_getRequestParam
*/
typedef enum 
{
	rpPathArgKeyword		= '----',
	rpSearchArgKeyword		= 'kfor', /**< Everything after ? in the url. */
	rpUserKeyword			= 'user', /**< The submitted user name. */
	rpPasswordKeyword		= 'pass', /**< The submitted password. */
	rpFromUser				= 'frmu',
	rpAddressKeyword		= 'addr', /**< The client's address. May not have been resolved, depending on the web server's configuration, in which case the value will be the IP address. */
	rpPostKeyword			= 'post', /**< Any submitted POST arguments. */
	rpMethodKeyword			= 'meth', /**< The request's method. */
	rpServerName			= 'svnm', /**< The server name according to the web server. */
	rpServerPort			= 'svpt', /**< The port the web server is running on.*/
	rpScriptName			= 'scnm', /**< The name of the requested file. */
	rpFileName				= rpScriptName,
	rpLassoName				= rpScriptName,
	rpContentType			= 'ctyp', /**< The content type header, if it was supplied. */
	rpContentLength			= 'CLen', /**< The length of the content, if any. */
	rpReferrerKeyword		= 'refr', /**< The referrer according to the user agent. */
	rpRefererKeyword		= rpReferrerKeyword,
	rpUserAgentKeyword		= 'Agnt', /**< The user agent as reported. */
	rpActionKeyword			= 'Kact', 
	rpActionPathKeyword		= 'Kapt',
	rpClientIPAddress		= 'Kcip', /**< The client's IP address. */
	rpFullRequestKeyword	= 'Kfrq', /**< All the submitted headedrs. */
	rpRealmsKeyword			= 'Prlm',
	rpCurrentRealmKeyword	= 'CRlm',
	
	rpCookiesKeyword		= 'koky', /**< The cookies header field. */
	
	rpIndexFile				= 'Pixf',

	rpSessionRootPath		= 'SPth', /**< The root of the current request. */
	rpSessionErrorFile		= 'Perf',
	rpSessionNoAccessFile	= 'Pnaf',
	rpSessionLogFile		= 'Plgf',

	rpSessionSiteName		= 'Site', /**< The name of the site handling the current request. */

	rpResolveFilePath		= 'RsFP', /**< Resolve the supplied url path name to that of the physical file. 
										   Requires that the url path name be supplied in the \a name member of the \a result parameter supplied to lasso_getRequestParam.
									   */

	rpWebFileData			= 'RmFl', /**< Return the file data for the given url path.
										   Requires that the url path name be supplied in the \a name member of the \a result parameter supplied to lasso_getRequestParam.
									   */
	rpRootFileData			= 'RmRF', /**< Return the file data for the given physical path.
										   Requires that the physical path name be supplied in the \a name member of the \a result parameter supplied to lasso_getRequestParam.
									   */
	rpAuthorizationKeyword  = 'Auth' /**< The authorzation header, if available. */

} RequestParamKeyword;

/**
	These constants are used to retrieve params from lasso_getLassoParam
*/
typedef enum 
{
	lpModulesFolderPath			= 'MPth',
	lpStartupItemsFolderPath	= 'SPth',
	lpLassoDataFilePath			= 'LDFP',
	lpLassoErrorsFilePath		= 'LEFP',
	lpDataRootFolderPath		= 'DRFP',

	// relative path to scripts root
	lpScriptsRoot				= 'SRot',
	// relative path to site scripts root (most likely includes lpScriptsRoot)
	lpScriptsSiteRoot			= 'SSit'

} LassoParamKeyword;

#ifdef __cplusplus
}
#endif

#endif
