/** \file osConfig.h
	\brief Cross-platform configuration
	
	This file used to set system configuration and build settings.
*/

#ifndef _OSCONFIG_H_
#define _OSCONFIG_H_

#if defined(EXPORT_LCAPI) || defined(LP_USE_ICU)
#include "unicode/utypes.h"
#else
typedef short UChar;
#endif

////////////////////////////////////
//  Determine the target platform.
////////////////////////////////////

// Microsoft Visual C++ defines the _WIN32 preprocessor identifier, Metrowerks defines the __INTEL__ preprocessor identifier
#if defined(WIN32) || defined(WIN64)
	#define	MACHINE_PC		1
	#define MACHINE_MAC		0
	#define MACHINE_UNIX		0
#elif macintosh
	#define	MACHINE_PC		0
	#define MACHINE_MAC		1
	#define MACHINE_UNIX		0
#else // unix & MacOS X (for now)
	#define	MACHINE_PC		0
	#define MACHINE_MAC		0
	#define MACHINE_UNIX		1
#endif // _WIN32

#ifdef __BIG_ENDIAN__ 
#	define MACHINE_LITTLE_ENDIAN 0
#	define MACHINE_BIG_ENDIAN 1
#else
#	define MACHINE_LITTLE_ENDIAN 1
#	define MACHINE_BIG_ENDIAN 0
#endif

#if MACHINE_UNIX && defined(__MWERKS__)
#define MACHINE_MACOSX 1
#endif

#if MACHINE_MACOSX
#	include <AvailabilityMacros.h>
#	ifdef MAC_OS_X_VERSION_10_5
#		define MODERN_MAC 1
#	endif
#endif

#if !MACHINE_LINUX
#ifdef FD_SETSIZE
#undef FD_SETSIZE
#endif
#define FD_SETSIZE 128
#endif

#define LPW(X) ((const UChar*)X)

#include <stdlib.h>

#if defined(WIN32) && !defined(strcasecmp)
#define strcasecmp _stricmp
#endif

/////////////////////////////////////
//  Determine the build type
/////////////////////////////////////

#ifdef NDEBUG
	#define DEBUG_BUILD			0
#else
	#define DEBUG_BUILD			1
#endif // NDEBUG
#define RELEASE_BUILD		!DEBUG_BUILD


/////////////////////////////////////
//  Define some basic datatypes
/////////////////////////////////////

#ifndef nil
	#define nil	0
#endif

#ifdef atoi
#undef atoi
#endif

#define atoi(X) strtol(X, NULL, 10)

typedef unsigned long osSize_t;

#if MACHINE_PC
	
#   define kEOL "\r\n"

	typedef unsigned short UniChar;	// UNICODE character

	typedef __int8 osInt8;
	typedef __int16 osInt16;
	typedef __int32 osInt32;
	typedef __int64 osInt64;

	typedef unsigned __int8 osUInt8;
	typedef unsigned __int16 osUInt16;
	typedef unsigned __int32 osUInt32;
	typedef unsigned __int64 osUInt64;

	#define OS_EXPORT				__declspec( dllexport )
	#define OS_IMPORT				__declspec( dllimport )

	//  Disable warnings on extern before template instantiation.
	#pragma warning( disable: 4231 )

	//  Disable warnings about debug names being truncated to 255 characters
	#pragma warning( disable: 4786 )

	//  Disable warnings for non-exported objects being members of exported objects (we'll get a link error if there really is a problem)
	#pragma warning( disable: 4251 )

	//  Tell the compiler we're using NT4 so we can use NT-specific calls
//	#define _WIN32_WINNT	0x0500

	//  Tell STL that bad_alloc is defined in the header file: <new>
//	#define __STL_BAD_ALLOC_DEFINED		1

	//  Speed up compilation by telling Windows not to include uncommonly-used stuff
	#define WIN32_LEAN_AND_MEAN

//	#define _CRT_SECURE_NO_WARNINGS
	
#endif // MACHINE_PC


#if MACHINE_UNIX
	
	#ifndef INVALID_SOCKET
	#define INVALID_SOCKET ((int)(~0))
	#endif

	#ifndef _PTHREADS
	#define _PTHREADS
	#endif
//	#define __USE_MALLOC
	#ifndef NULL
		#define NULL 0
	#endif // NULL
	
#   define kEOL "\n"

	typedef unsigned short UniChar;	// UNICODE character

#ifdef	MACHINE_MACOSX
#include <machine/types.h>
	
	typedef int8_t osInt8;
	typedef int16_t osInt16;
	typedef int32_t osInt32;
	typedef int64_t osInt64;

	typedef u_int8_t osUInt8;
	typedef u_int16_t osUInt16;
	typedef u_int32_t osUInt32;
	typedef u_int64_t osUInt64;
#else
#include <inttypes.h>
        typedef int8_t osInt8;
        typedef int16_t osInt16;
        typedef int32_t osInt32;        
        typedef int64_t osInt64;

        typedef uint8_t osUInt8;
        typedef uint16_t osUInt16;
        typedef uint32_t osUInt32;
        typedef uint64_t osUInt64;  
#endif

	#define OS_EXPORT				// !FIX!
	#define OS_IMPORT				// !FIX!
	#define BEGINEXPORTCLASS		// !FIX!
	#define ENDEXPORTCLASS			// !FIX!
	#define BEGINIMPORTCLASS		// !FIX!
	#define ENDIMPORTCLASS			// !FIX!
	
#endif // MACHINE_UNIX

typedef unsigned char * StringPtr;

// Variant should be able to support each of these types

typedef osUInt32 LP_TypeDesc;
typedef osInt64 record_id;

// if you're not using C++, you don't get these
#if __cplusplus
namespace LPTypes
{
const LP_TypeDesc lpTypeString		= 'TEXT';
const LP_TypeDesc lpTypeNull		= 'null';
const LP_TypeDesc lpTypeInteger		= 'long';
const LP_TypeDesc lpTypeBoolean		= 'bool';
const LP_TypeDesc lpTypePict		= 'PICT';

const LP_TypeDesc lpTypeBlob		= 'blob';
const LP_TypeDesc lpTypeBytes		= 'blob';
const LP_TypeDesc lpTypeDecimal		= 'doub';
const LP_TypeDesc lpTypeDateTime	= 'Date';
const LP_TypeDesc lpTypeArray		= 'arry';
const LP_TypeDesc lpTypeMap			= 'map_';
const LP_TypeDesc lpTypeTag			= 'code'; // new name. these 2 should be the same
const LP_TypeDesc lpTypeReference	= 'ref_';
const LP_TypeDesc lpTypePair		= 'pair';
const LP_TypeDesc lpTypeCustom		= 'Yers';

const LP_TypeDesc lpTypeDBInput		= 'Inpt';
const LP_TypeDesc lpTypeDBOutput	= 'Otpt';

const LP_TypeDesc kDateDataType	= 'Date';
const LP_TypeDesc kProtectionNone = 'none';
const LP_TypeDesc kProtectionReadOnly = 'nmod';
}
#endif
#endif // _OSCONFIG_H_
