/** \file osErrors.h
	\brief Enumeration of error codes
	
	This file contains the error codes which may be returned from any of the LassoCAPI calls.
*/

#ifndef _OSERRORS_H_
#define _OSERRORS_H_

#include "osConfig.h"

//typedef osInt32 osError;
const int kNumErrors = 70;

/**
	Enumeration of possible error codes.
*/
typedef enum {
	osErrNoErr = 0,
	osErrAssert = -10000,

	// stream errors
	osErrStreamReadError,
	osErrStreamWriteError,

	// memory errors
	osErrMemory,
	osErrInvalidMemoryObject,
	osErrOutOfMemory,
	osErrOutOfStackSpace,
	osErrCouldNotLockMemory,
	osErrCouldNotUnlockMemory,
	osErrCouldNotDisposeMemory,

	// file errors
	osErrFile,							// generic file error
	osErrFileInvalid,					// trying to work with an invalid file
	osErrFileInvalidAccessMode,			// trying to access a file in a mode that it doesn't support
	osErrCouldNotCreateOrOpenFile,		// could not create or open the file
	osErrCouldNotCloseFile,				// could not close the file
	osErrCouldNotDeleteFile,			// could not delete the file
	osErrFileNotFound,					// file does not exist
	osErrFileAlreadyExists,				// trying to create a file that already exists
	osErrFileCorrupt,					// file is corrupted
	osErrVolumeDoesNotExist,			// bad volume name
	osErrDiskFull,						// no room left on disk
	osErrDirectoryFull,					// no more items allowed inthe directory
	osErrIOError,						// I/O error
	osErrInvalidPathname,				// pathname is invalid
	osErrInvalidFilename,				// filename is invalid
	osErrFileLocked,					// file is locked
	osErrFileUnlocked,					// file is unlocked
	osErrFileIsOpen,					// file is open
	osErrFileIsClosed,					// file is closed
	osErrBOF,							// beginning of file reached
	osErrEOF,							// end of file reached
	osErrCouldNotWriteToFile,			// unable to complete a write operation to the file
	osErrCouldNotReadFromFile,			// unable to complete a read operation from the file

	// resource errors
	osErrResNotFound,
	osErrResource,

	// networking errors
	osErrNetwork,						// unknown networking error
	
	// security violation errors
	osErrInvalidUsername,				// the username supplied for the action is not valid
	osErrInvalidPassword,				// the password supplied for the action is not valid
	osErrInvalidDatabase,				// the database name supplied is not valid
	osErrNoPermission, 					// general permissions error
	osErrFieldRestriction, 				// the specified action is restricted
	
	// contiguous web errors
	osErrWebAddError,
	osErrWebUpdateError,
	osErrWebDeleteError,
		
	// general errors
	osErrInvalidParameter,				// an invalid parameter was passed to a function
	osErrOverflow,						// some memory passed to a function was too small to hold the results
	osErrNilPointer,					// a pointer was nil when it shouldn't have been
	osErrUnknownError,					// default when none of the cross-platform errors seem to fit

	osErrLoopAborted,			// a looping tag was aborted; all looping tags must catch this exception
	osErrSyntaxError,			// bad syntax used in a format file; parsing of the file was aborted	
	
	osErrDivideByZero,					
	osErrIllegalInstruction,
	osErrTagNotFound,
	osErrVarNotFound,

	osErrAborted,
	osErrFailure,

	osErrPreconditionFailed,
	osErrPostconditionFailed,
	osErrCriteriaNotMet,
	
	osErrIllegalUseOfFrozenInstance,

	// NOTE: All errors that aren't contiguous with the above errors
	//		 should be added below.  Also, any new error messages in the continguous
	//		 area should be added between osErrAssert and osErrUnknownError.

	// non contiguous web errors (backwards compatible)
	osErrWebRequiredFieldMissing  = -800,		// value missing for required field for Add
	osErrWebRepeatingRelatedField = -801,		// Adding repeating related fields isn't supported
	osErrWebNoSuchObject = -1728,				// same error number as errAENoSuchObject
	osErrWebTimeout = -1712,					// same error number as errAETimeout
	osErrWebActionNotSupported = -802,
	osErrConnectionInvalid = -609,			// same error number as connectionInvalid
	osErrLicenseInvalid = -610,				// same error number as connectionInvalid
	osErrWebModuleNotFound = -2000,
	osErrHTTPFileNotFound = 404,
	osErrDatasourceError = -3000				// third-party generic datasource error
	
} osError;


#endif
