<?LassoScript

	auth_admin;
	
	// [Admin_Object]
	// Parent type for helper types in admin that represent datasources, hosts, databases, tables, fields, etc.
	// 
	// [Admin_Object]
	//   -Database='database name' (required)
	//   -Table='table name' (required)
	//   -Keyfield='keyfield name' (required)
	//   -Keyvalue='keyvalue' (optional, if specified table is searched to fill map)
	//   -Readonly=(array) (optional list of readonly 

	define_type: 'admin_object', 'map', -prototype;
		
		local: 'database';
		local: 'table';
		local: 'keyfield';
		local: 'keyvalue';
		local: 'readonly';
		
		define_tag: 'onCreate',
				-optional='database',
				-optional='table',
				-optional='keyfield',
				-optional='keyvalue',
				-optional='readonly';
		
			// Remove extraneous map entries
			iterate: params, (local: 'param');
				if: #param->(isa: 'pair');
					self->(remove: #param->first);
				/if;
			/iterate;

			if: ((local: 'database') != null);
				self->'database' = #database;
			/if;

			if: ((local: 'table') != null);
				self->'table' = #table;
			/if;

			if: ((local: 'keyfield') != null);
				self->'keyfield' = #keyfield;
			/if;
			
			if: (local: 'readonly')->(isa: 'array');
				self->'readonly' = (local: 'readonly');
			else: !(self->'readonly'->(isa: 'array'));
				self->'readonly' = (array);
			/if;
			self->'readonly'->(insert: #keyfield);

			if: ((local: 'keyvalue') != null);
				self->'keyvalue' = (local: 'keyvalue');
				self->refresh;
			else;
				self->'keyvalue' = null;
			/if;
			
		/define_tag;
		
		define_tag: 'onConvert';
			return: self->parent;
		/define_tag;
		
		define_tag: 'onCompare', -required='value';
			if: #value === null;
				return: 1;
			else: (self->'keyvalue' != null) && (#value->(isa: 'integer') || #value->(isa: 'string'));
				if: (#value == self->'keyvalue');
					return: 0;
				else: (#value > self->'keyvalue');
					return: 1;
				else;
					return -1;
				/if;
			else;
				return: (self->parent == #value);
			/if;
		/define_tag;
		
		define_tag: 'refresh';
			local: 'inline' = (array:
					-search='action',
					-database=self->'database', 
					-table=self->'table', 
					-keyfield=self->'keyfield',
					-keyvalue=self->'keyvalue',
					-maxrecords=1);
			inline: -log=$__startup_loglevel__, #inline;
				if: (found_count == 1);
					iterate: (field_names), (local: 'field');
						self->(insert: #field = (string: (field: #field)));
					/iterate;
				/if;
			/inline;
			self->spruce;
		/define_tag;

		define_tag: 'store';
			local: 'inline' = (array:
					-update='action',
					-database=self->'database', 
					-table=self->'table', 
					-keyfield=self->'keyfield',
					-keyvalue=self->'keyvalue');
			iterate: self->keys, (local: 'key');
				if: #key == 'modified';
					#inline->(insert: #key = date->(format: '%Q %T'));
					loop_continue;
				else: #key == 'id_site';
					#inline->(insert: #key = site_id);
					loop_continue;
				else: #key == self->'keyfield';
					loop_continue;
				else: (self->'readonly' >> #key);
					loop_continue;
				/if;
				#inline->(insert: #key = self->(find: #key));
			/iterate;
			inline: -log=$__startup_loglevel__, #inline;
				if: (found_count == 1);
					iterate: (field_names), (local: 'field');
						self->(insert: #field = (string: (field: #field)));
					/iterate;
					self->spruce;
				/if;
			/inline;
		/define_tag;
		
		define_tag: 'delete';
			local: 'inline' = (array:
					-delete='action',
					-database=self->'database', 
					-table=self->'table', 
					-keyfield=self->'keyfield',
					-keyvalue=self->'keyvalue');
			inline: -log=$__startup_loglevel__, #inline;
			/inline;
		/define_tag;
		
		define_tag: 'spruce';
			// Placeholder for type specific extensions
		/define_tag;
	
	/define_type;

	define_tag: 'admin_listobjects',
			-required='database',
			-required='table',
			-required='keyfield',
			-optional='query';
		
		if: !(local_defined: 'query');
			local: 'query' = array;
		/if;
		
		local: 'output' = (array);
		
		inline: -log=$__startup_loglevel__, #query, 
				-search, 
				-database=#database, 
				-table=#table, 
				-keyfield=#keyfield, 
				-maxrecords='all';
			local: 'fields' = (field_names);
			protect;
				local: 'keyindex' = #fields->(findindex: #keyfield)->first;
			/protect;
			iterate: records_array, (local: 'record');
				local: 'temp' = (admin_object: -database=#database, -table=#table, -keyfield=#keyfield);
				iterate: #fields, (local: 'field');
					#temp->(insert: #field = (string: #record->(get: loop_count)));
				/iterate;
				protect;
					#temp->'keyvalue' = @#record->(get: #keyindex);
				/protect;
				#temp->spruce;
				#output->(insert: #temp);
			/iterate;
		/inline;
		
		return: @#output;
		
	/define_tag;

	define_type: 'admin_datasource', 'admin_object', 'map', -prototype;
	
		define_tag: 'onCreate',
				-optional='id';
			self->'database' = 'lasso_internal';
			self->'table' = 'security_datasources';
			self->'keyfield' = 'id';
			self->'keyvalue' = (local: 'id');
			if: ((local: 'id') != null);
				self->refresh;
			/if;
			self->'readonly'->(insert: 'name');
		/define_tag;
		
		define_tag: 'spruce';
			local: 'connectornames' = _admin_defaultconnectornames;
			if: #connectornames >> self->(find: 'name');
				self->(insert: 'label' = #connectornames->(find: self->(find: 'name')));
			else: self->(find: 'name') == '';
				self->(insert: 'label' = 'N/A');
			else;
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='global_prefs', -op='eq', 'store_key'='_admin_connector_alias:' + self->(find: 'name'), -search, -maxrecords=1;
					if: (found_count == 1);
						self->(insert: 'label' = (string: (field: 'data')));
					else;
						self->(insert: 'label' = self->(find: 'name'));
					/if;
				/inline;
			/if;
			self->'readonly'->(insert: 'label');
			
			if: self->(find: 'enabled') == 'Y';
				self->(insert: 'status' = 'Enabled');
			else;
				self->(insert: 'status' = 'Disabled');
			/if;
			self->'readonly'->(insert: 'status');
		/define_tag;
		
		define_tag: 'hosts',
				-optional='query';
	
			if: !(local_defined: 'query');
				local: 'query' = array;
			/if;
			
			#query->(insert: -op='eq');
			#query->(insert: 'id_datasource' = self->(find: 'id'));
			
			local: 'output' = (array);
			
			inline: -log=$__startup_loglevel__, #query, 
					-search, 
					-database='lasso_internal', 
					-table='security_datasource_databases', 
					-keyfield='id', 
					-maxrecords='all';
				local: 'fields' = (field_names);
				protect;
					local: 'keyindex' = #fields->(findindex: 'id')->first;
				/protect;
				iterate: records_array, (local: 'record');
					local: 'temp' = (admin_host);
					iterate: #fields, (local: 'field');
						#temp->(insert: #field = (string: #record->(get: loop_count)));
					/iterate;
					protect;
						#temp->'keyvalue' = @#record->(get: #keyindex);
					/protect;
					#temp->'datasource_object' = @self;
					#temp->spruce;
					#output->(insert: #temp);
				/iterate;
			/inline;
			
			return: @#output;
		/define_tag;
	
	/define_type;

	define_type: 'admin_host', 'admin_object', 'map', -prototype;
	
		local: 'datasource_object' = null;

		define_tag: 'onCreate',
				-optional='id';
			self->'database' = 'lasso_internal';
			self->'table' = 'security_datasource_hosts';
			self->'keyfield' = 'id';
			self->'keyvalue' = (local: 'id');
			if: ((local: 'id') != null);
				self->refresh;
			/if;
		/define_tag;
		
		define_tag: 'spruce';
			select: self->(find: 'id');
				case: '1';
					self->(insert: 'label' = 'Default');
				case;
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='global_prefs', -op='eq', 'store_key'='_admin_host_alias:' + self->(find: 'name'), -search, -maxrecords=1;
						if: (found_count == 1);
							self->(insert: 'label' = (string: (field: 'data')));
						else;
							self->(insert: 'label' = self->(find: 'name'));
						/if;
					/inline;
			/select;
			self->'readonly'->(insert: 'label');
			
			if: self->(find: 'enabled') == 'Y';
				self->(insert: 'status' = 'Enabled');
			else;
				self->(insert: 'status' = 'Disabled');
			/if;
			self->'readonly'->(insert: 'status');
		/define_tag;
		
		define_tag: 'datasource';
			if: (self->'datasource_object' == null);
				self->'datasource_object' = (admin_datasource: -id=self->(find: 'id_datasource'));
			/if;
			return: @self->'datasource_object';
		/define_tag;

		define_tag: 'validdatabase';
			local: 'valid' = (_database_names: self->datasource->(find: 'id'), self->(find: 'id'));
			iterate: self->databases, (local: 'database');
				if: (#database->(find: 'enabled') >> 'Y') && (#valid >> #database->(find: 'name'));
					return: #database->(find: 'alias');
				/if;
			/iterate;
			return: '';
		/define_tag;

		define_tag: 'databases',
				-optional='query';
	
			if: !(local_defined: 'query');
				local: 'query' = array;
			/if;
			
			#query->(insert: -op='eq');
			#query->(insert: 'id_host' = self->(find: 'id'));
			
			local: 'output' = (array);
			
			inline: -log=$__startup_loglevel__, #query, 
					-search, 
					-database='lasso_internal', 
					-table='security_datasource_databases', 
					-keyfield='id', 
					-maxrecords='all';
				local: 'fields' = (field_names);
				protect;
					local: 'keyindex' = #fields->(findindex: 'id')->first;
				/protect;
				iterate: records_array, (local: 'record');
					local: 'temp' = (admin_database);
					iterate: #fields, (local: 'field');
						#temp->(insert: #field = (string: #record->(get: loop_count)));
					/iterate;
					protect;
						#temp->'keyvalue' = @#record->(get: #keyindex);
					/protect;
					#temp->'host_object' = @self;
					#temp->spruce;
					#output->(insert: #temp);
				/iterate;
			/inline;
			
			return: @#output;
			
		/define_tag;
		
		define_tag: 'createdatabase',
				-required='name',
				-optional='enabled';

			local: 'id' = -1;
	
			fail_if: (#name == ''), -1, 'Please specify a database name.';
			fail_if: ((string_replaceregexp: #name, -find='\\w|\\$', -replace='') != ''), -1, 'Database names should consist of only letters, numbers, and the underscore character.';
	
			local: 'host' = self->(find: 'id');
			local: 'datasource' = self->datasource->(find: 'id');
			local: 'type' = self->datasource->(find: 'name');
	
			if: !(local_defined: 'enabled');
				local: 'enabled' = 'Y';
			/if;
			
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -maxrecords='all',
					-opbegin='or',
						-op='eq', 'alias' = #name,
						-op='eq', 'name' = #name,
					-opend='or';
				fail_if: (found_count > 0), -1, 'That database name "' + #name + '" already exists';
			/inline;

			select: #type;
				case: 'sqliteconnector';
							
					sqlite_createdb: #name;
				
				case: 'mysqlds';
				
					local: 'host_alias' = self->validdatabase;
					fail_if: (#host_alias == ''), -1, 'In order to create a new database, one database on the host must be enabled.';
					inline: -database=#host_alias, -sql='CREATE DATABASE `' + #name + '`';
						fail_if: (error_msg != error_noerror), -1, '' + error_msg + ' (' + error_code + ').';
					/inline;
					
				case;
				
					fail: -1, 'The data source "' + #type + '" is not supported.';
		
			/select;
			
			inline: -log=$__startup_loglevel__, -add, 
					-database='lasso_internal', 
					-table='security_datasource_databases',
					'id_datasource' = #datasource,
					'id_host' = self->(find: 'id'),
					'name' = #name,
					'alias' = '',
					'enabled' = #enabled,
					'modified' = date->(format: '%Q %T');
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='SELECT last_insert_rowid() AS id';
					local: 'id' = (field: 'id');
				/inline;
			/inline;
			
			_datasource_reload: (integer: #datasource);
			_security_flushcaches;
			
			return: #id;
			
		/define_tag;
		
	/define_type;

	define_type: 'admin_database', 'admin_object', 'map', -prototype;
	
		local: 'host_object' = null;
		local: 'datasource_object' = null;

		define_tag: 'onCreate',
				-optional='id';
			self->'database' = 'lasso_internal';
			self->'table' = 'security_datasource_databases';
			self->'keyfield' = 'id';
			self->'keyvalue' = (local: 'id');
			if: ((local: 'id') != null);
				self->refresh;
			/if;
			self->'readonly'->(insert: 'name');
		/define_tag;
		
		define_tag: 'spruce';			
			if: self->(find: 'enabled') == 'Y';
				if: (_database_names: self->(find: 'id_datasource'), self->(find: 'id_host'))->(find: self->(find: 'name')) === (array);
					self->(insert: 'status' = 'Not Available');
				else;
					self->(insert: 'status' = 'Enabled');
				/if;
			else: self->(find: 'enabled') == 'N';
				self->(insert: 'status' = 'Disabled');
			else;
				self->(insert: 'status' = 'New');
			/if;
			self->'readonly'->(insert: 'status');
			if: (self->(find: 'alias') == '');
				self->(insert: 'alias'=self->(find: 'name'));
				self->(insert: 'label'=self->(find: 'name'));
			else: (self->(find: 'alias') == self->(find: 'name'));
				self->(insert: 'label'=self->(find: 'name'));
			else;
				self->(insert: 'label'=self->(find: 'name') + ' (' + self->(find: 'alias') + ')');
			/if;
			self->'readonly'->(insert: 'label');
		/define_tag;
								
		define_tag: 'host';
			if: (self->'host_object' == null);
				self->'host_object' = (admin_host: -id=self->(find: 'id_host'));
			/if;
			return: @self->'host_object';
		/define_tag;
								
		define_tag: 'datasource';
			if: (self->'datasource_object' == null);
				self->'datasource_object' = (admin_datasource: -id=self->(find: 'id_datasource'));
			/if;
			return: @self->'datasource_object';
		/define_tag;
		
		define_tag: 'tables',
				-optional='query';
	
			if: !(local_defined: 'query');
				local: 'query' = array;
			/if;
			
			#query->(insert: -op='eq');
			#query->(insert: 'id_database' = self->(find: 'id'));
			
			local: 'output' = (array);
			
			inline: -log=$__startup_loglevel__, #query, 
					-search, 
					-database='lasso_internal', 
					-table='security_database_tables', 
					-keyfield='id', 
					-maxrecords='all';
				local: 'fields' = (field_names);
				protect;
					local: 'keyindex' = #fields->(findindex: 'id')->first;
				/protect;
				iterate: records_array, (local: 'record');
					local: 'temp' = (admin_table);
					iterate: #fields, (local: 'field');
						#temp->(insert: #field = (string: #record->(get: loop_count)));
					/iterate;
					protect;
						#temp->'keyvalue' = @#record->(get: #keyindex);
					/protect;
					#temp->'database_object' = @self;
					#temp->spruce;
					#output->(insert: #temp);
				/iterate;
			/inline;
			
			return: @#output;
			
		/define_tag;

		define_tag: 'createtable',
				-required='name',
				-optional='enabled',
				-optional='sql';
			
			local: 'id' = -1;
	
			fail_if: (#name == ''), -1, 'Please specify a table name.';
			fail_if: ((string_replaceregexp: #name, -find='\\w|\\$', -replace='') != ''), -1, 'Table names should consist of only letters, numbers, and the underscore character.';
	
			local: 'datasource' = self->datasource->(find: 'id');
			local: 'type' = self->datasource->(find: 'name');
			local: 'database_alias' = self->(find: 'alias');
			
			if: !(local_defined: 'enabled');
				local: 'enabled' = 'Y';
			/if;
			
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -maxrecords='all',
					-op='eq', 'id_database' = self->(find: 'id'),
					-opbegin='or',
						-op='eq', 'alias' = #name,
						-op='eq', 'name' = #name,
					-opend='or';
				fail_if: (found_count > 0), -1, 'That table name "' + #name + '" already exists';
			/inline;
					
			select: #type;
				case: 'sqliteconnector';
						
					if: !(local_defined: 'sql');
						local: 'sql' = 'CREATE TABLE ' + #name + ' (ID INTEGER PRIMARY KEY)';
					/if;
					
					inline: -database=#database_alias, -sql=#sql;
						fail_if: (error_msg != error_noerror), -1, '' + error_msg + ' (' + error_code + ').';
					/inline;
				
				case: 'mysqlds';

					if: !(local_defined: 'sql');
						local: 'sql' = 'CREATE TABLE `' + #name + '` (`ID` BIGINT PRIMARY KEY AUTO_INCREMENT)';
					/if;

					inline: -database=#database_alias, -sql=#sql;
						fail_if: (error_msg != error_noerror), -1, '' + error_msg + ' (' + error_code + ').';
					/inline;
					
				case;
					fail: -1, 'The data source "' + #type + '" is not supported.';
			/select;
			
			inline: -log=$__startup_loglevel__, -add, -database='lasso_internal', -table='security_database_tables', -keyfield='id',
					'id_database' = self->(find: 'id'),
					'name' = #name,
					'alias' = '',
					'enabled' = #enabled,
					'encoding' = 'utf-8',
					'modified' = date->(format: '%Q %T');
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='SELECT last_insert_rowid() AS id';
					local: 'id' = (field: 'id');
				/inline;
			/inline;
			
			_datasource_reload: (integer: #datasource);
			_security_flushcaches;
			
			return: #id;
			
		/define_tag;
		
	/define_type;

	define_type: 'admin_table', 'admin_object', 'map', -prototype;
	
		local: 'database_object' = null;
		local: 'host_object' = null;
		local: 'datasource_object' = null;

		define_tag: 'onCreate',
				-optional='id';
			self->'database' = 'lasso_internal';
			self->'table' = 'security_database_tables';
			self->'keyfield' = 'id';
			self->'keyvalue' = (local: 'id');
			if: ((local: 'id') != null);
				self->refresh;
			/if;
			self->'readonly'->(insert: 'name');
		/define_tag;
		
		define_tag: 'spruce';			
			if: self->(find: 'enabled') == 'Y';
				self->(insert: 'status' = 'Enabled');
			else: self->(find: 'enabled') == 'N';
				self->(insert: 'status' = 'Disabled');
			else;
				self->(insert: 'status' = 'New');
			/if;
			self->'readonly'->(insert: 'status');
			
			if: (self->(find: 'alias') == '');
				self->(insert: 'alias'=self->(find: 'name'));
				self->(insert: 'label'=self->(find: 'name'));
			else: (self->(find: 'alias') == self->(find: 'name'));
				self->(insert: 'label'=self->(find: 'name'));
			else;
				self->(insert: 'label'=self->(find: 'name') + ' (' + self->(find: 'alias') + ')');
			/if;
			self->'readonly'->(insert: 'label');
		/define_tag;
		
		define_tag: 'database';
			if: (self->'database_object' == null);
				self->'database_object' = (admin_database: -id=self->(find: 'id_database'));
			/if;
			return: @self->'database_object';
		/define_tag;
								
		define_tag: 'host';
			if: (self->'host_object' == null);
				self->'host_object' = @self->table->host;
			/if;
			return: @self->'host_object';
		/define_tag;
								
		define_tag: 'datasource';
			if: (self->'datasource_object' == null);
				if: (self->'host_object' != null);
					self->'datasource_object' = @self->'host_object'->datasource;
				else;
					self->'datasource_object' = @self->database->datasource;
				/if;
			/if;
			return: @self->'datasource_object';
		/define_tag;

		define_tag: 'fields',
				-optional='query',
				-optional='useschema';
	
			if: !(local_defined: 'query');
				local: 'query' = array;
			/if;
			
			#query->(insert: -op='eq');
			#query->(insert: 'id_table' = self->(find: 'id'));
			
			if: (local_defined: 'useschema');
				local: 'schema' = self->schema;
			else;
				inline: -log=$__startup_loglevel__, -database=self->database->(find: 'alias'), -table=self->(find: 'name'), -sql='SELECT count(*) FROM ' + self->(find: 'name'), -maxrecords=1;
					// Force Table To Load
				/inline;
			/if;
						
			local: 'output' = (array);

			inline: -log=$__startup_loglevel__, #query, 
					-search, 
					-database='lasso_internal', 
					-table='security_table_fields', 
					-keyfield='id', 
					-maxrecords='all';

				local: 'fields' = (field_names);
				protect;
					local: 'keyindex' = #fields->(findindex: 'id')->first;
				/protect;

				iterate: records_array, (local: 'record');

					local: 'temp' = (admin_field);

					iterate: #fields, (local: 'field');
						#temp->(insert: #field = (string: #record->(get: loop_count)));
					/iterate;

					protect;
						#temp->'keyvalue' = @#record->(get: #keyindex);
					/protect;

					#temp->'table_object' = @self;
					if: (local_defined: 'useschema') && (local_defined: 'schema') && (#schema->keys >> #temp->(find: 'name'));
						iterate: #schema->(find: #temp->(find: 'name')), (local: 'schema_temp');
							#temp->(insert: #schema_temp->first = #schema_temp->second);
							#temp->'readonly'->(insert: #schema_temp->first);
						/iterate;
					/if;
					#temp->spruce;
					#output->(insert: #temp);

				/iterate;
			/inline;
			
			return: @#output;
			
		/define_tag;

		define_tag: 'types';

			local: 'output' = map;

			local: 'datasource_type' = self->datasource->(find: 'name');

			select: #datasource_type;
			
				case: 'sqliteconnector';

					local: 'output' = (array:
						(map: 'type'='BLOB', 'name'='BLOB (Binary Data)'),
						(map: 'type'='CHAR', 'name'='CHAR (Character Data)'),
						(map: 'type'='FLOAT', 'name'='FLOAT (Numeric Data)'),
						(map: 'type'='INTEGER', 'name'='INTEGER (Numeric Data)'),
						(map: 'type'='TEXT', 'name'='TEXT (Character Data)'),
					);

				case: 'mysqlds';

					local: 'output' = (array:
						(map: 'type'='TINYINT', 'name'='TINYINT (Integer 8 Bit)', 'option'='Unsigned'),
						(map: 'type'='SMALLINT', 'name'='SMALLINT (Integer 16 Bit)', 'option'='Unsigned'),
						(map: 'type'='MEDIUMINT', 'name'='MEDIUMINT (Integer 24 Bit)', 'option'='Unsigned'),
						(map: 'type'='INT', 'name'='INT (Integer 32 Bit)', 'option'='Unsigned'),
						(map: 'type'='BIGINT', 'name'='BIGINT (Integer 64 Bit)', 'option'='Unsigned'),
						(map: 'type'='FLOAT', 'name'='FLOAT (Decimal 32 Bit)'),
						(map: 'type'='DOUBLE', 'name'='DOUBLE (Decimal 64 Bit)'),
						(map: 'type'='DECIMAL', 'name'='DECIMAL (Fixed Decimal)'),
						(map: 'type'='CHAR', 'name'='CHAR (String Fixed Max 255 Chars)', 'option'='Size,Binary'),
						(map: 'type'='VARCHAR', 'name'='VARCHAR (String Max 255 Chars)', 'option'='Size,Binary'),
						(map: 'type'='DATE', 'name'='DATE (%Y-%m-%d)'),
						(map: 'type'='TIME', 'name'='TIME (%H:%M:%S)'),
						(map: 'type'='DATETIME', 'name'='DATETIME (%Y-%m-%d %H:%M:%S)'),
						(map: 'type'='TIMESTAMP', 'name'='TIMESTAMP (Auto-Enter)'),
						(map: 'type'='ENUM', 'name'='ENUM (Value From List)'),
						(map: 'type'='SET', 'name'='SET (Values From List)'),
						(map: 'type'='TINYTEXT', 'name'='TINYTEXT (String 255 Chars)'),
						(map: 'type'='TEXT', 'name'='TEXT (String 64K Chars)'),
						(map: 'type'='MEDIUMTEXT', 'name'='MEDIUMTEXT (String 16M Chars)'),
						(map: 'type'='LONGTEXT', 'name'='LONGTEXT (String 4GB Chars)'),
						(map: 'type'='TINYBLOB', 'name'='TINYBLOB (Binary Data 255 Bytes)'),
						(map: 'type'='BLOB', 'name'='BLOB (Binary Data 64K Bytes)'),
						(map: 'type'='MEDIUMBLOB', 'name'='MEDIUMBLOB (Binary Data 16M Bytes)'),
						(map: 'type'='LONGBLOB', 'name'='LONGBLOB (Binary Data 4G Bytes)'),
					);

				case;
				
			/select;
				
			return: @#output;

		/define_tag;
		
		define_tag: 'indices';

			local: 'output' = array;
			
			local: 'database_alias' = self->database->(find: 'alias');
			local: 'datasource_type' = self->datasource->(find: 'name');
			select: #datasource_type;
			
				case: 'sqliteconnector';
					inline: -log=$__startup_loglevel__, -database=#database_alias, -table=self->(find: 'name'), -sql='SELECT count(*) FROM ' + self->(find: 'name'), -maxrecords=1;
						// Force Table To Load
					/inline;
					inline: -log=$__startup_loglevel__, -database=#database_alias, -sql='PRAGMA index_list(\'' + self->(find: 'name') + '\')';
						records;
							local: 'temp_schema' = (map:
								'name' = '',
								'fields' = (array),
								'options' = (array),
							);
							#temp_schema->(insert: 'name' = (field: 'name'));
							if: ((field: 'name') >> 'autoindex');
								#temp_schema->(insert: 'alias' = 'AUTOINDEX');
							else;
								#temp_schema->(insert: 'alias' = (field: 'name'));
							/if;
							
							local: 'options' = (array);
							if: (integer: (field: 'unique')) > 0;
								#temp_schema->(find: 'options')->(insert: 'UNIQUE');
							/if;
							
							inline: -log=$__startup_loglevel__, -database=#database_alias, -sql='PRAGMA index_info(\'' + (field: 'name') + '\')', -maxrecords='all';
								records;
									#temp_schema->(find: 'fields')->(insert: (field: 'name'));	
								/records;
							/inline;
							
							#output->(insert: #temp_schema);
						/records;
					/inline;

				case: 'mysqlds';
				
					inline: -log=$__startup_loglevel__, -database=#database_alias, -sql='SHOW INDEXES FROM `' + self->(find: 'name') + '`', -maxrecords='all';
						records;
							local: 'temp_schema' = (map:
								'name' = (field: 'key_name'),
								'alias' = (field: 'key_name'),
								'fields' = (array: (field: 'column_name')),
								'part' = (field: 'sub_part'),
								'options' = (array),
							);
							local: 'options' = (array);
							if: (integer: (field: 'non_unique')) == 0;
								#temp_schema->(find: 'options')->(insert: 'UNIQUE');
							/if;
							#output->(insert: #temp_schema);
						/records;
					/inline;
					
				case;
				
			/select;
				
			return: @#output;

		/define_tag;

		define_tag: 'schema';
			local: 'output' = map;
			
			local: 'database_alias' = self->database->(find: 'alias');
			local: 'datasource_type' = self->datasource->(find: 'name');

			select: #datasource_type;
				case: 'sqliteconnector';
				
					inline: -log=$__startup_loglevel__, -database=#database_alias, -table=self->(find: 'name'), -sql='SELECT count(*) FROM ' + self->(find: 'name'), -maxrecords=1;
						// Force Table To Load
					/inline;
					inline: -log=$__startup_loglevel__, -database=#database_alias, -sql='PRAGMA table_info(' + self->(find: 'name') + ')', -maxrecords='all';
						records;
							local: 'temp_schema' = (map:
								'name' = (field: 'name'),
								'type' = '',
								'display' = '',
								'options' = (array),
								'default' = (field: 'dflt_value'),
							);
							if: (field: 'type') >> '(';
								#temp_schema->(insert: 'type' = (string_replaceregexp: (field: 'type'), -find='.*?(\\w+).*', -replace='\\1'));
								#temp_schema->(insert: 'display' = (string_replaceregexp: (field: 'type'), -find='.*?\\(([^\\)]*)\\).*', -replace='\\1'));
							else;
								#temp_schema->(insert: 'type' = (string_replaceregexp: (field: 'type'), -find='.*?(\\w+).*', -replace='\\1'));
								#temp_schema->(insert: 'display' = '');
							/if;
							if: ((integer: (field: 'notnull')) > 0) && (#temp_schema->(find: 'options') !>> 'NOT NULL');
								#temp_schema->(find: 'options')->(insert: 'NOT NULL');
							/if;
							if: ((integer: (field: 'pk')) > 0) && (#temp_schema->(find: 'options') !>> 'PRIMARY KEY');
								#temp_schema->(find: 'options')->(insert: 'PRIMARY KEY');
							/if;

							#output->(insert: (field: 'name') = #temp_schema);
						/records;
					/inline;

				case: 'mysqlds';
				
					inline: -log=$__startup_loglevel__, -database=#database_alias, -table=self->(find: 'name'), -sql='SELECT count(*) FROM `' + self->(find: 'name') + '`', -maxrecords=1;
						// Force Table To Load
					/inline;

					inline: -log=$__startup_loglevel__, -database=#database_alias, -sql='SHOW FULL COLUMNS FROM `' + self->(find: 'name') + '`', -maxrecords='all';

						records;

							local: 'temp_schema' = (map:
								'name' = (field: 'field'),
								'type' = '',
								'display' = '',
								'options' = (array),
								'default' = (field: 'default'),
							);

							if: (field: 'type') >> '(';
								#temp_schema->(insert: 'type' = (string_replaceregexp: (field: 'type'), -find='.*?(\\w+).*', -replace='\\1'));
								#temp_schema->(insert: 'display' = (string_replaceregexp: (field: 'type'), -find='.*?\\(([^\\)]*)\\).*', -replace='\\1'));
							else;
								#temp_schema->(insert: 'type' = (string_replaceregexp: (field: 'type'), -find='.*?(\\w+).*', -replace='\\1'));
								#temp_schema->(insert: 'display' = '');
							/if;

							local: 'options' = (array);
							if: ((field: 'type') >> 'unsigned');
								#temp_schema->(find: 'options')->(insert: 'UNSIGNED');
							/if;
							if: ((field: 'collation') != null) && ((field: 'collation') >> '_bin');
								#temp_schema->(find: 'options')->(insert: 'BINARY');
							else: ((field: 'type') >> 'BINARY');
								#temp_schema->(find: 'options')->(insert: 'BINARY');
							/if;
							if: ((field: 'null') !>> 'YES') && (#temp_schema->(find: 'options') !>> 'NOT NULL');
								#temp_schema->(find: 'options')->(insert: 'NOT NULL');
							/if;
							if: ((field: 'extra') >> 'auto_increment') && (#temp_schema->(find: 'options') !>> 'AUTO_INCREMENT');
								#temp_schema->(find: 'options')->(insert: 'AUTO_INCREMENT');
							/if;
							if: ((field: 'key') >> 'PRI') && (#temp_schema->(find: 'options') !>> 'PRIMARY KEY');
								#temp_schema->(find: 'options')->(insert: 'PRIMARY KEY');
							/if;

							if: ((field: 'collation') != null);
								#temp_schema->(insert: 'collation' = (field: 'collation'));
							/if;

							#output->(insert: (field: 'field') = #temp_schema);
						/records;
					/inline;
					
			/select;

			return: @#output;

		/define_tag;
		
		define_tag: 'copydata', -required='database', -required='table', -optional='direct';

			local: 'datasource_type' = self->datasource->(find: 'name');
			local: 'database_alias' = self->database->(find: 'alias');
			local: 'database_name' = self->database->(find: 'name');
			local: 'table_alias' = self->(find: 'alias');
			local: 'table_name' = self->(find: 'name');

			if: (#datasource_type == 'FMRemoteDataSource') || (#datasource_type == 'FMServerAdvancedDataSource') || (#datasource_type == 'FMServer9DataSource');
				local: 'datasource_type' = 'filemaker';
			/if;

			select: #datasource_type;
				case: 'sqliteconnector';
		
					inline: -log=$__startup_loglevel__, -findall, -database=#database, -table=#table, -maxrecords='all';

						local: 'prefix' = 'INSERT INTO ' + (encode_sql92: #table_name) + ' (';
						local: 'fields' = field_names;
						iterate: #fields, (local: 'field');
							#prefix += '\'' + (encode_sql92: (string_replaceregexp: #field, -find='[: .]', -replace='_')) + ((loop_count != #fields->size) ? '\',' | '\'');
						/iterate;
						#prefix += ') VALUES (\'';
						local: 'suffix' = '\'); ';
						local: 'join' = '\', \'';
						local: 'count' = 0;
						local: 'sql' = '';
					
						iterate: records_array, (local: 'record');
							#sql += #prefix;
							iterate: #record, (local: 'field');
								#sql += (encode_sql92: #field) + (loop_count != #record->size ? #join | '');
							/iterate;
							#sql += #suffix;
							#count += 1;
							if: #count >= 100;
								inline: -log=$__startup_loglevel__, -database=#database_alias, -table=#table_alias, -sql='BEGIN TRANSACTION; ' + #sql + 'COMMIT TRANSACTION;', -maxrecords=0; 
								/inline;
								local: 'count' = 0;
								local: 'sql' = '';
							/if;
						/iterate;
						if: #sql != '';
							inline: -log=$__startup_loglevel__, -database=#database_alias, -table=#table_alias, -sql='BEGIN TRANSACTION; ' + #sql + 'COMMIT TRANSACTION;', -maxrecords=0; 
							/inline;
						/if;
					
					/inline;
		
				case: 'mysqlds';
				
					if: (local_defined: 'direct');
					
						local: 'sql' = 'INSERT INTO  `' + (encode_sql: #table_name) + '` SELECT * FROM `' + (encode_sql: #database) + '`.`' + (encode_sql: #table) + '`';
						inline: -log=$__startup_loglevel__, -database=#database_alias, -table=#table_alias, -sql=#sql, -maxrecords=0;
						/inline;
					
					else;
					
						inline: -log=$__startup_loglevel__, -findall, -database=#database, -table=#table, -maxrecords='all';

							local: 'prefix' = 'INSERT INTO `' + (encode_sql: #table_name) + '` (';
							local: 'fields' = field_names;
							iterate: #fields, (local: 'field');
								#prefix += '`' + (encode_sql92: (string_replaceregexp: #field, -find='[: .]', -replace='_')) + ((loop_count != #fields->size) ? '`,' | '`');
							/iterate;
							#prefix += ') VALUES (\'';
							local: 'suffix' = '\'); ';
							local: 'join' = '\',\'';
							local: 'count' = 0;
							local: 'sql' = '';
							
							iterate: records_array, (local: 'record');
								#sql += #prefix;
								iterate: #record, (local: 'field');
									#sql += (encode_sql: #field) + (loop_count != #record->size ? #join | '');
								/iterate;
								#sql += #suffix;
								#count += 1;
								if: #count >= 100;
									inline: -log=$__startup_loglevel__, -database=#database_alias, -table=#table_alias, -sql=#sql, -maxrecords=0; 
									/inline;
									local: 'count' = 0;
									local: 'sql' = '';
								/if;
							/iterate;
							if: #sql != '';
								inline: -log=$__startup_loglevel__, -database=#database_alias, -table=#table_alias, -sql=#sql, -maxrecords=0; 
								/inline;
							/if;
						
						/inline;
					
					/if;

				case;

					inline: -log=$__startup_loglevel__, -show, -database=#database_alias, -table=#table_alias, -maxrecords='all';
						local: 'field_names' = field_names;
					/inline;
					
					// Throttle
					local: 'found_count' = 1;
					local: 'skiprecords' = 0;
					local: 'maxrecords' = 100;
					
					while: #skiprecords < #found_count;
						inline: -log=$__startup_loglevel__, -findall, -database=#database, -table=#table, -maxrecords=#maxrecords, -skiprecords=#skiprecords;
							records;
								local: 'inline' = (array);
								iterate: #field_names, (local: 'field');
									if: (#field != '__record_id__');
										#inline->(insert: #field = (field: #field));
									else;
										#inline->(insert: #field = (keyfield_value));
									/if;
								/iterate;
								inline: -log=$__startup_loglevel__, -add, -database=#database_alias, -table=#table_alias, #inline, -maxrecords=0; 
								/inline;
							/records;
							local: 'found_count' = found_count;
							#skiprecords += #maxrecords;
						/inline;
					/while;
				
			/select;
		
		/define_tag;
		
		define_tag: 'sql', -optional='target', -optional='newname', -optional='usedata';

			local: 'sql' = '';
			local: 'datasource_type' = self->datasource->(find: 'name');
			local: 'database_alias' = self->database->(find: 'alias');
			local: 'database_name' = self->database->(find: 'name');
			local: 'table_alias' = self->(find: 'alias');
			local: 'table_name' = self->(find: 'name');

			if: !(local_defined: 'target');
				local: 'target' = #datasource_type;
			/if;

			if: (#datasource_type == 'FMRemoteDataSource') || (#datasource_type == 'FMServerAdvancedDataSource') || (#datasource_type == 'FMServer9DataSource');
				local: 'datasource_type' = 'filemaker';
			/if;

			if: (#target == 'sqliteconnector') && (#target == #datasource_type);
		
				inline: -log=$__startup_loglevel__, -database=#database_name, -table=#table_name, -sql='SELECT tbl_name,sql FROM sqlite_master WHERE tbl_name LIKE \'' + (encode_sql92: #table_name) + '\' AND type=\'table\'', -maxrecords='all';
					local: 'sql' = (string_replaceregexp: (field: 'sql'), -find=" DEFAULT '([, ])", -replace=" DEFAULT ''\\1");
				/inline;

			else: (#target == 'mysqlds') && (#target == #datasource_type);
		
				inline: -log=$__startup_loglevel__, -database=#database_name, -table=#table_name, -sql='SHOW CREATE TABLE ' + (encode_sql: #table_name), -maxrecords='all';
					local: 'sql' = (field: 'create table');
				/inline;

			else: (#datasource_type == 'filemaker') && (#target == 'mysqlds') && (local_defined: 'usedata');

				inline: -log=$__startup_loglevel__, -findall, -database=#database_name, -table=#table_name, -maxrecords=1;
					local: 'sql' = 'CREATE TABLE `' + (encode_sql: ((local: 'newname') != '' ? (local: 'newname') | #table_name)) + '` (';
	
					local: 'need_recordid' = (#datasource_type == 'filemaker');
					loop: (field_names: -count);
						local: 'field_name' = (encode_sql: (string_replaceregexp: (field_name: loop_count), -find='[: .]', -replace='_'));
						local: 'format' = '';

						local: 'repeat_count' = 0;
						repeating: (field_name: loop_count);
							#repeat_count += 1;
						/repeating;

						if: #field_name == '__Record_ID__';
							#sql += ' `' + #field_name + '` BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT,';
							local: 'need_recordid' = false;
						else: #repeat_count > 1;
							#sql += ' `' + #field_name + '` TEXT,';
							var: 'format' = 'string';
						else: (field_name: loop_count, -type) >> 'number';
							if: (string: (integer: (field: (field_name: loop_count)))) == (field: (field_name: loop_count));
								#sql += ' `' + #field_name + '` INTEGER,';
							else;
								#sql += ' `' + #field_name + '` DOUBLE,';
							/if;
						else: (field_name: loop_count, -type) >> 'date/time';
							protect;
								if: (date: (field: (field_name: loop_count)), -format='%D') != '';
									local: 'format' = 'date';
								/if;
							/protect;
							protect;
								if: (#format == '') && (date: (field: (field_name: loop_count)), -format='%T') != '';
									local: 'format' = 'time';
								/if;
							/protect;
							if: #format == 'date';
								#sql += ' `' + #field_name + '` DATE,';
							else: #format == 'time';
								#sql += ' `' + #field_name + '` TIME,';
							else;
								#sql += ' `' + #field_name + '` TEXT,';
							/if;
						else;
							#sql += ' ' + #field_name + ' TEXT,';
						/if;
					/loop;
					if: #need_recordid == true;
						#sql += ' `__Record_ID__` BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT';
					else;
						#sql->(removetrailing: ',');
					/if;
					#sql += ')';
				/inline;

			else: (#target == 'sqliteconnector');

				inline: -log=$__startup_loglevel__, -show, -database=#database_name, -table=#table_name, -maxrecords='all';
					local: 'sql' = 'CREATE TABLE ' + (encode_sql92: ((local: 'newname') != '' ? (local: 'newname') | #table_name)) + ' (';
	
					local: 'need_recordid' = (#datasource_type == 'filemaker');
					iterate: field_names, (local: 'field_name');
						if: #field_name == '__Record_ID__';
							#sql += ' ' + (encode_sql92: (string_replaceregexp: #field_name, -find='[: .]', -replace='_')) + ' INTEGER PRIMARY KEY,';
							local: 'need_recordid' = false;
						else;
							#sql += ' ' + (encode_sql92: (string_replaceregexp: #field_name, -find='[: .]', -replace='_')) + ' TEXT,';
						/if;
					/iterate;
					if: #need_recordid == true;
						#sql += ' __Record_ID__ INTEGER PRIMARY KEY';
					else;
						#sql->(removetrailing: ',');
					/if;
					#sql += ')';
				/inline;

			else: (#target == 'mysqlds');

				inline: -log=$__startup_loglevel__, -show, -database=#database_name, -table=#table_name, -maxrecords='all';
					local: 'sql' = 'CREATE TABLE `' + (encode_sql: ((local: 'newname') != '' ? (local: 'newname') | #table_name)) + '` (';
	
					local: 'need_recordid' = (#datasource_type == 'filemaker');
					iterate: field_names, (local: 'field_name');
						if: #field_name == '__Record_ID__';
							#sql += ' `' + (encode_sql: (string_replaceregexp: #field_name, -find='[: .]', -replace='_')) + '` BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT,';
							local: 'need_recordid' = false;
						else;
							#sql += ' `' + (encode_sql: (string_replaceregexp: #field_name, -find='[: .]', -replace='_')) + '` TEXT,';
						/if;
					/iterate;
					if: #need_recordid == true;
						#sql += ' `__Record_ID__` BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT';
					else;
						#sql->(removetrailing: ',');
					/if;
					#sql += ')';
				/inline;
			/if;
			
			if: ((local: 'newname') != '') && (#table_name != #newname);
				if: ((local: 'target') == 'sqliteconnector');
					#sql->(replace: 'CREATE TABLE ' + #table_name, 'CREATE TABLE ' + #newname);
				else: ((local: 'target') == 'mysqlds');
					#sql->(replace: 'CREATE TABLE `' + #table_name + '`', 'CREATE TABLE `' + #newname + '`');
				/if;
			/if;

			return: #sql;
			
		/define_tag;

	/define_type;

	define_tag: 'build_createtable',
			-required='database',
			-required='name',
			-optional='enabled';
			
		local: 'id' = -1;

		fail_if: (#name == ''), -1, 'Please specify a table name.';
		fail_if: ((string_replaceregexp: #name, -find='\\w|\\$', -replace='') != ''), -1, 'Table names should consist of only letters, numbers, and the underscore character.';

		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, -maxrecords=1;
			local: 'datasource' = (field: 'id_datasource');
			local: 'database_alias' = (field: 'alias');
			if: (#database_alias == '');
				local: 'database_alias' = (field: 'name');
			/if;
		/inline;
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasources', -keyfield='id', -keyvalue=#datasource, -maxrecords=1, -returnfield='name';
			local: 'type' = (field: 'name');
		/inline;
		
		if: !(local_defined: 'enabled');
			local: 'enabled' = 'Y';
		/if;
		
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -maxrecords='all',
				-op='eq', 'id_database' = #database,
				-opbegin='or',
					-op='eq', 'alias' = #name,
					-op='eq', 'name' = #name,
				-opend='or';
			fail_if: (found_count > 0), -1, 'That table name "' + #name + '" already exists';
		/inline;
				
		select: #type;
			case: 'sqliteconnector';
						
				inline: -database=#database_alias, -sql='CREATE TABLE ' + #name + ' (ID INTEGER PRIMARY KEY)';
					fail_if: (error_msg != error_noerror), -1, '' + error_msg + ' (' + error_code + ').';
				/inline;
			
			case: 'mysqlds';
							
				inline: -database=#database_alias, -sql='CREATE TABLE `' + #name + '` (`ID` BIGINT PRIMARY KEY AUTO_INCREMENT)';
					fail_if: (error_msg != error_noerror), -1, '' + error_msg + ' (' + error_code + ').';
				/inline;
				
			case;
				fail: -1, 'The data source "' + #type + '" is not supported.';
		/select;
		
		inline: -log=$__startup_loglevel__, -add, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id',
				'id_database' = #database,
				'name' = #name,
				'alias' = '',
				'enabled' = #enabled,
				'modified' = date->(format: '%Q %T');
			inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='SELECT last_insert_rowid() AS id';
				local: 'id' = (field: 'id');
			/inline;
		/inline;
		
		_datasource_reload: (integer: #datasource);
		_security_flushcaches;
		
		return: #id;
		
	/define_tag;

	define_type: 'admin_field', 'admin_object', 'map', -prototype;
	
		local: 'table_object' = null;
		local: 'database_object' = null;
		local: 'host_object' = null;
		local: 'datasource_object' = null;
		
		define_tag: 'onCreate',
				-optional='id';
			self->'database' = 'lasso_internal';
			self->'table' = 'security_table_fields';
			self->'keyfield' = 'id';
			self->'keyvalue' = (local: 'id');
			if: ((local: 'id') != null);
				self->refresh;
			/if;
			self->'readonly'->(insert: 'name');
		/define_tag;
		
		define_tag: 'spruce',
				-optional='extra';			
			if: self->(find: 'enabled') == 'Y';
				self->(insert: 'status' = 'Enabled');
			else: self->(find: 'enabled') == 'N';
				self->(insert: 'status' = 'Disabled');
			else;
				self->(insert: 'status' = 'New');
			/if;
			self->'readonly'->(insert: 'status');
			
			if: (self->(find: 'alias') == '') || (self->(find: 'alias') == self->(find: 'name'));
				self->(insert: 'label'=self->(find: 'name'));
			else;
				self->(insert: 'label'=self->(find: 'name') + ' (' + self->(find: 'alias') + ')');
			/if;
			self->'readonly'->(insert: 'label');
			
			self->(insert: 'index'=self->(find: 'name'));
			self->'readonly'->(insert: 'index');
			
			select: self->datasource->(find: 'name');
				case: 'mysqlds';
					if: self->(find: 'type') >> 'TEXT';
						self->(insert: 'index'=self->(find: 'name') + ' (255)');
					/if;	
				case;
			/select;
			
		/define_tag;
		
		define_tag: 'table';
			if: (self->'table_object' == null);
				self->'table_object' = (admin_table: -id=self->(find: 'id_table'));
			/if;
			return: @self->'table_object';
		/define_tag;
				
		define_tag: 'database';
			if: (self->'database_object' == null);
				self->'database_object' = @self->table->database;
			/if;
			return: @self->'database_object';
		/define_tag;
								
		define_tag: 'host';
			if: (self->'host_object' == null);
				self->'host_object' = @self->table->host;
			/if;
			return: @self->'host_object';
		/define_tag;
								
		define_tag: 'datasource';
			if: (self->'datasource_object' == null);
				if: (self->'host_object' != null);
					self->'datasource_object' = @self->'host_object'->datasource;
				else;
					self->'datasource_object' = @self->database->datasource;
				/if;
			/if;
			return: @self->'datasource_object';
		/define_tag;
		
	/define_type;

	define_tag: 'admin_default',
			-required='array',
			-optional='value';
		if: (#array->size == 0) || !(#array->(isa: 'array'));
			return;
		else: (local_defined: 'value');
			iterate: #array, (local: 'temp');
				if: #temp == #value;
					return: @#temp;
				/if;
			/iterate;
		/if;
		return: @#array->first;
	/define_tag;
	
	define_tag: 'admin_datasourcecascade',
			-optional='datasource',
			-optional='host',
			-optional='database',
			-optional='table',
			-optional='field',
			-optional='render',
			-optional='prefix',
			-optional='suffix',
			-optional='label',
			-optional='sqlonly',
			-optional='buildonly',
			-optional='datasourceonly';
			
		if: !(local_defined: 'render');
			local: 'render' = 'table';
		/if;
		if: !(local_defined: 'prefix');
			local: 'prefix' = '';
		/if;
		if: !(local_defined: 'suffix');
			local: 'suffix' = '';
		/if;
		if: !(local_defined: 'label');
			local: 'label' = '';
		/if;
		if: !(local_defined: 'class');
			if: (local_defined: 'forheader');
				local: 'class' = 'std_pop_medium';
			else: (local_defined: 'forsidebar');
				local: 'class' = 'sidebar';
			else;
				local: 'class' = 'std_pop_large';
			/if;
		/if;
		
		if: (local_defined: 'forheader') && !(local_defined: 'notabletag');
			local: 'notabletag' = true;
		else: (local_defined: 'forsidebar') && !(local_defined: 'notabletag');
			local: 'notabletag' = true;
		/if;
		
		if: (local_defined: 'forheader');
			local: 'output_pre' = '&nbsp;' + #label;
			local: 'output_mid' = ':&nbsp';
			local: 'output_end' = '';
		else: (local_defined: 'forsidebar');
			local: 'output_pre' = '<p><b>' + #label;
			local: 'output_mid' = '</b><br />';
			local: 'output_end' = '</p>';
		else;
			local: 'output_pre' = '<tr><th width="35%">' + #label;
			local: 'output_mid' = '</th><td>';
			local: 'output_end' = '</td></tr>';
		/if;
		
		local: 'output' = '';
		if: !(local_defined: 'notabletag');
			#output += '<table>';
		/if;
		local: 'error' = '';

		if: (local_defined: 'datasource');
			// Nothing
		else: (local_defined: 'host');
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host, -maxrecords=1, -returnfield='id_datasource';
				local: 'datasource' = (field: 'id_datasource');
			/inline;
		else: (local_defined: 'database');
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, -maxrecords=1, -returnfield='id_host', -returnfield='id_datasource';
				local: 'datasource' = (field: 'id_datasource');
				local: 'host' = (field: 'id_host');
			/inline;
		else: (local_defined: 'table');
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -keyvalue=#table, -maxrecords=1, -returnfield='id_database';
				local: 'database' = (field: 'id_database');
			/inline;
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, -maxrecords=1, -returnfield='id_datasource', -returnfield='id_datasource';
				local: 'datasource' = (field: 'id_datasource');
				local: 'host' = (field: 'id_host');
			/inline;
		else: (local_defined: 'field');
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_table_fields', -keyfield='id', -keyvalue=#field, -maxrecords=1, -returnfield='id_table';
				local: 'table' = (field: 'id_table');
			/inline;
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -keyvalue=#table, -maxrecords=1, -returnfield='id_database';
				local: 'database' = (field: 'id_database');
			/inline;
			inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, -maxrecords=1, -returnfield='id_host', -returnfield='id_datasource';
				local: 'datasource' = (field: 'id_datasource');
				local: 'host' = (field: 'id_host');
			/inline;
		else;
			// Default
			local: 'datasource' = 1;
		/if;
		
		if: !(local_defined: 'datasource');
			local: 'datasource' = 0;
		/if;
		if: !(local_defined: 'host');
			local: 'host' = 0;
		/if;
		if: !(local_defined: 'database');
			local: 'database' = 0;
		/if;
		if: !(local_defined: 'table');
			local: 'table' = 0;
		/if;
		if: !(local_defined: 'field');
			local: 'field' = 0;
		/if;

		local: 'inline_params' = (array: -op='eq', 'enabled'='Y');
		if: (local_defined: 'datasourceonly');
			local: 'inline_params' = (array:
				-op='eq', 'id'=#datasource,
			);
		else: (local_defined: 'sqlonly');
			local: 'inline_params' = (array:
				-opbegin='and',
					-op='neq', 'name'='SpotlightDS',
					-op='neq', 'name'='FMRemoteDataSource',
					-op='neq', 'name'='FMServerAdvancedDataSource',
					-op='neq', 'name'='FMServer9DataSource',
				-opend='and',
			);
		else: (local_defined: 'buildonly');
			local: 'inline_params' = (array:
				-opbegin='or',
					-op='eq', 'name'='SQLiteConnector',
					-op='eq', 'name'='MySQLDS',
				-opend='and',
			);
		else;
			local: 'inline_params' = (array:
				-op='neq', 'name'='SpotlightDS',
			);
		/if;

		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasources', -keyfield='id', #inline_params, -maxrecords='all';
			
			// Host Count
			local: 'hosts' = map;
			inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='security_datasource_hosts', -sql='SELECT id_datasource, COUNT(id) AS cnt FROM security_datasource_hosts GROUP BY id_datasource', -maxrecords='all';
				records;
					#hosts->(insert: (field: 'id_datasource') = (field: 'cnt'));
				/records;
			/inline;
			local: 'found_count' = #hosts->size;
			
			if: (#found_count == 0);
				local: 'datasource' = 0;
				local: 'host' = 0;
				local: 'database' = 0;
				local: 'table' = 0;
				local: 'field' = 0;
				if: !(local_defined: 'nodatasource') && (#error == '');
					local: 'error' = 'No Datasources Found.';
				/if;
			else: !(local_defined: 'nodatasource');
				local: 'found' = false;
				#output += #output_pre + 'Datasource' + #output_mid;
				
				// Look up labels
				local: 'labels' = _admin_defaultconnectornames;				
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='global_prefs', -op='bw', 'store_key'='_admin_connector_alias:', -search, -maxrecords='all';
					records;
						#labels->(insert: ((field: 'store_key') - '_admin_connector_alias:') = (string: (field: 'data')));
					/records;
				/inline;
				if: (local_defined: 'forsidebar');
					iterate: #labels->keys, (local: 'key');
						#labels->(find: #key) -= 'Lasso Connector for ';
					/iterate;
				/if;

				#output += '<select name="' + #prefix + 'datasource' + #suffix + '"' + ((local_defined: 'nodatasourcerefresh') ? '' | ' onchange="this.form.submit()"') + ' class="' + #class + '">';
				records;
					if: #hosts->(find: (field: 'id')) <= 0;
						loop_continue;
					else: (field: 'id') == (local: 'datasource');
						#found = true;
						#output += '<option value="' + (field: 'id') + '" selected="selected">' + (if_null: #labels->(find: (field: 'name')), (field: 'name')) + '</option>';
					else;
						#output += '<option value="' + (field: 'id') + '">' + (if_null: #labels->(find: (field: 'name')), (field: 'name')) + '</option>';
					/if;
				/records;
				#output += '</select>';
				#output += #output_end;
				if: (#found == false) || ((local: 'datasource') <= 0) || ((local: 'datasource') == '');
					local: 'datasource' = (field: 'id');
				/if;
			/if;
		/inline;
		
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -op='eq', 'id_datasource'=#datasource, -op='eq', 'enabled'='Y', -maxrecords='all';
			if: (found_count == 0);
				local: 'host' = 0;
				local: 'database' = 0;
				local: 'table' = 0;
				local: 'field' = 0;
				if: !(local_defined: 'nohost') && (#error == '');
					local: 'error' = 'No Hosts Found.';
				/if;
			else: !(local_defined: 'nohost');
				local: 'found' = false;
				#output += #output_pre + 'Host' + #output_mid;

				// Look up labels
				local: 'labels' = (map: 1 = 'Internal');				
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='global_prefs', -op='bw', 'store_key'='_admin_host_alias:', -search, -maxrecords='all';
					records;
						#labels->(insert: ((field: 'store_key') - '_admin_host_alias:') = (string: (field: 'data')));
					/records;
				/inline;

				#output += '<select name="' + #prefix + 'host' + #suffix + '"' + ((local_defined: 'nohostrefresh') ? '' | ' onchange="this.form.submit()"') + ' class="' + #class + '">';
				records;
					if: (field: 'id') == (local: 'host');
						#found = true;
						#output += '<option value="' + (field: 'id') + '" selected="selected">' + (if_null: #labels->(find: (field: 'id')), (field: 'name')) + '</option>';
					else;
						#output += '<option value="' + (field: 'id') + '">' + (if_null: #labels->(find: (field: 'id')), (field: 'name')) + '</option>';
					/if;
				/records;
				#output += '</select>';
				#output += #output_end;
				if: (#found == false) || ((local: 'host') <= 0) || ((local: 'host') == '');
					local: 'host' = (field: 'id');
				/if;
			/if;
		/inline;
		
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -op='eq', 'id_host'=#host, -op='eq', 'enabled'='Y', -maxrecords='all', ((#datasource == 1) ? (array: -opbegin='not', -op='ew', 'name'='-journal', -opend='not', -op='neq', 'name'=':memory:'));
			if: (found_count == 0);
				local: 'database' = 0;
				local: 'table' = 0;
				local: 'field' = 0;
				if: !(local_defined: 'nodatabase') && (#error == '');
					local: 'error' = 'No Databases Found.';
				/if;
			else: !(local_defined: 'nodatabase');
				local: 'found' = false;
				#output += #output_pre + 'Database' + #output_mid;
				#output += '<select name="' + #prefix + 'database' + #suffix + '"' + ((local_defined: 'nodatabaserefresh') ? '' | ' onchange="this.form.submit()"') + ' class="' + #class + '">';
				records;
					if: (field: 'id') == (local: 'database');
						#found = true;
						#output += '<option value="' + (field: 'id') + '" selected="selected">' + (field: 'name') + '</option>';
					else;
						#output += '<option value="' + (field: 'id') + '">' + (field: 'name') + '</option>';
					/if;
				/records;
				#output += '</select>';
				#output += #output_end;
				if: (#found == false) || ((local: 'database') <= 0) || ((local: 'database') == '');
					local: 'database' = (field: 'id');
				/if;
			/if;
		/inline;
		
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -op='eq', 'id_database'=#database, -op='eq', 'enabled'='Y', -maxrecords='all';
			if: (found_count == 0);
				local: 'table' = 0;
				local: 'field' = 0;
				if: !(local_defined: 'notable') && (#error == '');
					local: 'error' = 'No Tables Found.';
				/if;
			else: !(local_defined: 'notable');
				local: 'found' = false;
				#output += #output_pre + 'Table' + #output_mid;
				#output += '<select name="' + #prefix + 'table' + #suffix + '"' + ((local_defined: 'notablerefresh') ? '' | ' onchange="this.form.submit()"') + ' class="' + #class + '">';
				records;
					if: (field: 'id') == (local: 'table');
						#found = true;
						#output += '<option value="' + (field: 'id') + '" selected="selected">' + (field: 'name') + '</option>';
					else;
						#output += '<option value="' + (field: 'id') + '">' + (field: 'name') + '</option>';
					/if;
				/records;
				#output += '</select>';
				#output += #output_end;
				if: (#found == false) || ((local: 'table') <= 0) || ((local: 'table') == '');
					local: 'table' = (field: 'id');
				/if;
			/if;
		/inline;
		
		inline: -log=$__startup_loglevel__, -search, -database='lasso_internal', -table='security_table_fields', -keyfield='id', -op='eq', 'id_table'=#table, -op='eq', 'enabled'='Y', -maxrecords='all';
			if: (found_count == 0);
				local: 'field' = 00;
				if: !(local_defined: 'nofield') && (#error == '');
					local: 'error' = 'No Fields Found.';
				/if;
			else: !(local_defined: 'nofield');
				local: 'found' = false;
	 			if: (local_defined: 'keyfield');
					#output += #output_pre + 'Key Field' + #output_mid;
				else;
					#output += #output_pre + 'Field' + #output_mid;
				/if;
				#output += '<select name="' + #prefix + 'field' + #suffix + '"' + ((local_defined: 'nofieldrefresh') ? '' | ' onchange="this.form.submit()"') + ' class="' + #class + '">';
				records;
					if: (field: 'id') == (local: 'field');
						#found = true;
						#output += '<option value="' + (field: 'id') + '" selected="selected">' + (field: 'name') + '</option>';
					else;
						#output += '<option value="' + (field: 'id') + '">' + (field: 'name') + '</option>';
					/if;
				/records;
				#output += '</select>';
				#output += #output_end;
				if: (#found == false) || ((local: 'field') <= 0) || ((local: 'field') == '');
					local: 'field' = (field: 'id');
				/if;
			/if;
		/inline;
		
		if: !(local_defined: 'notabletag');
			#output += '</table>';
		/if;
		
		if: !(local_defined: 'noerror') && (#error != '');
			if: (local_defined: 'forheader');
				#output += '<span class="error">' + #error + '</span>';
			else: (local_defined: 'forsidebar');
				#output += '<p style="color: red"><b>' + #error + '</b></p>';
			else: !(local_defined: 'notabletag');
				#output += '<p class="error">' + #error + '</p><p>&nbsp;</p>';
			else;
				#output += '<tr><td colspan="2"><p class="error">' + #error + '</p></td></tr>';
			/if;
		/if;
		
		return: @#output;
	/define_tag;	

	// [Variable_Create]
	//
	// [Variable_Create: -name='variable name', -value='variable value', -session='session name']
	// Creates the specified variable with the specified value and adds to a session
	//
	// [Variable_Create: -name='variable name', -param, -cookie, -token, -default='default value']
	// Creates the specified variables with default value.  The first of [action_param], [cookie], or
	// [token_value] which is not-empty is used as the value for the variable.
	//
	// [Variable_Create: -name='variable name', -default='default value', -param, -session='session name']
	// Variable is created with default value and added to session, if an action param of the same name
	// is specified then the variable takes that value instead.
	
	define_tag: 'variable_create', 
			-required='name', 
			-optional='default', 
			-optional='value', 
			-optional='session', 
			-optional='token', 
			-optional='cookie', 
			-optional='param', 
			-priority='replace';
		if: (local_defined: 'session') && (#session != '');
			session_addvar: -name=#session, #name;
		/if;
		if: (local_defined: 'value');
			var: #name = (local: 'value');
			return;
		/if;
		if: ((var_defined: #name) == false) || ((var: #name) == null);
			var: #name = (local: 'default');
		/if;
		iterate: params, (local: 'param');
			select: #param;
				case: -param;
					local: 'cnt' = (action_param: #param->second, -count);
					if: #param->(isa: 'pair') && (#cnt > 0) && ((action_param: #param->second, #cnt) != '');
						var: #name = (action_param: #param->second);
						return;
					else: (#cnt > 0) && ((action_param: #name, #cnt) != '');
						var: #name = (action_param: #name, #cnt);
						return;
					/if;
				case: -cookie;
					if: #param->(isa: 'pair') && ((cookie: #param->second) != '');
						var: #name = (cookie: #param->second);
						return;
					else: ((cookie: #name) != '');
						var: #name = (cookie: #name);
						return;
					/if;
				case: -token;
					if: #param->(isa: 'pair') && ((token_value: #param->second) != '');
						var: #name = (token_value: #param->second);
						return;
					else: ((token_value: #name) != '');
						var: #name = (token_value: #name);
						return;
					/if;
			/select;
		/iterate;
	/define_tag;


?>	
