<?LassoScript

if: (action_params->size > 4);

	auth;
		
	protect;
		
	// LassoStudio.LassoApp
	
	// This file implements an XML server for Lasso Studio.
	
	// Required Inputs
	// m - The action which should be performed.
	
	// Modes
	// i - Information about the Lasso Service is returned.  Can be used as a status ping.  Default if no mode is specified.
	// d - Returns a list of databases which the current username can access.
	// t - Returns a list of tables for a databases.  Requires Database.
	// f - Returns a list of fields for a table.  Requires Database, Table.
	// q - Returns up to 10 records based on query parameters or default query.  Requires database, table.
	// v - Suppresses field value lists.  Requires database, Table.
	// x - Executes the code posted in source.
	// k - Checks the code posted in source.
	
	// Optional Inputs
	// d - The database for which information should be returned.
	// t - The table for which information should be returned.
	// f - The field for which information should be reuturned.
	// qa - The action to perform for the query.
	// qfx - Up to three query field parameters.
	// qvx - Up to three query field value parameters.
	// qox - Up to three query operator parameters.
	// qsf - Query sort field.
	// qso - Query sort order.
	// src - The source to be executed or checked.

	Variable: '_lassostudio_mode' = (Action_Param: 'm');
	
	Variable: '_lassostudio_database' = (Action_Param: 'd');
	Variable: '_lassostudio_table' = (Action_Param: 't');
	Variable: '_lassostudio_field' = (Action_Param: 'f');
	
	Variable: '_lassostudio_query_action' = (Action_Param: 'qa');
	Variable: '_lassostudio_query_field1' = (Action_Param: 'qf1');
	Variable: '_lassostudio_query_field2' = (Action_Param: 'qf2');
	Variable: '_lassostudio_query_field3' = (Action_Param: 'qf3');
	Variable: '_lassostudio_query_value1' = (Action_Param: 'qv1');
	Variable: '_lassostudio_query_value2' = (Action_Param: 'qv2');
	Variable: '_lassostudio_query_value3' = (Action_Param: 'qv2');
	Variable: '_lassostudio_query_operator1' = (Action_Param: 'qo1');
	Variable: '_lassostudio_query_operator2' = (Action_Param: 'qo2');
	Variable: '_lassostudio_query_operator3' = (Action_Param: 'qo3');
	Variable: '_lassostudio_query_sortfield' = (Action_Param: 'qsf');
	Variable: '_lassostudio_query_sortorder' = (Action_Param: 'qso');
	Variable: '_lassostudio_source' = (Action_Param: 'src');

	// The XML return value is stored in this variable.
	// Note: This is the only output from this file.
	Variable: '_lassostudio_output' = '';
	
	// The error and code variables store whether an error has occurred that prevents this file from finishing.
	Variable: '_lassostudio_error' = '';
	Variable: '_lassostudio_code' = '';

	// This is the code for mode 'info' which is included with each fragment.
	If: ($_lassostudio_error == '');
		$_lassostudio_output += '\n\t<ls_info ls_platform="' + (lasso_version: -lassoplatform) + '" ls_version="' + (lasso_version: -lassoversion) + '" ls_date="' + date + '" ls_name="Lasso Studio" ls_refresh=""></ls_info>';
	/If;
	
	handle_error;
		log_warning: 'Lasso Studio ' +  error_code + ' ' + error_msg;
	/handle_error;
	
	/protect;
	
	protect;
	
	// Dispatch based on mode.
	// Note: multiple modes can be concatenated.
	if: ($_lassostudio_mode >> 'd') || ($_lassostudio_mode >> 't') || ($_lassostudio_mode >> 'f') || ($_lassostudio_mode >> 'q') && ($_lassostudio_error == '');
					
		If: ($_lassostudio_mode >> 'd') && ($_lassostudio_error == '');
			// Set up list of databases accessible by the current username.
			Variable: '_lassostudio_count' = 0;
			Database_Names;
				// SQLite :memory: database doesn't have tables, so it's probably not
				// worth reporting back as it will usually cause a no tables found error.
				if: (database_nameitem != ':memory:');
					Variable: '_lassostudio_temp_type' = 'Unknown';
					if: ((Lasso_DatasourceModuleName: Database_NameItem) == 'mysqlds') || (Lasso_TagExists: 'Lasso_DataSourceIsMySQL') && (Lasso_DataSourceIsMySQL: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'MySQL';
					else: ((Lasso_DatasourceModuleName: Database_NameItem) == 'sqliteds') || (Lasso_TagExists: 'Lasso_DataSourceIsSQlite') && (Lasso_DataSourceIsSQlite: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'SQLite';
					else: ((Lasso_DatasourceModuleName: Database_NameItem) == 'internalmysqlds') || (Lasso_TagExists: 'Lasso_DataSourceIsLassoMySQL') && (Lasso_DataSourceIsLassoMySQL: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'Lasso MySQL';
					else: ((Lasso_DatasourceModuleName: Database_NameItem) == 'fmremotedatasource') || (Lasso_TagExists: 'Lasso_DatasourceIsFileMaker') && (Lasso_DataSourceIsFileMaker: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'FileMaker Pro';
					else: ((Lasso_DatasourceModuleName: Database_NameItem) == 'fmserveradvanceddatasource') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMakerSA') && (Lasso_DataSourceIsFileMakerSA: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'FileMaker SA';
					else: ((Lasso_DatasourceModuleName: Database_NameItem) == 'fmserver9datasource') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMaker9') && (Lasso_DataSourceIsFileMaker9: (Database_NameItem));
						Variable: '_lassostudio_temp_type' = 'FileMaker 9';
					/if;
					Variable: '_lassostudio_temp_database' = (Database_NameItem);
					$_lassostudio_output += '\n\t<ls_database ls_name="' + (encode_xml: $_lassostudio_temp_database) + '" ls_type="' + (encode_xml: $_lassostudio_temp_type) + '" ls_refresh=""></ls_database>';
					$_lassostudio_count = (loop_count);
					if: ($_lassostudio_database == '');
						if: (Database_NameItem != 'lasso_internal') && (Database_NameItem != 'site') && (Database_NameItem !=' lasso_site_1') && (Database_NameItem != 'lasso_admin');
							$_lassostudio_database = (Database_NameItem);
							$_lassostudio_mode += 't';
						/if;
					/if;
				/if;
			/Database_Names;
			
			If: ($_lassostudio_count == 0);
				Variable: '_lassostudio_error' = 'No databases are accessible to username ' + (Client_Username) + '.';
				Variable: '_lassostudio_code' = '-1';
			/If;
		/If;
		
		If: ($_lassostudio_mode >> 't') && ($_lassostudio_error == '');
			// Set up list of tables for a specific database.
			if: ($_lassostudio_database == '');
				Variable: '_lassostudio_error' = 'A database must be specified in order to get a list of tables.';
				Variable: '_lassostudio_code' = '-1';
				Fail: $_lassostudio_error, $_lassostudio_code;
			Else;
			
				Variable: '_lassostudio_count' = 0;
				$_lassostudio_output += '\n\t<ls_database ls_name="' + (encode_xml: $_lassostudio_database) + '" ls_default="true" ls_refresh="">';
				Database_TableNames: $_lassostudio_database;
					Variable: '_lassostudio_temp_table' = (Database_TableNameItem);
					$_lassostudio_output += '\n\t\t<ls_table ls_name="' + (encode_xml: $_lassostudio_temp_table) + '" ls_refresh=""></ls_table>';
					$_lassostudio_count = (loop_count);
					if: ($_lassostudio_table == '');
						$_lassostudio_table = (Database_TableNameItem);
						$_lassostudio_mode += 'f';
					/if;
				/Database_TableNames;
				$_lassostudio_output += '\n\t</ls_database>';

				If: ($_lassostudio_count == 0);
					Variable: '_lassostudio_error' = 'No tables were found within database ' + $_lassostudio_database + '.  The database may be disabled.';
					Variable: '_lassostudio_code' = '-1';
					Fail: $_lassostudio_error, $_lassostudio_code;
				/If;
			/If;
		/If;
		
		If: ($_lassostudio_mode >> 'f') && ($_lassostudio_error == '');
			// Set up list of fields for a specific table.
			if: ($_lassostudio_database == '');
				Variable: '_lassostudio_error' = 'A database and table must be specified in order to get a list of fields.';
				Variable: '_lassostudio_code' = '-1';
			else: ($_lassostudio_table == '');
				Variable: '_lassostudio_error' = 'A table must be specified in order to get a list of fields.';
				Variable: '_lassostudio_code' = '-1';
			Else;
				Variable: '_lassostudio_count' = 0;
				Variable: '_lassostudio_inline' = (array: -database=$_lassostudio_database, -table=$_lassostudio_table, -show='');
				Inline: $_lassostudio_inline;
					$_lassostudio_output += '\n\t<ls_database ls_name="' + (encode_xml: $_lassostudio_database) + '" ls_default="true" ls_refresh="">';
					$_lassostudio_output += '\n\t\t<ls_table ls_name="' + (encode_xml: $_lassostudio_table) + '" ls_default="true" ls_refresh="">';
					
					if: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmremotedatasource') || (Lasso_TagExists: 'Lasso_DatasourceIsFileMaker') && (Lasso_DataSourceIsFileMaker: (Database_NameItem));
						$_lassostudio_output += '\n\t\t\t<ls_field ls_name="{LS_TOKEN_RECORDID}" ls_type="number" ls_protection="none" ls_refresh=""></ls_field>';
					else: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmserveradvanceddatasource') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMakerSA') && (Lasso_DataSourceIsFileMakerSA: (Database_NameItem));
						$_lassostudio_output += '\n\t\t\t<ls_field ls_name="{LS_TOKEN_RECORDID}" ls_type="number" ls_protection="none" ls_refresh=""></ls_field>';
					else: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmserver9datasource') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMaker9') && (Lasso_DataSourceIsFileMaker9: (Database_NameItem));
						$_lassostudio_output += '\n\t\t\t<ls_field ls_name="{LS_TOKEN_RECORDID}" ls_type="number" ls_protection="none" ls_refresh=""></ls_field>';
					/If;
					
					Loop: (Field_Name: -Count);
						Variable: '_lassostudio_temp_field' = (Field_Name: (Loop_Count));
						Variable: '_lassostudio_temp_type' = (Field_Name: (Loop_Count), -Type);
						Variable: '_lassostudio_temp_protection' = (Field_Name: (Loop_Count), -Protection);
						Variable: '_lassostudio_temp_valuelist' = 'false';
						Value_List: (Field_Name: (Loop_Count));
							Variable: '_lassostudio_temp_valuelist' = 'true';
							Loop_Abort;
						/Value_List;
						$_lassostudio_output += '\n\t\t\t<ls_field ls_name="' + (encode_xml: $_lassostudio_temp_field) + '" ls_type="' + (encode_xml: $_lassostudio_temp_type) + '" ls_protection="' + (encode_xml: $_lassostudio_temp_protection) + '" ls_valuelist="' + (encode_xml: $_lassostudio_temp_valuelist) + '" ls_refresh="">';
						
						If: ($_lassostudio_mode !>> 'v') && ($_lassostudio_temp_valuelist == 'true');
							Value_List: (Field_Name: (Loop_Count));
								$_lassostudio_output += '\n\t\t\t\t<ls_value ls_name="' + (Encode_XML: Value_ListItem) + '" ls_refresh=""></ls_value>';
							/Value_List;
						/If;
						
						$_lassostudio_output += '\n\t\t\t</ls_field>';
						$_lassostudio_count = (loop_count);
					/Loop;
					$_lassostudio_output += '\n\t\t</ls_table>';
					$_lassostudio_output += '\n\t</ls_database>';
				/Inline;
				If: ($_lassostudio_count == 0);
					Variable: '_lassostudio_error' = 'No fields were found within table ' + $_lassostudio_table + '.  The table may be disabled.';
					Variable: '_lassostudio_code' = '-1';
				/If;
			/If;
		/If;
		
		If: ($_lassostudio_mode >> 'q') && ($_lassostudio_error == '');
			// Fetch query data for a specific database and table
			if: ($_lassostudio_database == '');
				Variable: '_lassostudio_error' = 'A database and table must be specified in order to perform a query.';
				Variable: '_lassostudio_code' = '-1';
			else: ($_lassostudio_table == '');
				Variable: '_lassostudio_error' = 'A table must be specified in order to perform a query.';
				Variable: '_lassostudio_code' = '-1';
			Else;
				Variable: '_lassostudio_inline' = (Array);
				$_lassostudio_inline->(insert: -database=$_lassostudio_database);
				$_lassostudio_inline->(insert: -table=$_lassostudio_table);
				$_lassostudio_inline->(insert: -maxrecords=10);
				$_lassostudio_inline->(insert: -operatorlogical='and');
				if: ($_lassostudio_mode >> 'v');
					if: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmremotedatasource') || (Lasso_TagExists: 'Lasso_DatasourceIsFileMaker') && (Lasso_DataSourceIsFileMaker: (Database_NameItem));
						$_lassostudio_inline->(insert: -novaluelists='');
					else: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmserveradvanceddatasource') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMakerSA') && (Lasso_DataSourceIsFileMakerSA: (Database_NameItem));
						$_lassostudio_inline->(insert: -novaluelists='');
					else: ((Lasso_DatasourceModuleName: $_lassostudio_database) == 'fmserveradvanced9') || (Lasso_TagExists: 'Lasso_DataSourceIsFileMaker9') && (Lasso_DataSourceIsFileMaker9: (Database_NameItem));
						$_lassostudio_inline->(insert: -novaluelists='');
					/If;
				/if;
				
				if: ($_lassostudio_query_action >> 'search');
					$_lassostudio_inline->(insert: -search='');

					if: $_lassostudio_query_sortfield != '';
						$_lassostudio_inline->(insert: -sortfield=$_lassostudio_query_sortfield);
						if: $_lassostudio_query_sortorder != '';
							$_lassostudio_inline->(insert: -sortorder=$_lassostudio_query_sortorder);
						/if;
					/if;

					loop: 3;
						Variable: '_lassostudio_field' = (Variable: '_lassostudio_query_field' + loop_count);
						Variable: '_lassostudio_operator' = (Variable: '_lassostudio_query_operator' + loop_count);
						Variable: '_lassostudio_value' = (Variable: '_lassostudio_query_value' + loop_count);
						
						if: ($_lassostudio_field != '');
							if: ($_lassostudio_operator != '');
								$_lassostudio_inline->(insert: -op=$_lassostudio_operator);
							/if;
							$_lassostudio_inline->(insert: $_lassostudio_field=$_lassostudio_value);
						/if;
					/loop;
					
				else; // 'findall'
					$_lassostudio_inline->(insert: -findall='');
					
					if: $_lassostudio_query_sortfield != '';
						$_lassostudio_inline->(insert: -sortfield=$_lassostudio_query_sortfield);
						if: $_lassostudio_query_sortorder != '';
							$_lassostudio_inline->(insert: -sortorder=$_lassostudio_query_sortorder);
						/if;
					/if;
					
				/if;
									
				Inline: $_lassostudio_inline;
					If: (Found_Count == 0);
						Variable: '_lassostudio_error' = 'No records found when performing query.';
						Variable: '_lassostudio_code' = '-1';
					Else: (Error_CurrentError) != (Error_NoError);
						Variable: '_lassostudio_error' = (Error_CurrentError);
						Variable: '_lassostudio_code' = (Error_CurrentError: -ErrorCode);
					Else;
						$_lassostudio_output += '\n\t<ls_database ls_name="' + (encode_xml: $_lassostudio_database) + '" ls_default="true" ls_refresh="">';
						$_lassostudio_output += '\n\t\t<ls_table ls_name="' + (encode_xml: $_lassostudio_table) + '" ls_default="true" ls_refresh="">';
						$_lassostudio_output += '\n\t\t\t<ls_query ls_name="query" ls_refresh="">';
						Records;
							$_lassostudio_output += '\n\t\t\t\t<ls_record ls_name="' + (loop_count) + '" ls_refresh="">';
							Loop: (Field_Name: -Count);
								Variable: '_lassostudio_temp_field' = (String: (Field_Name: (Loop_Count)));
								Variable: '_lassostudio_temp_value' = (String: (Field: $_lassostudio_temp_field));
								if: ($_lassostudio_temp_value->size > 256);
									$_lassostudio_temp_value = $_lassostudio_temp_value->(substring: 0, 253) + '...';
								/if;
								$_lassostudio_output += '\n\t\t\t\t\t<ls_column ls_name="' + (encode_xml: $_lassostudio_temp_field) + '" ls_value="' + (encode_xml: $_lassostudio_temp_value) + '" ls_refresh=""></ls_column>';
							/Loop;
							$_lassostudio_output += '\n\t\t\t\t</ls_record>';
						/Records;
						$_lassostudio_output += '\n\t\t\t</ls_query>';
						$_lassostudio_output += '\n\t\t</ls_table>';
						$_lassostudio_output += '\n\t</ls_database>';
					/If;
				/Inline;
			/If;
		/If;
			
	/if;
	
	handle_error;
		log_warning: 'Lasso Studio ' +  error_code + ' ' + error_msg;
	/handle_error;

	/protect;
	
	protect;
	
	If: ( ($_lassostudio_mode >> 'x') || ($_lassostudio_mode >> 'k') ) && ($_lassostudio_error == '') && ( (admin_currentgroups >> 'administrators') || (admin_currentgroups >> 'lassostudio') );

		Error_SetErrorCode: 0;
		Error_SetErrorMessage: '';

		Variable: '_lassostudio_error' = '';
		Variable: '_lassostudio_code' = '';

		if: ($_lassostudio_source == '');
		
			Variable: '_lassostudio_error' = 'No Code Found';
			Variable: '_lassostudio_code' = -1;

		else: ($_lassostudio_mode >> 'x');
			Variable: '_lassostudio_noerror' = true;
			Protect;
				Variable: '_lassostudio_result' = (Process: $_lassostudio_source, -Name='Lasso Studio Code');
				Handle_Error;
					$_lassostudio_output += '<ls_result>' + (Error_CurrentError: -ErrorCode) + ': ' + (String_ReplaceRegExp: (Error_CurrentError), -Find=' at: [pP]rocess[\\s\\S]*', -replace='') + '</ls_result>';
					Variable: '_lassostudio_noerror' = false;
				/Handle_Error;
			/Protect;
			if: $_lassostudio_noerror == true;
				$_lassostudio_output += '<ls_result>' + $_lassostudio_result + '</ls_result>';
			/if;
		else: ($_lassostudio_mode >> 'k');
			Variable: '_lassostudio_noerror' = true;
			Protect;
				Null: Tag->(Compile: $_lassostudio_source, -Name='Lasso Studio Code');
				Handle_Error;
					$_lassostudio_output += '<ls_result>' + (Error_CurrentError: -ErrorCode) + ': ' + (String_ReplaceRegExp: (Error_CurrentError), -Find=' at: [cC]ompile[\\s\\S]*', -replace='') + '</ls_result>';
					Variable: '_lassostudio_noerror' = false;
				/Handle_Error;
			/Protect;
			if: $_lassostudio_noerror == true;
				$_lassostudio_output += '<ls_result>Code compiled with no errors.</ls_result>';
			/if;
		/if;		
				
	/if;
	
	handle_error;
		log_warning: 'Lasso Studio ' +  error_code + ' ' + error_msg;
	/handle_error;

	/protect;

	If: $_lassostudio_error == '';
		Variable: '_lassostudio_error' = (Error_CurrentError);
		Variable: '_lassostudio_code' = (Error_CurrentError: -ErrorCode);
	/If;
	
	Variable: '_lassostudio_encoding' = 'iso-8859-1';
	if: (global_defined: '__encoding__');
		Variable: '_lassostudio_encoding' = (global: '__encoding__');
	/if;
	xml_serve: ('<?xml version="1.0" encoding="' + (encode_xml: $_lassostudio_encoding) + '" ?>\n\n<ls_fragment ls_version="8.0.6" lp_version="' + (encode_xml: (lasso_version: -lassoversion)) + '" ls_error="' + (encode_xml: $_lassostudio_error) + '" ls_code="' + (encode_xml: $_lassostudio_code) + '" ls_refresh="">' + $_lassostudio_output + '\n</ls_fragment>' + ('\n' * 256));
	
else;
	log_warning: 'Lasso Studio: Lasso Studio Connector Initialized.';
/if;

?>
<html>
	<head>
		<title>Lasso Studio 8</title>
	</head>
	<body>
		<div align="center">
			<p>Lasso Studio: 8.0.6</p>
			<p>Lasso Professional: [Lasso_Version: -LassoVersion]
		</div>
	</body>
</html>