<?LassoScript

//
// Define the Lasso Administration script
//

Log_Warning: 'Lasso Administration: Initializing';

// Check if the table exists
Local: 'table_exists' = false;
inline: -log=(global: '__startup_loglevel__'), -Database='lasso_admin', -SQL='SELECT * from sqlite_master WHERE type="table" AND name="admin_prefs"', -MaxRecords='all';
	if: (found_count) > 0;
		#table_exists = true;
	/if;
/inline;
if: (#table_exists == false);
	// Create table if it doesn't exist already
	var: '__need_reload__' = true;
	inline: -log=(global: '__startup_loglevel__'), -Database='lasso_admin',
		-sql='CREATE TABLE admin_prefs \
			( \
				id INTEGER, \
				type TEXT, \
				database_name TEXT, \
				table_name TEXT, \
				pref_name TEXT, \
				pref_value TEXT, \
				PRIMARY KEY (id) \
			)';
	/Inline;
/if;

//
// Define Lasso Administration Helper Tags
//

define_tag: '_admin_internaldatabase', -priority='replace';
	return: 'lasso_internal';
/define_tag;

define_tag:'_admin_refreshsecurity', -priority='replace';
	auth_admin;
	_security_flushcaches;
	var: '__admin_connectors_array' = null;
	var: '__admin_connector_id' = null;
	var: '__admin_hosts_array' = null;
	var: '__admin_host_id' = null;
	var: '__admin_databases_array' = null;
	var: '__admin_database_id' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_table_id' = null;
	var: '__admin_fields_array' = null;
	var: '__admin_groups_array' = null;
	var: '__admin_users_array' = null;
/define_tag;

define_prototype: '_admin_defaultconnectornames', (map:
		'internalmysqlds'='Lasso Connector for Lasso MySQL',
		'fmremotedatasource'='Lasso Connector for FileMaker Pro',
		'fmserveradvanceddatasource'='Lasso Connector for FileMaker SA',
		'fmserver9datasource'='Lasso Connector for FileMaker 9',
		'mysqlds'='Lasso Connector for MySQL',
		'sqliteconnector'='Lasso Internal',
		'oracle'='Lasso Connector for Oracle',
		'sqlserver'='Lasso Connector for SQL Server',
		'postgresql'='Lasso Connector for PostgreSQL',
		'odbc'='Lasso Connector for ODBC',
		'spotlightds'='Lasso Connector for Spotlight',
		'openbaseds'='Lasso Connector for OpenBase',
	);

define_tag:'_admin_listconnectors', -priority='replace';
	auth_admin;
	if: ((var: '__admin_connectors_array') != null) && ((var: '__admin_connectors_array')->size > 0);
		return: (var: '__admin_connectors_array');
	/if;

	// Find friendly connector names
	local: 'connector_map' = _admin_defaultconnectornames;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', -table='global_prefs', 'store_key'='_admin_connector_alias:', -search, -maxrecords='all';
		records;
			local: 'temp_name' = (string_replaceregexp: (string: (field: 'store_key')), -find='.*?:', -replace='');
			local: 'temp_alias' = (string: (field: 'data'));
			#connector_map->(insert: #temp_name = #temp_alias);
		/records;
	/inline;
	
	// Check for unavailable connectors (once)
	local: '__admin_unavailableconnectors' = (array);
//	local: '__admin_unavailableconnectors' = (global: '__admin_unavailableconnectors');
//	if: (#__admin_unavailableconnectors == null) && (#__admin_unavailableconnectors->size == 0);
//		local: '__admin_unavailableconnectors' = array;
//		inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasources', -op='neq', 'name'='_email_sender',
//			-returnfield='name', -MaxRecords='all';
//			records;
//				protect;
//					inline: -host=(array: -datasource=(field: 'name'));
//					/inline;
//					handle_error; 
//						#__admin_unavailableconnectors->(insert: (field: 'name'));
//					/handle_error;
//				/protect;
//			/records;
//		/inline;
//		global: '__admin_unavailableconnectors' = #__admin_unavailableconnectors;
//	/if;	

	local: 'output' = (array);
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasources', -op='neq', 'name'='_email_sender',
		-returnfield='name', -returnfield='id', -returnfield='enabled', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: #__admin_unavailableconnectors >> (field: 'name');
				#temp->(insert: 'status'='Unavailable');
			else: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#temp->(insert: 'alias'=(if_null: #connector_map->(find: (field: 'name')), (field: 'name')));
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_connectors_array' = #output;
	var: '__admin_hosts_array' = null;
	var: '__admin_databases_array' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_updateconnector', -required='connector', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_datasources', -keyfield='id', -keyvalue=#connector, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_defaultconnector', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_connectors_array') == null) || ((var: '__admin_connectors_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_connectors_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_connector_id') && ((var: '__admin_connector_id') > 0);
		local: 'temp' = (var: '__admin_connectors_array')->(find: (var: '__admin_connector_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_connectors_array')->size > 0);
		return: (var: '__admin_connectors_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_refreshconnector', -required='connector', -priority='replace';
	auth_admin;
	_datasource_reload: (integer: #connector);
/define_tag;

define_tag:'_admin_deleteconnector', -required='connector', -priority='replace';
	auth_admin;
	local: 'global_prefs' = (array);
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasources', -keyfield='id', -keyvalue=#connector;
		#global_prefs->(insert: '_admin_connector_alias: ' + (encode_sql92: (field: 'name')));
	/inline;
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasource_hosts', -op='eq', 'id_datasource'=#connector, -MaxRecords='all', -returnfield='id';
		records;
			#global_prefs->(insert: '_admin_host_alias: ' + (field: 'id'));
		/records;
	/inline;
	local: 'security_group_host_map' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_group_host_map.id FROM security_group_host_map \
				LEFT JOIN security_datasource_hosts ON security_group_host_map.id_host = security_datasource_hosts.id \
				WHERE security_datasource_hosts.id_datasource = ' + (integer: #connector);
		records;
			#security_group_host_map->(insert: (field: 'id'));
		/records;
	/inline;
	local: 'security_group_db_map' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_group_db_map.id FROM security_group_db_map \
				LEFT JOIN security_datasource_databases ON security_group_db_map.id_database = security_datasource_databases.id \
				WHERE security_datasource_databases.id_datasource = ' + (integer: #connector);
		records;
			#security_group_db_map->(insert: (field: 'id'));
		/records;
	/inline;
	local: 'security_database_tables' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_database_tables.id FROM security_database_tables \
				LEFT JOIN security_datasource_databases ON security_database_tables.id_database = security_datasource_databases.id \
				WHERE security_datasource_databases.id_datasource = ' + (integer: #connector);
		records;
			#security_database_tables->(insert: (field: 'id'));
		/records;
	/inline;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='BEGIN TRANSACTION;
			DELETE FROM global_prefs \
				WHERE store_key IN ("' + #global_prefs->(join: '","') + '");
			DELETE FROM security_group_table_map \
				WHERE id_table IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_group_db_map \
				WHERE id IN (' + #security_group_db_map->(join: ',') + ');
			DELETE FROM security_group_host_map \
				WHERE id IN (' + #security_group_host_map->(join: ',') + ');
			DELETE FROM security_table_fields \
				WHERE id_table IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_database_tables \
				WHERE id IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_datasource_databases \
				WHERE id_datasource = ' + (integer: #connector) + ';
			DELETE FROM security_datasource_hosts \
				WHERE id_datasource = ' + (integer: #connector) + ';
			DELETE FROM security_datasources \
				WHERE id = ' + (integer: #connector) + ';
			COMMIT TRANSACTION;';			
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_listhosts', -required='connector', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_hosts_array') != null) && ((var: '__admin_connector_id') == #connector);
		return: (var: '__admin_hosts_array');
	/if;

	// Find friendly host names
	local: 'host_map' = (map: 
		'1'='Lasso Internal',
	);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', -table='global_prefs', 'store_key'='_admin_host_alias:', -search, -maxrecords='all';
		records;
			local: 'temp_name' = (string_replaceregexp: (string: (field: 'store_key')), -find='.*?:', -replace='');
			local: 'temp_alias' = (string: (field: 'data'));
			#host_map->(insert: #temp_name = #temp_alias);
		/records;
	/inline;
	
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasource_hosts', -op='eq', 'id_datasource'=#connector, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			#temp->(insert: 'alias'=(if_null: #host_map->(find: (field: 'id')), ((field: 'name') + ':' + (field: 'port'))));
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_hosts_array' = #output;
	var: '__admin_connector_id' = #connector;
	var: '__admin_databases_array' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaulthost', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_hosts_array') == null) || ((var: '__admin_hosts_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_hosts_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_host_id') && ((var: '__admin_host_id') > 0);
		local: 'temp' = (var: '__admin_hosts_array')->(find: (var: '__admin_host_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_hosts_array')->size > 0);
		return: (var: '__admin_hosts_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatehost', -required='host', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletehost', -required='host', -priority='replace';
	auth_admin;
	local: 'security_group_db_map' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_group_db_map.id FROM security_group_db_map \
				LEFT JOIN security_datasource_databases ON security_group_db_map.id_database = security_datasource_databases.id \
				WHERE security_datasource_databases.id_host = ' + (integer: #host);
		records;
			#security_group_db_map->(insert: (field: 'id'));
		/records;
	/inline;
	local: 'security_database_tables' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_database_tables.id FROM security_database_tables \
				LEFT JOIN security_datasource_databases ON security_database_tables.id_database = security_datasource_databases.id \
				WHERE security_datasource_databases.id_host = ' + (integer: #host);
		records;
			#security_database_tables->(insert: (field: 'id'));
		/records;
	/inline;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='BEGIN TRANSACTION;
			DELETE FROM global_prefs \
				WHERE store_key = "' + ('_admin_host_alias:' + (integer: #host)) + '";
			DELETE FROM security_group_table_map \
				WHERE id_table IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_group_db_map \
				WHERE id IN (' + #security_group_db_map->(join: ',') + ');
			DELETE FROM security_group_host_map \
				WHERE id_host = ' + (integer: #host) + ';
			DELETE FROM security_table_fields \
				WHERE id_table IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_database_tables \
				WHERE id IN (' + #security_database_tables->(join: ',') + ');
			DELETE FROM security_datasource_databases \
				WHERE id_host = ' + (integer: #host) + ';
			DELETE FROM security_datasource_hosts \
				WHERE id = ' + (integer: #host) + ';
			COMMIT TRANSACTION;';		
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_listdatabases', -required='host', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_databases_array') != null) && ((var: '__admin_host_id') == #host);
		return: (var: '__admin_databases_array');
	/if;
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host;
		if: (string: (field: 'enabled')) >> 'Y';
			local: 'enabled' = 'Y';
		else;
			local: 'enabled' = 'N';
		/if;
	/inline;
	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_datasource_databases', -op='eq', 'id_host'=#host, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		if: #enabled >> 'Y';
			local: 'live' = (_database_names: (field: 'id_datasource'), (field: 'id_host'));
		else;
			local: 'live' = (array);
		/if;
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: #live->(find: (field: 'name')) != (array);
				if: (field: 'enabled') == 'Y';
					#temp->(insert: 'status'='Enabled');
				else: (field: 'enabled') == 'N';
					#temp->(insert: 'status'='Disabled');
				else;
					#temp->(insert: 'status'='New');
				/if;
			else;
				#temp->(insert: 'status'='Not Available');
			/if;
			if: #temp->(find: 'alias') == '';
				#temp->(insert: 'alias'=(field: 'name'));
			/if;
			if: ((field: 'alias') == '') || ((field: 'alias') == (field: 'name'));
				#temp->(insert: 'label'=(field: 'name'));
			else;
				#temp->(insert: 'label'=(field: 'name') + ' (' + (field: 'alias') + ')');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_databases_array' = #output;
	var: '__admin_host_id' = #host;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaultdatabase', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_databases_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_database_id') && ((var: '__admin_database_id') > 0);
		local: 'temp' = #array->(find: (var: '__admin_database_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatedatabase', -required='database', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletedatabase', -required='database', -priority='replace';
	auth_admin;
	local: 'security_group_table_map' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_group_table_map.id FROM security_group_table_map \
				LEFT JOIN security_database_tables ON security_group_table_map.id_table = security_database_tables.id \
				WHERE security_database_tables.id_database = ' + (integer: #database);
		records;
			#security_group_table_map->(insert: (field: 'id'));
		/records;
	/inline;
	local: 'security_table_fields' = (array);
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='SELECT security_table_fields.id FROM security_table_fields \
				LEFT JOIN security_database_tables ON security_table_fields.id_table = security_database_tables.id \
				WHERE security_database_tables.id_database = ' + (integer: #database);
		records;
			#security_table_fields->(insert: (field: 'id'));
		/records;
	/inline;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='BEGIN TRANSACTION;
			DELETE FROM security_group_table_map \
				WHERE id IN (' + #security_group_table_map->(join: ',') + ');
			DELETE FROM security_group_db_map \
				WHERE id_database = ' + (integer: #database) + ';
			DELETE FROM security_table_fields \
				WHERE id IN (' + #security_table_fields->(join: ',') + ');
			DELETE FROM security_database_tables \
				WHERE id_database = ' + (integer: #database) + ';
			DELETE FROM security_datasource_databases \
				WHERE id = ' + (integer: #database) + ';
			COMMIT TRANSACTION;';	
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_listtables', -required='database', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_tables_array') != null) && ((var: '__admin_database_id') == #database);
		return: (var: '__admin_tables_array');
	/if;

	local: 'db' = (var: '__admin_databases_array')->(find: #database);
	if: (#db != null) && (#db->size > 0);
		local: 'database_alias' = #db->(get:1)->second->(find: 'alias');
		local: 'database_status' = #db->(get:1)->second->(find: 'status');
		if: #database_status >> 'Enabled';
			inline: -log=(global: '__startup_loglevel__'), -show, -database=#database_alias; /inline;
		/if;
	/if;

	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_database_tables', -op='eq', 'id_database'=#database, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			if: #temp->(find: 'alias') == '';
				#temp->(insert: 'alias'=(field: 'name'));
			/if;
			if: ((field: 'alias') == '') || ((field: 'alias') == (field: 'name'));
				#temp->(insert: 'label'=(field: 'name'));
			else;
				#temp->(insert: 'label'=(field: 'name') + ' (' + (field: 'alias') + ')');
			/if;
			if: (local_defined: 'database_alias') && (#database_alias != '');
				#temp->(insert: 'database_alias' = #database_alias);
			/if;
			if: (local_defined: 'database_status') && (#database_status != '');
				#temp->(insert: 'database_status' = #database_status);
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_tables_array' = #output;
	var: '__admin_database_id' = #database;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaulttable', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_tables_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_table_id') && ((var: '__admin_table_id') > 0);
		local: 'temp' = #array->(find: (var: '__admin_table_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatetable', -required='table', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_database_tables', -keyfield='id', -keyvalue=#table, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletetable', -required='table', -priority='replace';
	auth_admin;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='BEGIN TRANSACTION;
			DELETE FROM security_group_table_map \
				WHERE id_table = ' + (integer: #table) + ';
			DELETE FROM security_table_fields \
				WHERE id_table = ' + (integer: #table) + ';
			DELETE FROM security_database_tables \
				WHERE id = ' + (integer: #table) + ';
			COMMIT TRANSACTION;';	
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_listfields', -required='table', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_fields_array') != null) && ((var: '__admin_table_id') == #table);
		return: (var: '__admin_fields_array');
	/if;

	local: 'tb' = (var: '__admin_tables_array')->(find: #table);
	if: (#tb != null) && (#tb->size > 0);
		local: 'database_alias' = #tb->(get:1)->second->(find: 'database_alias');
		local: 'table_alias' = #tb->(get:1)->second->(find: 'alias');
		local: 'database_status' = #tb->(get:1)->second->(find: 'database_status');
		if: #database_status >> 'Enabled';
			inline: -log=(global: '__startup_loglevel__'), -show, -database=#database_alias, -table=#table_alias; /inline;
		/if;
	/if;

	inline: -log=(global: '__startup_loglevel__'), -search, -database='lasso_internal', -table='security_table_fields', -op='eq', 'id_table'=#table, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_fields_array' = #output;
	var: '__admin_table_id' = #table;
	return: #output;
/define_tag;

define_tag:'_admin_defaultfield', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_fields_array') == null) || ((var: '__admin_fields_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_fields_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_field_id') && ((var: '__admin_field_id') > 0);
		local: 'temp' = (var: '__admin_fields_array')->(find: (var: '__admin_field_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_fields_array')->size > 0);
		return: (var: '__admin_fields_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatefield', -required='field', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_table_fields', -keyfield='id', -keyvalue=#field, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletefield', -required='field', -priority='replace';
	auth_admin;

	inline: -log=(global: '__startup_loglevel__'), -delete, -database='lasso_internal', -table='security_table_fields', -keyfield='id', -keyvalue=#field;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;

/define_tag;

define_tag:'_admin_listgroups', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_groups_array') != null);
		return: (var: '__admin_groups_array');
	/if;

	inline: -log=(global: '__startup_loglevel__'), -findall, -database='lasso_internal', -table='security_groups', -sortfield='name', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_groups_array' = #output;
	return: #output;
/define_tag;

define_tag:'_admin_defaultgroup', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_groups_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_addgroup', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	if: #params !>> 'id_site';
		#params->(insert: 'id_site'=site_id);
	/if;
	if: #params !>> 'created';
		#params->(insert: 'created'=date->(format: '%Q %T'));
	/if;
	inline: -add, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', 
			-table='security_groups', 
			#params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_updategroup', -required='group', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_groups', -keyfield='id', -keyvalue=#group, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_duplicategroup', -required='group', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	if: #params !>> 'id_site';
		#params->(insert: 'id_site'=site_id);
	/if;
	if: #params !>> 'created';
		#params->(insert: 'created'=date->(format: '%Q %T'));
	/if;
	inline: -search, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_groups', -keyfield='id', -keyvalue=#group, -maxrecords=1;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			inline: -add, 
					-log=(global: '__startup_loglevel__'), 
					-database='lasso_internal', 
					-table='security_groups', 
					-keyfield='id',
					#params;
				var: '_error' = (error_currenterror);
				var: '_code' = (error_currenterror: -errorcode);
				local: 'id' = (keyfield_value);
			/inline;
			if: #id != '';
				inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal',
						-sql='BEGIN TRANSACTION;
						INSERT INTO security_ug_map (id_user,id_group,precedence,admin,modified) \
							SELECT id_user,"' + #id + '",precedence,admin,modified FROM security_ug_map \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_db_map (id_database,id_group,allow,deny,modified) \
							SELECT id_database,"' + #id + '",allow,deny,modified FROM security_group_db_map \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_table_map (id_table,id_group,allow,deny,modified) \
							SELECT id_table,"' + #id + '",allow,deny,modified FROM security_group_table_map \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_host_map (id_host,id_group,allow,deny) \
							SELECT id_host,"' + #id + '",allow,deny FROM security_group_host_map \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_tag_map (id_tag,id_group,allow_exec,modified) \
							SELECT id_tag,"' + #id + '",allow_exec,modified FROM security_group_tag_map \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_file_perms (id_group,allow_root,allow_create,allow_read,allow_write,allow_move,allow_copy,allow_delete,allow_inspect,allow_any_extension,modified) \
							SELECT "' + #id + '",allow_root,allow_create,allow_read,allow_write,allow_move,allow_copy,allow_delete,allow_inspect,allow_any_extension,modified FROM security_group_file_perms \
								WHERE id_group = ' + (integer: #group) + '; \
						INSERT INTO security_group_generic_perms (id_group,email_allowed,schedule_allowed,modified) \
							SELECT "' + #id + '",email_allowed,schedule_allowed,modified FROM security_group_generic_perms \
								WHERE id_group = ' + (integer: #group) + '; \
						COMMIT TRANSACTION;';	
					var: '_error' = (error_currenterror);
					var: '_code' = (error_currenterror: -errorcode);
				/inline;
			/if;
		/records;
	/inline;
/define_tag;

define_tag:'_admin_deletegroup', -required='group', -priority='replace';
	auth_admin;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal',
			-sql='BEGIN TRANSACTION;
			DELETE FROM security_groups \
				WHERE id = ' + (integer: #group) + ';
			DELETE FROM security_ug_map \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_db_map \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_file_perms \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_generic_perms \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_host_map \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_table_map \
				WHERE id_group = ' + (integer: #group) + ';
			DELETE FROM security_group_tag_map \
				WHERE id_group = ' + (integer: #group) + ';
			COMMIT TRANSACTION;';	
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;


define_tag:'_admin_listusers', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_users_array') != null);
		return: (var: '__admin_users_array');
	/if;

	inline: -log=(global: '__startup_loglevel__'), -findall, -database='lasso_internal', -table='security_users', -sortfield='name', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_users_array' = #output;
	return: #output;
/define_tag;

define_tag:'_admin_defaultuser', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_users_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_adduser', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	if: #params !>> 'id_site';
		#params->(insert: 'id_site'=site_id);
	/if;
	if: #params !>> 'created';
		#params->(insert: 'created'=date->(format: '%Q %T'));
	/if;
	inline: -add, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', 
			-table='security_users', 
			#params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_updateuser', -required='user', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	inline: -update, 
			-log=(global: '__startup_loglevel__'), 
			-database='lasso_internal', -table='security_users', -keyfield='id', -keyvalue=#user, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deleteuser', -required='user', -priority='replace';
	auth_admin;
	inline: -log=(global: '__startup_loglevel__'), -database='lasso_internal', 
			-sql='BEGIN TRANSACTION;
			DELETE FROM security_users \
				WHERE id = ' + (integer: #user) + ';
			DELETE FROM security_ug_map \
				WHERE id_user = ' + (integer: #user) + ';
			COMMIT TRANSACTION;'			
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;

/define_tag;

define_tag: '_admin_servicepath', -priority='replace';
	return: (string: admin_lassoservicepath) + '/';
/define_tag;

?>
