<?lassoscript

auth_admin;

{
	//
	// Administrator User
	//
	// This section will create an administrator user, or will replace it if it is
	// missing.  Afterwards, it will flush the security cache to enable the changes.
	
	inline: -log=$__startup_loglevel__, 
			-search, 
			-database='lasso_internal', 
			-table='security_users',
			-keyfield='id',
			-op='id', 'id'=2, 
			-sortfield='id',
			-returnfield='id';
		if: (found_count) == 0;
			inline: -log=$__startup_loglevel__, 
					-add, 
					-database='lasso_internal', 
					-table='security_users',
					'id' = 2,
					'name' = 'administrator',
					'password' = '',
					'created' = date->(format: '%Q %T'),
					'id_site' = (site_id),
					'enabled' = 'Y';
				if: error_currenterror:-errorcode;
					var:'store_error'='Error creating site administrator username/password';
				/if;
			/inline;
		/if;
	/inline;
	
	inline: -log=$__startup_loglevel__, 
			-search,
			-database='lasso_internal',
			-table='security_ug_map',
			-keyfield='id',
			-op='eq', 'id_user' = 2,
			-op='eq', 'id_group' = 2,
			-returnfield='id';
		if: (found_count == 0);
			inline: -log=$__startup_loglevel__, 
					-add, 
					-database='lasso_internal', 
					-table='security_ug_map',
					'id_user' = 2,
					'id_group' = 2,
					'precedence' = 1,
					'admin' = 'Y',
					'modified' = date->(format: '%Q %T');
				if: error_currenterror:-errorcode;
					var:'store_error'='Error creating site administrator username/password';
				/if;
			/inline;
		/if;
	/inline;

	//
	// Anonymous User
	//
	// This section will fix a modified anonymous user, or will replace it if it is
	// missing.  Afterwards, it will flush the security cache to enable the changes.

	inline: -log=$__startup_loglevel__, 
		-database = 'lasso_internal',
		-sql = 'SELECT id, name, password, enabled FROM security_users WHERE id = 1';

		if: (found_count) == 0;
		
			// anonymous user was deleted
			log_warning: 'Adding missing anonymous user';

			inline: -log=$__startup_loglevel__, 
				-database = 'lasso_internal',
				-sql = 'INSERT INTO security_users (id, id_site, name, password, password_plain, created, enabled, comment) VALUES (1, ' + site_id + ', \'ANONYMOUS\', \'\', NULL, \'' + (date)->(format: '%Q %T') + '\', \'Y\', \'Any user without an explicit username and password.\')';
			/inline;

		else: ((field: 'name') != 'ANONYMOUS') || ((string: (field: 'password'))->(length) > 0) || ((field: 'enabled') != 'Y');

			// Anonymous user has been modified, so we are fixing it here.
			log_warning: 'Repairing anonymous user';

			inline: -log=$__startup_loglevel__, 
				-database = 'lasso_internal',
				-sql = 'update security_users set name = \'ANONYMOUS\', password = \'\', password_plain = NULL, enabled = \'Y\' where id = 1';
			/inline;

		/if;

	/inline;
	
	// check for the "OS Process Users" group and add it if it does not exist
	inline(-log='none', -database='lasso_internal', -table='security_groups', -op='eq', 'name'='OS Process Users', -search);
		if (found_count == 0);
			log_detail('OSProcess module automatically adding the "OS Process Users" group.');
			inline('id_site'=site_id, 'created'=date->format('%Q %T'), 'name'='OS Process Users', 'enabled'='Y', 'comment'='Automatically added by the OSProcess module. \
																	Users in this group are allowed to execute processes on this server.', -add);
			/inline;
		/if;
	/inline;

	//
	// -SQL Permissions
	//
	// This section populates the 'Q' permissions stored in security_group_host_map
	// to all of the databases and tables in security_group_database_map.  In addition,
	// Q permissions are cleansed from security_group_table_map and inserted into the
	// AnyTable entry for each database that has Q permission.
		
	// Check for the security_group_host_map and create it if it doesn't exist.
	Local: 'table_exists' = false;
	inline: -log=$__startup_loglevel__, -Database='lasso_internal', -SQL='SELECT * from sqlite_master WHERE type="table" AND name="security_group_host_map"';
		if: (found_count) > 0;
			#table_exists = true;
		/if;
	/inline;

	if: #table_exists == false;
		log_warning: 'Creating host permission table';
		// Create table if it doesn't exist already
		var: '__need_reload__' = true;
		inline: -log=$__startup_loglevel__, -Database='lasso_internal',
				-SQL='CREATE TABLE security_group_host_map \
				( \
					id INTEGER, \
					id_host INTEGER DEFAULT 0, \
					id_group INTEGER DEFAULT 0, \
					allow TEXT, \
					deny TEXT, \
					modified TEXT, \
					PRIMARY KEY (id) \
				)', 
				-maxrecords='all';
		/Inline;		
				
		// Populate security_group_host_map by inserting Q for any host that has Q permission for any databases
		local: 'allow_array' = (array);
		local: 'deny_array' = (array);
		inline: -log=$__startup_loglevel__, -Database='lasso_internal', 
				-SQL='SELECT DISTINCT map.allow,map.deny,map.id_group,dbs.id_host FROM security_group_db_map AS map LEFT JOIN security_datasource_databases AS dbs ON map.id_database=dbs.id WHERE ((map.allow LIKE "%Q%") OR (map.deny LIKE "%Q%")) AND (map.id_database <> 1)', 
				-maxrecords='all';
			records;
				local: 'temp' = (array: (field: 'id_host'), (field: 'id_group'), 'Q');
				if: ((string: (field: 'allow')) >> 'Q') && (#allow_array !>> #temp);
					#allow_array->(insert: #temp);
				/if;
				if: ((string: (field: 'deny')) >> 'Q') && (#deny_array !>> #temp);
					#deny_array->(insert: #temp);
				/if;
			/records;
		/inline;

		local: 'insert_array' = (array);
		iterate: #allow_array, (local: 'temp');
			if: (#deny_array >> #temp);
				#temp->(insert: 'Q');
				#deny_array->(removeall: #temp);
			else;
				#temp->(insert: '');
			/if;
			#insert_array->(insert: '"' + #temp->(join: '","') + '"');
		/iterate;
		iterate: #allow_array, (local: 'temp');
			#temp->(insert: '', 3);
			#insert_array->(insert: '"' + #temp->(join: '","') + '"');
		/iterate;
		
		if: #insert_array->size > 0;
			log_warning: 'Initializing host permission table';
			inline: -log=$__startup_loglevel__, -database='lasso_internal', 
					-SQL='BEGIN TRANSACTION; INSERT INTO security_group_host_map (id_host,id_group,allow,deny) VALUES (' + #insert_array->(join: '); INSERT INTO security_group_host_map (id_host,id_group,allow,deny) VALUES (') + '); COMMIT TRANSACTION;',
					-maxrecords='all';
			/inline;
		/if;
	/if;
	
	// Remove Q permission from all databases in security_group_database_map except All Databases entries.
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal',
			-table='security_group_db_map',
			-keyfield='id',
			-op='neq', 'id_database'=1,
			-opbegin='or',
				-op='cn', 'allow'='Q',
				-op='cn', 'deny'='Q',
			-op='opend',
			-maxrecords='all';
		local: 'actions' = (array);
		records;
			inline: -update,
					-log=$__startup_loglevel__, 
					-database='lasso_internal', 
					-table='security_group_db_map',
					-keyfield='id',
					-keyvalue=(field: 'id'),
					'allow' = ((string_replaceregexp: -find = ',,+', -replace = ',', (field: 'allow') - 'Q')->removeleading(',') & removetrailing(',') &),
					'deny' = ((string_replaceregexp: -find = ',,+', -replace = ',', (field: 'deny') - 'Q')->removeleading(',') & removetrailing(',') &),
					-statementonly;
				#actions->(insert: action_statement);
			/inline;
		/records;
	/inline;
	
	local: 'temp' = (array);
	iterate: #actions, (local: 'insert_temp');
		#temp->(insert: #insert_temp);
		if: (#actions->size == loop_count) || ((loop_count % 25) == 0);
			inline: -log=$__startup_loglevel__, -database='lasso_internal',
					-sql='BEGIN TRANSACTION; ' + #temp->(join: '; ') + '; COMMIT TRANSACTION;',
					-maxrecords='all';
			/inline;
			local: 'temp' = (array);
		/if;
	/iterate;

	// Remove Q permission from all tables in security_group_table_map except All Tables entries.
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal',
			-table='security_group_table_map',
			-keyfield='id',
			-op='neq', 'id_table'=1,
			-opbegin='or',
				-op='cn', 'allow'='Q',
				-op='cn', 'deny'='Q',
			-op='opend',
			-maxrecords='all';
		var: 'actions' = (array);
		records;
			inline: -update,
					-log=$__startup_loglevel__, 
					-database='lasso_internal', 
					-table='security_group_table_map',
					-keyfield='id',
					-keyvalue=(field: 'id'),
					'allow' = ((string_replaceregexp: -find = ',,+', -replace = ',', (field: 'allow') - 'Q')->removeleading(',') & removetrailing(',') &),
					'deny' = ((string_replaceregexp: -find = ',,+', -replace = ',', (field: 'deny') - 'Q')->removeleading(',') & removetrailing(',') &),
					-statementonly;
				#actions->(insert: action_statement);
			/inline;
		/records;
	/inline;

	local: 'temp' = (array);
	iterate: #actions, (local: 'insert_temp');
		#temp->(insert: #insert_temp);
		if: (#actions->size == loop_count) || ((loop_count % 25) == 0);
			inline: -log=$__startup_loglevel__, -database='lasso_internal',
					-sql='BEGIN TRANSACTION; ' + #temp->(join: '; ') + '; COMMIT TRANSACTION;',
					-maxrecords='all';
			/inline;
			local: 'temp' = (array);
		/if;
	/iterate;

	// Populate security_group_database_map based on the preferences in the host map.
	inline: -log=$__startup_loglevel__, -findall, 
			-database='lasso_internal', 
			-table='security_group_host_map', 
			-keyfield='id', 
			-maxrecords='all';
		local: 'host_map' = (map);
		records;
			if: (#host_map >> (field: 'id_host'));
				local: 'database_list' = #host_map->(find: (field: 'id_host'));
			else;
				local: 'database_list' = (array);
				inline: -log=$__startup_loglevel__, -search,
						-database='lasso_internal',
						-table='security_datasource_databases',
						-keyfield='id',
						-op='eq', 'id_host' = (field: 'id_host'),
						-maxrecords='all',
						-returnfield='id';
					records;
						#database_list->(insert: (field: 'id'));
					/records;
				/inline;
				#host_map->(insert: (field: 'id_host') = #database_list);
			/if;		

			local: 'need_allow' = ((string: (field: 'allow')) >> 'Q');
			local: 'need_deny' = ((string: (field: 'deny')) >> 'Q');
			local: 'update_list' = (array);
			
			if: #need_allow || #need_deny;
				log_warning: 'Updating host permissions for host ' + (field: 'id_host') ' and group ' + (field: 'id_group');
				if: #database_list->size > 0;
					local: 'sql' = 'SELECT id,allow,deny,id_database FROM security_group_db_map WHERE id_group=' + (field: 'id_group') + ' AND id_database in (' + #database_list->(join: ',') + ')';
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql, -maxrecords='all';
						records;
							#update_list->(insert: (field: 'id'));
							#database_list->(removeall: (field: 'id_database'));		
						/records;
					/inline;
				/if;
				if: #update_list->size > 0;
					local: 'sql' = '';
					if: #need_allow;
						#sql += 'UPDATE security_group_db_map SET allow="Q" WHERE id IN (' + #update_list->(join: ',') + ') AND (allow = "");';
						#sql += 'UPDATE security_group_db_map SET allow=(allow || ",Q") WHERE id IN (' + #update_list->(join: ',') + ') AND (allow != "") AND (allow NOT LIKE "%Q%");';
					/if;
					if: #need_deny;
						#sql += 'UPDATE security_group_db_map SET deny="Q" WHERE id IN (' + #update_list->(join: ',') + ') AND (deny = "");';
						#sql += 'UPDATE security_group_db_map SET deny=(deny || ",Q") WHERE id IN (' + #update_list->(join: ',') + ') AND (deny != "") AND (deny NOT LIKE "%Q%");';
					/if;
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql, -maxrecords='all';
					/inline;						
				/if;
				iterate: #database_list, (local: 'temp');
					if: #need_allow && #need_deny;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"Q","Q")';
					else: #need_allow;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"Q","")';
					else: #need_deny;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"","Q")';
					/if;
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql;
					/inline;						
				/iterate;
			/if;
			

		/records;
	/inline;
	
	// Add All Tables entries for all groups 
	inline: -log=$__startup_loglevel__, -findall, 
			-database='lasso_internal', 
			-table='security_groups', 
			-keyfield='id', 
			-maxrecords='all';
		records;
			// All Tables
			local: 'id_group' = (field: 'id');
			local: 'sql' = 'SELECT id FROM security_group_table_map WHERE id_group=' + (local: 'id_group') + ' AND id_table = 1';
			inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql, -maxrecords='all';
				if: (found_count > 1);
					local: 'sql' = 'DELETE FROM security_group_table_map WHERE id_group=' + (local: 'id_group') + ' AND id_table = 1';
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql;
					/inline;
				/if;
				if: (found_count == 1);
					if: (field: 'id') != '';
						local: 'sql' = '';
						#sql += 'UPDATE security_group_table_map SET allow="Q" WHERE id = ' + (field: 'id') + ' AND (allow = "");';
						#sql += 'UPDATE security_group_table_map SET allow=(allow || ",Q") WHERE id = ' + (field: 'id') + ' AND (allow != "") AND (allow NOT LIKE "%Q%");';
					/if;
				else;
					local: 'sql' = 'INSERT INTO security_group_table_map (id_group,id_table,allow,deny) VALUES (' + (local: 'id_group') + ',1,"Q","")';
				/if;
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql;
				/inline;
			/inline;
		/records;
	/inline;
	
	// Clean up any empty entries.
	inline: -log=$__startup_loglevel__, -database='lasso_internal', 
			-SQL='DELETE FROM security_group_host_map WHERE ((allow = "") OR (allow ISNULL)) AND ((deny = "") OR (deny ISNULL))';
	/inline;
	inline: -log=$__startup_loglevel__, -database='lasso_internal', 
			-SQL='DELETE FROM security_group_db_map WHERE ((allow = "") OR (allow ISNULL)) AND ((deny = "") OR (deny ISNULL))';
	/inline;
	inline: -log=$__startup_loglevel__, -database='lasso_internal', 
			-SQL='DELETE FROM security_group_table_map WHERE ((allow = "") OR (allow ISNULL)) AND ((deny = "") OR (deny ISNULL))';
	/inline;
	

	//
	// Default File Extensions
	//
	
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal',
			-table='global_prefs',
			-maxrecords='1',
			-returnfield='data',
			-op='eq', 'store_key'='lasso_file_extensions';
		if: (found_count == 0);
			
			local: 'default_extensions' = (array: '.txt','.text','.uld','.xml','.xsd', '.xsl','.pdf','.jpg','.gif','.png','.bmp','.psd','.tif','.rgb','.cmyk','.wsdl');
			#default_extensions->sort;
			
			inline: -add,
					-log=$__startup_loglevel__, 
					-database='lasso_internal',
					-table='global_prefs',
					-keyfield='store_key',
					'store_key'='lasso_file_extensions',
					'data'=#default_extensions->(join: ' ');
			/inline;
		/if;
	/inline;
	
	//
	// Default Script Extensions
	//
	
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal',
			-table='global_prefs',
			-maxrecords='1',
			-returnfield='data',
			-op='eq', 'store_key'='lasso_script_extensions';
		if: (found_count == 0);
			
			local: 'default_extensions' = (array: '.las','.lasso','.lassoapp','.htm','.html','.inc','.incl');
			#default_extensions->sort;
			
			inline: -add,
					-log=$__startup_loglevel__, 
					-database='lasso_internal',
					-table='global_prefs',
					-keyfield='store_key',
					'store_key'='lasso_script_extensions',
					'data'=#default_extensions->(join: ' ');
			/inline;
		/if;
	/inline;
	
	//
	// JDBC/FMSA/SQLite Table Level Encoding
	//

	local: 'iso_exceptions' = (array: 'security_datasources', 'security_datasource_hosts', 'security_datasource_databases', 'security_database_tables', 'security_table_fields');

	local: 'utf_array' = (array);
	local: 'iso_array' = (array);

	local: 'sql' = '
		SELECT security_database_tables.id
		FROM security_database_tables 
		LEFT JOIN security_datasource_databases 
			ON (security_database_tables.id_database = security_datasource_databases.id)
		LEFT JOIN security_datasources
			ON (security_datasource_databases.id_datasource = security_datasources.id)
		WHERE (security_datasources.name LIKE "%jdbc%") OR (security_datasources.name LIKE "%driver%") OR (security_datasources.name = "FMServerAdvancedDataSource") OR (security_datasources.name = "FMServer9DataSource") OR (security_datasources.name = "SQLiteConnector")
			AND (security_database_tables.encoding <> "utf-8")
			AND (security_database_tables.name NOT IN ("' + #iso_exceptions->(join: '","') + '"))
	';
	inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql, -maxrecords='all';
		iterate: records_array, (local: 'temp_record');
			#utf_array->(insert: #temp_record->(get:1));
		/iterate;
	/inline;

	local: 'sql' = '
		SELECT security_database_tables.id
		FROM security_database_tables 
		LEFT JOIN security_datasource_databases 
			ON (security_database_tables.id_database = security_datasource_databases.id)
		LEFT JOIN security_datasources
			ON (security_datasource_databases.id_datasource = security_datasources.id)
		WHERE (security_datasources.name = "SQLiteConnector")
			AND (security_database_tables.encoding <> "iso8859-1")
			AND (security_database_tables.name IN ("' + #iso_exceptions->(join: '","') + '"))
	';
	inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql, -maxrecords='all';
		iterate: records_array, (local: 'temp_record');
			#iso_array->(insert: #temp_record->(get:1));
		/iterate;
	/inline;

	if: #utf_array->size > 0;
		log_warning: 'Setting default encoding of SQLite, JDBC, and FileMaker SA tables to UTF-8';
		local: 'sql' = '
			UPDATE security_database_tables
			SET encoding = "utf-8"
			WHERE id IN (' + #utf_array->(join: ',') + ')
		';
		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql;
		/inline;
		var: '__need_reload__' = true;
	/if;
	
	if: #iso_array->size > 0;
		log_warning: 'Setting default encoding of security tables to ISO8859-1';
		local: 'sql' = '
			UPDATE security_database_tables
			SET encoding = "iso8859-1"
			WHERE id IN (' + #iso_array->(join: ',') + ')
		';
		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql=#sql;
		/inline;
		var: '__need_reload__' = true;
	/if;
	

	//
	// Enable the LDML8_Reference
	//
	
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='security_datasource_databases',
			-op='eq', 'id_datasource' = 1,
			-op='eq', 'id_host' = 1,
			-op='bw', 'name' = 'ldml8_reference',
			-maxrecords=1;
		if: (found_count > 0) && ((field: 'enabled') == '');
			inline: -log=$__startup_loglevel__, -update,
					-database='lasso_internal', 
					-table='security_datasource_databases',
					-keyfield='id',
					-keyvalue=(field: 'id'),
					'enabled' = 'Y',
					-maxrecords='all';
			/inline;
		/if;
	/inline;
	
	//
	// Fix the File Permissions table schema
	//
	
	inline: -log=$__startup_loglevel__, 
			-database='lasso_internal', 
			-sql='SELECT sql FROM sqlite_master WHERE type="table" AND tbl_name="security_group_file_perms"';
		if: (found_count == 1);
			if: ((field: 'sql') !>> 'allow_any_extension');
			
				protect;
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='BEGIN TRANSACTION';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='CREATE TEMPORARY TABLE security_group_file_temp AS SELECT * FROM security_group_file_perms';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='DROP TABLE security_group_file_perms';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='CREATE TABLE security_group_file_perms (id INTEGER PRIMARY KEY, id_group INTEGER NOT NULL, allow_root TINYTEXT, allow_create CHAR DEFAULT "N", allow_read CHAR DEFAULT "N", allow_write CHAR DEFAULT "N", allow_move CHAR DEFAULT "N", allow_copy CHAR DEFAULT "N", allow_delete CHAR DEFAULT "N", allow_inspect CHAR DEFAULT "N", allow_any_extension CHAR DEFAULT "N", modified VARCHAR(13) )';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='INSERT INTO security_group_file_perms (id, id_group, allow_root, allow_create, allow_read, allow_write, allow_move, allow_copy, allow_delete, allow_inspect, modified) SELECT id, id_group, allow_root, allow_create, allow_read, allow_write, allow_move, allow_copy, allow_delete, allow_inspect, modified FROM security_group_file_temp';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='DROP TABLE security_group_file_temp';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
		
					inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='COMMIT TRANSACTION';
						fail_if: (error_msg != error_noerror), error_code, error_msg;
					/inline;
					
					handle_error;
						inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='ROLLBACK TRANSACTION';
							fail_if: (error_msg != error_noerror), error_code, error_msg;
						/inline;
					/handle;
				/protect;
			
			/if;
		/if;
	/inline;

	//
	// Add Default Spotlight Host
	//
	if: (site_id != 0);
		inline: -log=$__startup_loglevel__, 
				-search, 
				-database='lasso_internal', 
				-table='security_datasources',
				-op='eq', 'name'='SpotlightDS',
				-returnfield='id',
				-maxrecords=1;
			inline: -log=$__startup_loglevel__, 
					-search, 
					-database='lasso_internal', 
					-table='security_datasource_hosts',
					'id_datasource'=(field: 'id'),
					-returnfield='id',
					-maxrecords=1;
				local: 'need_spotlight' = (found_count == 0);
			/inline;
			if: #need_spotlight;
				log_warning: 'Adding default Spotlight host';
				inline: -log=$__startup_loglevel__, 
						-add, 
						-database='lasso_internal', 
						-table='security_datasource_hosts',
						-keyfield='id',
						'id_datasource'=(field: 'id'),
						'enabled'='Y',
						-maxrecords=1;
					admin_setpref: -name='_admin_host_alias:' + (field: 'id'), -value='Spotlight';
				/inline;
				_datasource_reload: (integer: (field: 'id'));
				var: '__need_reload__' = true;
			/if;
		/inline;
	/if;
	
	//
	// Add Default Connectors
	//
	if: (site_id != 0) && ((admin_getpref: -name='_admin_defaultconnectors') !>> 'Y');
		admin_setpref: -name='_admin_defaultconnectors', -value='Y';
		local: 'connectors' = (array);
		inline: -log=$__startup_loglevel__, 
				-findall, 
				-database='lasso_internal', 
				-table='security_datasources',
				-returnfield='name',
				-maxrecords='all';
			records;
				#connectors->(insert: (field: 'name'));
			/records;
		/inline;
		iterate: _admin_defaultconnectornames, (local: 'defaultconnectorname');
			local: 'connector' = #defaultconnectorname->first;
			(#connector == 'internalmysqlds') ? loop_continue;
			((#connector == 'spotlightds') && (lasso_version !>> 'Mac OS X')) ? loop_continue;
			if: #connectors !>> #connector;
				inline: -log=$__startup_loglevel__, 
						-add, 
						-database='lasso_internal', 
						-table='security_datasources',
						'name'=#connector,
						'enabled'='Y',
						'id_site'=(site_id);
				/inline;
			/if;
		/iterate;
	/if;
}->run;

?>