<?LassoScript

	/*
		Library of routines to perform miscellaneous functions that used to be performed in Java modules
		Implements:
			[Euro: 12.54, -EuroFrom='BEF']	deprecated synonym for [Math_ConvertEuro]
			[Math_ConvertEuro: 12.54, -EuroFrom='BEF']
			[Include_CGI: -Action='http://blah', -Method='GET', -NoData, 'firstname'='john', 'lastname'='hill'] (Now uses the CURL tags)
			[Redirect_URL: 'http://blah.com']

		Problems:
			[Include_CGI] rewritten to use CURL tags, needs testing.

		History:
			06/15/2004	AJR - Moved to a pure LassoScript format
			04/06/2004	AJR - Added [Lasso_Uptime]
			03/08/2004	FMS - Updated [WAP_IsEnabled] to search only for Accept type.
			11/10/2003	AJR - Fixed a mistake in Math_ConvertEuro
			10/22/2003	AJR - Updated Include_CGI to work. Less error checking, but faster. It's actually safer too.
			10/22/2003	AJR - Modified the Euro tags to actually work with L7 better
			10/07/2003	AJR - Updated Redirect_URL to be much more efficient. It now requires that the headers have \r\n line endings though.
			09/15/2003	AJR - Updated _chop_mimes so that it wouldn't add empty pairs (which happened if you had a return at the end of your headers...grrr....)
			09/15/2003	AJR - Updated Redirect_URL to have up to date code, and handle extra mimes better
			02/18/2003	AJR - Replaced _chop_mimes with a new version that works better =)
			11/11/2002  AJR - Removed size - 1 from redirect URL which was messing up cookie sets.
			07/11/2002	AJR - Modified Include_CGI to fix a problem with hyphens and to only allow pairs of string|integer|decimals as parameters
			07/11/2002	AJR - Rewrite Include_CGI to use the CURL version of Include_URL
			06/13/2002	FMS - Converted to LassoScript, Changed to [_Include_URL], Rewrote [Euro]
			02/22/2002	GLH - Fix for difference between IIS and Apache HTTP_ WAP headers
			01/24/2002	GLH - Added extra error handling for malformed URLs
			12/07/2001	GLH - Added -port parameter to [Include_URL]
			11/26/2001	GLH - Added local variable 'Accepts' to prevent syntax error when not defined
			11/04/2001	GLH - Added Content-Type: application/x-www-form-urlencoded to [Include_CGI] method=POST
			09/10/2001	FMS - Added initialization for #PostParams and #GetParams in [Include_URL]
												Added parentheses around (Get: Loop_Count) calls in [Include_URL]
			09/07/2001	GLH - Changed _Chop_MIMEs to handle both \n and \r\n linebreaks
			04/16/2001	GLH - Username and Password upgraded to dashed params in [Include_URL]
			03/05/2001	GLH - upgraded to dashed params, now look for both legacy and current params
			02/26/2001	GLH - Added WAP tags
			01/23/2001	GLH - Added [Redirect_URL] tag
			01/04/2001	GLH - Added RetrieveMIMEHeaders feature to [Include_URL]
			01/03/2001	GLH - Added new optional params to [Include_URL]
			12/18/2000	GLH - Added username/password and extra MIME headers to [Include_CGI]
			12/15/2000	GLH - Added [Include_CGI]
			12/14/2000	GLH - Added [Include_URL]
			12/04/2000	GLH - changed [tag] to [define_tag]
			11/22/2000	GLH - Initial work
	*/



	/*
		_Chop_MIMEs:
			Helper routine to break apart MIME headers into an array

			The custom tag name begins with "_" in order to make it less likely that
			it will interfere with customer's tag names

		Problems:

		Inputs:
			*unnamed*	text containing raw MIME headers, separated by cr/lf pairs

		Output:
			Array of name/value pairs, each pair representing a MIME header
	*/

	define_tag: '_chop_mimes',
		-required = 'data',
		-priority = 'replace';

		if: #data->(find: '\r\n') > 0;
			local: 'breaks' = '\r\n';
		else: #data->(find: '\n') > 0;
			local: 'breaks' = '\n';
		else;
			local: 'breaks' = '\r';
		/if;

		local: 'split' = (string_replaceregexp:
			-find = (#breaks + '[ \t]+'),
			-replace = ' ',
			#data)->(split: #breaks);

		local: 'mimes' = (array);

		iterate: #split, (local: 'i');
			local: 'name' = #i->(split: ':')->(get: 1);
			local: 'value' = (string_removeleading: -pattern = (#name + ':'), #i);

			#name->trim;
			#value->trim;

			//only add a pair if the local name has some content
			if: (#name->size > 0);
				#mimes->(insert: (pair: #name = #value));
			/if;
		/iterate;

		return: @#mimes;

	/define_tag;



	/*
		Math_ConvertEuro (and Euro):
			Convert between different european currencies

		Problems:
			Probably dies with divide-by-zero if incorrect country code passed in EuroFrom

		Inputs:
			-euroFrom	numeric value to convert from Euros
			-euroTo		numeric value to convert to Euros

		Outputs:
			numeric value after conversion
	*/

	Define_Tag: 'ConvertEuro', -namespace='Math_',
		-priority = 'Replace',
		-required = 'Amount',
		-optional = 'EuroTo',
		-optional = 'EuroFrom';

		Local: 'convTable' = (map:
			'BEF' = 40.3399,
			'DEM' = 1.95583,
			'ESP' = 166.386,
			'FRF' = 6.55957,
			'IEP' = 0.787564,
			'ITL' = 1936.27,
			'LUF' = 40.3399,
			'NLG' = 2.20371,
			'ATS' = 13.7603,
			'PTE' = 200.482,
			'FIM' = 5.94573);

		If: (local_defined: 'EuroTo') && (#EuroTo->size > 0);
			Return: #Amount * #convTable->(find: #EuroTo);
		Else;
			Return: #Amount / #convTable->(find: #EuroFrom);
		/If;
	/Define_Tag;

	Define_Tag: 'Euro',
		-priority = 'Replace',
		-required = 'Amount',
		-optional = 'EuroTo',
		-optional = 'EuroFrom';

		return: (Math_ConvertEuro:
			#Amount,
			-EuroTo = (local: 'EuroTo'),
			-EuroFrom = (local: 'EuroFrom'));
	/Define_Tag;



	/*
		Include_CGI - CURL version
		Simple custom tag to allow Include_CGI to work with the new
		CURL tags, which gives us the perk of being able to do SSL
		web includes.
		//
		Required Params:
			-Action
		//
		Option Params:
			-Method = GET|POST
			-nodata
		//
		Returns:
			Body of the web page specified by the -Action parameter,
			Excluding the header information (may return emtpy string
			if action produced no result from the server).  Will
			return null if the -nodata parameter is specified.
	*/

	define_tag: 'CGI', -namespace='Include_',
		-priority = 'replace',
		-required = 'action',
		-required = 'method',
		-deprecated;

		local:
			'__cgiparams' = (array),
			'__nodata' = (params)->(find: '-nodata')->size == 1;

		iterate: params, (local: '__current');
			if: (#__current->type == 'pair') && (#__current->first != '-action') && (#__current->first != '-method');
				#__cgiparams->(insert: #__current);
			/if;
		/iterate;

		if: #method != 'get';
			local: '__url' = (include_url: #action, -postparams = #__cgiparams, -encodenone);
		else;
			local: '__url' = (include_url: #action, -getparams = #__cgiparams, -encodenone);
		/if;

		if: ! #__nodata;
			return: (string: #__url);
		/if;

	/define_tag;



	/*
		Redirect_URL:
			Modifies MIME headers to force remote browser to redirect to a different location
			Smart enough to leave any custome MIME headers, like cookies, in place

		Problems:
			Not enough error outputs
			Doesn't handle missing 'http://' gracefully

		Inputs:
			*unnamed*	full URL to be redirected to 'http://www.beachnet.com/~hstiles/cardtype.html'

		Outputs:
			none

		Side-Effects:
			modifies global variable __http_header__
	*/

	define_tag: 'URL', -namespace='Redirect_',
		-Priority='replace',
		-required='URL',
		-optional = 'NoAbort',
		-optional = 'Type';

		if: ($__http_header__->(isa: 'string')) && ($__http_header__ >> '\n') && ($__http_header__->(find: '\n') > 0);
			$__http_header__->(remove: 1, $__http_header__->(find: '\n'));
		/if;

		if: (local_defined: 'type') && ( (#type >> '301') || (#type >> 'move') );
			$__http_header__ = 'HTTP/1.0 301 MOVED\r\nLocation: ' #url '\r\nURI: ' #url '\r\n' $__http_header__;
		else;
			$__http_header__ = 'HTTP/1.0 302 FOUND\r\nLocation: ' #url '\r\nURI: ' #url '\r\n' $__http_header__;
		/if;
		
		// Abort so the redirect will happen immediately.
		if: ! (local_defined: 'NoAbort');
			abort;
		/if;
	/define_tag;


	// Lasso_Uptime globals
	global: '__Lasso_Uptime__' = date->serialize;
	(global: '__Lasso_Uptime__')->freezetype;
	(global: '__Lasso_Uptime__')->freezevalue;
	
	// NoProcess in LassoScript
	Define_Tag: 'NoProcess', -Container;
	/Define_Tag;

?>