<?LassoScript

Log_Warning: 'RPC: Defining Introspection Methods';

define_type:'base64', 'string';
	// for use within xml_rpc. signifies type of data
/define_type;

//
// Standard System Methods
//

define_tag:'system.listMethods', -rpc, -description='Lists the methods available on this server.';
	return: @$__RPC__->keys;
/define_tag;

define_tag:'system.methodHelp', -rpc, -required='name', -description='Returns the help message for the specific RP. Requires one parameter: the name of the RP to get help for.';
	local:'found' = @$__RPC__->find:#name;
	fail_if: #found == null, -1, 'The method could not be found.';
	return: #found->description;
/define_tag;

define_tag:'system.methodSignature', -rpc, -description='Lasso does not support method signatures.';
	fail:-1, 'Lasso does not support method signatures.';
/define_tag;

define_tag:'system.multiCall', -rpc, -description='Support for pipe-lining RPC';
	local:'ret'=array;
	iterate:params, local:'curr';
		protect;
			local:'methodName'=@(#curr->find:'methodName');
			local:'methodParams'=@(#curr->find:'params');
			local:'found'=@($__RPC__->find:#methodName);

			fail_if: #methodName == tag_name, -1, 'Recursive system.multiCall not permitted.';
			fail_if: !(#curr >> 'params'), -1, 'Parameters must be provided for the method.';
			fail_if: #found == null, -1, 'The method ' + #methodName + ' was not found.';

			#ret->insert:(@#found->(run:-params=@#methodParams));
			handle: (error_currenterror:-errorcode) != 0;
				#ret->insert:(map: 'faultCode'=(error_currenterror:-errorcode), 'faultString'=(error_currenterror));
			/handle;
		/protect;
	/iterate;
	return:@#ret;
/define_tag;

//
// Test Echo Method
//

define_tag:'test.echo', -rpc, -description='Returns the given parameters.';
	return:@params;
/define_tag;

//
// XML-RPC Validator Suite Methods
//

define_tag:'validator1.arrayOfStructsTest', -rpc, -required='structs';
	local:'res'=0;
	iterate:#structs, local:'curr';
		#res += #curr->find:'curly';
	/iterate;
	return:#res;
/define_tag;

define_tag:'validator1.countTheEntities', -rpc, -required='str';
	local:'ret'=map;
	#ret->insert:'ctLeftAngleBrackets'=(#str->(split:'<')->size - 1);
	#ret->insert:'ctRightAngleBrackets'=(#str->(split:'>')->size - 1);
	#ret->insert:'ctAmpersands'=(#str->(split:'&')->size - 1);
	#ret->insert:'ctApostrophes'=(#str->(split:'\'')->size - 1);
	#ret->insert:'ctQuotes'=(#str->(split:'"')->size - 1);
	return:@#ret;
/define_tag;

define_tag:'validator1.easyStructTest', -rpc, -required='struct';
	return: integer:(#struct->find:'moe') + (#struct->find:'larry') + (#struct->find:'curly');
/define_tag;

define_tag:'validator1.echoStructTest', -rpc, -required='struct';
	return:#struct;
/define_tag;

define_tag:'validator1.manyTypesTest', -rpc;
	return:@params;
/define_tag;

define_tag:'validator1.moderateSizeArrayCheck', -rpc, -required='ary';
	return:(#ary->get:1) + (#ary->last);
/define_tag;

define_tag:'validator1.nestedStructTest', -rpc, -required='struct';
	local:'target' = @(#struct->(find:'2000')->(find:'04')->(find:'01'));
	return: integer:(#target->find:'moe') + (#target->find:'larry') + (#target->find:'curly');
/define_tag;

define_tag:'validator1.simpleStructReturnTest', -rpc, -required='num';
	return:(map:'times10'=(#num * 10), 'times100'=(#num * 100), 'times1000'=(#num * 1000));
/define_tag;

//
// [XML_RPCCall] Tag
//
// This tag is an easier to use front-end for calling remote procedures and
// has a delightfully redundant name.
//
// [XML_RPCCall: -Host='http://127.0.0.1/Lasso/RPC.LassoApp', -Method='Test.Echo', -Params='Hello World!']
// Either -uri or -url can be used as default for host.
//
// Returns null if no result is received.  Returns a value of any data type if a single result is
// received.  Returns an array if multiple results are received.  Sets error code and message.
//

define_tag:'rpccall', -namespace='xml_', -optional='params', -optional='method', -optional='host', -optional='url', -optional='uri';
	local:'myparams' = null;
	local:'return' = null;
	
	if: (local_defined: 'uri');
		local: 'myhost' = #uri;
	else: (local_defined: 'url');
		local: 'myhost' = #url;
	else: (local_defined: 'host');
		local: 'myhost' = #host;
	else;
		local: 'myhost' = 'http://127.0.0.1/Lasso/RPC.LassoApp';
	/if;
	
	if: (local_defined: 'method');
		local: 'mymethod' = #method;
	else;
		local: 'mymethod' = 'test.echo';
	/if;
	
	if:local_defined:'params';
		if:#params->isa:'array';
			#myparams = #params;
		else;
			#myparams = array:#params;
		/if;
	else;
		#myparams = array;		
	/if;
	
	error_seterrorcode: 0;
	error_seterrormessage: '';
	protect;
		local: 'return'= (xml_rpc:#myparams)->(call: -uri=local:'myhost', -method=#mymethod);
		handle_error;
			return;
		/handle_error;
	/protect;
	if: (#return == array);
		return;
	else: (#return->type == 'array') && (#return->size == 1);
		return: #return->(get:1);
	else;
		return: #return;
	/if;
/define_tag;

?>