<?LassoScript

{

//
// Domain Name Server
//

	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='domain_name_server';
		if: (found_count) > 0;
			global: '__domain_name_server__' = (field: 'data');
			Log_Warning: 'Domain Name Server: Using ' + (field: 'data') + '.';
		else;
			global: '__domain_name_server__' = '';
		/if;
	/inline;


//
// Error Reporting
//

	local: 'error_reporting'=null;
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='error_reporting';
		if: (found_count) > 0;
			local: 'error_reporting' = (field: 'data');
		/if;
	/inline;
	if: #error_reporting === null;
		local: 'error_reporting' = 'minimal';
	/if;
	protect;
		lasso_errorreporting: (string: #error_reporting), -global;
		Log_Warning: 'Error Reporting: ' + (string: #error_reporting) + '.';
	/protect;


//
// Recursion Limit
//

	local: 'recursion_limit'=null;
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='recursion_limit';
		if: (found_count) > 0;
			local: 'recursion_limit' = (field: 'data');
		/if;
	/inline;
	if: #recursion_limit === null;
		local: 'recursion_limit' = 50;
	/if;
	protect;
		_recursion_limit: (integer: #recursion_limit);
		if: (integer: #recursion_limit) > 0;
			Log_Warning: 'Recursion Limit: ' + (integer: #recursion_limit) + '.';
		else;
			Log_Warning: 'Recursion Limit: No Limit.';
		/if;
	/protect;


//
// Format File Execution Time Limit
//

	local: 'execution_timelimit'=null;
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='execution_timelimit';
		if: (found_count) > 0;
			local: 'execution_timelimit' = (field: 'data');
		/if;
	/inline;
	if: #execution_timelimit === null;
		local: 'execution_timelimit' = 600;
	/if;
	protect;
		_execution_timelimit: (integer:#execution_timelimit);
		if: (integer: #execution_timelimit) > 0;
			Log_Warning: 'Format File Execution Time Limit: ' + (integer:#execution_timelimit) + ' seconds.';
		else;
			Log_Warning: 'Format File Execution Time Limit: No Limit.';
		/if;
	/protect;

//
// Default Page Encoding
//

	local: 'default_page_encoding'=null;
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='default_page_encoding';
		if: (found_count) > 0;
			local: 'default_page_encoding' = (field: 'data');
		/if;
	/inline;
	if: (#default_page_encoding === null) || !(string_validcharset: #default_page_encoding);
		local: 'default_page_encoding' = 'utf-8';
	/if;
	global: '__encoding__' = (string: #default_page_encoding);
	Log_Warning: 'Default Page Encoding: ' + (string: #default_page_encoding) + '.';

//
// Digest Authentication
//

	local: 'lasso_digest_auth'=null;
	inline: -log=$__startup_loglevel__, 
			-search,
			-database='lasso_internal', 
			-table='global_prefs',
			-keyfield='store_key',
			-keyvalue='lasso_digest_auth';
		if: (found_count) > 0;
			local: 'lasso_digest_auth' = (field: 'data');
		/if;
	/inline;
	if: (#lasso_digest_auth !>> 'Y');
		local: 'lasso_digest_auth' = 'N';
	else;
		local: 'lasso_digest_auth' = 'Y';
	/if;

	global: '__digest_auth__' = #lasso_digest_auth;
		
	if: (#lasso_digest_auth) !>> 'Y';
		Log_Warning: 'Authentication: Basic Only.';
	else;
		Log_Warning: 'Authentication: Digest and Basic.';
	/if;
	
}->run;


//
// Insert literals for 'all', 'eq', 'neq', etc. into tag map
//

iterate: (array: 'all', 'and', 'bw', 'cn', 'eq', 'ew', 'ft', 
		'gt', 'gte', 'lt', 'lte', 'neq', 'not', 'nrx', 'or', 'rx', 
		'none', 'minimal', 'full'), 
		(var: 'temp');
	define_constant: $temp, $temp;
/iterate;
var_remove: 'temp';

?>
