<?LassoScript

// namespace aware [tags] replacement
// does not offer full functionality, but should cover most bases - KRJ
define_type:'tags', -prototype;
	/*
	find
	values
	keys
	removeall
	iterator
	insert
	remove
	contains
	get
	onconvert
	insertfrom
	size 	
	*/
	define_tag:'onCreate';
		
	/define_tag;

	define_tag:'find', -required='p';
		return:tags_find(#p);
	/define_tag;
	
	define_tag:'values';
		local:'r' = array;
		iterate:tags_list, local:'i';
			#r->insert(@tags_find(#i));
		/iterate;
		return:#r;
	/define_tag;
	
	define_tag:'keys';
		return:tags_list;
	/define_tag;
	
	define_tag:'removeall';
	
	/define_tag;
	
	define_tag:'iterator';
	
	/define_tag;
	
	define_tag:'insert', -required='keyPair';
		namespace_import:#keyPair->second, -into=#keyPair->first;
	/define_tag;
	
	define_tag:'remove', -required='name';
		namespace_unload:#name;
	/define_tag;
	
	define_tag:'contains', -required='name';
		return:tags_list->contains(#name);
	/define_tag;
	
	define_tag:'>>', -required='name';
		return:tags_list >> #name;
	/define_tag;
	
	define_tag:'!>>', -required='name';
		return: tags_list !>> #name;
	/define_tag;
	
	define_tag:'get';
	
	/define_tag;
	
	define_tag:'onconvert';
		return:'the tags type';
	/define_tag;
	
	define_tag:'insertfrom';
	
	/define_tag;
	
	define_tag:'size';
		return:tags_list->size;
	/define_tag;

/define_type;

// [Log] Redefining log tag with encoding safe replacement.

define_tag: 'log', -optional='file', -optional='encoding', -container, -priority='replace';
	if: (local_defined: 'file');
		_log_helper: -file=#file, -encoding=(local: 'encoding'), -content=run_children;
	else;
		log_critical: (string: run_children);
	/if;
/define_tag;

define_tag: '_log_helper',
		-required='file', 
		-required='content',
		-required='encoding', 
		-privileged, 
		-priority='replace';
	if: !(local_defined: 'encoding') || (#encoding == '');
		if: ((lasso_version: -lassoplatform) >> 'mac');
			local: '_encoding' = 'mac';
		else;
			local: '_encoding' = 'iso-8859-1';
		/if;
	else;
			local: '_encoding' = #encoding;
	/if;
	fail_if: !(string_validcharset: #_encoding), -9956, 'Expected a valid character set but got ' + #encoding + '.';
	local: '_file' = (string: #file);
	#_file->trim;
	local: '_file' = (string_replace: #_file, -find='\\', -replace='/');
	fail_if: #_file->(beginswith: '///'), -9956, '[Log] Cannot write to fully qualified paths';
	fail_if: #_file->(endswith: '/'), -9956, '[Log] Cannot write to a directory';
	fail_if: (#_file >> '//'), -9956, '[Log] Cannot write to fully qualified paths';
	fail_if: (#_file >> '..'), -9956, '[Log] Cannot use .. in log paths';
	if: !(file_exists: #_file);
		file_create: #_file;
		fail_if: (file_currenterror: -errorcode) != 0, (file_currenterror: -errorcode), '[Log] ' + file_currenterror;
	/if;
	local: 'bytes' = bytes;
	#bytes->(importstring: #content, #_encoding);
	file_write: #file, #bytes;
/define_tag;

// [Database_FMContainer] tag that fetches the data from an FM container field

define_tag: 'FMContainer',
		-namespace='Database_',
		-required='field',
		-optional='repetition',
		-optional='type',
		-privileged;

	fail_if: (database_name == ''), -1, 'Error: Database not found or enabled.';
	fail_if: (table_name == ''), -1, 'Error: Table not found or enabled.';
	fail_if: ((field: #field) == null), -1, 'Error: Field not found or enabled.';


	local: '_repetition' = (math_max: 1, (integer: (local: 'repetition')));
	if: !(local_defined: 'type');
		local: '_type' = 'jpeg';
	else;
		local: '_type' = (string: #type);
	/if;

	local: 'table' = (table_name);
	local: 'keyvalue' = (keyfield_value);

	inline:
		-log = 'none',
		-database = 'lasso_internal',
		-sql = '\
			select \n\t\
				a.name as name, \n\t\
				b.name as host, \n\t\
				b.port as port, \n\t\
				case \n\t\t\
					when a.always_use_defaults = \'Y\' then \n\t\t\t\
						b.username \n\t\t\
					else \n\t\t\t\
						a.username \n\t\
				end as username, \n\t\
				case \n\t\t\
					when a.always_use_defaults = \'Y\' then \n\t\t\t\
						b.password \n\t\t\
					else \n\t\t\t\
						a.password \n\t\
				end as password, \n\t\
				b.schema as schema, \n\t\
				c.name as connector \n\
			from \n\t\
				security_datasource_databases as a \n\t\
				left join security_datasource_hosts as b on (a.id_host = b.id) \n\t\
				left join security_datasources as c on (b.id_datasource = c.id) \n\
			where \n\t\
				case \n\t\t\
					when a.alias is not null and a.alias <> \'\' then \n\t\t\t\
						a.alias \n\t\t\
					when a.name is not null and a.name <> \'\' then \n\t\t\t\
						a.name \n\t\t\
				end like \'' (encode_sql92: database_name ) '\' and \n\t\
				a.enabled = \'Y\' and \n\t\
				b.enabled = \'Y\' and \n\t\
				c.name in (\'FMServerAdvancedDataSource\', \'FMServer9DataSource\', \'FMRemoteDataSource\') \n\
			limit 1';

		fail_if: !found_count, -1728, 'The database "' database_name '" was not found, was not enabled, or was not a FileMaker Pro or FileMaker Server Advanced database.';

		local: 'database' = (field: 'name');
		local: 'username' = (field: 'username');
		local: 'password' = (field: 'password');
		local: 'url' = (field: 'host');
		local: 'port' = (field: 'port');
		local: 'connector' = (field: 'connector');

		if: !#username->size;
			#username = 'dummy';
		/if;
	/inline;

	local: 'result' = null;
	if: (#connector == 'FMRemoteDataSource');
		local: '_type' = (string: (local: 'type'));
		if: (#_type == '');
			local: '_type' = 'jpeg';
		/if;
		local: 'url' = 'http://' + #url + ':' + #port + '/FMPro?-db=' + #database + '&-lay=' + #table + '&' + #field + '=jpeg&-recid=' + #keyvalue + '&-img';
		local: 'result' = (include_url: #url, -username = #username, -password = #password);
	else: (#connector == 'FMServerAdvancedDataSource') || (#connector == 'FMServer9DataSource');
		// Return Null rather than erroring to prevent false error on empty container.
		if: ((field: #field) !>> '/fmi/xml/cnt/data');
			return: null;
		/if;

		local: 'url' = 'http://' + #url + ':' + #port + (field: #field);
		local: 'result' = (include_url: #url, -username = #username, -password = #password);
	/if;

	if: (#result >> 'HTTP 404: Not Found');
		return: null;
	/if;

	if: (#result >> 'HTTP 404: Not Found');
		return: null;
	/if;

	return: @#result;

/define_tag;

	// replacement for old repetition functionality
	define_type:'repetition', -prototype;

		//
		// IMPORTANT - [Repetition] has been deprecated.
		//
		// [If: (Repetition == 2)] should be changed to [If: (Loop_Count % 2 == 0)]
		//

		define_tag: 'onCompare', -required='modulo';

			if: (var_defined: '__loop_count__') && ($__loop_count__->size > 0);
				return: (((integer: $__loop_count__->last) % #modulo) != 0);
			/if;

			if: !(var_defined: '__repetition__');
				var: '__repetition__' = (map);
				log_warning: 'Warning: The [Repetition] tag has been deprecated.';
			/if;

			local: 'temp' = ((integer: $__repetition__->(find: #modulo)) + 1);
			$__repetition__->(insert: #modulo = #temp);
			return: ((#temp % #modulo) != 0);

		/define_tag;

	/define_type;

?>