<?LassoScript
	{

		//
		// Site ID - Try to fix the id_site fields
		//

		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='UPDATE security_users SET id_site = ' + site_id; /inline;
		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='UPDATE security_groups SET id_site = ' + site_id; /inline;
		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='UPDATE security_datasources SET id_site = ' + site_id; /inline;

		//
		// Errors - Removes any errors in excess of 1,000 in the _errors table
		//
		
		inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='SELECT id FROM errors ORDER BY id DESC LIMIT 1000,1';
			
			if: (found_count == 1);
			
				log_warning: 'Errors Table: Deleting excess error messages.';
			
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='DELETE FROM errors WHERE id <= ' + (field: 'id');
				/inline;
			
				inline: -log=$__startup_loglevel__, -database='lasso_internal', -sql='VACUUM errors';
				/inline;
			
			/if;
			
		/inline;
		
		//
		// LDML Reference - Update the LDML Reference database if a new version exists at server level
		//
		
		if: (site_id == 0);
			// Install LDML Reference
			var: 'src_path' = (admin_lassoservicepath + '/SQLiteDBs/LDML8_Reference');
			var: 'dst_array' = (array);
			inline: -log=$__startup_loglevel__, -search,
					-database='lasso_internal',
					-table='sites',
					-keyfield='id',
					-maxrecords='all',
					-op='gt', 'id'=0;
				records;
					protect;
						var: 'dst_folder' = (field: 'name') + '-' + (field: 'id');
						var: 'dst_path' = (admin_lassoservicepath + '/LassoSites/' + $dst_folder + '/SQLiteDBs/LDML8_Reference');
						if: (file_exists: $src_path) && (file_exists: $dst_path) && ((date: (file_moddate: $dst_path)) < (date: (file_moddate: $src_path)));
							log_warning: 'Lasso Reference: Updating database for site ' + (field: 'id') + ' ' + (field: 'name') + '.';
							file_copy: $src_path, $dst_path, -fileoverwrite;
						else: ((field: 'id') == 1) && (file_exists: $src_path) && !(file_exists: $dst_path) 
							log_warning: 'Lasso Reference: Installing database for site ' + (field: 'id') + ' ' + (field: 'name') + '.';
							file_copy: $src_path, $dst_path, -fileoverwrite;
						/if;
					/protect;
				/records;
			/inline;
		/if;
		
		//
		//	Update allowed file roots settings for LP802
		//
		/*
			At site 0 startup, if the security_site_paths table is empty 
			and if the upgrade procedure has not already been performed,
			the existing file permissions for each site are used to fill it with appropriate values.

			for each site, select all of security_group_file_perms.
				for each row
					if field allow_root is not an integer (it's a string)
						add a new row to security_site_paths with site id and this path
						grab the new id
						
						insert the id in place of the original string
						
						add a path for /// and assign it to the site's admin
							(this is for compatability with the previous system in
							an attempt to easy upgrading - 
							this entry should mimic the previous sytem exactly)
		*/
		if ( (site_id == 0) && (admin_getpref(-name='_admin_fileperms_upgrade') !>> 'Y') );
			admin_setpref(-name='_admin_fileperms_upgrade', -value='Y');
			// if the security_site_paths table is empty, then we perform an upgrade on all existing sites
			local(	'pathsCount'=integer, 
					'siteIDs'=array, 
					'currSiteID',
					'allowRoot',
					'filePermID',
					'insertID',
					'thisDBName');
			inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='security_site_paths', -sql='SELECT COUNT(*) AS count FROM security_site_paths');
				#pathsCount = integer(field('count'));
			/inline;
			if (#pathsCount == 0);
				// there are no paths. we assume this is the first time Lasso is being run after
				// an upgrade to LP802. the alternative is that this is the first time Lasso is being run
				
				define_tag('isAllDigits', -req='str');
					loop(#str->size);
						if (!#str->isDigit(loop_count));
							return(false);
						/if;
					/loop;
					return(true);
				/define_tag;
				
				// select all site ids
				inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='sites', -returnField='id', -findall);
					rows;
						#siteIDs->insert(integer(field('id')));
					/rows;
				/inline;
				iterate(#siteIDs, #currSiteID);
					#thisDBName = 'lasso_internal_' + string(#currSiteID);
					inline(-log=$__startup_loglevel__, -database=#thisDBName, -table='security_group_file_perms', 
							-returnField='id', -returnField='allow_root', -findall);
						if (found_count > 0);
							log_warning('File Permissions: Updating paths for site ' + #currSiteID);
						/if;
						rows;
							#allowRoot = string(field('allow_root'));
							#filePermID = integer(field('id'));
							if (!isAllDigits(#allowRoot));
								// need to upgrade this entry
								inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='security_site_paths', 
										-op='eq', 'id_site'=#currSiteID, -op='eq', 'path'=#allowRoot, -search, -maxrecords=1, -uselimit);
									if (found_count > 0);
										rows;
											// change the string to be the id of the matching record
											inline(-log=$__startup_loglevel__, -database=#thisDBName, -table='security_group_file_perms', 
													-keyField='id', -keyValue=#filePermID, 'allow_root'=field('id'), -update);
											/inline;
										/rows;
									else;
										// add a new row to security_site_paths
										inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='security_site_paths', 
												'id_site'=#currSiteID, 'path'=#allowRoot, -add);
											if (error_code == 0);
												// change the string to be the id of the matching record
												inline(-log=$__startup_loglevel__, -database=#thisDBName, -table='security_group_file_perms', 
														-keyField='id', -keyValue=#filePermID, 'allow_root'=keyField_value, -update);
												/inline;
											/if;
										/inline;
									/if;
								/inline;
							/if;
						/rows;
					/inline;
					// Add the root path only if this is an upgrade
					if (admin_getpref(-name='lasso_site_initialized', -site=#currSiteID) !>> 'Y');
						log_warning('File Permissions: Adding Web server root permission to site ' + #currSiteID);
						// add a / path for the admins group if it does not already exist
						local('rootOfItAll' = '/');
						inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='security_site_paths', -returnField='id', 'id_site'=#currSiteID, 
								-op='eq', 'path'=#rootOfItAll, -find);
							if (error_code == 0);
								local('newPathID');
								if (found_count == 0);
									// add the path
									inline(-log=$__startup_loglevel__,'id_site'=#currSiteID, 'path'=#rootOfItAll, -add);
										#newPathID = keyfield_value;
									/inline;
								else;
									#newPathID = integer(field('id'));
								/if;
							/if;
						/inline;
					else;
						log_warning('File Permissions: Adding file server root permission to site ' + #currSiteID);
						// add a /// path for the admins group if it does not already exist
						local('rootOfItAll', 'appPath'=admin_lassoservicepath);
						#rootOfItAll = (#appPath->isAlpha(1) && #appPath->get(2) == ':' ? (#appPath->get(1) + '://') | '///');
						inline(-log=$__startup_loglevel__, -database='lasso_internal', -table='security_site_paths', -returnField='id', 'id_site'=#currSiteID, 
								-op='eq', 'path'=#rootOfItAll, -find);
							if (error_code == 0);
								local('newPathID');
								if (found_count == 0);
									// add the path
									inline(-log=$__startup_loglevel__,'id_site'=#currSiteID, 'path'=#rootOfItAll, -add);
										#newPathID = keyfield_value;
									/inline;
								else;
									#newPathID = integer(field('id'));
								/if;
							/if;
						/inline;
					/if;
				/iterate;
			/if;
		/if;
		
	}->(run);

?>
