<?lassoscript

/* 
Notes:

1. Most significant datepart cannnot be greater than event rate (frequency)
2. -day and -weekday params are mutually exclusive (except for #3)
3. -day and -weekday params are mutually interchangeable when event rate is: -weekly
4. When the least significant datepart is -week, the -weekday param is implicitly set to the weekday of a starting date
5. In absence of -month param, -day param aquires a meaning of "day of year" when rate is -yearly
*/

define_type:'schedule', -namespace='date', -description='Provides utility functions for advanced date schedule calculations.';
	local:'kRates' = (map: '-daily'=4,'-weekly'=6,'-monthly'=7,'-yearly'=8);
	local:
		'minute'=null,
		'hour'=null,
		'day'=null,
		'weekday'=null,		
		'week'=null,
		'month'=null,
		'year'=null,
		'frequency'=null,
		'_denom'=0,
		'_rate'=0,
		'_fixedrate'=false,
		'format'='',
		'weekstart'='-sunday',
		'__dbg__'='';
	
	define_tag:'onconvert', -required='to';
		if: #to == 'string';
			local:'out' = 'schedule: (' + self->'frequency';
			if: self->'day';
				#out += ', day=' + self->'day';
			/if;
			if: self->'weekday';
				#out += ', weekday=' + self->'weekday';
			/if;
			if: self->'week';
				#out += ', week=' + self->'week';
			/if;
			if: self->'month';
				#out += ', month=' + self->'month';
			/if;
			if: self->'year';
				#out += ', year=' + self->'year';
			/if;
			if: self->'hour';
				#out += ', hour=' + self->'hour';
			/if;
			if: self->'minute';
				#out += ', minute=' + self->'minute';
			/if;
			return: #out + ')';
		else;
			fail: -10000, 'Couldn\'t convert schedule type to ' + #to;
		/if;
	/define_tag;

		
	define_tag:'oncreate';
		iterate:params, local:'p';
			if: #p->isa('string'); // rate?
				if: #p->beginswith('-annual'); // synonym
					#p = '-yearly';
				/if;
				if: self->'kRates'->find(#p);
					self->'frequency' = #p ;
					self->'_rate' = self->'rates'->find(#p);
				else if: #p == '-monday';
					self->'weekstart' = '-monday';
				/if;
			else: #p->isa('pair');
				select: #p->first;
				case: '-minute';
					self->'minute' = #p->second;
					if: self->'_denom' < 2; 
						self->'_denom' = 2; 
					/if;
				case: '-hour';
					self->'hour' = #p->second;
					if: self->'_denom' < 3 ; 
						self->'_denom' = 3; 
					/if;
				case: '-day';
					self->'day' = #p->second;
					if: self->'_denom' < 4; 
						self->'_denom' = 4; 
					/if;
				case: '-weekday';
					self->'weekday' = #p->second;
					if: self->'_denom' < 5; 
						self->'_denom' = 5; 
					/if;
				case: '-week';
					self->'week' = #p->second;
					if: self->'_denom' < 6; 
						self->'_denom' = 6; 
					/if;
				case: '-month';
					self->'month' = #p->second;
					if: self->'_denom' < 7; 
						self->'_denom' = 7; 
					/if;
				case: '-year';
					self->'year' = #p->second;
					if: self->'_denom' < 8; 
						self->'_denom' = 8; 
					/if;
				/select;
			/if;
		/iterate;
		
		if: self->'_denom' == 0 && self->'_rate' == 0;
			fail:-1, 'Insufficient params';
		/if;

		// missing/implicit rate?
		if: self->'_rate' == 0;
			if: self->'_denom' < 4;
				self->'_rate' = 4;
			else: self->'_denom' == 4 || self->'_denom' == 5;
				self->'_rate' = 6;
			else: self->'_denom' == 6;
				self->'_rate' = 7;
			else: self->'_denom' == 7 || self->'_denom' == 8;
				self->'_rate' = 8;
			/if;
		/if;
						
		if: self->'_rate' < self->'_denom';
			fail:-1, 'Rate should be greater or equal to the value of the most significant datetime parameter';
		/if;
		
		// '-day' is synonymous to '-weekday' when '-week' is present or rate is '-weekly'
		if: (self->'_rate' == 6 || self->'week') && self->'day' && !self->'weekday';
			self->'weekday' = self->'day';
			self->'day' = null;
		/if;
				
		// fixed-rate or fixed-time?
		if: self->'_rate' == self->'_denom';
			self->'_fixedrate' = true;
		/if;

		// adjust invalid year
		if: !self->'_fixedrate' && !self->'year'->isa('null') && self->'year' < 100;
			self->'year' += 2000;
		/if;
		
		// validate
		if: self->'weekday' != null && self->'_rate' != 6 && self->'week' == null;
			fail:-1, '-weekday param requires -weekly and/or -week keyword!';
		else if: self->'weekday' != null && self->'day' != null;
			fail:-1, '-weekday and -day params are mutually exclusive.';
		/if;
/*
		if: self->'_rate' == 8 && self->'month' == null;
			fail:-1, '-yearly param used without -month keyword';
		/if;
		if: self->'_rate' == 7 && self->'day' == null;
			fail:-1, '-monthly param requires -day keyword';
		/if;
		if: self->'_rate' == 6 && self->'day' == null && self->'weekday' == null;
			fail:-1, '-weekly param requires -day and/or -weekday keyword';
		/if;
*/
	/define_tag;
	
	
	define_tag:'checkdate', -required='d', -optional='start';		
		if: self->'weekday' != null && self->'weekday' != #d->dayofweek(self->'weekstart');
			return: false;
		/if;
		
		if: !self->'_fixedrate';
		
			if: self->'year' != null && self->'year' != #d->year;
				return: false;
			/if;			
			if: self->'month' != null && self->'month' != #d->month;
				return: false;
			/if;
			if: self->'week' != null && self->'week' != #d->week;
				return: false;
			/if;
			if: self->'day' < 0;
				local:'tmp' = date(#d, -day=1)->add(-month=1, -day=self->'day');
				if: #d->day != #tmp->day;
					return: false;
				/if;
			else if: self->'day' != null && self->'day' != #d->day;
				return: false;
			/if;
			
			if: self->'hour' != null && self->'hour' != #d->hour;
				return: false;
			/if;	

			if: self->'minute' != null && self->'minute' != #d->minute;
				return: false;
			/if;
					
			return: true;
		else; // fixed-interval
		
			if: !local_defined:'start';
				local:'start' = date;
			/if;
			
			local:'tmp' = #start;
			local:'n' = 1;
			
			while: #tmp < #d;
				#tmp = self->nextdate(#start, -occurance=#n);
				#n++;
				if: #tmp->format('%D') == #d->format('%D')
				&& (!self->'hour' || #tmp->hour == #d->hour) 
				&& (!self->'minute' || #tmp->minute == #d->minute);
					return: true;
				/if;
			/while;
			
			return: false;
		/if;
	/define_tag;
	
	
	define_tag:'nextdate', -optional='d', -optional='-occurence';
		local: 'diff' = null;
		
		if: !local_defined('d') || #d == null;
			local:'d' = date;
		else if: #d->isa('string');
			#d = date(#d);
		/if;
		if: !#d->isa('date');
			return: null; // or fail?
		/if;
		
		local:'out' = date(#d);
		local:'n' = integer(local:'occurence');
		
		if: self->'minute';
			#out->set(-minute=self->'minute');
		/if;
		if: self->'hour';
			#out->set(-hour=self->'hour');
		/if;
			
		if: !self->'_fixedrate';
			if: self->'day';
				#out->set(-day=self->'day');
			/if;
			if: self->'month';
				#out->set(-month=self->'month');
			/if;
			if: self->'year';
				#out->set(-year=self->'year');
			/if;
			
			if: #out >= #d && #n;
				#n--;	
			/if;

			if: #out < #d || #n > 0;
				select: self->'_rate';
				case: 8; //yearly
					#out->add(-year=#n);
				case: 7; //monthly
					#out->add(-month=#n);
				case: 6; //weekly
					#out->add(-week=#n);
				case: 4; //daily
					#out->add(-day=#n);
				/select;
			/if;
			
			if: self->'weekday' != null;
				if: #out->dayofweek(self->'weekstart') > self->'weekday';
					#out->add(-day=7 + self->'weekday' - #out->dayofweek(self->'weekstart'));
				else: #out->dayofweek(self->'weekstart') < self->'weekday';
					#out->add(-day=self->'weekday' -  #out->dayofweek(self->'weekstart'));
				/if;
			/if;
			
		else; // fixed-rate
	
			if: !#n;
				#n = 1;
			/if;
					
			if: self->'_rate' == 4; // daily
			
				if: self->'day';
					#n *= self->'day';
				/if;
				if: #out > #d;
					#n--;
				/if;
				#out->add(-day=#n);
				
			else: self->'_rate' == 6; // weekly
				if: self->'week';
					#n *= self->'week';
				/if;
				
				if: self->'weekday';
					local:'delta' = self->'weekday' - #out->dayofweek(self->'weekstart');
					if: #delta < 0;
						#delta += 7;
					else;
						#n--;
					/if;		
					#out->add(-day=#delta);	
				else if: #out > #d;
					#n--;
				/if;
				#out->add(-day=#n * 7);
				
			else: self->'_rate' == 7; // monthly
			
				if: self->'month';
					#n *= self->'month';
				/if;
				
				if: self->'day';
					if: #out->day < self->'day';
						#n--;
					/if;
					#out->add(-month=#n);
					#out->set(-day=self->'day');
					return: #out; // done
				/if;
				
				if: self->'weekday';
					local:'wkday' = self->'weekday';
				else;
					local:'wkday' = #d->dayofweek(self->'weekstart'); // derive from the beginning date
				/if;
				
				if: self->'week';
					local:'week' = self->'week';
				else;
					local:'week' = 1;
				/if;
	
				#out->add(-month=#n);			
				#out->set(-day=1);
				
				local:'delta' = #wkday - #out->dayofweek(self->'weekstart');
				if: #delta < 0;
					#delta += 7 * #week;
				else;
					#delta += 7 * (#week - 1);
				/if;
				#out->add(-day=#delta);
				
			else: self->'_rate' == 8; // annualy
			
				if: self->'year';
					#n *= self->'year';
				/if;

				if: (!self->'month' && self->'day' && #out->dayofyear > self->'day')
				|| (self->'month' && #out->month > self->'month')
				|| (#out->month == self->'month' && (!#self->'day' || #out->day == self->'day') && #out < #d);
				 	#n--;
				/if;
					
				#out->set(-year=#d->year + #n);
				
				if: !self->'month' && self->'day' && !self->'weekday' && !self->'week'; // nth day of year
					#out->set(-month=1, -day=self->'day');
					return: #out; // done
				/if;
		
				if: self->'day';
					#out->set(-doy=self->'day');
				/if;
				
				if: self->'month';
					#out->set(-month=self->'month');
				/if;
				
				if: self->'weekday';
					local:'wkday' = self->'weekday';
				else;
					local:'wkday' = #d->dayofweek(self->'weekstart'); // use start date
				/if;
				
				if: self->'week';
					local:'week' = self->'week';
				else;
					local:'week' = 1;
				/if;
				
				local:'delta' = #wkday - #out->dayofweek(self->'weekstart');
				if: #delta < 0;
					#delta += #week * 7;
				else;
					#delta += (#week - 1) * 7;
				/if;
				#out->add(-day=#delta);
			/if;
		/if;
		
		if: #out->dst < #d->dst;
			#out->add(-hour=-1);
		else if: #out->dst > #d->dst;
			#out->add(-hour=1);
		/if;
				
		return: #out;
	/define_tag;
	
	define_tag:'frequency';
		return: self->'frequency';
	/define_tag;
	
	define_tag:'minute';
		return: self->'minute';
	/define_tag;
	
	define_tag:'hour';
		return: self->'hour';
	/define_tag;

	define_tag:'weekday';
		return: self->'weekday';
	/define_tag;
	
	define_tag:'month';
		return: self->'month';
	/define_tag;

	define_tag:'year';
		return: self->'year';
	/define_tag;
/define_type;
	
?>

