<?LassoScript

	define_tag: 'TranslateBreaksToCRLF',
			-required = 'data',
			-namespace = 'email_';
		return: @(string_replaceregexp:
			-find = '\r\n|\n|\r',
			-replace = '\r\n',
			#data);
	/define_tag;


	// [Email_FindEmails]
	//
	// First parameter is a string containing one or more email address.
	//
	// This tag returns an array of all the email addresses contained within a string.
	//

	define_tag: 'FindEmails',
			-Priority='Replace',
			-namespace = 'email_';
		Local: 'emailArray' = (String_Replace: params->(get: 1), -find=';', -replace=',');
		Local: 'emailArray' = (String_FindRegExp: #emailArray, -find='[^ @\r\n,<>]*@[^ @\r\n,<>]*');
		Local: 'outArray' = (array);
		Loop: #emailArray->size;
			If: #emailArray->(get: loop_Count)->(get: 1) != '"';
				#outArray->(insert: #emailArray->(get: Loop_Count));
			/If;
		/Loop;
		return: @#outArray;
	/define_tag;

	// [Email_Compose] Type
	//
	// This type functions as a component of [Email_Send] and allows for emails
	// with attachments, HTML and text parts, and custom encodings to be created.
	//
	// [Email_Compose]
	//
	// The following four member tags allow different body parts and attachments to be added.
	//
	// [Email_Compose->AddTextPart]
	// [Email_Compose->AddHTMLPart]
	// [Email_Compose->AddAttachment]
	// [Email_Compose->AddPart]
	//
	// The following three member tags are used by [Email_Send]
	//
	// [Email_Compose->Recipients] - Returns an array of recipients of the message.  Includes to, cc, and bcc addresses.
	// [Email_Compose->From] - Returns the from address for the message.
	// [Email_Compose->Data] - Returns the MIME text of the message including all parts and attachments.
	// [Email_Compose->Summary] - Returns a summary of the essential headers of the message for display.
	//

	define_type: 'compose',
		-namespace = 'email_';

		local: 'mime_msg';
		local: 'mime_id';
		local: 'mime_hdrs';
		local: 'mime_recipients';
		local: 'mime_from';
		local: 'mime_contenttype';
		local: 'mime_transferencoding';
		local: 'mime_contentdisposition';
		local: 'mime_headertype';
		local: 'mime_charset';
		local: 'mime_boundary';
		local: 'mime_parts';
		local: 'mime_htmlindex';
		local: 'mime_textindex';
		local: 'mime_mode';
		local: 'mime_dirty';
		local: 'mime_hdrstoencode' = (array: 'subject','comment','to','cc','bcc','from','reply-to','sender');

		define_tag: 'onCreate',
				-optional='date',
				-optional='from',
				-optional='to',
				-optional='cc',
				-optional='bcc',
				-optional='subject',
				-optional='sender',
				-optional='replyto',
				-optional='body',
				-optional='html',
				-optional='contenttype',
				-optional='characterset',
				-optional='transferencoding',
				-optional='contentdisposition',
				-optional='headertype',
				-optional='extramimeheaders',
				-optional='attachments',
				-optional='htmlimages',
				-optional='parts';

			//
			// Recipients
			//
			self->'mime_recipients' = (array);
			if: (local_defined: 'to');
				self->'mime_recipients'->(merge: (Email_FindEmails: #to));
			/if;
			if: (local_defined: 'cc') && (local: 'cc')->size;
				self->'mime_recipients'->(merge: (Email_FindEmails: #cc));
			/if;
			if: (local_defined: 'bcc') && (local: 'bcc')->size;
				self->'mime_recipients'->(merge: (Email_FindEmails: #bcc));
			/if;

			//
			// From
			//
			fail_if: !(local_defined: 'from') && (self->'mime_recipients'->size > 0), -1, '[Email_Compose] A from address is required';
			self->'mime_from' = '';
			if: (local_defined: 'from');
				self->'mime_from' = (Email_FindEmails: #from)->(first);
			/if;

			//
			// Headers
			//
			self->'mime_hdrs' = (array);
			if: (local_defined: 'extramimeheaders') && #extramimeheaders->(isa: 'array');
				self->'mime_hdrs'->(merge: #extramimeheaders);
			/if;

			if: (self->'mime_hdrs' !>> 'MIME-Version') && (self->'mime_recipients'->size > 0);
				self->'mime_hdrs'->(insert: 'MIME-Version' = '1.0');
			/if;
			if: (local_defined: 'characterset');
				self->'mime_charset' = #characterset;
			else;
				self->'mime_charset' = 'utf-8';
			/if;
			if: (local_defined: 'contenttype');
				self->'mime_hdrs'->(removeall: 'Content-Type');
				if: #contenttype >> 'charset';
					self->'mime_hdrs'->(insert: 'Content-Type' = #contenttype);
					self->'mime_contenttype' = #contenttype;
					self->'mime_charset' = '';
				else;
					self->'mime_hdrs'->(insert: 'Content-Type' = #contenttype + '; charset=' + (self->'mime_charset'));
					self->'mime_contenttype' = #contenttype + '; charset=' + (self->'mime_charset');
				/if;
			else: (self->'mime_hdrs' !>> 'Content-Type');
				self->'mime_hdrs'->(insert: 'Content-Type' = 'text/plain; charset=' + (self->'mime_charset') '');
				self->'mime_contenttype' = 'text/plain; charset=' + (self->'mime_charset') '';
			else;
				protect;
					self->'mime_contenttype' = self->'mime_hdrs'->(find: 'Content-Type')->first->second;
				/protect;
			/if;
			if: (self->'mime_charset' == '');
				protect;
					self->'mime_charset' = (string_findregexp: self->'mime_hdrs'->(find: 'Content-Type')->first->second, -find='(?<=charset=)[0-9a-zA-Z_\'()+_,\\-./:=?]+(?=)')->(get:1);
				/protect;
			/if;
			if: (self->'mime_charset' == '');
				protect;
					self->'mime_charset' = (string_findregexp: self->'mime_hdrs'->(find: 'Content-Type')->first->second, -find='(?<=charset=)[0-9a-zA-Z_\'()+_,\\-./:=?]+')->(get:1);
				/protect;
			/if;

			if: (local_defined: 'transferencoding');
				self->'mime_hdrs'->(removeall: 'Content-Transfer-Encoding');
				self->'mime_hdrs'->(insert: 'Content-Transfer-Encoding' = #transferencoding);
				self->'mime_transferencoding' = #transferencoding;
			else: (self->'mime_hdrs' !>> 'Content-Transfer-Encoding');
				self->'mime_hdrs'->(insert: 'Content-Transfer-Encoding' = 'quoted-printable');
				self->'mime_transferencoding' = 'quoted-printable';
			else;
				protect;
					self->'mime_transferencoding' = self->'mime_hdrs'->(find: 'Content-Transfer-Encoding')->first->second;
				/protect;
			/if;

			if: (local_defined: 'contentdisposition');
				self->'mime_contentdisposition' = #contentdisposition;
			else;
				self->'mime_contentdisposition' = 'attachment';
			/if;

			if: (local_defined: 'headertype');
				self->'mime_headertype' = #headertype;
			else;
				self->'mime_headertype' = 'Q';
			/if;

			if: (self->'mime_hdrs' !>> 'X-Mailer') && (self->'mime_recipients'->size > 0);
				self->'mime_hdrs'->(insert: 'X-Mailer' = 'Lasso Professional ' + (string_replaceregexp: -find = '([\\.\\d]+).*', -replace = '\\1', (lasso_version: -lassoversion)));
			/if;

			if: (self->'mime_hdrs' !>> 'Message-ID') && (self->'mime_recipients'->size > 0);
				self->'mime_id' = lasso_uniqueid;
				self->'mime_hdrs'->(insert: 'Message-ID' = '<' + self->'mime_id' + '@' + {
						if: (global_defined: '__smtp_ip__');
							return: $__smtp_ip__;
						else: (admin_getpref: (map: 'lasso_smtp_lassoservice_ip' = ''))->(find: 'lasso_smtp_lassoservice_ip')->size;
							global: '__smtp_ip__' = (admin_getpref: (map: 'lasso_smtp_lassoservice_ip' = ''))->(find: 'lasso_smtp_lassoservice_ip');
						else;
							global: '__smtp_ip__' = '127.0.0.1';
						/if;

						return: $__smtp_ip__;
					}->eval + '>');
			/if;

			if: (local_defined: 'date') && #date->(isa: 'date');
				local: 'timezone' = #date->(format: '%z');
				if: !(#timezone->(beginswith: '-')) && !(#timezone->(beginswith:'+'));
					#timezone = '+' + #timezone;
				/if;
				self->'mime_hdrs'->(insert: 'Date' = #date->(format: '%a, %-d %b %Y %T ') + #timezone );
			else: (self->'mime_recipients'->size > 0);
				local: 'timezone' = date->(format: '%z');
				if: !(#timezone->(beginswith: '-')) && !(#timezone->(beginswith:'+'));
					#timezone = '+' + #timezone;
				/if;
				self->'mime_hdrs'->(insert: 'Date' = date->(format: '%a, %-d %b %Y %T ') + #timezone );
			/if;

			if: (local_defined: 'to');
				self->'mime_hdrs'->(insert: 'To' = #to);
			/if;
			if: (local_defined: 'cc') && (local: 'cc')->size;
				self->'mime_hdrs'->(insert: 'Cc' = #cc);
			/if;
			if: (local_defined: 'from');
				self->'mime_hdrs'->(insert: 'From' = #from);
			/if;
			if: (local_defined: 'subject');
				self->'mime_hdrs'->(insert: 'Subject' = #subject);
			/if;

			if: (local_defined: 'sender');
				self->'mime_hdrs'->(insert: 'Sender' = #sender);
			/if;

			if: (local_defined: 'replyto');
				self->'mime_hdrs'->(insert: 'Reply-To' = #replyto);
			/if;

			self->'mime_parts' = (array);
			self->'mime_mode' = '';

			if: (local_defined: 'body');
				self->(addtextpart: -data=#body);
			/if;

			if: (local_defined: 'html');
				if: (local_defined: 'htmlimages');
					self->(addhtmlpart: -data=#html, -images=#htmlimages);
				else;
					self->(addhtmlpart: -data=#html);
				/if;
			/if;

			if: (local_defined: 'attachment') && !(local_defined: 'attachments');
				local: 'attachments' = @#attachment;
			/if;
			if: (local_defined: 'attachments');
				iterate: (if_false: #attachments->(isa: 'array') || #attachments->(isa: 'map'), (array: #attachments), #attachments), (local: 'temp');
					if: #temp->(isa: 'pair');
						self->(addattachment: -name=#temp->first, -data=#temp->second);
					else;
						self->(addattachment: -path=#temp);
					/if;
				/iterate;
			/if;

			if: (local_defined: 'data');
				self->(addattachment: -data=#data, -name=(if_empty: (local: 'name'), 'Attachment'));
			/if;

			if: (local_defined: 'parts');
				iterate: (if_false: #parts->(isa: 'array'), (array: #parts), #parts), (local: 'temp');
					self->(addpart: -path=#temp);
				/iterate;
			/if;

			// Store list of headers that we encode using quoted-printable encoding
			self->'mime_hdrstoencode' = (array: 'subject','comment','to','cc','bcc','from','reply-to','sender');

			self->'mime_dirty' = true;
			
		/define_tag;

		define_tag: 'addtextpart',
			-optional='data',
			-optional='path';

			self->'mime_dirty' = true;

			if: (local_defined: 'data') && #data->(isa: 'compose');
				self->'mime_parts'->(insert: #data);
				self->'mime_mode' += 'text ';
				self->'mime_textindex' = self->'mime_parts'->size;
				return;
			/if;

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');
				local: '_raw' = (include: #path);
			else;
				fail: -1, '[Email_Compose] The text part could not be found.';
			/if;

			#_raw = (Email_TranslateBreaksToCRLF: #_raw);

			local: 'part' = (map:
				'_body' = #_raw,
				'_hdrs'=(array:
					'Content-Type' = (self->'mime_contenttype'),
					'Content-Transfer-Encoding' = (self->'mime_transferencoding'),
				),
				'_contenttype' = (self->'mime_contenttype'),
				'_transferencoding' = (self->'mime_transferencoding'),
				'_charset' = (self->'mime_charset'),
			);
			self->'mime_parts'->(insert: #part);
			self->'mime_mode' += 'text ';
			self->'mime_textindex' = self->'mime_parts'->size;
		/define_tag;

		define_tag: 'addhtmlpart',
			-optional='data',
			-optional='path',
			-optional='images';

			self->'mime_dirty' = true;

			if: (local_defined: 'data') && #data->(isa: 'compose');
				self->'mime_parts'->(insert: #data);
				self->'mime_mode' += 'html ';
				self->'mime_htmlindex' = self->'mime_parts'->size;
				return;
			/if;

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');
				local: '_raw' = (include: #path);
			else;
				fail: -1, '[Email_Compose] The HTML part could not be found.';
			/if;

			if: (local_defined: 'images');
			
				// Fix local HTML references to embedded images
				iterate: (#images->(isa: 'array') ? #images | (array: #images)), (local: 'image');
					if: #image->(isa: 'string');
						#_raw->(replace: 'src="' + #image + '"', 'src="cid:' + #image->(split: '/')->last->(split:'\\')->last + '"');
					else: #image->(isa: 'pair') && #image->first->(isa: 'string');
						#_raw->(replace: 'src="' + #image->first + '"', 'src="cid:' + #image->first + '"');
					/if;
				/iterate;
				
				local: 'comp' = (email_compose: 
						-html=#_raw, 
						-attachments=#images, 
						-contenttype='multipart/related', 
						-contentdisposition='inline');		
				self->'mime_parts'->(insert: @#comp);
				self->'mime_mode' += 'html ';
				self->'mime_htmlindex' = self->'mime_parts'->size;
			else;
			
				#_raw = (Email_TranslateBreaksToCRLF: #_raw);

				local: 'part' = (map:
					'_body' = #_raw,
					'_hdrs'=(array:
						'Content-Type' = 'text/html; charset=' + (self->'mime_charset'),
						'Content-Transfer-Encoding' = (self->'mime_transferencoding'),
					),
					'_contenttype' = 'text/html; charset=' + (self->'mime_charset'),
					'_transferencoding' = (self->'mime_transferencoding'),
					'_charset' = (self->'mime_charset'),
				);

				self->'mime_parts'->(insert: #part);
				self->'mime_mode' += 'html ';
				self->'mime_htmlindex' = self->'mime_parts'->size;
			/if;
		/define_tag;

		define_tag: 'addattachment',
			-optional='data',
			-optional='name',
			-optional='path',
			-optional='type';

			self->'mime_dirty' = true;

			if: (local_defined: 'data') && #data->(isa: 'compose');
				self->'mime_parts'->(insert: #data);
				self->'mime_mode' += 'attachment ';
				return;
			/if;

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');	
				local: '_raw' = (include_raw: #path);
			else;
				fail: -1, '[Email_Compose] An attachment could not be found.';
			/if;

			if: (#_raw == '');
				fail: -1, '[Email_Compose] The attachment was empty.';
			/if;

			if: (local_defined: 'name');
				local: '_name' = #name;
			else: (local_defined: 'path');
				local: '_name' = #path->(split: '/')->last->(split:'\\')->last;
			else;
				fail: -1, '[Email_Compose] An attachment name must be provided.';
			/if;

			local: '_base64' = (encode_base64: #_raw);
			local: '_array' = array;
			loop: -from=1, -to=#_base64->size, -by=76;
				#_array->(insert: #_base64->(substring: loop_count, 76));
			/loop;

			if: (local_defined: 'type') && (#type >> '/');
				local: '_type' = #type;
			else: #_raw->(beginswith: 'GIF89a');
				local: '_type' = 'image/gif';
			else: #_raw->(beginswith: '%PDF');
				local: '_type' = 'application/pdf';
			else;
				local: '_type' = (if_null: (mime_type: #_name), 'application/octet-stream');
			/if;
			
			local: 'part' = (map:
				'_body' = #_array->(join: '\r\n'),
				'_hdrs'=(array:
					'Content-Type' = #_type + '; name="' + #_name + '"',
					'Content-Transfer-Encoding' = 'base64',
					'Content-Disposition' = self->'mime_contentdisposition' + '; filename="' + #_name + '"',
					'Content-ID' = '<' + (encode_xml: #_name) + '>',
				),
				'_contenttype' = 'application/octet-stream; name="' + #_name + '"',
				'_transferencoding' = 'base64',
			);
			self->'mime_parts'->(insert: #part);
			self->'mime_mode' += 'attachment ';
		/define_tag;

		define_tag: 'addpart',
			-optional='data';

			self->'mime_dirty' = true;

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else;
				fail: -1, '[Email_Compose] A part could not be found.';
			/if;

			self->'mime_parts'->(insert: #_raw);
			self->'mime_mode' += 'part ';
		/define_tag;

		define_tag: 'data', -optional='prefix', -optional='force';

			if: (self->'mime_dirty' == false) && (self->'mime_msg' != '') && !(local_defined: 'force');
				return: (string_replace: self->'mime_msg', -find=self->'mime_id', -replace=lasso_uniqueid);
			/if;

			self->'mime_dirty' = false;

			local: 'output' = (array);

			if: (local_defined: 'prefix') && (self->'mime_boundary' == '');
				self->'mime_boundary' = '====_-' + (local: 'prefix') + '-' + (lasso_uniqueid) + '-_====';
			else: (self->'mime_boundary' == '');
				self->'mime_boundary' = '====_-LASSO-' + (lasso_uniqueid) + '-_====';
			/if;
			
			local: 'multi_part' = (self->'mime_parts'->size > 1);
			local: 'alternative' = ( (self->'mime_mode' >> 'html') && (self->'mime_mode' >> 'text') );
			local: 'mixed' = ( (#multi_part == true) && ( (self->'mime_mode' >> 'attachment') || (self->'mime_mode' >> 'part') ) );
			local: 'override' = ( (#mixed == true) && (self->'mime_contenttype' >> 'multipart/') );
			local: 'related' = (self->'mime_parts'->size == 1) && self->'mime_parts'->first->(isa: 'compose') && (self->'mime_parts'->first->'mime_contenttype' >> 'multipart/related');

			if: (#multi_part == true);
						
				if: (#override == true);
					// If the content-type is already multi-part then don't change it.
					self->'mime_hdrs'->(removeall: 'Content-Type');
					self->'mime_hdrs'->(insert: 'Content-Type' = (string_replaceregexp: self->'mime_contenttype', -find='.*(multipart/[^;]+).*', -replace='\\1') + '; boundary="' + self->'mime_boundary' + '"');
				else: (#mixed == true);
					self->'mime_hdrs'->(removeall: 'Content-Type');
					self->'mime_hdrs'->(insert: 'Content-Type' = 'multipart/mixed; boundary="' + self->'mime_boundary' + '"');
					self->'mime_contenttype' = 'multipart/mixed';
				else: (#alternative == true);
					self->'mime_hdrs'->(removeall: 'Content-Type');
					self->'mime_hdrs'->(insert: 'Content-Type' = 'multipart/alternative; boundary="' + self->'mime_boundary' + '"');
					self->'mime_contenttype' = 'multipart/alternative';
				else;
					self->'mime_hdrs'->(removeall: 'Content-Type');
					self->'mime_hdrs'->(insert: 'Content-Type' = 'multipart/mixed; boundary="' + self->'mime_boundary' + '"');
					self->'mime_contenttype' = 'multipart/mixed';
				/if;
				self->'mime_hdrs'->(removeall: 'Content-Transfer-Encoding');

			else: (self->'mime_parts'->size > 0);
			
				if: self->'mime_parts'->first->(isa: 'compose');
					self->'mime_hdrs'->(removeall: 'Content-Type');
					if: #related;
						self->'mime_hdrs'->(insert: 'Content-Type' = 'multipart/related; boundary="' + self->'mime_boundary' + '"');
						self->'mime_parts'->first->'mime_boundary' = self->'mime_boundary';
						self->'mime_contenttype' = 'multipart/related';
					else;
						self->'mime_hdrs'->(merge: self->'mime_parts'->first->'mime_hdrs'->(find: 'content-type'));
					/if;
					self->'mime_hdrs'->(removeall: 'Content-Transfer-Encoding');
					self->'mime_hdrs'->(merge: self->'mime_parts'->first->'mime_hdrs'->(find: 'content-transfer-encoding'));
					
				else: self->'mime_parts'->first->(isa: 'map');
					self->'mime_hdrs'->(removeall: 'Content-Type');
					self->'mime_hdrs'->(merge: self->'mime_parts'->first->(find: '_hdrs')->(find: 'content-type'));
					self->'mime_hdrs'->(removeall: 'Content-Transfer-Encoding');
					self->'mime_hdrs'->(merge: self->'mime_parts'->first->(find: '_hdrs')->(find: 'content-transfer-encoding'));
				/if;
				
			/if;

			iterate: self->'mime_hdrs', (local: 'hdr');
				// Only encode certain headers that we recognize
				if: (self->'mime_hdrstoencode' >> #hdr->first);
					(self->'mime_headertype' >> 'b')
						? #output->(insert: (encode_bheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'))
						| #output->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'));
				else: 'Message-ID' == #hdr->first;
					local: 'mime_id' = lasso_uniqueid;
					#output->(insert: (encode_qheader: -name=#hdr->first, -value=(string_replace: #hdr->second, -find=self->'mime_id', -replace=#mime_id), -charset='none'));
					self->'mime_id' = #mime_id;
				else;
					#output->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset='none'));
				/if;
			/iterate;

			#output->(insert: '');

			if: (self->'mime_parts'->size > 0);

				// Create the individual message parts

				local: 'parts' = (array);

				iterate: self->'mime_parts', (local: 'part');
				
					if: #part->(isa: 'map');
				
						local: 'tempout' = (array);
					
						if: (#multi_part == true);
							iterate: #part->(find: '_hdrs'), (local: 'hdr');
								// Only encode certain headers that we recognize
								if: (self->'mime_hdrstoencode' >> #hdr->first);
									(self->'mime_headertype' >> 'b')
										? #tempout->(insert: (encode_bheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'))
										| #tempout->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'));
								else;
									#tempout->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset='none'));
								/if;
							/iterate;
	
							#tempout->(insert: '');
						/if;
	
						if: (#part->(find: '_transferencoding') >> 'quoted-printable');
							#tempout->(insert: (encode_quotedprintable: #part->(find: '_body'), -charset=#part->(find: '_charset')));
						else: (string_validcharset: #part->(find: '_charset'));
							#tempout->(insert: (bytes: #part->(find: '_body'), #part->(find: '_charset')));
						else;
							#tempout->(insert: #part->(find: '_body'));
						/if;
	
						#parts->(insert: #tempout);
						
					else: #part->(isa: 'compose') || #part->(isa: 'string') || #part->(isa: 'bytes');

						#parts->(insert: @#part);
						
					/if;
 
				/iterate;

				// Merge the html and text parts in mixed messages

				if: (#multi_part == true) && (#alternative == true) && (#mixed == true);

					local: 'html' = @#parts->(get: self->'mime_htmlindex');
					local: 'text' = @#parts->(get: self->'mime_textindex');
					#parts->(remove: self->'mime_htmlindex');
					#parts->(remove: self->'mime_textindex');

					local: 'tempout' = (array);
									
					local: 'boundary' = '====_-ALT-' + (lasso_uniqueid) + '-_====';
					
					local: '_hdrs'=(array:
						'Content-Type' = 'multipart/alternative; boundary="' + #boundary + '"',
					);
				
					iterate: #_hdrs, (local: 'hdr');
						// Only encode certain headers that we recognize
						if: (self->'mime_hdrstoencode' >> #hdr->first);
							(self->'mime_headertype' >> 'b')
								? #tempout->(insert: (encode_bheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'))
								| #tempout->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'mime_charset'));
						else;
							#tempout->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset='none'));
						/if;
					/iterate;

					#tempout->(insert: '');
					#tempout->(insert: '--' + #boundary);
					if: #text->(isa: 'array');
						#tempout->(insertfrom: #text->iterator);
					else: #text->(isa: 'compose');
						#tempout->(insert: #text->data);
					else;
						#tempout->(insert: #text);
					/if;
					#tempout->(insert: '');
					#tempout->(insert: '--' + #boundary);
					if: #html->(isa: 'array');
						#tempout->(insertfrom: #html->iterator);
					else: #html->(isa: 'compose');
						#tempout->(insert: #html->(data: 'HTML'));
					else;
						#tempout->(insert: #html);
					/if;
					#tempout->(insert: '');
					#tempout->(insert: '--' + #boundary + '--');
					
					#parts->(insert: #tempout, 1);
				
				else: (#related == true);
				
					// Merge the single multipart/related part
					local: 'tempout' = #parts->first->data;
					local: 'temppos' = #tempout->(find: '\r\n--');
					#tempout->(remove: 1, #temppos);
					#tempout->(removeleading: '\r');
					#tempout->(removeleading: '\n');
					#parts->(remove);
					#parts->(insert: #tempout);
				/if;

				// Add the constructed parts to the output

				if: (#multi_part == true);
					#output->(insert: '--' + self->'mime_boundary');
				/if;
				
				iterate: #parts, (local: 'part');

					if: #part->(isa: 'array');
						#output->(insertfrom: #part->iterator);
					else: #part->(isa: 'compose');
						#output->(insert: #part->data);
					else;
						#output->(insert: #part);
					/if;
				
					if: (#multi_part == true);
						#output->(insert: '');
						if: loop_count == #parts->size;
							#output->(insert: '--' + self->'mime_boundary' + '--');
						else;
							#output->(insert: '--' + self->'mime_boundary');
						/if;
					/if;
				/iterate;
				
			/if;

			// Merge the output into a byte stream

			local: 'return' = bytes;
			local: 'valid' = (string_validcharset: self->'mime_charset');
			iterate: #output, (local: 'temp');
				if: #temp->(isa: 'bytes');
					#return += #temp;
				else: (#valid == true);
					#return += (bytes: (string: #temp), self->'mime_charset');
				else;
					#return += (bytes: (string: #temp));
				/if;
				#return += '\r\n';
			/iterate;

			self->'mime_msg' = #return;
			
			return: @#return;

		/define_tag;

		define_tag: 'recipients';
			return: self->'mime_recipients';
		/define_tag;

		define_tag: 'from';
			return: self->'mime_from';
		/define_tag;

		define_tag: 'onConvert';
			return: self->data;
		/define_tag;
		
		define_tag: 'find', -required='value';
			return: self->properties->first->(find: #value);
		/define_tag;

	/define_type;

	// [Email_Send]
	//
	// This tag is a drop-in replacement for [Email_Send].  It uses [Email_Compose] on the back-end in order
	// to create the message to be sent.
	//

	Define_Tag: 'Send',
		-Priority='replace',
		-namespace = 'email_',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority',
		-Optional='to',
		-Optional='cc',
		-Optional='bcc',
		-Optional='from',
		-Optional='replyto',
		-Optional='sender',
		-Optional='subject',
		-Optional='body',
		-Optional='html',
		-Optional='transferencoding',
		-Optional='contenttype',
		-Optional='characterset',
		-Optional='attachments',
		-Optional='extramimeheaders',
		-Optional='simpleform',
		-Optional='tokens',
		-Optional='merge',
		-Optional='date';

		fail_if: ((local_defined: 'from') && (!#from->size)) || (!(local_defined: 'from')), -1, '[Email_Send] Requires a -From address.';
		fail_if: !(local_defined: 'subject'), -1, '[Email_Send] Requires a -Subject.';
		fail_if: !(((local_defined: 'to') && (#to->size > 0)) || ((local_defined: 'cc') && (#cc->size > 0)) || ( (local_defined: 'bcc') && (#bcc->size > 0))), -1, '[Email_Send] Requires a recipient.';

		local(
			'_params' = params,
			'_host' = string(local('host')),
			'_username' = string(local('username')),
			'_password' = string(local('password'))
		);

		if((!#_username->size) && (!#_password->size) && (#_host >> '||') && (#_host->split('||')->size == 3));
			#_username = #_host->split('||')->get(1);
			#_password = #_host->split('||')->get(2);
			#_host = #_host->split('||')->get(3);
		else(#_host >> '||');
			#_host = #_host->split('||')->last;
		/if;

		if: (local_defined: 'simpleform');
			#_params->(removeall: '-simpleform');
			#_params->(removeall: '-body');

			if: !(local_defined: 'body');
				local: '_body' = string;
			else;
				local: '_body' = #body + '\n\n';
			/if;

			iterate: action_params, (local: 'i');
				if: !#i->first->(beginswith: '-');
					#_body += #i->first ': ' #i->second '\n';
				/if;
			/iterate;

			#_body += '\nForm was submitted by: ' client_ip '\n';
			#_body += 'Form was submitted on: ' date->(format: '%-D %-r %Z') '\n';
		
			#_params->(removeall: '-body');
			#_params->(insert: -body = @#_body);
		/if;

		local: 'data' = @(\email_compose->(run: -params=#_params, -name='email_compose'));
		local: 'mime_from' = #data->from;
		local: 'mime_recipients' = #data->recipients;
		local: 'mime_text' = #data->data;

		fail_if: (#mime_text == ''), -1, '[Email_Send] The MIME encoding for the email message could not be generated.';
		fail_if: (#mime_recipients == ''), -1, '[Email_Send] Requires one or more valid email addresses for recipients.';
		fail_if: (#mime_from == ''), -1, '[Email_Send] Requires a -From parameter with the sender\'s email address.';

		if: ((local_defined: 'date') == false) || (#date->(isa: 'date') == false);
			local: 'date' = date;
		/if;

		if: ((local: 'priority') == 'immediate') || (local_defined: 'immediate');
			Email_Immediate:
				-data=#mime_text,
				-from=#mime_from,
				-recipients=#mime_recipients,
				-host = #_host,
				-port = (local: 'port'),
				-timeout = (local: 'timeout'),
				-username = #_username,
				-password = #_password,
				-tokens = (local: 'tokens'),
				-merge = (local: 'merge');
		else;
			Email_Queue:
				-data=#mime_text,
				-from=#mime_from,
				-recipients=#mime_recipients,
				-priority=(local: 'priority'),
				-host = #_host,
				-port = (local: 'port'),
				-timeout = (local: 'timeout'),
				-username = #_username,
				-password = #_password,
				-tokens = (local: 'tokens'),
				-merge = (local: 'merge'),
				-date = #date;
		/if;
	/Define_Tag;

	//
	// [Email_Queue]
	//

	Define_Tag: 'Queue', -Priority='replace',
		-Privileged,
		-namespace = 'email_',
		-Optional='data',
		-Optional='recipients',
		-Optional='from',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority',
		-Optional='tokens',
		-Optional='merge',
		-Optional='date';

		fail_if: !(local_defined: 'data'), -1, '[Email_Queue] Requires a -Data parameter containing a MIME encoded email message.';
	
		if: #data->(isa: 'compose');		
			local: '_data' = #data->data;
			if: !(local_defined: 'recipients');
				local: 'recipients' = #data->recipients;
			/if;
			if: !(local_defined: 'from');
				local: 'from' = #data->from;
			/if;
		else;
			local: '_data' = #data;
		/if;

		fail_if: ((local: 'recipients') == ''), -1, '[Email_Queue] Requires a -Recipients parameter with one or more email addresses.';
		fail_if: ((local: 'from') == ''), -1, '[Email2_Queue] Requires a -From parameter with the sender\'s email address.';

		if: (local_defined: 'host');
			local: 'smtp_host' = (if_null: (local: 'host'), '');
			local: 'smtp_port' = (if_null: (local: 'port'), '');
			local: 'smtp_username' = (if_null: (local: 'username'), '');
			local: 'smtp_password' = (if_null: (local: 'password'), '');
			local: 'smtp_timeout' = (if_null: (local: 'timeout'), '');
		/if;

		local: 'queue_priority' = 2;
		if: (local_defined: 'priority') && (#priority->(isa: 'string'));
			if: #priority >> 'high';
				local: 'queue_priority' = 1;
			else; #priority >> 'low';
				local: 'queue_priority' = 3;
			/if;
		/if;
		
		if: ((local_defined: 'date') == false) || (#date->(isa: 'date') == false);
			local: 'date' = date;
		/if;

		local: 'id' = lasso_uniqueid;
		
		inline:
			-log=$__startup_loglevel__,
			-database='site',
			-table='smtp_queue',
			-keyfield='id',
			-sql='INSERT INTO smtp_queue \
				(id_site,mime_text,mime_recipients,mime_from,smtp_host,smtp_port,smtp_timeout,smtp_username,smtp_password,queue_date,queue_status,queue_error,queue_number,queue_priority,create_date,merge_tokens,merge_array) \
				VALUES (\ 
				\'' + #id + '\',
				X\'' + (encode_hex: #_data) + '\', \
				\'' + (encode_sql92: (if_false: #recipients->(isa: 'array'), (array: #recipients), #recipients)->(join: ',')) + '\', \
				\'' + (encode_sql92: #from) + '\', \
				\'' + (encode_sql92: (local: 'smtp_host')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_port')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_timeout')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_username')) + '\', \
				\'' + (encode_sql92: (local: 'smtp_password')) + '\', \
				\'' + (encode_sql92: #date->(format: '%Q %T')) + '\', \
				\'queued\', \
				\'\', \
				0, \
				3, \
				\'' + (encode_sql92: date->(format: '%Q %T')) + '\', \
				X\'' + (encode_hex: (local: 'tokens')->serialize) + '\', \
				X\'' + (encode_hex: (local: 'merge')->serialize) + '\');';
			if: error_code != 0;
				log_critical: 'Queue Error: ' error_msg;
			else;
				var: '_email_result' = #id;
			/if;
		/inline;

		if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
			Global: '__SMTP_Queue_Status__' = 'Active';
		/if;

	/Define_Tag;

	//
	// [Email_Immediate]
	//
	
	Define_Tag: 'Immediate', -Priority='replace',
		-privileged,
		-namespace = 'email_',
		-Required='data',
		-Optional='recipients',
		-Optional='from',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout';

		error_reset;

		fail_if: (#data == ''), -1, '[Email_Immediate] Requires a -Data parameter containing a MIME encoded email message.';

		if: #data->(isa: 'compose');		
			local: '_data' = #data->data;
			if: !(local_defined: 'recipients');
				local: 'recipients' = #data->recipients;
			/if;
			if: !(local_defined: 'from');
				local: 'from' = #data->from;
			/if;
		else;
			local: '_data' = #data;
		/if;

		fail_if: (#recipients == ''), -1, '[Email_Immediate] Requires a -Recipients parameter with one or more email addresses.';
		fail_if: (#from == ''), -1, '[Email_Immediate] Requires a -From parameter with the sender\'s email address.';

		if: ((local: 'host') != '');
			// Use all field values if host is set
			local: 'smtp_host' = (local: 'host');
			local: 'smtp_port' = (if_empty: (local: 'port'), 25);
			local: 'smtp_username' = (if_empty: (local: 'username'), '');
			local: 'smtp_password' = (if_empty: (local: 'password'), '');
			local: 'smtp_timeout' = (if_empty: (local: 'timeout'), 15);
		else;
			// Otherwise use all preference values
			Local: 'prefs' = (map: 
					'lasso_smtp_host' = '',
					'lasso_smtp_port' = 25,
					'lasso_smtp_username' = '',
					'lasso_smtp_password' = '',
					'lasso_smtp_timeout' = 15,
				);					
			Local: 'prefs' = (admin_getpref: #prefs);
			local: 'smtp_host' = #prefs->(find: 'lasso_smtp_host');
			local: 'smtp_port' = #prefs->(find: 'lasso_smtp_port');
			local: 'smtp_username' = #prefs->(find: 'lasso_smtp_username');
			local: 'smtp_password' = #prefs->(find: 'lasso_smtp_password');
			local: 'smtp_timeout' = #prefs->(find: 'lasso_smtp_timeout');
		/if;

		fail_if: (#smtp_host == ''), -1, '[Email_Immediate] Requires a -Host parameter.';

		// Reality Check
		if: ((integer: #smtp_port) <= 0);
			local: 'smtp_port' = 25;
		/if;
		if: ((integer: #smtp_timeout) <= 0);
			local: 'smtp_timeout' = 15;
		/if;

		// Mail Merge
		protect;
			if: (local_defined: 'tokens');
				local: '_data' = (email_merge: #_data, -tokens=#tokens);
			/if;
		/protect;
		
		protect;
			local: 'smtp' = (email_smtp);
			handle_error;
				fail: -1, '[Email_Immediate] Failed to initialize SMTP connector.';
			/handle_error;
		/protect;

		protect;
			if: (#smtp_username != '') && (#smtp_password != '');
				#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port), -username=#smtp_username, -password=#smtp_password, -timeout=(integer: #smtp_timeout));
			else;
				#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port), -timeout=(integer: #smtp_timeout));
			/if;
			handle_error;
				fail: -1, '[Email_Immediate] Failed to connect to host "' + #smtp_host + '".';
			/handle_error;
		/protect;

		protect;
			#smtp->(send: -from=#from, -recipients=#recipients, -message=#_data + '\r\n');
			handle_error;
				fail: -1, '[Email_Immediate] Error sending message: ' + error_code + ' ' + error_msg;
			/handle_error;
		/protect;

		handle: (local: 'smtp')->(isa: 'smtp');
			protect;
				#smtp->close;
			/protect;
		/handle;
		
	/Define_Tag;
	
	Define_Tag: 'Result',
			-namespace = 'email_';
		return: (var: '_Email_Result');
	/Define_Tag;

	Define_Tag: 'Status',
			-privileged,
			-namespace = 'email_',
			-optional='id',
			-optional='errors',
			-optional='full';
			
		if: !(local_defined: 'id');
			local: 'id' = (var: '_Email_Result');
		/if;
		
		fail_if: (#id == ''), -1, 'An email id must be specified to use this tag.';
		
		inline: -log=$__startup_loglevel__,
				-search,
				-database='site', 
				-table='smtp_queue',
				-keyfield='id',
				-op='eq', 'id_site' = #id,
				-maxrecords=1,
				-returnfield='id',
				-returnfield='queue_status';
			if: (found_count) == 0;
				return: 'sent';
			else;
				return: (field: 'queue_status');
			/if;
		/inline;
	/Define_Tag;
	
	Define_Tag: 'Token',
			-namespace='email_',
			-required='token';
		Return: '\r\n#' + (_substring: (string_replaceregexp: #token, -find='\\W', -replace=''), 1, 64) + '#\r\n';
	/Define_Tag;
	
	Define_Tag: 'Merge',
			-namespace='email_',
			-required='data',
			-required='tokens',
			-optional='charset',
			-optional='transferencoding';
		if: #data !>> '#';
			return: #data;
		/if;
		local: '_data' = #data;
		iterate: #tokens->keys, (local: 'token');
			local: 'find' = '#' + (_substring: (string_replaceregexp: #token, -find='\\W', -replace=''), 1, 64)+ '#';
			if: #_data >> ('\r\n' + #find + '\r\n');
				if: (local_defined: 'transferencoding') && (#transferencoding !>> 'quoted-printable');
					local: 'replace' = #tokens->(find: 'token');
				else: (local_defined: 'charset');
					local: 'replace' = (encode_quotedprintable: #tokens->(find: #token), -charset=(local: 'charset'));
				else;
					local: 'replace' = (encode_quotedprintable: #tokens->(find: #token));
				/if;
				#_data->(replace: ('\r\n' + #find + '\r\n'), ('=\r\n' + #replace + '=\r\n'));
			/if;
			if: #_data >> #find;
				if: (local_defined: 'transferencoding') && (#transferencoding !>> 'quoted-printable');
					local: 'replace' = #tokens->(find: 'token');
				else: (local_defined: 'charset');
					local: 'replace' = (encode_qheader: -name='Token', -value=(string: #tokens->(find: #token)), -charset=(local: 'charset'));
				else;
					local: 'replace' = (encode_qheader: -name='Token', -value=(string: #tokens->(find: #token)));
				/if;
				#replace -= 'Token: ';
				#_data->(replace: #find, #replace);
			/if;
		/iterate;
		return: @#_data;
	/Define_Tag;
	
	Define_Tag: 'Batch',
			-namespace='Email_',
			-Container,
			-description='Pauses the email queue while a batch of emails is queued for sending.';
		error_reset;
		if: ((Global: '__SMTP_Queue_Status__') !>> 'Abort') && ((Global: '__SMTP_Queue_Status__') !>> 'Pause');
			Global: '__SMTP_Queue_Status__' = 'Pause Batch';
		/if;
		Protect;
			Local: 'output' = Run_Children;
			Handle_Error;
				#output += '[Email_Batch] Caught error ' + error_code + ' ' + error_msg;
			/Handle_Error;
		/Protect;
		if: ((Global: '__SMTP_Queue_Status__') >> 'Batch');
			Global: '__SMTP_Queue_Status__' = 'Active';
		/if;
		Return: @#Output;
	/Define_Tag

?>
