<?LassoScript

	define_tag('hmac',
			-required = 'password',
			-required = 'token',
			-optional = 'digest',
			-optional = 'base64',
			-optional = 'hex',
			-optional = 'cram',
			-namespace = 'encrypt_');

		local(
			'pbytes' = bytes,
			'ibytes' = bytes,
			'obytes' = bytes,
			'output' = bytes);

		if(local_defined: 'digest');
			fail_if(cipher_list(-digest) !>> #digest, -9956, '[Encrypt_HMAC] The digest "' (string: #digest) + '" is not supported by Lasso on this machine.  Supported digest types include (' + (cipher_list: -digest)->(join: ', ') + ').'); 
		else;
			local: 'digest' = 'md5';
		/if;

		if(#password->size > 64);
			#pbytes = cipher_digest(#password, -digest=#digest);
		else: #password->isa('bytes');
			#pbytes = #password;
		else;
			#pbytes->importstring(#password, 'iso-8859-1');
		/if;

		iterate: #pbytes, local('byte');
			#ibytes->import8bits(integer(#byte)->bitxor(0x36) &);
			#obytes->import8bits(integer(#byte)->bitxor(0x5c) &);
		/iterate;

		if(#pbytes->size < 64);
			loop(64 - #pbytes->size);
				#ibytes->import8bits(0x36);
				#obytes->import8bits(0x5c);
			/loop;
		/if;

		if: #token->(isa: 'bytes');
			#ibytes += #token;
		else;
			#ibytes->importstring(#token, 'iso-8859-1');
		/if;

		#obytes += cipher_digest(#ibytes, -digest=#digest);

		#output = cipher_digest(#obytes, -digest=#digest);

		if(local_defined('hex') && ((#hex === null) || (#hex != false)));
			return('0x' + string_lowercase(encode_hex(#output)));
		else(local_defined('cram') && ((#cram === null) || (#cram != false)));
			return(string_lowercase(encode_hex(#output)));
		else(local_defined('base64') && ((#base64 === null) || (#base64 != false)));
			return(encode_base64(#output));
		else;
			return(@#output);
		/if;

	/define_tag;

	define_tag('crammd5',
			-required = 'password',
			-required = 'token',
			-namespace = 'encrypt_');
		
		return(encrypt_hmac(-password=#password, -token=#token, -digest='md5', -cram));
		
	/define_tag;

?>

