<?lassoscript

	define_type('smtp',
		-namespace = 'email_');

		local(
			'net' = net,
			'host' = 'localhost',
			'port' = 25,
			'timeout' = 60,
			'count' = 0);


		//->onCreate
		// nothing here yet
		define_tag('oncreate');
		/define_tag;


		// ->onConvert
		// We just print out the object host and port
		define_tag('onconvert');
			if(params->get(1) == 'string');
				return('SMTP (' self->'host' ':' self->'port' ')');
			/if;
		/define_tag;


		// ->onDestroy
		// We just print out the object host and port

		define_tag('ondestroy');
			protect;
				self->'net'->close;
			/protect;
		/define_tag;


		// ->Open
		// Open the connection and send basic commands to start the session
		define_tag('open',
			-required = 'host',
			-type = 'string',
			-optional = 'port',
			-type = 'integer',
			-optional = 'timeout',
			-type = 'integer',
			-optional = 'username',
			-type = 'string',
			-optional = 'password',
			-type = 'string',
			-optional='ssl');

			(global('__smtp_debug__') == true) ? log_always('SMTP OPEN ' + params);
			
			local(
				'net' = @self->'net',
				'auth' = false,
				'authtype' = '',
				'ehlo' = false,
				'commands' = string);

			if(global_defined('__smtp_ip__') && $__smtp_ip__->size);
				if((string_replaceregexp: $__smtp_ip__, -find='\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}', -replace='') == '');
					local('__local_smtp_ip__' = '[' + $__smtp_ip__ + ']');
				else;
					local('__local_smtp_ip__' = $__smtp_ip__);
				/if;
			else(global('__lassoservice_ip__')->size);
				local('__local_smtp_ip__' = '[' + $__lassoservice_ip__ + ']');
			else;
				local('__local_smtp_ip__' = '[127.0.0.1]');
			/if;

			if(local_defined('username') && local_defined('password') && #username->size && #password->size);
				#auth = true;
			else(local_defined('username') || local_defined('password'));

				if(local_defined('username') && local_defined('password') && (!#username->size) && (!#password->size));
					error_code = -1;
					error_msg = '[Email_SMTP] Username and Password were empty or null';
					return(false);
				else((local_defined('username') && (!#username->size)) || (!local_defined('username')));
					error_code = -1;
					error_msg = '[Email_SMTP] Username was empty or null';
					return(false);
				else((local_defined('password') && (!#password->size)) || (!local_defined('password')));
					error_code = -1;
					error_msg = '[Email_SMTP] Password was empty or null';
					return(false);
				/if;

			/if;

			self->'host' = #host;

			if(local_defined('port') && (#port->type == 'integer'));
				self->'port' = #port;
			/if;

			if(local_defined('timeout'));
				self->'timeout' = #timeout;
			/if;

			if: (local_defined: 'ssl');
				local: 'ssl' = (#ssl === null) || (#ssl != false);
			else;
				local: 'ssl' = false;
			/if;

			local_remove('host');
			local_remove('port');
			local_remove('timeout');

			local(
				'host' = @self->'host',
				'port' = @self->'port',
				'timeout' = @self->'timeout');

			// test EHLO [ ... ] (Use minimum 60 second connection timeout)
			if(#net->connect(#host, #port) == net_connectok);
				if(!self->command(-expect = 220, -timeout = (integer(math_min(60, #timeout)))));
					return(false);
				/if;

				#commands = self->command(-send = 'EHLO ' + #__local_smtp_ip__ + '\r\n', -expect = 250, -read, -multi);
				if(#commands !== false);
					if(#commands->find('\r\n'));
						local('eol' = '\r\n');
					else(#commands->find('\n'));
						local('eol' = '\n');
					else;
						local('eol' = '\r');
					/if;

					#ehlo = true;

					if(#ssl && #commands >> 'STARTTLS');
						self->command(-send = 'STARTTLS\r\n', -expect = 220);
						#net->settype(net_typessl);
					/if;

					if(#auth);
						iterate(#commands->split(#eol), local('i'));
							if(!#authtype->size);
								if((#i >> 'AUTH ') && (#i >> 'CRAM-MD5'));
									#authtype = 'cram-md5';
								else((#i >> 'AUTH ') && (#i >> 'DIGEST-MD5'));
									#authtype = 'digest-md5';
								else((#i >> 'AUTH ') && (#i >> 'LOGIN'));
									#authtype = 'login';
								/if;
							/if;
						/iterate;

						if(!#authtype->size);
							#authtype = 'login';
						/if;
					/if;
				else;
					#net->close;
				/if;

			/if;

			if(!#ehlo);
				if(#net->connect(#host, #port) == net_connectok);
					if(!self->command(-expect = 220, -timeout = (integer(math_min(60, #timeout)))));
						return(false);
					/if;

					if(!self->command(-send = 'HELO ' + #__local_smtp_ip__ + '\r\n', -expect = 250));
						#net->close;
						return(false);
					/if;

				else;
					#net->close;
					error_code = -8001;
					error_msg = '[Email_SMTP] Could not connect to server';
					return(false);
				/if;
			/if;

			// add auth stuff here
			if(#auth);
				if(#authtype == 'digest-md5');
					local(
						'authresp' = self->command(-send = 'AUTH DIGEST-MD5\r\n', -expect = 334, -read),
						'cram' = null);
					if(#authresp !== false);
						#authresp->trim;
						#authresp = string(decode_base64(#authresp->split(' ')->get(2)));
						(global('__smtp_debug__') == true) ? log_always('SMTP DIGEST-MD5 RESP ' + #authresp);
						local: 'digest' = Email_DigestChallenge(#authresp);
						#digest->(insert: 'username' = #username);
						#digest->(insert: 'password' = #password);
						#digest->(insert: 'digest-uri' = 'smtp/' + self->'host');
						#digest->(insert: 'cnonce' = (encrypt_md5: lasso_uniqueid));
						#digest->(insert: 'nc' = '00000001');
						#digest->(insert: 'charset' = 'utf-8');
						(global('__smtp_debug__') == true) ? log_always('SMTP DIGEST-MD5 SEND ' + email_digestresponse(#digest));
						#cram = encode_base64(email_digestresponse(#digest));
						if(!self->command(-send = #cram + '\r\n', -expect = 334));
							#net->close;
							return(false);
						/if;
						if(!self->command(-send = '\r\n', -expect = 235));
							#net->close;
							return(false);
						/if;
					else;
						#net->close;
						return(false);
					/if;
				else(#authtype == 'cram-md5');
					local(
						'authresp' = self->command(-send = 'AUTH CRAM-MD5\r\n', -expect = 334, -read),
						'cram' = null);
					if(#authresp !== false);
						#authresp->trim;
						#authresp = string(decode_base64(#authresp->split(' ')->get(2)));
						(global('__smtp_debug__') == true) ? log_always('SMTP CRAM-MD5 RESP ' + #authresp);
						(global('__smtp_debug__') == true) ? log_always('SMTP CRAM-MD5 SEND ' + #username + ' ' + encrypt_crammd5(#password, #authresp));
						#cram = encode_base64(#username ' ' encrypt_crammd5(#password, #authresp));
						if(!self->command(-send = #cram + '\r\n', -expect = 235));
							#net->close;
							return(false);
						/if;
					else;
						#net->close;
						return(false);
					/if;
				else(#authtype == 'login');
					local(
						'auth_user' = encode_base64(#username),
						'auth_pass' = encode_base64(#password));

					if(!self->command(-send = 'AUTH LOGIN\r\n', -expect = 334));
						#net->close;
						return(false);
					/if;

					if(!self->command(-send = #auth_user + '\r\n', -expect = 334));
						#net->close;
						return(false);
					/if;

					if(!self->command(-send = #auth_pass + '\r\n', -expect = 235));
						#net->close;
						return(false);
					/if;
				/if;
			/if;

			return(true);
		/define_tag;


		// ->Command
		// This will send a command to the SMTP server, and return the result if bad
		define_tag('command',
			-optional = 'send',
			-optional = 'expect',
			-optional = 'multi',
			-optional = 'read',
			-optional = 'timeout');

			local(
				'net' = @self->'net',
				'host' = @self->'host',
				'port' = @self->'port',
				'result' = null,
				'status' = null,
				'bytes' = bytes);

			if(!local_defined('timeout'));
				local('timeout' = @self->'timeout');
			/if;

			if(!local_defined('expect'));
				local('expect' = null);
			/if;

			if(local_defined('send') && #send->size);
				#status = #net->wait(#timeout, net_waitwrite);
				if(#status->isa('integer') && (#status == net_waittimeout));
					error_code = -8001;
					error_msg = '[Email_SMTP] SMTP server time out waiting for write ("' + (string: (local: 'send'))->trim & + '", ' + (local: 'expect') + ((local_defined:'multi') ? ', multi') + ((local_defined:'read') ? ', read') + ').';
					return(false);
				/if;

				// convert text to iso-8859-1 character set
				#bytes->importas(#send, 'iso8859-1');

				(global('__smtp_debug__') == true) ? log_always('SMTP -> ' + string_replace(string_replace(#send, -find='\n', -replace='\\n'), -find='\r', -replace='\\r'));

				null(#net->write(#bytes));
			/if;

			#status = #net->wait(#timeout, net_waitread);
			if(#status->isa('integer') && (#status == net_waittimeout));
				error_code = -8001;
				error_msg = '[Email_SMTP] SMTP server time out waiting for read ("' + (string: (local: 'send'))->trim & + '", ' + (local: 'expect') + ((local_defined:'multi') ? ', multi') + ((local_defined:'read') ? ', read') + ').';
				return(false);
			/if;

			#result = #net->read(32768);

			// Multi-line responses end with a line that contains e.g. "250"
			if(local_defined('multi'));
				while(#result !>> (#expect + ' '));
					#status = #net->wait(#timeout, net_waitread);
					if(#status->isa('integer') && (#status == net_waittimeout));
						loop_abort;
					/if;
					#result += #net->read(32768);
					if(loop_count > 100);
						loop_abort;
					/if;
				/while;
			/if;

			(global('__smtp_debug__') == true) ? log_always('SMTP <- ' + string_replace(string_replace(#result, -find='\n', -replace='\\n'), -find='\r', -replace='\\r'));
			
			if(#result->size);
				local('answerlist' = integer(string_findregexp(#result, -find='[0-9]+')->get(1)));
				if(#answerlist != #expect);
					(global('__smtp_debug__') == true) ? log_always('SMTP ERROR EXPECTED ' + #expect + ' GOT ' + #answerlist);
					error_code = #answerlist;
					error_seterrormessage('[Email_SMTP] Server Error: ' #result);
					return(false);
				else;
					error_code = 0;
				/if;
			else;
				(global('__smtp_debug__') == true) ? log_always('SMTP ERROR EMPTY');
				error_msg = '[Email_SMTP] SMTP server sent empty response (' + (local: 'send') + ', ' + (local: 'expect') + ', ' + (local_defined:'multi') + ', ' + (local_defined:'read') + ').';
				error_code = -8001;
				return(false);
			/if;

			if(local_defined('read'));
				return(#result);
			else;
				return(true);
			/if;

		/define_tag;


		// ->Send
		// We send the message here
		define_tag('send',
			-required = 'from',
			-type = 'string',
			-required = 'recipients',
			-type = 'array',
			-required = 'message');

			local(
				'net' = @self->'net',
				'timeout' = @self->'timeout',
				'_message' = (string_replace: #message, -find='\r\n.', -replace='\r\n..'));

			if(!self->command(-send = 'MAIL FROM:<' #from '>\r\n', -expect = 250));
				return(false);
			/if;

			iterate(#recipients, local('i'));
				if(!self->command(-send = 'RCPT TO:<' #i '>\r\n', -expect = 250));
					return(false);
				/if;
			/iterate;

			if(!self->command(-send = 'DATA\r\n', -expect = 354));
				return(false);
			/if;

			if(#net->wait(#timeout, net_waitwrite) == net_waitwrite);
				#net->write(#_message);
			/if;

			(global('__smtp_debug__') == true) ? log_always('SMTP DATA ' + #_message->size + ' BYTES');

			if(!self->command(-send = '\r\n.\r\n', -expect = 250));
				return(false);
			/if;

			return(true);
		/define_tag;


		// ->Close
		// Send QUIT and close the connection
		define_tag('close');
			local('net' = @self->'net');
			protect;
				self->command(-send = 'QUIT\r\n', -expect = 221);
			/protect;
			protect;
				#net->close;
				return(true);
			/protect;

			return(false);
		/define_tag;

	/define_type;

?>