<?lassoscript
/*
	->reset - Sets the iterators position back to its initial location.
	->atEnd - Returns true if the iterator can no longer step forward.
	->atBegin - Returns true if the iterator can no longer step backward.
	->forward - Attempts to step the iterator forward and returns true if it was successful.
	->backward Attempts to step the iterator backward and returns true if it was successful.
	->atFarLeft - Returns true if the iterator can no longer step left.
	->atFarRight - Returns true if the iterator can no longer step right.
	->left - Attemps to step the iterator left and returns true if it was successful.
	->right - Attempts to step the iterator right and returns true if it was successful.
	->atTop - Returns true if the iterator can no longer step upwards.
	->atBottom - Returns true if the iterator can no longer step downwards.
	->up - Attemps to step the iterator upwards and returns true if it was successful.
	->down - Attemps to step the iterator downwards and returns true if it was successful.
	->key - Returns the key associated with the element at the current position, if available.
	->value - Returns the element at the current position.
	->removeCurrent - Removes the element at the current iterator position from the host container. The iterator position is advanced to the next element.
	->insertAtCurrent:<any elem> - inserts the given element into the host container at the current iterator position.
*/

define_type:'iterator', -prototype, -description='Generic iterator type. Requires one parameter which is the container to iterate over. If the container has its own iterator implementation, it will be used instead. \
										The container must support random access to elements. The container must support the ->size member and random access to elements using ->get. An optional matcher can also be provided upon creation.';
	local:'pos'=1;
	local:'container'=null;
	local:'matcher'=null;
	local:'hasMatcher'=false;
	
	define_tag:'onCreate', -required='c', -optional='matcher';
		// see if #c has its own iterator;
		error_push;
		protect;
			if:local_defined('matcher');
				local:'tst' = @#c->iterator(#matcher);
			else;
				local:'tst' = @#c->iterator;
			/if;
			if:#tst != null;
				self->steal(#tst);
			/if;
			handle_error;
				self->'container' = @#c;
				if:local_defined('matcher');
					self->'hasMatcher' = true;
					self->'matcher' = @#matcher;
					if:!self->atEnd && self->'matcher' != self->value;
						self->forward;
					/if;
				/if;
			/handle_error;
		/protect;
		error_pop;
	/define_tag;
	
	define_tag:'reset';
		self->'pos' = 0;
	/define_tag;
	
	define_tag:'atEnd';
	   return: self->'pos' > self->'container'->size;
	/define_tag;
	
	define_tag:'atBegin';
	   return: self->'pos' == 1;
	/define_tag;
	
	define_tag:'forward';
		if:self->atEnd;
			return:false;
		/if;
		if:self->'hasMatcher';
			while:!self->atEnd;
				++self->'pos';
				if:self->atEnd || self->'matcher' == self->value;
					loop_abort;
				/if;
			/while;
		else;
			++self->'pos';
		/if;
		return:true;
	/define_tag;
	
	define_tag:'backward';
	   if:self->atBegin;
	       return:false;
	   /if;
		if:self->'hasMatcher';
			while:!self->atBegin;
				--self->'pos';
				if:self->atEnd || self->'matcher' == self->value;
					loop_abort;
				/if;
			/while;
		else;
			--self->'pos';
		/if;
	   return:true;
	/define_tag;
	
	define_tag:'key';
	   local:'item' = @self->'container'->get(self->'pos');
	   if:#item->isa('pair');
	       return:#item->first;
	   /if;
	   return:null;
	/define_tag;
	
	define_tag:'value';
	   local:'item' = @self->'container'->get(self->'pos');
	   if:#item->isa('pair');
	       return:#item->second;
	   /if;
	   return:#item;
	/define_tag;	
/define_type;

define_type:'reverseiterator', -prototype, -description='Generic reverseiterator type. Requires one parameter which is the container to iterate over. If the container has its own reverseiterator implementation, it will be used instead. \
										The container must support random access to elements. The container must support the ->size member and random access to elements using ->get. An optional matcher can also be provided upon creation.';
	local:'pos'=1;
	local:'container'=null;
	local:'matcher'=null;
	local:'hasMatcher'=false;
	
	define_tag:'onCreate', -required='c', -optional='matcher';
		// see if #c has its own iterator;
		error_push;
		protect;
			if:local_defined('matcher');
				local:'tst' = @#c->reverseiterator(#matcher);
			else;
				local:'tst' = @#c->reverseiterator;
			/if;
			if:#tst != null;
				self->steal(#tst);
			/if;
			handle_error;
				self->'container' = @#c;
				self->'pos' = #c->size;
				if:local_defined('matcher');
					self->'hasMatcher' = true;
					self->'matcher' = @#matcher;
					if:!self->atEnd && self->'matcher' != self->value;
						self->forward;
					/if;
				/if;
			/handle_error;
		/protect;
		error_pop;
	/define_tag;
	
	define_tag:'reset';
		self->'pos' = self->'container'->size;
	/define_tag;
	
	define_tag:'atEnd';
	   return: self->'pos' < 1;
	/define_tag;
	
	define_tag:'atBegin';
	   return: self->'pos' == self->'container'->size;
	/define_tag;
	
	define_tag:'forward';
	   if:self->atEnd;
	       return:false;
	   /if;
		if:self->'hasMatcher';
			while:!self->atEnd;
				--self->'pos';
				if:self->atEnd || self->'matcher' == self->value;
					loop_abort;
				/if;
			/while;
		else;
			--self->'pos';
		/if;
	   return:true;
	/define_tag;
	
	define_tag:'backward';
	   if:self->atBegin;
	       return:false;
	   /if;
		if:self->'hasMatcher';
			while:!self->atBegin;
				++self->'pos';
				if:self->atEnd || self->'matcher' == self->value;
					loop_abort;
				/if;
			/while;
		else;
			++self->'pos';
		/if;
	   return:true;
	/define_tag;
	
	define_tag:'key';
	   local:'item' = @self->'container'->get(self->'pos');
	   if:#item->isa('pair');
	       return:#item->first;
	   /if;
	   return:null;
	/define_tag;
	
	define_tag:'value';
	   local:'item' = @self->'container'->get(self->'pos');
	   if:#item->isa('pair');
	       return:#item->second;
	   /if;
	   return:#item;
	/define_tag;	
/define_type;

?>