<?LassoScript

	// we check for this because if Startup.LassoApp is messed up
	// and not loading, we don't want to rely on something that
	// is defined in Startup.LassoApp. If Global isn't defined,
	// we'll fall back onto Var to create the maps.
	if(lasso_tagexists('global'));
		global(
			'__lassoapp_mime_types__' = null,
			'__lassoapp_comp_types__' = null);
	else;
		var(
			'__lassoapp_mime_types__' = null,
			'__lassoapp_comp_types__' = null);
	/if;

	// we need to keep the mime text/html because if you
	// include any of the below text/... files, the content
	// type of the page will become what we specify here
	// hence if you include a .xml file in your LassoApp
	// and the mime for .xml files was text/xml, the page
	// with the include would come back as a text/xml
	// text/xml document, even though the parent page
	// probably was text/html to begin with
	$__lassoapp_mime_types__ = map(
		'lasso'  =  'text/html',
		'las'    =  'text/html',
		'css'    =  'text/css',
		'txt'    =  'text/html',
		'text'   =  'text/html',
		'htm'    =  'text/htm',
		'html'   =  'text/html',
		'inc'    =  'text/html',
		'incl'   =  'text/html',
		'xml'    =  'text/html',
		'xsd'    =  'text/html',
		'pdf'    =  'application/pdf',
		'jpg'    =  'image/jpeg',
		'jpeg'   =  'image/jpeg',
		'gif'    =  'image/gif',
		'png'    =  'image/png',
		'bmp'    =  'image/bmp',
		'js'	 = 	'text/javascript');

	// fill these out for the sake of completeness
	$__lassoapp_comp_types__ = map(
		'lasso'  =  true,
		'las'    =  true,
		'css'    =  true,
		'txt'    =  true,
		'text'   =  true,
		'htm'    =  true,
		'html'   =  true,
		'inc'    =  true,
		'incl'   =  true,
		'xml'    =  true,
		'xsd'    =  true,
		'pdf'    =  false,
		'jpg'    =  false,
		'jpeg'   =  false,
		'gif'    =  false,
		'png'    =  false,
		'bmp'    =  false,
		'js'	 =	false);


	define_tag('Unitize',
		-required = 'path',
		-required = 'strip',
		-required = 'buff',
		-namespace = 'LassoApp_');

		if(#path !>> '.');
			return;
		/if;

		local('localPath' = #path);
		#localPath->remove(1, #strip->size);
		#localpath->removeLeading('/');

		local('mime' = 'text/html'); // default
		local('comp' = true); // default
		local('ext' = #path->split('.')->last);

		local('mime' = $__lassoapp_mime_types__->find(#ext));
		if(#mime === null);
			local('mime' = 'text/html');
		/if;

		local('comp' = $__lassoapp_comp_types__->find(#ext));
		if((#comp === null) || (#comp == true));
			local('comp' = true);
		/if;

		#buff += '<unit name="' #localpath '" type="' #mime '">\r\n';
		if(#comp);
			#buff += (tag->compile(file_read(#path)) & serialize);
		else;
			// if it's not compiled, we need to make sure that it's a string
			// instead of a byte object
			if(#mime->beginswith('text/'));
				#buff += file_read(#path)->exportstring('utf-8')->serialize;
			else;
				#buff += file_read(#path)->serialize;
			/if;
		/if;
		#buff += '</unit>\r\n';
	/define_tag;


	define_tag('FlattenDir',
		-required = 'root',
		-namespace = 'LassoApp_');
		local('ret' = array);
		iterate: file_listdirectory(#root), local('i');
			if(#i->beginsWith('.'));
				// skip files and directories with a leading period
			else(file_isdirectory(#root + #i));
				#ret->merge(lassoapp_flattendir(#root + #i));
			else;
				#ret->insert(#root + #i);
			/if;
		/iterate;
		return(#ret);
	/define_tag;


	define_tag('Create',
		-required = 'root',
		-required = 'entry',
		-required = 'result',
		-namespace = 'LassoApp_');

		// Fix the root path
		local('_root' = #root);
		#_root->replace('\\', '/');
		if: lasso_version(-lassoplatform) >> 'Win32';
			local('_root' = string_replaceregexp(#_root, -find='([a-zA-Z]{1,2}:)/+', -replace='\\1//', -replaceonlyone));
		/if;
		if(! #_root->endswith('/'));
			#_root += '/';
		/if;
		local('exists' = file_exists(#_root));
		fail_if(file_currenterror(-errorcode) != 0, file_currenterror(-errorcode), '[LassoApp_Create] Root folder - ' + file_currenterror);
		if(#exists == false && (lasso_version(-lassoplatform) !>> 'Win32'));
			local('_root' = '//' + #_root);
		/if;
		fail_if(! file_exists(#_root), error_filenotfound(-errorcode), '[LassoApp_Create] Root folder - ' + error_filenotfound);

		local('_result' = #result);
		#_result->replace('\\', '/');
		if: lasso_version(-lassoplatform) >> 'Win32';
			local('_result' = string_replaceregexp(#_result, -find='([a-zA-Z]:)/+', -replace='\\1//', -replaceonlyone));
		/if;
		local('name' = #_result->split('/')->last);
		local('path' = #_result);
		#path->removetrailing(#name);
		local('exists' = file_exists(#path));
		fail_if(file_currenterror(-errorcode) != 0, file_currenterror(-errorcode), '[LassoApp_Create] Result folder - ' + file_currenterror);
		if(#exists == false);
			local('_result' = '//' + #_result);
			local('path' = '//' + #path);
			local('exists' = file_exists(#path));
		/if;
		fail_if((#exists == false), error_filenotfound(-errorcode), '[LassoApp_Create] Result folder - ' + error_filenotfound);

		local('_entry' = #entry);
		#_entry->replace('\\', '/');
		local('_entry' = #_entry->split('/')->last);

		local('laf' = '<?xml version="1.0" ?><lassoapp name="' #name '">\r\n');

		// entry file first
		local('main' = #_root + #_entry);
		log_detail('LassoApp Building Entry: ' #main);
		lassoapp_unitize(#main, #_root, #laf);
		fail_if(file_currenterror(-errorcode) != 0, file_currenterror(-errorcode), '[LassoApp_Create] Building LassoApp - ' + file_currenterror);

		iterate: lassoapp_flattendir(#_root), local('i');
			if(#i != #main);
				log_detail('LassoApp Building: ' #i);
				lassoapp_unitize(#i, #_root, #laf);
				fail_if(file_currenterror(-errorcode) != 0, file_currenterror(-errorcode), '[LassoApp_Create] Building LassoApp - ' + file_currenterror);
			/if;
		/iterate;

		#laf += '</lassoapp>';

		file_create(#_result, -fileoverwrite);
		file_write(#_result, compress(#laf));
		fail_if(file_currenterror(-errorcode) != 0, file_currenterror(-errorcode), '[LassoApp_Create] Writing result - ' + file_currenterror);
		log_detail('LassoApp Writing: ' #_result);
		lassoapp_dump(#name);
		return(error_code);
	/define_tag;

?>