﻿<?LassoScript

/*
		The ljax_target container tag allows individual sections of a page to be
	targetted and returned to the browser on an as-needed basis. Targettable
	sections are given unique names and wrapped in an ljax_target tag. Any code
	within the ljax_target tag is executed when that target name is specified.
	The page output, after executing all specified targets, should consist of
	zero or more valid XHTML nodes. Each node should contain a unique 'id'
	attribute. This attribute indicates where in the document the node should be
	inserted. The document will be searched for an element with a matching id
	and that node's contents will be replaced with the contants of the new node
	that was returned from the server. Multiple target names can be specified to
	return several different sections from the document.

		By specifying different targets, the contents within the ljax_target
	tags can be toggled on and off resulting in it either being included or
	excluded from the results for the containing page. If a ljax_target is
	turned off based on the specified -targets, then its contents are not
	executed. This means that any ljax_target tags contained within the
	turned-off ljax_target tag will never be executed no matter which targets
	they accept. Because of this, nested ljax_targets can only be used to make
	the output more resrictive.

	The data for a LJAX target is requested from within the client-side
	JavaScript by using the Lasso.includeTarget() function, located in the
	'lasso.js' file. That function accepts the following parameters:
	
	'targets' - The name or an array of the names of the target(s) to retrieve.
	'options' - An object containing properties for the following:
		'args' - The argument string or an element such as a form, anchor or 
		other input element that will be used to create the argument string 
		which will be sent with the request
		
		'func' - If this optional function is provided, it will be called to
		handle the resulting data. This function will be passed the
		XMLHttpRequest object used for the request and the value of the 'param'
		parameter
		
		'param' - The parameter which is passed to the 'func' to handle the 
		resulting data. This is only utilized if 'func' is provided
		
		'afterFunc' - This optional function will be called, with the 
		XMLHttpRequest as it's only parameter after completing the include.
		
		'argsoverride' - Optional object containing the values that will be 
		substituted for the matching values found in the 'args' object. For 
		example, if argsoverride contained a property x with a value of y, and
		the args object was a form which contained an input field x, the value
		of y would be used regardless of the actual value of x in the form.

// above here, always executed
ljax_target([-anytarget]); 
	// everything in here executed as long as there is some LJAX -target
	// it doesn't matter what target name(s)
/ljax_target;
// below here, always executed

ljax_target([-target=]'target_name_1');
	// this only executes if -target=target_name_1 was given
/ljax_target;

ljax_target([-target=]'target_name_2', -nooutput);
	// this only executes if -target=target_name_1 was given but it does not 
	// output any data to the page
/ljax_target;

ljax_target(-notarget);
	// this only executes if there is no LJAX -target
/ljax_target;

*/

	define_tag('target', -namespace='ljax_', -container);
		local('output'=null);
		\ljax_hastarget->(run: -params=params) ? #output = run_children;
		if(!local_defined('nooutput'));
			return(@#output);
		/if;
	/define_tag;
	
	define_tag('ljax_target', -container);/define_tag; // to compensate for a bug
/*
		The ljax_include tag allows an external file to be processed
	conditionally based on the -target supplied to the tag. The first
	parameter to this tag is the name of the file to be included, specified
	in the same manner as it would be to the include tag. The remaining
	parameters are given as they would be to the ljax_target tag.
	
	Example:
		ljax_include('files/thisone.lasso', -target='mytarget')
*/
	define_tag('include', -namespace='ljax_', -required='file');
		local('output'=null);
		\ljax_hastarget->(run: -params=params->(removeall: #file) & (removeall: -file) &) ? #output = include(#file);
		if(!local_defined('nooutput'));
			return(@#output);
		/if;
	/define_tag;

/*
	This utility tag is used by the two tags above to decide whether they
	should run their contents or not. 
*/
	define_tag('hastarget', -namespace='ljax_', -optional='target');

		!var_defined('LJAX') ? var('LJAX' = false);
		
		// cache __LJAX_Targets__ array
		if: !var_defined('__LJAX_Targets__') || !($__LJAX_Targets__->IsA('array'));
			var('__LJAX_Targets__' = client_getparams->find(-target)->merge(client_postparams->find(-target)) &);
			$__LJAX_Targets__->foreach( { params->first = params->first->second; return: true } );
		/if;
		
		if (($LJAX == false) || ($__LJAX_Targets__->size == 0));
			// no targets given
			if (local_defined('notarget'));
				return(true);
			/if;
		else (local_defined('anytarget') && !local_defined('target'));
			// any target
			return(true);
		else;
			// iterate through parameters
			iterate(params, local('curr'));
				if (#curr->isa('string') && !(#curr->waskeyword) && ($__LJAX_Targets__ >> #curr));
					// string parameter matches target name
					return(true);
				else (#curr->isa('pair') && (#curr->first == -target) && ($__LJAX_Targets__ >> #curr->second));
					// -target parameter matches target name
					return(true);
				else (#curr->isa('array'));
					// array parameter contains target name
					iterate(#curr, local('curr2'));
						if ($__LJAX_Targets__ >> #curr2);
							return(true);
						/if;
					/iterate;
				/if;
			/iterate;
		/if;
		return(false);
	/define_tag;

/*
	[LJAX_Start]
	[LJAX_End]
	These tags can be used to trim the output of a page when it is processed by LJAX.
	Any page output prior to [LJAX_Start] or after [LJAX_End] will be trimmed before
	being sent to the browser.  Note that Lasso code before and after these tags will
	be processed, only the output will be suppressed.
*/
	define_tag('start', -namespace='ljax_');
		!var_defined('LJAX') ? var('LJAX' = false);
		$LJAX ? return('###LJAXSTART###');
	/define_tag;
	define_tag('end', -namespace='ljax_');
		!var_defined('LJAX') ? var('LJAX' = false);
		$LJAX ? return('###LJAXEND###');
	/define_tag;
	
?>