<?LassoScript

namespace_using:'namespace_';
    
	define_constant('delimiter', '_');
    define_constant('global', '_global_');
    define_constant('page', '_page_');
    define_constant('current', '_current_');

	define_tag('fullPathExists', -namespace='_global_namespace_file_', -req='path', -privileged);
	
		// Tag returns the found path or empty for false
	
		local('servicepath'=admin_lassoservicepath);
		// there are two allowed roots
		// this given path is going to be in one of them
		// we are not permitted to check any directories above either of these
		// so we need to determine which root is being looked in and start from there
		local('globalRoot' = #servicepath + '/LassoLibraries/');
		local('localRoot' = #servicepath + '/../../LassoLibraries/');
	
		if (#path->beginsWith(#globalRoot));
			local('madePath' = #globalRoot);
		else (#path->beginsWith(#localRoot));
			local('madePath' = #localRoot);
		else;
			return('');
		/if;
		local('adjustedPath' = #path - #madePath);
		local('splt1' = #adjustedPath->split('/')->removeAll('') &);
		local('last' = #splt1->last);
		#splt1->remove;

		local('splt'=array);
		iterate(#splt1, local('i'));
			if (#i == '..');
				#splt->remove;
			else;
				#splt->insert(#i);
			/if;
		/iterate;
		
		// check all leading directory names
		iterate(#splt, local('i'));
			#i += '/';
			local('list' = file_listdirectory(#madePath));
			local('fnd' = #list->find(#i));
			(#fnd->size == 0) ? return('');
			#madePath += #fnd->first;
		/iterate;
		
		// check the file name
		local('list' = file_listdirectory(#madePath));
		local('fnd' = #list->find(#last));
		(#fnd->size > 0) ? return(#madePath + #fnd->first);
		local('fnd' = #list->find(#last + '/'));
		(#fnd->size > 0) ? return(#madePath + #fnd->first);
		return('');
	/define_tag;

	define_tag:'load', -required='namespace', -copy, -privileged;

		(#namespace == '') ? return: false;
		(#namespace == '_page_') ? return: false;
		(#namespace == '_global_') ? return: false;
		(#namespace >> '/') ? return: false;
		(#namespace >> '..') ? return: false;
		
		#namespace->(removeleading: '_page_');
		#namespace->(removeleading: '_global_');
		#namespace->removeleading('_');

		// transform namespace into a path
		// replace underscores with slashes
		local: 'namespacepath' = #namespace;
		#namespacepath->removetrailing('_');
		#namespacepath->(replace: '_', '/');
		local('servicepath'=admin_lassoservicepath);
		local:'dotLassoLocalPath' = #servicepath + '/LassoLibraries/' + #namespacepath + '.lasso';
		local:'dotLassoAppLocalPath' = #servicepath + '/LassoLibraries/' + #namespacepath + '.LassoApp';
		local:'dotFolderLocalPath' = #servicepath + '/LassoLibraries/' + #namespacepath + '/';
		local:'dotLassoGlobalPath' = #servicepath + '/../../LassoLibraries/' + #namespacepath + '.lasso';
		local:'dotLassoAppGlobalPath' = #servicepath + '/../../LassoLibraries/' + #namespacepath + '.LassoApp';
		local:'dotFolderGlobalPath' = #servicepath + '/../../LassoLibraries/' + #namespacepath + '/';
		
		local:'scope'='';
		local:'pathToUse'='';
		local:'processor'=null;
		
		global('_namespace_loader_global')->isa('map') == false ? global:'_namespace_loader_global' = map;
		var('_namespace_loader_active')->isa('set') == false ? var:'_namespace_loader_active' = set;
		
		if:(var: '_namespace_loader_active') >> #namespace;
			return: false;
		/if;
	
		if:(global: '_namespace_loader_global') >> #namespace;
			log_detail('Waiting for on-demand library for namespace ' #namespace '.');
			return: (global: '_namespace_loader_global')->(find: #namespace)->(wait: 1000);
		/if;
	
		if: (#pathToUse := _global_namespace_file_fullPathExists:#dotLassoLocalPath) != '';
			#scope = 'site';
			#processor = \process;
		else: (#pathToUse := _global_namespace_file_fullPathExists:#dotLassoAppLocalPath) != '';
			#scope = 'site';
			#processor = \lassoapp_process;
		else: (#pathToUse := _global_namespace_file_fullPathExists:#dotFolderLocalPath) != '';
			#scope = 'site';
			#processor = 'folder';
		else: (#pathToUse := _global_namespace_file_fullPathExists:#dotLassoGlobalPath) != '';
			#scope = 'global';
			#processor = \process;
		else: (#pathToUse := _global_namespace_file_fullPathExists:#dotLassoAppGlobalPath) != '';
			#scope = 'global';
			#processor = \lassoapp_process;
		else: (#pathToUse := _global_namespace_file_fullPathExists:#dotFolderGlobalPath) != '';
			#scope = 'global';
			#processor = 'folder';
		else;
			return:false;		
		/if;
	
		(global: '_namespace_loader_global')->insert(#namespace = thread_event);
		(var: '_namespace_loader_active')->insert(#namespace);
		log_detail('Loading ' #scope ' on-demand library for namespace ' #namespace '.');
		error_push;
		namespace_using: namespace_global + #namespace;
			protect;
				if: #processor->(isa: 'tag');
					#processor->invoke(file_read(#pathToUse));
				else;
					iterate(file_listdirectory(#pathToUse), (local: 'path'));
						if: #path->(endswith: '.lasso') || #path->(endswith: '.las') || #path->(endswith: '.html') || #path->(endswith: '.htm') || #path->(endswith: '.incl') || #path->(endswith: '.inc');
							\process->invoke(file_read(#pathToUse + #path), -name=#path);
						else: #path->(endswith: '.lassoapp');
							\lassoapp_process->invoke(file_read(#pathToUse + #path));
						/if;
					/iterate;
				/if;
				handle_error;
					log_critical('Error loading on-demand library ' #pathToUse ' for namespace ' #namespace ': ' error_code ' ' error_msg '.');
				/handle_error;
			/protect;
		/namespace_using;
		error_pop;
		(global: '_namespace_loader_global')->(find: #namespace)->signalall;
		(global: '_namespace_loader_global')->(remove: #namespace);
		(var: '_namespace_loader_active')->(remove: #namespace);
		return:true;
	/define_tag;

/namespace_using;

?>