<?LassoScript

	// Security Tags
	//
	// These tage provide additional security options beyond those provided
	// by [Auth] and [Auth_Admin].
	//
	// Check that the current user belongs to a group or one of a set of groups
	// To check that a user belongs to multiple groups use multiple [Auth_Group] tags.
	//
	// [Auth_Group: 'group']
	// [Auth_Group: (array: 'group', 'group')]
	//
	// Check for a specific Lasso Security username or one of a set of usernames
	//
	// [Auth_User: 'username']
	// [Auth_User: (array: 'username', 'username')]
	//
	// Check against a custom username/password or an array of usernames/passwords.
	// If two arrays are specified then common elements in each array match as username/password
	// pairs.  Both arrays should have the same length.
	//
	// [Auth_Custom: 'username', 'password']
	// [Auth_Custom: (array: 'username', 'username'), 'password']
	// [Auth_Custom: (array: 'username', 'username'), (array: 'password', 'password')]
	// [Auth_Custom: 'username', (array: 'password', 'password')]
	//
	// All tags take -Basic=true/false and -Digest=true/false parameter which control whether
	// Basic or Digest authentication is used.  Digest authentication will be used by default
	// if the preference in Site Admin is set to true.  Basic authentication is always used as a fallback.
	//
	// All tags take an optional -Realm parameter (default "Lasso Security").
	//
	// When in digest mode, all tags take optional -Nonce parameter (default [Lasso_UniqueID]), 
	// -Stale parameter and -Opaque parameter.
	//
	// All tags take an optional -NoAbort parameter that deactivates the abort.
	//
	// [Auth_Custom: 'username', 'password', 'realm', -NoAbort]
	//
	// Thank you to Bil Corry for contributions to the Digest Authentication qop implementation.
	// 

	Define_Tag: 'Group',
			-Namespace='Auth_',
			-Priority='Replace',
			-Optional='Group',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort';

		If: (local_defined: 'stale') && ((#stale === null) || (#stale == true));
			// Nothing
		Else: !(Local_Defined: 'Group');
			If: ((Admin_CurrentGroups) != (Array: 'ANYUSER'));
				Return;
			/If;
		Else: (#Group->(IsA: 'array'));
			Iterate: #Group, (Local: 'Temp');
				If: ((Admin_CurrentGroups) >> (String: #Temp));
					Return;
				/If;
			/Iterate;
		Else: ((Admin_CurrentGroups) >> (String: #Group));
			Return;
		/If;

		Local: 'Params' = (Array);
		Iterate: (Array: -Realm='Realm', -Basic='Basic', -Digest='Digest', -Nonce='Nonce', -Opaque='Opaque', -Stale='Stale', -Transparent='Transparent', -NoAbort='NoAbort', -ErrorResponse='ErrorResponse', -NoResponse='NoResponse'), (Local: 'Temp');
			If: (Local_Defined: #Temp->Second);
				#Params->(Insert: #Temp->First = (Local: #Temp->Second));
			/If;
		/Iterate;
		\Auth_Prompt->(Run: -Params=#Params);

	/Define_Tag;

	Define_Tag: 'User',
			-Namespace='Auth_',
			-Priority='Replace',
			-Optional='User',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort';

		If: (local_defined: 'stale') && ((#stale === null) || (#stale == true));
			// Nothing
		Else: !(Local_Defined: 'User');
			If: ((Admin_CurrentUsername) != 'ANONYMOUS') && ((Admin_CurrentUsername) != '');
				Return;
			/If;
		Else: (#User->(IsA: 'array')) && (#User >> (Admin_CurrentUsername));
			Return;
		Else: ((Admin_CurrentUsername) == (String: #User));
			Return;
		/If;

		Local: 'Params' = (Array);
		Iterate: (Array: -Realm='Realm', -Basic='Basic', -Digest='Digest', -Nonce='Nonce', -Opaque='Opaque', -Stale='Stale', -Transparent='Transparent', -NoAbort='NoAbort', -ErrorResponse='ErrorResponse', -NoResponse='NoResponse'), (Local: 'Temp');
			If: (Local_Defined: #Temp->Second);
				#Params->(Insert: #Temp->First = (Local: #Temp->Second));
			/If;
		/Iterate;
		\Auth_Prompt->(Run: -Params=#Params);

	/Define_Tag;

	Define_Tag: 'Custom',
			-Namespace='Auth_',
			-Priority='Replace',
			-Optional='User',
			-Optional='Password',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort',
			-Optional='AuthTag',
			-Optional='AuthTagParams';

		local: 'authorization' = (client_authorization);

		If: !(Local_Defined: 'Realm');
			Local: 'Realm' = 'Lasso Security';
		/If;

		If: (local_defined: 'stale') && ((#stale === null) || (#stale == true));
			// Nothing
		Else: (#authorization->(find: 'type') >> 'Digest');
			local: 'digest_user' = #authorization->(find: 'user');
			local: 'digest_nonce' = #authorization->(find: 'nonce');
			local: 'digest_uri' = #authorization->(find: 'uri');
			local: 'digest_realm' = #authorization->(find: 'realm');
			local: 'digest_method' = #authorization->(find: 'method');
			local: 'digest_response' = #authorization->(find: 'response');
			local: 'digest_opaque' = #authorization->(find: 'opaque');
			local: 'digest_algorithm' = #authorization->(find: 'algorithm');
			local: 'digest_nc' = #authorization->(find: 'nc');
			local: 'digest_cnonce' = #authorization->(find: 'cnonce');
			local: 'digest_qop' = #authorization->(find: 'qop');

			if: (local_defined:'authtag');
				local: 'user' = #digest_user;
				if:!(local_defined:'authtagparams');
					local:'authtagparams' = (array: -username=#user, -realm=#realm);
				else: !#authtagparams->(isa:'array');
					#authtagparams = (array: -username=#user, -realm=#realm, #authtagparams);
				else;
					#authtagparams->(insertfirst: -realm=#realm);
					#authtagparams->(insertfirst: -username=#user);
				/if;
				local: 'password' = \#authTag->(Run: -Params=#authtagparams); // get password for given user
			else;
				if: !(local_defined:'user') || !(local_defined:'password');
					fail: -1, 'Missing -user and/or -password, or -authtag.';
					abort; // for good measure
				/if;
			/if;
			
			if: #password !== null;
				If: (#User->(IsA: 'array')) && (#User >> #digest_user);
					If: (#Password->(IsA: 'array'));
						Local: 'Indices' = #User->(FindIndex: #digest_user);
						Iterate: #Indices, (Local: 'Temp');
							if: #digest_qop == 'auth';
								local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password->(Get: #Temp))) + ':' + #digest_nonce + ':' + #digest_nc + ':' + #digest_cnonce + ':' + #digest_qop + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
							else;
								local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password->(Get: #Temp))) + ':' + #digest_nonce + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
							/if;
							if: #response == #digest_response;
								return;	
							/if;
						/Iterate;
					Else;
						if: #digest_qop == 'auth';
							local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password)) + ':' + #digest_nonce + ':' + #digest_nc + ':' + #digest_cnonce + ':' + #digest_qop + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
						else;
							local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password)) + ':' + #digest_nonce + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
						/if;
						if: #response == #digest_response;
							return;	
						/if;
					/If;
				Else: (String: #digest_user)->(Equals: (String: #User));
					If: (#Password->(IsA: 'array'));
						Iterate: #Password, (Local: 'Temp');
							if: #digest_qop == 'auth';
								local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password->(Get: #Temp))) + ':' + #digest_nonce + ':' + #digest_nc + ':' + #digest_cnonce + ':' + #digest_qop + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
							else;
								local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password->(Get: #Temp))) + ':' + #digest_nonce + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
							/if;
							if: #response == #digest_response;
								return;	
							/if;
						/Iterate;
					Else;
						if: #digest_qop == 'auth';
							local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password)) + ':' + #digest_nonce + ':' + #digest_nc + ':' + #digest_cnonce + ':' + #digest_qop + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
						else;
							local: 'response' = (Encrypt_MD5: (Encrypt_MD5: #digest_user + ':' + #realm + ':' + (String: #Password)) + ':' + #digest_nonce + ':' + (Encrypt_MD5: #digest_method + ':' + #digest_uri));
						/if;
						if: #response == #digest_response;
							return;	
						/if;
					/If;
				/If;
			/if;
		else: (#authorization->(find: 'type') >> 'Basic');

			if: (local_defined:'authtag');
				local: 'user' = Client_Username;
				if:!(local_defined:'authtagparams');
					local:'authtagparams' = (array: -username=#user, -realm=#realm);
				else: !#authtagparams->(isa:'array');
					#authtagparams = (array: -username=#user, -realm=#realm, #authtagparams);
				else;
					#authtagparams->(insertfirst: -realm=#realm);
					#authtagparams->(insertfirst: -username=#user);
				/if;
				local: 'password' = \#authTag->(Run: -Params=#authtagparams); // get password for given user
			else;
				if: !(local_defined:'user') || !(local_defined:'password');
					fail: -1, 'Missing -user and/or -password, or -authtag.';
					abort; // for good measure
				/if;
			/if;

			if: #password !== null;
				If: (#User->(IsA: 'array')) && (#User >> Client_Username);
					If: (#Password->(IsA: 'array'));
						Local: 'Indices' = #User->(FindIndex: Client_Username);
						Iterate: #Indices, (Local: 'Temp');
							If: (#Password->Size >= #Temp) && ((String: Client_Password)->(Equals: (String: #Password->(Get: #Temp)), -Case));
								Return;
							/If;
						/Iterate;
					Else: ((String: Client_Password)->(Equals: (String: #Password), -Case));
						Return;
					/If;
				Else: (String: Client_Username)->(Equals: (String: #User));
					If: (#Password->(IsA: 'array'));
						Iterate: #Password, (Local: 'Temp');
							If: ((String: Client_Password)->(Equals: (String: #Password), -Case));
								Return;
							/If;
						/Iterate;
					Else: (String: Client_Password)->(Equals: (String: #Password), -Case);
						Return;
					/If;
				/If;
			/if;
		else;
			Local: 'Action_Params' = (Client_PostParams);
			#Action_Params->(Merge: Client_GetParams);
			If: #Action_Params >> '-Username';
				Local: 'Client_Username' = #Action_Params->(Find: '-Username')->Last->Second;
			Else;
				Local: 'Client_Username' = Client_Username;
			/If;

			if: (local_defined:'authtag');
				local: 'user' = #Client_Username;
				if:!(local_defined:'authtagparams');
					local:'authtagparams' = (array: -username=#user, -realm=#realm);
				else: !#authtagparams->(isa:'array');
					#authtagparams = (array: -username=#user, -realm=#realm, #authtagparams);
				else;
					#authtagparams->(insertfirst: -realm=#realm);
					#authtagparams->(insertfirst: -username=#user);
				/if;
				local: 'password' = \#authTag->(Run: -Params=#authtagparams); // get password for given user
			else;
				if: !(local_defined:'user') || !(local_defined:'password');
					fail: -1, 'Missing -user and/or -password, or -authtag.';
					abort; // for good measure
				/if;
			/if;

			if: #password !== null;

				If: #Action_Params >> '-Password';
					Local: 'Client_Password' = #Action_Params->(Find: '-Password')->Last->Second;
				Else;
					Local: 'Client_Password' = Client_Password;
				/If;
				If: (#User->(IsA: 'array')) && (#User >> #Client_Username);
					If: (#Password->(IsA: 'array'));
						Local: 'Indices' = #User->(FindIndex: #Client_Username);
						Iterate: #Indices, (Local: 'Temp');
							If: (#Password->Size >= #Temp) && ((String: #Client_Password)->(Equals: (String: #Password->(Get: #Temp)), -Case));
								Return;
							/If;
						/Iterate;
					Else: ((String: #Client_Password)->(Equals: (String: #Password), -Case));
						Return;
					/If;
				Else: (String: #Client_Username)->(Equals: (String: #User));
					If: (#Password->(IsA: 'array'));
						Iterate: #Password, (Local: 'Temp');
							If: ((String: #Client_Password)->(Equals: (String: #Password), -Case));
								Return;
							/If;
						/Iterate;
					Else: (String: #Client_Password)->(Equals: (String: #Password), -Case);
						Return;
					/If;
				/If;
			/if;
		/if;
		
		Local: 'Params' = (Array);
		Iterate: (Array: -Realm='Realm', -Basic='Basic', -Digest='Digest', -Nonce='Nonce', -Opaque='Opaque', -Stale='Stale', -Transparent='Transparent', -NoAbort='NoAbort', -ErrorResponse='ErrorResponse', -NoResponse='NoResponse'), (Local: 'Temp');
			If: (Local_Defined: #Temp->Second);
				#Params->(Insert: #Temp->First = (Local: #Temp->Second));
			/If;
		/Iterate;

		\Auth_Prompt->(Run: -Params=#Params);

	/Define_Tag;

	Define_Tag: 'Prompt',
			-Namespace='Auth_',
			-Priority='Replace',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort',
			-Optional='ErrorResponse',
			-Optional='NoResponse';

		local: 'headers' = (array);
		local: 'content' = '';
		
		If: !(Local_Defined: 'Realm');
			Local: 'Realm' = 'Lasso Security';
		/If;

		if: (local_defined: 'stale') && ((#stale === null) || (#stale == true));
				// rotate nonce
				Local: 'Nonce' = lasso_uniqueid;
		/If;

		If: !(Local_Defined: 'Nonce');
			Local: 'Nonce' = lasso_uniqueid;
		/If;

		If: !(Local_Defined: 'Opaque');
			Local: 'Opaque' = '';
		/If;
		
		If: ((Global: '__digest_auth__') >> 'Y');
			Local: 'Type' = 'basic/digest';
		Else;
			Local: 'Type' = 'basic';
		/If;
		
		If: (Local_Defined: 'Digest');
			If: (#Digest !== Null) && (#Digest != True);
				Local: 'Type' = 'basic';
			Else;
				Local: 'Type' = 'basic/digest';
			/If;
		/If;
		
		If: (Local_Defined: 'Basic') && (#Type >> 'digest');
			If: (#Basic !== Null) && (#Basic != True);
				Local: 'Type' = 'digest';
			/If;
		/If;

		#headers->(insert: 'HTTP/1.0 401 Unauthorized');

		if: (#Type >> 'digest');
			local: 'header' = '';
			#header += 'WWW-Authenticate: Digest ';
			#header += 'realm="' + #Realm + '", ';
			#header += 'qop="auth", ';
			#header += 'nonce="' + #nonce + '", ';
			if: (#opaque != '');
				#header += 'opaque="' + (encode_base64: #opaque) + '", ';
			/if;
			if: (local_defined: 'stale') && ((#stale === null) || (#stale == true));
				#header += 'stale="true", ';
			/if;
			#header += 'uri="' + response_filepath + '", ';
			#header += 'algorithm="md5"';
			#headers->(insert: #header);
		/if;
		#headers->(insert: 'Server: Lasso Professional (' + (Lasso_Version) + ')');
		#headers->(insert: 'Content-Type: text/html');

		if: (local_defined: 'NoResponse');
			local: 'content' = '';
		else: (local_defined: 'ErrorResponse');
			local: 'content' = (String: #ErrorResponse);
		else;
			local: 'content' = '\
				<html>\n\t\
					<head>\n\t\t\
						<title>Not Authorized</title>\n\t\
					</head>\n\t\
					<body>\n\t\t\
						<h1>Not Authorized</h1>\n\t\
					</body>\n\
				</html>\n';
		/if;
		if: (#Type >> 'basic');
			#headers->(insert: 'WWW-Authenticate: Basic realm="' + #Realm + '"');
		/if;

		if: (local_defined:'Transparent') && #transparent != false;
			$__http_header__ = #headers->(join: '\r\n') + '\r\n';
			return;
		else;
			Define_AtEnd: {
				$__html_reply__ = params->first;
				$__http_header__ = params->second->(join: '\r\n') + '\r\n';
			}=(array: #content, #headers);
	
			If: (Params !>> '-NoAbort');
				Abort;
			/If;
		/if;

	/Define_Tag;

	Define_Tag: 'Admin',
			-Namespace='Auth_',
			-Priority='Replace',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort';

		Local: 'Params' = (Params);
		#Params->(Insert: -Group='Administrators');
		
		Return: \Auth_Group->(Run: -Params=#Params, -Name='Auth_Group');

	/Define_Tag;

	Define_Tag: 'Auth',
			-Namespace='_global_',
			-Priority='Replace',
			-Optional='Realm',
			-Optional='Basic',
			-Optional='Digest',
			-Optional='Nonce',
			-Optional='Opaque',
			-Optional='Stale',
			-Optional='Transparent',
			-Optional='NoAbort';

		Local: 'Params' = (Params);
		#Params->(RemoveAll: -User);
		
		Return: \Auth_User->(Run: -Params=#Params, -Name='Auth_User');

	/Define_Tag;
		

	// Admin Tags
	//
	// These tage allow manipulations of users and groups by individual users and
	// group administrators.  They use -Privileged permission to modify the internal
	// security settings.
	//
	// [Admin_ChangeUser: 'Username', 'Old Password', 'New Password'] -> True/False
	// [Admin_CreateUser: 'Usenrame', 'Password'] -> True/False
	// [Admin_GroupAssignUser: 'Group', 'Username'] -> True/False
	// [Admin_GroupListUsers: 'Group'] -> True/False
	// [Admin_GroupRemoveUser: 'Group', 'Username']
	// [Admin_ListGroups]

	Define_Tag: 'ChangeUser', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='OldUsername',
			-Required='OldPassword',
			-Required='NewPassword';

		local: 'output' = false;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id from security_users where name like \'' encode_sql92(#oldusername) '\' and password=\'' (encrypt_md5: #oldpassword) '\'';
			if: (found_count > 0);
				inline:  -update,
						-log=$__startup_loglevel__,
						-database=database_name,
						-table=table_name,
						-keyfield='id',
						-keyvalue=(field: 'id'),
						'modified' = date->(format: '%Q %T'),
						'password_plain' = #newpassword,
						'password' = (encrypt_md5: #newpassword);
					if: error_code == 0;
						local: 'output' = true;
						_security_flushcaches;
					/if;
				/inline;
			/if;
		/inline;

		return: #output;

	/Define_Tag;

	Define_Tag: 'CreateUser', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='NewUsername',
			-Required='NewPassword',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'groups' = (admin_listgroups: -username=(local: 'username'), -password=(local: 'password'));
		else;
			local: 'groups' = (admin_listgroups);
		/if;
		if: (#groups === false) || (#groups->size == 0);
			return: false;
		/if;

		local: 'output' = false;
		
		fail_if: #newusername->size == 0, -9956, 'Username must have a size greater than zero.';
		
		if: (admin_userexists: #newusername);
			return: false;
		/if;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id from security_users where name like \'' encode_sql92(#newusername) '\'';
			if: (found_count == 0);
				if: #newpassword->size;
					local: 'passtmp' = (encrypt_md5: #newpassword);
				else;
					local: 'passtmp' = '';
				/if;

				inline: -log=$__startup_loglevel__, 
					-add,
					'name' = #newusername,
					'password' = #passtmp,
					'password_plain' = #newpassword,
					'id_site' = site_id,
					'enabled'='Y',
					'created' = date->(format: '%Q %T');

					if: error_code == 0;
						local: 'output' = true;
						_security_flushcaches;
					/if;

				/inline;
			/if;
		/inline;

		return: #output;

	/Define_Tag;

	Define_Tag: 'GroupAssignUser', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='Group',
			-Required='NewUsername',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'groups' = (admin_listgroups: -username=(local: 'username'), -password=(local: 'password'));
		else;
			local: 'groups' = (admin_listgroups);
		/if;
		if: (#groups === false) || (#groups->size == 0) || (#groups !>> #group);
			return: false;
		/if;

		local: 'output' = true;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id from security_users where name like \'' encode_sql92(#newusername) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_groups',
				-maxrecords='all',
				-sql = 'select id from security_groups where name like \'' encode_sql92(#group) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		local: 'output' = false;

		inline: -log=$__startup_loglevel__, -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-op='eq', 'id_group' = #group_id,
				-maxrecords='all';
			if: (found_count > 0);
				local: 'output' = true;
			/if;
		/inline;

		if: (#output === true);
			return: true;
		/if;

		inline: -add,
				-log=$__startup_loglevel__, 
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				'id_user' = #user_id,
				'id_group' = #group_id,
				'precedence' = 1,
				'admin' = 'N',
				'modified' = date->(format: '%Q %T');
			if: error_code == 0;
				local: 'output' = true;
				_security_flushcaches;
			/if;
		/inline;

		return: #output;

	/Define_Tag;

	Define_Tag: 'GroupListUsers', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='Group',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'groups' = (admin_listgroups: -username=(local: 'username'), -password=(local: 'password'));
		else;
			local: 'groups' = (admin_listgroups);
		/if;
		if: (#groups === false) || (#groups->size == 0) || (#groups !>> #group);
			return: false;
		/if;

		local: 'output' = (array);
		
		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_groups',
				-maxrecords='all',
				-sql = 'select id from security_groups where name like \'' encode_sql92(#group) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		local: 'ids' = (array);
		
		inline: -log=$__startup_loglevel__, -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_group' = #group_id,
				-returnfield='id_user',
				-maxrecords='all';
			records;
				#ids->(insert: -op='eq');
				#ids->(insert: 'id' = (field: 'id_user'));
			/records;
		/inline;
		
		if: #ids->size > 0;
			inline: -log=$__startup_loglevel__, -search,
					-database='lasso_internal',
					-table='security_users',
					-keyfield='id',
					-opbegin='or',
						#ids,
					-opend='or',
					-op='eq', 'enabled'='Y',
					-returnfield='name',
					-maxrecords='all';
				records;
					if: ((field: 'name') != '');
						#output->(insert: (field: 'name'));
					/if;
				/records;
			/inline;
		/if;
		
		return: #output;

	/Define_Tag;

	Define_Tag: 'UserListGroups', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='NewUsername',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'groups' = (admin_listgroups: -username=(local: 'username'), -password=(local: 'password'));
		else;
			local: 'groups' = (admin_listgroups);
		/if;
		
		local: 'output' = (array);

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id from security_users where name like \'' encode_sql92(#newusername) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		local: 'ids' = (array);

		inline: -log=$__startup_loglevel__, -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-returnfield='id_group',
				-maxrecords='all';
			records;
				#ids->(insert: -op='eq');
				#ids->(insert: 'id' = (field: 'id_group'));
			/records;
		/inline;

		inline: -log=$__startup_loglevel__, -search,
				-database='lasso_internal',
				-table='security_groups',
				-keyfield='id',
				-opbegin='or',
					#ids,
				-opend='or',
				-op='eq', 'enabled'='Y',
				-returnfield='name',
				-maxrecords='all';
			records;
				if: ((field: 'name') != '') && (#groups >> (field: 'name'));
					#output->(insert: (field: 'name'));
				/if;
			/records;
		/inline;

		return: #output;

	/Define_Tag;

	Define_Tag: 'GroupRemoveUser', -namespace='Admin_', -Privileged, -Priority='Replace',
			-Required='Group',
			-Required='NewUsername',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'groups' = (admin_listgroups: -username=(local: 'username'), -password=(local: 'password'));
		else;
			local: 'groups' = (admin_listgroups);
		/if;
		if: (#groups === false) || (#groups->size == 0) || (#groups !>> #group);
			return: false;
		/if;

		local: 'output' = null;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id from security_users where name like \'' encode_sql92(#newusername) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_groups',
				-maxrecords='all',
				-sql = 'select id from security_groups where name like \'' encode_sql92(#group) '\' and enabled=\'Y\'';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -log=$__startup_loglevel__, -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-op='eq', 'id_group' = #group_id,
				-maxrecords='all';
			records;
				inline: -log=$__startup_loglevel__, -delete,
						-database=(database_name),
						-table=(table_name),
						-keyfield=(keyfield_name),
						-keyvalue=(field: keyfield_name);
				/inline;
			/records;
		/inline;

		_security_flushcaches;
		return: true;

	/Define_Tag;

	Define_Tag: 'ListGroups', 
			-namespace='Admin_', 
			-Privileged, 
			-Priority='Replace',
			-Optional='Username',
			-Optional='Password';

		if: (local_defined: 'username');
			local: 'authusername' = (local: 'username');
			local: 'authpassword' = (local: 'password');
		else;
			local: 'authusername' = client_username;
			local: 'authpassword' = client_password;
		/if;

		local: 'output' = (array);

		inline: -log=$__startup_loglevel__,
				-database='lasso_internal',
				-table='security_users',
				-maxrecords='all',
				-sql = 'select id,enabled,password from security_users where name like \'' encode_sql92(#authusername) '\'';
			local: 'user_id' = (field: 'id');
			if: (found_count == 0) && (admin_userexists: #authusername);
				inline: -username=#authusername, -password=#authpassword;
					if: admin_currentusername == #authusername;
						local: 'user_id' = 2; // Server Administrator
					else;
						return: false;
					/if;
				/inline;
			else: (found_count == 0) || ((field: 'enabled') != 'Y') || ((field: 'password') != (encrypt_md5: #authpassword));
				return: false;
			/if;
		/inline;

		if: (#user_id == 2);
			inline: -log=$__startup_loglevel__, -search,
					-database='lasso_internal',
					-table='security_groups',
					-keyfield='id',
					-op='eq', 'enabled'='Y',
					-maxrecords='all';
				records;
					if: ((field: 'name') != '');
						#output->(insert: (field: 'name'));
					/if;
				/records;
			/inline;
		else;
			local: 'ids' = (array);

			inline: -log=$__startup_loglevel__, -search,
					-database='lasso_internal',
					-table='security_ug_map',
					-keyfield='id',
					-op='eq', 'id_user' = #user_id,
					-op='eq', 'admin' = 'Y',
					-returnfield='id_group',
					-maxrecords='all';
				records;
					#ids->(insert: -op='eq');
					#ids->(insert: 'id' = (field: 'id_group'));
				/records;
			/inline;
			
			if: #ids->size > 0;
				inline: -log=$__startup_loglevel__, -search,
						-database='lasso_internal',
						-table='security_groups',
						-keyfield='id',
						-opbegin='or',
							#ids,
						-opend='or',
						-op='eq', 'enabled'='Y',
						-maxrecords='all';
					records;
						if: ((field: 'name') != '');
							#output->(insert: (field: 'name'));
						/if;
					/records;
				/inline;
			/if;
		/if;

		return: #output;

	/Define_Tag;
	
	Define_Tag: 'Authorization', 
			-Namespace='Client_', 
			-Priority='Replace';
	
		Local: 'output' = (map: 'type'='None');
	
		local: 'authorization' = '';

		local: 'parse' = (_chop_mimes: client_headers);
		if: (client_headers >> 'HTTP_Authorization:');
			protect;
				local: 'authorization' = #parse->(find: 'HTTP_Authorization')->first->second;
			/protect;
		else: (client_headers >> 'Authorization:');
			protect;
				local: 'authorization' = #parse->(find: 'Authorization')->first->second;
			/protect;
		/if;

		if: (#authorization >> 'Digest');
			local: 'digest_user' = string;
			local: 'digest_nonce' = string;
			local: 'digest_uri' = string;
			local: 'digest_realm' = string;
			local: 'digest_qop' = string;
			local: 'digest_algorithm' = string;
			local: 'digest_opaque' = string;
			local: 'digest_method' = string;
			local: 'digest_response' = string;
			local: 'digest_nc' = string;
			local: 'digest_cnonce' = string;

			define_tag: 'extract', -required='data', -required='field';
				if: (#data >> #field);
					local: 'pattern' = '[, ]' + #field + '="([^"]*?)"';
					local: 'result' = (string_findregexp: #data, -find=#pattern);
					if: #result->size > 1;
						return: #result->second;
					/if;
					local: 'pattern' = '[, ]' + #field + '=([^, ]*?)[, ]';
					local: 'result' = (string_findregexp: #data, -find=#pattern);
					if: #result->size > 1;
						return: #result->second;
					/if;
				/if;
				return: '';
			/define_tag;

			local: 'digest_method' = (string_uppercase: client_formmethod);
			local: 'digest_user' = (extract: #authorization, 'username');
			local: 'digest_nonce' = (extract: #authorization, 'nonce');
			local: 'digest_uri' = (extract: #authorization, 'uri');
			local: 'digest_realm' = (extract: #authorization, 'realm');
			local: 'digest_qop' = (extract: #authorization, 'qop');
			local: 'digest_algorithm' = (extract: #authorization, 'algorithm');
			local: 'digest_opaque' = (extract: #authorization, 'opaque');
			local: 'digest_response' = (extract: #authorization, 'response');
			local: 'digest_cnonce' = (extract: #authorization, 'cnonce');
			local: 'digest_nc' = (extract: #authorization, 'nc');

			#output->(insert: 'user' = #digest_user);
			#output->(insert: 'nonce' = #digest_nonce);
			#output->(insert: 'nc' = #digest_nc);
			#output->(insert: 'cnonce' = #digest_cnonce);
			#output->(insert: 'response' = #digest_response);
			#output->(insert: 'uri' = #digest_uri);
			#output->(insert: 'realm' = #digest_realm);
			#output->(insert: 'qop' = #digest_qop);
			#output->(insert: 'algorithm' = #digest_algorithm);
			#output->(insert: 'method' = #digest_method);
			if: #digest_opaque->size;
				#output->(insert: 'opaque' = (decode_base64: #digest_opaque));
			else;
				#output->(insert: 'opaque' = '');			
			/if;
			#output->(insert: 'type' = 'Digest');
	
		else: (#authorization >> 'Basic');
		
			#output->(insert: 'type' = 'Basic');
			
		/if;
		
		Return: #output;
	
	/Define_Tag;

?>