<?LassoScript

// driver for 'memory'
define_type:'driver', 'session_driver', -namespace='memory_session_', -prototype, -privileged;

	define_type('entry');
		local('lastTouched'=date);
		local('expireMinutes'=1440);
		local('key'=null);
		local('value'=null); 
		
		define_tag('touch');
			self->'lastTouched' = date;
		/define_tag;
		
		define_tag('hasExpired');
			local('dateCopy' = self->'lastTouched');
			#dateCopy->add(-minute=self->'expireMinutes');
			return(date > #dateCopy);
		/define_tag;
	/define_type;
	
	local('map'=map);
	local('maxInMemory'=500); // max entries before the map gets pruned
	local('idealInMemory'=300); // map will be pruned to this number
	local('lock'=thread_lock);
    local('nextPruneDelta'=5*60); // minimum amount of time between ->prune in seconds
    local('nextPrune' = date + #nextPruneDelta);  // can't ->prune until at least this time

	define_tag:'startup';
		
	/define_tag;
	
	define_tag:'validateSessionsTable', -optional='db', -optional='tb', -privileged;
		return:true;
	/define_tag;
	
	define_tag:'createTable', -required='db', -type='string', -privileged;

	/define_tag;
	
	define_tag:'fetchdata', -required='id', -type='string', -privileged;		
		self->'lock'->lock;
		
		if(self->'map' >> #id);		
			// need check for expiry	
			local('item' = @self->'map'->find(#id));
			if (#item->hasExpired);
				self->'map'->remove(#id);
			else;
				return:string(decompress(#item->touch & value));
			/if;
		/if;
		
		handle;
			self->'lock'->unlock;
		/handle;
		return:null;
	/define_tag;
	
	define_tag:'savedata', -required='id', -type='string', -required='data', -type='string', -privileged;	
		self->'lock'->lock;
		self->'map'->find(#id)->'value' = compress(#data);
		handle;
			self->'lock'->unlock;
		/handle;
	/define_tag;
	
	define_tag:'init', -required='id', -type='string', -required='expires', -type='integer', -privileged;
		self->placeInMap(#id, '', #expires);
	/define_tag;
	
	define_tag:'kill', -required='id', -type='string', -privileged;
		self->removeFromMap(#id);
	/define_tag;
	
	define_tag:'expire', -privileged, -async;
		self->'lock'->lock;
		self->prune;
		handle;
			self->'lock'->unlock;
		/handle;
	/define_tag;
	
	define_tag('placeInMap', -req='id', -req='data', -req='expires');
			local('entry') = self->entry;
			#entry->'value' = compress(#data);
			#entry->'expireMinutes' = #expires;
			self->'lock'->lock;
			
			self->'map'->insert(#id=@#entry);
			if (self->'map'->size > self->'maxInMemory' && self->'nextPrune' < date);
				self->prune;
			/if;
			handle;
				self->'lock'->unlock;
			/handle;	
	/define_tag;

	define_tag('removeFromMap', -req='id');
			self->'lock'->lock;
			self->'map'->remove(#id);
			handle;
				self->'lock'->unlock;
			/handle;	
	/define_tag;
	
	define_tag('prune');
        self->'nextPrune' = date + self->'nextPruneDelta';  // can't ->prune until at least this time
		local('now' = date);	
		// assumes a write lock is already held
		local('toRemove'=array);
		// remove expired sessions first
		iterate(self->'map'->keys, local('i'));
			local_reset('curr' = @self->'map'->find(#i));
			if(#curr != null && #curr->hasExpired);
				#toRemove->insert(#i);
			/if;
		/iterate;
		
		iterate(#toRemove, #i);
			self->'map'->remove(#i);
		/iterate;		
		
	/define_tag;
	
	define_tag('onDestroy');
	
	/define_tag;
	
/define_type;

?>