<?LassoScript

define_tag: 'setdriver', -namespace='session_';
	if: (lasso_datasourceismysql: (global:'_lasso_session_database'));
		global: '_session_driver' = mysql_session_driver;
	else: (lasso_datasourceIsSqlServer: (global:'_lasso_session_database'));
		global: '_session_driver' = sqlserver_session_driver;
	else: (global:'_lasso_session_database') == 'memory';
		global: '_session_driver' = memory_session_driver;
	else;
		global: '_session_driver' = sqlite_session_driver;
	/if;
/define_tag;

// base type for session_driver
define_type:'driver',
	-namespace = 'session_';
	
	define_tag:'startup';
	/define_tag;
	
	define_tag:'validateSessionsTable', -privileged;
	/define_tag;
	
	define_tag:'createTable', -required='db', -type='string', -privileged;
	/define_tag;
	
	define_tag:'fetchdata', -required='id', -type='string', -privileged;
	/define_tag;
	
	define_tag:'savedata', -required='id', -type='string', -required='data', -type='string', -privileged;
	/define_tag;
	
	define_tag:'init', -required='id', -type='string', -required='expires', -type='integer', -privileged;	
	/define_tag;
	
	define_tag:'kill', -required='id', -type='string', -privileged;
	/define_tag;
	
	define_tag:'expire', -privileged, -async;
	/define_tag;	
	
	define_tag:'abort', -required='id', -type='string', -privileged;
	/define_tag;	
	
/define_type;


// driver for SQLite with MEMORY
library(lassoapp_link('sessionMemoryDriver.lasso'));


// driver for SQLite
define_type:'driver', 'session_driver', -namespace='sqlite_session_', -prototype, -privileged;

	define_tag('onCreate');
		// check to see if we have the global thread lock
		if (global('_sqlite_session_driver_lock') == null);
			global('_sqlite_session_driver_lock') = thread_lock;
			global('_sqlite_session_driver_map') = map;
			
			{
				// check the map for entries to write
				while($_sqlite_session_driver_map != null);
				
					protect;
						local('theWriteMap'=null);
						if ($_sqlite_session_driver_lock->lock);
							if ($_sqlite_session_driver_map->size);
								#theWriteMap = $_sqlite_session_driver_map;
								$_sqlite_session_driver_map = map;
							/if;
							handle;
								$_sqlite_session_driver_lock->unlock;
							/handle;
						/if;
						
						if (#theWriteMap->size > 0);
							inline: -log=$__startup_loglevel__, -database=(globals->(find: '_lasso_session_database')), -table=(globals->(find: '_lasso_session_table')),
								-sql="BEGIN EXCLUSIVE TRANSACTION";
								local('errMsg'='');
								local('it'=#theWriteMap->iterator);
								
								while(!#it->atEnd);
									protect;
										inline: -log=$__startup_loglevel__, -database=(globals->(find: '_lasso_session_database')), -table=(globals->(find: '_lasso_session_table')),
												-sql="UPDATE " (globals->(find: '_lasso_session_table')) " SET data=X'" encode_hex(compress(#it->value)) "', last_access='" date->format('%Q %T') "' WHERE session_key='" encode_sql92(#it->key) "'";
											if (error_code != 0);
												#errMsg += '(' error_code ' ' error_msg ' ' #it->key ') '; 
											/if;
										/inline;
									/protect;
									#it->forward;
								/while;
								handle;
									inline: -log=$__startup_loglevel__, -database=(globals->(find: '_lasso_session_database')), -table=(globals->(find: '_lasso_session_table')),
										-sql="COMMIT TRANSACTION";
									/inline;
								/handle;
							/inline;
							#theWriteMap = null;
							if (#errMsg->size);
								log_critical('Errors occurred while writing session data: ' #errMsg); 
							/if;
						/if;					
					/protect;
					sleep(10000);				
				/while;
				handle;
					global('_sqlite_session_driver_lock') = null;
					global('_sqlite_session_driver_map') = null;
				/handle;
			}->asAsync(-Name='Sessions: SQLite Driver');
		/if;		
	/define_tag;

	define_tag:'startup';
		
	/define_tag;
	
	define_tag:'validateSessionsTable', -optional='db', -optional='tb', -privileged;
		if: !(local_defined: 'db');
			local: 'db' = (globals->(find: '_lasso_session_database'));
		/if;
		if: !(local_defined: 'tb');
			local: 'tb' = (globals->(find: '_lasso_session_table'));
		/if;
		inline: -log=$__startup_loglevel__, -database=#db, -table=#tb, -show;
			local: 'fields' = (array: 'id','session_key','data','last_access','expire_minutes');
			iterate:field_names, local:'_tmp';
				#fields->(removeall: #_tmp);
			/iterate;
			if: (error_currenterror != error_noerror);
				return:false;
			else: (#fields->size > 0);
				return:false;
			/if;
		/inline;
		return:true;
	/define_tag;
	
	define_tag:'createTable', -required='db', -type='string', -privileged;
		Local: 'table_exists' = false;

		inline: -log=$__startup_loglevel__, -Database=#db, -SQL='SELECT * from sqlite_master WHERE type="table" AND name="sessions"', -MaxRecords=1;
			if: (found_count) > 0;
				#table_exists = true;
			/if;
		/inline;

		if: (#table_exists == false);
			inline: -log=$__startup_loglevel__, -database=#db,
					-sql='CREATE TABLE sessions
							(
								id INTEGER PRIMARY KEY,
								session_key TEXT NOT NULL UNIQUE,
								data BLOB,
								last_access TEXT,
								expire_minutes INTEGER DEFAULT 15
							)';
			/inline;
		/if;
	/define_tag;
	
	define_tag:'fetchdata', -required='id', -type='string', -privileged;
		protect;
			if ($_sqlite_session_driver_lock->lock);
				local('found'=@$_sqlite_session_driver_map->find(#id));
				if (#found != null);
					return(#found);
				/if;
				handle;
					$_sqlite_session_driver_lock->unlock;
				/handle;
			/if;
		/protect;
		inline(-log=$__startup_loglevel__, -database='site', -table='sessions', 
				-sql='SELECT data,last_access,expire_minutes FROM sessions WHERE session_key=\'' encode_sql92(#id) '\' LIMIT 1');
			
			fail_if:error_code != 0, error_code, error_msg;
			if:found_count == 0;
				return:null;
			/if;
			
			local('date' = date(field('last_access')));
			local('now' = date);
			#date->add(-minute=integer(field('expire_minutes')));

			if (#now > #date); // it expired
				return:null;
			/if;
			if: ((field: 'data') >> '<?xml') || ((field: 'data') >> '<mapType>');
				return(string(field: 'data'));
			else;
				return(string(decompress(field:'data')));
			/if;
		/inline;
	/define_tag;
	
	define_tag:'savedata', -required='id', -type='string', -required='data', -type='string', -privileged;
		protect;
			if ($_sqlite_session_driver_lock->lock);
				$_sqlite_session_driver_map->insert(#id = #data);
				handle;
					$_sqlite_session_driver_lock->unlock;
				/handle;
				return:true;
			/if;
		/protect;
		inline: -log=$__startup_loglevel__, -database=(globals->(find: '_lasso_session_database')), -table=(globals->(find: '_lasso_session_table')),
				-sql="PRAGMA synchronous = NORMAL; UPDATE " (globals->(find: '_lasso_session_table')) " SET data=X'" encode_hex(compress(#data)) "', last_access='" date->format('%Q %T') "' WHERE session_key='" encode_sql92(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'init', -required='id', -type='string', -required='expires', -type='integer', -privileged;	
		inline: -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="PRAGMA synchronous = NORMAL; INSERT INTO " (global:'_lasso_session_table') " (session_key, data, last_access, expire_minutes) VALUES ('" encode_sql92(#id) "', X'" encode_hex(compress(bytes(''))) "', '" date->format('%Q %T') "', " integer(#expires) ")";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'kill', -required='id', -type='string', -privileged;
		protect;
			if ($_sqlite_session_driver_lock->lock);
				$_sqlite_session_driver_map->remove(#id);
				handle;
					$_sqlite_session_driver_lock->unlock;
				/handle;
			/if;
		/protect;
		inline: -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="PRAGMA synchronous = NORMAL; DELETE FROM " (global:'_lasso_session_table') " WHERE session_key = '" encode_sql92(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'expire', -privileged, -async;
		inline: -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="PRAGMA synchronous = NORMAL; DELETE FROM " (global:'_lasso_session_table') " WHERE (strftime('%s', datetime('now','localtime')) - strftime('%s', last_access) > (expire_minutes*60))";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
/define_type;

// driver for MySQL
define_type:'mysql_session_driver', 'session_driver', -prototype, -privileged;

	define_tag:'startup';
		
	/define_tag;
	
	define_tag:'validateSessionsTable', -optional='db', -optional='tb', -privileged;
		if: !(local_defined: 'db');
			local: 'db' = (globals->(find: '_lasso_session_database'));
		/if;
		if: !(local_defined: 'tb');
			local: 'tb' = (globals->(find: '_lasso_session_table'));
		/if;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=#db, -table=#tb, -show;
			local: 'fields' = (array: 'id','session_key','data','last_access','expire_minutes');
			iterate:field_names, local:'_tmp';
				#fields->(removeall: #_tmp);
			/iterate;
			if: (error_currenterror != error_noerror);
				return:false;
			else: (#fields->size > 0);
				return:false;
			/if;
		/inline;
		return:true;
	/define_tag;
	
	define_tag:'createTable', -required='db', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=#db,
				-sql='CREATE TABLE sessions
						(
							id BIGINT PRIMARY KEY AUTO_INCREMENT,
							session_key CHAR(255) NOT NULL,
							data LONGBLOB,
							last_access TIMESTAMP,
							expire_minutes INT DEFAULT 15,
							UNIQUE (session_key)
						) CHARACTER SET UTF8';
		/inline;
	/define_tag;
	
	define_tag:'fetchdata', -required='id', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), 
				-table=(global:'_lasso_session_table'),
				-sql='SELECT data FROM ' (global:'_lasso_session_table') ' WHERE (session_key = "' encode_sql(#id) '") AND (last_access + INTERVAL expire_minutes MINUTE >= NOW()) LIMIT 1';
			fail_if:error_code != 0, error_code, error_msg;
			if:found_count == 0;
				return:null;
			/if;
			return:(field:'data');
		/inline;
	/define_tag;
	
	define_tag:'savedata', -required='id', -type='string', -required='data', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="UPDATE " (global:'_lasso_session_table') " SET data='" encode_sql(#data) "', last_access=NOW() WHERE session_key='" encode_sql(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'init', -required='id', -type='string', -required='expires', -type='integer', -privileged;	
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="INSERT INTO " (global:'_lasso_session_table') " (session_key, data, last_access, expire_minutes) VALUES ('" encode_sql(#id) "', '', NOW(), " integer(#expires) ")";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'kill', -required='id', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="DELETE FROM " (global:'_lasso_session_table') " WHERE session_key = '" encode_sql(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'expire', -privileged, -async;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="DELETE FROM " (global:'_lasso_session_table') " WHERE last_access + INTERVAL expire_minutes MINUTE < NOW()";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
/define_type;

// driver for SQL Server
define_type:'sqlserver_session_driver', 'session_driver', -prototype, -privileged;

	define_tag:'startup';
		
	/define_tag;
	
	define_tag:'validateSessionsTable', -optional='db', -optional='tb', -privileged;
		if: !(local_defined: 'db');
			local: 'db' = (globals->(find: '_lasso_session_database'));
		/if;
		if: !(local_defined: 'tb');
			local: 'tb' = (globals->(find: '_lasso_session_table'));
		/if;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=#db, -table=#tb, -show;
			local: 'fields' = (array: 'id','session_key','data','last_access','expire_minutes');
			iterate:field_names, local:'_tmp';
				#fields->(removeall: #_tmp);
			/iterate;
			if: (error_currenterror != error_noerror);
				return:false;
			else: (#fields->size > 0);
				return:false;
			/if;
		/inline;
		return:true;
	/define_tag;
	
	define_tag:'createTable', -required='db', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=#db,
				-sql='CREATE TABLE [sessions] (
							[id] bigint PRIMARY KEY IDENTITY,
							[session_key] char(255) NOT NULL,
							[data] varbinary(max) NULL,
							[last_access] datetime NULL,
							[expire_minutes] int NULL DEFAULT 15
						);CREATE UNIQUE NONCLUSTERED INDEX [session_key_idx]
						ON [sessions] ([session_key] ASC)
						WITH (PAD_INDEX = OFF,
							IGNORE_DUP_KEY = OFF,
							STATISTICS_NORECOMPUTE = OFF,
							SORT_IN_TEMPDB = OFF,
							ONLINE = OFF,
							ALLOW_ROW_LOCKS = ON,
							ALLOW_PAGE_LOCKS = ON)';
		/inline;
	/define_tag;
	
	define_tag:'fetchdata', -required='id', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), 
				-table=(global:'_lasso_session_table'),
				-sql='SELECT TOP 1 CAST(data AS VARCHAR(max)) AS data FROM '+(global:'_lasso_session_table')+' WHERE (session_key = \''+encode_sql92(#id)+'\') AND (DATEDIFF(mi,last_access,GETDATE()) <= expire_minutes)';
			fail_if:error_code != 0, error_code, error_msg;
			if:found_count == 0;
				return:null;
			/if;
			return:(field:'data');
		/inline;
	/define_tag;
	
	define_tag:'savedata', -required='id', -type='string', -required='data', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="UPDATE " (global:'_lasso_session_table') " SET data=CAST('"+encode_sql92(#data)+"' AS varbinary(max)), last_access=GETDATE() WHERE session_key='" encode_sql92(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'init', -required='id', -type='string', -required='expires', -type='integer', -privileged;	
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="INSERT INTO " (global:'_lasso_session_table') " (session_key, last_access, expire_minutes) VALUES ('" encode_sql92(#id) "', GETDATE(), " integer(#expires) ")";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'kill', -required='id', -type='string', -privileged;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="DELETE FROM " (global:'_lasso_session_table') " WHERE session_key = '" encode_sql92(#id) "'";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
	define_tag:'expire', -privileged, -async;
		inline: -timeout=3, -log=$__startup_loglevel__, -database=(global:'_lasso_session_database'), -table=(global:'_lasso_session_table'),
				-sql="DELETE FROM " (global:'_lasso_session_table') " WHERE DATEDIFF(mi,last_access,GETDATE()) > expire_minutes";
			fail_if:error_code != 0, error_code, error_msg;
		/inline;
	/define_tag;
	
/define_type;

?>
