<?LassoScript

	// Records.Lasso
	//
	// Implements:
	// [Records_Map]
	//
	// Copyright  2007 LassoSoft, LLC

	// [Records_Map] returns a map of maps representing the current found set.
	// 
	// The key for the outer map is the key field value for each record.  The
	// keyfield can be specified explicitly with the -KeyField parameter, otherwise
	// the [KeyField_Name] is used, a field named "ID", or as a last resort the
	// first field.
	// 
	// -ReturnField can be specified multiple times to explicitly state which
	// fields should be included.  -ExcludeField can be specified multiple times to
	// knock certain fields out of the results.
	// 
	// -Fields=(Array) can be used to specify alternate field names than those
	// provided by [Field_Names].  The number of fields should be equal to or less
	// than the number returned by the search.
	//
	// -Type='array' controls whether the outer data type is a map (by default) or
	// an array.
	//
	// Note that unless -Type='array' is used the map returned by this tag contains
	// only one record per key field value andthe sorting of the [Records_Array] is
	// lost.

	Define_Tag: 'Map', -Namespace='Records_',
			-Optional='KeyField',
			-Optional='ReturnField',
			-Optional='ExcludeField',
			-Optional='Fields',
			-Optional='Type';
		Local: '_fields' = (Local_Defined: 'fields') && #fields->(IsA: 'array') ? #fields | Field_Names;
		Fail_If: #_fields->size == 0, -1, 'No fields found for [Records_Map]';
		Local: '_keyfield' = (Local: 'keyfield');
		If: #_fields !>> #_keyfield;
			Local: '_keyfield' = (KeyField_Name);
			If: #_fields !>> #_keyfield;
				Local: '_keyfield' = 'ID';
				If: #_fields !>> #_keyfield;
					Local: '_keyfield' = #_fields->First;
				/If;
			/If;
		/If;
		Local: '_index' = #_fields->(FindPosition: #_keyfield)->First;
		Local: '_return' = (Local_Defined: 'returnfield') ? (Params->(Find: -ReturnField)->(ForEach: {Params->First = Params->First->Second; Return: True}) &) | @#_fields;
		Local: '_exclude' = (Local_Defined: 'excludefield') ? (Params->(Find: -ExcludeField)->(ForEach: {Params->First = Params->First->Second; Return: True}) &) | Array;
		Local: '_output' =  (Local_Defined: 'type') && #type >> 'array' ? array | map;
		Iterate: Records_Array, (Local: '_record');
			Local: '_temp' = Map;
			Iterate: #_fields, (Local: '_field');
				((#_return >> #_field) && (#_exclude !>> #_field)) ? #_temp->(Insert: #_field = #_record->(Get: Loop_Count));
			/Iterate;
			#_output->(Insert: #_record->(Get: #_index) = #_temp);
		/Iterate;
		Return: @#_output;
	/Define_Tag;

?>