<?LassoScript

	// URL Rewrite
	{
		Error_Reset;
		Protect;
		
			Log_Warning: 'URL Rewrite: Initializing';
		
			// Check if the rewrite_rule table exists
			Local: 'table_rewrite_rule_exists' = false;
			Local: 'table_rewrite_trigger_exists' = false;
			Local: 'table_rewrite_action_exists' = false;
			Inline: -log=$__startup_loglevel__,
					-Database='site',
					-SQL='SELECT name FROM sqlite_master WHERE type="table" AND name LIKE "rewrite_%"',
					-MaxRecords='all';
				records;
					if: (field: 'name') == 'rewrite_rule';
						local: 'table_rewrite_rule_exists' = true;
					else: (field: 'name') == 'rewrite_trigger';
						local: 'table_rewrite_trigger_exists' = true;
					else: (field: 'name') == 'rewrite_action';
						local: 'table_rewrite_action_exists' = true;
					/if;
				/records;
			/inline;
		
			if: #table_rewrite_rule_exists == false;
				// Create table
				var: '__need_reload__' = true;
				Inline: -log=$__startup_loglevel__,
						-Database='site',
						-SQL='CREATE TABLE rewrite_rule \
						(\
							id INTEGER, \
							name TEXT, \
							description TEXT, \
							status TEXT, \
							option TEXT, \
							bool TEXT, \
							sort INTEGER, \
							modified TEXT, \
							PRIMARY KEY (id) \
						)';
				/Inline;
			/if;
			
			if: #table_rewrite_trigger_exists == false;
				// Create table
				var: '__need_reload__' = true;
				Inline: -log=$__startup_loglevel__,
						-Database='site',
						-SQL='CREATE TABLE rewrite_trigger \
						(\
							id INTEGER, \
							id_rule INTEGER, \
							type TEXT, \
							name TEXT, \
							comp TEXT, \
							value TEXT, \
							sort INTEGER, \
							modified TEXT, \
							PRIMARY KEY (id) \
						)';
				/Inline;
			/if;
			
			if: #table_rewrite_action_exists == false;
				// Create table
				var: '__need_reload__' = true;
				Inline: -log=$__startup_loglevel__,
						-Database='site',
						-SQL='CREATE TABLE rewrite_action \
						(\
							id INTEGER, \
							id_rule INTEGER, \
							type TEXT, \
							name TEXT, \
							value TEXT, \
							sort INTEGER, \
							modified TEXT, \
							PRIMARY KEY (id) \
						)';
				/Inline;
			/if;
		
			handle_error;
				log_warning: 'URL Rewrite: Unhandled Error (1): ' + error_msg + ' (' + error_code + ')';
			/handle_error;
		
		/protect;

	}->(Run);
	
	define_type: 'URL_Rewrite';
		
		local: 'rules' = (array);
		
		define_tag: 'oncreate';
			inline: -log=$__startup_loglevel__,
					-search,
					-database='site', 
					-table='rewrite_rule',
					-op='eq', 'status'='Y',
					-maxrecords='all',
					-sortfield='sort';
				records;
					local: 'triggers' = (array);
					inline: -log=$__startup_loglevel__,
							-search,
							-table='rewrite_trigger',
							-op='eq', 'id_rule'=(field: 'id'),
							-maxrecords='all',
							-sortfield='sort';
						records;
							#triggers->(insert: self->\trigger = (array: (field: 'type'), (field: 'name'), (field: 'comp'), (field: 'value')));
						/records;
					/inline;
					local: 'actions' = (array);
					inline: -log=$__startup_loglevel__,
							-search,
							-table='rewrite_action',
							-op='eq', 'id_rule'=(field: 'id'),
							-maxrecords='all',
							-sortfield='sort';
						records;
							#actions->(insert: self->\action = (array: (field: 'type'), (field: 'name'), (field: 'value')));
						/records;
					/inline;
					if: #actions->size > 0;
						if: #triggers->size == 0;
							#triggers->(insert: (array: 'true','','',''));
						/if;
						self->'rules'->(insert: self->\rule = (array: #triggers, (field: 'bool'), #actions, (field: 'option')));
					/if;
				/records;
			/inline;
		/define_tag;
		
		define_tag: 'onconvert';
			return: 'URL Rewrite';
		/define_tag;
		
		define_tag: 'oncompare',
				-required='value';
			return: #value != 'rewrite';
		/define_tag;
		
		define_tag: 'invoke';
			
			if: (string_findregexp: response_filepath, -find='/(?:ldmlreference)\\.\\d*\\.?lassoapp', -ignorecase)->size > 0;
				_security_flushcaches;
				redirect_url: (response_filepath - 'ldml') + '?' + client_getargs;
			else: (string_findregexp: response_filepath, -find='/(?:siteadmin|reference|serveradmin|groupadmin|lassostudio|ljax|rpc|databasebrowser)\\.\\d*\\.?lassoapp', -ignorecase)->size > 0;
				return;
			/if;
			
			var: '__abort__' = false;
			
			error_push;
			
			iterate: self->'rules', (local: 'rule');
				protect;
					local: 'stop' = #rule->invoke;
					handle_error;
						log_warning: 'URL Rewrite: ' + error_code + ' ' + error_msg;
					/handle_error;
				/protect;
				if: #stop || $__abort__;
					loop_abort;
				/if;
			/iterate;
			
			error_pop;

			$__abort__ ? abort;
			
		/define_tag;
			
		define_tag: 'trigger',
				-required='field',
				-required='name',
				-required='comp',
				-required='value';
			
			local: 'field_array' = (array: 'client_url', 'server_name', 'server_port', 'response_fileexists', 'response_filepath', 'client_getargs', 'client_ip', 'client_header', 'cookie', 'action_param', 'client_getparams', 'client_postparams', 'error_code', 'error_msg', 'true', 'global', 'variable');
			fail_if: #field_array !>> #field, -9956, 'URL Rewrite: Trigger field "' + #field + '" not found.';
			if: #field == 'true';
				return: true;
			/if;
			
			error_push;

			protect;
				local: 'field_value' = \#field->(invoke: #name);
				handle_error;
					fail: -9956, 'URL Rewrite: Trigger field "' + #field + '" failed.';
				/handle_error;
			/protect;

			if: #comp == 'rx' || #comp == 'nrx';
				protect;
					local: 'regexp_value' = (string_findregexp: #field_value, -find=#value, -ignorecase);
					$__triggers__->(insert: #regexp_value->size > 0 ? #regexp_value, '');
					handle_error;
						fail: -9956, 'URL Rewrite: Trigger regular expression "' + #value + '" failed.';
					/handle_error;
				/protect;
			else;
				$__triggers__->(insert: #value);
			/if;
			
			error_pop;
			
			(global: '__rewrite_debug__') ? log_always: 'Rewrite Trigger: ' + #comp + ' ' #field_value + ' ' + #field_value->type + ' ' + (local: 'value') + ' ' + (local: 'value')->type;
			
			select: #comp;
				case: 'true';
					return: #field_value == true;
				case: 'false';
					return: #field_value == false;
				case: 'eq';
					return: #field_value == #value;
				case: 'cn';
					return: #field_value >> #value;
				case: 'rx';
					return: #regexp_value->size > 0;
				case: 'bw';
					return: #field_value->(beginswith: #value);
				case: 'ew';
					return: #field_value->(endswith: #value);
				case: 'gt';
					return: #field_value > #value;
				case: 'gte';			
					return: #field_value >= #value;
				case: 'lt';
					return: #field_value < #value;
				case: 'lte';
					return: #field_value <= #value;
				case: 'ncn';
					return: !(#field_value >> #value);
				case: 'nrx';
					return: #regexp_value->size == 0;
				case: 'neq';			
					return: #field_value != #value;
				case;
					fail: -9956, 'URL Rewrite: Trigger operator "' + #comp + '" not found.';
			/select;
			return: false;
			
		/define_tag;
		
		define_tag: 'action',
				-required='action',
				-required='name',
				-required='value';

			local: '_name' = #name;
			local: '_value' = #value;

			local: 'token_array' = (array: 'server_name', 'server_port', 'response_path', 'response_filepath', 'response_filename',
					'var', 'variable', 'global', 'cookie', 'action_param', 'client_getparam', 'client_postparam');

			if: #_name >> '[';
				iterate: (string_findregexp: #_name, -find='\\[.*?\\]'), (local: 'token');
					local: 'tag' = (string_replaceregexp: #token, -find='\\[(.*?)[:\\]].*', -replace='\\1');
					if: #tag == 'trigger';
						local: 'index0' = (string_findregexp: #token, -find='\\d+');
						local: 'index1' = (#index0->size >= 1) ? (integer: #index0->first) | 1;
						local: 'index2' = (#index0->size >= 2) ? (integer: #index0->second) | 0;
						local: 'tag_value' = '';
						protect;
							local: 'tag_value' = $__triggers__->(get: #index1)->(get: #index2 + 1);
						/protect;
						protect;
							if: #tag_value == '';
								local: 'tag_value' = $__triggers__->(get: #index1);
							/if;
						/protect;
						#_name->(replace: #token, #tag_value);
					else: #token_array >> #tag;
						local: 'tag_value' = (string_replaceregexp: #token, -find='.*["\'](.*?)["\'].*', -replace='\\1');
						#_name->(replace: #token, \#tag->(run: -name=#tag, -params=(array: #tag_value)));
					/if;
				/iterate;
			/if;
			
			if: #_value >> '[';
				iterate: (string_findregexp: #_value, -find='\\[.*?\\]'), (local: 'token');
					local: 'tag' = (string_replaceregexp: #token, -find='\\[(.*?)[:\\]].*', -replace='\\1');
					if: #tag == 'trigger';
						local: 'index0' = (string_findregexp: #token, -find='\\d+');
						local: 'index1' = (#index0->size >= 1) ? #index0->first | 1;
						local: 'index2' = (#index0->size >= 2) ? #index0->second | 1;
						local: 'tag_value' = '';
						protect;
							local: 'tag_value' = $__triggers__->(get: #index1)->(get: #index2);
						/protect;
						protect;
							if: #tag_value == '';
								local: 'tag_value' = $__triggers__->(get: #index1);
							/if;
						/protect;
						#_value->(replace: #token, #tag_value);
					else: #token_array >> #tag;
						local: 'tag_value' = (string_replaceregexp: #token, -find='.*["\'](.*?)["\'].*', -replace='\\1');
						#_value->(replace: #token, \#tag->(run: -name=#tag, -params=(array: #tag_value)));
					/if;
				/iterate;
			/if;
		
			(global: '__rewrite_debug__') ? log_always: 'Rewrite Action: ' + #action + ' ' + (local: 'name') + ' ' + (local: 'name')->type + ' ' + (local: 'value') + ' ' + (local: 'value')->type;

			select: #action;
				case: 'variable';
					variable: #_name = #_value;
				case: 'global';
					global: #_name = #_value;
				case: 'redirect_url';
					define_atend: \redirect_url=(array:#_name);
					var: '__abort__' = true;
				case: 'include';
					namespace_using: '_page_';
						$__html_reply__ += (include: #_name);
					/namespace_using;
				case: 'include_once';
					namespace_using: '_page_';
						$__html_reply__ += (include_once: #_name);
					/namespace_using;
				case: 'library';
					namespace_using: '_page_';
						library: #_name;
					/namespace_using;
				case: 'library_once';
					namespace_using: '_page_';
						library_once: #_name;
					/namespace_using;
				case: 'log_critical';
					log_critical: #_name;
				case: 'log_warning';
					log_warning: #_name;
				case: 'log_detail';
					log_detail: #_name;
				case: 'log_deprecated';
					log_deprecated: #_name;
				case: 'log_sql';
					log_sql: #_name;
				case: 'log_always';
					log_always: #_name;
				case: 'fail';
					fail: (if_empty: (integer: #_value), -1), #_name;
				case: 'abort';
					var: '__abort__' = true;
				case;
					fail: -9956, 'URL Rewrite: Action "' + #action + '" not found.';
			/select;
		/define_tag;
		
		define_tag: 'rule',
				-required='triggers',
				-required='bool',
				-required='actions',
				-required='option';

			var: '__triggers__' = (array);
			var: '__abort__' = false;
			
			error_push;
			
			protect;
				if: #bool == 'or';
					iterate: #triggers, (local: 'trigger');
						local: 'trigger_result' = #trigger->invoke;
						(global: '__rewrite_debug__') ? log_always: '--> ' + #trigger_result;
						if: #trigger_result == true;
							iterate: #actions, (local: 'action');
								#action->invoke;
								$__abort__ ? loop_abort;
							/iterate;
							loop_abort;
						/if;
					/iterate;
					return: (#option >> 'stop');
				else;
					iterate: #triggers, (local: 'trigger');
						local: 'trigger_result' = #trigger->invoke;
						(global: '__rewrite_debug__') ? log_always: '--> ' + #trigger_result;
						if: #trigger_result == false;
							return: false;
						/if;
					/iterate;
					iterate: #actions, (local: 'action');
						#action->invoke;
						$__abort__ ? loop_abort;
					/iterate;
					return: (#option >> 'stop');
				/if;
				handle_error;
					log_warning: 'URL Rewrite: ' + error_code + ' ' + error_msg;
				/handle_error;
			/protect;
			
			error_pop;
			
			return: false;

		/define_tag;
		
	/define_type;
	
	loop: -from=site_atbegin->size, -to=1, -by=-1;
		if: site_atbegin->(get: loop_count) == 'rewrite';
		  site_atbegin->(remove: loop_count);
		/if;
	/loop;
	if: (site_id != 0) && ((admin_getpref: 'lasso_urlrewrite') !>> 'N');
		define_atbegin: url_rewrite;
	/if;
		
?>