<?LassoScript

	// Returns a named parameter within a custom tag, deprecated
	define_tag: 'param', -namespace='named_', -required='name', -priority='replace', -copy, -deprecated;
		iterate: params_up, (local: 'temp');
			if: #temp->(isa: 'pair') && (#temp->first == #name);
				return: #temp->second;
			/if;
		/iterate;
		return: null;
	/define_tag;



	// Convert args to pair array, used internally to parse mime
	define_tag:'_chop_args', -priority='replace';
		local:'out' = (array);
		if: ((params)->(size) == 2) && (params)->(get:1)->(length);
			local:'delim' = (params)->(get:2),'temp'='';
			#out = (params)->(get:1)->(split:#delim);

			loop:(#out->size);
				#temp = #out->(get:loop_count)->(split:'=');
				if: (#temp->(size) == 2);
					#out->(get:loop_count) = (pair:(decode_url:#temp->(get:1)) = (decode_url:#temp->(get:2)));
				else: (#temp->(size) == 1);
					#out->(get:loop_count) = (pair:(decode_url:#temp->(get:1)) = '');
				/if;
			/loop;
		/if;
		return:#out;
	/define_tag;


	// 
	// [String_ValidCharSet: 'charset']
	//
	// Check whether a character set is valid or not
	//
	
	define_tag: 'validcharset', 
			-namespace='string_', 
			-priority='replace', 
			-required='charset';
		if: (#charset == '');
			return: false;
		/if;
		local: 'test' = bytes;
		#test->(importstring: 'test frg string', #charset);
		return: (#test->size > 0);
	/define_tag;

	//
	// [If_True: test, truevalue, falsevalue]
	// [If_False: test, falsevalue, truevalue]
	//
	// Returns first value if condition is as specified, otherwise second value
	//
	// [If_Null: value, default] 
	// [If_Empty: value, default]
	//
	// Returns value if not null or empty, otherwise returns defaults
	//
	
	define_tag: 'true',
			-required='test',
			-required='true',
			-required='false',
			-namespace = 'if_';
		if: (boolean: (local: 'test'));
			return: (local: 'true');
		else;
			return: (local: 'false');
		/if;
	/define_tag;

	define_tag: 'false',
			-required='test',
			-required='false',
			-required='true',
			-namespace = 'if_';
		if: !(boolean: (local: 'test'));
			return: (local: 'false');
		else;
			return: (local: 'true');
		/if;
	/define_tag;

	define_tag: 'null',
			-required='value',
			-required='default',
			-namespace = 'if_';
		if: ((local: 'value') === null);
			return: (local: 'default');
		else;
			return: (local: 'value');
		/if;
	/define_tag;

	define_tag: 'empty',
			-required='value',
			-required='default',
			-namespace = 'if_';
		if: ((local: 'value') == '');
			return: (local: 'default');
		else;
			return: (local: 'value');
		/if;
	/define_tag;

	//
	// [Math_Range: $value, -min=#, -max=#]
	//
	// Returns the value if it is between the min and max, otherwise returns closes extreme
	//
	
	define_tag: 'range',
			-required='val',
			-required='min',
			-required='max',
			-namespace = 'math_';
		if: (#val < #min);
			return: #min;
		else: (#val > #max);
			return: #max;
		else;
			return: #val;
		/if;
	/define_tag;

	//
	// [Admin_GetPref]
	// [Admin_SetPref]
	// [Admin_RemovePref]
	//
	// Can only be used by administrators
	//

	define_tag: 'getpref',
			-required='name',
			-optional='default',
			-optional='site',
			-namespace = 'admin_';

		auth_admin;
		
		error_push;

		if: (local_defined: 'site') && ((integer: #site) > 0) && (site_id == 0);
			local: '_database' = 'lasso_internal_' + (integer: #site);
		else;
			local: '_database' = 'lasso_internal';
		/if;
		local: '_table' = 'global_prefs';

		if: #name->(isa: 'map');

			local: 'inline' = (array);
			iterate: #name->keys, (local: 'key');
				#inline->(insert: -op='eq');
				#inline->(insert: 'store_key' = #key);
			/iterate;

			if: (local_defined: 'default') && #default->(isa: 'map');
				local: 'output' = #default;
			else: #name->(isa: 'map');
				local: 'output' = #name;
			else;
				local: 'output' = map;
			/if;

			inline: -search,
					-database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__,
					-opbegin='or',
					#inline,
					-opend='or',
					-maxrecords='all',
					-returnfield='store_key',
					-returnfield='data';
				records;
					#output->(insert: (field: 'store_key') = (field: 'data'));
				/records;
			/inline;

			error_pop;

			return: @#output;

		else;

			inline: -search,
					-database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__, 
					-op='eq', 'store_key'=#name,
					-maxrecords=1,
					-returnfield='data',
					-uselimit;
				error_pop;
				if: (found_count > 0);
					return: (field: 'data');
				else;
					return: (local: 'default');
				/if;
			/inline;

		/if;
		
		error_pop;

	/define_tag;

	define_tag: 'setpref',
			-required='name',
			-optional='value',
			-optional='site',
			-namespace = 'admin_';

		auth_admin;
		
		error_push;

		if: (local_defined: 'site') && ((integer: #site) > 0) && (site_id == 0);
			local: '_database' = 'lasso_internal_' + (integer: #site);
		else;
			local: '_database' = 'lasso_internal';
		/if;
		local: '_table' = 'global_prefs';

		if: #name->(isa: 'map');

			local: 'deletes' = (array);
			local: 'inserts' = (array);
			iterate: #name, (local: 'temp');
				#deletes->(insert: #temp->first);
				#inserts->(insert: 'INSERT INTO ' + #_table + '(store_key,data) VALUES (\'' + #temp->first + '\',\'' + #temp->second +'\');');
			/iterate;
			
			inline: -database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__, 
					-sql='DELETE FROM ' + #_table + ' WHERE store_key IN (\'' + #deletes->(join: '\',\'') + '\')';
			/inline;

			inline: -database=#_database, 
					-table=#_table, 
					-log=$__startup_loglevel__,
					-sql='BEGIN TRANSACTION; ' + #inserts->(join: ' ') + ' COMMIT TRANSACTION;';
			/inline;
			
		else;

			inline: -delete,
					-database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__,
					-keyfield='store_key',
					-keyvalue=#name;
			/inline;

			inline: -add,
					-database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__,
					'store_key' = #name,
					'data' = (local: 'value'),
					-maxrecords=0;
			/inline;

		/if;
		
		error_pop;

	/define_tag;

	define_tag: 'removepref',
			-required='name',
			-optional='site',
			-namespace = 'admin_';

		auth_admin;
		
		error_push;

		if: (local_defined: 'site') && ((integer: #site) > 0) && (site_id == 0);
			local: '_database' = 'lasso_internal_' + (integer: #site);
		else;
			local: '_database' = 'lasso_internal';
		/if;
		local: '_table' = 'global_prefs';

		if: #name->(isa: 'map');

			local: 'deletes' = (array);
			iterate: #name, (local: 'temp');
				#deletes->(insert: #temp->first);
			/iterate;
			
			inline: -database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__, 
					-sql='DELETE FROM ' + #_table + ' WHERE store_key IN (\'' + #deletes->(join: '\',\'') + '\')';
			/inline;
			
		else;

			inline: -delete,
					-database=#_database,
					-table=#_table,
					-log=$__startup_loglevel__,
					-keyfield='store_key',
					-keyvalue=#name;
			/inline;

		/if;

		error_pop;

	/define_tag;
	
	define_tag: '_chartfx_activation_string', -privileged;
	  local: 'output' = (admin_getpref: 'chartfx_activation_string');
	  return: @#output;
	/define_tag;

?>