<?LassoScript

define_type:'find_soap_ops', -prototype, -description='Utility type which compares equally against operation notes which do NOT have soap:operation';
	define_tag:'onCompare', -required='to';
		if: #to->extract("*[local-name()='operation' and namespace-uri()='" $SOAP "']")->size == 0;
			return: 0;
		/if;
		return:-1;
	/define_tag;
/define_type;

namespace_using:'wsdl';

define_tag:'makeTest', -required='name', -required='ns';
	return: "[local-name() = '" #name "']";// and namespace-uri() = '" #ns "']";
/define_tag;

define_tag:'getOperation', -required='xml', -type='xml', 
                                -required='named', -type='string';
    return:#xml->extractOne("/*" wsdl_makeTest('definitions', $WSDL) "/*" wsdl_makeTest('binding', $WSDL) "/*[local-name()='operation' and @name = '" #named "']"); /* */
/define_tag;

define_tag:'listOperations', -required='xml', -type='xml';
	return: #xml->extract("/*" wsdl_makeTest('definitions', $WSDL) 
							"/*" wsdl_makeTest('binding', $WSDL) 
								"/*" wsdl_makeTest('operation', $WSDL) )->removeAll(find_soap_ops) &; /* */
/define_tag;

define_tag:'getOpSoapAction', -required='xml', -type='xml';
    return:#xml->extract("normalize-space(*[namespace-uri()='" $SOAP "' and local-name()='operation']/@soapAction)");
/define_tag;

define_tag:'getOpLocation', -required='xml', -type='xml', -required='op';
	local:'opX' = @wsdl_getOperation(#xml, #op);
	local:'nm' = #opX->extract('normalize-space(parent::*/@name)');
	return: #xml->extract('normalize-space(/*' wsdl_makeTest('definitions', $WSDL) 
																'/*' wsdl_makeTest('service', $WSDL) 
																'/*' wsdl_makeTest('port', $WSDL) 
																"[contains(@binding, '" #nm "')]/*" wsdl_makeTest('address', $SOAP) 
																'/@location)');
/define_tag;

define_tag:'getOpMessageTypes', -required='xml', -type='xml';
    local:'opName'=#xml->extract('normalize-space(@name)');
    local:'bindingType'=#xml->extract('normalize-space(parent::*/@type)');
    local:'tns' = #xml->extract('normalize-space(parent::*/parent::*/@targetNamespace)');
    local:'normalized' = xsd_ref:xsd_fullyQualifyName(#xml, #tns, #bindingType);
    local:'name' = #normalized->name;

    // find a portType whose parent definitions has a targetNamespace as specified
    local:'fnd' = #xml->extractOne(
        "//*[local-name()='definitions' and @targetNamespace='" 
        #tns 
        "']/*[local-name()='portType' and @name = '" #name "']/*[local-name()='operation' and @name = '" #opName "']"
        );
        
    if: #fnd->isa:'xml';
        local:'inmsg' = xsd_fullyQualifyName(#fnd, #tns, #fnd->extractOne("normalize-space(*[local-name()='input']/@message)"));
        local:'outmsg' = xsd_fullyQualifyName(#fnd, #tns, #fnd->extractOne("normalize-space(*[local-name()='output']/@message)"));
        return:pair:#inmsg=#outmsg;
    /if;

    fail:-1, 'Could not find message types.';
/define_tag;

define_tag:'getOpBodyUse', -required='xml', -type='xml';
    return:#xml->extract("normalize-space(*" wsdl_makeTest('input', $WSDL) "/*" wsdl_makeTest('body', $SOAP) "/@use)");
/define_tag;

define_tag:'getOpBodyStyle', -required='xml', -type='xml';
    local:'x' = #xml->extractOne("normalize-space(*" wsdl_makeTest('operation', $SOAP) "/@style)");
    if:null == #x;
    	#x = #xml->extractOne("normalize-space(../*" wsdl_makeTest('binding', $SOAP) "[@style]/@style)");
    /if;
    return:#x;
/define_tag;

define_tag:'getTargetNamespace', -required='xml', -type='xml';
	return:#xml->extract('normalize-space(/*' wsdl_makeTest('definitions', $WSDL) '/@targetNamespace)');
/define_tag;

define_tag:'getServices', -required='xml', 
		-description='Returns an array of the service elements in the wsdl.';
	return: (#xml->extract:"/*[local-name()='definitions']/*[local-name()='service']");/* */
/define_tag;

define_tag:'getServiceDocumentation', -required='srvc', -description='Returns the documentation element for the service.';
	local:'r' = @#srvc->(extract:"*[local-name()='documentation']/text()");
	if:#r && #r->size;
		return: #r->(get:1)->contents;
	/if;
	return:'';
/define_tag;

define_tag:'getPortsForService', -required='srvc', 
		-description='Returns the ports for the given service.';
	return:@(#srvc->extract:"*[local-name()='port']");
/define_tag;

define_tag:'getPortAddress', -required='port', -description='Returns the address location for the port.';
	local:'r'=@#port->(extract:"*[local-name()='address']/@location");
	if:#r->size;
		return:#r->(get:1)->contents;
	/if;
	return:'';
/define_tag;

define_tag:'getBinding', -required='xml', -required='name',
		-description='Returns the binding for the given name.';
	
	local:'r'=@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='binding'][@name='" + #name + "']"));/* */
	if:#r->size;
		return:@(#r->get:1);
	/if;
/define_tag;

define_tag:'getBindingForOperation', -required='xml', -required='name';
	local:'str'= "/*[local-name()='definitions']/*[local-name()='binding']/*[local-name()='operation' and @name='";/* */
	#str += #name;
	#str += "']/..";
	local:'r' = @(#xml->extract:#str);
	if:#r->size;
		return:#r->get:1;
	/if;
/define_tag;

define_tag:'getMessageParts', -required='xml', -required='name',
		-description='Returns the message for the given name.';
	local: 'result' = #xml->extract("/*[local-name()='definitions']/*[local-name()='message' and @name='" + #name + "']/*[local-name()='part']");/* */
	// Ensure result is an Array
	#result == null ? return: array;
	return: @#result;
/define_tag;

define_tag:'getPortType', -required='xml', -required='name',
		-description='Returns the portType for the given name.';
	return:@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='portType'and @name='" + #name + "']"))->get:1;/* */
/define_tag;

define_tag:'getBindingOperations', -required='binding', 
		-description='Returns the operations listed for the given binding.';
	return:@(#binding->extract:"*[local-name()='operation']");
/define_tag;

define_tag:'getPortTypeOperation', -required='xml', -required='name';
	return:@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='portType']/*[local-name()='operation'][@name='" + #name + "']"))->get:1;/* */
/define_tag;

define_tag:'isSoapOperation', -required='op', -description='Checks the element name to see if it is a SOAP operation.';
	return:#op->name == (#soapns + 'operation');
/define_tag;



define_tag:'getMessageTrioFromPortType', -required='portType', -required='opName';
	local:'str'="*[local-name()='operation' and @name='";
	#str += #opName;
	#str += "']/*[local-name()='input' or local-name()='output' or local-name()='fault']";/* */
	local:'r'=@(#portType->extract:#str);
	if:#r->size;
		return:#r;
	/if;
/define_tag;

define_tag:'getMessageNamed', -required='xml', -required='name';
	return:#xml->extractOne("/*[local-name()='definitions']/*[local-name()='message' and @name='" #name "']");
/define_tag;

define_tag:'extract', -required='xml', -required='opName';
	local:'ret'=map;
	// find the definitions node
	// should only be one. if there are multiple, get only the first
	local:'defs'=(#xml->extract:"/*[local-name()='definitions']");/* */
	if:#defs->size;
		#xml = #defs->get:1;
	/if;
	fail_if:((xml_removeNamespace:#xml->name) != 'definitions'), -1, 'The file does not appear to be a valid wsdl file. ' + #xml->name;
	
	// grab the namespaces
	local:'wsdlns'='';
	local:'soapns'='';
	local:'xsdns'='';
	local:'targetNs'=(xml_getAttribute:#xml, 'targetNamespace');	
	local:'tns'='';
	
	local:'namespaces'= @(#xml->namespaces);
	iterate:#namespaces, local:'curns';
		// don't process default namespace
		if:#curns->first; // default ns will have null prefix
			select:#curns->second;
				case:'http://schemas.xmlsoap.org/wsdl/';
					#wsdlns = #curns->first;
				case:'http://www.w3.org/2001/XMLSchema';
					#xsdns = #curns->first;
				case:'http://schemas.xmlsoap.org/wsdl/soap/';
					#soapns = #curns->first;
				case:#targetNs;
					#tns = #curns->first;
			/select;
		/if;
	/iterate;
	if:#soapns != '';
		#soapns += ':';
	/if;
	if:#tns != '';
		#tns += ':';
	/if;
	
	// find the binding for the spec'd operation
	local:'binding'= @(wsdl_getBindingForOperation:#xml, #opName);
	// find the portType for this binding
	local:'portType'= @(wsdl_getPortType:#xml, xml_removeNamespace:(xml_getAttribute:#binding, 'type'));
	// grab the input, output and fault nodes
	local:'iofNames' = @(wsdl_getMessageTrioFromPortType:#portType, #opName);
	
	local:'input', 'output', 'fault';
	
	iterate:#iofNames, local:'scratch';
		select:xml_removeNamespace:(#scratch->name);
			case:'input';
				#input = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
			case:'output';
				#output = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
			case:'fault';
				#fault = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
		/select;
	/iterate;
	
	local:'inputTypes'=array;
	local:'outputTypes'=array;
	
	// find the schema section(s)
	local:'schemas'=@(xml_getSchemas:#xml);
	
	define_tag:'getSchemaPart', -required='schemas', -required='typeName';
		iterate:#schemas, local:'i';
			local:'answer'= @(#i->extractOne:"*[@name='" + #typeName + "']");
			if:#answer->isa:'xml';
				return:@#answer;
			/if;
		/iterate;
	/define_tag;
	
	fail_if:#schemas->size == 0, -1, 'Could not find any valid schemas in the WSDL.';
	
	iterate:#input->children, #scratch;
		if:#scratch->name == 'part';
			local:'typeName' = xml_removeNamespace:(xml_getAttribute:#scratch, 'type');
			#inputTypes->insert:@(getSchemaPart:#schemas, #typename);
		/if;
	/iterate;
	
	iterate:#output->children, #scratch;
		if:#scratch->name == 'part';
			local:'typeName' = xml_removeNamespace:(xml_getAttribute:#scratch, 'type');
			#outputTypes->insert:@(getSchemaPart:#schemas, #typename);
		/if;		
	/iterate;
	
	#ret->insert:'operation'='opName';
	#ret->insert:'input'=@#inputTypes;
	#ret->insert:'output'=@#outputTypes;
	return:@#ret;
/define_tag;

/namespace_using;
//wsdl_extract:xml:include_url:'http://soap.amazon.com/schemas/AmazonWebServices.wsdl', 'ArtistSearchRequest';

?>