import com.omnipilot.lassopro.*;
import com.lowagie.text.*;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.*;
import java.util.*;
import java.io.*;
import java.awt.Color;

/**
 * Basic Documentation:To handle the functionality of the PDF Tags of the LassoScript.
 * Responsiblities:
 * 1.To provide implementation methods for tags like PDF_DOC.PDF_FONT,PDF_TABLE,PDF_GRAPHIC.
 * @author	- Mindfire Solutions (www.mindfiresolutions.com)
 */

public class PDFTag extends LassoTagModule
{
	private int gerr = 0;

	public PDFTag ()
	{

	}

	public void	finalize()
	{

	}

	/**
	 * This method primarily handles the PDF_Doc tag and its accessor and member tags.
	 */

	public int tagPDFDocFunc (LassoCall lasso, int action)
	{
	int err = 1;

	try
	{
		PDFDoc pdfdoc	=	null;
		pdfdoc 			= 	new PDFDoc();
		LassoValue result 		= new LassoValue();

		IntValue j = new IntValue();
		lasso.getTagParamCount(j);

		//get the -file parameter.
		String filename	= getParamValue(lasso,"-file");

		LassoTypeRef outWaterMark 	= new LassoTypeRef();
		LassoTypeRef ele 			= new LassoTypeRef();
		LassoValue typeName 		= new LassoValue();

		String strWaterMarkImage	= null;
		float leftWaterMark 		= 0;
		float topWaterMark 			= 0;

		// get the -watermark parameter.
		if( lasso.findTagParam2("-watermark", outWaterMark ) == ERR_NOERR &&
			lasso.typeGetName(outWaterMark, typeName) == ERR_NOERR)
		{
			if((typeName.name()).equalsIgnoreCase("map"))
			{
				LassoTypeRef keyRef = new LassoTypeRef();
				lasso.typeAllocString(keyRef, "image");

				if(ERR_NOERR == lasso.mapFindElement(outWaterMark, keyRef, ele))
				{
					LassoValue outStringValue = new LassoValue();
					if(ERR_NOERR != lasso.typeGetString(ele, outStringValue))
					{
						strWaterMarkImage = null;
					}
					else
					{
						strWaterMarkImage = outStringValue.name();

						//get left and top values
						lasso.typeAllocString(keyRef, "left");
						DecimalValue fVal = new DecimalValue();
						if(ERR_NOERR == lasso.mapFindElement(outWaterMark, keyRef, ele))
						{
							if(ERR_NOERR == lasso.typeGetDecimal(ele, fVal))
								 leftWaterMark = fVal.floatValue();
						}

						lasso.typeAllocString(keyRef, "top");
						if(ERR_NOERR == lasso.mapFindElement(outWaterMark, keyRef, ele))
						{
							if(ERR_NOERR == lasso.typeGetDecimal(ele, fVal))
								 topWaterMark = fVal.floatValue();
						}
					}
				}
				else
				{
					strWaterMarkImage = null;
				}
			}
		}

		if(filename != null)
		{
			//EARLIER COMMENTED AS THIS METHOD WAS NOT DEFINED IN THE DLL
		//	if (lasso.operationAllowed(LassoSecureOperations.FILE_CREATE, filename) != ERR_NOERR)
		//		return setError(lasso, "File could not be created in specified path", LassoErrors.CouldNotCreateOrOpenFile);
			filename = internalToFullPath(lasso, filename);
		}

		if(strWaterMarkImage != null)
			strWaterMarkImage = internalToFullPath(lasso, strWaterMarkImage);


		//optionally get -format parameter
		String 	format	= getParamValue(lasso,"-format");

		pdfdoc.createDoc(filename, strWaterMarkImage, leftWaterMark, topWaterMark, format );


		//optionally get -width,-height | -size  parameter
		long width = -1;
		long height= -1;

		String strWidth	= getParamValue(lasso,"-width");
		if(strWidth != null) width = (new Long(strWidth)).longValue();

		String strHeight= getParamValue(lasso,"-height");
		if(strHeight != null && strWidth != null)
		{
			height = (new Long(strHeight)).longValue();
			Rectangle rec = new Rectangle(width,height);
			err = pdfdoc.setPageSize(rec);
		}

		//get the -size parameter.
		String strSize;
		if(height < 0 && width < 0)
		{
			strSize = getParamValue(lasso,"-size");
			err = pdfdoc.setPageSize(strSize);

		}

		LassoTypeRef res 	= new LassoTypeRef();
		LassoTypeRef element= new LassoTypeRef();

		float [] tempArr	= new float[4];
		float marginLeft 	= 0;
		float marginRight 	= 0;
		float marginTop 	= 0;
		float marginBottom	= 0;

		// get the -margins parameter.
		if( lasso.findTagParam2("-margins", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
		{
			if((typeName.name()).equalsIgnoreCase("array"))
			{
				IntValue count = new IntValue();
				if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
				{
					for ( int i = 0; i < count.intValue(); i++ )
					{
						lasso.arrayGetElement(res, i, element);

						LassoValue typeName2 = new LassoValue();

						lasso.typeGetName(element,typeName2);

						DecimalValue outDecimalValue = new DecimalValue();
						float fValue = 0;

						if((typeName2.name()).equalsIgnoreCase("decimal"))
						{

						lasso.typeGetDecimal(element, outDecimalValue);
						fValue = outDecimalValue.floatValue();

						}
						else if((typeName2.name()).equalsIgnoreCase("integer"))
						{

						IntValue outIntValue = new IntValue();
						lasso.typeGetInteger(element, outIntValue);
						fValue = outIntValue.intValue();
						}
						tempArr[i] = fValue;
					}
				}
			}
			//call the setmargins function
			marginLeft 	= tempArr[0];
			marginRight = tempArr[1];
			marginTop	= tempArr[2];
			marginBottom= tempArr[3];
			err = pdfdoc.setMargins(marginLeft, marginRight, marginTop, marginBottom);
		}

		//optionally get -color parameter
		String strColor	= getParamValue(lasso,"-color");
		pdfdoc.setDocColor(strColor);

		//optionally get -usedate parameter
		IntValue bValue	= getBooleanValue(lasso,"-usedate");

		if(bValue.intValue() != 0)
			pdfdoc.setUseDate();

		//optionally get 'author' parameter
		String strAuthor	= getParamValue(lasso,"author");
		if(strAuthor != null)
			pdfdoc.setDocAuthor(strAuthor);

		//optionally get 'title' parameter
		String strTitle	= getParamValue(lasso,"title");
		if(strTitle != null)
			pdfdoc.setDocTitle(strTitle);

		//optionally get 'subject' parameter
		String strSubject = getParamValue(lasso,"subject");
		if(strSubject != null)
			pdfdoc.setDocSubject(strSubject);

		//optionally get 'keywords' parameter
		String strKeywords = getParamValue(lasso,"keywords");
		if(strKeywords != null)
			pdfdoc.setDocKeywords(strKeywords);

		//optionally get 'creator' parameter
		String strCreator = getParamValue(lasso,"creator");
		if(strCreator != null)
			pdfdoc.setDocCreator(strCreator);

		//optionally get -nocompress  parameter

		bValue	= getBooleanValue(lasso,"-nocompress");

		if(bValue.intValue() == 1)
			pdfdoc.setCompress(false);
		else
			pdfdoc.setCompress(true);

		//optionally get -pageheader parameter
		String strHeader	= getParamValue(lasso,"-pageheader");
		if(strHeader!=null) pdfdoc.setHeader(strHeader);

		//optionally get -pageno  parameter
		int pageno = 0;
		String strPageNo	= getParamValue(lasso,"-pageno");
		if(strPageNo != null)
		{
		 pageno = (new Integer(strPageNo)).intValue();
		 pdfdoc.setDocPageNumber(pageno);
		}

		/**
		 * After all the initializers have been called call the operation() function to open up the document.
		 */

		pdfdoc.openDoc();


		/**
		 *  PDF_Doc Tag
		 *	Handling the Accessors & the Member tags of PDF_Doc
		 */

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Doc") == ERR_NOERR )
		{

			/**
		 	 * Handling the Add accessor method of PDF_Doc tag.
		     */
			LassoTypeRef add = new LassoTypeRef();
			if (lasso.typeAllocTag(add,"PDFTag", "myDocAddFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "Add", add ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}


			/**
		 	 * Handling the InsertPage member tag of PDF_Doc tag.
		 	 * This method adds a new page.
		     */

			LassoTypeRef insertPage = new LassoTypeRef();
			if (lasso.typeAllocTag(insertPage,"PDFTag", "myInsertPageFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "InsertPage", insertPage)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the GetMargins accessor method of PDF_Doc tag.
		 	 * This method returns margins of the document.
		     */
			LassoTypeRef getmargins = new LassoTypeRef();
			if (lasso.typeAllocTag(getmargins,"PDFTag", "myGetMarginsFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetMargins", getmargins ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetColor accessor method of PDF_Doc tag.
		 	 * This method returns color of the document.
		     */

			LassoTypeRef getcolor = new LassoTypeRef();
			if (lasso.typeAllocTag(getcolor,"PDFTag", "myGetDocColorFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetColor", getcolor ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetFormat accessor method of PDF_Doc tag.
		 	 * This method returns format  of the document.
		     */

			LassoTypeRef getformat = new LassoTypeRef();
			if (lasso.typeAllocTag(getformat,"PDFTag", "myGetDocFormatFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetFormat", getformat ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetHeaders accessor method of PDF_Doc tag.
		 	 * This method gets all the specified headers of  the document.
		     */

			LassoTypeRef getheaders = new LassoTypeRef();
			if (lasso.typeAllocTag(getheaders,"PDFTag", "myGetDocHeadersFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetHeaders", getheaders ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetHeader accessor method of PDF_Doc tag.
		 	 * This method returns the Header of the document.
		     */

			LassoTypeRef getheader = new LassoTypeRef();
			if (lasso.typeAllocTag(getheader,"PDFTag", "myGetDocHeaderFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetHeader", getheader ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetSize accessor method of PDF_Doc tag.
		 	 * This method returns the size of the document.
		     */

			LassoTypeRef getsize = new LassoTypeRef();
			if (lasso.typeAllocTag(getsize,"PDFTag", "myGetSizeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetSize", getsize ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetPageNumber accessor method of PDF_Doc tag.
		 	 * This method returns the page number of the document.
		     */

			LassoTypeRef getpgnum = new LassoTypeRef();
			if (lasso.typeAllocTag(getpgnum,"PDFTag", "myGetPageNumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetPageNumber", getpgnum ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetPageNumber member tag of PDF_Doc tag.
		 	 * This method sets the pagecount number of the document.
		     */

			LassoTypeRef setpgnum = new LassoTypeRef();
			if (lasso.typeAllocTag(setpgnum,"PDFTag", "mySetPageNumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetPageNumber", setpgnum ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the AddPage member tag of PDF_Doc tag.
		 	 * This method signals that a new page has to be started.
		     */

			LassoTypeRef addpage = new LassoTypeRef();
			if (lasso.typeAllocTag(addpage,"PDFTag", "myAddPageFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddPage", addpage )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the NewPage accessor method of PDF_Doc tag.
		 	 * This method starts a new page after commitijng the previous page.
		     */

			LassoTypeRef newpage = new LassoTypeRef();
			if (lasso.typeAllocTag(newpage,"PDFTag", "myAddPageFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "NewPage", newpage )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the SetColor member tag of PDF_Doc tag.
		 	 * This method sets the color of  the document.
		     */

			LassoTypeRef drawcolor = new LassoTypeRef();
			if (lasso.typeAllocTag(drawcolor,"PDFTag", "mySetColorFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "SetColor", drawcolor)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the SetLineWidth of PDF_Doc tag.
		 	 * This method sets the line width to the specified value.
		     */

			LassoTypeRef linewidth = new LassoTypeRef();
			if (lasso.typeAllocTag(linewidth,"PDFTag", "mySetLineWidthFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "SetLineWidth", linewidth)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the Line member tag of PDF_Doc tag.
		 	 * This method draws an line on the document.
		     */

			LassoTypeRef line = new LassoTypeRef();
			if (lasso.typeAllocTag(line,"PDFTag", "myLineFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Line", line)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the Rect member tag of PDF_Doc tag.
		 	 * This method draws an Rectangle on the document.
		     */

			LassoTypeRef rect = new LassoTypeRef();
			if (lasso.typeAllocTag(rect,"PDFTag", "myRectFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Rect", rect)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the MoveTo member tag of PDF_Doc tag.
		 	 * This method moves the pen position to the given coordinates  on the document.
		     */

			LassoTypeRef moveto = new LassoTypeRef();
			if (lasso.typeAllocTag(moveto,"PDFTag", "myMoveToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "MoveTo", moveto)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the CurveTo member tag of PDF_Doc tag.
		 	 * This method draws a Curve on the document.
		     */

			LassoTypeRef curveto = new LassoTypeRef();
			if (lasso.typeAllocTag(curveto,"PDFTag", "myCurveToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "CurveTo", curveto)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the Circle member tag of PDF_Doc tag.
		 	 * This method draws an circle on the document.
		     */

			LassoTypeRef circle = new LassoTypeRef();
			if (lasso.typeAllocTag(circle, "PDFTag", "myCircleFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Circle", circle)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the Arc member tag of PDF_Doc tag.
		 	 * This method draws an arc on the document.
		     */

			LassoTypeRef arc = new LassoTypeRef();
			if (lasso.typeAllocTag(arc, "PDFTag", "myArcFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Arc", arc)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the ClosePathFillStroke member tags of PDF_Doc tag.
		 	 * This method closes the path after drawing operations.
		     */

			LassoTypeRef closepath = new LassoTypeRef();
			if (lasso.typeAllocTag(closepath, "PDFTag", "myClosePathFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "ClosePath", closepath)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddText member tag of PDF_Doc tag.
		 	 * This method adds a Text or a Chunk to the document.
		     */

			LassoTypeRef addtext = new LassoTypeRef();
			if (lasso.typeAllocTag(addtext, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddText", addtext )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the DrawText member tag of PDF_Doc tag.
		 	 * This method adds a Text or a Chunk to the document at a specified position.
		     */

			LassoTypeRef drawtext = new LassoTypeRef();
			if (lasso.typeAllocTag(drawtext, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "DrawText", drawtext )) != ERR_NOERR )
				{
					return err;
				}
			}
			/**
		 	 * Handling the AddPhrase member tag of PDF_Doc tag.
		 	 * This method adds a Phrase to the document.
		     */

			LassoTypeRef addphrase = new LassoTypeRef();
			if (lasso.typeAllocTag(addphrase, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddPhrase", addphrase )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddParagraph member tag of PDF_Doc tag.
		 	 * This method adds a Paragraph to the document.
		     */

			LassoTypeRef addparagraph = new LassoTypeRef();
			if (lasso.typeAllocTag(addparagraph, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddParagraph", addparagraph)) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddSection member tag of PDF_Doc tag.
		 	 * This method adds a Section to the document.
		     */

			LassoTypeRef addsection = new LassoTypeRef();
			if (lasso.typeAllocTag(addsection, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddSection", addsection )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddChapter member tag of PDF_Doc tag.
		 	 * This method adds a chapter to the document.
		     */

			LassoTypeRef addchapter = new LassoTypeRef();
			if (lasso.typeAllocTag(addchapter, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddChapter", addchapter )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddImage member method of PDF_Doc tag.
		 	 * This method adds an Image to the document.
		     */

			LassoTypeRef addimage = new LassoTypeRef();
			if (lasso.typeAllocTag(addimage , "PDFTag", "myAddImageFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddImage", addimage )) != ERR_NOERR )
				{
					return err;
				}
			}

	     	/**
		 	 * Handling the AddList accessor method of PDF_Doc tag.
		 	 * This method adds a list to the document containing the Listitems.
		     */

			LassoTypeRef addlist = new LassoTypeRef();
			if (lasso.typeAllocTag(addlist, "PDFTag", "myAddTextFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddList", addlist )) != ERR_NOERR )
				{
					return err;
				}
			}

      	/** LassoTypeRef eletype = new LassoTypeRef();
			if (lasso.typeAllocTag(eletype, "PDFTag", "myGetElementTypeFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "GetElementType", eletype )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef ele = new LassoTypeRef();
			if (lasso.typeAllocTag(ele, "PDFTag", "myGetElementFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "GetElement", ele )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef elecount = new LassoTypeRef();
			if (lasso.typeAllocTag(elecount, "PDFTag", "myGetElementCountFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "GetElementCount", elecount )) != ERR_NOERR )
				{
					return err;
				}
			}
		*/


			/**
		 	 * Handling the Close member method of PDF_Doc tag.
		 	 * This method closes  the document and commits every thing to the disk.
		     */

			LassoTypeRef closedoc = new LassoTypeRef();
			if (lasso.typeAllocTag(closedoc,"PDFTag", "myCloseDocFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Close", closedoc )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddGraphics member method of PDF_Doc tag.
		 	 * This method adds the Graphics object to the document.


			LassoTypeRef addgraphics = new LassoTypeRef();
			if (lasso.typeAllocTag(addgraphics,"PDFTag", "myAddGraphicsFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddGraphics", addgraphics )) != ERR_NOERR )
				{
					return err;
				}
			}
			*/

			/**
		 	 * Handling the AddTable membertag method of PDF_Doc tag.
		 	 * This method adds the table object to the  Document.
		     */

			LassoTypeRef addtable = new LassoTypeRef();
			if (lasso.typeAllocTag(addtable,"PDFTag", "myAddTableFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddTable", addtable )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the AddBarCode membertag method of PDF_Doc tag.
		 	 * This method adds the barcode object to the  Document.
		     */

			LassoTypeRef addbarcode = new LassoTypeRef();
			if (lasso.typeAllocTag(addbarcode,"PDFTag", "myAddBarcodeFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddBarcode", addbarcode )) != ERR_NOERR )
				{
					return err;
				}
			}

			/**
		 	 * Handling the SetFont accessor method of PDF_Doc tag.
		 	 * This method sets the font for the document.
		     */

			LassoTypeRef setfont = new LassoTypeRef();
			if (lasso.typeAllocTag(setfont,"PDFTag", "mySetFontFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "SetFont", setfont )) != ERR_NOERR )
				{
					return err;
				}
			}


			LassoTypeRef oncnvt = new LassoTypeRef();
			if (lasso.typeAllocTag(oncnvt,"PDFTag", "myOnConvertFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "onconvert", oncnvt )) != ERR_NOERR )
				{
					return err;
				}
			}

			/* added nov 2003 */
			LassoTypeRef bytes = new LassoTypeRef();
			if (lasso.typeAllocTag(bytes,"PDFTag", "myGetBytes" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Data", bytes )) != ERR_NOERR )
				{
					return err;
				}
			}

// Acro-Form Stuff July-13, 2002
			LassoTypeRef addtf = new LassoTypeRef();
			if (lasso.typeAllocTag(addtf,"PDFTag", "myAddTextFieldFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddTextField", addtf )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addpf = new LassoTypeRef();
			if (lasso.typeAllocTag(addpf,"PDFTag", "myAddPasswordFieldFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddPasswordField", addpf )) != ERR_NOERR )
				{
					return err;
				}
			}


			LassoTypeRef addta = new LassoTypeRef();
			if (lasso.typeAllocTag(addta,"PDFTag", "myAddTextAreaFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddTextArea", addta )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addcb = new LassoTypeRef();
			if (lasso.typeAllocTag(addcb,"PDFTag", "myAddCheckBoxFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddCheckBox", addcb )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addrb = new LassoTypeRef();
			if (lasso.typeAllocTag(addrb,"PDFTag", "myAddRadioButtonFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddRadioButton", addrb )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addrg = new LassoTypeRef();
			if (lasso.typeAllocTag(addrg,"PDFTag", "myAddRadioGroupFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddRadioGroup", addrg )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addcmb = new LassoTypeRef();
			if (lasso.typeAllocTag(addcmb,"PDFTag", "myAddComboBoxFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddComboBox", addcmb )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addsl = new LassoTypeRef();
			if (lasso.typeAllocTag(addsl,"PDFTag", "myAddSelectListFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddSelectList", addsl )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef addhf = new LassoTypeRef();
			if (lasso.typeAllocTag(addhf,"PDFTag", "myAddHiddenFieldFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddHiddenField", addhf )) != ERR_NOERR )
				{
					return err;
				}
			}


			LassoTypeRef addsb = new LassoTypeRef();
			if (lasso.typeAllocTag(addsb,"PDFTag", "myAddSubmitButtonFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddSubmitButton", addsb )) != ERR_NOERR )
				{
					return err;
				}
			}


			LassoTypeRef addrstb = new LassoTypeRef();
			if (lasso.typeAllocTag(addrstb,"PDFTag", "myAddResetButtonFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "AddResetButton", addrstb )) != ERR_NOERR )
				{
					return err;
				}
			}

// End Acro-Form


			if((err = lasso.typeSetCustomJavaObject(me, pdfdoc)) != ERR_NOERR)
			{
				return setError(lasso, "PDF_Doc:Error setting pdf_doc object", err);
			}

			lasso.returnTagValue(me);

		} //end for MEMBER TAGS.
	}
		catch(Exception e) {
		e.printStackTrace();
		lasso.setResultMessage(e.toString());
		lasso.setResultCode(LassoErrors.UnknownError);
		err = LassoErrors.UnknownError;
		return err;
		}

		return 0;
	}


	/** PDF_Doc Tag
	 *	Implementation of all the Member Tags functions of PDF_Doc.
	 *	GetColor,GetSize,GetMargins,AddText,AddPhrase etc...........
	 */

	/**
	 * This method adds an object to the document.
	 * @param lasso
	 * @param action
	 */
	public int myDocAddFunc(LassoCall lasso, int action)
	{
		int err 				= 0;
		LassoTypeRef me			= new LassoTypeRef();
		PDFDoc retPdfDoc		= null;
		PDFImage pdfImage		= null;
		PDFList pdfList			= null;
		PDFText pdfText			= null;
		PDFBarCode pdfBarCode	= null;
		PDFTable pdfTable		= null;

		String	sAlign = "";
		IntValue iWrap = null;
		String	sLeft = "";
		String	sTop = "";
		String	sWidth = "";
		String	sHeight = "";

		if (lasso.getTagSelf(me) == ERR_NOERR )
		retPdfDoc = this.getPdfDoc(lasso, me);

		LassoTypeRef outValue = new LassoTypeRef();
		err = lasso.getTagParam2(0, outValue );

		if(err == ERR_NOERR)
		{
			LassoValue typeName = new LassoValue();
			if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
			{
				String objectTypeName = typeName.name();

				if(objectTypeName.equalsIgnoreCase("String"))
				{
					String strContent = getString(lasso);	//the String to be added to the list

					if(retPdfDoc!= null && strContent != null)
					{
						if((err = retPdfDoc.docAddString(strContent)) == ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding String", err);
					}
				}
				//if object is an IMAGE
				else if(objectTypeName.equalsIgnoreCase("PDF_Image"))
				{
					pdfImage =	this.getPdfImage(lasso, outValue);

					sAlign = getParamValue(lasso,"-align");
					iWrap = getBooleanValue(lasso,"-wrap");
					sLeft = getParamValue(lasso,"-left");
					sTop = getParamValue(lasso,"-top");
					sWidth = getParamValue(lasso,"-width");
					sHeight = getParamValue(lasso,"-height");
					boolean bWrap = false;
					float fLeft = -1;
					float fTop = -1;
					float fWidth = -1;
					float fHeight = -1;

					if(sAlign==null)
						sAlign="";
					bWrap = (iWrap.intValue()==1);
					if(sLeft!=null && !sLeft.trim().equals(""))
						fLeft = Float.parseFloat(sLeft);
					if(sTop!=null && !sTop.trim().equals(""))
						fTop = Float.parseFloat(sTop);
					if(sWidth!=null && !sWidth.trim().equals(""))
						fWidth = Float.parseFloat(sWidth);
					if(sHeight!=null && !sHeight.trim().equals(""))
						fHeight = Float.parseFloat(sHeight);

					if(pdfImage!=null)
					{
						if( (err = retPdfDoc.docAddImage(pdfImage, sAlign, bWrap, fLeft, fTop, fWidth, fHeight))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding Image", err);
					}
				}
				//if object is a BarCode
				else if(objectTypeName.equalsIgnoreCase("PDF_BarCode"))
				{
					pdfBarCode =	this.getPdfBarCode(lasso, outValue);

					sAlign = getParamValue(lasso,"-align");
					iWrap = getBooleanValue(lasso,"-wrap");
					sLeft = getParamValue(lasso,"-left");
					sTop = getParamValue(lasso,"-top");
					sWidth = getParamValue(lasso,"-width");
					sHeight = getParamValue(lasso,"-height");
					boolean bWrap = false;
					float fLeft = -1;
					float fTop = -1;
					float fWidth = -1;
					float fHeight = -1;

					if(sAlign==null)
						sAlign="";
					bWrap = (iWrap.intValue()==1);
					if(sLeft!=null && !sLeft.trim().equals(""))
						fLeft = Float.parseFloat(sLeft);
					if(sTop!=null && !sTop.trim().equals(""))
						fTop = Float.parseFloat(sTop);
					if(sWidth!=null && !sWidth.trim().equals(""))
						fWidth = Float.parseFloat(sWidth);
					if(sHeight!=null && !sHeight.trim().equals(""))
						fHeight = Float.parseFloat(sHeight);

					if(pdfBarCode!=null)
					{
						if( (err = retPdfDoc.docAddBarCode(pdfBarCode, sAlign, bWrap, fLeft, fTop, fWidth, fHeight))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding BarCode", err);
					}
				}
				//if object is a Table
				else if(objectTypeName.equalsIgnoreCase("PDF_Table"))
				{
					pdfTable =	this.getPdfTable(lasso, outValue);
					if(pdfTable!=null)
					{
						if( (err = retPdfDoc.docAddTable(pdfTable))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding Table", err);
					}
				}
				//if object is a List
				else if(objectTypeName.equalsIgnoreCase("PDF_List"))
				{
					pdfList =	this.getPdfList(lasso, outValue);
					if(pdfList!=null)
					{
						if( (err = retPdfDoc.docAddList(pdfList))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding List", err);
					}
				}
				/* if object is a PDF_Text */
				else if(objectTypeName.equalsIgnoreCase("PDF_Text"))
				{
					pdfText =	this.getPdfText(lasso, outValue);
					if(pdfText!=null)
					{
						if( (err = retPdfDoc.docAddText(pdfText))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_DOC:(Add) Error adding Text", err);
					}
				}

			}
		}
		return setError(lasso, "PDF_DOC:(Add)Unknown Error", err);
	}

	/**
	 * This method inserts a new page.
	 * @param lasso
	 * @param action
	 */
	public int myInsertPageFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		PDFRead pdfRead		= null;
		int err				= 0;

		LassoTypeRef outValue0 = new LassoTypeRef();
		err = lasso.getTagParam2(0, outValue0 );

		if(err == ERR_NOERR)
		{
			LassoValue typeName = new LassoValue();
			if( lasso.typeGetName(outValue0,typeName) == ERR_NOERR)
			{
				LassoTypeRef outValue1 = new LassoTypeRef();
				IntValue iValue = new IntValue();
				err = lasso.getTagParam2(1, outValue1);
				err = lasso.typeGetInteger( outValue1, iValue);

				int pageNo = iValue.intValue();

				if((typeName.name()).equalsIgnoreCase("PDF_Read"))//if object is a PDF_Read
				{
					pdfRead =	this.getPdfRead(lasso, outValue0);

					IntValue iNewPage = getBooleanValue(lasso,"-newpage");
					String	sTop = getParamValue(lasso,"-top");
					String	sLeft = getParamValue(lasso,"-left");
					String	sWidth = getParamValue(lasso,"-width");
					String	sHeight = getParamValue(lasso,"-height");
					IntValue iProportional	= getBooleanValue(lasso,"-proportional");
					String	sRotate = getParamValue(lasso,"-rotate");
					IntValue iFlipHorizontal = getBooleanValue(lasso,"-fliphorizontal");
					IntValue iFlipVertical = getBooleanValue(lasso,"-flipvertical");

					if(lasso.getTagSelf(me) == ERR_NOERR )
						retPdfDoc = this.getPdfDoc(lasso, me);

					int[] docSize = retPdfDoc.getDocSize();
					int docWidth = docSize[0];
					int docHeight = docSize[1] ;

					boolean bNewPage = (iNewPage.intValue()==1);

					float fTop = 0;
					if(sTop != null && !sTop.equals(""))
						fTop = Float.parseFloat(sTop);

					float fLeft = 0;
					if(sLeft != null && !sLeft.equals(""))
						fLeft = Float.parseFloat(sLeft);

					//ratio of the template's height to doc's height
					float fHeight = 0;
					if(sHeight != null && !sHeight.equals(""))
					{
						if(sHeight.indexOf("%")==-1)
						{
							fHeight = Float.parseFloat(sHeight);
							fHeight /= docHeight;
						}
						else
						{
							sHeight = sHeight.substring(0,sHeight.length()-1);
							fHeight = Float.parseFloat(sHeight);
							fHeight /= 100;
						}
					}

					//ratio of the template's width to doc's width
					float fWidth = 0;
					if(sWidth != null && !sWidth.equals(""))
					{
						if(sWidth.indexOf("%")==-1)
						{
							fWidth = Float.parseFloat(sWidth);
							fWidth /= docWidth;
						}
						else
						{
							sWidth = sWidth.substring(0,sWidth.length()-1);
							fWidth = Float.parseFloat(sWidth);
							fWidth /= 100;
						}
					}

					boolean bProportional = false;
					bProportional = (iProportional.intValue()==1);

					float fRotate = 0;
					if(sRotate!= null && !sRotate.equals(""))
						fRotate = Float.parseFloat(sRotate);

					boolean bFlipHorizontal = false;
					bFlipHorizontal = (iFlipHorizontal.intValue()==1);

					boolean bFlipVertical = false;
					bFlipVertical = (iFlipVertical.intValue()==1);

					if(retPdfDoc != null && pdfRead!=null)
					{
						if((err = retPdfDoc.docInsertPage(pdfRead, pageNo, bNewPage, fTop, fLeft,fWidth, fHeight,bProportional,fRotate,bFlipHorizontal, bFlipVertical)) == ERR_NOERR)
							return err;
						else
							return  setError(lasso, "PDF_Doc:(InsertPage) Error Inserting a New Page", err);
					}
					return  setError(lasso, "PDF_Doc:(InsertPage) Unknown Error", err);
				}
				else
					return  setError(lasso, "PDF_Doc:(InsertPage) Olny PDF_Read can be used to insert", 1);
			}
		}
		else
			return  setError(lasso, "PDF_Doc:(InsertPage) Unknown Error", 1);

		return 0;
	}

	/**
	 * This method returns the margins of the document.
	 * @param lasso
	 * @param action
	 */
	public int myGetMarginsFunc (LassoCall lasso, int action)
	{
		PDFDoc retPdfDoc	= null;
		LassoTypeRef me		= new LassoTypeRef();
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR)
			retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{
			float[] marginsArr = new float[4];
			marginsArr = retPdfDoc.getDocMargins();

			LassoTypeRef data 		= new LassoTypeRef();
			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef[] initial	= new LassoTypeRef[4];
			LassoTypeRef x1 		= new LassoTypeRef();
			LassoTypeRef x2 		= new LassoTypeRef();
			LassoTypeRef y1 		= new LassoTypeRef();
			LassoTypeRef y2 		= new LassoTypeRef();

			lasso.typeAllocDecimal(x1, (double)marginsArr[0]);
			lasso.typeAllocDecimal(x2, (double)marginsArr[1]);
			lasso.typeAllocDecimal(y1, (double)marginsArr[2]);
			lasso.typeAllocDecimal(y2, (double)marginsArr[3]);

			initial[0]=x1;
			initial[1]=x2;
			initial[2]=y1;
			initial[3]=y2;
			lasso.typeAllocArray(mArray, initial);
			lasso.returnTagValue(mArray);
			return setError(lasso, "PDF_Doc:(GetMargin) Error getting the margins", err);
		}

		return setError(lasso, "PDF_Doc:(GetMargin) Unknown Error", err);
	}


	/**
	 * This method returns the size of the document.The width and the height.
	 * @param lasso
	 * @param action
	 */

	public int myGetSizeFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFDoc retPdfDoc	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{
			int[] sizeArr = new int[2];
			sizeArr = retPdfDoc.getDocSize();


			LassoTypeRef data 		= new LassoTypeRef();
			LassoTypeRef szArray	= new LassoTypeRef();
			LassoTypeRef[] size 	= new LassoTypeRef[2];
			LassoTypeRef w	 		= new LassoTypeRef();
			LassoTypeRef y	 		= new LassoTypeRef();

			lasso.typeAllocInteger(w,  (long)sizeArr[0]);
			lasso.typeAllocInteger(y,  (long)sizeArr[1]);

			size[0]=w;
			size[1]=y;
			lasso.typeAllocArray(szArray, size);
			lasso.returnTagValue(szArray);
			return setError(lasso, "PDF_Doc:(GetSize) Error getting the size", err);
		}

		return setError(lasso, "PDF_Doc:(GetSize) Unknown Error", err);
	}

	/**
	 * This method returns the color of the document.
	 * @param lasso
	 * @param action
	 */

	public int myGetDocColorFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetColor) Error while calling GetColor", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			String sData 		= retPdfDoc.getDocColor();
			if(sData == null) sData = "Unable to get color";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the format of the document.
	 * @param lasso
	 * @param action
	 */

	public int myGetDocFormatFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetFormat) Error while calling GetFormat", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			String sData 		= retPdfDoc.getDocFormat();

			if(sData == null) sData = "Unable to get format";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method returns the header of the document.
	 * @param lasso
	 * @param action
	 */

	public int myGetDocHeaderFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		String strHead 	= null;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetHeader) Error while calling GetHeader", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			strHead 			= getString(lasso);
			String sData 		= retPdfDoc.getDocHeader(strHead);

			if(sData == null) sData = "Unable to get Header";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	public int myOnConvertFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		String strHead 	= null;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(OnConvert) Error while calling OnConvert", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{

			LassoTypeRef ref 	= new LassoTypeRef();
			String sData 		= null;

			sData = retPdfDoc.getPDFString();
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns all the specified headers of the document.
	 * @param lasso
	 * @param action
	 */
	public int myGetDocHeadersFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling GetHeaders", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef mp 	= new LassoTypeRef();
			LassoTypeRef elems[] 	= new LassoTypeRef[2];

			int size = 0;

			String temp = null;

			err = lasso.typeAllocMap(mp, null);

			LassoTypeRef mpkey = new LassoTypeRef();
			LassoTypeRef mpval = new LassoTypeRef();

			temp = retPdfDoc.getDocTitle();

			if(temp != null)
			{
				lasso.typeAllocString( mpkey, "Title" );
				lasso.typeAllocString( mpval, temp );
				lasso.mapSetElement( mp, mpkey, mpval );
			}

			temp = retPdfDoc.getDocSubject();

			if(temp != null)
			{
				lasso.typeAllocString( mpkey, "Subject" );
				lasso.typeAllocString( mpval, temp );
				lasso.mapSetElement( mp, mpkey, mpval );
			}

			temp = retPdfDoc.getDocAuthor();

			if(temp != null)
			{
				lasso.typeAllocString( mpkey, "Author" );
				lasso.typeAllocString( mpval, temp );
				lasso.mapSetElement( mp, mpkey, mpval );
			}

			temp = retPdfDoc.getDocCreator();

			if(temp != null)
			{
				lasso.typeAllocString( mpkey, "Creator" );
				lasso.typeAllocString( mpval, temp );
				lasso.mapSetElement( mp, mpkey, mpval );
			}

			temp = retPdfDoc.getDocKeywords();

			if(temp != null)
			{
				lasso.typeAllocString( mpkey, "Keywords" );
				lasso.typeAllocString( mpval, temp );
				lasso.mapSetElement( mp, mpkey, mpval );
			}


			if(err == ERR_NOERR)
				lasso.returnTagValue(mp); // returns the map
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the page number of the document.
	 * @param lasso
	 * @param action
	 */

	public int myGetPageNumFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFDoc retPdfDoc	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{
			int iPageNum 	= 0;
			LassoTypeRef pn = new LassoTypeRef();
			iPageNum 		= retPdfDoc.getDocPageNumber();
			lasso.typeAllocInteger(pn,  (long)iPageNum);
			lasso.returnTagValue(pn);
			return setError(lasso, "PDF_Doc:(GetPageNumber) Error getting the Page Number", err);
		}
		return setError(lasso, "PDF_Doc:(GetPageNumber) Unknown Error ", err);
	}

	/**
	 * This method sets the page number of the document.
	 * @param lasso
	 * @param action
	 */

	public int mySetPageNumFunc (LassoCall lasso, int action)
	{
		String strPageNum	= null;
		int iPageNum 		= 0;
		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		int err				= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			strPageNum = getString(lasso);
			if(strPageNum != null)
			{
				iPageNum = Integer.parseInt(strPageNum);
				if((err = retPdfDoc.setDocPageNumber(iPageNum)) == ERR_NOERR)
					return err;
				else
					return  setError(lasso, "PDF_Doc:(SetPageNumber) Error setting the Page Number", err);
			}
			else  return LassoErrors.InvalidParameter;

		}
		return setError(lasso, "PDF_Doc:(SetPageNumber) Unknown Error", err);
	}

	/**
	 * This method signals that a new page has to be started.
	 * @param lasso
	 * @param action
	 */
	public int myAddPageFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		int err				= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			if((err = retPdfDoc.docAddPage()) == ERR_NOERR)
				return err;
			else
				return  setError(lasso, "PDF_Doc:(AddPage) Error Adding a New Page", err);
		}
		return  setError(lasso, "PDF_Doc:(AddPage) Unknown Error", err);
	}

	/**
	 * This method adds the Graphics object to the document.
	 * @param lasso
	 * @param action
	 */

	public int myAddGraphicsFunc (LassoCall lasso, int action)
	{
		int err = 0;
		try
		{
			LassoTypeRef me			= new LassoTypeRef();
			PDFDoc retPdfDoc 		= null;
			PDFGraphics pdfgraphics = null;


			if(lasso.getTagSelf(me) == ERR_NOERR )
				retPdfDoc = this.getPdfDoc(lasso, me);

			LassoTypeRef outValue 	= new LassoTypeRef();
			err 					= lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Graphic"))
						pdfgraphics =	this.getPdfGraphics(lasso, outValue);
				}

			}
			if(retPdfDoc != null && pdfgraphics != null)
			{
				if((err = retPdfDoc.addGraphics(pdfgraphics.myGraphics)) == ERR_NOERR)
					return err;
			}
			return  setError(lasso, "PDF_Doc:(AddGraphics)Adding the Graphics Object", err);
		}
		catch(Exception e)
		{
			return  setError(lasso, "PDF_Doc:(AddGraphics)Unknown Error", err);
		}
	}


	/**
	 * This method  adds the table object to  the document.
	 * @param lasso
	 * @param action
	 */

	public int myAddTableFunc (LassoCall lasso, int action)
	{
		int err = 0;
		try
		{
			LassoTypeRef me			= new LassoTypeRef();
			PDFDoc retPdfDoc 		= null;
			PDFTable pdfTable 		= null;


			if(lasso.getTagSelf(me) == ERR_NOERR )
				retPdfDoc = this.getPdfDoc(lasso, me);

			LassoTypeRef outValue = new LassoTypeRef();
			err = lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Table"))
						pdfTable =	this.getPdfTable(lasso, outValue);
				}

			}
			if(retPdfDoc != null && pdfTable != null)
			{
				if((err = retPdfDoc.addTable(pdfTable.myTable)) == ERR_NOERR)
					return err;
			}
			return  setError(lasso, "PDF_Doc:(AddTable)Error Adding Table Object to Document", err);
		}
		catch(Exception e)
		{
			return  setError(lasso, "PDF_Doc:(AddTable)Unknown Error", err);
		}
	}


	/**
	 * This method  adds the barcode object to  the document.
	 * @param lasso
	 * @param action
	 */

	public int myAddBarcodeFunc (LassoCall lasso, int action)
	{
		int err = 0;
		try
		{
			LassoTypeRef me				= new LassoTypeRef();
			PDFDoc retPdfDoc 			= null;
			PDFBarCode pdfBarcode 		= null;


			if(lasso.getTagSelf(me) == ERR_NOERR )
				retPdfDoc = this.getPdfDoc(lasso, me);

			float llx, lly;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddBarcode: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}


			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddBarcode: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = f.floatValue();

			LassoTypeRef outValue = new LassoTypeRef();
			err = lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Barcode"))
						pdfBarcode =	this.getPdfBarCode(lasso, outValue);
				}

			}
			if(retPdfDoc != null && pdfBarcode != null)
			{
				if((err = retPdfDoc.addBarcode(pdfBarcode, llx, lly)) == ERR_NOERR)
					return err;
			}
			return  setError(lasso, "PDF_Doc:(AddBarcode)Error adding the Barcode object to Document", err);
		}
		catch(Exception e)
		{
			return  setError(lasso, "PDF_Doc:(AddBarcode)Unknown Error", err);
		}
	}



	/**
	 * This method sets the font of the document.
	 * @param lasso
	 * @param action
	 */

	public int mySetFontFunc (LassoCall lasso, int action)
	{
		int err = 0;
		try
		{
			LassoTypeRef me			= new LassoTypeRef();
			PDFDoc retPdfDoc 		= null;
			PDFFont pF 				= null;


			if(lasso.getTagSelf(me) == ERR_NOERR )
				retPdfDoc = this.getPdfDoc(lasso, me);

			LassoTypeRef outValue = new LassoTypeRef();
			err = lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Font"))
						pF =	this.getPdfFont(lasso, outValue);
				}

			}

			if(retPdfDoc != null && pF != null)
			{
				if((err = retPdfDoc.setFont(pF.getFont())) == ERR_NOERR)
					return err;
			}
			return  setError(lasso, "PDF_Doc:(setFont)Error setting the Font", err);
		}
		catch(Exception e)
		{
			return  setError(lasso, "PDF_Doc:(setFont)Unknown Error", err);
		}
	}


	/**
	 * This method sets the width of the line.
	 * @param lasso
	 * @param action
	 */

	public int mySetLineWidthFunc (LassoCall lasso, int action)
	{
		String strLineWidth = null;
		float fLineWidth 	= 0;
		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		int err				= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			strLineWidth = getString(lasso);
			if(strLineWidth != null)
			{
				fLineWidth = Float.parseFloat(strLineWidth);
				if(fLineWidth >= 0)
				{
					if((err = retPdfDoc.docSetLineWidth(fLineWidth)) == ERR_NOERR)
						return err;
					else
						return setError(lasso, "PDF_Doc:(SetLineWidth)Unable to set LineWidth", err);
				}
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return setError(lasso, "PDF_Doc:(SetLineWidth)Unknown Error", err);
	}

	/**
	 * This method draws a line on the document.
	 * @param lasso
	 * @param action
	 */

	public int myLineFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		String strX1 = null;
		String strY1 = null;
		String strX2 = null;
		String strY2 = null;

	/*	int iX1 = 0;
		int iY1 = 0;
		int iX2 = 0;
		int iY2 = 0;*/
		float iX1 = 0;
		float iY1 = 0;
		float iX2 = 0;
		float iY2 = 0;
		int err	= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		boolean bLineTo = true;
		int tagCount = 0;
		IntValue k = new IntValue();
		lasso.getTagParamCount(k);
		tagCount = k.intValue();

		if(tagCount == 4) bLineTo = false;
		else if(tagCount == 2) bLineTo = true;
		else
		{
			return setError(lasso, "PDF_Doc:(Line)Parameters not sufficient", err);
		}


		if(retPdfDoc != null)
		{

			LassoValue X1 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(0, X1 ))
			{
				strX1 = X1.name();
			}

			LassoValue Y1 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(1, Y1 ))
			{
				strY1 = Y1.name();
			}


			if(bLineTo == false)
			{
				LassoValue X2 = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(2, X2 ))
				{
					strX2 = X2.name();
				}

				LassoValue Y2 = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(3, Y2 ))
				{
					strY2 = Y2.name();
				}

				if(strX2 != null)
				//	iX2 = Integer.parseInt(strX2);
					iX2 = Float.parseFloat(strX2);
				if(strY2 != null)
				//	iY2 = Integer.parseInt(strY2);
					iY2 = Float.parseFloat(strY2);
			}

			if(strX1 != null)
			//	iX1 = Integer.parseInt(strX1);
				iX1 = Float.parseFloat(strX1);

			if(strY1 != null)
			//	iY1 = Integer.parseInt(strY1);
				iY1 = Float.parseFloat(strY1);

			if(bLineTo == false)
			{
				if((err = retPdfDoc.docLine(iX1,iY1,iX2,iY2)) == ERR_NOERR)
				 	return err;
				else
					return setError(lasso, "PDF_Doc:(Line)Error drawing line", err);
			}
			else
			{

				if((err = retPdfDoc.docLineTo(iX1,iY1)) == ERR_NOERR)
					return err;
				else
					return 	setError(lasso, "PDF_Doc:(Line)Error drawing line", err);

			}

		}
		return setError(lasso, "PDF_Doc:(Line)Unknown Error", err);
	}


	/**
	 * This method sets the color for the subsequent drawing operations on the document.
	 * @param lasso
	 * @param action
	 */

	public int mySetColorFunc (LassoCall lasso, int action)
	{

		String strType 	= null;	//one of 'stroke' ,'fill' or both.
		String strColor = null; //one of 'grey','rgb','cmyk','spot' or 'pattern'.
		String strC1 	= null;	//decimal number for color
		String strC2	= null;	//decimal number for color
		String strC3 	= null;	//decimal number for color
		String strC4 	= null;	//decimal number for color

		float fC1 		= 0;
		float fC2 		= 0;
		float fC3 		= 0;
		float fC4 		= 0;

		boolean bStroke = false;
		boolean bFill	= false;
		boolean bBoth 	= false;

		int err 		= 0;


		LassoTypeRef me = new LassoTypeRef();
		PDFDoc retPdfDoc = null;
		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);


		LassoValue type = new LassoValue();
		if(ERR_NOERR == lasso.getTagParam(0, type ))
		{
			strType = type.name();
		}

		LassoValue clr = new LassoValue();
		if(ERR_NOERR == lasso.getTagParam(1, clr ))
		{
			strColor = clr.name();
		}


		if(strType != null)
		{
			if(strType.equalsIgnoreCase("stroke"))
				bStroke = true;
			else if(strType.equalsIgnoreCase("fill"))
				bFill = true;
			else if(strType.equalsIgnoreCase("both"))
			{
				bStroke = true;
				bFill = true;
			}
		}


		int tagCount	= 0;
		IntValue k 		= new IntValue();

		lasso.getTagParamCount(k);
		tagCount = k.intValue();
		if(tagCount == 3)
		{
			LassoValue C1 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(2, C1 ))
			{
				strC1 = C1.name();
				if(strC1 != null)
					fC1 = Float.parseFloat(strC1);
			}

		}
		else if(tagCount == 5)
		{
			LassoValue C1 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(2, C1 ))
			{
				strC1 = C1.name();
				if(strC1 != null)
					fC1 = Float.parseFloat(strC1);
			}

			LassoValue C2 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(3, C2 ))
			{
				strC2 = C2.name();
				if(strC2 != null)
					fC2 = Float.parseFloat(strC2);
			}

			LassoValue C3 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(4, C3 ))
			{
				strC3 = C3.name();
				if(strC3 != null)
					fC3 = Float.parseFloat(strC3);
			}


		}
		else if(tagCount == 6)
		{

			LassoValue C1 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(2, C1 ))
			{
				strC1 = C1.name();
				if(strC1 != null)
					fC1 = Float.parseFloat(strC1);
			}

			LassoValue C2 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(3, C2 ))
			{
				strC2 = C2.name();
				if(strC2 != null)
					fC2 = Float.parseFloat(strC2);
			}

			LassoValue C3 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(4, C3 ))
			{
				strC3 = C3.name();
				if(strC3 != null)
					fC3 = Float.parseFloat(strC3);
			}

			LassoValue C4 = new LassoValue();

			if(ERR_NOERR == lasso.getTagParam(4, C4 ))
			{
				strC4 = C4.name();
				if(strC4 != null)
					fC4 = Float.parseFloat(strC4);
			}
		}

		if(retPdfDoc != null)
		{

			if(bStroke == true && bFill == false && strColor.equalsIgnoreCase("gray"))
			 	return  retPdfDoc.docSetGrayStroke(fC1);

			else if (bStroke == true && bFill == false && strColor.equalsIgnoreCase("rgb"))
				return  retPdfDoc.docSetRGBColorStrokeF(fC1,fC2,fC3);

			else if (bStroke == true && bFill == false && strColor.equalsIgnoreCase("cmyk"))
				return  retPdfDoc.docSetCMYKColorStrokeF(fC1,fC2,fC3,fC4);

			else if (bFill == true && bStroke == false && strColor.equalsIgnoreCase("gray"))
				return  retPdfDoc.docSetGrayFill(fC1);

			else if (bFill == true && bStroke == false && strColor.equalsIgnoreCase("rgb"))
				return  retPdfDoc.docSetRGBColorFillF(fC1,fC2,fC3);

			else if (bFill == true && bStroke == false && strColor.equalsIgnoreCase("cmyk"))
				return  retPdfDoc.docSetCMYKColorFillF(fC1,fC2,fC3,fC4);

			else if (bFill == true && bStroke == true && strColor.equalsIgnoreCase("gray"))
			{
				retPdfDoc.docSetGrayFill(fC1);
				retPdfDoc.docSetGrayStroke(fC1);
				return 0;
			}
			else if (bFill == true && bStroke == true && strColor.equalsIgnoreCase("rgb"))
			{
				retPdfDoc.docSetRGBColorFillF(fC1,fC2,fC3);
				retPdfDoc.docSetRGBColorStrokeF(fC1,fC2,fC3);
				return 0;
			}

			else if (bFill == true && bStroke == true && strColor.equalsIgnoreCase("cmyk"))
				retPdfDoc.docSetCMYKColorFillF(fC1,fC2,fC3,fC4);
				retPdfDoc.docSetCMYKColorStrokeF(fC1,fC2,fC3,fC4);
				return 0;
		}
		return setError(lasso, "PDF_Doc:(SetColor)Error setting the color", err);
	}

	/**
	 * This method draws a rectangle on the document.
	 * @param lasso
	 * @param action
	 */

	public int myRectFunc (LassoCall lasso, int action)
	{
		int	err		= 0;
		try
		{
			String strX 	= null;	//the X-coordinate value.
			String strY		= null;	//the Y-coordinate value.
			String strWidth = null;	//the width value.
			String strHeight= null;	//the height value.



			float iX 		= 0;
			float iY 		= 0;
			float iWidth 	= 0;
			float iHeight = 0;

			LassoTypeRef me = new LassoTypeRef();
			PDFDoc retPdfDoc = null;
			if(lasso.getTagSelf(me) == ERR_NOERR )
				retPdfDoc = this.getPdfDoc(lasso, me);
			if(retPdfDoc != null)
			{

				LassoValue X = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(0, X ))
				{
					strX = X.name();
					if(strX != null) iX = Float.parseFloat(strX);
				}

				LassoValue Y = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(1, Y ))
				{
					strY = Y.name();
					if(strY != null) iY = Float.parseFloat(strY);
				}

				LassoValue width = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(2, width ))
				{
					strWidth = width.name();
					if(strWidth != null) iWidth = Float.parseFloat(strWidth);
				}

				LassoValue height = new LassoValue();
				if(ERR_NOERR == lasso.getTagParam(3, height ))
				{
					strHeight = height.name();
					if(strHeight != null) iHeight = Float.parseFloat(strHeight);
				}

				if((err = retPdfDoc.docRect(iX,iY,iWidth,iHeight)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Doc:(Rectangle)Error drawing Rectangle", err);
			}
			return setError(lasso, "PDF_Doc:(Rectangle)Unknown Error ", err);
		}
		catch(Exception e)	{
			return setError(lasso, "PDF_Doc:(Rectangle)Unknown Error ", err);
		}

	}

	/**
	 * This method draws an arc on  the document.
	 * @param lasso
	 * @param action
	 */

	public int myArcFunc (LassoCall lasso, int action)
	{
		String strX 	= null;	//the X-coordinate value.
		String strY 	= null;	//the Y-coordinate value.
		String strR 	= null;	//the radius value.
		String strAlpha = null;	//the starting angle value.
		String strBeta 	= null;	//the ending angle value.

		float iX 			= 0;
		float iY 			= 0;
		float iR 			= 0;
		float iAlpha 		= 0;
		float iBeta 		= 0;
		int err			= 0;

		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{

			LassoValue X = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(0, X ))
			{
				strX = X.name();
				if(strX != null) iX = Float.parseFloat(strX);
			}

			LassoValue Y = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(1, Y ))
			{
				strY = Y.name();
				if(strY != null) iY = Float.parseFloat(strY);
			}

			LassoValue R = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(2, R ))
			{
				strR = R.name();
				if(strR != null) iR = Float.parseFloat(strR);
			}

			LassoValue alpha = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(3, alpha ))
			{
				strAlpha = alpha.name();
				if(strAlpha != null) iAlpha = Float.parseFloat(strAlpha);
			}

			LassoValue beta = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(4, beta ))
			{
				strBeta = beta.name();
				if(strBeta != null) iBeta = Float.parseFloat(strBeta);
			}

			if((err = retPdfDoc.docArc(iX, iY, iR, iAlpha, iBeta)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Doc:(Arc)Error drawing Arc", err);
		}
		return setError(lasso, "PDF_Doc:(Arc)Unknown Error", err);
	}

	/**
	 * This method moves the pen position to the specified co-ordinates on  the document.
	 * @param lasso
	 * @param action
	 */

	public int myMoveToFunc (LassoCall lasso, int action)
	{
		String strX = null; //the X-coordinate value
		String strY = null; //the Y-coordinate value

		float iX	 	= 0;
		float iY 		= 0;
		int err		= 0;

		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc	= null;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{

			LassoValue X = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(0, X ))
			{
				strX = X.name();
				if(strX != null) iX = Float.parseFloat(strX);
			}

			LassoValue Y = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(1, Y ))
			{
				strY = Y.name();
				if(strY != null) iY = Float.parseFloat(strY);
			}

			if((err = retPdfDoc.docMoveTo(iX,iY)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Doc:(MoveTo)Error in MoveTo", err);
		}
		return setError(lasso, "PDF_Doc:(MoveTo)Error in MoveTo", err);
	}

	/**
	 * This method draws an bezier arc on  the document.
	 * @param lasso
	 * @param action
	 */

	public int myCurveToFunc (LassoCall lasso, int action)
	{
		String strX1 	= null;	//the X1-coordinate value.
		String strY1 	= null;	//the Y1-coordinate value.
		String strX2 	= null;	//the X2-coordinate value.
		String strY2 	= null;	//the Y2-coordinate value.
		String strX3 	= null;	//the X3-coordinate value.
		String strY3 	= null;	//the Y3-coordinate value.

		float iX1			= 0;
		float iY1 		= 0;
		float iX2 		= 0;
		float iY2 		= 0;
		float iX3 		= 0;
		float iY3 		= 0;
		int err			= 0;

		LassoTypeRef me = new LassoTypeRef();
		PDFDoc retPdfDoc = null;
		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);


		if(retPdfDoc != null)
		{
			LassoValue X1 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(0, X1 ))
			{
				strX1 = X1.name();
				if(strX1 != null)	iX1 = Float.parseFloat(strX1);
			}

			LassoValue Y1 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(1, Y1 ))
			{
				strY1 = Y1.name();
				if(strY1 != null)	iY1 = Float.parseFloat(strY1);

			}

			LassoValue X2 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(2, X2 ))
			{
				strX2 = X2.name();
				if(strX2 != null)	iX2 = Float.parseFloat(strX2);

			}

			LassoValue Y2 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(3, Y2 ))
			{
				strY2 = Y2.name();
				if(strY2 != null)	iY2 = Float.parseFloat(strY2);

			}

			LassoValue X3 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(4, X3 ))
			{
				strX3 = X3.name();
				if(strX3 != null)	iX3 = Float.parseFloat(strX2);
			}

			LassoValue Y3 = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(5, Y3 ))
			{
				strY3 = Y3.name();
				if(strY3 != null)	iY3 = Float.parseFloat(strY3);
			}


			if((err = retPdfDoc.docCurveTo(iX1,iY1,iX2,iY2,iX3,iY3)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Doc:(CurveTo)Error in  drawing CurveTo", err);

		}
		return setError(lasso, "PDF_Doc:(CurveTo)Unknown Error ", err);
	}

	/**
	 * This method draws an circle arc on  the document.
	 * @param lasso
	 * @param action
	 */

	public int myCircleFunc (LassoCall lasso, int action)
	{
		String strX 	= null;	//the X-coordinate of the center
		String strY 	= null;	//the Y-coordinate of the center
		String strRadius= null;	//the radius value.

		float iX 			= 0;
		float iY 			= 0;
		float iRadius		= 0;
		int err			= 0;

		LassoTypeRef me = new LassoTypeRef();
		PDFDoc retPdfDoc = null;
		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoValue X = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(0, X ))
			{
				strX = X.name();
				if(strX != null)	iX = Float.parseFloat(strX);
			}

			LassoValue Y = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(1, Y ))
			{
				strY = Y.name();
				if(strY != null)	iY = Float.parseFloat(strY);
			}

			LassoValue radius = new LassoValue();
			if(ERR_NOERR == lasso.getTagParam(2, radius ))
			{
				strRadius = radius.name();
				if(strRadius != null)	iRadius = Float.parseFloat(strRadius);
			}


			if((err = retPdfDoc.docCircle(iX,iY,iRadius)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Doc:(Circle)Error Drawing Circle ", err);
		}
		return setError(lasso, "PDF_Doc:(Circle)Unknown Error ", err);
	}

	/**
	 * This method closes the path after a drawing operation on the document.
	 * @param lasso
	 * @param action
	 */

	public int myClosePathFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		int	err				= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			if(ERR_NOERR == retPdfDoc.docClosePathFillStroke())
				return err;
			else
				return setError(lasso, "PDF_Doc:(ClosePath)Error in ClosePath ", err);
		}
		return setError(lasso, "PDF_Doc:(ClosePath)Unknown Error ", err);
	}

	/**
	 * This method gets the font  value of the PDFFont object.
	 * @param lasso
	 */
	public Font getFontParam(LassoCall lasso)
	{
		int err = 0;
		try
		{
			PDFFont pF 				= null;
			LassoTypeRef outValue 	= new LassoTypeRef();
			err 					= lasso.findTagParam2("-font", outValue );


			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Font"))
					{
						pF =	this.getPdfFont(lasso, outValue);
						if(pF!=null)
							return 	pF.getFont();
					}
				}
			}

			return  null;
		}
		catch(Exception e)
		{
			return  null;
		}
	}



	/**
	 * This method gets the font  value of the PDFFont object.
	 * @param lasso
	 */
	public Font getFontParam2(LassoCall lasso)
	{
		int err = 0;
		try
		{
			PDFFont pF 				= null;
			LassoTypeRef outValue 	= new LassoTypeRef();
			err 					= lasso.getTagParam2(0, outValue );


			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Font"))
					{
						pF =	this.getPdfFont(lasso, outValue);
						if(pF!=null)
							return 	pF.getFont();
					}
				}
			}

			return  null;
		}
		catch(Exception e)
		{
			return  null;
		}
	}

	/**
	 * This method adds a chunk on to the document.
	 * @param lasso
	 * @param action
	 */

	public int myAddTextFunc (LassoCall lasso, int action)
	{
		int err 				= 0;
		LassoTypeRef me			= new LassoTypeRef();
		PDFDoc retPdfDoc		= null;
		PDFText pdfText			= null;

		if (lasso.getTagSelf(me) == ERR_NOERR )
		retPdfDoc = this.getPdfDoc(lasso, me);

		LassoValue tagName	= new LassoValue();
		lasso.getTagName(tagName);

		boolean bAddPhrase		= false;
		boolean bAddText 		= false;
		boolean bAddParagraph 	= false;
		boolean bAddSection 	= false;
		boolean bAddChapter 	= false;
		boolean bAddList		= false;
		boolean bDrawText 		= false;


		if(tagName.data().equalsIgnoreCase("addtext"))
			bAddText = true;
		else if(tagName.data().equalsIgnoreCase("addphrase"))
			bAddPhrase = true;
		else if(tagName.data().equalsIgnoreCase("addparagraph"))
			bAddParagraph = true;
		else if(tagName.data().equalsIgnoreCase("addsection"))
			bAddSection = true;
		else if(tagName.data().equalsIgnoreCase("addchapter"))
			bAddChapter = true;
		else if(tagName.data().equalsIgnoreCase("addlist"))
			bAddList = true;
		else if(tagName.data().equalsIgnoreCase("drawtext"))
			bDrawText = true;

	/** PDF_Doc Tag
	 *	Handling the Optional parameters of the PDF_Doc member tags.
	 *  Like font,anchor,name,goto,file etc...
	 */

			String	strAnchor	= getParamValue(lasso,"-anchor");

			String	strName 	= getParamValue(lasso,"-name");

			String	strGoTo		= getParamValue(lasso,"-goto");

			String	strFile		= getParamValue(lasso,"-file");

		if(bAddText)
		{
			String strText 	= null;
			strText 		= getString(lasso);	//the chunk text to be added to the dcument.

			if(retPdfDoc != null && strText != null)
			{
				if((err = retPdfDoc.docAddText(strText, getFontParam(lasso), strAnchor, strName, strGoTo, strFile)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_DOC:(AddText) Error adding a Text", err);
			}
			return setError(lasso, "PDF_DOC:(AddText)Unknown Error", err);
		}

		if(bAddPhrase)
		{
			String strPhrase 	= getString(lasso);	//the phrase text.
			LassoValue ldresult = new LassoValue();
			String strLeading 	= getParamValue(lasso,"-leading");
			float fLeading		= 0;
			if(strLeading != null)
				fLeading 		= Float.parseFloat(strLeading);

			if(retPdfDoc != null && strPhrase !=  null)
			{
				if((err = retPdfDoc.docAddPhrase( strPhrase, fLeading, getFontParam(lasso), strAnchor, strName, strGoTo, strFile)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_DOC:(AddPhrase) Error adding a Phrase ", err);
			}
			return setError(lasso, "PDF_DOC(AddPhrase):Unknown Error", err);

		}
		if(bAddParagraph)
		{
			String strParagraph 	= null;
			strParagraph 			= getString(lasso);	//the paragraph text.

			String strAlignment = getParamValue(lasso,"-alignment"); //code added 12-09-2002

			String strMarginLeft = getParamValue(lasso,"-marginleft");
			String strMarginRight = getParamValue(lasso,"-marginright");
			float fMarginLeft		= 0;
			float fMarginRight		= 0;

			if(strMarginLeft != null)
				fMarginLeft 		= Float.parseFloat(strMarginLeft);
			if(strMarginRight != null)
				fMarginRight 		= Float.parseFloat(strMarginRight);

			String strLeading 	= getParamValue(lasso,"-leading");  //code added 12-09-2002
			float fLeading		= 0;
			if(strLeading != null)
				fLeading 		= Float.parseFloat(strLeading);


			if(retPdfDoc != null && strParagraph != null)
			{
				if((err = retPdfDoc.docAddParagraph( strParagraph,fLeading,fMarginRight,fMarginLeft,strAlignment,getFontParam(lasso), strAnchor, strName, strGoTo, strFile)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_DOC:(AddParagraph) Error adding a Paragraph ", err);

			}
			return setError(lasso, "PDF_DOC(AddParagraph):Unknown Error", err);
		}

		if(bAddSection)
		{
			//PDF_TEXT should be added

			LassoTypeRef outValue = new LassoTypeRef();
			err = lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					String objectTypeName = typeName.name();

					String strIndent 	= getParamValue(lasso,"-sectionindent");
					float fIndent		= 0;
					if(strIndent != null)
						fIndent		= Float.parseFloat(strIndent);

					boolean bHideNo		= false;
					IntValue bValue	= getBooleanValue(lasso,"-hidenumber");
					bHideNo = (bValue.intValue()==1);

					/* if object is a PDF_Text */
					if(objectTypeName.equalsIgnoreCase("PDF_Text"))
					{
						pdfText =	this.getPdfText(lasso, outValue);
						if(pdfText!=null)
						{
							if( (err = retPdfDoc.docAddSection(pdfText, fIndent, bHideNo))==ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_DOC:(Add) Error adding Section", err);
						}
					}
				}
			}
			return setError(lasso, "PDF_DOC(AddSection):Unknown Eroror", err);

		}

		if(bAddChapter)
		{
			//PDF_TEXT should be added
			LassoTypeRef outValue = new LassoTypeRef();
			err = lasso.getTagParam2(0, outValue );

			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					String objectTypeName = typeName.name();

					String strChapNumber= getParamValue(lasso,"-number");
					int iChapNumber 	= 0;
					if(strChapNumber != null)
						iChapNumber 	= Integer.parseInt(strChapNumber);

					boolean bHideNo		= false;
					IntValue bValue	= getBooleanValue(lasso,"-hidenumber");
					bHideNo = (bValue.intValue()==1);

					/* if object is a PDF_Text */
					if(objectTypeName.equalsIgnoreCase("PDF_Text"))
					{
						pdfText =	this.getPdfText(lasso, outValue);
						if(pdfText!=null)
						{
							if( (err = retPdfDoc.docAddChapter(pdfText, iChapNumber, bHideNo))==ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_DOC:(Add) Error adding Chapter", err);
						}
					}
				}
			}
			return setError(lasso, "PDF_DOC(AddSection):Unknown Eroror", err);
		}


		//code added 12-09-2002
		if(bDrawText)
		{
			float fLeft 		= 0;
			float fTop 			= 0;
			String strLeft		= null;	//the left position or the point,
			String strTop		= null;	//the top position or the point.

			String strText 		= null;
			strText 			= getString(lasso);	//the text.

			String strAlignment = null;
			strAlignment = getParamValue(lasso,"-alignment"); //code added 12-09-2002


			strLeft		= getParamValue(lasso,"-left");
			strTop 		= getParamValue(lasso,"-top");


			if(strLeft != null)
				fLeft = Float.parseFloat(strLeft);
			if(strTop !=  null)
				fTop= Float.parseFloat(strTop);




			String strWidth = getParamValue(lasso,"-width");
			String strHeight = getParamValue(lasso,"-height");
			float fWidth		= 0;
			float fHeight		= 0;


			if(strWidth != null)
				fWidth 		= Float.parseFloat(strWidth);
			if(strHeight != null)
				fHeight 		= Float.parseFloat(strHeight);

			String strLeading 	= getParamValue(lasso,"-leading");  //code added 12-09-2002
			float fLeading		= 0;
			if(strLeading != null)
				fLeading 		= Float.parseFloat(strLeading);

			if(retPdfDoc != null && strText != null)
			{
				if((err = retPdfDoc.docDrawText( strText, getFontParam(lasso), fTop,fLeft,fWidth,fHeight,fLeading,strAlignment)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_DOC:(DrawText) Error Drawing a Text ", err);

			}
			return setError(lasso, "PDF_DOC(DrawText):Unknown Error", err);
		}




		if(bAddList)
		{
			LassoTypeRef res 	= new LassoTypeRef();
			LassoValue typeName = new LassoValue();
			int arrSize 		= 0;
			String[] listArr 	= null;
			String strFormat 	= getParamValue(lasso,"-format");

			if( lasso.getTagParam2(0, res) == ERR_NOERR &&
				lasso.typeGetName(res,typeName) == ERR_NOERR)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();

					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						arrSize = count.intValue();
						listArr	= new String[arrSize];

						for ( int k = 0; k < arrSize; k++ )
						{
							LassoTypeRef element= new LassoTypeRef();
							LassoValue outStringValue = new LassoValue();

							lasso.arrayGetElement(res, k, element);
							lasso.typeGetString(element, outStringValue);
							listArr[k] = outStringValue.name();
						}
					}
				}

			}
		//	This part of the code is commented because passing strings in an Array is not working.

			if(retPdfDoc != null )
			{
				if((err = retPdfDoc.docAddList(listArr, strFormat, getFontParam(lasso), strAnchor, strName, strGoTo, strFile)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_DOC:(AddList) Error adding a List ", err);
			}

		}

		return setError(lasso, "PDF_DOC:(AddList) Error adding a List", err);
	}


	/**
	 * This method adds an Image on  the document.
	 * @param lasso
	 * @param action
	 */

	public int myAddImageFunc (LassoCall lasso, int action)
	{
		String strImage 	= null; //the path of th image file
		String strLeft		= null;	//the left position or the point,
		String strTop		= null;	//the top position or the point.

		float fLeft 		= 0;
		float fTop 			= 0;

		LassoTypeRef me 	= new LassoTypeRef();
		PDFDoc retPdfDoc 	= null;
		int err				= 0;


		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			strImage	= getParamValue(lasso,"-image");
			if(strImage != null)
				strImage = internalToFullPath(lasso, strImage);
			else
				return setError(lasso, "PDF_Doc:(AddImage)Unable to add Image to the document", err);

			strLeft		= getParamValue(lasso,"-left");
			strTop 		= getParamValue(lasso,"-top");


			if(strLeft != null)
				fLeft = Float.parseFloat(strLeft);
			if(strTop !=  null)
				fTop= Float.parseFloat(strTop);

			if((err = retPdfDoc.docAddImage(strImage, fLeft, fTop)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Doc:(AddImage)Unable to add Image to the document", err);
		}
		return setError(lasso, "PDF_Doc:(AddImage)Unknown Error", err);
	}

	/**
	 * This method returns the element type.
	 * @param lasso
	 * @param action
	 */
	public int myGetElementTypeFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetElementType) Error while calling GetElementType", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			int sData 			= retPdfDoc.getElementType();
			err  				= lasso.typeAllocInteger(ref, sData);

			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns an element.
	 * @param lasso
	 * @param action
	 */
	public int myGetElement (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetElement) Error while calling GetElement", err);


		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			int sData = retPdfDoc.getElement();
			err  = lasso.typeAllocInteger(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns  element count.
	 * @param lasso
	 * @param action
	 */

	public int myGetElementCount (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(GetElementCount) Error while calling GetElementCount", err);

		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			String sData 		= retPdfDoc.getElementCount();
			if(sData == null)
				sData 			= "Unable to get element count";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/** added Nov 2003
	* gets the data as bytes from the current pdf;works only after doc's
	  been closed
	* @param lasso
	* @param action
	*/
	public int myGetBytes(LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc:(Data) Error while calling Data", err);

		PDFDoc retPdfDoc = this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			byte[] data 		= retPdfDoc.getBytes();

			if(data == null)
				data = "Unable to get data from pdf_doc".getBytes();
			return lasso.outputTagData(data);
		}

		return 0;
	}

	/**
	 * This method closes the document and commits it on the disk.
	 * @param lasso
	 * @param action
	 */

	public int myCloseDocFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me 	= new LassoTypeRef();
		lasso.getTagSelf(me);

		PDFDoc retPdfDoc 	= this.getPdfDoc(lasso, me);
		if(retPdfDoc != null)
			retPdfDoc.closeDoc();
		return 0;

	}

	//Added in july 2003
	/**
	 * This method returns the PDFList object.
	 * @param lasso
	 * @param me
	 */

	public PDFList getPdfList(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFList)lasso.typeGetCustomJavaObject(me);
		return null;
	}

	//Added in july 2003
	/**
	 * This method returns the PDFImage object.
	 * @param lasso
	 * @param me
	 */

	public PDFImage getPdfImage(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFImage)lasso.typeGetCustomJavaObject(me);
		return null;
	}

	//Added in july 2003
	/**
	 * This method returns the PDFText object.
	 * @param lasso
	 * @param me
	 */

	public PDFText getPdfText(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFText)lasso.typeGetCustomJavaObject(me);
		return null;
	}

	/**
	 * This method returns the PDFRead object.
	 * @param lasso
	 * @param me
	 */

	public PDFRead getPdfRead(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFRead)lasso.typeGetCustomJavaObject(me);
		return null;
	}

	/**
	 * This method returns the PDFDoc object.
	 * @param lasso
	 * @param me
	 */

	public PDFDoc getPdfDoc(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFDoc)lasso.typeGetCustomJavaObject(me);
		return null;
	}


	/**
	 * This method returns the PDFGraphics object.
	 * @param lasso
	 * @param me
	 */

	public PDFGraphics getPdfGraphics(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFGraphics)lasso.typeGetCustomJavaObject(me);
		return null;
	}


	/**
	 * This method returns the parameter value as a string.
	 * @param lasso
	 * @param me
	 */

	public String getString(LassoCall lasso)
	{
		LassoValue text = new LassoValue();
		if(ERR_NOERR == lasso.getTagParam(0, text ))
		{
			return text.name();	//returning the parameter value.
		}
		return null;
	}

	/**
	 * This method returns the parameter value as a string.
	 * @param lasso
	 * @param paramName
	 */

	public String getParamValue(LassoCall lasso, String paramName)
	{
		LassoValue text = new LassoValue();
		if(ERR_NOERR == lasso.findTagParam(paramName, text ))
		{
			return text.data();	//returning the parameter value as a string.
		}
		return null;
	}




	/* 	PDF_Graphic Tag
	 *	Handling of all the member tags.
	 */



	public int tagPDFGraphicFunc (LassoCall lasso, int action)
	{
		int err;

		PDFGraphics pdfG = new PDFGraphics();
		LassoValue result = new LassoValue();

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Graphic") == ERR_NOERR )
		{
			LassoTypeRef drawline = new LassoTypeRef();
			if (lasso.typeAllocTag(drawline,"PDFTag", "myGrDrawLineFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "DrawLine", drawline )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef drawrectangle = new LassoTypeRef();
			if (lasso.typeAllocTag(drawrectangle,"PDFTag", "myDrawRectangleFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "DrawRectangle", drawrectangle )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef drawellipse = new LassoTypeRef();
			if (lasso.typeAllocTag(drawellipse,"PDFTag", "myDrawEllipseFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "DrawEllipse", drawellipse )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef drawarc = new LassoTypeRef();
			if (lasso.typeAllocTag(drawarc,"PDFTag", "myDrawArcFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "DrawArc", drawarc )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef newpath = new LassoTypeRef();
			if (lasso.typeAllocTag(newpath,"PDFTag", "myNewPathFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "NewPath", newpath )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef closepath = new LassoTypeRef();
			if (lasso.typeAllocTag(closepath,"PDFTag", "myGrClosePathFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "ClosePath", closepath )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef curveto = new LassoTypeRef();
			if (lasso.typeAllocTag(curveto,"PDFTag", "myGrCurveToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "CurveTo", curveto )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef curvefromto = new LassoTypeRef();
			if (lasso.typeAllocTag(curvefromto,"PDFTag", "myGrCurveFromToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "CurveFromTo", curvefromto )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef lineto = new LassoTypeRef();
			if (lasso.typeAllocTag(lineto,"PDFTag", "myGrLineToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "LineTo", lineto )) != ERR_NOERR )
				{
					return err;
				}
			}

			LassoTypeRef moveto = new LassoTypeRef();
			if (lasso.typeAllocTag(moveto,"PDFTag", "myGrMoveToFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "MoveTo", moveto )) != ERR_NOERR )
				{
					return err;
				}
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfG)) != ERR_NOERR)
			{
				return setError(lasso, "PDF_Graphics:Error setting pdf_graphics object", err);
			}


			lasso.returnTagValue(me);

		} //end for MEMBER TAGS.
		return 0;


	}

	/* 	PDF_Graphic Tag
	 *	Implementation of all the Member Tags.
	 *	DrawLine,DrawEllipse,DrawArc,NewPath,ClosePath....etc
	 */


	/**
	 * This method draws a line on the document.
	 * @param lasso
	 * @param action
	 */

	public int myGrDrawLineFunc (LassoCall lasso, int action)
	{
		int err = 0;

		LassoTypeRef me 		= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		LassoTypeRef res2 		= new LassoTypeRef();
		LassoTypeRef element2 	= new LassoTypeRef();
		LassoValue typeName2 	= new LassoValue();

		int [] fromArr 			= new int[2];
		int [] toArr 			= new int[2];
		int iX1					= 0;
		int iX2 				= 0;
		int iY1 				= 0;
		int iY2					= 0;

		if (lasso.getTagSelf(me) != ERR_NOERR ) return -1;
		PDFGraphics g = this.getPdfGraphics(lasso, me);
		if(g != null)
		{
			if( lasso.findTagParam2("-from", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();
					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						for ( int i = 0; i < count.intValue(); i++ )
						{
							lasso.arrayGetElement(res, i, element);
							lasso.typeGetName(element,typeName);
							IntValue outIntValue = new IntValue();
							int iValue = 0;
							lasso.typeGetInteger(element, outIntValue);
							iValue = outIntValue.intValue();
							fromArr[i] = iValue;
						}
					}
				}

	 		}

	 		if( lasso.findTagParam2("-to", res2 ) == ERR_NOERR &&
			lasso.typeGetName(res2, typeName2) == ERR_NOERR)
			{
				if((typeName2.name()).equalsIgnoreCase("array"))
				{
					IntValue count2 = new IntValue();
					if ( lasso.arrayGetSize(res2, count2 ) == ERR_NOERR )
					{
						for ( int j = 0; j < count2.intValue(); j++ )
						{
							lasso.arrayGetElement(res2, j, element2);
							lasso.typeGetName(element2,typeName2);
							IntValue outIntValue = new IntValue();
							int iValue = 0;
							lasso.typeGetInteger(element2, outIntValue);
							iValue = outIntValue.intValue();
							toArr[j] = iValue;
						}
					}
				}

	 		}

	 		LassoValue resdata 	= new LassoValue();
	 		String strLineWidth = null;
	 		float fLineWidth 	= 0;

			err = lasso.findTagParam("-width", resdata );
			if(err == ERR_NOERR)
			{
				strLineWidth	= resdata.data();
				if(strLineWidth != null)
					fLineWidth = Float.parseFloat(strLineWidth);

			}

			String strColor = null;
			err = lasso.findTagParam("-color", resdata );
			if(err == ERR_NOERR)
			{
				strColor = resdata.data();

			}
	 		iX1 = fromArr[0];
			iY1 = fromArr[1];
			iX2 = toArr[0];
			iY2 = toArr[1];

			if((err = g.grDrawLine(iX1, iY1, iX2, iY2, (int)fLineWidth, strColor)) == ERR_NOERR)
				return  err;
			else
				return setError(lasso, "PDF_Graphic:(DrawLine) Error drawing a Line", err);

		}
		return setError(lasso, "PDF_Graphic:(DrawLine) Unknown Error", err);
	}

	/**
	 * This method draws a rectangle on the document.
	 * @param lasso
	 * @param action
	 */

	public int myDrawRectangleFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me 		= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[4];
		int iX1 				= 0;
		int iY1 				= 0;
		int iWidth 				= 0;
		int iHeight 			= 0;
		int err					= 0;

		if( lasso.findTagParam2("-bounds", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
		{

			if (lasso.getTagSelf(me) != ERR_NOERR ) return -1;
			PDFGraphics g = this.getPdfGraphics(lasso, me);

			if(g != null)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();
					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						for ( int i = 0; i < count.intValue(); i++ )
						{
							int iValue = 0;
							lasso.arrayGetElement(res, i, element);
							IntValue outIntValue = new IntValue();
							lasso.typeGetInteger(element, outIntValue);
							iValue = outIntValue.intValue();
							boundsArr[i] = iValue;
						}
					}
				}

				LassoValue resdata = new LassoValue();
				String strColor = null;
				err = lasso.findTagParam("-color", resdata );
				if(err == ERR_NOERR)
				{
					strColor	= resdata.data();

				}

				iX1 	= boundsArr[0];	//the X-Coordinate of the rectangle.
				iY1 	= boundsArr[1];	//the Y-Coordinate of the rectangle.
				iWidth 	= boundsArr[2];	//the Width value of the rectangle.
				iHeight = boundsArr[3];	//the Height value of the rectangle.

				if((err = g.grDrawRectangle(iX1, iY1, iWidth, iHeight, strColor)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Graphic:(DrawRectangle) Error drawing a Rectangle", err);

				}
				return setError(lasso, "PDF_Graphic:(DrawRectangle) Unknown Error", err);
		}
		return setError(lasso, "PDF_Graphic:(DrawRectangle) Unknown Error", err);

	}

	/**
	 * This method draws an ellipse on the document.
	 * @param lasso
	 * @param action
	 */

	public int myDrawEllipseFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me 		= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[4];
		int iX1 				= 0;
		int iY1 				= 0;
		int iX2 				= 0;
		int iY2					= 0;
		int err					= 0;

		if( lasso.findTagParam2("-bounds", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
		{

			if (lasso.getTagSelf(me) != ERR_NOERR ) return -1;
			PDFGraphics g = this.getPdfGraphics(lasso, me);

			if(g != null)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();
					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						for ( int i = 0; i < count.intValue(); i++ )
						{
							int iValue = 0;
							lasso.arrayGetElement(res, i, element);
							IntValue outIntValue = new IntValue();
							lasso.typeGetInteger(element, outIntValue);
							iValue = outIntValue.intValue();
							boundsArr[i] = iValue;
						}
					}
				}



				iX1 	= boundsArr[0];	//the X1-Coordinate of one corner of the rectangle.
				iY1 	= boundsArr[1];	//the Y1-Coordinate of one corner of the rectangle.
				iX2 	= boundsArr[2];	//the X2-Coordinate of one corner of the rectangle.
				iY2		= boundsArr[3];	//the Y2-Coordinate of one corner of the rectangle.

				if((err = g.grDrawEllipse(iX1, iY1, iX2, iY2)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Graphic:(DrawEllipse) Error drawing a Ellipse", err);

				}
				return setError(lasso, "PDF_Graphic:Unknown Error", err);
		}
		return setError(lasso, "PDF_Graphic:(DrawEllipse) Unknown Error", err);

	}

	/**
	 * This method draws an arc on the document.
	 * @param lasso
	 * @param action
	 */

	public int myDrawArcFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me 		= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[4];
		int iX1 				= 0;
		int iY1 				= 0;
		int iX2 				= 0;
		int iY2		 			= 0;
		float fStart 			= 0;
		float fExtent 			= 0;
		int err		   			= 0;
		PDFGraphics g   		= null;

		if( lasso.findTagParam2("-bounds", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
		{

			if (lasso.getTagSelf(me) == ERR_NOERR )
				g = this.getPdfGraphics(lasso, me);

			if(g != null)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();
					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						for ( int i = 0; i < count.intValue(); i++ )
						{
							int iValue = 0;
							lasso.arrayGetElement(res, i, element);
							IntValue outIntValue = new IntValue();
							lasso.typeGetInteger(element, outIntValue);
							iValue = outIntValue.intValue();
							boundsArr[i] = iValue;
						}
					}
				}

				LassoValue resdata = new LassoValue();
				String strStart = null;
				err = lasso.findTagParam("-start", resdata );
				if(err == ERR_NOERR)
				{
					strStart	= resdata.data();
					fStart = Float.parseFloat(strStart);

				}
				String strExtent = null;
				err = lasso.findTagParam("-extent", resdata );
				if(err == ERR_NOERR)
				{
					strExtent	= resdata.data();
					fExtent = Float.parseFloat(strExtent);

				}

				iX1	= boundsArr[0];	//the X1-Coordinate point.
				iY1	= boundsArr[1];	//the Y1-Coordinate point.
				iX2	= boundsArr[2];	//the X2-Coordinate point.
				iY2	= boundsArr[3];	//the Y2-Coordinate point.

				if((err = g.grDrawArc(iX1, iY1, iX2, iY2, (int)fStart, (int)fExtent)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Graphic:(DrawArc) Error drawing a Arc", err);

				}
				return setError(lasso, "PDF_Graphic:(DrawArc) Unknown Error", err);
		}
		return setError(lasso, "PDF_Graphic:(DrawArc) Unknown Error", err);

	}

	/**
	 * This method draws an new path on the document.
	 * @param lasso
	 * @param action
	 */

	public int myNewPathFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me	= new LassoTypeRef();
		int err = 0;
		PDFGraphics g = null;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);
			if(g != null)
			{
				if ((err = g.grDrawNewPath()) == ERR_NOERR )
					return err;
				else
					return setError(lasso, "PDF_Graphic:(NewPath) Error in NewPath", err);
			}
			return setError(lasso, "PDF_Graphic:(NewPath) Unknown Error", err);
	}

	/**
	 * This method closes the path after drawing operations are completed.
	 * @param lasso
	 * @param action
	 */

	public int myGrClosePathFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me	= new LassoTypeRef();
		int err			= 0;
		PDFGraphics g 	= null;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);
			if(g != null)
			{
				if ((err = g.grClosePath()) == ERR_NOERR )
					return err;
				else
					return setError(lasso, "PDF_Graphic:(ClosePath) Error ClosingPath", err);
			}
			return setError(lasso, "PDF_Graphic:(ClosePath) Unknown Error", err);

	}

	/**
	 * This method draws an curve on the document.
	 * @param lasso
	 * @param action
	 */

	public int myGrCurveToFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me			= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[6];
		PDFGraphics g 			= null;

		int iX1 				= 0;
		int iY1 				= 0;
		int iX2 				= 0;
		int iY2 				= 0;
		int iX3 				= 0;
		int iY3 				= 0;
		int err 				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);
			if(g != null)
			{
				if(lasso.getTagParam2( 0, res ) == ERR_NOERR)
				{
					if(lasso.typeGetName(res,typeName) == ERR_NOERR)
					{
						if((typeName.name()).equalsIgnoreCase("array"))
						{
							IntValue count = new IntValue();
							if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
							{
								for ( int i = 0; i < count.intValue(); i++ )
								{
									int iValue = 0;
									lasso.arrayGetElement(res, i, element);
									IntValue outIntValue = new IntValue();
									lasso.typeGetInteger(element, outIntValue);
									iValue = outIntValue.intValue();
									boundsArr[i] = iValue;
								}
							}
						}

						iX1 	= boundsArr[0];	//the X1-Coordinate point.
						iY1 	= boundsArr[1];	//the Y1-Coordinate point.
						iX2 	= boundsArr[2];	//the X2-Coordinate point.
						iY2		= boundsArr[3];	//the Y2-Coordinate point.
						iX3 	= boundsArr[4];	//the X3-Coordinate point.
						iY3		= boundsArr[5];	//the Y3-Coordinate point.

						if ((err = g.grDrawCurveTo(iX1, iY1, iX2, iY2, iX3, iY3)) == ERR_NOERR )
							return err;
						else
							return setError(lasso, "PDF_Graphic:(CurveTo) Error drawing Curve", err);

					}

				}

			}

			return setError(lasso, "PDF_Graphic:(CurveTo) Unknown Error ", err);
	}

	/**
	 * This method draws an curve on the document.
	 * @param lasso
	 * @param action
	 */

	public int myGrCurveFromToFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me			= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[4];
		PDFGraphics g			= null;

		int iX1 				= 0;
		int iY1 				= 0;
		int iX3 				= 0;
		int iY3 				= 0;
		int err					= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);
			if(g != null)
			{
				if(lasso.getTagParam2( 0, res ) == ERR_NOERR)
				{
					if(lasso.typeGetName(res,typeName) == ERR_NOERR)
					{
						if((typeName.name()).equalsIgnoreCase("array"))
						{
							IntValue count = new IntValue();
							if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
							{
								for ( int i = 0; i < count.intValue(); i++ )
								{
									int iValue = 0;
									lasso.arrayGetElement(res, i, element);
									IntValue outIntValue = new IntValue();
									lasso.typeGetInteger(element, outIntValue);
									iValue = outIntValue.intValue();
									boundsArr[i] = iValue;
								}
							}
						}

						iX1 	= boundsArr[0];	//the X1-Coordinate point.
						iY1 	= boundsArr[1];	//the Y1-Coordinate point.
						iX3 	= boundsArr[2];	//the X2-Coordinate point.
						iY3		= boundsArr[3];	//the Y2-Coordinate point.

						if ((err = g.grDrawCurveFromTo(iX1, iY1, iX3, iY3)) == ERR_NOERR )
							return err;
						else
							return setError(lasso, "PDF_Graphic:(CurveFromTo)Error drawing a Curve. ", err);

					}
				}
			}
			return setError(lasso, "PDF_Graphic:(CurveFromTo) Unknown Error ", err);
	}

	/**
	 * This method draws an curve on the document.
	 * @param lasso
	 * @param action
	 */

	public int myGrLineToFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me			= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[2];
		PDFGraphics g			= null;

		int iX1 				= 0;
		int iY1 				= 0;
		int err					= 0;
		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);
			if(g != null)
			{
				if(lasso.getTagParam2( 0, res ) == ERR_NOERR)
				{
					if(lasso.typeGetName(res,typeName) == ERR_NOERR)
					{
						if((typeName.name()).equalsIgnoreCase("array"))
						{
							IntValue count = new IntValue();
							if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
							{
								for ( int i = 0; i < count.intValue(); i++ )
								{
									int iValue = 0;
									lasso.arrayGetElement(res, i, element);
									IntValue outIntValue = new IntValue();
									lasso.typeGetInteger(element, outIntValue);
									iValue = outIntValue.intValue();
									boundsArr[i] = iValue;
								}

								if(count.intValue() > 0)
								{
								iX1 	= boundsArr[0];
								iY1 	= boundsArr[1];
								if ((err = g.grDrawLineTo(iX1, iY1 )) == ERR_NOERR )
									return err;
								else
									return setError(lasso, "PDF_Graphic:(LineTo) Error drawing Line ", err);
								}
							}
						}
					}
				}

			}
			return setError(lasso, "PDF_Graphic:(LineTo) Unknown Error", err);
		}

	/**
	 * This method moves the pen to the specified position on the document.
	 * @param lasso
	 * @param action
	 */

	public int myGrMoveToFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me			= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element 	= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		int [] boundsArr 		= new int[2];
		PDFGraphics g			= null;

		int iX1 				= 0;
		int iY1 				= 0;
		int err					= 0;


		if (lasso.getTagSelf(me) == ERR_NOERR )
			g = this.getPdfGraphics(lasso, me);

		if(g != null)
		{
			if(lasso.getTagParam2( 0, res ) == ERR_NOERR)
			{
				if(lasso.typeGetName(res,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("array"))
					{
						IntValue count = new IntValue();
						if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
						{
							for ( int i = 0; i < count.intValue(); i++ )
							{
								int iValue = 0;
								lasso.arrayGetElement(res, i, element);
								IntValue outIntValue = new IntValue();
								lasso.typeGetInteger(element, outIntValue);
								iValue = outIntValue.intValue();
								boundsArr[i] = iValue;
							}

							if(count.intValue() > 0)
							{
							iX1 	= boundsArr[0];
							iY1 	= boundsArr[1];

							if ((err = g.grDrawMoveTo(iX1, iY1 )) == ERR_NOERR )
								return err;
							else
								return setError(lasso, "PDF_Graphic:(MoveTo) Error in MoveTo", err);
							}
						}
					}
				}
			}
		}

		return setError(lasso, "PDF_Graphic:(MoveTo) Unknown error ", err);
	}

	/**
	 * Sets the Error message for the LassoScript.
	 */

	int setError(LassoCall lasso, String errMsg, int err)
	{
		lasso.setResultMessage(errMsg);
		lasso.setResultCode(err);
		return err;
	}

	/* This method returns a PDFBarcode class object from the vector on the basis of the key value.
	 * Returns the PDFBarcode object.
	 */

	public PDFBarCode getPdfBarCode(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFBarCode)lasso.typeGetCustomJavaObject(me);
		return null;
	}


	/* This method returns a PDFFont class object from the vector on the basis of the key value.
	 * Returns the PDFFont object.
	 */

	public PDFFont getPdfFont(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFFont)lasso.typeGetCustomJavaObject(me);
		return null;
	}


	/* This method returns a PDFTable class object from the vector on the basis of the key value.
	 * Returns the PDFTable object.
	 */

	public PDFTable getPdfTable(LassoCall lasso, LassoTypeRef me)
	{
		if ( me != null )
			return (PDFTable)lasso.typeGetCustomJavaObject(me);
		return null;
	}


	/* 	PDF_Font Tag
	 *	Stores all the specificatiobs of a font:
	 *	font family, size, style, color etc.
	 */

	public int tagPDFFontFunc (LassoCall lasso, int action)
	{
		int err = 0;

		PDFFont pdfF = new PDFFont();

		//optionally get -face parameter
		String sFace = getParamValue(lasso,"-face");

		//optionally get -file parameter
		String sFile = getParamValue(lasso,"-file");

		if(sFile != null)
			sFile = internalToFullPath(lasso, sFile);

		//optionally get -size parameter
		String sSize = getParamValue(lasso,"-size");

		//optionally get -color parameter
		String sColor = getParamValue(lasso,"-color");

		//optionally get -encoding parameter
		String sEncoding = getParamValue(lasso,"-encoding");


		int size = -1;
		if(sSize != null)
			size = Integer.parseInt(sSize);

		//optionally get -embed parameter
		boolean bEmbed = false;
		IntValue bValue	= getBooleanValue(lasso,"-embed");

		if(bValue.intValue() == 1)
			bEmbed = true;

		try{
		 	pdfF.createFont(sFace, sFile, size, sColor, sEncoding, bEmbed);
		}
		catch(Exception e)
		{

			return setError(lasso, "PDF_Font: " + e.getMessage(), LassoErrors.UnknownError);
//			e.printStackTrace();
//			return LassoErrors.UnknownError;
		}

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Font") == ERR_NOERR )
		{
			//GET THE TEXT WIDTH
			LassoTypeRef getWidth = new LassoTypeRef();
			err = lasso.typeAllocTag(getWidth, "PDFTag", "myGetTextWidth" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "TextWidth", getWidth);
			}

			//setface
			LassoTypeRef setface = new LassoTypeRef();
			err = lasso.typeAllocTag(setface, "PDFTag", "mySetFaceFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetFace", setface );
			}

			//setcolor
			LassoTypeRef scolor = new LassoTypeRef();
			err = lasso.typeAllocTag(scolor, "PDFTag", "mySetFontColorFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetColor", scolor );
			}

			//setsize
			LassoTypeRef ssize = new LassoTypeRef();
			err = lasso.typeAllocTag(ssize, "PDFTag", "mySetSizeFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetSize", ssize );
			}

			//setencoding
			LassoTypeRef setencoding = new LassoTypeRef();
			err = lasso.typeAllocTag(setencoding, "PDFTag", "mySetEncodingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetEncoding", setencoding );
			}

			//setstyle
			LassoTypeRef setstyle = new LassoTypeRef();
			err = lasso.typeAllocTag(setstyle, "PDFTag", "mySetStyleFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetStyle", setstyle );
			}

			/*
			//setbold
			LassoTypeRef setbold = new LassoTypeRef();
			err = lasso.typeAllocTag(setbold, "PDFTag", "mySetBoldFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetBold", setbold );
			}
			*/
			//setitalic
			/*
			LassoTypeRef setitalic = new LassoTypeRef();
			err = lasso.typeAllocTag(setitalic, "PDFTag", "mySetItalicFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetItalic", setitalic );
			}
			*/
			//setunderline
			LassoTypeRef setunderline = new LassoTypeRef();
			err = lasso.typeAllocTag(setunderline,"PDFTag", "mySetUnderLineFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetUnderline", setunderline );
			}

			//getface
			LassoTypeRef getface = new LassoTypeRef();
			err = lasso.typeAllocTag(getface, "PDFTag", "myGetFaceFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetFace", getface );
			}

			//getcolor
			LassoTypeRef getcolor = new LassoTypeRef();
			err = lasso.typeAllocTag(getcolor, "PDFTag", "myGetColorFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetColor", getcolor );
			}

			//getsize
			LassoTypeRef getsize = new LassoTypeRef();
			err = lasso.typeAllocTag(getsize, "PDFTag", "myGetFontSizeFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetSize", getsize );
			}

			//getencoding
			LassoTypeRef getencoding = new LassoTypeRef();
			err = lasso.typeAllocTag(getencoding, "PDFTag", "myGetEncodingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetEncoding", getencoding );
			}


			//getstyle
			LassoTypeRef getstyle = new LassoTypeRef();
			err = lasso.typeAllocTag(getstyle, "PDFTag", "myGetStyleFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetStyle", getstyle );
			}


			//getpsfontname
			LassoTypeRef getpsfontname = new LassoTypeRef();
			err = lasso.typeAllocTag(getpsfontname, "PDFTag", "myGetPSFontNameFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetPSFontName", getpsfontname );
			}


			//istruetype
			LassoTypeRef istruetype = new LassoTypeRef();
			err = lasso.typeAllocTag(istruetype, "PDFTag", "myIsTrueTypeFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "IsTrueType", istruetype );
			}

			//getsupportedenc
			LassoTypeRef getsupportedenc = new LassoTypeRef();
			err = lasso.typeAllocTag(getsupportedenc, "PDFTag", "myGetSupportedEncodingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetSupportedEncodings", getsupportedenc );
			}

			//getfullfontname
			LassoTypeRef getfullfontname = new LassoTypeRef();
			err = lasso.typeAllocTag(getfullfontname, "PDFTag", "myGetFullFontNameFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetFullFontName", getfullfontname );
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfF)) != ERR_NOERR)
			{
				return setError(lasso, "PDF_Font: Error setting pdf_font object", err);
			}

			lasso.returnTagValue(me);


		} //end for MEMBER TAGS.
		return 0;


	}

	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int mySetFaceFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetFace", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String strFace = getString(lasso);
			if(strFace != null)
			{
				pF.setFontFace(strFace);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the font color.
	 * @param lasso
	 * @param action
	 */

	public int mySetFontColorFunc (LassoCall lasso, int action)
	{
		int err				=0;
		boolean bParseColor = false;
		LassoTypeRef me 	= new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetColor", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String strColor = getString(lasso);
			if(strColor != null)
				err = pF.setFontColor(strColor);

			if(err != ERR_NOERR)
				return setError(lasso, "PDF_Font: Error while setting the color of the font", err);
			else
				return 0;
		}
		return setError(lasso, "PDF_Font: Error while setting the color of the font", err);
	}

	/**
	 * This method sets the font size.
	 * @param lasso
	 * @param action
	 */

	public int mySetSizeFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetSize", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String strSize = getString(lasso);
			int size = Integer.parseInt(strSize);

			if(strSize != null)
			{
				pF.setFontSize(size);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the font encoding value.
	 * @param lasso
	 * @param action
	 */

	public int mySetEncodingFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetEncoding", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String strEnc = getString(lasso);
			if(strEnc != null)
			{
				pF.setFontEncoding(strEnc);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the font style.
	 * @param lasso
	 * @param action
	 */

	public int mySetStyleFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetStyle", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			pF.setFontNormal();

			IntValue bValue	= getBooleanValue(lasso,"-italic");
			if(bValue.intValue() == 1)
				pF.setFontItalic(true);
			else
				pF.setFontItalic(false);

			bValue	= getBooleanValue(lasso,"-bold");
			if(bValue.intValue() == 1)
				pF.setFontBold(true);
			else
				pF.setFontBold(false);

			bValue	= getBooleanValue(lasso,"-underline");	//underscore chenged to underline
			if(bValue.intValue() == 1)
				pF.setFontUnderLine(true);
			else
				pF.setFontUnderLine(false);
		}
		return 0;
	}

	/**
	 * This method sets the font to bold style.
	 * @param lasso
	 * @param action
	 */

	//setBold and setItalic commented on 13-09-2002

/*	public int mySetBoldFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetBold", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String sBold = getString(lasso);
			boolean bBold = false;
			if(sBold != null)
			{
				if(sBold.equalsIgnoreCase("true"))
					pF.setFontBold(true);
				else
					pF.setFontBold(false);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}
*/
	/**
	 * This method sets the font to italic style.
	 * @param lasso
	 * @param action
	 */

/*	public int mySetItalicFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetItalic", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String sItalic = getString(lasso);
			boolean bItalic = false;
			if(sItalic != null)
			{
				if(sItalic.equalsIgnoreCase("true"))
					pF.setFontItalic(true);
				else
					pF.setFontItalic(false);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}
*/
	/**
	 * This method sets the font to Underline style.
	 * @param lasso
	 * @param action
	 */

	public int mySetUnderLineFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling SetUnderLine", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			String sUnderLine	= getString(lasso);
			boolean bUnderLine 	= false;

			if(sUnderLine != null)
			{
				if(sUnderLine.equalsIgnoreCase("true"))
					pF.setFontUnderLine(true);
				else
					pF.setFontUnderLine(false);
			}
			else
					pF.setFontUnderLine(true); // code added 13-09-2002 (by default made to true)
		}
		return 0;
	}

	//ADDed Aug 2003
	/**
	 * This method returns the  font face value.
	 * @param lasso
	 * @param action
	 */
	public int myGetTextWidth(LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling TextWidth", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String strContent = getString(lasso);
			if(strContent==null)
				strContent = "";
			float fTextWidth = pF.getTextWidth(strContent);
			err  = lasso.typeAllocDecimal(ref, (double)fTextWidth);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;

	}


	/**
	 * This method returns the  font face value.
	 * @param lasso
	 * @param action
	 */

	public int myGetFaceFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetFace", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sFace = pF.getFontFace();
			if(sFace == null)
				sFace = "No font face specified";
			err  = lasso.typeAllocString(ref, sFace);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font color value.
	 * @param lasso
	 * @param action
	 */

	public int myGetColorFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetColor", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sData = pF.getFontColor();
			if(sData == null) sData = "Unable to get color";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font size value.
	 * @param lasso
	 * @param action
	 */

	public int myGetFontSizeFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetSize", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref	= new LassoTypeRef();
			int lSize 			= pF.getFontSize();

			err  = lasso.typeAllocInteger(ref, (long) lSize);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font encoding value.
	 * @param lasso
	 * @param action
	 */

	public int myGetEncodingFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetEncoding", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sData =pF.getFontEncoding();

			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font style value.
	 * @param lasso
	 * @param action
	 */

	public int myGetStyleFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetStyle", err);


		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef[] initial	= new LassoTypeRef[3];
			LassoTypeRef bref 		= new LassoTypeRef();
			LassoTypeRef sref 		= new LassoTypeRef();
			LassoTypeRef pref1 		= new LassoTypeRef();
			LassoTypeRef pref2 		= new LassoTypeRef();
			LassoTypeRef pref3 		= new LassoTypeRef();

			boolean bItalic 		= pF.getItalic();
			boolean bBold 			= pF.getBold();
			boolean bUnderScore 	= pF.getUnderScore();


			err = lasso.typeAllocString(sref, "-italic");
			err = lasso.typeAllocBoolean(bref, bItalic);
			err	=  lasso.typeAllocPair(pref1, sref, bref);

			err = lasso.typeAllocString(sref, "-bold");
			err = lasso.typeAllocBoolean(bref, bBold);
			err =  lasso.typeAllocPair(pref2, sref, bref);

			err = lasso.typeAllocString(sref, "-underline");  //underscore changed to underline
			err = lasso.typeAllocBoolean(bref, bUnderScore);
			err =  lasso.typeAllocPair(pref3, sref, bref);

			initial[0]	= pref1;
			initial[1]	= pref2;
			initial[2]	= pref3;

			err = lasso.typeAllocArray(mArray, initial);
			if(err == ERR_NOERR)
				lasso.returnTagValue(mArray);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font font name.
	 * @param lasso
	 * @param action
	 */
	public int myGetPSFontNameFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetPSFontName", err);


		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sData = pF.getPSFontName();
			if(sData == null) sData = "No PS Font Name found";
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method checks if the font is of TrueType..
	 * @param lasso
	 * @param action
	 */
	//todo: implement this in PDFFont
	public int myIsTrueTypeFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling IsTrueType", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			boolean bTrueType = pF.isTrueType();

			err  = lasso.typeAllocBoolean(ref, bTrueType);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  supported encoding values of the font.
	 * @param lasso
	 * @param action
	 */
	public int myGetSupportedEncodingFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetSupportedEncoding", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef mArray 	= new LassoTypeRef();
			//LassoTypeRef initial[] 	= new LassoTypeRef[1];
			LassoTypeRef sref 		= new LassoTypeRef();
			err 					= lasso.typeAllocArray(mArray, null );
			int size 				= 0;
			String[] sEncs 			= pF.getSupportedEncodings();

			for(int k = 0; k < sEncs.length; k++)
			{
				err = lasso.typeAllocString(sref, sEncs[k]); //use iteration here
				// adding the elements to the array
				size++;
				err = lasso.arraySetElement(mArray, size, sref);
			}

			if(err == ERR_NOERR)
				lasso.returnTagValue(mArray);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  font full name value.
	 * @param lasso
	 * @param action
	 */
	public int myGetFullFontNameFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Font: Error while calling GetFullFontName", err);

		PDFFont pF = this.getPdfFont(lasso, me);
		if(pF != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sData = pF.getFontFace();
			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * The PDF_Table tag is handled here.
	 * @param lasso
	 * @param action
	 */

	//Code for -Width added on 12-09-2002
	public int tagPDFTableFunc (LassoCall lasso, int action)
	{
		int err = 0;

		PDFTable pdfTbl = new PDFTable();

		//optionally get -cols parameter
		String sCols = getParamValue(lasso,"-cols");

		//optionally get -rows parameter
		String sRows = getParamValue(lasso,"-rows");

		//optionally get -spacing parameter
		String sSpacing = getParamValue(lasso,"-spacing");

		//optionally get -padding parameter
		String sPadding = getParamValue(lasso,"-padding");

		//optionally get -width parameter
		String sWidth = getParamValue(lasso,"-width");

		//optionally get -BorderWidth parameter
		String sBrWidth = getParamValue(lasso,"-BorderWidth");

		//optionally get -BorderColor parameter
		String sBrColor = getParamValue(lasso,"-BorderColor");

		//optionally get -BackgroundColor parameter
		String sBgColor = getParamValue(lasso,"-BackgroundColor");

		//optionally get -Rotate parameter
		String sRotate = getParamValue(lasso,"-Rotate");


		int iCols = -1;
		if(sCols != null)
			iCols = Integer.parseInt(sCols);

		int iRows = -1;
		if(sRows != null)
			iRows = Integer.parseInt(sRows);

		int iSpacing = -1;
		if(sSpacing != null)
			iSpacing = Integer.parseInt(sSpacing);

		int iPadding = -1;
		if(sPadding != null)
			iPadding = Integer.parseInt(sPadding);

		int iWidth = -1;
		if(sWidth != null)
			iWidth = Integer.parseInt(sWidth);

		int iBWidth = -1;
		if(sBrWidth != null)
			iBWidth = Integer.parseInt(sBrWidth);



		LassoValue typeName		= new LassoValue();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoTypeRef element	= new LassoTypeRef();

		float [] tempArr	= null;

		// get the -ColWidth parameter.
		if( lasso.findTagParam2("-ColWidth", res ) == ERR_NOERR &&
			lasso.typeGetName(res,typeName) == ERR_NOERR)
		{
			tempArr	= new float[iCols];

			if((typeName.name()).equalsIgnoreCase("array"))
			{
				IntValue count = new IntValue();
				if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
				{

					for ( int i = 0; i < count.intValue(); i++ )
						{
							lasso.arrayGetElement(res, i, element);

							LassoValue typeName2 = new LassoValue();

							lasso.typeGetName(element,typeName2);

							DecimalValue outDecimalValue = new DecimalValue();
							float fValue = 0;

							if((typeName2.name()).equalsIgnoreCase("decimal"))
							{

							lasso.typeGetDecimal(element, outDecimalValue);
							fValue = outDecimalValue.floatValue();

							}
							else if((typeName2.name()).equalsIgnoreCase("integer"))
							{

							IntValue outIntValue = new IntValue();
							lasso.typeGetInteger(element, outIntValue);
							fValue = outIntValue.intValue();
							}
							tempArr[i] = fValue;
						}
				}
			}

		}

		try{
			pdfTbl.CreateTable(iCols, iRows, iSpacing, iPadding, iWidth, iBWidth, tempArr, sBrColor, sBgColor, sRotate); //iWidth added on 12-09-2002
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return LassoErrors.UnknownError;
		}

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Table") == ERR_NOERR )
		{
			//setBorderWidth
			LassoTypeRef setbw = new LassoTypeRef();
			err = lasso.typeAllocTag(setbw, "PDFTag", "mySetBorderWidthFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetBorderWidth", setbw );
			}

			//setBorderColor
			LassoTypeRef setbc = new LassoTypeRef();
			err = lasso.typeAllocTag(setbc, "PDFTag", "mySetBorderColorFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetBorderColor", setbc );
			}

			//setPadding
			LassoTypeRef setpadding = new LassoTypeRef();
			err = lasso.typeAllocTag(setpadding, "PDFTag", "mySetPaddingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetPadding", setpadding );
			}

			//setSpacing
			LassoTypeRef setspacing = new LassoTypeRef();
			err = lasso.typeAllocTag(setspacing, "PDFTag", "mySetSpacingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetSpacing", setspacing );
			}

			//setAlignment
			LassoTypeRef setalignment = new LassoTypeRef();
			err = lasso.typeAllocTag(setalignment, "PDFTag", "mySetAlignmentFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "SetAlignment", setalignment );
			}

			//code added 12-09-2002
			//set coloumn width
/*			LassoTypeRef setcoloumnwidth = new LassoTypeRef();
			if (lasso.typeAllocTag(setcoloumnwidth,"PDFTag", "mySetColoumnWidthFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "SetColoumnWidth", setcoloumnwidth )) != ERR_NOERR )
				{
					return err;
				}
			}
*/
			//code added 12-09-2002
			/**
		 	 * Handling the GetColoumnWidths accessor method of PDF_Table tag.
		 	 * This method returns array of percentage.
		     */
/*			LassoTypeRef getcoloumnwidth = new LassoTypeRef();
			if (lasso.typeAllocTag(getcoloumnwidth,"PDFTag", "myGetColoumnWidthFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetColoumnWidth", getcoloumnwidth ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}
*/
			//getBorderWidth
			LassoTypeRef getbw = new LassoTypeRef();
			err = lasso.typeAllocTag(getbw, "PDFTag", "myGetBorderWidthFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetBorderWidth", getbw );
			}

			//getBorderColor
			LassoTypeRef getbc = new LassoTypeRef();
			err = lasso.typeAllocTag(getbc, "PDFTag", "myGetBorderColorFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetBorderColor", getbc );
			}

			//getPadding
			LassoTypeRef getpadding = new LassoTypeRef();
			err = lasso.typeAllocTag(getpadding, "PDFTag", "myGetPaddingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetPadding", getpadding );
			}

			//getSpacing
			LassoTypeRef getspacing = new LassoTypeRef();
			err = lasso.typeAllocTag(getspacing, "PDFTag", "myGetSpacingFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetSpacing", getspacing );
			}

			//getAlignment
			LassoTypeRef getalignment = new LassoTypeRef();
			err = lasso.typeAllocTag(getalignment, "PDFTag", "myGetAlignmentFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetAlignment", getalignment );
			}

			//getColoumnCount
			LassoTypeRef getcc = new LassoTypeRef();
			err = lasso.typeAllocTag(getcc, "PDFTag", "myGetColoumnCountFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetColumnCount", getcc);
			}

			//getRowCount
			LassoTypeRef getrc = new LassoTypeRef();
			err = lasso.typeAllocTag(getrc, "PDFTag", "myGetRowCountFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetRowCount", getrc );
			}

			//getAbsWidth
			LassoTypeRef getabsw = new LassoTypeRef();
			err = lasso.typeAllocTag(getabsw, "PDFTag", "myGetAbsWidthFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "GetAbsWidth", getabsw );
			}

			//Add text
			//Insert tag renamed to Add in Lasso 7 realease.
			LassoTypeRef tbladd = new LassoTypeRef();
			err = lasso.typeAllocTag(tbladd, "PDFTag", "myInsertFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "Add", tbladd );
			}

			//Insert text
			//Insert tag still there for backward compatibility.
			LassoTypeRef tblinsert = new LassoTypeRef();
			err = lasso.typeAllocTag(tblinsert, "PDFTag", "myInsertFunc" );

			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "Insert", tblinsert );
			}

			//Get
//			LassoTypeRef tblget = new LassoTypeRef();
//			err = lasso.typeAllocTag(tblget, "PDFTag", "myGetFunc" );
//			if(err == ERR_NOERR)
//			{
//				err = lasso.typeAddMember(me , "Get", tblget );
//			}


			//Remove
			LassoTypeRef tblremove = new LassoTypeRef();
			err = lasso.typeAllocTag(tblremove, "PDFTag", "myRemoveFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "Remove", tblremove );
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfTbl)) != ERR_NOERR)
			{
				return setError(lasso, "PDF_Table: Error removing a cell", err);
			}



			lasso.returnTagValue(me);

		} //end for MEMBER TAGS.
		return 0;


	}

	/**
	 * This method Sets the table border width value.
	 * @param lasso
	 * @param action
	 */
	public int mySetBorderWidthFunc (LassoCall lasso, int action)
	{
		String strBorderWidth 	= null;
		float fBorderWidth 		= 0;
		LassoTypeRef me 		= new LassoTypeRef();
		PDFTable retPdfTable 	= null;
		int err					= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			strBorderWidth = getString(lasso);
			if(strBorderWidth != null)
			{
			fBorderWidth = Float.parseFloat(strBorderWidth);
			if((err = retPdfTable.setTableBorderWidth(fBorderWidth)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Table:(SetBorderWidth)Unable to set BorderWidth of the table", err);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return setError(lasso, "PDF_Table:(SetBorderWidth)Unknown Error", err);
	}

	/**
	 * This method Sets the table border color.
	 * @param lasso
	 * @param action
	 */

	public int mySetBorderColorFunc (LassoCall lasso, int action)
	{
		int err			= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling SetBorderColor", err);

		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			String strColor = getString(lasso);
			if(strColor != null)
				retPdfTable.setTableBorderColor(strColor);
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method Sets the table padding value.
	 * @param lasso
	 * @param action
	 */

	public int mySetPaddingFunc (LassoCall lasso, int action)
	{
		String strPadding 		= null;
		float fPadding 			= 0;
		LassoTypeRef me 		= new LassoTypeRef();
		PDFTable retPdfTable 	= null;
		int err					= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			strPadding = getString(lasso);
			if(strPadding != null)
			{
			fPadding = Float.parseFloat(strPadding);
			if((err = retPdfTable.setTablePadding(fPadding)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Table:(SetPadding)Unable to set the Padding value of the table", err);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return setError(lasso, "PDF_Table:(SetPadding)Unknown Error", err);
	}


	/**
	 * This method Sets the table spacing value.
	 * @param lasso
	 * @param action
	 */
	public int mySetSpacingFunc (LassoCall lasso, int action)
	{
		String strSpacing 		= null;
		float fSpacing			= 0;
		LassoTypeRef me 		= new LassoTypeRef();
		PDFTable retPdfTable 	= null;
		int err					= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			strSpacing = getString(lasso);
			if(strSpacing != null)
			{
			fSpacing = Float.parseFloat(strSpacing);
			if((err = retPdfTable.setTableSpacing(fSpacing)) == ERR_NOERR)
				return err;
			else
				return setError(lasso, "PDF_Table:(SetSpacing)Unable to set the Spacing value of the table", err);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return setError(lasso, "PDF_Table:(SetSpacing)Unknown Error", err);
	}


	/**
	 * This method Sets the table Alignment value.
	 * @param lasso
	 * @param action
	 */

	public int mySetAlignmentFunc (LassoCall lasso, int action)
	{
		String strAlignment 	= null;
		int iAlignment			= 0;
		LassoTypeRef me 		= new LassoTypeRef();
		PDFTable retPdfTable 	= null;
		int err					= 0;

		if(lasso.getTagSelf(me) == ERR_NOERR )
			retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			strAlignment = getString(lasso);
			if(strAlignment != null)
			{

				if(strAlignment.equalsIgnoreCase("left"))
					iAlignment = 0;
				else if(strAlignment.equalsIgnoreCase("center"))
					iAlignment = 1;
				else if(strAlignment.equalsIgnoreCase("right"))
					iAlignment = 2;

				if((err = retPdfTable.setTableAlignment(iAlignment)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Table:(SetAlignment)Unable to set the Alignment value for the table", err);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return setError(lasso, "PDF_Table:(SetAlignment)Unknown Error", err);
	}

	// code added 12-09-2002
	/**
	 * This method sets the coloumn width
	 * @param lasso
	 * @param action
	 */

/*	public int mySetColoumnWidthFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me 		= new LassoTypeRef();
		LassoTypeRef res 		= new LassoTypeRef();
		LassoValue typeName 	= new LassoValue();
		Float[] colWidthArr 	= null;
		int arrSize 			= 0;
		int cc 					= 0;
		int err					= 0;


			PDFTable retPdfTable = this.getPdfTable(lasso, me);

			if(retPdfTable != null)
			{
				cc = retPdfTable.getTableColoumnCount();
				int [] boundsArr 		= new int[cc];

			if( lasso.getTagParam2(0, res) == ERR_NOERR &&
				lasso.typeGetName(res,typeName) == ERR_NOERR)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();

					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						arrSize = count.intValue();

						if(arrSize == cc)
						{
							colWidthArr	= new Float[arrSize];

							for ( int k = 0; k < arrSize; k++ )
							{
								LassoTypeRef element= new LassoTypeRef();
								LassoValue outDecimalValue = new LassoValue();

								lasso.arrayGetElement(res, k, element);
								lasso.typeGetString(element, outStringValue);
								colWidthArr[k] = outDecimalValue.value();
							}
						}
					}

				}
				if((err = retPdfTable.setColoumnWidth(colWidthArr)) == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Table:(SetColoumnWidth) Error setting the coloumn Width", err);

			}
			return setError(lasso, "PDF_Table:(SetColoumnWidth) Error setting the coloumn Width", err);
	}

	}
*/
	/**
	 * This method returns the coloumn width.
	 * @param lasso
	 * @param action
	 */

/*	public int myGetColoumnWidthFunc(LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		int err				= 0;

		PDFTable retPdfTable = this.getPdfTable(lasso, me);

		if(retPdfTable != null)
		{
			float[] coloumnwidthArr = new float[1];
			coloumnwidthArr = retPdfTable.getColoumnWidth();

			LassoTypeRef data 		= new LassoTypeRef();
			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef[] initial	= new LassoTypeRef[1];

			LassoTypeRef x1 		= new LassoTypeRef();


			lasso.typeAllocDecimal(x1, (double)coloumnwidthArr[0]);


			initial[0]=x1;

			lasso.typeAllocArray(mArray, initial);
			lasso.returnTagValue(mArray);
			return setError(lasso, "PDF_Table:(GetColoumnWidth) Error getting the coloumn width", err);
		}

		return setError(lasso, "PDF_Table:(GetColoumnWidth) Unknown Error", err);
	}



*/
	/**
	 * This method Gets the table borderwidth value .
	 * @param lasso
	 * @param action
	 */

	public int myGetBorderWidthFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetBorderWidth", err);

		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			float fBorderWidth = retPdfTable.getTableBorderWidth();

			err  = lasso.typeAllocDecimal(ref, (double) fBorderWidth);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method Gets the table table border color value .
	 * @param lasso
	 * @param action
	 */

	public int myGetBorderColorFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetTableBorderColor", err);

		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			String sData 		= retPdfTable.getTableBorderColor();

			if(sData == null) sData = "Unable to get table border color";

			err  = lasso.typeAllocString(ref, sData);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method Gets the table padding value .
	 * @param lasso
	 * @param action
	 */

	public int myGetPaddingFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetPadding", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			float fTblPadding 	= retPdfTable.getTablePadding();
			err  = lasso.typeAllocDecimal(ref, (double) fTblPadding);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method Gets the table spacing value .
	 * @param lasso
	 * @param action
	 */

	public int myGetSpacingFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetSpacing", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			float fTblSpacing = retPdfTable.getTableSpacing();

			err  = lasso.typeAllocDecimal(ref, (double) fTblSpacing);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method Gets the table Alignment value .
	 * @param lasso
	 * @param action
	 */

	public int myGetAlignmentFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetAlignment", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			int iTblAlignment	= retPdfTable.getTableAlignment();

			err  = lasso.typeAllocInteger(ref, (long) iTblAlignment);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method Gets the table Coloumn Count value .
	 * @param lasso
	 * @param action
	 */

	public int myGetColoumnCountFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetColoumnCount", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			int iTblColsCount 	= retPdfTable.getTableColoumnCount();

			err  = lasso.typeAllocInteger(ref, (long) iTblColsCount);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method Gets the table Row Count value .
	 * @param lasso
	 * @param action
	 */

	public int myGetRowCountFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetRowCount", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref 	= new LassoTypeRef();
			int iTblRowsCount 	= retPdfTable.getTableRowCount();

			err  = lasso.typeAllocInteger(ref, (long) iTblRowsCount);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method Gets the Absolute Width value .
	 * @param lasso
	 * @param action
	 */

	public int myGetAbsWidthFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling GetAbsoluteWidth", err);


		PDFTable retPdfTable = this.getPdfTable(lasso, me);
		if(retPdfTable != null)
		{
			LassoTypeRef ref	 = new LassoTypeRef();
			String sData		 = retPdfTable.getTableAbsWidth();
			int iTblAbsWidth 	 = 0;

			if(sData != null)
				iTblAbsWidth = Integer.parseInt(sData);
			err  = lasso.typeAllocInteger(ref, (long)iTblAbsWidth);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method is modified to accept Text Objects instead of Text strings.
	 * @param lasso
	 * @param action
	 */

	public int myInsertFunc (LassoCall lasso, int action) throws Exception
	{
		LassoTypeRef me 	= new LassoTypeRef();
		LassoValue tagName	= new LassoValue();
		PDFTable pT 		= null;
		PDFText  pTxt 		= null;
		PDFImage pImg		= null;
		PDFBarCode pBcode   = null;

		int err = 0;
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling Add/Insert", err);

		lasso.getTagName(tagName );

		if(tagName.data().equalsIgnoreCase("Add") ||
			tagName.data().equalsIgnoreCase("Insert"))
		{

			String	strCols		= getParamValue(lasso,"-col");
			String	strRow 		= getParamValue(lasso,"-row");
			String	strColSpan	= getParamValue(lasso,"-colspan");
			String	strRowSpan	= getParamValue(lasso,"-rowspan");

			//code added 12-09-2002
			String  strVAlign   = getParamValue(lasso,"-VerticalAlignment");
			String  strHAlign   = getParamValue(lasso,"-HorizontalAlignment");

			//-Leading Parameter is removed in Lasso 7
			//String  strLeading  = getParamValue(lasso,"-Leading");

			String  strBColor   = getParamValue(lasso,"-BorderColor");
			String  strBWidth   = getParamValue(lasso,"-BorderWidth");

			String	strHeader	= "false";
			String	strNoWrap	= "false";

			IntValue bValue	= getBooleanValue(lasso,"-header");

			if(bValue.intValue() == 1)
				strHeader = "true";

			bValue	= getBooleanValue(lasso,"-nowrap");
			if(bValue.intValue() == 1)
				strNoWrap = "true";


			String	strXabsPos	= getParamValue(lasso,"-x");
			String	strYabsPos	= getParamValue(lasso,"-y");
			PDFTable retPdfTable = this.getPdfTable(lasso, me);

			LassoTypeRef outValue = new LassoTypeRef();

			err = lasso.getTagParam2(0, outValue );
			if(err == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Table"))
					{
						pT = this.getPdfTable(lasso, outValue);

						if(retPdfTable != null && pT != null)
						{
							if((err = retPdfTable.insert(pT, strCols, strRow, strColSpan, strRowSpan, strHeader, strNoWrap, strXabsPos, strYabsPos,strVAlign,strHAlign,strBColor,strBWidth)) == ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_Table:(Add) Error inserting a Table", err);
						}
					}

					else if((typeName.name()).equalsIgnoreCase("PDF_Text"))
					{
						pTxt = this.getPdfText(lasso, outValue);

						if(retPdfTable != null && pTxt != null)
						{
							if((err = retPdfTable.insert(pTxt, strCols, strRow, strColSpan, strRowSpan, strHeader, strNoWrap, strXabsPos, strYabsPos,strVAlign,strHAlign,strBColor,strBWidth)) == ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_Table:(Add) Error inserting a Text", err);
						}
					}

					else if((typeName.name()).equalsIgnoreCase("PDF_Image"))
					{
						pImg = this.getPdfImage(lasso, outValue);

						if(retPdfTable != null && pImg != null)
						{
							if((err = retPdfTable.insert(pImg, strCols, strRow, strColSpan, strRowSpan, strHeader, strNoWrap, strXabsPos, strYabsPos,strVAlign,strHAlign,strBColor,strBWidth)) == ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_Table:(Add) Error inserting a Image", err);
						}
					}
					else if((typeName.name()).equalsIgnoreCase("PDF_Barcode"))
					{
						pBcode = this.getPdfBarCode(lasso, outValue);

						if(retPdfTable != null && pBcode != null)
						{
							if((err = retPdfTable.insert(pBcode, strCols, strRow, strColSpan, strRowSpan, strHeader, strNoWrap, strXabsPos, strYabsPos,strVAlign,strHAlign,strBColor,strBWidth)) == ERR_NOERR)
								return err;
							else
								return setError(lasso, "PDF_Table:(Add) Error inserting a Barcode", err);
						}
					}
					else
					{
						String strText = null;	//incase Text needs to be inserted into the table.

						if((typeName.name()).equalsIgnoreCase("String"))
						{
							strText = getString(lasso);

							if(retPdfTable != null && strText != null)
							{
								if((err = retPdfTable.insert(strText, strCols, strRow, strColSpan, strRowSpan, strHeader, strNoWrap, strXabsPos, strYabsPos,strVAlign,strHAlign,strBColor,strBWidth)) == ERR_NOERR)
									return err;
								else
									return setError(lasso, "PDF_Table:(Add) Error inserting a Text", err);
							}
						}
					}
				}
			}
			return setError(lasso, "PDF_Table:(Add)Unknown Error", err);
		}
		return setError(lasso, "PDF_Table:(Add)Unknown Error", err);
	}


	/**
	 * This method returns the element at the position row, column
	 * @param lasso
	 * @param action
	 */

	public int myGetFunc (LassoCall lasso, int action)
	{

		int err 		= 0;
		int iCols 		= 0;
		int iRows 		= 0;


		LassoTypeRef me 	= new LassoTypeRef();
		LassoValue tagName	= new LassoValue();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling Function to return an Element at a particular position.", err);

		lasso.getTagName(tagName );

		if(tagName.data().equalsIgnoreCase("Get"))
		{

			String	strCols		= getParamValue(lasso,"-col");

			String	strRow 		= getParamValue(lasso,"-row");

			if(strCols != null)
				iCols = Integer.parseInt(strCols);
			if(strRow != null)
				iRows = Integer.parseInt(strRow);

			PDFTable retPdfTable = this.getPdfTable(lasso, me);
			if(retPdfTable != null )
			{
			//	if((err = retPdfTable.getTableElement(iRows, iCols)) == ERR_NOERR)
			//		return err; //return the element Object here.
			//	else
			//		return setError(lasso, "PDF_Table:(Get) Error getting a Element", err);
			}
			return setError(lasso, "PDF_Table:(Get)Unknown Error", err);
		}
		return setError(lasso, "PDF_Table:(Get)Unknown Error", err);
	}

	/**
	 * This method removes the element at the position row, column
	 * @param lasso
	 * @param action
	 */

	public int myRemoveFunc (LassoCall lasso, int action)
	{

		int err 		= 0;
		int iCols 		= 0;
		int iRows 		= 0;


		LassoTypeRef me = new LassoTypeRef();
		LassoValue tagName	= new LassoValue();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Table: Error while calling Remove", err);

		lasso.getTagName(tagName );

		if(tagName.data().equalsIgnoreCase("Get"))
		{

			String	strCols		= getParamValue(lasso,"-col");

			String	strRow 		= getParamValue(lasso,"-row");

			if(strCols != null)
				iCols = Integer.parseInt(strCols);
			if(strRow != null)
				iRows = Integer.parseInt(strRow);

			PDFTable retPdfTable = this.getPdfTable(lasso, me);
			if(retPdfTable != null )
			{
				if(iRows == 0)
					err = retPdfTable.remove(-1, iCols);
				else if(iCols == 0)
					err = retPdfTable.remove(iRows, -1);

				if(err == ERR_NOERR)
					return err;
				else
					return setError(lasso, "PDF_Table:(Remove) Error removing a row/coloumn", err);
			}
			return setError(lasso, "PDF_Table:(Remove)Unknown Error", err);
		}
		return setError(lasso, "PDF_Table:(Remove)Unknown Error", err);
	}

	//////////////////////////////////////////////////////////////////////////

	// Added Jul 03
	/**
	 * The PDF_Image tag is handled here.
	 * @param lasso
	 * @param action
	 */
	public int tagPDFImageFunc (LassoCall lasso, int action)
	{
		int err = 0;
		PDFImage pdfImg = new PDFImage();

		/* one of the following three must be used */
		//1. local path to an image file
		String sFile = getParamValue(lasso,"-file");
		//2. URL to an image
		String sUrl = getParamValue(lasso,"-url");
		//3. raw string of bits representing the image
		String sRaw = getParamValue(lasso,"-raw");


		//optionally get -height parameter
		String sHeight = getParamValue(lasso,"-height");

		//optionally get -width parameter
		String sWidth = getParamValue(lasso,"-width");

		//optionally get -proportional parameter
		IntValue iProportional = getBooleanValue(lasso,"-proportional");

		//optionally get -rotate parameter
		String sRotate= getParamValue(lasso,"-rotate");

		float fHeight = 0;
		if(sHeight != null && !sHeight.equals(""))
			fHeight = Float.parseFloat(sHeight);

		float fWidth = 0;
		if(sWidth != null && !sWidth.equals(""))
			fWidth = Float.parseFloat(sWidth);

		boolean bProportional = (iProportional.intValue()==1);

		float fRotate = 0;
		if(sRotate!= null && !sRotate.equals(""))
			fRotate = Float.parseFloat(sRotate);

		/* need to handle the other tags like fliphorizontal flip vertical as well */

		if(sFile==null)  sFile = "";
		if(sUrl==null)  sUrl = "";
		if(sRaw==null)  sRaw = "";

		//have to handle it differently
		try{
			if( sFile.equals("") && sUrl.equals("") && sRaw.equals("") )
				throw new Exception("image file location or raw image must be specified");

			sFile = internalToFullPath(lasso, sFile);

			pdfImg.CreateImage(sFile,sUrl,sRaw,fHeight,fWidth,bProportional,fRotate);
		}catch(Exception e){
			e.printStackTrace();
			return LassoErrors.UnknownError;
		}

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Image") == ERR_NOERR )
		{
			if((err = lasso.typeSetCustomJavaObject(me, pdfImg)) != ERR_NOERR)
				return setError(lasso, "PDF_Image:Error setting pdf_image object", err);

			lasso.returnTagValue(me);
		}
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////////////////////


	/**
	 * The PDF_Read tag is handled here.
	 * @param lasso
	 * @param action
	 */
	public int tagPDFReadFunc (LassoCall lasso, int action)
	{
		int err = 0;
		PDFRead pdfRead = new PDFRead();


		String sFile = getParamValue(lasso,"-File");

		if(sFile==null)  sFile = "";

		try{
			if( sFile.equals(""))
				throw new Exception("Location of the file to be opened for reading needs to be specified");

			sFile = internalToFullPath(lasso, sFile);
			pdfRead.CreatePDFRead(sFile);
		}catch(Exception e){
			e.printStackTrace();
			return LassoErrors.UnknownError;
		}
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Read") == ERR_NOERR )
		{
			//getPageCount
			LassoTypeRef getCount = new LassoTypeRef();
			err = lasso.typeAllocTag(getCount, "PDFTag", "myGetPageCountFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "PageCount", getCount );
			}

			//getPageSize
			LassoTypeRef getPageSize = new LassoTypeRef();
			err = lasso.typeAllocTag(getPageSize, "PDFTag", "myGetPageSizeFunc" );
			if(err == ERR_NOERR)
			{
				err = lasso.typeAddMember(me , "PageSize", getPageSize );
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfRead)) != ERR_NOERR)
				return setError(lasso, "PDF_Read:Error setting PDF_Read object", err);
			lasso.returnTagValue(me);

		} //end of member tags for PDF_Read.

		return 0;

	}

	/**
	 * This method returns the  page count value.
	 * @param lasso
	 * @param action
	 */

	public int myGetPageCountFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Read: Error while calling PageCount", err);

		PDFRead pR = this.getPdfRead(lasso, me);
		if(pR != null)
		{
			LassoTypeRef ref	= new LassoTypeRef();
			int pCount 			= pR.getCount();

			err  = lasso.typeAllocInteger(ref, pCount);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  page count value.
	 * @param lasso
	 * @param action
	 */

	public int myGetPageSizeFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Read: Error while calling PageSize", err);

		PDFRead pr = this.getPdfRead(lasso, me);
		if(pr != null)
		{
			String strIndex = getString(lasso);
			int index = 1;
			try{
				if(strIndex!=null && !strIndex.equals(""))
					index = Integer.parseInt(strIndex);
			}catch(NumberFormatException ignore){
			}
			LassoTypeRef ref	= new LassoTypeRef();
			Rectangle rect		= pr.getPageSize(index);

			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef[] initial	= new LassoTypeRef[4];
			LassoTypeRef l 		= new LassoTypeRef();
			LassoTypeRef t 		= new LassoTypeRef();
			LassoTypeRef w 		= new LassoTypeRef();
			LassoTypeRef h 		= new LassoTypeRef();

			lasso.typeAllocDecimal(l, (double)rect.left());
			lasso.typeAllocDecimal(t, (double)rect.top());
			lasso.typeAllocDecimal(w, (double)rect.width());
			lasso.typeAllocDecimal(h, (double)rect.height());

			initial[0]=l; initial[1]=t;	initial[2]=w; initial[3]=h;

			err = lasso.typeAllocArray(mArray, initial);

			if(err == ERR_NOERR)
				lasso.returnTagValue(mArray);
			else
				return err;
		}
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////



	// Added jul 03
	/**
	 * The PDF_List tag is handled here.
	 * @param lasso
	 * @param action
	 */
	public int tagPDFListFunc (LassoCall lasso, int action)
	{
		int err = 0;
		PDFList pdfList = new PDFList();

		// format
		String sFormat = getParamValue(lasso,"-format");
		// bullet
		String sBullet = getParamValue(lasso,"-bullet");
		//indent
		String sIndent = getParamValue(lasso,"-indent");

		//Font object
		Font pFont = getFontParam(lasso);

		//optionally get -align parameter
		String sAlign = getParamValue(lasso,"-align");

		//optionally get -color parameter
		String sColor = getParamValue(lasso,"-color");

		//optionally get -backgroundcolor parameter
		String sBackgroundColor= getParamValue(lasso,"-backgroundcolor");

		//optionally get -leading parameter
		String sLeading = getParamValue(lasso,"-leading");


		//optionally get -indentleft parameter
		String sIndentLeft = getParamValue(lasso,"-indentleft");

		//optionally get -indentright parameter
		String sIndentRight = getParamValue(lasso,"-indentright");

		if(sFormat==null || sFormat.equals(""))
			sFormat = "bullet";

		if(sBullet==null)
			sBullet = "";

		float fIndent = 0;
		if(sIndent!=null && !sIndent.equals(""))
			fIndent = Float.parseFloat(sIndent);

		if(sAlign==null || sAlign.equals(""))
			sAlign = "left";

		if(sBackgroundColor==null)
			sBackgroundColor = "";

		float fLeading = 0;
		if(sLeading!=null && !sLeading.equals(""))
			fLeading = Float.parseFloat(sLeading);

		float fIndentLeft = 0;
		if(sIndentLeft!=null && !sIndentLeft.equals(""))
			fIndentLeft = Float.parseFloat(sIndentLeft);

		float fIndentRight = 0;
		if(sIndentRight!=null && !sIndentRight.equals(""))
			fIndentRight = Float.parseFloat(sIndentRight);

		try{
			pdfList.CreateList(sFormat,sBullet,fIndent,pFont,sAlign,sColor,sBackgroundColor,
								fLeading, fIndentLeft,fIndentRight);
		}catch(Exception e){
			e.printStackTrace();
			return LassoErrors.UnknownError;
		}

		/**
		 *  PDF_List Tag
		 *	Handling the Accessors & the Member tags of PDF_List
		 */
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_List") == ERR_NOERR )
		{
			/**
		 	 * Handling the Add member tag of PDF_List tag.
		 	 * This method adds an object to the list.
		     */

			LassoTypeRef add = new LassoTypeRef();
			if (lasso.typeAllocTag(add, "PDFTag", "myListAddFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Add", add )) != ERR_NOERR )
				{
					return err;
				}
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfList)) != ERR_NOERR)
				return setError(lasso, "PDF_List:Error setting pdf_list object", err);

			lasso.returnTagValue(me);
		}

		return 0;
	}

	/**
	 * This method adds an object to the list.
	 * @param lasso
	 * @param action
	 */
	public int myListAddFunc(LassoCall lasso, int action)
	{
		int err 				= 0;
		LassoTypeRef me			= new LassoTypeRef();
		PDFList retPdfList		= null;
		PDFImage pdfImage		= null;
		PDFList pdfList			= null;
		PDFText pdfText			= null;
		PDFBarCode pdfBarCode	= null;
		PDFTable pdfTable		= null;

		if (lasso.getTagSelf(me) == ERR_NOERR )
		retPdfList = this.getPdfList(lasso, me);

		LassoTypeRef outValue = new LassoTypeRef();
		err = lasso.getTagParam2(0, outValue );

		if(err == ERR_NOERR)
		{
			LassoValue typeName = new LassoValue();
			if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
			{
				String objectTypeName = typeName.name();
				if(objectTypeName.equalsIgnoreCase("String"))
				{
					String strContent = getString(lasso);	//the String to be added to the list

					if(retPdfList!= null && strContent != null)
					{
						if((err = retPdfList.listAddString(strContent)) == ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding String", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Image"))//if object is an IMAGE
				{
					pdfImage =	this.getPdfImage(lasso, outValue);
					if(retPdfList!=null && pdfImage!=null)
					{
						String	sAlign = getParamValue(lasso,"-align");
						IntValue iWrap = getBooleanValue(lasso,"-wrap");
						boolean bWrap = (iWrap.intValue()==1);

						if( (err = retPdfList.listAddImage(pdfImage,sAlign,bWrap))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding Image", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_BarCode"))//if object is a BARCODE
				{
					pdfBarCode = this.getPdfBarCode(lasso, outValue);
					if(retPdfList!=null && pdfBarCode!=null)
					{
						if( (err = retPdfList.listAddBarCode(pdfBarCode))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding BarCode", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Table"))//if object is a TABLE
				{
					pdfTable = this.getPdfTable(lasso, outValue);
					if(retPdfList!=null && pdfTable!=null)
					{
						if( (err = retPdfList.listAddTable(pdfTable))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding Table", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_List"))//if object is a LIST
				{
					pdfList =	this.getPdfList(lasso, outValue);
					if(retPdfList!=null && pdfList!=null)
					{
						if( (err = retPdfList.listAddList(pdfList))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding List", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Text"))//if object is a TEXT
				{
					pdfText =	this.getPdfText(lasso, outValue);
					if(retPdfList!=null && pdfText!=null)
					{
						if( (err = retPdfList.listAddText(pdfText))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_LIST:(Add) Error adding Text", err);
					}
				}
			}
		}

		return setError(lasso, "PDF_LIST:(Add)Unknown Error", err);

	}

	//////////////////////////////////////////////////////////////////////////

	// Added Aug 03
	/**
	 * The PDF_Text tag is handled here.
	 * @param lasso
	 * @param action
	 */
	public int tagPDFTextFunc (LassoCall lasso, int action)
	{
		int err = 0;
		PDFText pdfText = new PDFText();

		String sContent = null;

		// type
		String sType = getParamValue(lasso,"-type");

		//Font object
		Font pFont = getFontParam(lasso);
		//optionally get -color parameter : hexString
		String sColor = getParamValue(lasso,"-color");
		//optionally get -backgroundcolor parameter
		String sBackgroundColor= getParamValue(lasso,"-backgroundcolor");
		// underLine : boolean
		IntValue iUnderLine = getBooleanValue(lasso,"-underline");
		//strikethru : boolean
		IntValue iStrikeThru = getBooleanValue(lasso,"-strikethru");
		//textrise : decimal
		String sTextRise = getParamValue(lasso,"-textrise");
		//anchor : URL String
		String sAnchor = getParamValue(lasso,"-anchor");
		//name : String
		String sName = getParamValue(lasso,"-name");
		//Goto : String
		String sGoTo = getParamValue(lasso,"-goto");
		//file : String
		String sFile = getParamValue(lasso,"-file");

		//OPTIONALLY GET -LEADING PARAMETER FOR PHRASE/PARAGRAPH
		String sLeading = getParamValue(lasso,"-leading");

		/* FOLLOWING THREE ARE FOR PARAGRAPH */
		//optionally get -align parameter
		String sAlign = getParamValue(lasso,"-align");
		//optionally get -indentleft parameter
		String sIndentLeft = getParamValue(lasso,"-indentleft");
		//optionally get -indentright parameter
		String sIndentRight = getParamValue(lasso,"-indentright");
		
		if(sType==null || sType.equals("") ||
			!(sType.equalsIgnoreCase("chunk") || sType.equalsIgnoreCase("phrase")) )
		{
			sType = "paragraph";
		}

		float fTextRise = 0;
		if(sTextRise!=null && !sTextRise.equals(""))
			fTextRise = Float.parseFloat(sTextRise);

		if(sAlign==null || sAlign.equals(""))
			sAlign = "left";

		if(sBackgroundColor==null)
			sBackgroundColor = "";

		float fLeading = 0;
		if(sLeading!=null && !sLeading.equals(""))
			fLeading = Float.parseFloat(sLeading);

		float fIndentLeft = 0;
		if(sIndentLeft!=null && !sIndentLeft.equals(""))
			fIndentLeft = Float.parseFloat(sIndentLeft);

		float fIndentRight = 0;
		if(sIndentRight!=null && !sIndentRight.equals(""))
			fIndentRight = Float.parseFloat(sIndentRight);

		boolean bUnderLine = (iUnderLine.intValue()==1);

		boolean bStrikeThru = (iStrikeThru.intValue()==1);

		try{
			LassoTypeRef outValue = new LassoTypeRef();
			if(lasso.getTagParam2(0, outValue ) == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					String objectTypeName = typeName.name();

					if(objectTypeName.equalsIgnoreCase("String"))
						sContent = getString(lasso);
				}
			}

			pdfText.CreateText(sContent,sType,pFont,sColor,sBackgroundColor,bUnderLine, bStrikeThru,
								fTextRise,sAnchor,sName,sGoTo,sFile,fLeading,sAlign,
								fIndentLeft,fIndentRight);
		}catch(Exception e){
			e.printStackTrace();
			return LassoErrors.UnknownError;
		}

		/**
		 *  PDF_Text Tag
		 *	Handling the Accessors & the Member tags of PDF_Text
		 */
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Text") == ERR_NOERR )
		{
			/**
		 	 * Handling the Add member tag of PDF_Text tag.
		 	 * This method adds an object.
		     */

			LassoTypeRef add = new LassoTypeRef();
			if (lasso.typeAllocTag(add, "PDFTag", "myTextAddFunc" ) == ERR_NOERR)
			{
				if ((err = lasso.typeAddMember(me , "Add", add )) != ERR_NOERR )
				{
					return err;
				}
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfText)) != ERR_NOERR)
				return setError(lasso, "PDF_Text:Error setting pdf_text object", err);

			lasso.returnTagValue(me);
		}

		return 0;
	}

	// Added Aug 03
	/**
	 * This method adds an object to PDF_Text.
	 * @param lasso
	 * @param action
	 */
	public int myTextAddFunc(LassoCall lasso, int action)
	{
		int err 				= 0;
		LassoTypeRef me			= new LassoTypeRef();
		PDFText retPdfText		= null;
		PDFImage pdfImage		= null;
		PDFBarCode pdfBarCode	= null;
		PDFTable pdfTable		= null;
		PDFText pdfText			= null;

		if (lasso.getTagSelf(me) == ERR_NOERR )
		retPdfText = this.getPdfText(lasso, me);

		LassoTypeRef outValue = new LassoTypeRef();
		err = lasso.getTagParam2(0, outValue );

		if(err == ERR_NOERR)
		{
			LassoValue typeName = new LassoValue();
			if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
			{
				String objectTypeName = typeName.name();

				if(objectTypeName.equalsIgnoreCase("String"))
				{
					String strContent = getString(lasso);	//the String to be added to the list

					if(retPdfText!= null && strContent != null)
					{
						if((err = retPdfText.textAddString(strContent)) == ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_TEXT:(Add) Error adding String", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Image"))//if object is an IMAGE
				{
					pdfImage =	this.getPdfImage(lasso, outValue);

					String	sAlign = getParamValue(lasso,"-align");
					IntValue iWrap = getBooleanValue(lasso,"-wrap");
					boolean bWrap = (iWrap.intValue()==1);

					if(retPdfText!= null && pdfImage!=null)
					{
						if( (err = retPdfText.textAddImage(pdfImage,sAlign,bWrap))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_TEXT:(Add) Error adding Image", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Barcode"))//if object is a BARCODE
				{
					pdfBarCode =	this.getPdfBarCode(lasso, outValue);

					String	sAlign = getParamValue(lasso,"-align");
					IntValue iWrap = getBooleanValue(lasso,"-wrap");
					boolean bWrap = (iWrap.intValue()==1);

					if(retPdfText!= null && pdfBarCode!=null)
					{
						if( (err = retPdfText.textAddBarCode(pdfBarCode,sAlign,bWrap))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_TEXT:(Add) Error adding BarCode", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Table"))//if object is a TABLE
				{
					pdfTable =	this.getPdfTable(lasso, outValue);

					String	sAlign = getParamValue(lasso,"-align");

					if(retPdfText!= null && pdfTable!=null)
					{
						if( (err = retPdfText.textAddTable(pdfTable,sAlign))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_TEXT:(Add) Error adding Table", err);
					}
				}
				else if((typeName.name()).equalsIgnoreCase("PDF_Text"))//if object is a TEXT
				{
					pdfText = this.getPdfText(lasso, outValue);

					if(retPdfText!= null && pdfText!=null)
					{
						if( (err = retPdfText.textAddText(pdfText))==ERR_NOERR)
							return err;
						else
							return setError(lasso, "PDF_TEXT:(Add) Error adding Text", err);
					}
				}
			}
		}

		return setError(lasso, "PDF_TEXT:(Add)Unknown Error", err);

	}


	//////////////////////////////////////////////////////////////////////////

	public int tagPDFBarcodeFunc (LassoCall lasso, int action)
	{
		int err = 1;

		PDFBarCode pdfbarcode =	null;
		pdfbarcode 			= 	new PDFBarCode();

		//get the -file parameter.
		String filename	= getParamValue(lasso,"-file");
		if(filename != null)
			filename = internalToFullPath(lasso, filename);

		//get the -type parameter.
		String strType	= getParamValue(lasso,"-type");

		//optionally get -color parameter
		String strColor	= getParamValue(lasso,"-color");

		//optionally get -usedate parameter
		String strCode	= getParamValue(lasso,"-code");

		//optionally get -supplemental  parameter
		String strSupplemental	= getParamValue(lasso,"-supplemental");
		float fSuplemental = -1;
		if(strSupplemental != null)
		{

			fSuplemental = Float.parseFloat(strSupplemental);

		}

		//optionally get -generateChecksum parameter
//		String sGenerateChecksum = getParamValue(lasso,"-generateChecksum");


		IntValue bValue = getBooleanValue(lasso,"-generateChecksum");
		boolean bGenerateChecksum = false;
		if(bValue.intValue() == 1)
			bGenerateChecksum = true;

		//optionally get -showCode39StartStop parameter
		bValue = getBooleanValue(lasso,"-showCode39StartStop");

		boolean bshowCode39StartStop = true;

		if(bValue.intValue() == 0)
			bshowCode39StartStop = false;

		//optionally get -showEANGaurdBars parameter
		bValue = getBooleanValue(lasso,"-showEANGuardBars");

		String sBHeight = getParamValue(lasso,"-barheight");
		String sBWidth = getParamValue(lasso,"-barwidth");
		String sBaseLine = getParamValue(lasso,"-baseline");
		IntValue iShowCheckSum = getBooleanValue(lasso,"-showchecksum");
		Font pFont = getFontParam(lasso);
		String sBarMultiplier = getParamValue(lasso,"-barmultiplier");
		String sTextSize = getParamValue(lasso,"-textsize");
		String sTextAlignment = getParamValue(lasso,"-textalignment");


		boolean bshowEANGaurdBars = true;

		if(bValue.intValue() == 0)
			bshowEANGaurdBars = false;

		float fBarHeight = 0;
		if(sBHeight!=null && !sBHeight.equals(""))
			fBarHeight = Float.parseFloat(sBHeight);

		float fBarWidth = 0;
		if(sBWidth!=null && !sBWidth.equals(""))
			fBarWidth = Float.parseFloat(sBWidth);

		float fBaseLine = 0;
		if(sBaseLine!=null && !sBaseLine.equals(""))
			fBaseLine = Float.parseFloat(sBaseLine);

		boolean bShowCheckSum = (iShowCheckSum.intValue()==1);

		float fBarMultiplier = 0;
		if(sBarMultiplier!=null && !sBarMultiplier.equals(""))
			fBarMultiplier = Float.parseFloat(sBarMultiplier);

		float fTextSize = 0;
		if(sTextSize!=null && !sTextSize.equals(""))
			fTextSize = Float.parseFloat(sTextSize);
		if(sTextAlignment==null)
			sTextAlignment = "";

	 	pdfbarcode.createBarCode(filename, strType, strColor, strCode, fSuplemental,
	 						bGenerateChecksum, bshowCode39StartStop, bshowEANGaurdBars,
	 						fBarHeight, fBarWidth,fBaseLine, bShowCheckSum,pFont,
							fBarMultiplier,fTextSize,sTextAlignment);
		/**
		 *  PDF_Doc Tag
		 *	Handling the Accessors & the Member tags of PDF_Doc
		 */

		LassoTypeRef me = new LassoTypeRef();
		if (lasso.typeAllocCustom(me, "PDF_Barcode") == ERR_NOERR )
		{

			/**
		 	 * Handling the SetBarHeight accessor method of PDF_BarCode tag.
		 	 * This method sets the  height of the barcode.
		     */
			LassoTypeRef setbarheight = new LassoTypeRef();
			if (lasso.typeAllocTag(setbarheight,"PDFTag", "mySetBarHeightFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetBarHeight", setbarheight ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetBarWidth accessor method of PDF_BarCode tag.
		 	 * This method sets the height of the barcode.
		     */
			LassoTypeRef setbarwidth = new LassoTypeRef();
			if (lasso.typeAllocTag(setbarwidth,"PDFTag", "mySetBarWidthFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetBarWidth", setbarwidth ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetBaseLine accessor method of PDF_BarCode tag.
		 	 * This method sets the baseline of the barcode.
		     */
			LassoTypeRef setbaseline = new LassoTypeRef();
			if (lasso.typeAllocTag(setbaseline,"PDFTag", "mySetBaseLineFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetBaseline", setbaseline) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}


			/**
		 	 * Handling the SetShowCheckSum accessor method of PDF_BarCode tag.
		 	 * This method sets the ShowCheckSum of the barcode.
		     */
			LassoTypeRef setshowchecksum = new LassoTypeRef();
			if (lasso.typeAllocTag(setshowchecksum,"PDFTag", "mySetShowCheckSumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetShowChecksum", setshowchecksum) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetCode accessor method of PDF_BarCode tag.
		 	 * This method sets the Code of the barcode.
		     */
			LassoTypeRef setcode = new LassoTypeRef();
			if (lasso.typeAllocTag(setcode,"PDFTag", "mySetCodeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetCode", setcode) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetFont accessor method of PDF_BarCode tag.
		 	 * This method sets the Font of the barcode.
		     */
			LassoTypeRef setfont = new LassoTypeRef();
			if (lasso.typeAllocTag(setfont,"PDFTag", "mySetBCFontFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetFont", setfont) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetGenerateChecksum accessor method of PDF_BarCode tag.
		 	 * This method sets the GenerateChecksum of the barcode.
		     */
			LassoTypeRef setgeneratechecksum = new LassoTypeRef();
			if (lasso.typeAllocTag(setgeneratechecksum,"PDFTag", "mySetGenerateChecksumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetGenerateChecksum", setgeneratechecksum) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetBarMultiplier accessor method of PDF_BarCode tag.
		 	 * This method sets the BarMultiplier of the barcode.
		     */
			LassoTypeRef setbarmultiplier = new LassoTypeRef();
			if (lasso.typeAllocTag(setbarmultiplier,"PDFTag", "mySetBarMultiplierFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetBarMultiplier", setbarmultiplier) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetTextSize accessor method of PDF_BarCode tag.
		 	 * This method sets the TextSize of the barcode.
		     */
			LassoTypeRef settextsize = new LassoTypeRef();
			if (lasso.typeAllocTag(settextsize,"PDFTag", "mySetTextSizeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetTextSize", settextsize) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the SetTextAlignment accessor method of PDF_BarCode tag.
		 	 * This method sets the TextAlignment of the barcode.
		     */
			LassoTypeRef settextalignment = new LassoTypeRef();
			if (lasso.typeAllocTag(settextalignment,"PDFTag", "mySetTextAlignmentFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "SetTextAlignment", settextalignment) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetSize accessor method of PDF_Barcode tag.
		 	 * This method returns the size of the barcode.
		     */

			LassoTypeRef getsize = new LassoTypeRef();
			if (lasso.typeAllocTag(getsize,"PDFTag", "myGetBCSizeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetSize", getsize ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetBarHeight accessor method of PDF_Barcode tag.
		 	 * This method returns the BarHeight of the barcode.
		     */

			LassoTypeRef getbarheight = new LassoTypeRef();
			if (lasso.typeAllocTag(getbarheight,"PDFTag", "myGetBarHeightFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetBarHeight", getbarheight ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetBarHeight accessor method of PDF_Barcode tag.
		 	 * This method returns the height of the barcode.
		     */

			LassoTypeRef getbarwidth = new LassoTypeRef();
			if (lasso.typeAllocTag(getbarwidth,"PDFTag", "myGetBarHeightFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetBarWidth", getbarwidth ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetBaseline accessor method of PDF_Barcode tag.
		 	 * This method returns the baseline of the barcode.
		     */

			LassoTypeRef getbaseline = new LassoTypeRef();
			if (lasso.typeAllocTag(getbaseline,"PDFTag", "myGetBaselineFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetBaseline", getbaseline ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetCode accessor method of PDF_Barcode tag.
		 	 * This method returns the baseline of the barcode.
		     */

			LassoTypeRef getcode = new LassoTypeRef();
			if (lasso.typeAllocTag(getcode,"PDFTag", "myGetBCCodeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetCode", getcode ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetType accessor method of PDF_Barcode tag.
		 	 * This method returns the type of the barcode.
		     */

			LassoTypeRef gettype = new LassoTypeRef();
			if (lasso.typeAllocTag(gettype,"PDFTag", "myGetTypeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetType", gettype ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetFont accessor method of PDF_Barcode tag.
		 	 * This method returns the Font of the barcode.
		     */

			LassoTypeRef getfont = new LassoTypeRef();
			if (lasso.typeAllocTag(getfont,"PDFTag", "myGetFontFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetFont", getfont ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetFont accessor method of PDF_Barcode tag.
		 	 * This method returns the Font of the barcode.
		     */

			LassoTypeRef getbarmultiplier = new LassoTypeRef();
			if (lasso.typeAllocTag(getbarmultiplier,"PDFTag", "myGetBarMultiplierFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetBarMultiplier", getbarmultiplier ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetTextSize accessor method of PDF_Barcode tag.
		 	 * This method returns the TextSize of the barcode.
		     */

			LassoTypeRef gettextsize = new LassoTypeRef();
			if (lasso.typeAllocTag(gettextsize,"PDFTag", "myGetTextSizeFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetTextSize", gettextsize ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GetTextAlignment accessor method of PDF_Barcode tag.
		 	 * This method returns the TextAlignment of the barcode.
		     */

			LassoTypeRef gettextalignment = new LassoTypeRef();
			if (lasso.typeAllocTag(gettextalignment,"PDFTag", "myGetTextAlignmentFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GetTextAlignment", gettextalignment ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the ShowChecksum accessor method of PDF_Barcode tag.
		 	 * This method returns the ShowChecksum of the barcode.
		     */

			LassoTypeRef showchecksum = new LassoTypeRef();
			if (lasso.typeAllocTag(showchecksum,"PDFTag", "myShowChecksumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "ShowChecksum", showchecksum ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the GenerateChecksum accessor method of PDF_Barcode tag.
		 	 * This method returns the Checksum of the barcode.
		     */

			LassoTypeRef generatechecksum = new LassoTypeRef();
			if (lasso.typeAllocTag(generatechecksum,"PDFTag", "myGenerateChecksumFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "GenerateChecksum", generatechecksum ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the ShowEANGuardBars accessor method of PDF_Barcode tag.
		 	 * This method returns the EANGuardBars of the barcode.
		     */

			LassoTypeRef showeanguardbars = new LassoTypeRef();
			if (lasso.typeAllocTag(showeanguardbars,"PDFTag", "myShowEANGuardBarsFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "ShowEANGuardBars", showeanguardbars ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			/**
		 	 * Handling the ShowCode39StartStop accessor method of PDF_Barcode tag.
		 	 * This method returns the Code39StartStop value of the barcode.
		     */

			LassoTypeRef showcode39startstop = new LassoTypeRef();
			if (lasso.typeAllocTag(showcode39startstop,"PDFTag", "myShowCode39StartStopFunc" ) == ERR_NOERR)
			{
				if (lasso.typeAddMember(me , "ShowCode39StartStop", showcode39startstop ) != ERR_NOERR )
				{
					return LassoErrors.UnknownError;
				}
			}

			if((err = lasso.typeSetCustomJavaObject(me, pdfbarcode)) != ERR_NOERR)
			{
				return setError(lasso, "PDF_Barcode:Error calling showcode39startstop ", err);
			}


			lasso.returnTagValue(me);

		} //end for MEMBER TAGS.



		return 0;
	}



	/**
	 * This method sets the barcode height
	 * @param lasso
	 * @param action
	 */

	public int mySetBarHeightFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling SetBarHeight", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strHeight = getString(lasso);

			if(strHeight != null)
			{
				float fHeight = Float.parseFloat(strHeight);
				pBC.setBarHeight(fHeight);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode width
	 * @param lasso
	 * @param action
	 */

	public int mySetBarWidthFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling SetBarWidth", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strWidth = getString(lasso);

			if(strWidth != null)
			{
				float fWidth = Float.parseFloat(strWidth);
				pBC.setBarWidth(fWidth);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode baseline
	 * @param lasso
	 * @param action
	 */

	public int mySetBaseLineFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling SetBaseLine", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strBaseLine = getString(lasso);

			if(strBaseLine != null)
			{
				float fBaseLine = Float.parseFloat(strBaseLine);
				pBC.setBaseline(fBaseLine);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode showchecksum
	 * @param lasso
	 * @param action
	 */

	public int mySetShowCheckSumFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setShowChecksum", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strShowCheckSum = getString(lasso);

			if(strShowCheckSum != null)
			{
				if(strShowCheckSum.equalsIgnoreCase("True"))
					pBC.setShowChecksum(true);
				else if(strShowCheckSum.equalsIgnoreCase("False"))
					pBC.setShowChecksum(false);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode showchecksum
	 * @param lasso
	 * @param action
	 */

	public int mySetCodeFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setCode", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strCode = getString(lasso);

			if(strCode != null)
			{
				pBC.setCode(strCode);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode showchecksum
	 * @param lasso
	 * @param action
	 */

	public int mySetBCFontFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling set BarCodeFont", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			pBC.setFont(getFontParam2(lasso));
		}
		return 0;
	}

	/**
	 * This method sets the barcode generatechecksum
	 * @param lasso
	 * @param action
	 */

	public int mySetGenerateChecksumFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setGenerateChecksum", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strGenerateCheckSum = getString(lasso);

			if(strGenerateCheckSum != null)
			{
				if(strGenerateCheckSum.equalsIgnoreCase("True"))
					pBC.setGenerateChecksum(true);
				else if(strGenerateCheckSum.equalsIgnoreCase("False")	)
					pBC.setGenerateChecksum(false);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode setBarMultiplier
	 * @param lasso
	 * @param action
	 */

	public int mySetBarMultiplierFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setBarMultiplier", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strBarMultiplier = getString(lasso);

			if(strBarMultiplier != null)
			{
				float fBarMultiplier = Float.parseFloat(strBarMultiplier);
				pBC.setBarMultiplier(fBarMultiplier);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode setBarMultiplier
	 * @param lasso
	 * @param action
	 */

	public int mySetTextSizeFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setTextSize", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strTextSize = getString(lasso);

			if(strTextSize != null)
			{
				float fTextSize = Float.parseFloat(strTextSize);
				pBC.setTextSize(fTextSize);
			}
			else
				return LassoErrors.InvalidParameter;
		}
		return 0;
	}

	/**
	 * This method sets the barcode setBarMultiplier
	 * @param lasso
	 * @param action
	 */

	public int mySetTextAlignmentFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling setTextAlignment", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			String strTextAlign = getString(lasso);

			if(strTextAlign != null)
			{
				pBC.setTextAlignment(strTextAlign);
			}
			else
				return LassoErrors.InvalidParameter;
		}

		return 0;
	}

	/**
	 * This method returns the  font size value.
	 * @param lasso
	 * @param action
	 */

	public int myGetBCSizeFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling Get BarcodeSize", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref	= new LassoTypeRef();
			Rectangle rect		= pBC.getSize();

			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef[] initial	= new LassoTypeRef[4];
			LassoTypeRef l 		= new LassoTypeRef();
			LassoTypeRef t 		= new LassoTypeRef();
			LassoTypeRef w 		= new LassoTypeRef();
			LassoTypeRef h 		= new LassoTypeRef();

			lasso.typeAllocDecimal(l, (double)rect.left());
			lasso.typeAllocDecimal(t, (double)rect.top());
			lasso.typeAllocDecimal(w, (double)rect.width());
			lasso.typeAllocDecimal(h, (double)rect.height());

			initial[0]=l; initial[1]=t;	initial[2]=w; initial[3]=h;

			err = lasso.typeAllocArray(mArray, initial);

			if(err == ERR_NOERR)
				lasso.returnTagValue(mArray);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the bar height value.
	 * @param lasso
	 * @param action
	 */

	public int myGetBarHeightFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling GetBarHeight", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref		= new LassoTypeRef();
			float fHeight 			= pBC.getBarHeight();

			err  = lasso.typeAllocDecimal(ref, (long) fHeight);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method returns the bar Width value.
	 * @param lasso
	 * @param action
	 */

	public int myGetBarWidthFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling GetBarWidth", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref		= new LassoTypeRef();
			float fWidth	 			= pBC.getBarWidth();

			err  = lasso.typeAllocDecimal(ref, (long) fWidth);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the bar BaseLine value.
	 * @param lasso
	 * @param action
	 */

	public int myGetBaselineFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling GetBaseLine", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref		= new LassoTypeRef();
			float fBaseline	 			= pBC.getBaseline();

			err  = lasso.typeAllocDecimal(ref, (float) fBaseline);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method returns the  Barcode code.
	 * @param lasso
	 * @param action
	 */
	public int myGetBCCodeFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling Get BarcodeCode", err);


		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			String sCode = pBC.getCode();
			if(sCode == null) sCode = "No Code found";
			err  = lasso.typeAllocString(ref, sCode);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  Barcode Type
	 * @param lasso
	 * @param action
	 */
	public int myGetTypeFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling Get BarCodeType", err);


		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			int iType = pBC.getType();
			err  = lasso.typeAllocInteger(ref, iType);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the  Barcode Font
	 * @param lasso
	 * @param action
	 */
	public int myGetFontFunc (LassoCall lasso, int action)
	{

		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling GetFont", err);


		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref = new LassoTypeRef();

			BaseFont bf = pBC.getFont();
			Color clr = pBC.getBarTextColor();
			float ts = pBC.getTextSize();

			LassoTypeRef outValue = new LassoTypeRef();

			if (lasso.typeAlloc("PDF_Font", null, outValue) == ERR_NOERR)
			{
				LassoValue typeName = new LassoValue();
				if( lasso.typeGetName(outValue,typeName) == ERR_NOERR)
				{
					if((typeName.name()).equalsIgnoreCase("PDF_Font"))
					{
						PDFFont pF = this.getPdfFont(lasso, outValue);
						if(pF!=null)
						{
							pF.resetFont(bf, clr, ts);
							err = lasso.returnTagValue(outValue);
							return err;
						}
					}
				}
			}
		}

		return 0;
	}


	/**
	 * This method returns the bar BaseLine value.
	 * @param lasso
	 * @param action
	 */

	public int myGetBarMultiplierFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling getBarMultiplier", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref		= new LassoTypeRef();
			float fBarMultiplier	 	= pBC.getBarMultiplier();

			err  = lasso.typeAllocDecimal(ref, (float) fBarMultiplier);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}

	/**
	 * This method returns the bar TextSize value.
	 * @param lasso
	 * @param action
	 */

	public int myGetTextSizeFunc (LassoCall lasso, int action)
	{
		int err 		= 0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling getTextSize", err);

		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref		= new LassoTypeRef();
			float fTextSize	 	= pBC.getTextSize();

			err  = lasso.typeAllocDecimal(ref, (float) fTextSize);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method returns the  Barcode Type
	 * @param lasso
	 * @param action
	 */
	public int myGetTextAlignmentFunc (LassoCall lasso, int action)
	{
		int err = 0;
		LassoTypeRef me = new LassoTypeRef();
		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Barcode: Error while calling getTextAlignment", err);


		PDFBarCode pBC = this.getPdfBarCode(lasso, me);
		if(pBC != null)
		{
			LassoTypeRef ref = new LassoTypeRef();
			int iAlignment = pBC.getTextAlignment();

			err  = lasso.typeAllocInteger(ref, iAlignment);
			if(err == ERR_NOERR)
				err = lasso.returnTagValue(ref);
			else
				return err;
		}

		return 0;
	}


	/**
	 * This method returns the show Checksum
	 * @param lasso
	 * @param action
	 */

	public int myShowChecksumFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFBarCode retPdfBC	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfBC = this.getPdfBarCode(lasso, me);

		if(retPdfBC != null)
		{

			boolean bShowCheckSum = retPdfBC.showChecksum();

			LassoTypeRef chkSum		= new LassoTypeRef();
			LassoTypeRef boolChkSum 		= new LassoTypeRef();

			lasso.typeAllocBoolean(boolChkSum,bShowCheckSum );
			lasso.returnTagValue(boolChkSum);
			return err;
		}

		return err;
	}

	/**
	 * This method returns the show Checksum
	 * @param lasso
	 * @param action
	 */

	public int myGenerateChecksumFunc (LassoCall lasso, int action)
	{
		LassoTypeRef me		= new LassoTypeRef();
		PDFBarCode retPdfBC	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfBC = this.getPdfBarCode(lasso, me);

		if(retPdfBC != null)
		{

			boolean bGenerateCheckSum = retPdfBC.generateChecksum();

			LassoTypeRef chkSum		= new LassoTypeRef();
			LassoTypeRef boolChkSum 		= new LassoTypeRef();

			lasso.typeAllocBoolean(boolChkSum,bGenerateCheckSum );
			lasso.returnTagValue(boolChkSum);
			return err;
		}

		return err;

	}

	/**
	 * This method returns the show Checksum
	 * @param lasso
	 * @param action
	 */

	public int myShowEANGuardBarsFunc (LassoCall lasso, int action)
	{

		LassoTypeRef me		= new LassoTypeRef();
		PDFBarCode retPdfBC	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfBC = this.getPdfBarCode(lasso, me);

		if(retPdfBC != null)
		{

			boolean bShowEannGaurd = retPdfBC.showEANGuardBars();

			LassoTypeRef chkSum			= new LassoTypeRef();
			LassoTypeRef boolChkSum 	= new LassoTypeRef();

			lasso.typeAllocBoolean(boolChkSum,bShowEannGaurd );
			lasso.returnTagValue(boolChkSum);
			return err;
		}

		return err;

	}

	/**
	 * This method returns the show Checksum
	 * @param lasso
	 * @param action
	 */

	public int myShowCode39StartStopFunc (LassoCall lasso, int action)
	{

		LassoTypeRef me		= new LassoTypeRef();
		PDFBarCode retPdfBC	= null;
		int err				= 0;

		if (lasso.getTagSelf(me) == ERR_NOERR )
			retPdfBC = this.getPdfBarCode(lasso, me);

		if(retPdfBC != null)
		{

			boolean bShowEannGaurd = retPdfBC.showCode39StartStop();

			LassoTypeRef chkSum			= new LassoTypeRef();
			LassoTypeRef boolChkSum 	= new LassoTypeRef();

			lasso.typeAllocBoolean(boolChkSum,bShowEannGaurd );
			lasso.returnTagValue(boolChkSum);
			return err;
		}

		return err;
	}



// Acro-Form Stuff July-13, 2002

/*
 * @param		llx			lower left x
 * @param		lly			lower left y
 * @param		urx			upper right x
 * @param		ury			upper right y


 fontSize, 171, 440, 300, 490);

 */

	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddTextFieldFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddTextField", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextField: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextField: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextField: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextField: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextField: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
				strValue = "";

			retPdfDoc.addTextField(strName, strValue, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddPasswordFieldFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddPasswordField", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddPasswordField: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddPasswordField: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddPasswordField: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddPasswordField: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddPasswordField: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
				strValue = "";

			retPdfDoc.addPasswordField(strName, strValue, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddTextAreaFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddTextArea", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextArea: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextArea: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextArea: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextArea: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddTextArea: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
				strValue = "";

			retPdfDoc.addTextAreaField(strName, strValue, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddCheckBoxFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddCheckBox", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
			{
				lasso.setResultMessage("PDF_Doc->AddCheckBox: -value parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			IntValue bValue = getBooleanValue(lasso,"-checked");
			boolean bChecked = false;
			if(bValue.intValue() == 1)
				bChecked = true;

			retPdfDoc.addCheckBoxField(strName, strValue, bChecked, llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddRadioButtonFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddRadioButton", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strGroup = getParamValue(lasso,"-group");
			if(strGroup == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -group parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioButton: -value parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			retPdfDoc.addRadButton(strGroup, strValue, llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddRadioGroupFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddRadioGroup", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{
			String strGroup = getParamValue(lasso,"-name");
			if(strGroup == null)
			{
				lasso.setResultMessage("PDF_Doc->AddRadioGroup: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			retPdfDoc.addRadGroup(strGroup);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddComboBoxFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddComboBox", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strDefault = getParamValue(lasso,"-default");

			IntValue bValue = getBooleanValue(lasso,"-editable");
			boolean bEditable = false;
			if(bValue.intValue() == 1)
				bEditable = true;


			LassoTypeRef res 	= new LassoTypeRef();
			LassoValue typeName = new LassoValue();
			int arrSize 		= 0;
			String[] options1 = null;
			String[] options2 = null;
			String[][] 	 options = null;

			if( lasso.findTagParam2("-values", res) == ERR_NOERR &&
				lasso.typeGetName(res,typeName) == ERR_NOERR)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();

					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						arrSize = count.intValue();
						options1	= new String[arrSize];
						options2	= new String[arrSize];

						for ( int k = 0; k < arrSize; k++ )
						{
							LassoTypeRef element= new LassoTypeRef();

							lasso.arrayGetElement(res, k, element);
							LassoValue typeName2 = new LassoValue();
							lasso.typeGetName(element,typeName2);

							if((typeName2.name()).equalsIgnoreCase("string"))
							{
								LassoValue outStringValue = new LassoValue();
								lasso.typeGetString(element, outStringValue);

								options1[k] = outStringValue.name();
								options2[k] = outStringValue.name();
							}
							else if((typeName2.name()).equalsIgnoreCase("pair"))
							{
								LassoTypeRef outFirst 	= new LassoTypeRef();
								LassoTypeRef outSecond 	= new LassoTypeRef();
								LassoValue outStringValue1 = new LassoValue();
								LassoValue outStringValue2 = new LassoValue();

								lasso.pairGetFirst( element, outFirst );
								lasso.typeGetString(outFirst, outStringValue1);
								lasso.pairGetSecond( element, outSecond );
								lasso.typeGetString(outSecond, outStringValue2);

								options1[k] = outStringValue1.name();
								options2[k] = outStringValue2.name();
							}
						}
					}

					if(arrSize > 0)
					{
						options = new String[arrSize][2];

						for ( int l = 0; l < arrSize; l++ )
						{
							options[l][0] = options1[l];
							options[l][1] = options2[l];
						}
					}
				}

			}


			if(arrSize < 1)
			{
				lasso.setResultMessage("PDF_Doc->AddComboBox: -values parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			retPdfDoc.addComboBoxField(strName, options, strDefault, bEditable, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddSelectListFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddSelectList", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strDefault = getParamValue(lasso,"-default");


			LassoTypeRef res 	= new LassoTypeRef();
			LassoValue typeName = new LassoValue();
			int arrSize 		= 0;
			String[] options1 = null;
			String[] options2 = null;
			String[][] 	 options = null;

			if( lasso.findTagParam2("-values", res) == ERR_NOERR &&
				lasso.typeGetName(res,typeName) == ERR_NOERR)
			{
				if((typeName.name()).equalsIgnoreCase("array"))
				{
					IntValue count = new IntValue();

					if ( lasso.arrayGetSize(res, count ) == ERR_NOERR )
					{
						arrSize = count.intValue();
						options1	= new String[arrSize];
						options2	= new String[arrSize];

						for ( int k = 0; k < arrSize; k++ )
						{
							LassoTypeRef element= new LassoTypeRef();

							lasso.arrayGetElement(res, k, element);
							LassoValue typeName2 = new LassoValue();
							lasso.typeGetName(element,typeName2);

							if((typeName2.name()).equalsIgnoreCase("string"))
							{
								LassoValue outStringValue = new LassoValue();
								lasso.typeGetString(element, outStringValue);

								options1[k] = outStringValue.name();
								options2[k] = outStringValue.name();
							}
							else if((typeName2.name()).equalsIgnoreCase("pair"))
							{
								LassoTypeRef outFirst 	= new LassoTypeRef();
								LassoTypeRef outSecond 	= new LassoTypeRef();
								LassoValue outStringValue1 = new LassoValue();
								LassoValue outStringValue2 = new LassoValue();

								lasso.pairGetFirst( element, outFirst );
								lasso.typeGetString(outFirst, outStringValue1);
								lasso.pairGetSecond( element, outSecond );
								lasso.typeGetString(outSecond, outStringValue2);

								options1[k] = outStringValue1.name();
								options2[k] = outStringValue2.name();
							}
						}
					}

					if(arrSize > 0)
					{
						options = new String[arrSize][2];

						for ( int l = 0; l < arrSize; l++ )
						{
							options[l][0] = options1[l];
							options[l][1] = options2[l];
						}
					}
				}

			}


			if(arrSize < 1)
			{
				lasso.setResultMessage("PDF_Doc->AddSelectList: -values parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			retPdfDoc.addSelectListField(strName, options, strDefault, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddHiddenFieldFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddHiddenField", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{
			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddHiddenField: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
 			if(strValue == null || strValue.length() < 1)
			{
				lasso.setResultMessage("PDF_Doc->AddHiddenField: -value parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			retPdfDoc.addHidenField(strName, strValue);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddSubmitButtonFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddSubmitButton", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -value parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strUrl = getParamValue(lasso,"-url");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddSubmitButton: -url parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strCaption = getParamValue(lasso,"-caption");
			if(strCaption == null) strCaption = strValue;

			retPdfDoc.addSubmitButton(strName, strCaption, strValue, strUrl, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	/**
	 * This method sets the font face.
	 * @param lasso
	 * @param action
	 */

	public int myAddResetButtonFunc (LassoCall lasso, int action)
	{
		int err			=0;
		LassoTypeRef me = new LassoTypeRef();

		if (lasso.getTagSelf(me) != ERR_NOERR )
			return setError(lasso, "PDF_Doc: Error while calling AddResetButton", err);


		PDFDoc retPdfDoc 		= null;
		retPdfDoc = this.getPdfDoc(lasso, me);

		if(retPdfDoc != null)
		{

			float llx, lly, urx, ury;

			DecimalValue f = null;
			f = getDecimalValue(lasso, "-left");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -left parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			llx = f.floatValue();

			f = getDecimalValue(lasso, "-top");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -top parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			ury = retPdfDoc.getConventionalTop(f.floatValue());


			f = getDecimalValue(lasso, "-width");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -width parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			urx = llx + f.floatValue();

			f = getDecimalValue(lasso, "-height");
			if(f == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -height parameter missing/invalid");
				lasso.setResultCode(gerr);
				return gerr;
			}

			lly = ury - f.floatValue();


			String strName = getParamValue(lasso,"-name");
			if(strName == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -name parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strValue = getParamValue(lasso,"-value");
			if(strValue == null)
			{
				lasso.setResultMessage("PDF_Doc->AddResetButton: -value parameter missing/invalid");
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			String strCaption = getParamValue(lasso,"-caption");
			if(strCaption == null) strCaption = strValue;

			retPdfDoc.addRstButton(strName, strCaption, strValue, getFontParam(lasso), llx, lly, urx, ury);
		}
		return 0;
	}


	public DecimalValue getDecimalValue(LassoCall lasso, String param)
	{
		LassoTypeRef outRef = new LassoTypeRef();
		DecimalValue outValue = new DecimalValue();

		if((gerr = lasso.findTagParam2(param, outRef )) == ERR_NOERR)
		{
			if((gerr = lasso.typeGetDecimal(outRef, outValue )) == ERR_NOERR)
				return outValue;
			else
				return null;
		}
		else
			return null;
	}

	public IntValue getBooleanValue(LassoCall lasso, String param)
	{
		LassoValue text = new LassoValue();
		if(ERR_NOERR == lasso.findTagParam(param, text ))
		{
			String bStr = text.data();
			if(bStr != null)
			{
				if(bStr.equalsIgnoreCase("true"))
					return new IntValue(1);
				else
					return new IntValue(0);
			}
			return new IntValue(1);
		}
		else
		{
			if(ERR_NOERR == lasso.tagParamIsDefined(param))
			{
				return new IntValue(1);
			}
			else
				return new IntValue(-1);
		}
	}



	// converts an internal path (full or relative) into a full platform specific path
	String internalToFullPath(LassoCall lasso, String inPath)
	{
		String qualifiedPath = null;
		String resolvedPath = null;
		qualifiedPath = lasso.fullyQualifyPath( inPath );
		resolvedPath = lasso.resolvePath( qualifiedPath );
		return lasso.getPlatformSpecificPath( resolvedPath );
	}



// End Acro-Form

	///////////////////////////////////////////////////////////////////////


	public void registerLassoModule() {
	// registers the Tag
	registerTagModule("PDFTag", "PDF_Doc", "tagPDFDocFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Implement PDF_Doc tag");
	// registerTagModule("PDFTag", "PDF_Graphic", "tagPDFGraphicFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Implement PDF_Graphic tag");
	registerTagModule("PDFTag", "PDF_Font", "tagPDFFontFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Font tag");
	registerTagModule("PDFTag", "PDF_Table", "tagPDFTableFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Table tag");
	registerTagModule("PDFTag", "PDF_Barcode", "tagPDFBarcodeFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Barcode tag");
	registerTagModule("PDFTag", "PDF_Image", "tagPDFImageFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Image tag");
	registerTagModule("PDFTag", "PDF_List", "tagPDFListFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_List tag");
	registerTagModule("PDFTag", "PDF_Text", "tagPDFTextFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Text tag");
	registerTagModule("PDFTag", "PDF_Read", "tagPDFReadFunc", FLAG_INITIALIZER| FLAG_SUBSTITUTION, "To Test PDF_Read tag");
	}


}


